#!/bin/bash

set -e

SERVICE_NAME="终端管控"

python_script=stup-domain.py

curPath=$(readlink -f "$(dirname "$0")")

cd "$curPath"

options=(all web db nginx master redis freeipa aptly server)
options_list="[ all | web | db | nginx | master | redis | freeipa | aptly | server]"
port_array=("6379" "3306" "8000" "4505" "4506" "53" "443" "389" "636" "88" "464" "80")


function err_and_quit () {
    printf "\n\n\033[33mError occured during setup. \nPlease fix possible problems and run the script again.\033[m\n\n"
    exit 1;
}

function check_python_executable() {
    if [[ "$PYTHON" != "" && -x $PYTHON ]]; then
        return 0
    fi

    if which python3 2>/dev/null 1>&2; then
        PYTHON=python3
    elif which python3.6 2>/dev/null 1>&2; then
        PYTHON=python3.6
    elif which python36 2>/dev/null 1>&2; then
        PYTHON=python36
    elif which python3.5 2>/dev/null 1>&2; then
        PYTHON=python3.5
    elif which python35 2>/dev/null 1>&2; then
        PYTHON=python35
    elif which python2.7 2>/dev/null 1>&2; then
        PYTHON=python2.7
    elif which python27 2>/dev/null 1>&2; then
        PYTHON=python27
    else
        echo
        echo "Can't find a python executable of version 2.7|3.5|3.6 or above in PATH"
        echo "Install python 2.7|3.5|3.6 before continue."
        echo "Or if you installed it in a non-standard PATH, set the PYTHON enviroment varirable to it"
        echo
        exit 1
    fi
}


function check_python () {
    echo "Checking python on this machine ..."
    check_python_executable
    if ! which $PYTHON 2>/dev/null 1>&2; then
        echo "No $PYTHON found on this machine. Please install it first."
        err_and_quit;
    fi
    echo
}


function f_usage() {
   if [ "$1" == "start" ]; then
       echo "USAGE: start [options]"
       echo "OPTIONS:"
       echo "       $options_list"
       echo "for example:"
       echo "       ./domain.sh start web"
       return
   elif [ "$1" == "stop" ]; then
       echo "USAGE: stop [options]"
       echo "OPTIONS:"
       echo "       $options_list"
       echo "for example:"
       echo "       ./domain.sh start web"
       return
   elif [ "$1" == "restart" ]; then
       echo "USAGE: restart [options]"
       echo "OPTIONS:"
       echo "       $options_list"
       echo "for example:"
       echo "       ./domain.sh restart web"
       return
   fi
   echo "OPTIONS:"
   printf "       init            \033[33m用以服务部署时初始化配置\033[m\n"
   printf "       check-images    \033[33m用以检测终端管控容器是否缺失\033[m\n"
   echo "       start   $options_list"
   echo "       stop    $options_list"
   echo "       restart $options_list"
}

function contains() {
    local n=$#
    local value=${!n}
    for ((i=1;i < $#;i++)) {
        if [ "${!i}" == "${value}" ]; then
            echo "y"
            return 0
        fi
    }
    echo "n"
    return 1
}


function check_port(){
    echo "*** 端口检测 ***"

    for((i=0;i<${#port_array[@]};i++));
    do
        echo "*** 检测 [ ${port_array[$i]} ] 端口"
        name=$(lsof -i:${port_array[$i]} |grep LISTEN |tail -1|awk '"$1"!=""{print $1}')
        if [ "$name" != "" ]; then
            if [ "$name" != "docker-pr" ]; then
                read -p "注意 [ ${port_array[$i]} ] 端口被 [ $name ] 占用，是否杀掉 (y / n):" is_kill
                if [ "$is_kill" == "y" ]; then
                       pid=$(lsof -i:${port_array[$i]}|tail -1|awk '"$1"!=""{print $2}')
                       kill -9 "$pid"
                       echo "kill $name"
                       if [ "$name" == "systemd-r" ]; then
                              systemctl stop systemd-resolved.service
                              systemctl disable systemd-resolved.service
                       fi
                       if [ "$name" == "dnsmasq" ]; then
                              systemctl stop dnsmasq.service
                              systemctl disable dnsmasq.service
                       fi
                       if [ "$name" == "httpd" ]; then
                              systemctl stop httpd.socket
                              systemctl stop httpd.service
                              systemctl disable httpd.service
                       fi
                #else
                #    printf "***\033[33m 域控服务停止继续\033[m ***\n"
                #    return 0
                fi
            fi
        fi
    done
}

function check_images(){
       sleep 1
       echo "--- 镜像检测 ---"

       webssh=`docker ps |grep webssh | grep -v "grep" | awk '{print $2}'`
       if [ "$webssh" == "" ]; then
           sleep 1
           printf "***\033[33m webssh 未运行\033[m ***\n"
       else
           echo "*** $webssh 存在 ***"
       fi

       nginx=`docker ps |grep nginx | grep -v "grep" | awk '{print $2}'`
       if [ "$nginx" == "" ]; then
           sleep 1
           printf "***\033[33m nginx 未运行\033[m ***\n"
       else
           echo "*** $nginx 存在 ***"
       fi

       mariadb=`docker ps |grep mariadb | grep -v "grep" | awk '{print $2}'`
       if [ "$mariadb" == "" ]; then
           sleep 1
           printf "***\033[33m mariadb 未运行\033[m ***\n"
       else
           echo "*** $mariadb 存在 ***"
       fi

       redis=`docker ps |grep redis | grep -v "grep" | awk '{print $2}'`
       if [ "$redis" == "" ]; then
           sleep 1
           printf "***\033[33m redis 未运行\033[m ***\n"
       else
           echo "*** $redis 存在 ***"
       fi

       salt=`docker ps |grep nfschina/salt | grep -v "grep" | awk '{print $2}'`
       if [ "$salt" == "" ]; then
           sleep 1
           printf "***\033[33m salt 未运行\033[m ***\n"
       else
           echo "*** $salt 存在 ***"
       fi

       domain=`docker ps |grep nfschina/domain | grep -v "grep" | awk '{print $2}'`
       if [ "$domain" == "" ]; then
           sleep 1
           printf "***\033[33m domain 未运行\033[m ***\n"
       else
           echo "*** $domain 存在 ***"
       fi

       aptly=`docker ps |grep nfs-aptly | grep -v "grep" | awk '{print $2}'`
       if [ "$aptly" == "" ]; then
           sleep 1
           printf "***\033[33m aptly 未运行\033[m ***\n"
       else
           echo "*** $aptly 存在 ***"
       fi

       freeipa=`docker ps |grep freeipa | grep -v "grep" | awk '{print $2}'`
       if [ "$freeipa" == "" ]; then
           sleep 1
           printf "***\033[33m freeipa 未运行\033[m ***\n"
       else
           echo "*** $freeipa 存在 ***"
       fi

       domain=`docker ps |grep nfschina/domain | grep -v "grep" | awk '{print $2}'`
       if [ "$domain" == "" ]; then
           sleep 1
           printf "***\033[33m domain 未运行\033[m ***\n"
       else
           echo "*** $domain 存在 ***"
       fi


       sleep 1
       echo "--- 镜像检测完成 ---"
}

function init(){
       check_python;
       export PYTHON=$PYTHON
       echo "--- 准备初始化 ---"
       sleep 2

       read -p "注意 重复初始化会导致配置不一致问题，是否继续 (yes / no):" is_ok
       if [ "$is_ok" != "yes" ]; then
           echo "--- 已退出 ---"
           return
       fi
       echo "--- 进入配置初始化 ---"
       sleep 1
       echo 3
       sleep 1
       echo 2
       sleep 1
       echo 1

       exec $PYTHON "$python_script" "$@"
}

ipa_rewrite="
# VERSION 6 - DO NOT REMOVE THIS LINE

RewriteEngine on

# By default forward all requests to /ipa. If you don't want IPA
# to be the default on your web server comment this line out.
RewriteRule ^/$ https://$DOMAIN_SERVER_FQDN/ipa/ui [L,NC,R=301]
#RewriteRule ^/$ http://$DOMAIN_SERVER_FQDN/mainweb [L,NC,R=301]

# Redirect to the fully-qualified hostname. Not redirecting to secure
# port so configuration files can be retrieved without requiring SSL.
RewriteCond %{HTTP_HOST}    !^$DOMAIN_SERVER_FQDN$ [NC]
RewriteRule ^/ipa/(.*)      http://$DOMAIN_SERVER_FQDN/ipa/$1 [L,R=301]

# Redirect to the secure port if not displaying an error or retrieving
# configuration.
RewriteCond %{SERVER_PORT}  !^443$
RewriteCond %{REQUEST_URI}  !^/ipa/(errors|config|crl)
RewriteCond %{REQUEST_URI}  !^/ipa/[^\?]+(\.js|\.css|\.png|\.gif|\.ico|\.woff|\.svg|\.ttf|\.eot)$
RewriteRule ^/ipa/(.*)      https://$DOMAIN_SERVER_FQDN/ipa/$1 [L,R=301,NC]

# Rewrite for plugin index, make it like it's a static file
RewriteRule ^/ipa/ui/js/freeipa/plugins.js$    /ipa/wsgi/plugins.py [PT]
"
ipa_rewrite_conf=$curPath/docker/freeipa-docker/data/etc/httpd/conf.d/ipa-rewrite.conf

function set_db_master_slave() {
  ./stup-db-master-slave.sh
}

function set_nfsdomain_net() {
   nfsdomain_net=`docker network list |grep nfsdomain-net | grep -v "grep" | awk '{print $2}'`
   if [ "$nfsdomain_net" == "" ]; then
    echo "create nfsdomain network"
    docker network create nfsdomain-net
   fi
}

function set_chrony_conf() {
    filename="/etc/chrony.conf"
    allow_all="allow all"
    local_stratum="local stratum 10"
    need_restart=false
    # 检查文件是否存在
    if [ -f "$filename" ]; then
            # 检查文件中是否存在未注释的模式
        if ! grep -qP "(?<!#)\b$allow_all\b" "$filename"; then
            # 如果不存在未注释的模式，则追加
            echo "$allow_all" >> "$filename"
            echo "已将 $allow_all 追加到文件 $filename 中。"
            need_restart=true
        fi
        if ! grep -qP "(?<!#)\b$local_stratum\b" "$filename"; then
            # 如果不存在未注释的模式，则追加
            echo "$local_stratum" >> "$filename"
            echo "已将 $local_stratum 追加到文件 $filename 中。"
            need_restart=true
        fi
        # 根据标记决定是否重启 chronyd 服务
        if [ "$need_restart" = true ]; then
            echo "create chrony conf"
            systemctl restart chronyd.service
        fi
    else
        echo "文件 $filename 不存在。"
    fi

}

function init_freeipa_policy() {
    echo '开始freeipa初始化'
    export $(grep -v '^#' .env | xargs)
    ipa_rewrite="
# VERSION 6 - DO NOT REMOVE THIS LINE

RewriteEngine on

# By default forward all requests to /ipa. If you don't want IPA
# to be the default on your web server comment this line out.
RewriteRule ^/$ https://$DOMAIN_SERVER_FQDN/ipa/ui [L,NC,R=301]
#RewriteRule ^/$ http://$DOMAIN_SERVER_FQDN/mainweb [L,NC,R=301]

# Redirect to the fully-qualified hostname. Not redirecting to secure
# port so configuration files can be retrieved without requiring SSL.
RewriteCond %{HTTP_HOST}    !^$DOMAIN_SERVER_FQDN$ [NC]
RewriteRule ^/ipa/(.*)      http://$DOMAIN_SERVER_FQDN/ipa/\$1 [L,R=301]

# Redirect to the secure port if not displaying an error or retrieving
# configuration.
RewriteCond %{SERVER_PORT}  !^443$
RewriteCond %{REQUEST_URI}  !^/ipa/(errors|config|crl)
RewriteCond %{REQUEST_URI}  !^/ipa/[^\?]+(\.js|\.css|\.png|\.gif|\.ico|\.woff|\.svg|\.ttf|\.eot)$
RewriteRule ^/ipa/(.*)      https://$DOMAIN_SERVER_FQDN/ipa/\$1 [L,R=301,NC]

# Rewrite for plugin index, make it like it's a static file
RewriteRule ^/ipa/ui/js/freeipa/plugins.js$    /ipa/wsgi/plugins.py [PT]
"
    reprepro_debian_distributions="
Origin: nfs-terminal
Label: nfs-terminal
Codename: stretch
Architectures: amd64
Components: main
Description: Apt repository for nfs-terminal
DebOverride: override.stretch
DscOverride: override.stretch
SignWith: 76C3C2A0
"
    reprepro_nginx_conf="
server {
  listen 80;
  server_name $DOMAIN_IP;

  access_log /var/log/nginx/packages-access.log;
  error_log /var/log/nginx/packages-error.log;

  location / {
    root /var/www/repos/apt/debian;
    index index.html;
  }

  location /tiger {
    alias /var/www/repos/apt/tiger;
    index index.html;
  }

  location /nfs4 {
    alias /var/www/repos/apt/nfs4;
    index index.html;
  }

  location /kylin {
    alias /var/www/repos/apt/kylinV10/;
    index index.html;
  }
  location /kylin-v10 {
    alias /var/www/repos/apt/kylinV10/;
    index index.html;
  }

  location /kylin-v10spl {
    alias /var/www/repos/apt/kylinV10SPL/;
    index index.html;
  }

  location /uos-20pro {
    alias /var/www/repos/apt/uos20PRO/;
    index index.html;
  }

  location /x86_64 {
    alias /var/www/repos/apt/x86_64;
    index index.html;
  }

  location /sw_64 {
    alias /var/www/repos/apt/sw_64;
    index index.html;
  }

  location /mips64 {
    alias /var/www/repos/apt/mips64;
    index index.html;
  }

  location /loongarch64 {
    alias /var/www/repos/apt/loongarch64;
    index index.html;
  }

  location /aarch64 {
    alias /var/www/repos/apt/aarch64;
    index index.html;
  }

  location ~ /(.*)/conf {
    deny all;
  }

  location ~ /(.*)/db {
    deny all;
  }
}
"
    sudo sh -c "echo $DOMAIN_IP $DOMAIN_SERVER_FQDN >> /etc/hosts"
    ipa=`docker images |grep  nfs-freeipa-server | grep -v "grep" | awk '{print $1}'`
    if [ "$ipa" == "nfs-freeipa-server" ]; then  # 老的镜像
      # 利用登录接口进行初始化检测
      while [[ "$(curl -k -s \
          -H Referer:https://$DOMAIN_SERVER_FQDN/ipa \
          -H "Content-Type:application/x-www-form-urlencoded" \
          -H "Accept:text/plain" \
          -w "%{http_code}" \
          --data "user=$DOMAIN_SERVER_ADMIN&password=$DOMAIN_SERVER_ADMIN_PASSWD" \
          -X POST \
          https://$DOMAIN_SERVER_FQDN/ipa/session/login_password)" != "200" ]];
      do
          echo '等待freeipa初始化...'
          sleep 10;
      done
      echo 'freeipa初始化完成...'
      sleep 5
      echo '...'
      sleep 5
      echo '...'
      sleep 5
      echo '...'
      sleep 5
      echo '...'
      echo 'freeipa配置默认策略...'
      docker-compose exec freeipa bash /freeipa-policy-init.sh
      sleep 5
      echo '...'
      sleep 5
      echo '...'
      sleep 5
      echo '...'
      sleep 5
      echo '...'
      echo "$ipa_rewrite" > $ipa_rewrite_conf
      sleep 5
      echo '...'
      docker-compose exec freeipa bash /max_open_files.sh
      docker-compose restart freeipa
      sleep 5
      echo '...'
      sleep 15
      docker-compose exec freeipa sh -c 'dsconf ldap://$DOMAIN_SERVER_FQDN -D "cn=Directory Manager" -w $DOMAIN_SERVER_ADMIN_PASSWD config replace nsslapd-maxdescriptors=${MAX_OPEN_FILES}'
      sleep 3
      echo '...'
      docker-compose exec freeipa sh -c 'ldapsearch -x -H ldap://$DOMAIN_SERVER_FQDN -D "cn=Directory Manager" -w $DOMAIN_SERVER_ADMIN_PASSWD -b "cn=config" -s base "(objectclass=*)" nsslapd-maxdescriptors'
      echo '完成freeipa默认策略配置'
      sleep 3
    else  # 新镜像 nfsdomain/ipa:v1
      sleep 5
      echo '...'
      sleep 5
      echo '...'
      sleep 10
      echo '...'
      sleep 5
      echo '...'
      echo '...'
      sleep 10
    fi
}

function load_aptly() {
    echo '准备载入aptly仓库数据...'
    if rpm -q nfs-domain-client-depends >/dev/null; then
        ./update-nfsaptly.sh nfs-4.0.tar.gz
        ./update-nfsaptly.sh nfs-5.0.tar.gz
        echo '载入aptly仓库数据完成...'
    else
        echo "未安装nfs-domain-client-depends"
    fi
    echo
    echo
}

function print_info() {
    printf "***\033[33m $SERVICE_NAME 初始化完成\033[m ***\n"
    printf "\033[33m 可通如下方式访问：\033[m \n"
    printf "\033[33m IP:   $DOMAIN_IP \033[m \n"
    printf "\033[33m 域名： $DOMAIN_SERVER_FQDN \033[m \n"
    printf "\033[33m 默认管理用户及密码：\033[m \n"
    printf "\033[33m admin / nfschina@2023 \033[m \n"
    printf "***\033[33m $SERVICE_NAME 初始化完成\033[m ***\n"
}

destination_rsyslog_conf=/etc/rsyslog.conf
source_rsyslog_conf=$curPath/docker/elk-docker/rsyslog.conf
destination_nfsdomain_handler_conf=/etc/rsyslog.d/rsyslog-nfsdomain-handler.conf
source_nfsdomain_handler_conf=$curPath/docker/elk-docker/rsyslog-nfsdomain-handler.conf
function set_rsyslog() {
  if [ -f $destination_rsyslog_conf ] || [ -d /etc/rsyslog.d/ ]; then
    if [ -d $curPath/docker/elk-docker ]; then
      cat $source_rsyslog_conf > $destination_rsyslog_conf
      cat $source_nfsdomain_handler_conf > $destination_nfsdomain_handler_conf
    fi
  else
    echo "$destination_rsyslog_conf or /etc/rsyslog.d/ 不存在"
  fi
}

function start(){
       if [ $(contains "${options[@]}" "$1") != "y" ]; then
           f_usage start
           return
       fi

       if [ "$1" == "all" ]; then
           check_port;
           set_nfsdomain_net
           docker-compose up -d
           set_chrony_conf
           #set_db_master_slave
           #init_freeipa_policy
           load_aptly
           set_rsyslog
           print_info
           return
       elif [ "$1" == "web" ]; then
           docker-compose up -d web
           return
       elif [ "$1" == "db" ]; then
           docker-compose up -d db
           return
       elif [ "$1" == "nginx" ]; then
           docker-compose up -d nginx
           return
       elif [ "$1" == "master" ]; then
           docker-compose up -d master
           return
       elif [ "$1" == "redis" ]; then
           docker-compose up -d redis
           return
       elif [ "$1" == "aptly" ]; then
           docker-compose up -d nfs-aptly
           return
       elif [ "$1" == "freeipa" ]; then
           docker-compose up -d freeipa
           init_freeipa_policy
           return
       elif [ "$1" == "webssh" ]; then
           docker-compose up -d webssh
           return
       elif [ "$1" == "server" ]; then
           check_port;
           docker-compose start
           return
       fi
}
function stop(){
       if [ $(contains "${options[@]}" "$1") != "y" ]; then
           f_usage stop
           return
       fi

       if [ "$1" == "all" ]; then
           printf "***\033[33m 注意：建议使用其他选项启用或停用服务\033[m ***\n"
           sleep 1
           docker-compose down
           return
       elif [ "$1" == "web" ]; then
           docker-compose stop web
           return
       elif [ "$1" == "db" ]; then
           docker-compose stop db
           return
       elif [ "$1" == "nginx" ]; then
           docker-compose stop nginx
           return
       elif [ "$1" == "master" ]; then
           docker-compose stop master
           return
       elif [ "$1" == "redis" ]; then
           docker-compose stop redis
           return
       elif [ "$1" == "freeipa" ]; then
           docker-compose stop freeipa
           return
       elif [ "$1" == "webssh" ]; then
           docker-compose stop webssh
           return
       elif [ "$1" == "server" ]; then
           docker-compose stop
           P_ID=`ps -ef | grep -w "rpc_server.py" | grep -v "grep" | awk '{print $2}'`
            if [ "$P_ID" == "" ]; then
               echo "*** rpc_server.py not running ***"
            else
               echo "*** rpc_server.py process pid is:$P_ID ***"
               echo "*** begin kill rpc_server.py process,kill is:$P_ID ***"
               kill -9 $P_ID
            fi
           return
       fi
}

case "$1" in
   "init")
       init
       ;;
   "check-images")
       check_images
       ;;
   "start")
       start "$2"
       ;;
   "stop")
       stop "$2"
       ;;
   "restart")
       if [ $(contains "${options[@]}" "$2") != "y" ]; then
           f_usage restart
       else
           stop "$2"
           sleep 3
           start "$2"
           echo "*** restart "$2" ***"
       fi
       ;;
   *)
   f_usage
   ;;

esac
exit 0
