import datetime
import os
import json
import uuid
import requests
from python_freeipa import ClientMeta, ClientLegacy, exceptions as freeip_exceptions
import logging
from django.conf import settings

from saltapi.tasks import check_sudo_policy
from accounts import models
from accounts.models import AccountEntity
from devices.models import Devices
from devices.serializers import DeviceSerializer

logger = logging.getLogger(__name__)


def get_ipa_client():
    client = ClientMeta(settings.DOMAIN_SERVER_FQDN, verify_ssl=False)
    client.login(settings.DOMAIN_SERVER_ADMIN, settings.DOMAIN_SERVER_ADMIN_PASSWD)
    return client


def check_user(o_id, o_userpassword):
    client = ClientMeta(settings.DOMAIN_SERVER_FQDN, verify_ssl=False)
    res = {
        'exist': False,
        'msg': ''
    }
    try:
        client.login(o_id, o_userpassword)
    except freeip_exceptions.InvalidSessionPassword as e:
        # logger.info(e)
        res['exist'] = False
        res['msg'] = '用户不存在或密码错误'
        return res
    except freeip_exceptions.PasswordExpired as e:
        # 用户名密码正确，但密码过期
        res['exist'] = True
        res['msg'] = '密码过期'
        return res
    except freeip_exceptions.UserLocked as e:
        # logger.info(e)
        res['exist'] = False
        res['msg'] = '用户已被锁定'
        return res
    else:
        client.logout()
        res['exist'] = True
        res['msg'] = '登录成功'
        return res
    # finally:
    #    res['exist'] = False
    #    res['msg'] = '未知错误'
    #    return res


def add_user(a_uid, o_cn, password=None, o_krbpasswordexpiration=None):
    client = get_ipa_client()
    res_add_user = True

    try:
        # givenname和sn都是必选字段
        # 当前将givenname配置为用户名称，sn配置为a_uid
        res = pwpolicy_show()
        result = res.get('result')

        def _get_value(res, key):
            return int(res.get(key)[0]) if res.get(key) else None

        max_life = _get_value(result, 'krbmaxpwdlife')
        if o_krbpasswordexpiration:
            max_life = datetime.datetime.now() + datetime.timedelta(days=(365 * 30))
            max_life = max_life.strftime("%Y-%m-%d %H:%M:%SZ")
        else:
            max_life = None
        logger.info(f"*******{max_life}****")
        gn = o_cn if not o_cn.isspace() else a_uid
        user = client.user_add(
            a_uid,
            o_givenname=gn,
            o_sn=a_uid,
            o_cn=gn,
            o_displayname=gn,
            o_gecos=gn,
            o_userpassword=password if password else settings.DEFAULT_USER_PASSWORD,
            o_krbpasswordexpiration=max_life,
            o_loginshell='/bin/bash',
        )
        logger.info(user)
        res_add_user = True
    except Exception as e:
        res_add_user = False
        if f'user with name "{a_uid}" already exists' == str(e):
            res_add_user = False
        else:
            logger.error(e)
    finally:
        client.logout()
        return res_add_user


def user_mod(a_uid, o_cn):
    client = get_ipa_client()
    try:
        gn = o_cn if not o_cn.isspace() else a_uid
        res = client.user_mod(a_uid, o_cn=gn, o_givenname=gn, o_sn=gn, o_displayname=gn, o_gecos=gn)
        logger.info(res)
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def user_del(a_uid):
    client = get_ipa_client()
    try:
        res = client.user_del(a_uid)
        logger.info(res)
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def reset_user_pwd(a_uid, passwd=None, o_krbpasswordexpiration=None):
    client = get_ipa_client()
    try:
        res = client.user_mod(
            a_uid,
            o_userpassword=settings.DEFAULT_USER_PASSWORD if passwd is None else passwd,
        )
        if o_krbpasswordexpiration:
            max_life = datetime.datetime.now() + datetime.timedelta(days=(365 * 30))
            max_life = max_life.strftime("%Y-%m-%d %H:%M:%SZ")
            client.user_mod(
                a_uid,
                o_krbpasswordexpiration=max_life
            )
        # logger.info(res)
        status = 'success'
    except freeip_exceptions.NotFound as e:
        logger.error(e)
        status = 'notfound'
    except Exception as e:
        logger.error(e)
        status = 'unknown'
    finally:
        client.logout()
    return status


def user_status_disable(a_uid):
    client = get_ipa_client()
    try:
        res = client.user_disable(a_uid)
        logger.info(res)
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def user_status_enable(a_uid):
    client = get_ipa_client()
    try:
        res = client.user_enable(a_uid)
        logger.info(res)
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def user_status(a_uid):
    client = get_ipa_client()
    try:
        res = client.user_status(a_uid)
        logger.info(res)
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def user_unlock(a_uid):
    client = get_ipa_client()
    try:
        res = client.user_unlock(a_uid)
        logger.info(res)
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def user_show(a_uid):
    client = get_ipa_client()
    res = {
        'value': a_uid,
        'memberof_hbacrule': []
    }
    try:
        show_res = client.user_show(a_uid)
        logger.info(res)
        res['memberof_hbacrule'] = show_res.get('result').get('memberof_hbacrule')
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()
        return res


def user_find(a_uid):
    client = get_ipa_client()
    # res = client.user_find(a_uid)
    try:
        res = client.user_show(a_uid)
    except Exception as e:
        logger.info(e)
        return False
    return res


def hbacrule_add(a_cn):
    client = get_ipa_client()
    try:
        res = client.hbacrule_add(
            a_cn,
            # 默认配置service为all
            o_servicecategory='all',
        )
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def hbacrule_add_user(a_cn, o_user):
    client = get_ipa_client()
    try:
        res = client.hbacrule_add_user(a_cn, o_user=o_user)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def hbacrule_add_host(a_cn, o_host):
    client = get_ipa_client()
    try:
        res = client.hbacrule_add_host(a_cn, o_host=o_host)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def hbacrule_remove_user(a_cn, o_user):
    client = get_ipa_client()
    try:
        res = client.hbacrule_remove_user(a_cn, o_user=o_user)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def hbacrule_remove_host(a_cn, o_host):
    client = get_ipa_client()
    try:
        res = client.hbacrule_remove_host(a_cn, o_host=o_host)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


# HBAC Services : crond ftp gdm gdm-password gssftp kdm login proftpd pure-ftpd sshd su su-l sudo sudo-i systemd-user vsftpd
def hbacrule_add_service(a_cn, o_hbacsvc):
    client = get_ipa_client()
    try:
        res = client.hbacrule_add_service(a_cn, o_hbacsvc=o_hbacsvc)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def hbacrule_del(a_cn):
    client = get_ipa_client()
    try:
        res = client.hbacrule_del(a_cn)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def pwpolicy_show(a_cn='global_policy'):
    client = get_ipa_client()
    try:
        res = client.pwpolicy_show(a_cn)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def pwpolicy_mod(a_cn='global_policy',
                 o_krbmaxpwdlife=None,
                 o_krbminpwdlife=None,
                 o_krbpwdhistorylength=None,
                 o_krbpwdmindiffchars=None,
                 o_krbpwdminlength=None,
                 o_cospriority=None,
                 o_krbpwdmaxfailure=None,
                 o_krbpwdfailurecountinterval=None,
                 o_krbpwdlockoutduration=None,
                 ):
    """
    o_krbmaxpwdlife: 天
    o_krbminpwdlife: 小时
    o_krbpwdfailurecountinterval: 秒
    o_krbpwdlockoutduration: 秒
    """
    client = get_ipa_client()
    try:
        return client.pwpolicy_mod(
            a_cn,
            o_krbmaxpwdlife,
            o_krbminpwdlife,
            o_krbpwdhistorylength,
            o_krbpwdmindiffchars,
            o_krbpwdminlength,
            o_cospriority,
            o_krbpwdmaxfailure,
            o_krbpwdfailurecountinterval,
            o_krbpwdlockoutduration,
        )
    except Exception as e:
        logger.error(e)
        return {'msg': str(e)}
    finally:
        client.logout()


def change_passwd(a_principal, a_password, a_current_password):
    client = get_ipa_client()
    try:
        # res = client.passwd(a_principal, a_password, a_current_password)
        res = client.change_password(a_principal, a_password, a_current_password)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def hbacrule_find(o_cn):
    client = get_ipa_client()
    res = {
        'count': 0,
        'memberuser_user': [],
        'memberhost_host': []
    }
    try:
        hbacrule_res = client.hbacrule_find(o_cn=o_cn)
        res['count'] = hbacrule_res.get('count')
        if hbacrule_res.get('count') != 0:
            res['memberuser_user'] = hbacrule_res.get('result')[0].get('memberuser_user')
            res['memberhost_host'] = hbacrule_res.get('result')[0].get('memberhost_host')
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()
        return res


def hbacrule_enable(host_name):
    client = get_ipa_client()
    try:
        a_cn = 'hbac_' + host_name
        res = client.hbacrule_enable(a_cn)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def hbacrule_disable(host_name):
    client = get_ipa_client()
    try:
        a_cn = 'hbac_' + host_name
        res = client.hbacrule_disable(a_cn)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def hbac_add_user_to_host(account_id, host_name):
    '''
    新建一条规则，用于用户与主机的绑定关系
    '''
    account = AccountEntity.objects.get(pk=account_id)
    rule_name = 'hbac_' + host_name
    hbacrule_add(rule_name)
    hbacrule_add_host(rule_name, host_name)
    hbacrule_add_user(rule_name, account.name)


def host_disable(a_fqdn):
    """
    TODO: 目前不知道disable后如何enable
    """
    client = get_ipa_client()
    try:
        res = client.host_disable(a_fqdn)
        return res
    except Exception as e:
        logger.error(e)
    finally:
        client.logout()


def sudorule_all_add_user(a_cn, s_user, policy_no=None):
    """
    将用户添加到freeipa 的sudo服务组
    :param a_cn:
    :param s_user: 用户名 列表
    :return:
    """
    client = get_ipa_client()
    account_items = models.AccountEntity.objects.all()
    for o_user in s_user:
        try:
            res = client.sudorule_add_user(a_cn, o_user=o_user)
            logger.info(res)
            # 更新表中的策略状态
            account_items.filter(name=o_user).update(policy_status=1)
            account_id = account_items.get(name=o_user).id
            # device_items = Devices.objects.filter(accounts_id=account_id).exclude(is_domain=0).all().defer(
            #     'device_root').order_by('-id')
            device_items = Devices.objects.filter(bind_account=account_id).all()
            celery_task = check_sudo_policy(
                DeviceSerializer(device_items, many=True).data,  # celery参数不能是query_set类型
                {'on_off': 'on', 'domain_name': os.environ.get("DOMAIN_NAME"), 'domain_user': o_user,
                 "account_id": account_id},
                **{'policy_no': policy_no},
            )
        except Exception as e:
            account_items.filter(name=o_user).update(policy_status=0)
            logger.error(e)
    client.logout()


def sudorule_all_remove_user(a_cn, s_user, policy_no=None):
    """
    将用户移除到freeipa 的sudo服务组
    :param a_cn:
    :param s_user: 用户名 列表
    :return:
    """
    client = get_ipa_client()
    account_items = models.AccountEntity.objects.all()
    for o_user in s_user:
        try:
            res = client.sudorule_remove_user(a_cn, o_user=o_user)
            logger.info(res)
            # 更新表中的策略状态
            account_items.filter(name=o_user).update(policy_status=1)
            account_id = account_items.get(name=o_user).id
            # device_items = Devices.objects.filter(accounts_id=account_id).exclude(is_domain=0).all().defer(
            #     'device_root').order_by('-id')
            device_items = Devices.objects.filter(bind_account=account_id).all()
            celery_task = check_sudo_policy(
                DeviceSerializer(device_items, many=True).data,  # celery参数不能是query_set类型
                {'on_off': 'off', 'domain_name': os.environ.get("DOMAIN_NAME"), 'domain_user': o_user,
                 'account_id': account_id},
                **{'policy_no': policy_no},
            )
        except Exception as e:
            account_items.filter(name=o_user).update(policy_status=0)
            logger.error(e)
    client.logout()

def dnsrecord_show(a_idnsname):
    '''
    {'result': {'objectclass': ['top', 'idnsrecord'], 'idnsname': [{'__dns_name__': 'server102'}], 'arecord': ['172.30.98.49'], 'dn': 'idnsname=server102,idnsname=nfsdomain.com.,cn=dns,dc=nfsdomain,dc=com'}, 'value': {'__dns_name__': 'server102'}, 'summary': None}
    '''
    client = get_ipa_client()
    try:
        res = client.dnsrecord_show(
            a_dnszoneidnsname=os.environ.get('DOMAIN_NAME'),
            a_idnsname=a_idnsname,
        )
        return res
    except freeip_exceptions.NotFound as e:
        logger.error(e) # 'server103: DNS resource record not found'
        return False
    except Exception as e:
        logger.error(e)
    return False

def dnsrecord_add(a_idnsname, o_a_part_ip_address):
    '''
    e.g :
    ipa dnsrecord-add nfsdomain.com server --a-rec 172.16.127.128
    a_dnszoneidnsname=nfsdomain.com
    a_idnsname=server
    o_a_part_ip_address=172.30.98.211
    {'result': {'objectclass': ['top', 'idnsrecord'], 'arecord': ['172.30.98.49'], 'idnsname': [{'__dns_name__': 'server100'}], 'dn': 'idnsname=server100,idnsname=nfsdomain.com.,cn=dns,dc=nfsdomain,dc=com'}, 'value': {'__dns_name__': 'server100'}, 'summary': None}
    '''
    client = get_ipa_client()
    logger.info(os.environ.get('DOMAIN_NAME'))
    logger.info(a_idnsname)
    logger.info(o_a_part_ip_address)

    try:
        res = client.dnsrecord_add(
            a_dnszoneidnsname=os.environ.get('DOMAIN_NAME'),
            a_idnsname=a_idnsname,
            o_a_part_ip_address=o_a_part_ip_address
        )
        return res
    except Exception as e:
        logger.error(e)
    return False

def dnsrecord_del(a_idnsname):
    client = get_ipa_client()
    # client.dnsrecord_del(os.environ.get('DOMAIN_NAME'), a_idnsname, a_port_ip_address={"del":ip})
    client.dnsrecord_del(os.environ.get('DOMAIN_NAME'), a_idnsname, o_del_all=True)
    return


def ipa_backup_restore(command, file_path=None):
    url = f"http://{settings.DOMAIN_SERVER_FQDN}/command/"

    if command == 'ipa-backup':
        payload = json.dumps({
                "task_id":str(uuid.uuid1()),
                "command":command,
                "args":"--data --online"
            })
    else:
        payload = json.dumps({
            "task_id": str(uuid.uuid1()),
            "command": command,
            "args": file_path
        })
    headers = {
        'Content-Type': 'application/json'
    }
    try:
        response = requests.request("POST", url, headers=headers, data=payload)
        if response.status_code != 202:
            return False
        logger.info(response.text)
        res = json.loads(response.text)
        return res
    except requests.exceptions.RequestException as e:
        logging.error(f"请求错误: {e}")
    except Exception as e:
        logging.error(f"未知错误: {e}")
