import re
import copy
import logging
from ldap3 import Server, Connection, SUBTREE
from organizations.models import OrgnizationEntity, LdapOrgnization
from accounts.models import Ldap, LdapSyncStatus, LdapAccountEntity, AccountEntity
from accounts.freeipa import add_user, get_ipa_client, reset_user_pwd
from saltapi.utils import DefaultPolicy
logger = logging.getLogger(__name__)

EXCLUDE_GROUP = '离职保留人员'

class LdapAD(object):
    def __init__(self, host, port, admin_user_dn, admin_password, base_dn, base_ou=None):
        self.ldap_host = host + ':' + str(port)

        # self.ldap_port = int(port)
        self.base_dn = base_dn
        self.base_ou = base_ou
        self.admin_user_dn = admin_user_dn
        self.admin_password = admin_password

        self.user_update_num = 0
        self.user_add_num = 0
        self.user_fail_num = 0
        self.user_clash_num = 0
        self.user_sync_num = 0

        self.group_update_num = 0
        self.group_add_num = 0
        self.group_fail_num = 0
        self.group_clash_num = 0
        self.group_sync_num = 0


        try:
            # 连接ldap服务器
            self.s = Server(host=self.ldap_host, port=None, use_ssl=False, get_info='ALL')
            self.conn = Connection(self.s, user=self.admin_user_dn, password=self.admin_password,
                              # auto_bind='NONE',
                              #                   version=3,
                              # authentication='SIMPLE', client_strategy='SYNC', auto_referrals=True,
                              check_names=True,
                              read_only=True,
                              raise_exceptions=True
                              )
            self.conn.bind()  # conn.last_error()
        except Exception as e:
            self.conn = False
            logger.info('连接LDAP服务器失败')
            logger.info(e)

    def test_connection(self):
        return self.conn

    def get_users(self, ldap_info, user=None, password=None):
        if not self.conn:
            return
        default_policy = DefaultPolicy()
        client = get_ipa_client()
        update_user = {}
        search_base_dn = []
        if self.base_ou:
            for ou in self.base_ou:
                search_base_dn.append('{},{}'.format(ou, self.base_dn))   # 'OU=Rearch Center,DC=cpu,DC=cn'  # ou=Rearch Center 基础ou
        else:
            search_base_dn.append(self.base_dn )  # 'dc=cpu-os,dc=com'
        search_filter = '(objectClass=user)'  # 查询所有人，
        if user:
            search_filter = f'(objectClass=user)(cn={user})'  # 查询某个人：'(objectClass=user)(cn=xiaoming)'
        attributes = [
            'displayName',
            'sAMAccountName',
            'userPrincipalName',
            'mail',
            'userAccountControl',
            # 'memberOf',
            'distinguishedName',
            'objectGUID',
            'employeeID',
            "telephoneNumber",
            "co",
            "st",
            "l",
            "streetAddress",
            # "postalAddress"
            # 'unicodePwd'
        ]
        for search_base in search_base_dn:
            self.conn.search(
                search_base=search_base, search_filter=search_filter,
                search_scope=SUBTREE, attributes=attributes
            )
            # 查询结果
            for entry in self.conn.response:
                # logger.info(entry)
                if entry['type'] != 'searchResEntry':
                    continue

                cn_name = entry.get('attributes').get('displayName')  # 姓名
                cn_name = cn_name if cn_name else ''
                email = entry.get('attributes').get('mail')  # 用户邮箱
                email = email if email else ''
                telephoneNumber = entry.get('attributes').get('telephoneNumber')  # mobile
                telephoneNumber = telephoneNumber if telephoneNumber else None

                name = entry.get('attributes').get('sAMAccountName')  # 用户ID, 值唯一
                userPrincipalName = entry.get('attributes').get('userPrincipalName')  # 用户登录账号
                userPrincipalName = userPrincipalName if userPrincipalName else None
                user_status = entry.get('attributes').get('userAccountControl')  # 用户状态 512 514
                # 'groups': entry.get('attributes').get('memberOf'),  # 用户所在群组
                dn = entry.get('attributes').get('distinguishedName')  # 用户DN, DN中包含群组信息
                guid = entry.get('attributes').get('objectGUID')  # GUID
                employeeID = entry.get('attributes').get('employeeID')  # 工号
                co = entry.get('attributes').get('co')  # address
                st = entry.get('attributes').get('st')  # address
                l = entry.get('attributes').get('l')  # address
                postalAddress = entry.get('attributes').get('streetAddress')  # address
                if isinstance(st, str):
                    st = [st]
                if isinstance(l, str):
                    l = [l]
                if isinstance(postalAddress, str):
                    postalAddress = [postalAddress]
                address = st + l + postalAddress
                address = address if address else None
                # logger.info("国家:", co)
                # logger.info("省:", st)
                # logger.info("市:", l)
                # logger.info("街道:", postalAddress)

                # ldap中用户的状态
                if user_status in ('514', '4096', '66050', 514, 4096, 66050):  # 禁用
                    user_status = 1
                else:
                    user_status = 0
                user = {
                    'cn_name': cn_name,  # 姓名
                    'name': name,  # 用户ID
                    'email': email,  # 用户邮箱
                    'userPrincipalName': userPrincipalName,  # 用户登录账号
                    'system_status': user_status,  # 用户状态 512 514
                    'dn': dn,  # 用户DN, DN中包含群组信息  'CN=xxx,OU=user,OU=武汉研发组,OU=产品研发部,OU=Rearch Center,DC=cpu,DC=cn
                    'guid': guid,  # GUID
                    'ldap_id': ldap_info.id,
                    # 'phone': telephoneNumber,
                    # 'co': co,
                    # 'st': st,
                    # 'l': l,
                    # 'postalAddress': postalAddress,
                    # 'address':address
                }
                # logger.info(user)
                ldap_user = LdapAccountEntity.objects.filter(name=name).first()
                # ldap用户所在的ldap组
                # pattern = r'CN=([^,]+)'
                # match_obj = re.search(pattern, dn)
                # if match_obj:
                #     ou = dn.rsplit(match_obj.group(0)+',')[-1]   # 'OU=user,OU=武汉研发组,OU=产品研发部,OU=Rearch Center,DC=cpu,DC=cn
                # else:
                #     ou = dn
                ou = []
                for i in dn.split(','):
                    if not i.startswith('CN='):
                        ou.append(i)
                ou = ','.join(ou)
                if f'OU={EXCLUDE_GROUP}' in ou:
                    continue
                self.user_sync_num += 1
                ldap_org = LdapOrgnization.objects.filter(dn=ou).exclude(org_id=None).first()
                if not ldap_user:
                    # ldap用户未迁移过
                    ldap_user = LdapAccountEntity.objects.create(**user)
                    account = AccountEntity.objects.filter(name=name).first()
                    if account:  # 创建过此用户
                        # if ldap_org:
                        #     if account.org_info_id != ldap_org.org_id:
                        #         account.org_info_id = ldap_org.org_id  # 移动所在组
                        # if account.system_status != system_status:
                        #     account.system_status = system_status  # 改变状态
                        # if account.cn_name != cn_name:
                        #     account.cn_name = cn_name
                        # if account.email != email:
                        #     account.email = email
                        # account.save()
                        self.user_clash_num += 1
                    else:  # 未创建过此用户
                        # 用户所在ldap的组是否已经创建，若未创建，用户放到跟组下
                        org_id = 1
                        if ldap_org:
                            org_id = ldap_org.org_id
                        # 创建用户
                        account = AccountEntity.objects.create(
                            name=name, cn_name=cn_name, email=email, user_status=user_status,
                            org_info_id=org_id, create_type=1, phone=telephoneNumber, address=address,
                            policy_status=1
                        )
                        # ldap用户加到freeipa
                        add_user(account.name, account.cn_name, ldap_info.default_password)
                        self.user_add_num += 1
                        if int(default_policy.get_default_policy().get('root')) == 1:
                            client.sudorule_add_user('sudo_admin', o_user=name)
                            account.policy_status = 1
                            account.is_root = 1
                            account.save()
                    ldap_user.account_id = account.id
                    ldap_user.save()
                else:
                    # ldap用户已迁移过，信息是否变更
                    before_change = {
                        'cn_name': ldap_user.cn_name,
                        'name': ldap_user.name,
                        'email': ldap_user.email,
                        'userPrincipalName': ldap_user.userPrincipalName,
                        'user_status': ldap_user.system_status,
                        'dn': ldap_user.dn,
                        'guid': ldap_user.guid,
                    }
                    account = AccountEntity.objects.filter(name=name).first()
                    if account:
                        if ldap_user.ldap_id != ldap_org.ldap_id:
                            self.user_clash_num += 1
                            continue
                        if account.create_type != 1:
                            self.user_clash_num += 1
                            continue
                        change = False
                        if ldap_org.org_id != account.org_info_id:
                            change = True
                            account.org_info_id = ldap_org.org_id
                        if account.user_status != user_status:
                            change = True
                            account.user_status = user_status  # 改变状态
                        if account.cn_name != cn_name:
                            change = True
                            account.cn_name = cn_name
                        if account.email != email:
                            change = True
                            account.email = email
                        if account.phone != telephoneNumber:
                            change = True
                            account.phone = telephoneNumber
                        if account.address != address:
                            change = True
                            account.address = address

                        if ldap_user.dn != dn:
                            change = True
                            ldap_user.dn = dn
                            if ldap_org:
                                if account.org_info_id != ldap_org.org_id:
                                    account.org_info_id = ldap_org.org_id  # 移动所在组
                                    account.save()
                        if ldap_user.cn_name != cn_name:
                            change = True
                            ldap_user.cn_name = cn_name
                        if ldap_user.email != email:
                            change = True
                            ldap_user.email = email
                        if ldap_user.userPrincipalName != userPrincipalName:
                            change = True
                            ldap_user.userPrincipalName = userPrincipalName
                        if int(ldap_user.system_status) != user_status:
                            change = True
                            ldap_user.system_status = user_status
                        if ldap_user.guid != guid:
                            change = True
                            ldap_user.guid = guid
                        if change:
                            # update_user[name] = before_change
                            ldap_user.save()
                            account.save()
                            self.user_update_num += 1
                        else:
                            self.user_clash_num += 1
                    else:
                        # 创建用户
                        account = AccountEntity.objects.create(
                            name=name, cn_name=cn_name, email=email, user_status=user_status,
                            org_info_id=ldap_org.org_id, create_type=1
                        )
                        # ldap用户加到freeipa
                        add_user(account.name, account.cn_name, ldap_info.default_password if not password else password)
                        self.user_add_num += 1
                        ldap_user.account_id = account.id
                        ldap_user.save()
        return

    def auth_user(self, username, password):
        if not self.conn:
            return {'status':False, 'message':'连接LDAP服务器失败,请检查LDAP服务器或管理员密码是否过期'}
        search_filter = '(sAMAccountName={})'.format(username)
        # 定义LDAP查询条件
        res = self.conn.search(search_base=self.base_dn,
                          search_filter=search_filter,
                          search_scope=SUBTREE,
                          attributes=['cn', 'givenName', 'mail', 'sAMAccountName', 'userAccountControl', 'objectGUID'],
                          )
        try:
            if res:
                entry = self.conn.response[0]
                # logger.info(entry)
                dn = entry['dn']
                user_status = entry.get('attributes').get('userAccountControl')  # 用户状态 512 514
                # attr_dict = entry['attributes']
                # logger.info('attr_dic:%s' % attr_dict)

                try:
                    # 这个connect是通过你的用户名和密码还有上面搜到的入口搜索来查询的
                    auth_conn = Connection(self.s, user=dn, password=password, check_names=True, lazy=False,
                                           raise_exceptions=False, read_only=True)
                    auth_conn.bind()
                    # logger.info(auth_conn.result["description"])

                    # 正确-success 不正确-invalidCredentials
                    if auth_conn.result["description"] == "success":  # success or invalidCredentials
                        stu = reset_user_pwd(username, password, o_krbpasswordexpiration=True)
                        if user_status in ('514', '4096', '66050', 514, 4096, 66050):  # 禁用
                            return {'status':False, 'message':'此账号禁用'}
                        if stu == 'notfound':
                            return {'status':False, 'message':'此账号暂未同步'}
                        return {'status':True, 'message':'认证通过'}
                    else:
                        return {'status':False, 'message':'认证失败，用户名或密码错误!'}
                except Exception as e:
                    logger.info(e)
                    return {'status':False, 'message':'认证失败，用户名或密码错误!'}
        except KeyError as e:
            logger.info(e)
            return {'status':False, 'message':'认证失败，用户名或密码错误!'}

    def get_groups(self, ldap_info):
        if not self.conn:
            return
        update_group = {}
        search_filter = '(objectclass=organizationalUnit)'
        attributes = ['ou', 'distinguishedName', 'objectGUID']
        search_base_dn = []
        if self.base_ou:
            for ou in self.base_ou:
                search_base_dn.append('{},{}'.format(ou, self.base_dn))   # 'OU=Rearch Center,DC=cpu-os,DC=com'  # ou=Rearch Center 基础ou
        else:
            search_base_dn.append(self.base_dn )  # 'dc=cpu-os,dc=com'
            if not  LdapOrgnization.objects.filter(dn=self.base_dn,ldap_id=ldap_info.id):
                LdapOrgnization.objects.create(name=self.base_dn, dn=self.base_dn, guid=None, parent_dn=self.base_dn, ldap_id=ldap_info.id)

        for search_base in search_base_dn:
            # 发送搜索请求并获取结果
            self.conn.search(search_base=search_base, search_filter=search_filter, search_scope=SUBTREE, attributes=attributes)
            # 保存群组的DN和名称的映射
            group_dn_map = {
                'DC=cpu-os,DC=com': {
                    'dn': 'DC=cpu-os,DC=com',
                    'parts_dn': '',
                    'ou': ''
                },
            }

            for entry in self.conn.response:
                if entry['type'] != 'searchResEntry':
                    continue
                # logger.info(entry.get('attributes'))
                # {'ou': ['Rearch Center'], 'distinguishedName': 'OU=Rearch Center,DC=cpu-os,DC=com'}
                # {'ou': ['CPU'], 'distinguishedName': 'OU=CPU,OU=Rearch Center,DC=cpu-os,DC=com'}
                # {'ou': ['CPU/OS深度融合研究部/设计部'], 'distinguishedName': 'OU=CPU/OS深度融合研究部/设计部,OU=CPU,OU=Rearch Center,DC=cpu-os,DC=com'}
                name = entry.get('attributes').get('ou')[0]
                dn = entry.get('attributes').get('distinguishedName')
                guid = entry.get('attributes').get('objectGUID')
                parent_dn = dn.split('{},'.format(name))[-1]
                if name == EXCLUDE_GROUP:
                    continue
                self.group_sync_num += 1
                ldap_org = LdapOrgnization.objects.filter(guid=guid,ldap_id=ldap_info.id).first()
                if not ldap_org:
                    LdapOrgnization.objects.create(name=name, dn=dn, guid=guid, parent_dn=parent_dn,ldap_id=ldap_info.id)
                else:
                    if ldap_org.dn != dn:
                        before_change = {
                            'dn':ldap_org.dn,
                            'name':ldap_org.name,
                            'parent_dn':ldap_org.parent_dn,
                            'ldap_id':ldap_org.ldap_id,
                        }
                        ldap_org.dn = dn
                        ldap_org.name = name
                        ldap_org.parent_dn = parent_dn
                        ldap_org.save()
                        update_group[guid] = before_change

        def add_ldap_group_to_org(ldap_groups):
            all = []
            for group in ldap_groups:
                parent = LdapOrgnization.objects.filter(dn=group.parent_dn).exclude(org_id=None).first()
                if not parent:
                    all.append(group)
                    continue
                if group.guid in update_group:
                    g = OrgnizationEntity.objects.filter(id=group.org_id).first()
                    if g:
                        g.name = group.name
                        g.save()
                        self.group_update_num += 1
                        continue
                # g = OrgnizationEntity.objects.filter(id=group.org_id).first()
                if not group.org_id:
                    g = OrgnizationEntity.objects.create(name=group.name, parent_id=parent.org_id, create_type=1,
                                                         passwd_extra_status=1)
                    group.org_id = g.id
                    group.save()
                    self.group_add_num += 1
                else:
                    g = OrgnizationEntity.objects.filter(id=group.org_id).first()
                    g.parent_id = parent.org_id
                    g.save()
                    self.group_clash_num += 1
            if len(all) != 0:
                # add_ldap_group_to_org(all, sync_num, add_num, update_num, fail_num, clash_num)
                add_ldap_group_to_org(all)
            return

        ldap_group = []
        base_group = LdapOrgnization.objects.filter(ldap_id=ldap_info.id, dn__in=search_base_dn).all()
        if ldap_info.point:
            point_org = OrgnizationEntity.objects.filter(id=ldap_info.point).first()
        else:
            point_org = OrgnizationEntity.objects.filter(id=1).first()
        for group in base_group:
            g = OrgnizationEntity.objects.filter(id=group.org_id).first()
            # g = OrgnizationEntity.objects.filter(name=group.name, parent_id=point_org.id)
            if not g:
                g = OrgnizationEntity.objects.create(name=group.name, parent_id=point_org.id, create_type=1,
                                                     passwd_extra_status=1)
                group.org_id = g.id
                group.save()
                self.group_add_num += 1
            else:
                g.parent_id = point_org.id
                g.save()
                self.group_clash_num += 1

            ldap_group.extend(LdapOrgnization.objects.filter(ldap_id=ldap_info.id, dn__endswith=group.dn).exclude(
                dn=group.dn).all())
        ldap_group = [i for i in ldap_group]
        return add_ldap_group_to_org(ldap_group)

    def sync_group_user(self):
        ldap_info = Ldap.objects.filter(base_dn=self.base_dn).first()
        ldap_sync_status = LdapSyncStatus.objects.create(ldap=ldap_info)
        self.get_groups(ldap_info)
        self.group_fail_num = self.group_sync_num - (self.group_add_num + self.group_update_num + self.group_clash_num)
        ldap_sync_status.add_group = self.group_add_num
        ldap_sync_status.update_group = self.group_update_num
        ldap_sync_status.fail_group = self.group_fail_num if self.group_fail_num > 0 else 0
        ldap_sync_status.clash_group = self.group_clash_num

        self.get_users(ldap_info)
        self.user_fail_num = self.user_sync_num - (self.user_add_num + self.user_update_num + self.user_clash_num)
        ldap_sync_status.add_user = self.user_add_num
        ldap_sync_status.update_user = self.user_update_num
        ldap_sync_status.fail_user = self.user_fail_num if self.user_fail_num > 0 else 0
        ldap_sync_status.clash_user = self.user_clash_num

        if self.user_fail_num > 0:
            ldap_sync_status.sync_status = 2
            ldap_sync_status.save()
            return False
        ldap_sync_status.sync_status = 1
        ldap_sync_status.save()
        return True

def get_ldap_ad():
    ldap_obj = Ldap.objects.exclude(id=1).last()
    if not ldap_obj:
        return False
    base_ou = [ou.base_ou for ou in ldap_obj.ldapbaseou_set.all()]
    ldap = LdapAD(
        ldap_obj.host, ldap_obj.port, ldap_obj.admin_user_dn, ldap_obj.admin_password,
        ldap_obj.base_dn, base_ou
    )
    return ldap