# -*- coding: utf-8 -*-
import json
import re
import logging
from django.conf import settings
from django.core.cache import caches
from django_redis import get_redis_connection

from rest_framework import serializers
from accounts.models import AccountEntity, PasswordAccountRule, Ldap, \
    LdapBaseOU, LdapSyncStatus, LocalUserLogin, PasswordAccountProhibitModification, \
    AuthSource, AuthRecords, BlackWhiteLists, BlackWhiteListsRule, BackupRestore, DNSRecords
from organizations.models import OrgnizationEntity, LdapOrgnization

logger = logging.getLogger(__name__)

conn = get_redis_connection('redis')


class AccountEntitySerializer(serializers.ModelSerializer):
    class Meta:
        model = AccountEntity
        fields = ('id', 'cn_name', 'name', 'phone', 'user_status', 'address', 'remarks', 'office_address')


class AccountSerializer(serializers.ModelSerializer):
    uid = serializers.SerializerMethodField()
    is_active = serializers.SerializerMethodField()
    org_info_name = serializers.SerializerMethodField()

    class Meta:
        model = AccountEntity
        fields = ('uid', 'cn_name', 'name', 'phone', 'user_status', 'address', 'remarks', 'office_address', 'org_info',
                  'org_info_name', 'is_active')

    def get_uid(self, obj):
        return obj.id

    def get_org_info_name(self, obj):
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=obj.org_info_id).first()
        if items:
            return items.name
        else:
            return None

    def get_is_active(self, obj):
        device_user_active_dict_str = conn.hgetall("device_user_active")
        active_user_list = []
        if device_user_active_dict_str:
            for k, v in device_user_active_dict_str.items():
                if v:
                    active_user_list += json.loads(v.decode('utf-8'))
        if obj.name in active_user_list:
            return 1
        else:
            return 0


class BindAccountSerializer(serializers.ModelSerializer):
    org_info_name = serializers.SerializerMethodField()
    uid = serializers.SerializerMethodField()
    is_active = serializers.SerializerMethodField()

    class Meta:
        model = AccountEntity
        fields = ('uid', 'cn_name', 'name', 'user_status', 'org_info_name', 'is_active')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.active_user_list = self.active_account()

    @staticmethod
    def active_account():
        device_user_active_dict_str = conn.hgetall("device_user_active")
        active_user_list = []
        if device_user_active_dict_str:
            for k, v in device_user_active_dict_str.items():
                if v:
                    active_user_list += json.loads(v.decode('utf-8'))
        return active_user_list

    def get_is_active(self, obj):
        if obj.name in self.active_user_list:
            return 1
        else:
            return 0

    def get_uid(self, obj):
        return obj.id

    def get_org_info_name(self, obj):
        org_info_items = OrgnizationEntity.objects.all()
        if org_info_items.filter(id=obj.org_info_id).exists():
            org_info_name = org_info_items.get(id=obj.org_info_id).name
        else:
            org_info_name = None

        return org_info_name


class PasswordAccountRuleSerializer(serializers.ModelSerializer):
    """
    权限序列化
    """

    class Meta:
        model = PasswordAccountRule
        fields = ('id', 'accounts_check_status', 'account_mix_length', 'is_lower', 'is_upper', 'is_num', 'is_char',
                  'is_offline_login', 'offline_cache_days')


class LdapSerializer(serializers.ModelSerializer):
    point_name = serializers.SerializerMethodField()

    def get_point_name(self, obj):
        if obj.point:
            org = OrgnizationEntity.objects.filter(id=obj.point).first()
        else:
            org = OrgnizationEntity.objects.filter(id=1).first()
        return org.name

    ssl_file_name = serializers.SerializerMethodField()

    def get_ssl_file_name(self, obj):
        if obj.ssl_file:
            return obj.ssl_file.rsplit('/')[-1]
        return ''

    base_ou = serializers.SerializerMethodField()

    def get_base_ou(self, obj):
        base_ou_list = []
        queryset = LdapBaseOU.objects.filter(ldap_id=obj.id).all()
        if queryset:
            for i in queryset:
                base_ou_list.append(i.base_ou)
        return base_ou_list

    sync_status = serializers.SerializerMethodField()

    def get_sync_status(self, obj):
        sync_status = obj.ldapsyncstatus_set.order_by('-sync_at').first()
        if sync_status:
            return sync_status.get_sync_status_display()
        return ''

    ldap_type = serializers.SerializerMethodField()

    def get_ldap_type(self, obj):
        return obj.get_ldap_type_display()

    last_sync_at = serializers.SerializerMethodField()

    def get_last_sync_at(self, obj):
        sync_status = obj.ldapsyncstatus_set.order_by('-sync_at').first()
        if sync_status:
            return sync_status.sync_at.strftime("%Y-%m-%d %H:%M:%S")
        return ''

    class Meta:
        model = Ldap
        fields = (
            'id', 'name', 'host', 'port', 'base_dn', 'base_ou', 'admin_user_dn', 'admin_password', 'ssl',
            'default_password', 'ssl_file',
            'point',
            'point_name',
            'ssl_file_name',
            'sync_status',
            'created_at',
            'last_update_at',
            'last_sync_at',
            'ldap_type',
            'last_sync_at',
        )

    # def validate_default_password(self, value):
    #     if not value or value == '':
    #         value = 'Aa12345678'
    #     return value

    def validate_name(self, value):
        if re.findall(r"[~!@#$%^&*()\-_=+{}|\[\]\\:\";\'<>?,./]+", value):
            raise serializers.ValidationError('不允许特殊字符')
        # if Ldap.objects.filter(name=value).first():
        #     raise serializers.ValidationError(value + ' 已经存在')
        return value

    def validate_host(self, value):
        if re.findall(r"[~!@#$%^&*()\-_=+{}|\[\]\\\";\'<>?,]+", value):
            raise serializers.ValidationError('不允许特殊字符')
        # if Ldap.objects.filter(host=value).first():
        #     raise serializers.ValidationError(value + ' 已经存在')
        return value

    def validate_base_dn(self, value):
        if re.findall(r"[~!@#$%^&*()+{}|\[\]\\\";\'<>?./]+", value):
            raise serializers.ValidationError('不允许特殊字符')
        if len(value.split(',')) != 2:
            raise serializers.ValidationError(' 基准DN不规范，案例：DC=example,DC=com')
        value = re.sub(r"(dc=)([^\s,]+)", lambda m: m.group(1).lower() + m.group(2).lower(), value)
        value = re.sub(r"dc", "DC", value)
        return value

    # def validate_base_ou(self, value):
    #     if re.findall(r"[~!@#$%^&*()+{}|\[\]\\\";\'<>?./]+", value):
    #         raise serializers.ValidationError('不允许特殊字符')
    #     return value

    def save_base_ou(self, instance, base_ou):
        if base_ou == []:
            LdapBaseOU.objects.filter(ldap_id=instance).all().delete()
            return
        LdapBaseOU.objects.filter(ldap_id=instance).all().delete()
        for ou in base_ou:
            ou = re.sub(r"ou", "OU", ou)
            LdapBaseOU.objects.create(base_ou=ou, ldap=instance)

    def validate(self, attrs):
        ssl = attrs.get('ssl')
        ssl_file = attrs.get('ssl_file')
        if ssl == 1:
            if not ssl_file or ssl_file == '':
                ldap_obj = Ldap.objects.filter(name=attrs.get('name')).first()
                if ldap_obj:
                    if ldap_obj.ssl_file:
                        return attrs
                raise serializers.ValidationError({'ssl_file': '缺少ssl证书'})
        base_ou = self.context.get('request').data.get('base_ou')
        for value in base_ou:
            if re.findall(r"[~!@#$%^&*()+{}|\[\]\\\";\'<>?./]+", value):
                raise serializers.ValidationError({'base_ou': '不允许特殊字符'})
        attrs['base_ou'] = base_ou
        return attrs

    def update(self, instance, validated_data):
        # if validated_data.get('base_dn'):
        #     validated_data.pop('base_dn')  # base_dn 不可修改
        # if validated_data.get('point'):
        #     if instance.point != validated_data.get('point'):
        #         ldap_base_ou = instance.ldapbaseou_set.all()
        #         if ldap_base_ou:
        #             for base_ou in ldap_base_ou:
        #                 ldap_org = LdapOrgnization.objects.filter(dn=base_ou.base_ou + ',' + instance.base_dn).first()
        #                 org = OrgnizationEntity.objects.filter(id=ldap_org.org_id).first()
        #                 if org:
        #                     org.parent_id = validated_data.get('point')
        #                     org.save()
        #         else:
        #             ldap_org = LdapOrgnization.objects.filter(dn=instance.base_dn).first()
        #             org = OrgnizationEntity.objects.filter(id=ldap_org.org_id).first()
        #             if org:
        #                 org.parent_id = validated_data.get('point')
        #                 org.save()
        if not validated_data.get('default_password') or validated_data.get('default_password') == '':
            validated_data['default_password'] = 'Aa12345678'
        if instance.name != validated_data.get('name'):
            if Ldap.objects.filter(name=validated_data.get('name')).first():
                raise serializers.ValidationError(validated_data.get('name') + ' 已经存在')
        if instance.host != validated_data.get('host'):
            if Ldap.objects.filter(host=validated_data.get('host')).first():
                raise serializers.ValidationError(validated_data.get('host') + ' 已经存在')
        if instance.base_dn != validated_data.get('base_dn'):
            if Ldap.objects.filter(base_dn=validated_data.get('base_dn')).first():
                raise serializers.ValidationError(validated_data.get('base_dn') + ' 已经存在')
        base_ou = validated_data.pop('base_ou')
        self.save_base_ou(instance, base_ou)
        super(LdapSerializer, self).update(instance, validated_data)
        return instance

    def create(self, validated_data):
        if Ldap.objects.filter(name=validated_data.get('name')).first():
            raise serializers.ValidationError(validated_data.get('name') + ' 已经存在')
        if Ldap.objects.filter(host=validated_data.get('host')).first():
            raise serializers.ValidationError(validated_data.get('host') + ' 已经存在')
        if Ldap.objects.filter(base_dn=validated_data.get('base_dn')).first():
            raise serializers.ValidationError(validated_data.get('base_dn') + ' 已经存在')
        if not validated_data.get('default_password') or validated_data.get('default_password') == '':
            validated_data['default_password'] = 'Aa12345678'
        base_ou = validated_data.pop('base_ou')
        instance = super(LdapSerializer, self).create(validated_data)
        self.save_base_ou(instance, base_ou)
        return instance


class LdapSyncStatusSerializer(serializers.ModelSerializer):
    sync_status = serializers.SerializerMethodField()

    def get_sync_status(self, obj):
        return obj.get_sync_status_display()

    class Meta:
        model = LdapSyncStatus
        fields = (
            'id',
            'add_user', 'update_user',
            'add_user', 'add_group',
            'update_user', 'update_group',
            'clash_user', 'clash_group',
            'fail_user', 'fail_group',
            'sync_at', 'sync_status'
        )


class LocalUserLoginSerializer(serializers.ModelSerializer):
    class Meta:
        model = LocalUserLogin
        fields = '__all__'


class PwdProhibitModificationSerializer(serializers.ModelSerializer):
    class Meta:
        model = PasswordAccountProhibitModification
        fields = ('id', 'status', 'range', 'org_id', 'account_id')


class AuthSourceSerializer(serializers.ModelSerializer):
    update_time = serializers.SerializerMethodField()
    update_status = serializers.SerializerMethodField()

    class Meta:
        model = AuthSource
        fields = ('id', 'name', 'addr', 'sync', 'sync_at', 'point', 'created_at', 'update_time', 'update_status')

    def get_update_time(self, obj):
        records_items = AuthRecords.objects.all()
        if records_items.count() == 0:
            return None
        else:
            return records_items.order_by('-sync_at').first().sync_at.strftime("%Y-%m-%d %H:%M:%S")

    def get_update_status(self, obj):
        records_items = AuthRecords.objects.all()
        if records_items.count() == 0:
            return 0
        else:
            return records_items.order_by('-sync_at').first().sync_status


class AuthRecordsSerializer(serializers.ModelSerializer):
    class Meta:
        model = AuthRecords
        fields = '__all__'


class BlackWhiteListsSerializer(serializers.ModelSerializer):
    class Meta:
        model = BlackWhiteLists
        fields = '__all__'


class BlackWhiteListsRuleSerializer(serializers.ModelSerializer):
    class Meta:
        model = BlackWhiteListsRule
        fields = '__all__'


class BackupRestoreSerializer(serializers.ModelSerializer):
    class Meta:
        model = BackupRestore
        fields = '__all__'


class DNSRecordsSerializer(serializers.ModelSerializer):
    class Meta:
        model = DNSRecords
        fields = '__all__'