# -*- coding: utf-8 -*-
import logging
import os
import json
import datetime
import pandas as pd

from celery_task.celery import app
from menumanager.models import MenuInfo, MenuSubAction
from adminlog.models import AdminLog
from users.models import User
from utils.utils import get_uuid
from utils.client_log_helper import ClientLogHelper
from clientlog.models import ClientLogAction, ClientLogModule, ClientLog
from nfs_domain_mainserver.settings import SALTFILESYSTEMPATH
from devices.web_consumers import send_msg_to_web
from message_center.models import MessageCenter


logger = logging.getLogger(__name__)


@app.task(queue='adminlog')
def admin_log_add(request_dict, response_dict):
    """
    celery异步任务:管理员日志处理,分析request和response,将日志记录到数据库中
    request: 不能直接传ASGIRequest对象,response同理
    """
    try:
        # 获取属性
        admin_name = request_dict['request_user']
        admin_name = str(admin_name).strip()
        request_method = request_dict['request_method']
        request_path = request_dict['request_path']
        request_path = str(request_path).strip()
        request_body = request_dict['request_body']
        request_query_params= request_dict['request_meta_data'].get('QUERY_STRING','')
        request_id = request_dict['request_meta_data'].get('HTTP_REQUESTID', '')
        request_is_file = request_dict['request_is_file']
        request_file = request_dict['request_file']
        request_content_type = request_dict['request_content_type']
        # response
        response_status_code = response_dict['response_status_code']
        response_content = response_dict['response_content']

        if admin_name is None:
            return
        #################### AnonymousUser的处理 ######################
        is_allow = False
        if admin_name == 'AnonymousUser':
            # 平台管理员登录时,admin_name是匿名的,需要从参数中获取
            if request_path.endswith('/mainapi/user/login/'):
                try:
                    request_body_json = json.loads(request_body)
                    if 'username' in request_body_json:
                        admin_name = request_body_json.get('username')
                except Exception as exp:
                    logger.error(f"platform admin login error:{exp}")
            # 客户端的用户(不管是域用户还是系统用户,登录时都会调用此接口,故在这里记录客户端登录日志)
            elif request_path.endswith('/mainapi/device/login/'):
                try:
                    request_body_json = json.loads(request_body)
                    device_uuid = request_body_json.get('device_uuid')
                    domain_user = request_body_json.get('domain_user')
                    client_log = ClientLog(
                        device_uuid=device_uuid,
                        account_name=domain_user,
                        module=ClientLogModule.ACCOUNT.value,
                        action=ClientLogAction.ACCOUNT_LOGIN.value,
                        result_status=1, #登录成功
                        operate_uuid=get_uuid(),
                        request=request_body,
                        response=json.dumps({'msg':'pong'})
                    )
                    client_log_helper = ClientLogHelper()
                    client_log_helper.add(client_log=client_log)
                except Exception as exp:
                    logger.error(f"client user login error:{exp}")
            # 如果是文件上传,允许放行,当普通的POST处理即可
            elif request_is_file and request_path.startswith('/mainapi/patch/patch_file/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/software/deb/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/account/batch/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/account/batch/ldap/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/account/ldap/ssl_file/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/ca/file/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/filemanager/file/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/org/batch/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/org/batch/ldap/'):
                is_allow = True
            elif request_is_file and request_path.startswith('/mainapi/org/batch/ldap/'):
                is_allow = True
        #################### AnonymousUser的处理 ######################
        # prometheus的访问不处理
        if request_path.endswith('/metrics'):
            return
        # GET请求暂不记录到数据库
        if request_method == 'GET':
            return
        # 获取管理员
        admin = User.objects.filter(username=admin_name).first()
        if not admin and not is_allow:
            return

        # 判断是否是文件上传
        if request_is_file:
            # 文件存在
            request_body = json.dumps(request_file)

        # 获取菜单的module和action
        module_action = _get_module_action_(request_dict)

        # 请求对象request处理
        module = module_action['module']
        action = module_action['action']
        #module_val = module_action['module_val']
        action_val = module_action['action_val']
        desc = f"管理员{admin_name}{action_val}"
        req_json = {}
        req_json["method"] = request_method
        req_json["path"] = request_path
        req_json["body"] = request_body
        req_json["query_params"] = request_query_params
        req_json['request_id'] = request_id
        
        # 返回对象response处理
        action_is_success = False
        status_code = 0
        content = ''
        if response_status_code > 0:
            status_code = response_status_code
        # 捕获返回内容,并解析里面的处理状态
        if response_content:
            try:
                # 如果是文件、流、图片
                content_type_list = ['application/ms-excel', 'application/octet-stream', 'image/png', 'image/jpg', 'image/jpeg']
                if request_content_type in content_type_list:
                    action_is_success = True
                else:
                    content = response_content
                    content_json = json.loads(content)
                    # 从response的返回对象中查询code,如果没有code就用status_code判断
                    if 'code' in content_json:
                        if int(content_json['code']) == 200 or int(content_json['code']) == 201 or int(content_json['code']) == 2000:
                            action_is_success = True
                    elif status_code == 200 or status_code == 201:
                        action_is_success = True
            except Exception as e1:
                logger.error(f"解析response结果异常:{e1}, content:{response_content}")
                # 如果response不是json格式,如返回了'404 Not Found'或'500'或nginx页面,都以失败处理
        else:
            # 如果返回对象是空的,就以response.status_code为准
            if status_code == 200 or status_code == 201:
                action_is_success = True

        resp_json = {
            "status_code":  status_code,
            "content": content,
        }

        # 将日志记录到数据库
        # 操作时间不要微妙
        now = datetime.datetime.now()
        operation_time = now.replace(microsecond=0)
        AdminLog.objects.create(
            admin_name=admin_name,
            admin_nickname=admin.nickname if admin else '',
            module=module,
            action=action,
            result_status=action_is_success,
            desc=desc,
            request=json.dumps(req_json),
            response=json.dumps(resp_json),
            operation_time=operation_time
        )
    except Exception as exp:
        # 业务抛出了异常,也是需要处理的
        logger.error("管理员操作日志异常: %s" % exp)


def _get_module_action_(request_dict):
    """
    根据request和数据库表menu_info获取到操作module和action
    """
    module_action = {
        'module': '',
        'module_val': '',
        'action': '',
        'action_val': ''
    }
    request_method = request_dict['request_method']
    request_path = request_dict['request_path']
    request_body = request_dict['request_body']
    # 
    request_path = str(request_path).strip()
    request_method = str(request_method).strip()

    # 判断数据库引擎,根据不同的数据库SQL语句不同
    db_engine = os.environ.get("DB_BACKEND")
    # 默认mysql
    db_regexp = " %s REGEXP `url_match` "
    # 如果是人大金仓数据库
    if db_engine == 'kingbase':
        db_regexp = " %s ~ url_match "

    # 从菜单表中获取requsest.path对应的菜单信息
    menu_info = MenuInfo.objects.extra(
                where=[db_regexp], # 顺序不能乱,值要在前面
                params=[request_path]
            ).first()
    if not menu_info:
        return module_action
    
    # 获取菜单的基本action
    menu_action = menu_info.action
    sub_action_str = f"{menu_info.action}_{request_method.lower()}"        
    sub_action_queryset = MenuSubAction.objects.filter(sub_action=sub_action_str).first()
    sub_action_desc_str = None
    
    # 生成子行为和子描述
    method_desc_str = None
    if request_method == 'GET':
        method_desc_str = "查询"
    elif request_method == 'POST':
        method_desc_str = "添加"
    elif request_method == 'PUT':
        method_desc_str = "修改"
    elif request_method == 'DELETE':
        method_desc_str = "删除"
    else:
        method_desc_str = ''
    # 默认值 = [方法描述:父行为描述]
    sub_action_desc_str = f"{method_desc_str}:{menu_info.desc}"
    # 如果子行为中存在,就用子行为的描述
    if sub_action_queryset:
        sub_action_desc_str = sub_action_queryset.sub_action_desc

    # 根据请求参数分解出子行为
    # 特殊处理1:域账号批量处理
    if menu_action == 'account_info_account_batch_operation' and request_body is not None:
        request_body_json = json.loads(request_body)
        # 如果是删除
        if 'is_delete' in request_body_json:
            if request_body_json.get('is_delete') == 1:
                sub_action_str = 'account_info_account_batch_operation_delete'
                sub_action_desc_str = f"删除域账号"
        # 如果是重置密码
        elif 'is_rest_passwd' in request_body_json:
            if request_body_json.get('is_rest_passwd') == 1:
                sub_action_str = 'account_info_account_batch_operation_rest_passwd'
                sub_action_desc_str = f"重置域账号密码"
        # 如果是改变状态
        elif 'is_status' in request_body_json:
            if request_body_json.get('is_status') == 0:
                sub_action_str = 'account_info_account_batch_operation_enable'
                sub_action_desc_str = f"启用域账号"
            elif request_body_json.get('is_status') == 1:
                sub_action_str = 'account_info_account_batch_operation_disable'
                sub_action_desc_str = f"禁用域账号"
            elif request_body_json.get('is_status') == 2:
                sub_action_str = 'account_info_account_batch_operation_unlock'
                sub_action_desc_str = f"解锁域账号"
        # 如果是调整部门
        elif 'to_organ_id' in request_body_json:
            sub_action_str = 'account_info_account_batch_operation_org_put'
            sub_action_desc_str = f"调整域账号部门"
        # 如果是导出
        elif 'is_down_file' in request_body_json:
            if request_body_json.get('is_down_file') == 1:
                sub_action_str = 'account_info_account_batch_operation_down_file'
                sub_action_desc_str = f"导出域账号"
    # 特殊处理2:部门批量处理
    elif menu_action == 'org_info_organ_batch_operation' and request_body is not None:
        request_body_json = json.loads(request_body)
        # 如果是删除
        if 'is_delete' in request_body_json:
            if request_body_json.get('is_delete') == 1:
                sub_action_str = 'org_info_organ_batch_operation_delete'
                sub_action_desc_str = f"删除部门"
        # 如果是导出
        elif 'is_down_file' in request_body_json:
            if request_body_json.get('is_down_file') == 1:
                sub_action_str = 'org_info_organ_batch_operation_down_file'
                sub_action_desc_str = f"导出部门"
    # 特殊处理3:补丁批量处理
    elif menu_action == 'patch_policy_batch_operation' and request_body is not None:
        request_body_json = json.loads(request_body)
        # 如果是删除
        if 'is_delete' in request_body_json:
            if request_body_json.get('is_delete') == 1:
                sub_action_str = 'patch_policy_batch_operation_delete'
                sub_action_desc_str = f"删除补丁"
        elif 'is_status' in request_body_json:
        # 如果是启动
            if request_body_json.get('is_status') == 1:
                    sub_action_str = 'patch_policy_batch_operation_enable'
                    sub_action_desc_str = f"启用补丁"
            # 如果是启动
            elif request_body_json.get('is_status') == 0:
                    sub_action_str = 'patch_policy_batch_operation_disable'
                    sub_action_desc_str = f"禁用补丁"
    # 特殊处理4:终端批量处理
    elif menu_action == 'device_info_device_switch' and request_body is not None:
        request_body_json = json.loads(request_body)
        # 如果是删除
        if 'is_delete' in request_body_json:
            if request_body_json.get('is_delete') == 1:
                sub_action_str = 'device_info_device_switch_delete'
                sub_action_desc_str = f"删除终端"
        # 如果是改变状态
        elif 'is_up' in request_body_json:
            if request_body_json.get('is_up') == 'true':
                sub_action_str = 'device_info_device_switch_enable'
                sub_action_desc_str = f"启用终端"
            elif request_body_json.get('is_up') == 'false':
                sub_action_str = 'device_info_device_switch_disable'
                sub_action_desc_str = f"禁用终端"
        # 如果是改变状态
        elif 'is_power' in request_body_json:
            if request_body_json.get('is_power') == 0:
                sub_action_str = 'device_info_device_switch_power_shutdown'
                sub_action_desc_str = f"终端关机"
            elif request_body_json.get('is_power') == 1:
                sub_action_str = 'device_info_device_switch_power_reboot'
                sub_action_desc_str = f"终端重启"
        # 如果是调整部门
        elif 'to_organ_id' in request_body_json:
            sub_action_str = 'device_info_device_switch_org_put'
            sub_action_desc_str = f"调整终端部门"
        # 如果是导出
        elif 'is_down_file' in request_body_json:
            if request_body_json.get('is_down_file') == 1:
                sub_action_str = 'device_info_device_switch_down_file'
                sub_action_desc_str = f"导出终端"
    # 特殊处理5:软件策略批量处理
    elif menu_action == 'software_policy_batch_operation' and request_body is not None:
        request_body_json = json.loads(request_body)
        # 如果是删除
        if 'is_delete' in request_body_json:
            if request_body_json.get('is_delete') == 1:
                sub_action_str = 'software_policy_batch_operation_delete'
                sub_action_desc_str = f"删除软件策略"
        elif 'is_status' in request_body_json:
        # 如果是启动
            if request_body_json.get('is_status') == 1:
                    sub_action_str = 'software_policy_batch_operation_enable'
                    sub_action_desc_str = f"启用软件策略"
            # 如果是启动
            elif request_body_json.get('is_status') == 0:
                    sub_action_str = 'software_policy_batch_operation_disable'
                    sub_action_desc_str = f"禁用软件策略"
    # 特殊处理6:补丁批量处理
    elif menu_action == 'patch_info_patch_2_xlsx' and request_body is not None:
        request_body_json = json.loads(request_body)
        # 如果是删除
        if 'is_delete' in request_body_json:
            if request_body_json.get('is_delete') == 1:
                sub_action_str = 'patch_info_patch_2_xlsx_delete'
                sub_action_desc_str = f"删除补丁"
        # 如果是下载
        if 'is_down_file' in request_body_json:
            if request_body_json.get('is_down_file') == 1:
                sub_action_str = 'patch_info_patch_2_xlsx_down_file'
                sub_action_desc_str = f"导出补丁"

    # 如果不存在就新增
    if not sub_action_queryset:
        # 插入数据
        MenuSubAction.objects.create(
            action=menu_action,
            sub_action=sub_action_str,
            sub_action_desc=sub_action_desc_str
        )
        # 返回对象封装
        module_action['module'] = menu_info.module
        module_action['module_val'] = menu_info.desc
        module_action['action'] = sub_action_str
        module_action['action_val'] = sub_action_desc_str
    else:
        # 返回对象封装
        module_action['module'] = menu_info.module
        module_action['module_val'] = menu_info.desc
        module_action['action'] = sub_action_str
        module_action['action_val'] = sub_action_desc_str

    return module_action


@app.task(queue='default')
def export_adminlog_data(user_name, data, operate_start_time, operate_end_time, log_file_name):
    """
    导出数据,并发送消息
    """
    try:
        # 文件名
        now = datetime.datetime.now()
        timestamp_str = now.strftime('%Y%m%d%H%M%S')
        filename = f'nfs-domain_{log_file_name}日志_{timestamp_str}.xlsx'
        # 如果开始时间存在,则文件名重构为:'{operate_start_time}--{operate_end_time}_{log_content_desc}日志_{timestamp_str}.xlsx'
        if operate_start_time:
            # 格式化为datetime
            operate_start_time_format = datetime.datetime.strptime(operate_start_time, '%Y-%m-%d %H:%M:%S')
            # 如果结束时间没有,则默认今天
            if not operate_end_time:
                operate_end_time = datetime.datetime.strptime(now.strftime('%Y-%m-%d %H:%M:%S'), '%Y-%m-%d %H:%M:%S')
            operate_end_time_format = datetime.datetime.strptime(operate_end_time, '%Y-%m-%d %H:%M:%S')
            # 只要年-月-日
            filename = f'{operate_start_time_format.date()}-{operate_end_time_format.date()}_{log_file_name}日志_{timestamp_str}.xlsx'

        # 将数据转换为DataFrame
        df = pd.DataFrame(data)
        df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"{log_file_name}日志导出完成, {filename}, 总共{len(data)}条数据",
            'create_time': now,
            'target_module': f"{log_file_name}",
            'message_user': user_name,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, user_name)
    except Exception as e:
        logger.error(f"export_clienglog_data error:{e}")