# -*- coding: utf-8 -*-
import logging
import json
import datetime

from rest_framework.views import Response
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.pagination import PageNumberPagination
from rest_framework.viewsets import ModelViewSet
from django.db.models import Q
from django.utils.dateparse import parse_datetime

from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.permission import IsAdminUser
from utils.response import FitJSONRenderer
from utils.throttling import UserRateThrottle

from adminlog.models import AdminLog
from adminlog.serializers import AdminLogSerializer
from menumanager.models import MenuInfo, MenuSubAction
from users.models import User
from adminlog.tasks import export_adminlog_data

logger = logging.getLogger(__name__)

class AdminLogSearchViewSet(ModelViewSet):
    '''
    管理员操作日志查询
    '''
    serializer_class = AdminLogSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    
    # 当前时间
    now_time = datetime.datetime.now()
    # 今天 0 点
    zero_today = now_time - datetime.timedelta(hours=now_time.hour, minutes=now_time.minute, seconds=now_time.second, microseconds=now_time.microsecond)
        # 今天 23:59:59
    last_today = zero_today + datetime.timedelta(hours=23, minutes=59, seconds=59)
    # 昨天 23:59:59
    last_yerterday = last_today - datetime.timedelta(days=1)

    def __get_actions_by_module(self, module):
        """
        内部方法:根据module获取module下的所有子行为
        """
        module_action_list = []
        module_action = MenuInfo.objects.filter(module=module)
        for item in module_action:
            module_action_list.append(item.action)
        sub_action_list = MenuSubAction.objects.filter(action__in=module_action_list).order_by('sub_action_desc')
        res_list = []
        for item in sub_action_list:
            item_info = {
                'action': item.sub_action,
                'action_desc': item.sub_action_desc
            }
            res_list.append(item_info)
        #res_list = sorted(res_list, key=lambda x: x['action_desc'])
        return res_list

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def actions(self, request, *args, **kwargs):
        '''
        根据模块获取模块下的子行为列表
        '''
        module_str = request.query_params.get('module')
        res_list = self.__get_actions_by_module(module=module_str)
        
        return Response(data=res_list, status=status.HTTP_200_OK)
    
    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def admins(self, request, *args, **kwargs):
        '''
        获取所有管理员的账号列表
        '''
        admin_list = []
        admin_queryset = User.objects.all()
        for item in admin_queryset:
            admin_item = {
                'username': item.username,
                'nickname': item.nickname,
            }
            admin_list.append(admin_item)
        return Response(data=admin_list, status=status.HTTP_200_OK)
    

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def login(self, request, *args, **kwargs):
        '''
        登录登出日志(登录登出比较特殊,它是属于用户模块,但是只关注登录和登出)
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')

        queryset = AdminLog.objects.filter(module='user', action__in=('user_login_post', 'user_logout_post'))
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except Exception as exp :
            logger.error(f"error={exp}")
            return Response("error", status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    # 密码的特殊处理
                    if request_body_json is not None and 'password' in request_body_json:
                        request_body_json['password'] = '******'
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': '登录' if item.action == 'user_login_post'  else '登出',
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def org(self, request, *args, **kwargs):
        '''
        组织机构日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='org')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='org', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)
    

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def account(self, request, *args, **kwargs):
        '''
        终端用户日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        desc = request.query_params.get('desc')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='account')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='account', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            #
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    # 密码的特殊处理
                    if request_body_json is not None and 'password' in request_body_json:
                        request_body_json['password'] = '******'
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def device(self, request, *args, **kwargs):
        '''
        终端日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        desc = request.query_params.get('desc')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='device')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='device', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            #
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            # 尝试解析request里面的body字段和response里面的content字段
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def software(self, request, *args, **kwargs):
        '''
        软件日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        desc = request.query_params.get('desc')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='software')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='software', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            #
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            # 尝试解析request里面的body字段和response里面的content字段
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def patch(self, request, *args, **kwargs):
        '''
        补丁日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        desc = request.query_params.get('desc')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='patch')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='patch', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            #
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            # 尝试解析request里面的body字段和response里面的content字段
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def tasks(self, request, *args, **kwargs):
        '''
        任务日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        desc = request.query_params.get('desc')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='tasks')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='tasks', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            #
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            # 尝试解析request里面的body字段和response里面的content字段
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def logs(self, request, *args, **kwargs):
        '''
        日志的日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        desc = request.query_params.get('desc')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='logs')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='logs', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            #
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            # 尝试解析request里面的body字段和response里面的content字段
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    # 密码的特殊处理
                    if request_body_json is not None and 'password' in request_body_json:
                        request_body_json['password'] = '******'
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def user(self, request, *args, **kwargs):
        '''
        平台管理员日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        desc = request.query_params.get('desc')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='user')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='user', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            #
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            # 尝试解析request里面的body字段和response里面的content字段
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    # 密码的特殊处理
                    if request_body_json is not None and 'password' in request_body_json:
                        request_body_json['password'] = '******'
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def salt(self, request, *args, **kwargs):
        '''
        策略日志查询
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        desc = request.query_params.get('desc')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='salt')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='salt', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            #
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == item.action:
                    action_desc = item2['action_desc']
                    break
            # 尝试解析request里面的body字段和response里面的content字段
            request_json = {}
            response_json = {}
            # 尝试解析request和request里面的body字段
            try:
                request_json = json.loads(item.request)
                if 'body' in request_json and request_json['body'] is not None:
                    request_body_json = json.loads(request_json['body'])
                    request_json['body'] = request_body_json
            except:
                pass
            # 尝试解析response和response里面的content字段
            try:
                response_json = json.loads(item.response)
                if 'content' in response_json and response_json['content'] is not None:
                    response_content_json = json.loads(response_json['content'])
                    response_json['content'] = response_content_json
            except:
                pass
            item_info = {
                'id': item.id,
                'admin_name': item.admin_name,
                'admin_nickname': item.admin_nickname,
                'action': item.action,
                'action_desc': action_desc,
                'result_status': item.result_status,
                'result_status_val': '成功' if item.result_status == 1  else '失败',
                'desc': item.desc,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_login(self, request, *args, **kwargs):
        '''
        日志导出:用户登录登出
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')

        queryset = AdminLog.objects.filter(module='user', action__in=('user_login_post', 'user_logout_post'))
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = '登录' if ret_item.action == 'user_login_post'  else '登出'
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_用户登录登出')
        return Response("管理员日志_用户登录登出导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)
    

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_org(self, request, *args, **kwargs):
        '''
        日志导出:组织机构管理
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='org')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='org', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_组织机构管理')
        return Response("管理员日志_组织机构管理导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)

 
    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_account(self, request, *args, **kwargs):
        '''
        日志导出:域用户管理
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='account')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='account', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_域用户管理')
        return Response("管理员日志_域用户管理导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)
   

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_device(self, request, *args, **kwargs):
        '''
        日志导出:终端
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='device')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='device', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_终端管理')
        return Response("管理员日志_终端管理导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)
   

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_software(self, request, *args, **kwargs):
        '''
        日志导出:软件
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='software')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='software', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_软件管理')
        return Response("管理员日志_软件日志管理中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_patch(self, request, *args, **kwargs):
        '''
        日志导出:补丁
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='patch')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='patch', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_补丁管理')
        return Response("管理员日志_补丁管理导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_tasks(self, request, *args, **kwargs):
        '''
        日志导出:任务
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='tasks')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='tasks', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_任务管理')
        return Response("管理员日志_任务管理导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_logs(self, request, *args, **kwargs):
        '''
        日志导出:日志管理
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='logs')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='logs', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_日志管理')
        return Response("管理员日志_日志管理导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_user(self, request, *args, **kwargs):
        '''
        日志导出:管理员管理
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='user')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='user', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_管理员管理')
        return Response("管理员日志_管理员管理导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_salt(self, request, *args, **kwargs):
        '''
        日志导出:策略管理
        '''
        admin_name = request.query_params.get('admin_name')
        admin_nickname = request.query_params.get('admin_nickname')
        action = request.query_params.get('action')
        result_status = request.query_params.get('result_status')
        desc = request.query_params.get('desc')
        operate_time_type = request.GET.get('operate_time_type')

        # action列表
        sub_action_list = []
        menu_sub_actions = self.__get_actions_by_module(module='salt')
        for item in menu_sub_actions:
            sub_action_list.append(item['action'])
        queryset = AdminLog.objects.filter(module='salt', action__in=sub_action_list)
        # 查询条件
        if admin_name:
            admin_name = str(admin_name).split(',')
            queryset = queryset.filter(admin_name__in=admin_name)
        if admin_nickname:
            queryset = queryset.filter(Q(admin_nickname__icontains=admin_nickname))
        if action:
            queryset = queryset.filter(action=action)
        if desc:
            queryset = queryset.filter(Q(desc__icontains=desc))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 行为描述
            action_desc = ''
            for item2 in menu_sub_actions:
                if item2['action'] == ret_item.action:
                    action_desc = item2['action_desc']
                    break
            # 转成中文描述
            data_item["姓名"] = ret_item.admin_nickname
            data_item["用户账号"] = ret_item.admin_name
            data_item["描述"] = ret_item.desc
            data_item["行为"] = action_desc
            data_item["结果"] = '成功' if ret_item.result_status == 1  else '失败'
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_adminlog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '管理员日志_策略管理')
        return Response("管理员日志_策略管理导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)
