import os, time, json, logging
import ipaddress
from django.db import close_old_connections
from django.conf import settings

from devices.models import Devices
from devices.serializers import DeviceSerializer
from browser import models, filter, serializers
from browser.models import USER_POLICY_TYPE, APPLICATION_POLICY_TYPE, BOOKMARK_POLICY_TYPE, POLICY_DEFAULT_FILE_NAME, \
    get_policy_int
from saltapi import tasks

from django.db.models import Q
from concurrent.futures import ThreadPoolExecutor

logger = logging.getLogger(__name__)

executor = ThreadPoolExecutor(max_workers=10)


# 根据策略获取设备列表 （复制补丁策略逻辑代码）
def get_device_list(browserPolicyInfo):
    device_items = Devices.objects.exclude(is_domain=0).all()

    range_type = browserPolicyInfo['range_type']
    range_value = browserPolicyInfo['range_val']

    # (0, '按部门'), (1, '按终端'), (2, '按 IP 网段'), (2, '按人员')
    if range_type == 0:
        organ_list = range_value.split(',')
        if device_items.filter(org_info_id__in=organ_list).exists():
            device_list = device_items.filter(org_info_id__in=organ_list).values_list('id', flat=True)
            device_list = [str(i) for i in device_list]
            res_device = ','.join(device_list) if len(device_list) != 0 else None
        else:
            res_device = None
    elif range_type == 1:
        res_device = range_value
    elif range_type == 2:
        ip_list = range_value.split(',')
        mid_ip_list = []
        for i in ip_list:
            start_ip, end_ip = i.split('-')
            # 解析起始和结束IP地址为IPv4Address对象
            start_ip = ipaddress.IPv4Address(start_ip)
            end_ip = ipaddress.IPv4Address(end_ip)
            range_ip_list = [str(ip) for ip in range(int(start_ip), int(end_ip) + 1)]
            mid_ip_list = range_ip_list + mid_ip_list
        if device_items.filter(device_ip__in=mid_ip_list).exists():
            device_list = device_items.filter(device_ip__in=mid_ip_list).values_list('id', flat=True)
            device_list = [str(i) for i in device_list]
            res_device = ','.join(device_list) if len(device_list) != 0 else None
        else:
            res_device = None
    elif range_type == 3:
        account_list = range_value.split(',')
        if device_items.filter(bind_account__id__in=account_list).exists():
            device_list = device_items.filter(bind_account__id__in=account_list).values_list('id', flat=True)
            device_list = [str(i) for i in device_list]
            res_device = ','.join(device_list) if len(device_list) != 0 else None
        else:
            res_device = None
    else:
        res_device = None
    return res_device


# ---------------------------------------------------------------------------v1--------------------------------

def get_default_policy_file_path(type):
    policy_file = f"{type}/{POLICY_DEFAULT_FILE_NAME}.json"
    return os.path.join(settings.BROWSER_POLICY_FILE_PATH, policy_file)


def get_default_policy_form_file_path(type):
    policy_file = f"{type}/{POLICY_DEFAULT_FILE_NAME}-form.json"
    return os.path.join(settings.BROWSER_POLICY_FILE_PATH, policy_file)


# 查询机构的策略  目前是根据是否进行过批量下发
def get_policy_by_org(policy_type, org_id):
    bpo = models.BrowserPolicyOrgInfo.objects.filter(org_id=org_id, policy_type=get_policy_int(policy_type)).first()
    if not bpo:
        # 使用默认策略 默认策略ID=0
        return 0
    else:
        return bpo.policy_id


# 下发策略文件后JID更新事件处理
def update_jids(account_id, policy_type, jid):
    logger.info(f"update_jids run,account is {account_id},policy_type is {policy_type},jid is {jid}")

    policy_account_obj = None
    if policy_type == USER_POLICY_TYPE:
        policy_account_obj = models.BrowserUserPolicyAccount.objects.filter(account_id=account_id).first()
    if policy_type == APPLICATION_POLICY_TYPE:
        policy_account_obj = models.BrowserApplicationPolicyAccount.objects.filter(account_id=account_id).first()
    if policy_type == BOOKMARK_POLICY_TYPE:
        policy_account_obj = models.BrowserBookmarkPolicyAccount.objects.filter(account_id=account_id).first()

    if policy_account_obj:
        jobids = policy_account_obj.jobids
        if not jobids:
            policy_account_obj.jobids = jid
        else:
            policy_account_obj.jobids = ",".join([jobids, jid])
    policy_account_obj.save()
    close_old_connections()


# 分发策略文件前，重置策略-账户表的状态 jobid信息 增加序列号的值
def pre_deploy_to_device(policy_account_obj):
    current_seq = policy_account_obj.update_seq
    policy_account_obj.update_seq = current_seq + 1
    policy_account_obj.jobids = ""
    policy_account_obj.status = 0
    policy_account_obj.save(force_update=True)
    return policy_account_obj


# 账户-机构变化事件处理
def check_account_org_change(account_id, org_id):
    policy_account_obj = models.BrowserUserPolicyAccount.objects.filter(account_id=account_id).first()
    if policy_account_obj:
        policy_account_obj.org_id = org_id
        policy_id = get_policy_by_org(USER_POLICY_TYPE, org_id)
        if policy_id == 0:
            policy_account_obj.policy_id = None
        else:
            policy_account_obj.policy_id = policy_id
        policy_account_obj.save()
        gen_json_file_by_policy_type(account_id=account_id, policy_id=policy_id, policy_type=USER_POLICY_TYPE)
    else:
        logger.info(f"No item fount in user policy accout for account is {account_id}")
        return

    policy_account_obj = models.BrowserApplicationPolicyAccount.objects.filter(account_id=account_id).first()
    if policy_account_obj:
        policy_account_obj.org_id = org_id
        policy_id = get_policy_by_org(APPLICATION_POLICY_TYPE, org_id)
        if policy_id == 0:
            policy_account_obj.policy_id = None
        else:
            policy_account_obj.policy_id = policy_id
        policy_account_obj.save()
        gen_json_file_by_policy_type(account_id=account_id, policy_id=policy_id, policy_type=APPLICATION_POLICY_TYPE)
    else:
        logger.info(f"No item fount in application policy accout for account is {account_id}")
        return
    policy_account_obj = models.BrowserBookmarkPolicyAccount.objects.filter(account_id=account_id).first()
    if policy_account_obj:
        policy_account_obj.org_id = org_id
        policy_id = get_policy_by_org(BOOKMARK_POLICY_TYPE, org_id)
        if policy_id == 0:
            policy_account_obj.policy_id = None
        else:
            policy_account_obj.policy_id = policy_id
        policy_account_obj.save()
        gen_json_file_by_policy_type(account_id=account_id, policy_id=policy_id, policy_type=BOOKMARK_POLICY_TYPE)
    else:
        logger.info(f"No item fount in bookmark policy accout for account is {account_id}")
        return

    deployed_browser_policy_for_account(account_id)


# 账户-设备绑定变化事件处理
def preAddPolicyAccountInfo(account_id, org_id, add_account_ids, device_id):
    is_add = False
    bupa = models.BrowserUserPolicyAccount.objects.filter(account_id=account_id).first()
    if bupa is None:
        is_add = True
        bupa = models.BrowserUserPolicyAccount()
        bupa.account_id = account_id
        bupa.org_id = org_id
        policy_id = get_policy_by_org(USER_POLICY_TYPE, org_id)
        if policy_id == 0:
            bupa.status = 3
        else:
            bupa.policy_id = policy_id
        bupa.save(force_insert=True)
        logger.info(f"add new record for browser_user_policy_account,account is {account_id}")
    gen_json_file_by_policy_type(account_id=account_id, policy_id=bupa.policy_id, policy_type=USER_POLICY_TYPE)
    bapa = models.BrowserApplicationPolicyAccount.objects.filter(account_id=account_id).first()
    if bapa is None:
        bapa = models.BrowserApplicationPolicyAccount()
        bapa.account_id = account_id
        bapa.org_id = org_id
        policy_id = get_policy_by_org(APPLICATION_POLICY_TYPE, org_id)
        if policy_id == 0:
            bapa.status = 3
        else:
            bapa.policy_id = policy_id
        bapa.save(force_insert=True)
        logger.info(f"add new record for browser_application_policy_account,account is {account_id}")
    gen_json_file_by_policy_type(account_id=account_id, policy_id=bapa.policy_id, policy_type=APPLICATION_POLICY_TYPE)
    bmpa = models.BrowserBookmarkPolicyAccount.objects.filter(account_id=account_id).first()
    if bmpa is None:
        bmpa = models.BrowserBookmarkPolicyAccount()
        bmpa.account_id = account_id
        bmpa.org_id = org_id
        policy_id = get_policy_by_org(BOOKMARK_POLICY_TYPE, org_id)
        if policy_id == 0:
            bmpa.status = 3
        else:
            bmpa.policy_id = policy_id
        bmpa.save(force_insert=True)
        logger.info(f"add new record for browser_bookmark_policy_account,account is {account_id}")
    gen_json_file_by_policy_type(account_id=account_id, policy_id=bmpa.policy_id, policy_type=BOOKMARK_POLICY_TYPE)
    # 首次,添加到策略-账户信息表 ，如果是已存在表中 并且只是绑定终端有变 发送对应帐号策略到终端 不全发送                     
    if is_add:
        deployed_browser_policy_for_account(account_id)
    elif account_id in add_account_ids:
        deployed_browser_policy_for_account_and_device(account_id, device_id)
    close_old_connections()


# 更新或创建策略文件
def createOrUpdateJsonFile(file_path, content):
    if not os.path.isfile(file_path):

        json_file = open(file_path, 'w', encoding='utf-8')
        json_file.write(content)
        json_file.close()
    else:
        json_file = open(file_path, 'w', encoding='utf-8')
        json_file.write(content)
        json_file.close()
        # logger.info(f"createOrUpdateJsonFile ,file_path is {file_path}")


# 生成策略文件
def gen_json_file_by_policy_type(account_id, policy_id, policy_type):
    policy_info = None
    if policy_type == USER_POLICY_TYPE:
        policy_info = models.BrowserUserPolicyInfo.objects.filter(pk=policy_id).first()
    if policy_type == APPLICATION_POLICY_TYPE:
        policy_info = models.BrowserApplicationPolicyInfo.objects.filter(pk=policy_id).first()
    if policy_type == BOOKMARK_POLICY_TYPE:
        policy_info = models.BrowserBookmarkPolicyInfo.objects.filter(pk=policy_id).first()

    if not policy_info:
        with open(get_default_policy_form_file_path(policy_type), 'r') as json_file:
            # 加载 JSON 数据
            policy_form_content = json_file.read()
        with open(get_default_policy_file_path(policy_type), 'r') as json_file:
            # 加载 JSON 数据
            policy_json_content = json_file.read()
    else:
        policy_form_content = policy_info.policy_form_content
        policy_json_content = policy_info.policy_json_content

    # policy form json
    policy_form_file_path = get_default_policy_form_file_path(policy_type)
    policy_form_file_path = policy_form_file_path.replace(POLICY_DEFAULT_FILE_NAME, str(account_id))
    createOrUpdateJsonFile(policy_form_file_path, policy_form_content)

    # policy content json
    policy_file_path = get_default_policy_file_path(policy_type)
    policy_file_path = policy_file_path.replace(POLICY_DEFAULT_FILE_NAME, str(account_id))
    createOrUpdateJsonFile(policy_file_path, policy_json_content)


# 根据账户 策略类型policy_type 分发策略文件到device
def deployed_browser_policy_to_device(policy_type, policy_account_obj, device):
    account_id = policy_account_obj.account_id
    update_seq = policy_account_obj.update_seq
    policy_id = policy_account_obj.policy_id
    logger.info(
        f"run deployed_browser_policy_to_device 类型{policy_type} 策略{policy_id} 账户{account_id} 分发到设备{device.id}")

    tasks.browser_deploy_policy.delay(device=DeviceSerializer(device).data, policy_type=policy_type,
                                      account_id=account_id,
                                      update_seq=update_seq, policy_id=policy_id)
    return


# 根据策略类型 策略-账户信息  下发用户的policy_type策略到用户绑定的设备
def deployed_browser_policy_by_type(policy_type, policy_account_obj):
    account_id = policy_account_obj.account_id
    logger.info(f"run deployed_browser_policy_by_type ,account is {account_id} and policy type is {policy_type}")
    bind_devices = Devices.objects.exclude(is_domain=0).all().filter(bind_account=account_id)
    if bind_devices:
        for device in bind_devices:
            wrapped_task = deployed_browser_policy_to_device(policy_type, policy_account_obj, device)
            executor.submit(wrapped_task)

    else:
        logger.info(f"no device to deployed for account {account_id}")

    return


def deployed_browser_policy_for_account_and_device(account_id, device_id):
    device = Devices.objects.exclude(is_domain=0).all().filter(pk=device_id).first()
    logger.info(f"run deployed_browser_policy_for_account ,account is {account_id},device is {device.device_hostname}")
    policy_account_obj = models.BrowserUserPolicyAccount.objects.filter(account_id=account_id).first()
    if not policy_account_obj:
        logger.info("policy_account_obj begin 1 2 3....")
        time.sleep(3)
        logger.info("policy_account_obj end 3 2 1....")
        policy_account_obj = models.BrowserUserPolicyAccount.objects.filter(account_id=account_id).first()
    logger.info(f"policy_account_obj is  {policy_account_obj}")
    if policy_account_obj:
        pre_deploy_to_device(policy_account_obj)
        wrapped_task = deployed_browser_policy_to_device(USER_POLICY_TYPE, policy_account_obj, device)
        executor.submit(wrapped_task)
    policy_account_obj = models.BrowserApplicationPolicyAccount.objects.filter(account_id=account_id).first()
    logger.info(f"policy_account_obj is  {policy_account_obj}")
    if policy_account_obj:
        pre_deploy_to_device(policy_account_obj)
        wrapped_task = deployed_browser_policy_to_device(APPLICATION_POLICY_TYPE, policy_account_obj, device)
        executor.submit(wrapped_task)

        policy_account_obj = models.BrowserBookmarkPolicyAccount.objects.filter(account_id=account_id).first()
        logger.info(f"policy_account_obj is  {policy_account_obj}")
        if policy_account_obj:
            pre_deploy_to_device(policy_account_obj)
            wrapped_task = deployed_browser_policy_to_device(BOOKMARK_POLICY_TYPE, policy_account_obj, device)
            executor.submit(wrapped_task)
    else:
        logger.info(f"no device to deployed for account {account_id}")
    # 根据账户ID，查找到绑定设备，下发全部类型的策略文件到终端


def deployed_browser_policy_for_account(account_id):
    logger.info(f"run deployed_browser_policy_for_account ,account is {account_id}")
    bind_devices = Devices.objects.exclude(is_domain=0).all().filter(bind_account=account_id)

    if not bind_devices:
        logger.info("bind_devices begin 3 2 1....")
        time.sleep(3)
        logger.info("bind_devices end 3 2 1....")
        bind_devices = Devices.objects.exclude(is_domain=0).all().filter(bind_account=account_id)

    if bind_devices:
        policy_account_obj = models.BrowserUserPolicyAccount.objects.filter(account_id=account_id).first()
        if not policy_account_obj:
            logger.info("policy_account_obj begin 3 2 1....")
            time.sleep(3)
            logger.info("policy_account_obj end 3 2 1....")
            policy_account_obj = models.BrowserUserPolicyAccount.objects.filter(account_id=account_id).first()
        logger.info(f"policy_account_obj is  {policy_account_obj}")
        if policy_account_obj:
            pre_deploy_to_device(policy_account_obj)
            for device in bind_devices:
                wrapped_task = deployed_browser_policy_to_device(USER_POLICY_TYPE, policy_account_obj, device)
                executor.submit(wrapped_task)
        policy_account_obj = models.BrowserApplicationPolicyAccount.objects.filter(account_id=account_id).first()
        logger.info(f"policy_account_obj is  {policy_account_obj}")
        if policy_account_obj:
            pre_deploy_to_device(policy_account_obj)
            for device in bind_devices:
                wrapped_task = deployed_browser_policy_to_device(APPLICATION_POLICY_TYPE, policy_account_obj, device)
                executor.submit(wrapped_task)

        policy_account_obj = models.BrowserBookmarkPolicyAccount.objects.filter(account_id=account_id).first()
        logger.info(f"policy_account_obj is  {policy_account_obj}")
        if policy_account_obj:
            pre_deploy_to_device(policy_account_obj)
            for device in bind_devices:
                wrapped_task = deployed_browser_policy_to_device(BOOKMARK_POLICY_TYPE, policy_account_obj, device)
                executor.submit(wrapped_task)
    else:
        logger.info(f"no device to deployed for account {account_id}")
    close_old_connections()


def deploy_policy_to_device_task(device):
    if not device:
        pass
    else:
        tasks.browser_default_policy.delay(device=DeviceSerializer(device).data)


def deploy_policy_to_device(device_id):
    device = Devices.objects.filter(pk=device_id).first()
    wrapped_task = deploy_policy_to_device_task(device)
    executor.submit(wrapped_task)


def translate_to_policy(formDict):
    policy_json = {}
    # 复制
    disable_copy = formDict.get('disable_copy', False)
    if disable_copy is True:
        policy_json["CopyPreventionSettings"] = {"enable": ["*"],
                                                 "disable": [],
                                                 "minimum_data_size": 0}

    # 开发者工具
    disable_developer_tool = formDict.get('disable_developer_tool', False)
    if disable_developer_tool is True:
        policy_json["DeveloperToolsAvailability"] = 2
    # 查看源代码
    disable_view_source = formDict.get('disable_view_source', False)
    if disable_view_source is True:
        policy_json["URLBlocklist"] = ["view-source:*"]
    # 下载
    disable_download = formDict.get('disable_download', False)
    if disable_download is True:
        policy_json["DownloadRestrictions"] = 3
    # 打印
    disable_printing = formDict.get('disable_printing', False)
    if disable_printing is True:
        policy_json["PrintingEnabled"] = False

    # 首页 https://chromeenterprise.google/policies/#RestoreOnStartup
    homepage = formDict.get('homepage', {})
    homepage_type = homepage.get('type', None)
    homepage_type_map = {
        "restore": 1,
        "customize": 4,
        "blank": 4,
        "newtab": 5,
        "restore_and_customize": 6
    }
    if homepage_type in homepage_type_map:
        policy_json["RestoreOnStartup"] = homepage_type_map[homepage_type]
    if homepage_type in ["customize", "blank", "restore_and_customize"]:
        if homepage_type == "blank":
            policy_json["RestoreOnStartupURLs"] = ['about:blank']
        else:
            homepage_url = list(map(lambda e: e.get('url', None), homepage.get('urls', [])))
            policy_json["RestoreOnStartupURLs"] = homepage_url

    # bookmark = formDict.get('bookmark', {})

    # managed_bookmarks = bookmark.get('managed', None)

    return json.dumps(policy_json)


def translate_to_application_policy(formDict):
    policy_json = {}
    policy_json["NTPUseMostVisitedTiles"] = False
    policy_json["CustomLinksList"] = []
    ntp = formDict.get('ntp', {})
    if ntp:
        applications = ntp.get("applications", [])
        policy_json["CustomLinksList"] = applications
    return json.dumps(policy_json)


def translate_to_bookmark_policy(formDict):
    policy_json = {}
    policy_json["ManagedBookmarks"] = []
    bookmark = formDict.get('bookmark', {})
    if bookmark:
        bookmarks = bookmark.get("list", [])
        folder_name = bookmark.get("toplevel_name")
        if folder_name:
            bookmarks.append({"toplevel_name": folder_name})
        policy_json["ManagedBookmarks"] = bookmarks

    return json.dumps(policy_json)


def saveOrUpdatePolicyOrgInfo(policy_type, policy_id, organ_ids):
    if not policy_id or not organ_ids:
        return
    for org_id in organ_ids:
        bpo = models.BrowserPolicyOrgInfo.objects.filter(org_id=org_id, policy_type=policy_type).first()

        if not bpo:
            bpo = models.BrowserPolicyOrgInfo()
            bpo.policy_id = policy_id
            bpo.org_id = org_id
            bpo.policy_type = policy_type
        bpo.policy_id = policy_id
        bpo.save()
