import logging
import json

from django_filters.rest_framework import DjangoFilterBackend
from django_redis import get_redis_connection
from rest_framework import viewsets, status
from rest_framework.decorators import action
from rest_framework.filters import OrderingFilter, SearchFilter
from rest_framework.response import Response

from nfs_domain_mainserver import settings
from accounts.models import AccountEntity
from organizations.models import OrgnizationEntity
from devices.models import Devices
from browser import models, filter, serializers,service
from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer
 
from browser.util import USER_POLICY_TYPE,APPLICATION_POLICY_TYPE ,BOOKMARK_POLICY_TYPE,POLICY_DEFAULT_FILE_NAME
from browser.util import translate_to_policy,translate_to_application_policy,translate_to_bookmark_policy,saveOrUpdatePolicyOrgInfo
from browser.util import gen_json_file_by_policy_type,deployed_browser_policy_to_device,deployed_browser_policy_by_type,pre_deploy_to_device

conn = get_redis_connection('redis')
 
logger = logging.getLogger(__name__)
 
class BrowserPolicyOrgUtil():
    @staticmethod
    def organ_down_list(num):
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        # logger.info(f'***organ_down_list*******{organ_down_list}*********')
        return organ_down_list

# 浏览器-用户策略  默认  
class BrowserUserPolicyDefaultViewSet(viewsets.ModelViewSet):
     
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)

    def list(self, request, *args, **kwargs):
         
        res_obj = service.get_default_form_json(USER_POLICY_TYPE)
        if not res_obj:
            return Response("默认策略文件缺失", status=status.HTTP_404_NOT_FOUND) 
        return Response(res_obj, status.HTTP_200_OK)
        

    def update(self, request, *args, **kwargs):
        item = request.data.copy() 
        res = service.update_default_file(USER_POLICY_TYPE,item)
        if not res:
            return Response("默认策略文件缺失", status=status.HTTP_404_NOT_FOUND) 
        return Response(item,status=status.HTTP_200_OK)

# 浏览器-用户策略  信息
class BrowserUserPolicyInfoViewSet(viewsets.ModelViewSet):
    queryset = models.BrowserUserPolicyInfo.objects.all()
    serializer_class = serializers.BrowserUserPolicyInfoSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.BrowserUserPolicyInfoFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__' 

    def filter_queryset(self, queryset):
        queryset = super(BrowserUserPolicyInfoViewSet, self).filter_queryset(self.queryset)
        return queryset

    @staticmethod
    def _change_item_(item):
        policy_form_content = item['policy_form_content']
        item['policy_json_content']= translate_to_policy(json.loads(policy_form_content))
        return item

    def create(self, request, *args, **kwargs):
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        serializer = self.get_serializer(data=item)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
       
        return Response('数据创建完成', status=status.HTTP_201_CREATED)

    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        policyAccountSet = models.BrowserUserPolicyAccount.objects.filter(policy_id=self.kwargs['pk'])
        for userPolicyAccount in policyAccountSet:
            gen_json_file_by_policy_type(userPolicyAccount.account_id,userPolicyAccount.policy_id,USER_POLICY_TYPE)
            pre_deploy_to_device(userPolicyAccount)
            deployed_browser_policy_by_type(USER_POLICY_TYPE,userPolicyAccount)
        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        policyAccount = models.BrowserUserPolicyAccount.objects.filter(policy_id=self.kwargs['pk']).first()
        if policyAccount:
           return Response("策略使用中，不能删除",status=status.HTTP_226_IM_USED) 
        self.perform_destroy(instance)
        return Response(status=status.HTTP_204_NO_CONTENT)
    
    @action(methods=['get'], url_path='all',detail=False, authentication_classes=[TokenAuthentication])
    def get_all_policy_names(self, request, *args, **kwargs):
        policy_list = models.BrowserUserPolicyInfo.objects.values("id","name")
 
        return Response(policy_list, status=status.HTTP_200_OK)
# 浏览器-用户策略  信息-ACCOUNT
class BrowserUserPolicyAccountViewSet(viewsets.ModelViewSet):
    queryset = models.BrowserUserPolicyAccount.objects.all()
    serializer_class = serializers.BrowserUserPolicyAccountSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.BrowserUserPolicyAccountFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__' 

    
    def filter_queryset(self, queryset):
        queryset = super(BrowserUserPolicyAccountViewSet, self).filter_queryset(self.queryset)
        return queryset

    def get_queryset(self):
         
        policy__name_param = self.request.query_params.get('policy__name__contains',None) 
        if policy__name_param:
            if "默认" in policy__name_param :
                self.queryset = self.queryset.filter(policy_id__isnull=True) 
            else:
                self.queryset = self.queryset.filter(policy__name__contains=policy__name_param) 
        query_org_id = self.request.query_params.get('org_id',None) 
        if query_org_id:
            org_down_list = BrowserPolicyOrgUtil.organ_down_list(query_org_id)
            self.queryset = self.queryset.filter(org_id__in=org_down_list)
        return self.queryset 
    
    @staticmethod
    def _change_item_(self,item):   
        return item
    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        policy_id = item['policy_id']
        if not policy_id:
            return  Response("没有策略组信息")
        item = self._change_item_(self,item)
        item['creator'] = request.user.username
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        # update device policy file
         
        logger.info("begin deploy policy file to device")
        policyAccount = models.BrowserUserPolicyAccount.objects.filter(pk= self.kwargs['pk']).first()
        gen_json_file_by_policy_type(policyAccount.account_id,policyAccount.policy_id,USER_POLICY_TYPE)
        pre_deploy_to_device(policyAccount)
        deployed_browser_policy_by_type(USER_POLICY_TYPE,policyAccount)

        return Response(serializer.data)
    
    @action(methods=['get'], url_path='jobs',detail=True, authentication_classes=[TokenAuthentication])
    def get_jobs(self, request, *args, **kwargs):
        res = service.get_jobs(self.kwargs['pk'],USER_POLICY_TYPE)
        if res:
            return Response(data=res)
        return Response("no user policy account data")
       
    @action(methods=['get'], url_path='logs',detail=True, authentication_classes=[TokenAuthentication])
    def get_logs(self, request, *args, **kwargs):
        res = service.get_logs(self.kwargs['pk'],USER_POLICY_TYPE)
        if res:
            return Response(data=res)
        return Response("no user policy account log data")    
    
    @action(methods=['put'], url_path='multiple-org-change-policy',detail=False, authentication_classes=[TokenAuthentication])
    def multiple_org_change_policy(self, request, *args, **kwargs):
        """
        按照部门 策略 批量修改
        """
        organ_ids = request.data.get('org_info_id')
        policy_id = request.data.get('policy_id')

        if policy_id is None:
            return Response({'code': 4001, 'msg': '策略 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if organ_ids is None:
            return Response({'code': 4001, 'msg': '机构为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if not isinstance(organ_ids, list):
            return Response({'code': 4001, 'msg': '机构必须传列表'}, status=status.HTTP_400_BAD_REQUEST)
        accounts = AccountEntity.objects.all().filter(org_info_id__in=organ_ids)
        service.deploy_policy_to_mult_account(USER_POLICY_TYPE,policy_id,organ_ids,accounts)
        if not accounts:
            return Response('部门下没有查询到用户', status=status.HTTP_400_BAD_REQUEST)
          
        return Response("部门批量更新策略成功", status=status.HTTP_200_OK)
# ----------------------------------------------------------------------------------------------------------

# 浏览器-应用策略  默认 
class BrowserApplicationPolicyDefaultViewSet(viewsets.ModelViewSet):
     
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)

    def list(self, request, *args, **kwargs):

        res_obj = service.get_default_form_json(APPLICATION_POLICY_TYPE)
        if not res_obj:
            return Response("默认策略文件缺失", status=status.HTTP_404_NOT_FOUND) 
        return Response(res_obj, status.HTTP_200_OK)
        

    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        res = service.update_default_file(APPLICATION_POLICY_TYPE,item)
        if not res:
            return Response("默认策略文件缺失", status=status.HTTP_404_NOT_FOUND) 
        return Response(item,status=status.HTTP_200_OK)

# 浏览器-应用策略 信息
class BrowserApplicationPolicyInfoViewSet(viewsets.ModelViewSet):
    queryset = models.BrowserApplicationPolicyInfo.objects.all()
    serializer_class = serializers.BrowserApplicationPolicyInfoSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.BrowserApplicationPolicyInfoFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__' 

    def filter_queryset(self, queryset):
        queryset = super(BrowserApplicationPolicyInfoViewSet, self).filter_queryset(self.queryset)
        return queryset

    @staticmethod
    def _change_item_(item):
        policy_form_content = item['policy_form_content']
        item['policy_json_content']= translate_to_application_policy(json.loads(policy_form_content))
        return item

    def create(self, request, *args, **kwargs):
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        serializer = self.get_serializer(data=item)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
       
        return Response('数据创建完成', status=status.HTTP_201_CREATED)

    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        policyAccountSet = models.BrowserApplicationPolicyAccount.objects.filter(policy_id=self.kwargs['pk'])
        for userPolicyAccount in policyAccountSet:
            gen_json_file_by_policy_type(userPolicyAccount.account_id,userPolicyAccount.policy_id,APPLICATION_POLICY_TYPE)
            pre_deploy_to_device(userPolicyAccount)
            deployed_browser_policy_by_type(APPLICATION_POLICY_TYPE,userPolicyAccount)
        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        policyAccount = models.BrowserApplicationPolicyAccount.objects.filter(policy_id=self.kwargs['pk']).first()
        if policyAccount:
           return Response("策略使用中，不能删除",status=status.HTTP_226_IM_USED) 
        self.perform_destroy(instance)
        return Response(status=status.HTTP_204_NO_CONTENT)
    
    @action(methods=['get'], url_path='all',detail=False, authentication_classes=[TokenAuthentication])
    def get_all_policy_names(self, request, *args, **kwargs):
        policy_list = models.BrowserApplicationPolicyInfo.objects.values("id","name")
 
        return Response(policy_list, status=status.HTTP_200_OK)
    
# 浏览器-应用策略 信息-ACCOUNT
class BrowserApplicationPolicyAccountViewSet(viewsets.ModelViewSet):
    queryset = models.BrowserApplicationPolicyAccount.objects.all()
    serializer_class = serializers.BrowserApplicationPolicyAccountSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.BrowserApplicationPolicyAccountFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__' 

    def filter_queryset(self, queryset):
        queryset = super(BrowserApplicationPolicyAccountViewSet, self).filter_queryset(self.queryset)
        return queryset

    def get_queryset(self):
         
        policy__name_param = self.request.query_params.get('policy__name__contains',None)
         
        if policy__name_param:
            if "默认" in policy__name_param :
                self.queryset = self.queryset.filter(policy_id__isnull=True)
                 
            else:
                self.queryset = self.queryset.filter(policy__name__contains=policy__name_param)
        query_org_id = self.request.query_params.get('org_id',None) 
        if query_org_id:
            org_down_list = BrowserPolicyOrgUtil.organ_down_list(query_org_id)
            self.queryset = self.queryset.filter(org_id__in=org_down_list)        
        return self.queryset 
    @staticmethod
    def _change_item_(self,item):   
        return item
    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        policy_id = item['policy_id']
        if not policy_id:
            return  Response("没有策略组信息")
        item = self._change_item_(self,item)
        item['creator'] = request.user.username
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        # update device policy file
         
        logger.info("begin deploy policy file to device")
        policyAccount = models.BrowserApplicationPolicyAccount.objects.filter(pk= self.kwargs['pk']).first()
        gen_json_file_by_policy_type(policyAccount.account_id,policyAccount.policy_id,APPLICATION_POLICY_TYPE)
        pre_deploy_to_device(policyAccount)
        deployed_browser_policy_by_type(APPLICATION_POLICY_TYPE,policyAccount)

        return Response(serializer.data)
    
    @action(methods=['get'], url_path='jobs',detail=True, authentication_classes=[TokenAuthentication])
    def get_jobs(self, request, *args, **kwargs):
        res = service.get_jobs(self.kwargs['pk'],APPLICATION_POLICY_TYPE)
        if res:
            return Response(data=res)
        return Response("no app policy account data")
    
         
    @action(methods=['get'], url_path='logs',detail=True, authentication_classes=[TokenAuthentication])
    def get_logs(self, request, *args, **kwargs):
        res = service.get_logs(self.kwargs['pk'],APPLICATION_POLICY_TYPE)
        if res:
            return Response(data=res)
        return Response("no   policy account log data") 
    @action(methods=['put'], url_path='multiple-org-change-policy',detail=False, authentication_classes=[TokenAuthentication])
    def multiple_org_change_policy(self, request, *args, **kwargs):
        """
        按照部门 策略 批量修改
        """
        organ_ids = request.data.get('org_info_id')
        policy_id = request.data.get('policy_id')

        if policy_id is None:
            return Response({'code': 4001, 'msg': '策略 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if organ_ids is None:
            return Response({'code': 4001, 'msg': '机构为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if not isinstance(organ_ids, list):
            return Response({'code': 4001, 'msg': '机构必须传列表'}, status=status.HTTP_400_BAD_REQUEST)
        accounts = AccountEntity.objects.all().filter(org_info_id__in=organ_ids)
        service.deploy_policy_to_mult_account(APPLICATION_POLICY_TYPE,policy_id,organ_ids,accounts)
        if not accounts:
            return Response('部门下没有查询到用户', status=status.HTTP_400_BAD_REQUEST)
          
        return Response("部门批量更新策略成功", status=status.HTTP_200_OK)
        

 
    
# ----------------------------------------------------------------------------------------------------------

# 浏览器-收藏策略  默认 
class BrowserBookmarkPolicyDefaultViewSet(viewsets.ModelViewSet):
     
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)

    def list(self, request, *args, **kwargs):
         
        res_obj = service.get_default_form_json(BOOKMARK_POLICY_TYPE)
        if not res_obj:
            return Response("默认策略文件缺失", status=status.HTTP_404_NOT_FOUND) 
        return Response(res_obj, status.HTTP_200_OK)
        

    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        res = service.update_default_file(BOOKMARK_POLICY_TYPE,item)
        if not res:
            return Response("默认策略文件缺失", status=status.HTTP_404_NOT_FOUND) 
        return Response(item,status=status.HTTP_200_OK)
       

# 浏览器-收藏策略 信息
class BrowserBookmarkPolicyInfoViewSet(viewsets.ModelViewSet):
    queryset = models.BrowserBookmarkPolicyInfo.objects.all()
    serializer_class = serializers.BrowserBookmarkPolicyInfoSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.BrowserBookmarkPolicyInfoFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__' 

    def filter_queryset(self, queryset):
        queryset = super(BrowserBookmarkPolicyInfoViewSet, self).filter_queryset(self.queryset)
        return queryset

    @staticmethod
    def _change_item_(item):
        policy_form_content = item['policy_form_content']
        item['policy_json_content']= translate_to_bookmark_policy(json.loads(policy_form_content))
        return item

    def create(self, request, *args, **kwargs):
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        serializer = self.get_serializer(data=item)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
       
        return Response('数据创建完成', status=status.HTTP_201_CREATED)

    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        policyAccountSet = models.BrowserBookmarkPolicyAccount.objects.filter(policy_id=self.kwargs['pk'])
        for userPolicyAccount in policyAccountSet:
            gen_json_file_by_policy_type(userPolicyAccount.account_id,userPolicyAccount.policy_id,BOOKMARK_POLICY_TYPE)
            pre_deploy_to_device(userPolicyAccount)
            deployed_browser_policy_by_type(BOOKMARK_POLICY_TYPE,userPolicyAccount)
        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        policyAccount = models.BrowserBookmarkPolicyAccount.objects.filter(policy_id=self.kwargs['pk']).first()
        if policyAccount:
           return Response("策略使用中，不能删除",status=status.HTTP_226_IM_USED) 
        self.perform_destroy(instance)
        return Response(status=status.HTTP_204_NO_CONTENT)
    
    @action(methods=['get'], url_path='all',detail=False, authentication_classes=[TokenAuthentication])
    def get_all_policy_names(self, request, *args, **kwargs):
        policy_list = models.BrowserBookmarkPolicyInfo.objects.values("id","name")
 
        return Response(policy_list, status=status.HTTP_200_OK)
      
    
# 浏览器-收藏策略 信息-ACCOUNT 
class BrowserBookmarkPolicyAccountViewSet(viewsets.ModelViewSet):
    queryset = models.BrowserBookmarkPolicyAccount.objects.all()
    serializer_class = serializers.BrowserBookmarkPolicyAccountSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.BrowserBookmarkPolicyAccountFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__' 

    def filter_queryset(self, queryset):
        queryset = super(BrowserBookmarkPolicyAccountViewSet, self).filter_queryset(self.queryset)
        return queryset

    def get_queryset(self):
         
        policy__name_param = self.request.query_params.get('policy__name__contains',None)
         
        if policy__name_param:
            if "默认" in policy__name_param   :
                self.queryset = self.queryset.filter(policy_id__isnull=True)
                 
            else:
                self.queryset = self.queryset.filter(policy__name__contains=policy__name_param)
        query_org_id = self.request.query_params.get('org_id',None) 
        if query_org_id:
            org_down_list = BrowserPolicyOrgUtil.organ_down_list(query_org_id)
            self.queryset = self.queryset.filter(org_id__in=org_down_list)     
        return self.queryset 
    @staticmethod
    def _change_item_(self,item):   
        return item
    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        policy_id = item['policy_id']
        if not policy_id:
            return  Response("没有策略组信息")
        item = self._change_item_(self,item)
        item['creator'] = request.user.username
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        # update device policy file
         
        logger.info("begin deploy policy file to device")
        policyAccount = models.BrowserBookmarkPolicyAccount.objects.filter(pk= self.kwargs['pk']).first()
        gen_json_file_by_policy_type(policyAccount.account_id,policyAccount.policy_id,BOOKMARK_POLICY_TYPE)
        pre_deploy_to_device(policyAccount)
        deployed_browser_policy_by_type(BOOKMARK_POLICY_TYPE,policyAccount)

        return Response(serializer.data)
    
    @action(methods=['get'], url_path='jobs',detail=True, authentication_classes=[TokenAuthentication])
    def get_jobs(self, request, *args, **kwargs):
        res = service.get_jobs(self.kwargs['pk'],BOOKMARK_POLICY_TYPE)
        if res:
            return Response(data=res)
        return Response("no bookmark policy account data")
         
    @action(methods=['get'], url_path='logs',detail=True, authentication_classes=[TokenAuthentication])
    def get_logs(self, request, *args, **kwargs):
        res = service.get_logs(self.kwargs['pk'],BOOKMARK_POLICY_TYPE)
        if res:
            return Response(data=res)
        return Response("no bookmark policy account log data") 
    @action(methods=['put'], url_path='multiple-org-change-policy',detail=False, authentication_classes=[TokenAuthentication])
    def multiple_org_change_policy(self, request, *args, **kwargs):
        """
        按照部门 策略 批量修改
        """
        organ_ids = request.data.get('org_info_id')
        policy_id = request.data.get('policy_id')

        if policy_id is None:
            return Response({'code': 4001, 'msg': '策略 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if organ_ids is None:
            return Response({'code': 4001, 'msg': '机构为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if not isinstance(organ_ids, list):
            return Response({'code': 4001, 'msg': '机构必须传列表'}, status=status.HTTP_400_BAD_REQUEST)
         
        
        accounts = AccountEntity.objects.all().filter(org_info_id__in=organ_ids)
        service.deploy_policy_to_mult_account(BOOKMARK_POLICY_TYPE,policy_id,organ_ids,accounts)
        if not accounts:
            return Response('部门下没有查询到用户', status=status.HTTP_400_BAD_REQUEST)
          
        return Response("部门批量更新策略成功", status=status.HTTP_200_OK)

 