import datetime
import os
from http.client import ImproperConnectionState
from django.shortcuts import render

import logging
import json
from django.conf import settings
from rest_framework.decorators import action
from rest_framework.views import APIView
from rest_framework.parsers import FormParser  # 解析urlencoded数据包
from rest_framework.parsers import MultiPartParser
from rest_framework.viewsets import ModelViewSet

from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer
from utils.throttling import AnonRateThrottle, UserRateThrottle
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.views import Response
from rest_framework import status
from rest_framework.pagination import PageNumberPagination
from django.db.models import Q
from saltapi import tasks
from utils.RedisHelper import RedisHelper

from saltapi.utils import SaltAPI, DefaultPolicy
from camanager.models import CaDistributeModel, CaInfoModel

from devices.models import Devices
from devices.serializers import DeviceSerializer
from camanager.serializers import CaInfoModelSerializer
from software.serializers import SoftwareInfoModelSerializer
from organizations.models import OrgnizationEntity
from accounts.models import AccountEntity

logger = logging.getLogger(__name__)


# 上传ca file,save to disk
class CaFileUploadView(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)
    parsers_classes = [MultiPartParser, FormParser]

    def put(self, request, filename, format=None):

        # file_obj = request.data['file']
        file_obj = request.FILES.get('file')
        full_filename = '/'.join([settings.CERT_FILES_PATH, filename])
        # image_path = '/'.join([settings.WALLPAPERPATH, image_name])
        try:
            # save file to local
            with open(full_filename, 'wb') as f:
                for chunk in file_obj.chunks():
                    f.write(chunk)
            salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
            if os.environ.get('DEPLOY_ARCH') == 'cluster':
                if not salt_api.api_certfile_add(full_filename, filename):
                    return Response({
                        'code': status.HTTP_400_BAD_REQUEST,
                        'msg': '上传失败',
                        'data': ''
                    })
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '上传完成',
                'data': full_filename
            })
        except Exception as e:
            logger.info('save ca file error' + str(e))
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '上传失败',
                'data': ''
            })


# CA分页列表  新增
class CAInfoList(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    #  分页查询
    def get(self, request, *args, **kwargs):

        # cainfos = CaInfoModel.objects.all()
        cainfos = CaInfoModel.objects.filter(is_deleted=0)
        query_name = request.query_params.get('name')
        if query_name:
            # cainfos = cainfos.filter(Q(name__contains=query_name) | Q(main_version__contains=query_name))
            cainfos = cainfos.filter(Q(name__contains=query_name))
        query_org = request.query_params.get('ca_org')
        if query_org:
            # cainfos = cainfos.filter(Q(name__contains=query_name) | Q(main_version__contains=query_name))
            cainfos = cainfos.filter(Q(ca_org__contains=query_org))

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(cainfos, request)
        except:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'error',
                'success': False,
                'data': []
            })
        res_list = []
        for cainfo in ret:
            res_list.append({
                'sid': cainfo.id,
                'name': cainfo.name,
                'ca_org': cainfo.ca_org,
                'ca_expired': cainfo.ca_expired,
                'ca_desc': cainfo.ca_desc,
                'created_at': cainfo.created_at,
                'last_update': cainfo.last_update,
                'is_default_install': cainfo.is_default_install,
            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '查询Ca信息列表完成',
            'total': len(cainfos),
            'success': True,
            'data': res_list
        })

    # 新增 CA
    def post(self, request, *args, **kwargs):

        req_data = request.data

        cainfo = CaInfoModel()
        cainfo.name = req_data.get('name')
        cainfo.custome_filename = req_data.get('filename')
        cainfo.ca_desc = req_data.get('ca_desc')
        cainfo.ca_org = req_data.get('ca_org')
        cainfo.ca_expired = req_data.get('ca_expired')

        # 判断 是否存在
        check_set = CaInfoModel.objects.filter(name=req_data['name'], is_deleted=0)
        if check_set:
            return Response({
                'code': status.HTTP_226_IM_USED,
                'msg': 'CA信息已存在'
            })

        cainfo.save()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '新增CA信息成功'
        })


# CA信息 详细  删除
class CaInfoDetail(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # CA详细信息
    def get(self, request, *args, **kwargs):
        uid = kwargs.get('id')
        try:
            cainfo = CaInfoModel.objects.get(pk=uid)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '查询不到CA信息'
            })
        # ser_account = AccountEntitySerializer(account, many=True)
        if cainfo:
            res_obj = {
                'id': cainfo.id,
                'name': cainfo.name,
                'ca_desc': cainfo.ca_desc,
                'created_at': cainfo.created_at,
            }
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '查询CA信息完成',
                'data': res_obj
            })
        else:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '获取不到CA信息',
                'data': {}
            })

    # 删除软件信息
    def delete(self, request, *args, **kwargs):
        uid = kwargs.get('id')
        try:
            cainfo = CaInfoModel.objects.get(pk=uid)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'CA信息不存在'
            })
        cainfo.delete()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除CA信息完成'
        })


# CA下发分页列表  新增
class CaDistributeList(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    #  分页查询
    def get(self, request, *args, **kwargs):

        cainfos = CaDistributeModel.objects.all()
        query_name = request.query_params.get('query_name')
        if query_name:
            # cainfos = cainfos.filter(Q(name__contains=query_name) | Q(main_version__contains=query_name))
            cainfos = cainfos.filter(Q(name__contains=query_name))

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(cainfos, request)
        except:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'error',
                'success': False,
                'data': []
            })
        res_list = []
        for cainfo in ret:
            res_list.append({
                'sid': cainfo.id,
                'name': cainfo.name,
                'org_id': cainfo.org_id,
                'custom_path': cainfo.custom_path,
            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '查询Ca下发信息列表完成',
            'total': len(cainfos),
            'success': True,
            'data': res_list
        })

    # 新增 CA
    def post(self, request, *args, **kwargs):

        req_data = request.data

        cainfoDistribute = CaDistributeModel()
        cainfoDistribute.name = req_data.get('name')
        cainfoDistribute.org_id = req_data.get('org_id')
        cainfoDistribute.custom_path = req_data.get('custom_path')
        if not cainfoDistribute.name or not cainfoDistribute.org_id:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请求参数不完整,缺少证书或部门信息'
            })
        # 判断 是否存在
        cainfo = CaInfoModel.objects.filter(name=cainfoDistribute.name).first()
        if not cainfo:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'CA信息不存在'
            })
        cainfoDistribute.save()
        devices = []
        # 通过部门ID --> 终端用户--> 设备 的关系 查找要安装软件到设备
        try:
            org = OrgnizationEntity.objects.get(pk=cainfoDistribute.org_id)
        except Exception as e:
            logger.error('部门信息不存在 ,id is %s', cainfoDistribute.org_id)
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '部门信息不存在'
            })

        accounts = AccountEntity.objects.filter(org_info=org)
        for account in accounts:
            account_devices = Devices.objects.filter(accounts_id=account.id)
            for device in account_devices:
                devices.append(device)
        if not devices:
            return Response({
                'code': status.HTTP_406_NOT_ACCEPTABLE,
                'msg': '部门下没有设备信息可用'
            })
        for dev in devices:
            celery_task = tasks.ca_apply_policy.delay(
                DeviceSerializer(dev).data,
                # SoftwareInstalledSerializer(softwareInstalledInfos, many=True).data
                CaInfoModelSerializer(cainfo).data
            )
            logger.info(celery_task)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '新增CA下发信息成功'
        })


# CA策略保存 查询
class CAInfoPolicyList(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    #  查询所有默认证书
    def get(self, request, *args, **kwargs):

        cainfos = CaInfoModel.objects.filter(is_deleted=0).filter(is_default_install=1)
        query_name = request.query_params.get('name')
        if query_name:
            # cainfos = cainfos.filter(Q(name__contains=query_name) | Q(main_version__contains=query_name))
            cainfos = cainfos.filter(Q(name__contains=query_name))
        query_org = request.query_params.get('ca_org')
        if query_org:
            # cainfos = cainfos.filter(Q(name__contains=query_name) | Q(main_version__contains=query_name))
            cainfos = cainfos.filter(Q(ca_org__contains=query_org))

        res_list = []
        for cainfo in cainfos:
            res_list.append({
                'sid': cainfo.id,
                'name': cainfo.name,
                'ca_org': cainfo.ca_org,
                'ca_expired': cainfo.ca_expired,
                'ca_desc': cainfo.ca_desc,
                'created_at': cainfo.created_at,
                'last_update': cainfo.last_update,

            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '查询所有默认证书完成',
            'total': len(cainfos),
            'success': True,
            'data': res_list
        })

    #  更新CA 系统默认下发状态
    def put(self, request, *args, **kwargs):
        ids = kwargs.get('ids')
        sids = ids.split(',')
        cainfos = CaInfoModel.objects.filter(is_deleted=0)
        for cainfo in cainfos:
            cid = str(cainfo.id)
            if cid in sids:
                cainfo.is_default_install = 1
            else:
                cainfo.is_default_install = 0
            cainfo.save()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '更新CA信息完成'
        })


# CA策略查询何设置  switch
class CAInfoPolicySwitch(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    #  查询
    def get(self, request, *args, **kwargs):
        default_policy = DefaultPolicy()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '查询策略开关完成',
            'success': True,
            'data': int(default_policy.get_default_policy().get('cert'))
        })

    #  更新
    def put(self, request, *args, **kwargs):
        switch = kwargs.get('switch')
        default_policy = DefaultPolicy()
        default_policy.set_default_policy('cert', switch)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '更新默认策略开关完成'
        })


class CaInfoViewSet(ModelViewSet):
    queryset = CaInfoModel.objects.all()
    serializer_class = CaInfoModelSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = None
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        filename = instance.custome_filename
        self.perform_destroy(instance)
        full_filename = '/'.join([settings.CERT_FILES_PATH, filename])
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        if os.environ.get('DEPLOY_ARCH') == 'cluster':
            if not salt_api.api_certfile_delete(filename):
                return Response(f'删除文件失败', status=status.HTTP_400_BAD_REQUEST)
        try:
            os.remove(full_filename)
        except Exception as exp:
            logger.info(f"删除文件失败，{exp}")

        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(methods=['get', 'put'], detail=False, permission_classes=[IsAuthenticated], url_name='base_info')
    def base_info(self, request, pk=None):
        """
        获取默认CA信息
        :param request:
        :param pk:
        :return:
        """
        default_policy = DefaultPolicy()
        if request.method == 'GET':
            data = {'switch': int(default_policy.get_default_policy().get('cert'))}
            return Response(data, status=status.HTTP_200_OK)
        elif request.method == 'PUT':
            switch = request.data.get('switch')
            default_policy.set_default_policy('cert', switch)
            return Response('修改CA状态成功', status=status.HTTP_200_OK)
        else:
            return Response(f'方法{request.method}不被允许', status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, permission_classes=[IsAuthenticated], url_name='ca_info')
    def ca_info(self, request, pk=None):
        """
        更新默认CA信息
        :param request:
        :param pk:
        :return:
        """
        uploaded_files = self.request.FILES.getlist('fileList')

        if len(uploaded_files) > 10:
            return Response('最多一次支持10个文件上传', status=status.HTTP_400_BAD_REQUEST)
        if len(uploaded_files) == 0:
            return Response('请上传文件', status=status.HTTP_400_BAD_REQUEST)

        try:
            id_list = []
            for fileobj in uploaded_files:
                logger.info(f'{fileobj.__dict__}')
                filename = fileobj._name
                full_filename = '/'.join([settings.CERT_FILES_PATH, filename])
                with open(full_filename, 'wb') as f:
                    for chunk in fileobj.chunks():
                        f.write(chunk)
                salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
                if os.environ.get('DEPLOY_ARCH') == 'cluster':
                    if not salt_api.api_certfile_add(full_filename, filename):
                        return Response(f'上传失败', status=status.HTTP_400_BAD_REQUEST)
                res = self.queryset.create(name=filename,
                                           custome_filename=filename,
                                           created_at=datetime.datetime.now(),
                                           last_update=datetime.datetime.now(),
                                           is_default_install=1, is_deleted=0
                                           )

                id_list.append(str(res.id))
        except Exception as e:
            return Response(f'上传失败, 错误信息{e}', status=status.HTTP_400_BAD_REQUEST)

        return Response(','.join(id_list), status=status.HTTP_200_OK)
