# -*- coding: utf-8 -*-
import logging
import datetime
import pandas as pd

from celery_task.celery import app
from utils.client_log_helper import ClientLogHelper
from clientlog.models import ClientLog
from nfs_domain_mainserver.settings import SALTFILESYSTEMPATH
from devices.web_consumers import send_msg_to_web
from message_center.models import MessageCenter

logger = logging.getLogger(__name__)

@app.task(queue='default')
def client_log_add(client_log):
    """
    celery异步任务:客户端日志处理:添加日志
    """
    # 设备没有就不记录日志
    if not client_log or 'device_uuid' not in client_log or 'module' not in client_log or 'action' not in client_log:
        return
    
    client_log = ClientLog(
        device_uuid=client_log.get('device_uuid'),
        device_cnname=client_log.get('device_cnname', ''),
        device_org_id=client_log.get('device_org_id', 0),
        account_name=client_log.get('account_name', None),
        account_cnname=client_log.get('account_cnname', None),
        account_org_id=client_log.get('account_org_id', None),
        module=client_log.get('module'),
        action=client_log.get('action'),
        operate_uuid=client_log.get('operate_uuid', None),
        result_status=client_log.get('result_status', None),
        request=client_log.get('request', None),
        response=client_log.get('response', None),
        device_info_json=client_log.get('device_info_json', None),
        account_info_json=client_log.get('account_info_json', None)
    )
    client_log_helper = ClientLogHelper()
    client_log_helper.add(client_log)


@app.task(queue='default')
def client_log_update_result_status(client_log):
    """
    celery异步任务:客户端日志处理:更新日志的操作结果
    """
    # 设备没有就不记录日志
    if not client_log or 'operate_uuid' not in client_log or 'result_status' not in client_log:
        return
    
    client_log = ClientLog(
        operate_uuid=client_log.get('operate_uuid'),
        result_status=client_log.get('result_status'),
        response=client_log.get('response', None),
    )
    client_log_helper = ClientLogHelper()
    client_log_helper.update_result_status(client_log)


@app.task(queue='default')
def export_clienglog_data(user_name, data, operate_start_time, operate_end_time, log_file_name):
    """
    导出数据,并发送消息
    """
    try:
        # 文件名
        now = datetime.datetime.now()
        timestamp_str = now.strftime('%Y%m%d%H%M%S')
        filename = f'nfs-domain_{log_file_name}日志_{timestamp_str}.xlsx'
        # 如果开始时间存在,则文件名重构为:'{operate_start_time}--{operate_end_time}_{log_content_desc}日志_{timestamp_str}.xlsx'
        if operate_start_time:
            # 格式化为datetime
            operate_start_time_format = datetime.datetime.strptime(operate_start_time, '%Y-%m-%d %H:%M:%S')
            # 如果结束时间没有,则默认今天
            if not operate_end_time:
                operate_end_time = datetime.datetime.strptime(now.strftime('%Y-%m-%d %H:%M:%S'), '%Y-%m-%d %H:%M:%S')
            operate_end_time_format = datetime.datetime.strptime(operate_end_time, '%Y-%m-%d %H:%M:%S')
            # 只要年-月-日
            filename = f'{operate_start_time_format.date()}-{operate_end_time_format.date()}_{log_file_name}日志_{timestamp_str}.xlsx'

        # 将数据转换为DataFrame
        df = pd.DataFrame(data)
        df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"{log_file_name}日志导出完成, {filename}, 总共{len(data)}条数据",
            'create_time': now,
            'target_module': f"{log_file_name}",
            'message_user': user_name,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, user_name)
    except Exception as e:
        logger.error(f"export_clienglog_data error:{e}")