# -*- coding: utf-8 -*-
import json
import logging
import datetime
from rest_framework.views import Response
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.pagination import PageNumberPagination
from rest_framework.viewsets import ModelViewSet
from django.utils.dateparse import parse_datetime
from django.db.models import Q

from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.permission import IsAdminUser
from utils.response import FitJSONRenderer
from utils.throttling import UserRateThrottle

from clientlog.models import ClientLog, ClientLogAction, ClientLogModule
from clientlog.serializers import ClientLogSerializer
from organizations.models import OrgnizationEntity
from clientlog.tasks import export_clienglog_data


logger = logging.getLogger(__name__)

class ClientLogSearchViewSet(ModelViewSet):
    '''
    客户端日志查询
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    serializer_class = ClientLogSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    
    # 当前时间
    now_time = datetime.datetime.now()
    # 今天 0 点
    zero_today = now_time - datetime.timedelta(hours=now_time.hour, minutes=now_time.minute, seconds=now_time.second, microseconds=now_time.microsecond)
        # 今天 23:59:59
    last_today = zero_today + datetime.timedelta(hours=23, minutes=59, seconds=59)
    # 昨天 23:59:59
    last_yerterday = last_today - datetime.timedelta(days=1)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def device_join_domain(self, request, *args, **kwargs):
        '''
        终端加域日志
        '''
        device_cnname = request.GET.get('device_cnname')
        device_org_id = request.GET.get('device_org_id')
        account_cnname = request.GET.get('account_cnname')
        account_name = request.GET.get('account_name')
        result_status = request.GET.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        queryset = ClientLog.objects.filter(module=ClientLogModule.DEVICE.value, action=ClientLogAction.JOIN_DOMAIN.value)
        if device_cnname:
            queryset = queryset.filter(Q(device_cnname__icontains=device_cnname))
        if device_org_id:
            device_org_id = str(device_org_id).split(',')
            queryset = queryset.filter(device_org_id__in=device_org_id)
        if account_cnname:
            queryset = queryset.filter(Q(account_cnname__icontains=account_cnname))
        if account_name:
            queryset = queryset.filter(Q(account_name__icontains=account_name))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')

        # 操作时间格式化,如果失败记录日志
        if operate_time_start:
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            # 获取机构名称
            device_org = OrgnizationEntity.objects.filter(id=item.device_org_id).first()
            result_status_str = ''
            if item.result_status == 1:
                result_status_str = '成功'
            elif  item.result_status == 0:
                result_status_str = '失败'
            request_json = {}
            response_json = {}
            # 尝试解析request
            try:
                request_json = json.loads(item.request)
            except:
                pass
            # 尝试解析response
            try:
                response_json = json.loads(item.response)
            except:
                pass
            item_info = {
                'id': item.id,
                'device_uuid': item.device_uuid,
                'device_cnname': item.device_cnname,
                'device_org_id': item.device_org_id,
                'device_org_name': device_org.name if device_org else '',
                'account_cnname': item.account_cnname,
                'account_name': item.account_name,
                'result_status': item.result_status,
                'result_status_val': result_status_str,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def device_exit_domain(self, request, *args, **kwargs):
        '''
        终端退域日志
        '''
        device_cnname = request.GET.get('device_cnname')
        device_org_id = request.GET.get('device_org_id')
        account_cnname = request.GET.get('account_cnname')
        account_name = request.GET.get('account_name')
        result_status = request.GET.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        queryset = ClientLog.objects.filter(module=ClientLogModule.DEVICE.value, action=ClientLogAction.EXIT_DOMAIN.value)
        if device_cnname:
            queryset = queryset.filter(Q(device_cnname__icontains=device_cnname))
        if device_org_id:
            device_org_id = str(device_org_id).split(',')
            queryset = queryset.filter(device_org_id__in=device_org_id)
        if account_cnname:
            queryset = queryset.filter(Q(account_cnname__icontains=account_cnname))
        if account_name:
            queryset = queryset.filter(Q(account_name__icontains=account_name))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            # 获取机构名称
            device_org = OrgnizationEntity.objects.filter(id=item.device_org_id).first()
            result_status_str = ''
            if item.result_status == 1:
                result_status_str = '成功'
            elif  item.result_status == 0:
                result_status_str = '失败'
            request_json = {}
            response_json = {}
            # 尝试解析request
            try:
                request_json = json.loads(item.request)
            except:
                pass
            # 尝试解析response
            try:
                response_json = json.loads(item.response)
            except:
                pass
            item_info = {
                'id': item.id,
                'device_uuid': item.device_uuid,
                'device_cnname': item.device_cnname,
                'device_org_id': item.device_org_id,
                'device_org_name': device_org.name if device_org else '',
                'account_cnname': item.account_cnname,
                'account_name': item.account_name,
                'result_status': item.result_status,
                'result_status_val': result_status_str,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def account_login(self, request, *args, **kwargs):
        '''
        终端用户登录登出日志
        '''
        account_name = request.GET.get('account_name')
        account_cnname = request.GET.get('account_cnname')
        account_org_id = request.GET.get('account_org_id')
        device_cnname = request.GET.get('device_cnname')
        result_status = request.GET.get('result_status')
        action = request.GET.get('action') #登录或登出
        operate_time_type = request.GET.get('operate_time_type')
        queryset = ClientLog.objects.filter(module=ClientLogModule.ACCOUNT.value)
        if device_cnname:
            queryset = queryset.filter(device_cnname=device_cnname)
        if account_org_id:
            account_org_id = str(account_org_id).split(',')
            queryset = queryset.filter(account_org_id=account_org_id)
        if account_cnname:
            queryset = queryset.filter(account_cnname=account_cnname)
        if account_cnname:
            queryset = queryset.filter(Q(account_cnname__icontains=account_cnname))
        if account_name:
            queryset = queryset.filter(Q(account_name__icontains=account_name))
        if action:
            queryset = queryset.filter(action=action)
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_400_BAD_REQUEST)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            # 获取机构名称
            device_org = OrgnizationEntity.objects.filter(id=item.account_org_id).first()
            result_status_str = ''
            if item.result_status == 1:
                result_status_str = '成功'
            elif  item.result_status == 0:
                result_status_str = '失败'
            request_json = {}
            response_json = {}
            # 尝试解析request
            try:
                request_json = json.loads(item.request)
            except:
                pass
            # 尝试解析response
            try:
                response_json = json.loads(item.response)
            except:
                pass
            item_info = {
                'id': item.id,
                'account_name': item.account_name,
                'account_cnname': item.account_cnname,
                'account_org_id': item.account_org_id,
                'account_org_name': device_org.name if device_org else '',
                'device_cnname': item.device_cnname,
                'action': item.action,
                'action_desc': '登录' if item.action == ClientLogAction.ACCOUNT_LOGIN.value else '登出',
                'result_status': item.result_status,
                'result_status_val': result_status_str,
                'request': request_json,
                'response': response_json,
                'operation_time': item.operation_time.strftime(format_str),
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_device_join_domain(self, request, *args, **kwargs):
        '''
        日志导出:终端加域
        '''
        device_cnname = request.GET.get('device_cnname')
        device_org_id = request.GET.get('device_org_id')
        account_cnname = request.GET.get('account_cnname')
        account_name = request.GET.get('account_name')
        result_status = request.GET.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        queryset = ClientLog.objects.filter(module=ClientLogModule.DEVICE.value, action=ClientLogAction.JOIN_DOMAIN.value)
        if device_cnname:
            queryset = queryset.filter(Q(device_cnname__icontains=device_cnname))
        if device_org_id:
            device_org_id = str(device_org_id).split(',')
            queryset = queryset.filter(device_org_id__in=device_org_id)
        if account_cnname:
            queryset = queryset.filter(Q(account_cnname__icontains=account_cnname))
        if account_name:
            queryset = queryset.filter(Q(account_name__icontains=account_name))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')

        # 操作时间格式化,如果失败记录日志
        if operate_time_start:
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():    
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 获取机构名称
            device_org_name = ''
            device_org = OrgnizationEntity.objects.filter(id=ret_item.device_org_id).first()
            if device_org:
                device_org_name = device_org.name
            result_status_str = ''
            if ret_item.result_status == 1:
                result_status_str = '成功'
            elif  ret_item.result_status == 0:
                result_status_str = '失败'
            
            # 转成中文描述
            data_item["终端别名"] = ret_item.device_cnname
            data_item["姓名"] = ret_item.account_cnname
            data_item["用户账号"] = ret_item.account_name
            data_item["所属部门"] = device_org_name
            data_item["行为"] = '加域'
            data_item["结果"] = result_status_str
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)

        # 异步任务:日志下载
        export_clienglog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '客户端日志_终端加域')
        return Response("客户端日志_终端加域日志导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)

    
    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_device_exit_domain(self, request, *args, **kwargs):
        '''
        日志导出:终端退域
        '''
        device_cnname = request.GET.get('device_cnname')
        device_org_id = request.GET.get('device_org_id')
        account_cnname = request.GET.get('account_cnname')
        account_name = request.GET.get('account_name')
        result_status = request.GET.get('result_status')
        operate_time_type = request.GET.get('operate_time_type')
        queryset = ClientLog.objects.filter(module=ClientLogModule.DEVICE.value, action=ClientLogAction.EXIT_DOMAIN.value)
        if device_cnname:
            queryset = queryset.filter(Q(device_cnname__icontains=device_cnname))
        if device_org_id:
            device_org_id = str(device_org_id).split(',')
            queryset = queryset.filter(device_org_id__in=device_org_id)
        if account_cnname:
            queryset = queryset.filter(Q(account_cnname__icontains=account_cnname))
        if account_name:
            queryset = queryset.filter(Q(account_name__icontains=account_name))
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 获取机构名称
            device_org_name = ''
            device_org = OrgnizationEntity.objects.filter(id=ret_item.device_org_id).first()
            if device_org:
                device_org_name = device_org.name
            result_status_str = ''
            if ret_item.result_status == 1:
                result_status_str = '成功'
            elif  ret_item.result_status == 0:
                result_status_str = '失败'
            # 转成中文描述
            data_item["终端别名"] = ret_item.device_cnname
            data_item["姓名"] = ret_item.account_cnname
            data_item["用户账号"] = ret_item.account_name
            data_item["所属部门"] = device_org_name
            data_item["行为"] = '退域'
            data_item["结果"] = result_status_str
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)

        # 异步任务:日志下载
        export_clienglog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '客户端日志_终端退域')
        return Response("客户端日志_终端退域日志导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def export_account_login(self, request, *args, **kwargs):
        '''
        日志导出:终端用户登录登出
        '''
        account_name = request.GET.get('account_name')
        account_cnname = request.GET.get('account_cnname')
        account_org_id = request.GET.get('account_org_id')
        device_cnname = request.GET.get('device_cnname')
        result_status = request.GET.get('result_status')
        action = request.GET.get('action') #登录或登出
        operate_time_type = request.GET.get('operate_time_type')
        queryset = ClientLog.objects.filter(module=ClientLogModule.ACCOUNT.value)
        if device_cnname:
            queryset = queryset.filter(device_cnname=device_cnname)
        if account_org_id:
            account_org_id = str(account_org_id).split(',')
            queryset = queryset.filter(account_org_id=account_org_id)
        if account_cnname:
            queryset = queryset.filter(account_cnname=account_cnname)
        if account_cnname:
            queryset = queryset.filter(Q(account_cnname__icontains=account_cnname))
        if account_name:
            queryset = queryset.filter(Q(account_name__icontains=account_name))
        if action:
            queryset = queryset.filter(action=action)
        if result_status:
            queryset = queryset.filter(result_status=result_status)
        # 日期查询
        operate_time_start = None
        operate_time_end = None
        # 筛选类型:近一周,近1个月,近3个月
        if operate_time_type:
            if operate_time_type == 'one_week':
                # 一周前的0点
                one_week_last_zero = self.zero_today - datetime.timedelta(days=7)
                one_week_last_zero = one_week_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_week_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'one_month':
                # 一月前(按30天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=30)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)                
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'three_month':
                # 三个月前(按90天来算)的0点
                one_month_last_zero = self.zero_today - datetime.timedelta(days=90)
                one_month_last_zero = one_month_last_zero.replace(hour=0, minute=0, second=0, microsecond=0)
                operate_time_start = one_month_last_zero.strftime('%Y-%m-%d %H:%M:%S')
                operate_time_end = self.last_yerterday.strftime('%Y-%m-%d %H:%M:%S')
            elif operate_time_type == 'custom':
                # 客户自定义时间
                operate_time_start = request.GET.get('operate_time_start')
                operate_time_end = request.GET.get('operate_time_end')
        if operate_time_start:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_start_format = parse_datetime(operate_time_start)
                queryset = queryset.filter(operation_time__gte=operate_time_start_format)
            except:
                logger.error(f'parse operate_time_start error, operate_time_start={operate_time_start}')
        if operate_time_end:
            # 操作时间格式化,如果失败记录日志
            try:
                operate_time_end_format = parse_datetime(operate_time_end)
                queryset = queryset.filter(operation_time__lte=operate_time_end_format)
            except:
                logger.error(f'parse operate_time_end error, operate_time_end={operate_time_end}')
        if not queryset.exists():            
            return Response('没有数据', status=status.HTTP_400_BAD_REQUEST)
        
        # 倒序
        queryset = queryset.order_by("-id")
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        data = []
        for ret_item in queryset:
            data_item = {}
            # 获取机构名称
            device_org_name = ''
            device_org = OrgnizationEntity.objects.filter(id=ret_item.account_org_id).first()
            if device_org:
                device_org_name = device_org.name
            result_status_str = ''
            if ret_item.result_status == 1:
                result_status_str = '成功'
            elif  ret_item.result_status == 0:
                result_status_str = '失败'

            # 转成中文描述
            data_item["姓名"] = ret_item.account_cnname
            data_item["用户账号"] = ret_item.account_name
            data_item["终端别名"] = ret_item.device_cnname
            data_item["所属部门"] = device_org_name
            data_item["行为"] = '登录' if ret_item.action == ClientLogAction.ACCOUNT_LOGIN.value else '登出'
            data_item["结果"] = result_status_str
            data_item["行为时间"] = ret_item.operation_time.strftime(format_str)
            data.append(data_item)
        # 异步任务:日志下载
        export_clienglog_data.delay(request.user.username, data, operate_time_start, operate_time_end, '客户端日志_终端用户登录登出')
        return Response("客户端日志_终端用户登录登出日志导出中, 请稍后前往消息中心查看", status=status.HTTP_200_OK)

