"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2024/1/23 下午1:52
@File: client_consumers.py
@Desc: 终端连接服务器websocket
       redis 缓存keys的用途  ：  client_channel_name  终端设备通道表
                               actives              终端在线信息表
                               device_user_active   终端用户信息在线表
                               remote_assist        远程协助参数记录表
                               client_websocket_unsent 终端不在线服务器给终端发送消息缓存表

"""
import copy
import datetime
import json
import logging
import os
try:
    import aioredis
    aioredis_import = 'aioredis'
except ImportError:
    from redis import asyncio as aioredis
    aioredis_import = 'asyncio'
from channels.generic.websocket import AsyncWebsocketConsumer
from channels.layers import get_channel_layer
from django_redis import get_redis_connection
from asgiref.sync import async_to_sync, sync_to_async

from accounts.freeipa import check_user, user_find
from accounts.ldaps import get_ldap_ad
from accounts.models import AccountEntity
from devices.models import Devices
from devices.utils import bind_device_account
from nfs_domain_mainserver import settings
from organizations.models import OrgnizationEntity
from patch_management.models import PatchDeviceLog, PatchInfo, PatchFile
from remote_assist.models import RemoteAssistDevice, StatusEnum
from saltapi.models import SaltJidDevice
from software.models import SoftwareInstalledModel
from tasks.models import TaskDeviceExec, Tasks
from utils.RedisHelper import RedisHelper
from utils.authentication import DomainLicenseAuth
from utils.redis_domain_device import RedisCacheDevice
from utils.sch_soap_task_member import get_soapmember, SoapMember
from utils.utils import get_uuid, read_licenseB, bytes_to_string, is_satisfy_black_lists, sm4_decrypt, sm4_hash
from utils.reports_helper import ReportsHelper
from clientlog.models import ClientLogAction, ClientLogModule, ClientLog
from clientlog.tasks import client_log_add, client_log_update_result_status

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')
channel_layer = get_channel_layer()


class ClientConsumer(AsyncWebsocketConsumer):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.is_connected = None
        self.client_uuid = None
        self.client_info = None
        self.group_name = 'client_channel'  # 广播频道
        self.actions = {
            '1-1-0': self.verification_code,  # 1-1-0  发送验证码
            '1-1-1': self.add_domain,  # 1-1-1  加域
            '1-1-2': self.add_domain_result,  # 1-1-2  加域成功和失败
            '1-1-3': self.exit_domain,  # 1-1-3  退域
            '1-1-4': self.exit_domain_result,  # 1-1-4  退域 成功或失败
            '1-1-5': self.account_info,  # 1-1-5  域用户信息
            '9-1-1': self.account_online,  # 9-1-1  用户登陆（获取用户在线信息）
            '9-1-2': self.account_verify,  # 9-1-2 校验用户密码信息包括短信验证校验
            '9-2-1': self.device_install_info,  # 9-2-1  处理终端软件安装信息
            # 9-5-6  查询终端补丁信息
            '9-5-7': self.device_patch_status,  # 9-5-7  返回终端补丁安装状态
            # 9-6-6,  补丁下发命令
            '9-6-7': self.patch_install_status,  # 9-6-7  补丁安装终端返回信息处理
            '9-7-1': self.device_auto_domain,  # 9-7-1  客户端自主加域 salt
            '9-8-1': self.device_ver_info,  # 9-8-1  获取终端版本准备远程连接
            '9-8-5': self.device_remote,  # 9-8-5  终端反馈远程协助结果（当需要终端用户点击确认远程时）
            '10-1-1': self.device_remote_policy,  # 10-1-1  终端反馈远程协助策略
            '9-9-7': self.client_unsent_msg,  # 9-9-7 获取终端对应的未发送指令
            '9-9-8': self.heart_beat,  # 9-9-8  心跳包 检查终端是否在域中
            '9-9-9': None,  # 9-9-9  终端主动断开websocket
            # '10-1-1': self.group_policy_receiver,  # 10-1-1  组策略消息处理
        }
        # 统一websocket服务端返回信息
        self.server_result_msg = {'sender': 'SERVER', 'sn': None, 'data': None, 'message': None}

    async def connect(self):
        # 创建异步 Redis 连接
        if aioredis_import == 'aioredis':
            self.redis_conn = await aioredis.create_redis(
                # 使用3号库，websocket连接通道存在redis3号库中
                "redis://{}:{}/3".format(os.environ.get("REDIS_HOST"), os.environ.get('REDIS_PORT'))
            )
        else:
            self.redis_conn = await aioredis.from_url(
                # 使用3号库，websocket连接通道存在redis3号库中
                "redis://{}:{}/3".format(os.environ.get("REDIS_HOST"), os.environ.get('REDIS_PORT'))
            )

        # 当 WebSocket 连接建立时调用
        await self.accept()
        self.client_info = self.scope["client"] if "client" in self.scope.keys() else None
        client_uuid = self.scope['query_string'].decode('utf-8')
        client_uuid = client_uuid.split('hostname=')[-1]
        if not bool(client_uuid):
            client_uuid = None
        else:
            self.client_uuid = client_uuid
            # 使用异步方式管理 Redis 连接
            client_channel_name = await self.redis_conn.hget('client_channel_name', self.client_uuid)
            if client_channel_name:
                client_channel_name = client_channel_name.decode('utf-8')
            if self.channel_name != client_channel_name:
                await self.redis_conn.hset('client_channel_name', self.client_uuid, self.channel_name)

            # 记录终端在线信息
            if not await self.redis_conn.hget('actives', self.client_uuid):
                await self.redis_conn.hset('actives', self.client_uuid, 1)

            # 统计数据入库:设备的上线时间
            reports_helper = ReportsHelper()
            await reports_helper.save_history_device_online_up(self.client_uuid)

        logger.info(f"终端{self.client_info[0]}的主机名称({client_uuid}), 连接服务器")
        # 加入 chat group
        await self.channel_layer.group_add(
            self.group_name,  # 广播频道
            self.channel_name  # 通道名称
        )
        self.is_connected = True
        # device connected,send policy to device
        # server_msg = self.get_device_policy()
        # await self.send(text_data=json.dumps(server_msg, ensure_ascii=False))
        # logger.info(f"向终端{self.client_info[0]}的主机名称（{self.client_uuid}）发送的消息{server_msg}")

    async def disconnect(self, close_code):
        logging.info(f"WebSocket connection closed with code: {close_code}")
        # 当 WebSocket 连接关闭时调用
        if self.client_uuid is not None:
            device_user_active = await self.redis_conn.hget('device_user_active', self.client_uuid)
            if device_user_active:
                device_user_active = device_user_active.decode('utf-8')
                device_user_active = json.loads(device_user_active)
            device_user_active = device_user_active if isinstance(device_user_active, list) else []

            # 使用异步方式管理 Redis 连接
            await self.redis_conn.hdel('client_channel_name', self.client_uuid)  # 清除终端连接通道信息
            await self.redis_conn.hdel('actives', self.client_uuid)  # 清除终端在线信息
            await self.redis_conn.hdel('device_user_active', self.client_uuid)  # 清除终端上用户在线信息

            # 统计数据入库:1-设备的下线时间,2-终端用户的下线时间
            reports_helper = ReportsHelper()
            await reports_helper.save_history_device_online_down(self.client_uuid)

            if device_user_active:
                for item in device_user_active:
                    await reports_helper.save_history_account_online_down(item, self.client_uuid)

        # 离开 chat group
        await self.channel_layer.group_discard(
            self.group_name,
            self.channel_name
        )
        self.is_connected = False
        logger.info(f"终端{self.client_info[0]}的主机名称（{self.client_uuid}）, 断开服务器连接")

    async def receive(self, text_data=None, bytes_data=None):
        # 当从 WebSocket 接收到消息时调用
        text_data_json = json.loads(text_data)

        # 在这里可以添加处理接收到的消息的逻辑
        # 发送消息给 特定通道
        await self.channel_layer.send(
            self.channel_name,
            {
                'type': 'get_message',
                'message': text_data_json
            }
        )

    async def get_message(self, event):
        # 当从 send_message 中收到消息时调用，广播消息
        message = event['message']
        server_msg = await self.msg_sn_input(message)

        # 使用异步方式获取 client_channel_name
        client_channel_name = await self.redis_conn.hget('client_channel_name',
                                                         self.client_uuid) if self.client_uuid is not None else None

        # 如果没有 client_channel_name 或 server_msg 为 None，直接返回
        if client_channel_name is None or server_msg is None:
            return
        # 发送消息到 WebSocket， 当前客户端连接存在再发送
        if self.is_connected:
            if server_msg.get("sn") != '9-9-8':
                logger.info(f"向终端{self.client_info[0]}的主机名称（{self.client_uuid}）发送的消息{server_msg}")
            await self.send(text_data=json.dumps(server_msg, ensure_ascii=False))

    async def send_message(self, event):
        server_msg = event['message']
        logger.info(f"向终端{self.client_info[0]}的主机名称（{self.client_uuid}）发送的消息{server_msg}")
        await self.send(text_data=json.dumps(server_msg, ensure_ascii=False))

    @sync_to_async
    def msg_sn_input(self, option):
        """
        通过指令转为相应的消息处理机制
        :param option: {"sender": "CLIENT","sn": "1-1-1","data": None,}
        :return:
        """
        client_sn = option.get('sn')
        msg_data = option.get('data')

        # 检查选项是否在字典中
        if client_sn in self.actions:
            # 执行与选项对应的操作
            server_msg = self.actions[client_sn](msg_data)
        else:
            # 执行默认操作
            server_msg = self.default_action(option)

        return server_msg

    """********以下是指令对应的方法*******"""

    def verification_code(self, msg_data):
        """
        获取验证码
        :param msg_data: {sender: client，sn: 1-1-0，data: {string userNum: xxx}}
        :return: {"sender": "SERVER",
        "sn": "1-1-0",
        "data": {"code": 400 | 200,"userNum": xxx，"message": '发送失败！短信异常员工号不存在。'}
        }
        """
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        logger.info("======= 1-1-0 ======")
        logging.info(msg_data)
        req_uuid = get_uuid()
        user_num = msg_data.get('userNum')
        domain_auth = DomainLicenseAuth()
        if not domain_auth.license_b:
            result_msg.update({'sn': '1-1-0',
                               'data': {"code": 400, "message": '服务器未激活,请联系管理员', 'uuid': req_uuid}
                               })
            return result_msg
        sm = get_soapmember()
        if not bool(sm):
            result_msg.update({'sn': '1-1-0',
                               'data': {"code": 400, "message": '服务器未添加统一认证,请联系管理员', 'uuid': req_uuid}
                               })
            return result_msg
        account = AccountEntity.objects.filter(name=f'{sm.user_prefix}{user_num}').first()
        if account:
            domain_user_create_type = account.create_type
        else:
            domain_user_create_type = 0
        if (user_num != settings.DOMAIN_SERVER_ADMIN) and (domain_user_create_type == 2) and (
                sm.client.connect()):
            soap_data = {
                "userId": user_num,
            }
            send_res = sm.send_code(soap_data)
            if send_res.get('status') == 'error':
                result_msg.update({'sn': '1-1-0',
                                   'data': {"code": 400, "message": send_res.get('message'), "userNum": user_num,
                                            'uuid': req_uuid}})
                return result_msg
        result_msg.update({'sn': '1-1-0', 'data': {"code": 200, "userNum": user_num, 'uuid': req_uuid}})
        return result_msg

    def add_domain(self, msg_data):
        """
        终端请求是否可以加域
        :param msg_data: 服务接收的消息
                        {sender: client
                        sn: 1-1-1
                        data: {
                          string ip: 127.0.0.1
                          string domain_user: test001
                          string domain_user_pwd: 123456
                          string device_uid: *****    设备ID (MAC+SN+硬盘序列号)
                          string device_sid: ****     设备SID
                          string device_cnname: xxx
                          int freejoin: 1  0不是，1 是
                          string authType: M_PASSWORD|密码(默认), M_FINGER|指纹, M_FINGER_CARD|指纹卡, M_OTP|短信
                          string verifyCode:  为短信认证时必输入
                          string fingerPassword:  为指纹卡时必输入
                          string fingerData:  为指纹时必输入
                        }}
        :return: 服务发送消息
                 {"sender": "SERVER",
                        "sn": "1-1-1",
                        "data": {
                            "code": 200,
                            "message": f"{hostname} 可以加域",
                            "user":"nfs-test001",
                            "domain_server_fqdn": server.os-domain.cpu-os.com
                            "adddomaincmd": 'ipa-client-install
                            --server server.os-domain.cpu-os.com
                            --domain os-domain.cpu-os.com
                            --hostname test-9-13.os-domain.cpu-os.com
                            --password 12345678
                            --principal admin
                            --ntp-server server.os-domain.cpu-os.com
                            --force-join --mkhomedir --unattended  '
                        }  200|可以加域  400|不可以加域
                }
        """
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        req_uuid = get_uuid()
        msg_data['uuid'] = req_uuid
        logger.info(f"处理 1-1-1，接收消息 {msg_data}")
        device_ip = msg_data.get('ip', None)
        domain_user = msg_data.get('domain_user')
        domain_user_pwd = msg_data.get('domain_user_pwd')
        device_cnname = msg_data.get('device_cnname', None)
        device_uuid = device_hostname = msg_data.get('device_hostname')
        device_uid = msg_data.get('device_uid')
        device_sid = msg_data.get('device_sid')
        freejoin = msg_data.get('freejoin')
        auth_type = msg_data.get('authType', 'M_PASSWORD')
        verify_code = msg_data.get('verifyCode')
        finger_data = msg_data.get('fingerData')
        finger_password = msg_data.get('fingerPassword')
        domain_auth = DomainLicenseAuth()
        if not domain_auth.license_b:
            result_msg.update({"sn": "1-1-1",
                               "data": {"code": 400, "message": '服务器未激活,请联系管理员', 'uuid': req_uuid}})
            return result_msg
        mac = os.environ.get("DOMAIN_MAC")
        license_info = read_licenseB(mac, domain_auth.license_b)
        max_users = bytes_to_string(license_info.get('max_users_res', 0))
        if Devices.objects.exclude(is_domain=0).all().count() >= int(max_users):
            result_msg.update({"sn": "1-1-1",
                               "data": {"code": 400, "message": '超出最大终端管控数,请联系管理员', 'uuid': req_uuid}})
            return result_msg
        if device_ip is None:
            result_msg.update({"sn": "1-1-1",
                               "data": {"code": 400, "message": '缺少设备ip', 'uuid': req_uuid}
                               })
            return result_msg
        if device_cnname is None:
            result_msg.update({"sn": "1-1-1",
                               "data": {"code": 400, "message": '缺少设备别名', 'uuid': req_uuid}
                               })
            return result_msg
        if device_hostname is None:
            result_msg.update({"sn": "1-1-1",
                               "data": {"code": 400, "message": '缺少设备主机名称', 'uuid': req_uuid}
                               })
            return result_msg
        if RedisCacheDevice().device_is_domain(device_hostname):
            result_msg.update({"sn": "1-1-1",
                               "data": {"code": 400, "message": '主机名已被使用，请修改终端别名', 'uuid': req_uuid}
                               })
            return result_msg
        # device_cnname = init_device_hostname(domain_user)
        if is_satisfy_black_lists(domain_user):
            result_msg.update({"sn": "1-1-1",
                               "data": {"code": 400, "message": '用户符合黑名单规则,禁止加域', 'uuid': req_uuid}
                               })
            return result_msg
        ldap_ad = get_ldap_ad()
        sm = get_soapmember()
        if not freejoin:
            if not domain_user or not domain_user_pwd:
                result_msg.update({"sn": "1-1-1",
                                   "data": {"code": 400, "message": '缺少域用户或密码', 'uuid': req_uuid}
                                   })
                return result_msg
            if domain_user != settings.DOMAIN_SERVER_ADMIN and ldap_ad:
                account = AccountEntity.objects.filter(name=domain_user).first()
                if account:
                    if account.create_type == 1:
                        check_res = ldap_ad.auth_user(domain_user, sm4_decrypt(domain_user_pwd, settings.SM4_KEY))
                        if not check_res.get('status'):
                            result_msg.update({"sn": "1-1-1",
                                               "data": {"code": 400, "message": check_res.get('message'),
                                                        'uuid': req_uuid}
                                               })
                            return result_msg
                    else:
                        check_res = check_user(domain_user, sm4_decrypt(domain_user_pwd, settings.SM4_KEY))
                        if not check_res.get('exist'):
                            result_msg.update({"sn": "1-1-1",
                                               "data": {"code": 400, "message": '用户不存在或密码错误或已被锁定',
                                                        'uuid': req_uuid}
                                               })
                            return result_msg
                else:
                    result_msg.update({"sn": "1-1-1",
                                       "data": {"code": 400, "message": '此账号暂未同步', 'uuid': req_uuid}
                                       })
                    return result_msg
            elif domain_user != settings.DOMAIN_SERVER_ADMIN and sm:
                account = AccountEntity.objects.filter(name=f'{sm.user_prefix}{domain_user}').first()
                if account:
                    if (account.create_type == 2) and (sm.client.connect()):
                        soap_data = {
                            "appId": sm.appid,
                            "userId": domain_user,
                            "password": sm4_decrypt(domain_user_pwd, sm.soap_sm4_keys),
                            "requestId": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
                            "authType": auth_type,
                            "requestName": None,
                            "requestReason": None,
                            'authFlag': "1"
                        }
                        if auth_type == 'M_OTP':
                            soap_data['verifyCode'] = verify_code
                        elif auth_type == 'M_FINGER':
                            soap_data['fingerData'] = finger_data
                        elif auth_type == 'M_FINGER_CARD':
                            soap_data['fingerPassword'] = finger_password
                        check_res = sm.get_auth_by_sm4(soap_data)
                        if check_res.get('status') == 'error' and check_res.get('code') not in (10009, 10200):
                            result_msg.update({"sn": "1-1-1",
                                               "data": {"code": 400, "message": check_res.get('message'),
                                                        'uuid': req_uuid}
                                               })
                            return result_msg
                        domain_user = f'{sm.user_prefix}{domain_user}'
                    else:
                        check_res = check_user(domain_user, sm4_decrypt(domain_user_pwd, settings.SM4_KEY))
                        if not check_res.get('exist'):
                            result_msg.update({"sn": "1-1-1",
                                               "data": {"code": 400, "message": '用户不存在或密码错误或已被锁定',
                                                        'uuid': req_uuid}
                                               })
                            return result_msg
                else:
                    result_msg.update({"sn": "1-1-1",
                                       "data": {"code": 400, "message": '此账号暂未同步', 'uuid': req_uuid}
                                       })
                    return result_msg
            else:
                check_res = check_user(domain_user, sm4_decrypt(domain_user_pwd, settings.SM4_KEY))
                if not check_res.get('exist'):
                    result_msg.update({"sn": "1-1-1",
                                       "data": {"code": 400, "message": '用户不存在或密码错误或已被锁定',
                                                'uuid': req_uuid}
                                       })
                    return result_msg
        else:
            if sm:
                domain_user = f'{sm.user_prefix}{domain_user}'
            if not user_find(domain_user):
                result_msg.update({"sn": "1-1-1",
                                   "data": {"code": 400, "message": '用户不存在', 'uuid': req_uuid}
                                   })
                return result_msg
        if redis_conn.get(f'{device_hostname}_adding_domain'):
            result_msg.update({"sn": "1-1-1",
                               "data": {"code": 400, "message": f"{device_hostname} 正在加域"}
                               })
            return result_msg
        device, is_create = Devices.objects.get_or_create(device_uuid=device_uuid)
        if device.is_domain:
            result_msg.update({
                "sn": "1-1-1",
                "data": {"code": 400, "message": '终端已在域中', 'uuid': req_uuid}
            })
            return result_msg
        device.device_ip = device_ip
        device.device_hostname = device_hostname
        device.device_cnname = device_cnname
        device.device_uid = device_uid
        device.device_sid = device_sid
        device.save()
        # domain_uuid = str(uuid.uuid4())
        # conn.hset(domain_uuid, {'salt':False, 'freeipa':False}, 60 * 30)
        cmd = f'ipa-client-install --server {os.environ.get("DOMAIN_SERVER_FQDN")} --domain {os.environ.get("DOMAIN_NAME")} --hostname {device_hostname} --password  {os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD")} --principal admin --no-ntp --enable-dns-update --force-join --unattended'
        result_msg.update({"sn": "1-1-1",
                           "data": {
                               "code": 200,
                               "message": f"{device_hostname} 可以加域",
                               "adddomaincmd": sm4_hash(cmd, settings.SM4_KEY),
                               'uuid': req_uuid,
                               'user': domain_user.lower(),
                               'domain_server_fqdn': os.environ.get("DOMAIN_SERVER_FQDN"),
                               'device_cnname': device_cnname
                           }
                           })
        redis_conn.set(f'{device_hostname}_adding_domain', 1, ex=100, nx=True)

        # 记录客户端日志(异步)
        client_log = {
            'device_uuid': device_uuid,
            'device_cnname': device.device_cnname,
            'device_org_id': device.org_info.id,
            'account_name': domain_user,
            'module': ClientLogModule.DEVICE.value,
            'action': ClientLogAction.JOIN_DOMAIN.value,
            'operate_uuid': req_uuid,
            'request': json.dumps(msg_data)
        }
        client_log_add.delay(client_log=client_log)
        
        return result_msg

    def add_domain_result(self, msg_data):
        """
        客户端加域结果
        :param msg_data: 服务端收到的消息
                {sender: 1
                sn: 1-1-2
                data: {
                  string device_hostname: xxx.os-domain.cpu-os.com
                  string domain_user: xxx
                  string ip: 120.7.0.1
                  string message: 加域失败原因  当客户端加域失败后，将分析的原因发送给后端，后端在将失败的原因返回给客户端
                  int result: 0  0|加域成功 1|加域失败
                  string log: xxxxxxxx (暂时不用)
                }}
        :return: 服务发送消息
                 {
                 "sender": "SERVER",
                 "sn": "1-1-2",
                 "data": {"code": 200, "message": "加域成功"}  200|加域成功  400|加域失败
                 }
        """
        logger.info(msg_data)
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        result = msg_data.get('result')
        req_uuid = msg_data.get('uuid')
        domain_user = msg_data.get('domain_user')
        device_ip = msg_data.get('ip')
        message = msg_data.get('message')
        device_hostname = msg_data.get('device_hostname')
        if result == 0:
            device = Devices.objects.filter(device_uuid=device_hostname).first()
            account = AccountEntity.objects.filter(name=domain_user).first()
            device.add_domain_time = datetime.datetime.now()
            device.is_domain = 1
            if domain_user != os.environ.get('DOMAIN_SERVER_ADMIN'):
                account = AccountEntity.objects.filter(name=domain_user).first()
                device.org_info = account.org_info
                device.bind_account.add(account)
                device.belong_user = account.cn_name if account.cn_name else domain_user
            device.save()
            bind_device_account(device_hostname, account.id)
            result_msg.update({
                "sn": "1-1-2",
                "data": {"code": 200, "message": '加域成功', 'uuid': req_uuid}
            })
            signal = {
                'event_type': 'install',
                'device_ip': device_ip,
                'device_hostname': device_hostname,
                'success': True,
                'msg': f'终端{device_hostname}加域成功',
                'os': msg_data.get('os') or 'Nfsdesktop',
                'os_release': msg_data.get('os_release') or '5.0',
                'os_arch': msg_data.get('os_arch') or 'x86_64'
            }
            # 更新客户端日志中的操作结果(异步)
            client_log = {
                'operate_uuid': req_uuid,
                'result_status': 1,
                'response': json.dumps(msg_data)
            }
            client_log_update_result_status.delay(client_log=client_log)

            public = RedisHelper()
            public.public(json.dumps(signal), chan='freeipa')
            return result_msg
        # alter_time = datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S.%f')
        # full_ret = {"cmd":"_return","id":device_hostname,"success":False, "return":{"ws.domain.log":log}}
        # SaltJidDevice.objects.add_jid('', 'ws.domain', jid, device_hostname, full_ret.get('return'), 0,
        #                               full_ret,
        #                               alter_time, ['ws.domain'])
        else:
            result_msg.update({
                "sn": "1-1-2",
                "data": {"code": 400, "message": f'加域失败 {message}', 'uuid': req_uuid}
            })
            signal = {
                'event_type': 'install',
                'device_ip': device_ip,
                'device_hostname': device_hostname,
                'success': False,
                'msg': f'终端{device_hostname}加域失败',
            }
            # 更新客户端日志中的操作结果(异步)
            client_log = {
                'operate_uuid': req_uuid,
                'result_status': 0,
                'response': json.dumps(msg_data)
            }
            client_log_update_result_status.delay(client_log=client_log)

            public = RedisHelper()
            public.public(json.dumps(signal), chan='freeipa')
            return result_msg

    def exit_domain(self, msg_data):
        """
        终端退域
        :param msg_data:
        :return: 服务发送消息
                 {
                 "sender": "SERVER",
                 "sn": "1-1-3",
                 "data": {
                    "exit_domain": 1
                    "auto_exit": 1  # 自主退域
                    "device_hostname": test.com
                    "uuid": "7a19654c-c1a8-11ef-9b83-0242ac12000b"
                    }
                 }
        """
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        result_msg.update(msg_data)
        device_hostname = result_msg.get('device_hostname')
        req_uuid = result_msg.get('uuid')
        device = Devices.objects.filter(device_uuid=device_hostname).first()
        if not device:
            return
        device_belong_user = AccountEntity.objects.filter(id=device.accounts_id).first()
        if msg_data.get('auto_exit'):
            signal = {
                'event_type': 'delete',
                'device_ip': device.device_ip,
                'device_hostname': device_hostname,
                'success': True,
                'msg': '{message} 自主退域成功'.format(message=device_hostname)
            }
            public = RedisHelper()
            public.public(json.dumps(signal), chan='freeipa')
        # 记录客户端日志(异步)
        client_log = {
            'device_uuid': device.device_uuid,
            'device_cnname': device.device_cnname,
            'device_org_id': device.org_info.id if device.org_info else 0,
            'account_name': device_belong_user.name if device_belong_user else '',
            'account_cnname': device_belong_user.cn_name if device_belong_user else '',
            'account_org_id': device_belong_user.org_info.id if device_belong_user else 0,
            'module': ClientLogModule.DEVICE.value,
            'action': ClientLogAction.EXIT_DOMAIN.value,
            'operate_uuid': req_uuid if req_uuid else get_uuid(),
            'result_status': 1,  # 退域结果
            'request': json.dumps(msg_data),
            'response': json.dumps(result_msg),
            'device_info_json': json.dumps({
                'device_hostname': device.device_hostname,
                'device_ip': device.device_ip
            }),
            'account_info_json': json.dumps({
                'email': device_belong_user.email if device_belong_user else ''
            })
        }
        client_log_add.delay(client_log=client_log)

        return result_msg

    @staticmethod
    def exit_domain_result(msg_data):
        """
        :param msg_data: 终端返回退域结果
                         {
                           sender: 1
                           sn: 1-1-4
                           data: {
                                   string device_hostname: xxx.os-domain.cpu-os.com
                                   int result: 0  0|退域成功 1|退域失败
                           }
                        }
        :return: None
        """
        device_hostname = msg_data.get('device_hostname')
        result = msg_data.get('result')
        if result:
            return
        req_uuid = msg_data.get('uuid')
        device = Devices.objects.filter(device_uuid=device_hostname).first()
        if not device:
            return
        device_belong_user = AccountEntity.objects.filter(id=device.accounts_id).first()
        signal = {
            'event_type': 'delete',
            'device_ip': device.device_ip,
            'device_hostname': device_hostname,
            'success': True,
            'msg': '{message} 自主退域成功'.format(message=device_hostname)
        }
        public = RedisHelper()
        public.public(json.dumps(signal), chan='freeipa')
        # 记录客户端日志(异步)
        client_log = {
            'device_uuid': device.device_uuid,
            'device_cnname': device.device_cnname,
            'device_org_id': device.org_info.id if device.org_info else 0,
            'account_name': device_belong_user.name if device_belong_user else '',
            'account_cnname': device_belong_user.cn_name if device_belong_user else '',
            'account_org_id': device_belong_user.org_info.id if device_belong_user else 0,
            'module': ClientLogModule.DEVICE.value,
            'action': ClientLogAction.EXIT_DOMAIN.value,
            'operate_uuid': req_uuid if req_uuid else get_uuid(),
            'result_status': 1,  # 退域结果
            'request': json.dumps(msg_data),
            'response': json.dumps(msg_data),
            'device_info_json': json.dumps({
                'device_hostname': device.device_hostname,
                'device_ip': device.device_ip
            }),
            'account_info_json': json.dumps({
                'email': device_belong_user.email if device_belong_user else ''
            })
        }
        client_log_add.delay(client_log=client_log)
        return None

    def account_info(self, msg_data):
        """
        获取域用户信息
        :param msg_data: 服务收到消息
                 {"sender": "Client",
                 "sn": "1-1-5",
                 "data": {"name": '*******'}   账号
                 }
        :return: 服务发送消息
                 {"sender": "SERVER",
                 "sn": "1-1-5",
                 "data": {
                 "code": 200,                       200|成功；400|失败
                 "cn_name": "huchl",                姓名
                 "name": "huchl",                   账号
                 "organ_name": "部门＋",             组织名称
                 "depart_name": "部门＋",            部门名称
                 "create_type": 0},                 0|自检用户，1|ldap用户，2|统一认证用户
                 "message": null
                 }
        """
        logger.info(f"处理 1-1-5，接收消息 {msg_data}")
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        account_name = msg_data.get('name')
        queryset = AccountEntity.objects.all()
        if queryset.filter(name=account_name).exists():
            account_obj = AccountEntity.objects.get(name=account_name)
            account_org_id = account_obj.org_info_id
            organ_name = self.get_organ_name(account_org_id)
            res_dict = {
                "code": 200,
                "cn_name": account_obj.cn_name,
                "name": account_obj.name,
                "organ_name": organ_name,
                "depart_name": OrgnizationEntity.objects.get(
                    id=account_org_id).name if OrgnizationEntity.objects.filter(id=account_org_id).exists() else None,
                "create_type": account_obj.create_type
            }
            result_msg.update({"sn": "1-1-5", "data": res_dict})
        else:
            result_msg.update({"sn": "1-1-5", "data": {"code": 400, "message": f'{account_name}不存在'}})

        return result_msg

    @staticmethod
    def get_organ_name(organ_id):
        """获取组织信息，即获取跟部门下的部门名称"""
        queryset_organ = OrgnizationEntity.objects.all()
        root_organ = queryset_organ.filter(parent_id=1).values_list('id', flat=True)
        super_id = organ_id
        while True:
            if (super_id in root_organ) or (super_id == 1):
                # logger.info(f"{root_organ},,,,{super_id},,{organ_id}")
                break
            else:
                super_id = queryset_organ.get(id=organ_id).parent_id
        organ_name = queryset_organ.get(id=super_id).name
        return organ_name

    @staticmethod
    def account_online(msg_data):
        """
        用户在线信息
        :param msg_data: 用于获取用户在线信息,用户与终端为多对多关系，如果用户登陆三台终端，离线两台仍然显示在线
                        {
                         sender: CLIENT
                         sn: 9-1-1
                         data: {
                           string hostname: wzx.os-domain.cpu-os.com
                           string username: wzx12345,huchl,mfk
                           }
                        }
        :return: None
        """
        # logger.info(f"9-1-1 {msg_data}, {type(msg_data)}")
        hostname = msg_data.get("hostname")
        username = msg_data.get("username")
        username_list = username.split(',')
        if RedisCacheDevice().device_is_domain(hostname):
            device_user_active = redis_conn.hget('device_user_active', hostname)
            if device_user_active:
                device_user_active = device_user_active.decode('utf-8')
                device_user_active = json.loads(device_user_active)
            device_user_active = device_user_active if isinstance(device_user_active, list) else []
            if username_list == device_user_active:
                pass
            else:
                new_list = [item for item in username_list if item not in ['nfs', 'admin', 'root']]
                device_user_active = json.dumps(new_list, ensure_ascii=False)
                redis_conn.hset('device_user_active', hostname, device_user_active)

                # 统计数据入库:终端用户的上线时间
                reports_helper = ReportsHelper()
                for item in new_list:
                    reports_helper.save_history_account_online_up(item, hostname)
        else:
            pass

        return None

    def account_verify(self, msg_data):
        """
        用户短信验证码，密码校验
        :param msg_data: 用于获取用户在线信息,用户与终端为多对多关系，如果用户登陆三台终端，离线两台仍然显示在线
                         data: {
                           string hostname: wzx.os-domain.cpu-os.com
                           string username: wzx12345
                           string sms:
                           string passwd
                           }
        :return: data: {
                           into   code   200|成功 400|失败
                           string hostname: wzx.os-domain.cpu-os.com
                           string username: wzx12345
                           string sms_result: ''|不处理  ‘true’|正确  ‘false’|失败
                           string passwd_result: ''|不处理  ‘true’|正确  ‘false’|失败
                           string message  错误信息
                           string sms_message  短信校验错误信息
                           string passwd_message 密码校验错误信息
                        }
        """
        logger.info(f"9-1-2 {msg_data}, {type(msg_data)}")
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        result_msg.update({"sn": "9-1-2", "data": {}})
        result_data = result_msg.get('data', {})
        hostname = msg_data.get("hostname")
        username = msg_data.get("username")
        sms = msg_data.get("sms", None)
        passwd = msg_data.get("passwd", None)
        soap_server = SoapMember()
        soap_user = f'{soap_server.user_prefix}{username}'
        account_items = AccountEntity.objects.all()
        result_data.update({"code": 200, "message": None, 'hostname': hostname, 'username': username, 'sms_result': '',
                            'passwd_result': ''})  # 默认返回数据结构
        if account_items.filter(name=username, create_type=0).exists():
            # 本地用户检查密码校验
            if passwd is not None:
                check_res = check_user(username, sm4_decrypt(passwd, settings.SM4_KEY))
                res_exist = check_res.get('exist')
                res_msg = check_res.get('msg')
                if res_exist:
                    result_data.update({"code": 200,
                                        "message": res_msg,
                                        'hostname': hostname,
                                        'username': username,
                                        'sms_result': '',
                                        'passwd_result': 'true'})
                else:
                    result_data.update({"code": 400,
                                        "message": res_msg,
                                        'hostname': hostname,
                                        'username': username,
                                        'sms_result': '',
                                        'passwd_result': 'false'})
            else:
                result_data.update({"code": 400,
                                    "message": '用户密码为空',
                                    'hostname': hostname,
                                    'username': username,
                                    'sms_result': '',
                                    'passwd_result': ''})
        else:
            # 统一认证用户短信校验
            soap_data = {"appId": soap_server.appid, "userId": soap_user,
                         "password": sm4_decrypt(passwd, soap_server.soap_sm4_keys),
                         "requestId": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
                         'authType': 'M_OTP',
                         'verifyCode': sms,
                         "requestName": None, "requestReason": None, "authFlag": "1"
                         }
            if soap_server.client.connect() != 1:
                result_data.update({"code": 400,
                                    "message": "统一认证服务连接失败",
                                    'hostname': hostname,
                                    'username': soap_user,
                                    'sms_result': '',
                                    'passwd_result': ''})
            else:
                # 短信校验
                sms_check_res = soap_server.get_auth_by_sm4(soap_data)
                if sms_check_res.get('status') == 'error' and sms_check_res.get('code') not in (10009, 10200):
                    result_data.update({"code": 400,
                                        "sms_message": sms_check_res.get('message', None),
                                        'hostname': hostname,
                                        'username': soap_user,
                                        'sms_result': 'false',
                                        'passwd_result': ''})
                else:
                    result_data.update({"code": 200,
                                        "sms_message": sms_check_res.get('message', None),
                                        'hostname': hostname,
                                        'username': soap_user,
                                        'sms_result': 'true',
                                        'passwd_result': ''})
                # 密码校验
                soap_data.update({'authType': 'M_PASSWORD', 'verifyCode': None})
                passwd_check_res = soap_server.get_auth_by_sm4(soap_data)
                if passwd_check_res.get('status') == 'error' and passwd_check_res.get('code') not in (10009, 10200):
                    result_data.update({"code": 400,
                                        'passwd_message': passwd_check_res.get('message', None),
                                        'passwd_result': 'false'})
                else:
                    result_data.update({'passwd_message': passwd_check_res.get('message', None),
                                        'passwd_result': 'true'})
        return result_msg

    def device_install_info(self, msg_data):
        """
        获取终端软件安装信息
        {
                "sender": 1,
                "sn": "9-2-1",
                "data": {
                    "taskid": "22",
                    "hostname": "*****.os",
                    "result": 0,  0|成功  1|失败
                    "error": "××××" 安装错误信息
                }
            }
        """
        logger.info(f"9-2-1: {msg_data}")
        jid = "{:%Y%m%d%H%M%S%f}".format(datetime.datetime.utcnow())
        jid_msg = {
            'fun': '9-2-1',
            'fun_args': msg_data,
            'jid': jid,
            'minion': None,
            'ret': None,
            'success': None,
            'full_ret': None,
            'alter_time': datetime.datetime.now()
        }
        try:
            task_id = msg_data.get('taskid')
            task_id = str(task_id)
            hostname = msg_data.get('hostname')
            result = msg_data.get('result')
            error_msg = msg_data.get('error')
            if int(result) == 0:
                """安装成功"""
                jid_msg.update({'minion': hostname,
                                'ret': json.dumps({'result': int(result), 'error_msg': error_msg}, ensure_ascii=False),
                                'success': 0,
                                'full_ret': msg_data})
            elif int(result) == 1:
                """安装失败"""
                jid_msg.update({'minion': hostname,
                                'ret': json.dumps({'result': int(result), 'error_msg': error_msg}, ensure_ascii=False),
                                'success': 0,
                                'full_ret': msg_data})
            else:
                pass

            # 写入日志
            SaltJidDevice.objects.create(**jid_msg)
        except Exception as exp:
            logger.info(f"处理9-2-1: {msg_data} 错误 {exp}")

    @staticmethod
    def device_patch_status(msg_data):
        """
        终端所有补丁状态
        :param msg_data: 返回终端补丁安装状态
                        sender: CLIENT
                        sn: 9-5-7
                        data: {
                          device_id: ***，
                          patch_status_list: [{'patchname':"****","status":"0|1|2"}，{'patchname':"****","status":"0|1|2"}]
                        }
        :return: None
        """
        logger.info(f"9-5-7 {msg_data}, {type(msg_data)}")
        device_id = msg_data.get("device_id")
        patch_status_list = msg_data.get("patch_status_list")
        if '[' in patch_status_list:
            patch_status_list = json.loads(patch_status_list)
        else:
            patch_status_list = []
        if (not patch_status_list) and (isinstance(patch_status_list, list)):
            PatchDeviceLog.objects.filter(device_id=device_id).update(
                install_status=0,
                install_time=datetime.datetime.now(),
                install_err_msg=None,
                is_send_status=0,
                send_msg_time=datetime.datetime.now(),
                is_receive_status=0
            )
        else:
            if patch_status_list is None:
                patch_status_list = []
            patch_file_items = PatchFile.objects.all()
            patch_info_items = PatchInfo.objects.all()
            for i in patch_status_list:
                # logger.info(f"******{i},{type(i)}")
                patch_name = i.get('patchname')
                install_status = i.get('status')
                if install_status == 0:
                    install_status = 1
                elif install_status == 1:
                    install_status = 0
                elif install_status == 2:
                    install_status = 2
                else:
                    install_status = 0
                if patch_file_items.filter(name=patch_name).exists():
                    patch_file_id_list = patch_file_items.filter(name=patch_name).values_list('id', flat=True)
                    patch_id_list = patch_info_items.filter(patch_file_id__in=patch_file_id_list).values_list('id',
                                                                                                              flat=True)
                    patch_device_log_items = PatchDeviceLog.objects.filter(device_id=device_id,
                                                                           patch_id__in=patch_id_list)
                    if install_status == 0:
                        patch_device_log_items.update(
                            install_status=install_status,
                            install_time=datetime.datetime.now(),
                            install_err_msg=None,
                            is_send_status=0,
                            send_msg_time=datetime.datetime.now(),
                            is_receive_status=0
                        )
                    elif install_status == 1:
                        patch_device_log_items.update(
                            install_status=install_status,
                            install_time=datetime.datetime.now(),
                            install_err_msg='成功',
                            is_send_status=1,
                            send_msg_time=datetime.datetime.now(),
                            is_receive_status=1
                        )
                    else:
                        patch_device_log_items.update(
                            install_status=install_status,
                            install_time=datetime.datetime.now(),
                            is_send_status=1,
                            send_msg_time=datetime.datetime.now(),
                            is_receive_status=1
                        )
                else:
                    pass
        return None

    @staticmethod
    def patch_install_status(msg_data):
        """
        补丁安装终端返回信息处理
        :param msg_data: { sender: CLIENT
                           sn: 9-6-7
                           data: {'reason': 2, 'res': 1, 'taskid': '11-3011'}
                         }
        :return: None
        """
        logger.info(f"9-6-7 {msg_data}, {type(msg_data)}")
        task_id = msg_data.get("taskid")
        device_id = task_id.split('-')[0]
        patch_id = task_id.split('-')[1]
        install_status = msg_data.get("res")
        if install_status == 0:
            install_status = 2
        elif install_status == 1:
            install_status = 0
        else:
            install_status = 0
        install_err_msg = msg_data.get("reason")
        if install_err_msg == 0:
            install_err_msg = '成功'
        elif install_err_msg == 1:
            install_err_msg = '空间不足'
        elif install_err_msg == 2:
            install_err_msg = '补丁非法(补丁跟系统版本不一致，名称校验）'
        elif install_err_msg == 3:
            install_err_msg = '下载失败'
        elif install_err_msg == 4:
            install_err_msg = '安装异常'
        else:
            pass
        PatchDeviceLog.objects.filter(device_id=device_id, patch_id=patch_id).update(
            install_status=install_status,
            install_err_msg=install_err_msg,
            install_time=datetime.datetime.now(),
            is_receive_status=1
        )
        return None

    def device_auto_domain(self, msg_data):
        """
        :param msg_data:
                        { sender: 1
                          sn: 9-7-1
                          data: {
                          string hostname: 127.0.0.1@os-domain.cpu-os.com
                          string ip: 127.0.0.1
                          int type: 1  1|申请加域 2|客户端自主加域成功 3|客户端自主加域失败
                          int is_domain: 2  1|加域模式  2|管控模式
                          }
                        }
        :return:  {'sender': 'SERVER', 'sn': 9-7-1, 'data': None, }
        """
        result_msg = copy.deepcopy(self.server_result_msg)
        req_type = msg_data.get('type')
        hostname = msg_data.get('hostname')
        ip = msg_data.get('ip')
        is_domain = msg_data.get('is_domain')
        if req_type == 1:
            if not RedisCacheDevice().device_is_domain(hostname):
                if not redis_conn.get(f'{ip}_adding_domain'):
                    result_msg.update({"sn": "9-7-1", "data": {"code": 200, "message": f"{hostname} 可以加域"}})
                    return result_msg
                else:
                    result_msg.update({"sn": "9-7-1", "data": {"code": 400, "message": f"{hostname} 正在加域"}})
                    return result_msg
            else:
                result_msg.update({"sn": "9-7-1", "data": {"code": 400, "message": f"{hostname} 已经在域中"}})
                return result_msg
        elif req_type == 2:
            signal = {
                'event_type': 'client_own_domain',
                'device_ip': ip,
                'device_hostname': hostname,
                'success': True,
                'msg': f'终端{ip}自主加域成功',
                'is_domain': is_domain
            }
            public = RedisHelper()
            public.public(json.dumps(signal), chan='freeipa')
            result_msg.update({"sn": "9-7-1", "data": "pong"})
            return result_msg
        elif req_type == 3:
            signal = {
                'event_type': 'client_own_domain',
                'device_ip': ip,
                'device_hostname': hostname,
                'success': False,
                'msg': f'终端{ip}自主加域失败',
                'is_domain': is_domain
            }
            public = RedisHelper()
            public.public(json.dumps(signal), chan='freeipa')
            return None
        else:
            result_msg.update({"sn": "9-7-1", "data": {"code": 400, "message": "type类型传参错误"}})
            return result_msg

    def device_ver_info(self, msg_data):
        """
        获取终端版本准备远程连接
        :param msg_data: { sender: 1
                           sn: 9-8-1
                           data: {
                                string hostname: wzx-pc
                                string os: Found/UOS
                                string osversion: 4/5
                                string  clientversion: 1.0.1-m22
                                }
                           }
        :return: None
        """
        # hostname = msg_data.get("hostname")
        clientversion = msg_data.get("clientversion")
        device_ver_dict = {"version": clientversion}

        remote_device_dict = redis_conn.hget(settings.REMOET_ASSIST_KEY, self.client_uuid)
        if remote_device_dict:
            remote_device_dict = remote_device_dict.decode('utf-8')
            remote_device_dict = json.loads(remote_device_dict)
            redis_device_ver = remote_device_dict.get("version")
            if redis_device_ver != clientversion:
                remote_device_dict.update({"version": clientversion})
                redis_conn.hset(settings.REMOET_ASSIST_KEY, self.client_uuid, remote_device_dict)
        else:
            redis_conn.hset(settings.REMOET_ASSIST_KEY, self.client_uuid,
                            json.dumps(device_ver_dict, ensure_ascii=False))

        # logger.info(f'9-8-1: hostname {hostname} channel_info {self.client_uuid}  host_info: {remote_device_dict}')

    def device_remote(self, msg_data):
        """
        终端反馈远程协助结果（当需要终端用户点击确认远程时）
        :param msg_data: {
            "sender": 1,
            "sn": "9-8-5",
            "data": {
                "hostname": "wzx-pc",
                "op": "start",          start|启动, stop|关闭
                "url": "********",
                "errmsg":""             空|成功，"*****"|失败原因 远程协助服务启动失败,远程协助服务关闭失败
            }
        }
        :return: None
        """
        logger.info(f'9-8-5:  msg_data: {msg_data}')
        url = msg_data.get("url")
        errmsg = msg_data.get("errmsg", None)
        vnc_status = msg_data.get("op")

        remote_device_dict = redis_conn.hget(settings.REMOET_ASSIST_KEY, self.client_uuid)
        if remote_device_dict:
            remote_device_dict = remote_device_dict.decode('utf-8')
            remote_device_dict = json.loads(remote_device_dict)
            remote_device_dict.update({"url": url, "errmsg": errmsg, "op": vnc_status})
            redis_conn.hset(settings.REMOET_ASSIST_KEY, self.client_uuid,
                            json.dumps(remote_device_dict, ensure_ascii=False))
        else:
            pass

        return None

    def device_remote_policy(self, msg_data):
        """
        终端反馈远程协助策略
        :param msg_data: {
            "sender": "CLIENT",
            "sn": "10-1-1",
            "data": {
                'enabled': '',
                'argee': '',
                'needpasswd': '',
                'passwd': '',
                'timestamp': '',
            }
        }
        :return: None
        """
        logger.info(f'10-1-1:  msg_data: {msg_data}')
        errmsg = msg_data.get("errmsg", None)

        device = Devices.objects.filter(device_uuid=self.client_uuid).first()
        if device:
            ra = RemoteAssistDevice.objects.filter(device_id=device.id).first()
            if ra:
                if errmsg:
                    ra.status = StatusEnum.FAIL
                else:
                    ra.status = StatusEnum.SUCCESS
                ra.save()

        return None

    def client_unsent_msg(self, msg_data):
        """
        设备在线下发终端未处理的指令
        {
                "sender": client,
                "sn": "9-9-7",
                "data": {
                    "hostname": "*****.os"
                }
            }
        """
        hostname = msg_data.get('hostname')
        logger.info(f'9-9-7:  msg_data: {msg_data}')
        # 查看管道不在线没发送的消息
        device_unsent_message = redis_conn.hget('client_websocket_unsent', hostname)
        if device_unsent_message is None:
            pass
        else:
            device_unsent_message = device_unsent_message.decode('utf-8')
            device_unsent_message = json.loads(device_unsent_message)
            if len(device_unsent_message) == 0:
                redis_conn.hdel('client_websocket_unsent', hostname)
            else:
                err_list = []
                for i in device_unsent_message:
                    try:
                        msg_to_client(1, self.channel_name, i)
                    except Exception as exp:
                        err_list.append(i)
                if len(err_list) == 0:
                    redis_conn.hdel('client_websocket_unsent', hostname)
                else:
                    redis_conn.hset('client_websocket_unsent', hostname, json.dumps(err_list, ensure_ascii=False))

    def heart_beat(self, msg_data):
        """
        心跳程序
        :param msg_data:  {
                            "data": { "hostname": "nfs5huchl.os-domain.cpu-os.com",
                                      "status": 1},
                            "sender": 1,
                            "sn": "9-9-8"
                           }
        :return:
        """
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        hostname = msg_data.get("hostname")
        device_status = msg_data.get("status")
        if RedisCacheDevice().device_is_domain(hostname):
            result_msg.update({"sn": "9-9-8", "message": "pong"})
        else:
            if device_status == 1:
                result_msg.update({"sn": "9-9-8", "message": "pong"})
                # result_msg.update({"sn": "1-1-3", "data": {"exit_domain": 1}})  # 终端不在域中触发终端自动退域
            else:
                result_msg.update({"sn": "9-9-8",
                                   "message": {
                                       "code": 400,
                                       "data": None,
                                       "message": f"{hostname}不在域中"
                                   }})
        return result_msg

    @staticmethod
    def default_action(message):
        """
        默认收到信息转回去
        :param message:
        :return:
        """
        return message

    def get_device_policy(self):
        """
        设备策略信息
        """
        msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        msg.update({"message":"policy group infomation.","sn": "10-1-1", "data": {"version": "v1.0","msgid":"getmsgid","time":"time"
            ,"client_version":{"least":"1.0","latest":"1.9"},"service":"policy_group","policys":[]
        }})
        # TODO 需要接口 获取设备的策略列表
        msg['data']['policys'].append({"type":"usb","val":"off","code":10})
        msg['data']['policys'].append({"type":"root","val":"false","code":10})
        return msg

    def group_policy_receiver(self,msg_data):
        msg = ""
        logger.info("TODO 处理返回到策略执行结果")
        logger.info(msg_data)
        msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        msg.update({"message":"ok","sn": "10-1-1", "data": {} })
        return msg



def msg_to_client(send_type, channel_name, message):
    """
    :param send_type: 0|广播消息，1|点对点消息
    :param channel_name: 通道名称
    :param message: 需要发送的消息
    :return:
    """
    if send_type == 0:
        async_to_sync(channel_layer.group_send)(
            channel_name,  # 广播频道名称
            {
                "type": "send_message",
                "message": message
            }
        )
    elif send_type == 1:
        async_to_sync(channel_layer.send)(
            channel_name,  # 点对点通道名称
            {
                "type": "send_message",
                "message": message,
            },
        )
    else:
        pass
