import logging
import asyncio
import json
import subprocess
from django.conf import settings

from organizations.models import OrgnizationEntity
from .models import Devices, DevicesHardware, DevicesHistory
from celery_task import task_result
from saltapi.utils import SaltAPI
from saltapi.tasks import system_activation_policy
from saltapi.output import check_output
from accounts.models import AccountEntity
from accounts.freeipa import hbac_add_user_to_host
from groups.models import Groups, GroupsDetail
from utils.authentication import DomainLicenseAuth

logger = logging.getLogger(__name__)


def get_device(device_hostname):
    device = Devices.objects.filter(device_hostname=device_hostname).first()
    if not device:
        return False
    return device


def del_device(device_hostname):
    Devices.objects.filter(device_hostname=device_hostname).delete()
    return


def del_device_group(device_id):
    GroupsDetail.objects.filter(device_id=device_id).delete()
    return


def bind_device_account(device_hostname, accounts_id):
    device = get_device(device_hostname)
    if not device:
        return False
    device.accounts_id = accounts_id
    device.save()
    hbac_add_user_to_host(accounts_id, device.device_hostname)
    return True


def save_device_history(device, op_type):
    path = [device.org_info.name]
    parent = device.org_info.parent
    while parent.id != 1:
        path.insert(0, parent.name)
        parent = parent.parent
    account = AccountEntity.objects.filter(id=device.accounts_id).first()
    DevicesHistory.objects.create(
        op_type=op_type,
        device_uuid=device.device_uuid,
        device_ip=device.device_ip,
        device_username=device.device_username,
        device_passwd=device.device_passwd,
        device_auto_domain=device.device_auto_domain,
        device_auto_client=device.device_auto_client,
        device_hostname=device.device_hostname,
        device_cnname=device.device_cnname,
        device_addr=device.device_addr,
        accounts_cnname=account.cn_name,
        accounts_name=account.name,
        os=device.os,
        os_release=device.os_release,
        os_arch=device.os_arch,
        org_info=' / '.join(path),
    )


def get_install_minion_result(device):
    result = task_result.get_celery_result(device.install_minion_task)
    if result in ('PENDING', 'RETRY', 'STARTED'):
        return 2, '已申请加域，当前状态：salt client 正在安装'
    if result == 'FAILED':
        return 3, '安装 salt client 任务失败, 请选择其他方式安装'
    if not result or not result[device.device_hostname]:
        return 1, '安装 salt client 策略失败,请稍后再试,'
    res = result[device.device_hostname]
    if res.get('_error'):
        return 1, '安装 salt client 策略失败,请选择其他方式安装。 error:%s' % res.get('stderr')
    return 0, 'salt client 安装成功'


def get_install_freeipa_result(task_id, option):
    result = task_result.get_celery_result(task_id)
    if result in ('PENDING', 'RETRY', 'STARTED'):
        return 2, '已申请加域，当前状态：组件 正在安装，请稍等'
    if result == 'FAILED':
        return 3, '安装 组件 任务失败, 请选择其他方式安装［1001］'
    if result == 1002:
        return 1, '安装 组件 策略失败,请稍后再试［1002］,'
    if result == 1003:
        return 1, '安装 组件 策略失败,请选择其他方式安装［1003］'
    if result == 1004:
        return 1, '安装 组件 策略失败,请选择其他方式安装［1004］'
    if result == 1005:
        return 1, '安装 组件 策略失败,请选择其他方式安装［1005］'
    if result == 1006:
        return 1, '安装 组件 策略失败,请选择其他方式安装［1006］'
    if result == 1007:
        return 1, '安装 组件 策略失败,请选择其他方式安装［1007］'
    return 0, '组件 安装成功'


def system_activation(device):
    auth = DomainLicenseAuth()
    device = {
        'id': device.id,
        'os': device.os,
        'device_ip': device.device_ip,
        'device_hostname': device.device_hostname,
        'os_release': device.os_release,
    }
    if auth.kms_active == '1':
        if auth.kms_server.startswith('https://'):
            kms_server = auth.kms_server
        elif auth.kms_server.startswith('http://'):
            kms_server = auth.kms_server
        else:
            kms_server = 'http://{0}'.format(auth.kms_server)
        system_activation_policy.delay(
            [device],
            kms_server,
            singal=False
        )


def save_device_hardware(device):
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    grains_item = ['uuid', 'hwaddr_interfaces', 'ip4_interfaces', 'lsb_distrib_description', 'cpuarch', 'mem_total',
                   'productname', 'kernelrelease', 'manufacturer', 'cpu_model', 'ipv4', 'lsb_distrib_id',
                   'lsb_distrib_release']  # 'nodename'
    device_grains_item = salt_api.grain(device.device_hostname, grains_item).get(device.device_hostname)
    real_ip = salt_api.cmd_run(device.device_hostname,
                               "ss -nt | grep :4505 | awk '{print $4}' | awk -F: '{print $1}'").get(
        device.device_hostname)
    cpu_model = device_grains_item.get('cpu_model')
    if cpu_model == 'Unknown':
        cpu_model = salt_api.cmd_run(device.device_hostname,
                                     "dmidecode -t processor | awk '/Version: /{print $2}'").get(
            device.device_hostname)

    hwaddr_interfaces = device_grains_item.get('hwaddr_interfaces')
    ip4_interfaces = device_grains_item.get('ip4_interfaces')
    device_network_list = []
    logger.info(f"****{hwaddr_interfaces}********{ip4_interfaces}")
    for k, hwaddr in hwaddr_interfaces.items():
        hwaddr_ipv4 = {}
        if k == 'lo': continue
        hwaddr_ipv4['model'] = k
        hwaddr_ipv4['mac'] = hwaddr
        hwaddr_ipv4['ip_addres'] = ip4_interfaces[k][0] if ip4_interfaces[k] else ''
        device_network_list.append(hwaddr_ipv4)
    device_grains_item['device_network_list'] = device_network_list
    details_basic = {
        'network': json.dumps(device_grains_item.get('device_network_list')),  # 网卡
        'cpu_model': cpu_model,  # 处理器
        'mainboard': device_grains_item.get('manufacturer'),  # 主板 / 厂商
        'kernelrelease': device_grains_item.get('kernelrelease'),  # 内核版本
        'productname': device_grains_item.get('productname'),  # 产品机型
        'mem_total': device_grains_item.get('mem_total'),  # 内存
        'cpuarch': device_grains_item.get('cpuarch'),  # cpu 架构
        'os': device_grains_item.get('lsb_distrib_description'),  # 操作系统
    }
    device_os = device_grains_item.get('lsb_distrib_id')
    os_release = device_grains_item.get('lsb_distrib_release')
    os_arch = device_grains_item.get('cpuarch').lower()
    device.os = device_os
    device.os_release = os_release
    device.os_arch = os_arch
    ens3_ip = ip4_interfaces.get('ens3')
    eth0_ip = ip4_interfaces.get('eth0')
    enp6s0_ip = ip4_interfaces.get('enp6s0')
    ipv4 = device_grains_item.get('ipv4')

    # real_ip 去重功能
    ip_list = real_ip.split()
    # 使用 set 数据结构进行去重，但排除 '127.0.0.1'
    unique_ips = set(ip for ip in ip_list if ip != '127.0.0.1')
    # 将去重后的 IP 地址再次转换为字符串
    real_ip = ' '.join(unique_ips)
    logger.info(f"real_ip value is {real_ip}")

    if real_ip:
        device.device_ip = real_ip
    elif ens3_ip:
        device.device_ip = ens3_ip[0]
    elif eth0_ip:
        device.device_ip = eth0_ip[0]
    elif enp6s0_ip:
        device.device_ip = enp6s0_ip[0]
    else:
        for ip in ipv4:
            if ip != '127.0.0.1':
                device.device_ip = ip
                break
    device.save()
    # if device.device_ip not in ipv4:
    #     for ip in ipv4:
    #         if ip != '127.0.0.1':
    #             device.device_ip = ip
    #             device.save()
    #             break

    DevicesHardware.objects.update_or_create(defaults=details_basic, device_id=device.id)
    return


def clear_device_hardware(device):
    device_hardware = DevicesHardware.objects.filter(device_id=device.id).first()
    if device_hardware:
        device_hardware.delete()


def get_up_3_organ_name(organ_id):
    """
    获取部门名层级
    获取所有部门名
    :param organ_id: 账号对应的部门id
    :return: 返回信息 name1/name2/name3
    """
    up_3_organ_name = []
    organ_items = OrgnizationEntity.objects.all()
    organ_id_temp = organ_id
    break_id = 1
    try:
        while True:
            if organ_items.filter(id=organ_id_temp).exists():
                organ_obj = organ_items.get(id=organ_id_temp)
                up_3_organ_name.append(organ_obj.name)
                organ_id_temp = organ_obj.parent_id
                if organ_obj.id == break_id:
                    break
            else:
                # logger.info(f"获取父节点失败 id为{organ_id_temp}")
                break
    except Exception as e:
        pass
        # logger.info(f'获取上级部门信息错误，错误信息为{e}')
    finally:
        if len(up_3_organ_name) != 0:
            up_3_organ_name.reverse()
            up_3_organ_name = '/'.join(up_3_organ_name)
        else:
            up_3_organ_name = None
        # logger.info(f"*****{up_3_organ_name}")
        return up_3_organ_name


class Ping(object):
    def __init__(self):
        self.is_active = {}

    def sync_ping_host(self, host):
        res = subprocess.Popen('ping -c 1 -w 1 ' + host, shell=True,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE)
        exit_code = res.wait()
        if exit_code != 0:
            return False
        else:
            return True

    async def async_ping_host(self, device):
        loop = asyncio.get_event_loop()
        future = loop.run_in_executor(None, self.sync_ping_host, device.device_ip)
        res = await future
        if res:
            self.is_active[device.device_hostname] = 1
        else:
            self.is_active[device.device_hostname] = 0

    def run_async_ping_host(self, devices):
        new_loop = asyncio.new_event_loop()
        asyncio.set_event_loop(new_loop)
        tasks = [self.async_ping_host(device) for device in devices]
        loop = asyncio.get_event_loop()
        loop.run_until_complete(asyncio.wait(tasks))
        return self.is_active
