# -*- coding: utf-8 -*-
import datetime
import ipaddress
import os
import threading

import pandas as pd
import logging
import json
import base64
import socket

from django.db import close_old_connections
from django.http import FileResponse, StreamingHttpResponse
from django.conf import settings
from django.db.models import Q
from django.core.cache import \
    caches  # caches['redis'].get('res_key') 使用redis缓存， caches['default'].get('res_key') 使用default缓存
from django_filters.rest_framework import DjangoFilterBackend
from django_redis import get_redis_connection
from rest_framework.decorators import action
from rest_framework.filters import OrderingFilter, SearchFilter
from rest_framework.views import APIView
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.views import Response
from rest_framework import status
from rest_framework.viewsets import ModelViewSet

from accounts.serializers import BindAccountSerializer
from accounts.views import AccountList
from devices.web_consumers import send_msg_to_web
from message_center.models import MessageCenter
from nfs_domain_mainserver.settings import BASE_DIR, SALTFILESYSTEMPATH
from organizations.views import DEPT_ROOT_ID
from users.models import User
from users.role_permisson import get_org_permission_list
from utils.authentication import TokenAuthentication, DomainLicenseAuth
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer
from utils.sch_patch import patch_task, PatchTask
from utils.software_policy_analyse import SoftwarePolicyTask
from utils.throttling import AnonRateThrottle, UserRateThrottle
from utils.rpc_client import SaltRpcClient
from utils.RedisHelper import RedisHelper
from utils.utils import pager_currency, read_licenseB, bytes_to_string, auto_load_salt_shh, auto_install_client, \
    check_auto_result, set_roster, get_uuid
from utils.sch_soap_task_member import SoapMember
from accounts.models import AccountEntity
from organizations.models import OrgnizationEntity
from saltapi.models import SSHDevicePolicy, USBDevicePolicy, FirewalldDevicePolicy, SaltJidDevice, \
    WallpaperDevicePolicy, Wallpaper, ScreensaversDevicePolicy, Screensavers
from saltapi.utils import SaltAPI, SaltSHH, DefaultPolicy
from saltapi.tasks import install_minion, install_freeipa, delete_freeipa, compress_upload_file_policy
from saltapi.singnals import run_user_default_policy, clear_user_policy, clear_wait_policy, clear_device_policy, \
    clear_software_installed_infos, clear_device_domain_info, clear_device_jid, clear_device_user_active, \
    clear_task_infos
from devices.models import Devices, DevicesHardware
from devices.serializers import DeviceSerializer
from devices.client_consumers import msg_to_client
from devices.utils import get_install_minion_result, get_install_freeipa_result, Ping, bind_device_account, \
    clear_device_hardware, del_device, del_device_group, get_up_3_organ_name
from accounts.freeipa import hbacrule_disable, hbacrule_enable, check_user, hbacrule_del, \
    hbacrule_add_user, hbacrule_remove_user
from system.models import DomainRule
from system.serializers import DomainRuleSerializer
from openpyxl import load_workbook
from io import BytesIO
from clientlog.models import ClientLogAction, ClientLogModule
from clientlog.tasks import client_log_add

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')


class PingAPIView(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request):
        domain_rule_queryset = DomainRule.objects.filter(id=1).first()
        domain_rule_data = DomainRuleSerializer(domain_rule_queryset).data
        return Response({'res_ping': 'pong',
                         'domain_rule': domain_rule_data
                         })


class Login(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def post(self, request):
        device_uuid = request.data.get('device_uuid')
        domain_user = request.data.get('domain_user')
        # logger.info(f"********{device_uuid}\r\n{domain_user}\r\n")
        if domain_user == os.environ.get("DOMAIN_SERVER_ADMIN"):
            return Response('pong')
        device = Devices.objects.filter(device_uuid=device_uuid).exclude(is_domain=0).first()
        if not device:
            return Response('pong')
        account = AccountEntity.objects.filter(name=domain_user).first()
        if not account:
            return Response('pong')

        # 更新用户最后登录时间
        last_login_time = datetime.datetime.now()  # 获取当前本地时间
        account.last_login_time = last_login_time.strftime('%Y-%m-%d %H:%M:%S')
        account.save()

        # if account.name != domain_user:
        #     return Response('pong')
        run_user_default_policy(device)
        PatchTask().__query_device_patch__(device_uuid)  # 启动补丁下发检查终端状态
        SoftwarePolicyTask().software_login_device(device_uuid)
        return Response('pong')


class DeviceHostnameInit(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request, device_uuid):

        domain_auth = DomainLicenseAuth()
        if not domain_auth.license_b:
            return Response({'code': 4001, 'data': {'success': False, 'msg': '服务器未激活,请联系管理员'}})
        # mac = get_mac_address()
        mac = os.environ.get("DOMAIN_MAC")
        license_info = read_licenseB(mac, domain_auth.license_b)

        if not license_info.get('register_res'):
            return Response({'code': 4001, 'data': {'success': False, 'msg': '服务器激活码错误,请联系管理员'}})

        max_users = bytes_to_string(license_info.get('max_users_res', 0))

        domain_name = os.environ.get("DOMAIN_NAME")
        # domain_name = '@nfschina.com'
        if not domain_name:
            return Response({'code': 4001, 'data': {'success': False, 'msg': '未设置域名,请联系管理员'}})

        if Devices.objects.exclude(is_domain=0).all().count() >= int(max_users):
            return Response({'code': 4001, 'data': {'success': False, 'msg': '超出最大终端管控数,请联系管理员'}})

        device = Devices.objects.filter(device_uuid=device_uuid).first()
        if device and device.device_hostname:
            device_hostname = device.device_hostname
        else:
            while True:
                # uid = ''.join(uuid.uuid4().__str__().split('-'))
                device_hostname = device_uuid + '.' + domain_name
                if not Devices.objects.filter(device_hostname=device_hostname).first():
                    break
        return Response({
            'code': 2000,
            'data': {
                'success': True,
                'device_hostname': device_hostname
            },
        })


class DeviceDomainInfo(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request, device_uuid):
        device = Devices.objects.filter(device_uuid=device_uuid).exclude(is_domain=0).first()
        if not device or not device.is_domain:
            return Response({
                'code': 2000,
                'data': {
                    'is_domain': False,
                    'msg': '设备未完成加域'
                },
            })

        accounts_id = device.accounts_id
        account = AccountEntity.objects.filter(id=accounts_id).first()
        device_info = {'is_domain': True}
        if account:
            device_info['cn_name'] = account.cn_name
            device_info['name'] = account.name
            org = account.org_info
            device_info['org_name'] = org.name
        else:
            device_info['cn_name'] = None
            device_info['name'] = None
            device_info['org_name'] = None
        return Response({'code': 2000, 'data': device_info})


class DeviceDomain(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def __device_install_minion(self, device):
        device_ip = device.device_ip
        root = device.device_root
        root_pwd = device.device_root_pwd

        roster = {
            device_ip: {
                'host': device_ip,
                'user': root,
                'passwd': root_pwd,
            }
        }
        if root != 'root':
            # roster[device_ip]['tty'] = True
            roster[device_ip]['sudo'] = True
        install_minion_task = install_minion.delay(roster=roster, tgt=device_ip)
        return install_minion_task.id

    def __device_install_freeipa(self, device, accounts_id):
        install_minion_task = install_freeipa.delay(device, accounts_id)
        return install_minion_task.id

    def get(self, request, task_id):
        res = {}
        redis_cache = caches['redis']
        device_uuid = redis_cache.get(task_id)
        # device = Devices.objects.filter(device_uuid=device_uuid).first()
        # if not device:
        #     res['code'] = 4001
        #     res['data'] = {'loop': False, 'msg': 'task_id 不存在'}
        #     return Response(res)
        tag, msg = get_install_freeipa_result(task_id, device_uuid)
        if tag == 1:
            res['code'] = 4001
            res['data'] = {'loop': False, 'msg': msg}
            return Response(res)
        elif tag == 2:
            res['code'] = 2000
            res['data'] = {'loop': True, 'msg': msg, 'task_id': task_id}
            return Response(res)
        elif tag == 3:
            res['code'] = 4001
            res['data'] = {'loop': False, 'msg': msg}
            return Response(res)
        res['code'] = 2000
        res['data'] = {'loop': False, 'msg': '设备加域成功'}
        # if not device.is_domain:
        #     device.is_domain = 1
        #     device.save()
        return Response(res)

    def post(self, request):
        # device add domain
        device_uuid = request.data.get('device_hostname')
        device_ip = request.data.get('ip')
        domain_user = request.data.get('domain_user')
        domain_user_pwd = request.data.get('domain_user_pwd')
        device_cnname = request.data.get('device_hostname')
        device_hostname = request.data.get('device_hostname')
        if not device_ip:
            return Response({'code': 2000, 'data': {'success': False, 'msg': '缺少设备ip'}})
        if not domain_user or not domain_user_pwd:
            return Response({'code': 2000, 'data': {'success': False, 'msg': '缺少域用户或密码'}})
        # if domain_admin != os.environ.get('DOMAIN_SERVER_ADMIN') or domain_admin_pwd != os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD"):
        #     return Response({'code': 2000, 'data': {'success': False, 'msg': '域管理员帐号或密码错误'}})
        if not device_hostname:
            return Response({'code': 2000, 'data': {'success': False, 'msg': '缺少设备hostname'}})
        if not device_cnname:
            return Response({'code': 2000, 'data': {'success': False, 'msg': '缺少设备别名'}})
        try:
            device_cnname = device_cnname.split(f'.{os.environ.get("DOMAIN_NAME")}')[0]
        except Exception as exp:
            pass
        sm = SoapMember()
        accounts_items = AccountEntity.objects.all()
        if accounts_items.filter(name=f'{sm.user_prefix}{domain_user}').exists():
            domain_user_create_type = accounts_items.get(name=f'{sm.user_prefix}{domain_user}').create_type
        else:
            domain_user_create_type = 0

        if (domain_user != settings.DOMAIN_SERVER_ADMIN) and (sm.client.connect()) and (domain_user_create_type == 2):
            soap_data = {
                "appId": sm.appid,
                "userId": domain_user,
                "password": domain_user_pwd,
                "requestId": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
                "authType": 'M_PASSWORD',
                "requestName": None,
                "requestReason": None,
                'authFlag': "1"
            }
            check_res = sm.get_auth_by_sm4(soap_data)
            if check_res.get('status') == 'error' and check_res.get('code') not in (10009, 10200):
                return Response({'code': 4001, 'data': {'success': False, 'msg': check_res.get('message')}})
            domain_user = f'{sm.user_prefix}{domain_user}'
        else:
            check_res = check_user(domain_user, domain_user_pwd)
            if not check_res.get('exist'):
                return Response({'code': 4001, 'data': {'success': False, 'msg': check_res.get('msg')}})
        # show_res = user_show(domain_user)
        # if show_res.get('memberof_hbacrule'):
        #     # if domain_user in show_res.get('memberof_hbacrule'):
        #     return Response({'code': 4001, 'data': {'success': False, 'msg': '{0} 已在域策略中，不可加域'}})

        if Devices.objects.filter(device_uuid=device_uuid).exclude(is_domain=0).first():
            return Response({
                'code': 2000,
                'data': {
                    'success': False,
                    'msg': '主机名已被使用，请修改终端别名'
                },
            })
        domain_name = os.environ.get("DOMAIN_NAME")
        # domain_name = '@nfschina.com'
        if not domain_name:
            return Response({'code': 4001, 'data': {'success': False, 'msg': '未设置域名,请联系管理员'}})

        device = Devices.objects.filter(device_uuid=device_uuid).first()
        if not device:
            device = Devices.objects.create(device_uuid=device_uuid)
        elif device.is_domain:
            return Response({
                'code': 2000,
                'data': {
                    'success': False,
                    'msg': '终端已在域中'
                },
            })
        device.device_ip = device_ip
        device.device_hostname = device_hostname
        device.device_cnname = device_cnname
        if domain_user != os.environ.get('DOMAIN_SERVER_ADMIN'):
            account = AccountEntity.objects.filter(name=domain_user).first()
            device.org_info = account.org_info
            device.bind_account.add(account)
            device.belong_user = account.cn_name if account.cn_name else domain_user
            task_id = self.__device_install_freeipa(DeviceSerializer(device).data, account.id)
        else:

            task_id = self.__device_install_freeipa(DeviceSerializer(device).data, domain_user)
        try:
            device.save()
        except Exception as exp:
            logger.info(f"*202***{exp}")

        redis_cache = caches['redis']
        redis_cache.set(task_id, device_uuid, 60 * 30)

        return Response({
            'code': 2000,
            'data': {
                'success': True,
                'msg': '终端加域申请已受理',
                'task_id': task_id
            },
        }
        )

    def delete(self, request):
        device_id = request.GET.get('device_id')
        device = Devices.objects.filter(id=device_id).exclude(is_domain=0).first()
        if not device:
            return Response({
                'code': 400,
                'msg': '终端未加域'
            })

        conn = get_redis_connection('redis')
        actives = True if conn.hget('actives', device.device_hostname) == b'1' else False
        if not actives:
            res = delete_domain(device)
            logger.info(f"res 删除 saltmaster key 结果 {res}")
            return Response({'code': 200, 'msg': '已强制删除'})
        task = delete_freeipa.delay(DeviceSerializer(device).data)
        return Response({
            'code': 200,
            'msg': '终端退域申请已受理',
            'task_id': task.id
        }
        )


def delete_domain(device):
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    try:
        clear_user_policy(device)
    except Exception as e:
        logger.info('--- clear_user_policy error ---')
        logger.info(e)
    try:
        clear_device_policy(device)
    except Exception as e:
        logger.info('--- clear_device_policy error ---')
        logger.info(e)
    try:
        clear_device_hardware(device)
    except Exception as e:
        logger.info('--- clear_device_hardware error ---')
        logger.info(e)
    try:
        clear_wait_policy(device)
    except Exception as e:
        logger.info('--- clear_wait_policy error ---')
        logger.info(e)
    try:
        clear_software_installed_infos(device)
    except Exception as e:
        logger.info('--- clear software install log error ---')
        logger.info(e)
    # try:
    #     clear_task_infos(device)
    # except Exception as e:
    #     logger.info('--- clear task install log error ---')
    #     logger.info(e)
    try:
        clear_device_user_active(device)
    except Exception as e:
        logger.info('--- clear device user active log error ---')
        logger.info(e)
    try:
        SaltRpcClient().remove_ssh_host(device.device_ip)
    except Exception as e:
        logger.info('--- clear salt ssh hostname error ---')
        logger.info(e)
    # try:
    #     clear_device_jid(device)
    # except Exception as e:
    #     logger.info('--- clear salt jid log error ---')
    #     logger.info(e)
    # 下发离线策略
    # delete_freeipa.delay(DeviceSerializer(device).data)
    # freeipa delete hbac rule
    rule_name = 'hbac_' + device.device_hostname
    hbacrule_del(rule_name)
    device.bind_account.clear()  # 清除可登录终端的人员
    del_device_group(device.id)
    clear_device_domain_info(device)
    res = salt_api.delete_key(device.device_hostname)
    logger.info(f"res 删除 saltmaster key 结果 {res}")
    # del_device(device.device_hostname)
    return res


class GetClient(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        ip = os.environ.get("DOMAIN_IP")
        domain_server = os.environ.get("DOMAIN_NAME")
        port = 8082
        if not ip or not domain_server:
            return Response({
                'code': 400,
                'msg': '未配置服务器IP或域名'
            })
        url = 'wget -O nfsdomain.sh http://{ip}:{port}/mainapi/client/ && sudo bash nfsdomain.sh {ip} server.{domain_server}'.format(
            domain_server=domain_server,
            port=port,
            ip=ip
        )
        return Response({
            'code': 200,
            'data': {
                'url': url
            }
        })


class GetClient2(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request):
        ip = os.environ.get("DOMAIN_IP")
        domain_server = os.environ.get("DOMAIN_NAME")
        port = 8082
        if not ip or not domain_server:
            return Response({
                'code': 400,
                'msg': '未配置服务器IP或域名'
            })
        default_policy = DefaultPolicy()
        client_2 = default_policy.get_default_policy().get('install_client_2')
        if client_2 == '0':
            url = None
        else:
            url = 'wget -O nfsdomain.sh http://{ip}:{port}/mainapi/client_2/ && sudo bash nfsdomain.sh {ip} server.{domain_server} 2'.format(
                domain_server=domain_server,
                port=port,
                ip=ip
            )
        return Response({
            'code': 200,
            'data': {
                'url': url
            }
        })


class DownloadClient(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request, file='install-domain-client_1.0.sh'):
        file_path = settings.SALTFILESYSTEMPATH + '/' + file
        try:
            response = FileResponse(open(file_path, 'rb'))
            response['Content-Type'] = 'application/octet-stream'
            response['Content-Disposition'] = 'attachment; filename={file}'.format(file=file)
            return response
        except Exception:
            return Response(status=status.HTTP_404_NOT_FOUND)


class DownloadClient2(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request, file='install-domain-client_2.0.sh'):
        file_path = settings.SALTFILESYSTEMPATH + '/' + file
        try:
            response = FileResponse(open(file_path, 'rb'))
            response['Content-Type'] = 'application/octet-stream'
            response['Content-Disposition'] = 'attachment; filename={file}'.format(file=file)
            return response
        except Exception:
            return Response(status=status.HTTP_404_NOT_FOUND)


# 设备列表
class DeviceAll(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    @classmethod
    def organ_down_list(cls, num):
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        return organ_down_list

    def get(self, request):
        device_all = Devices.objects.exclude(is_domain=0).all().defer('device_root').order_by('-id')

        # 根据部门过滤数据，超级管理员不需要
        user = User.objects.filter(username=request.user.username).first()
        org_list = get_org_permission_list(user)

        # 根据部门筛选数据
        if org_list:
            device_all = device_all.filter(org_info__in=org_list)

        name = request.GET.get('name')
        cn_name = request.GET.get('cn_name')
        org_id = request.GET.get('org_id')
        org_name = request.GET.get('org_name')
        device_ip = request.GET.get('device_ip')
        device_addr = request.GET.get('device_addr')
        device_hostname = request.GET.get('device_hostname')
        device_cnname = request.GET.get('device_cnname')
        is_active = request.GET.get('is_active')
        is_up = request.GET.get('is_up')
        belong_user = request.GET.get('belong_user')

        if name:
            account = AccountEntity.objects.filter(name=name).first()
            if account:
                device_all = device_all.filter(accounts_id=account.id)
            else:
                return Response({
                    'code': 200,
                    "pageCount": 1,
                    "current": 1,
                    "pageSize": 10,
                    "total": 0,
                    "data": []
                })
        if cn_name:
            account_list = AccountEntity.objects.filter(cn_name__icontains=cn_name).values_list('id', flat=True)
            if account_list:
                device_all = device_all.filter(accounts_id__in=account_list)
            else:
                return Response({
                    'code': 200,
                    "pageCount": 1,
                    "current": 1,
                    "pageSize": 10,
                    "total": 0,
                    "data": []
                })
        if org_id:
            if int(user.org_global_switch_device) == 1:
                org_ids_list = [org_id]
            else:
                org_ids_list = self.organ_down_list(org_id)

            if org_id == 1:
                pass
            else:
                device_all = device_all.filter(org_info_id__in=org_ids_list)
        if device_ip:
            device_all = device_all.filter(device_ip__icontains=device_ip)
        if device_addr:
            device_all = device_all.filter(device_addr=device_addr)
        if device_hostname:
            device_all = device_all.filter(device_hostname=device_hostname)
        if device_cnname:
            device_all = device_all.filter(device_cnname__icontains=device_cnname)
        if org_name:
            org_id_list = OrgnizationEntity.objects.filter(name__icontains=org_name).values_list('id', flat=True)
            device_all = device_all.filter(org_info__in=org_id_list)

        if is_up:
            if is_up == 'true':
                is_up = 1
            else:
                is_up = 0
            device_all = device_all.filter(is_up=is_up)
        if belong_user:
            logger.info("belong_user %s" % belong_user)
            device_all = device_all.filter(belong_user__icontains=belong_user)
        conn = get_redis_connection('redis')
        total_number = device_all.count()
        if is_active in ('true', 'false'):
            total = []
            for device in device_all:
                device_info = {}
                device_info['is_active'] = True if conn.hget('actives', device.device_hostname) else False
                if is_active == 'true' and device_info['is_active'] == True:
                    total.append(device)
                    continue
                if is_active == 'false' and device_info['is_active'] == False:
                    total.append(device)
                    continue
            device_all = total
            total_number = len(device_all)

        page_info = pager_currency(request, total_number)
        devices = device_all[page_info.get('start'):page_info.get('limit')]
        # ping = Ping()
        # actives = ping.run_async_ping_host(devices)
        res_list = []
        for device in devices:
            device_info = {}
            device_ip = device.device_ip

            # device_info['is_active'] = True if device.device_hostname in actives else False
            device_info['is_active'] = True if conn.hget('actives', device.device_hostname) == b'1' else False

            # if is_active == 'true' and device_info['is_active'] == False: continue
            # if is_active == 'false' and device_info['is_active'] == True: continue
            path = [device.org_info.name]
            parent = device.org_info.parent
            while parent.id != 1:
                path.insert(0, parent.name)
                parent = parent.parent
            org_info = ' / '.join(path)

            device_info['device_uuid'] = device.device_uuid
            device_info['id'] = device.id
            device_info['device_ip'] = device_ip
            device_info['device_addr'] = device.device_addr
            device_info['device_hostname'] = device.device_hostname
            device_info['device_cnname'] = device.device_cnname
            device_info['is_up'] = True if device.is_up else False
            device_info['org_id'] = device.org_info_id
            device_info['org_name'] = get_up_3_organ_name(device.org_info_id)
            device_info['org_info'] = org_info
            device_info['belong_user'] = device.belong_user

            accounts_id = device.accounts_id
            account = AccountEntity.objects.filter(id=accounts_id).first()
            if account:
                device_info['cn_name'] = account.cn_name
                device_info['name'] = account.name
            else:
                device_info['cn_name'] = None
                device_info['name'] = None
            res_list.append(device_info)
        return Response({
            'code': 200,
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': total_number,
            'data': res_list
        })


# 基础信息
class DeviceDetailsBasic(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, id):
        device = Devices.objects.filter(id=id).exclude(is_domain=0).first()
        conn = get_redis_connection('redis')
        device_info = {}
        if not device:
            return Response({
                'code': 200,
                'data': device_info
            })
        device_info['is_active'] = True if conn.hget('actives', device.device_hostname) == b'1' else False
        device_info['is_up'] = True if device.is_up else False
        device_info['is_delete'] = True if device.is_delete else False
        device_info['device_uuid'] = device.device_uuid
        device_info['device_addr'] = device.device_addr
        device_info['device_hostname'] = device.device_hostname
        device_info['device_cnname'] = device.device_cnname
        device_info['note'] = device.note
        device_info['device_ip'] = device.device_ip
        device_info['belong_user'] = device.belong_user
        device_info['device_root_pwd'] = str(base64.b64encode(device.device_root_pwd.encode("utf-8")), "utf-8")
        device_info['device_uid'] = device.device_uid
        device_info['device_sid'] = device.device_sid
        accounts_id = device.accounts_id
        account = AccountEntity.objects.filter(id=accounts_id).first()
        if account:
            device_info['cn_name'] = account.cn_name
            device_info['name'] = account.name
        else:
            device_info['cn_name'] = None
            device_info['name'] = None
        device_info['org_name'] = get_up_3_organ_name(device.org_info_id)
        device_info['org_id'] = device.org_info.id
        return Response({
            'code': 200,
            'data': device_info
        })

    def post(self, request, id):
        device = Devices.objects.filter(id=id).exclude(is_domain=0).first()
        accounts_items = AccountEntity.objects.filter(id=device.accounts_id)
        is_up = request.data.get('is_up')
        is_delete = request.data.get('is_delete')
        device_addr = request.data.get('device_addr', None)
        accounts_id = request.data.get('accounts_id', None)
        device_cnname = request.data.get('device_cnname', None)
        note = request.data.get('note', None)
        org_id = request.data.get('org_id', None)
        belong_user = request.data.get('belong_user', None)

        if is_up in ("true", "false"):
            # freeipa 启用/禁用hbac
            if is_up == "true":
                device.is_up = 1
                hbacrule_enable(device.device_hostname)
            else:
                device.is_up = 0
                hbacrule_disable(device.device_hostname)
        if is_delete in (0, 1):
            device.is_delete = is_delete
        if device_cnname:
            if device.device_cnname == device_cnname:
                pass
            elif Devices.objects.filter(device_cnname=device_cnname).exclude(is_domain=0).first():
                return Response({
                    'code': 400,
                    'msg': '设备别名已存在'
                })
            device.device_cnname = device_cnname
        if device_addr != None:
            device.device_addr = device_addr
        if note != None:
            device.note = note
        if belong_user:
            device.belong_user = belong_user
        if org_id:
            device.org_info_id = org_id
        device.save()
        if accounts_id != None:
            bind_device_account(device.device_hostname, accounts_id)
        return Response({
            'code': 200,
            'msg': '修改完成'
        })


# 硬件信息
class DeviceDetailsHardware(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, id):
        device = Devices.objects.filter(id=id).exclude(is_domain=0).first()
        # salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        # is_active = salt_api.test_ping(device_ip)
        # ping = Ping()
        # actives = ping.sync_ping_host(device.device_ip)
        # if not actives:
        #     return Response({
        #         'code': 400,
        #         'msg': '设备不在线'
        #     })
        device_hardware = DevicesHardware.objects.filter(device_id=id).first()
        details_basic = {}
        if not device_hardware:
            return Response({
                'code': 200,
                'data': {
                    'device_network_list': [{"model": None, "mac": None, "ip_addres": None}]
                }
            })
        details_basic['device_network_list'] = json.loads(device_hardware.network)
        details_basic['cpu_model'] = device_hardware.cpu_model
        details_basic['mainboard_name'] = device_hardware.mainboard
        details_basic['kernelrelease'] = device_hardware.kernelrelease
        details_basic['productname'] = device_hardware.productname
        details_basic['mem_total'] = device_hardware.mem_total
        details_basic['cpuarch'] = device_hardware.cpuarch
        details_basic['lsb_distrib_description'] = device_hardware.os

        return Response({
            'code': 200,
            'data': details_basic
        })


class DeviceDetailsMount(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, id):
        device_ip = Devices.objects.filter(id=id).exclude(is_domain=0).first().device_ip
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        # is_active = salt_api.test_ping(device_ip)
        ping = Ping()
        actives = ping.sync_ping_host(device_ip)
        if not actives:
            return Response({
                'code': 400,
                'msg': '设备不在线'
            })
        res = salt_api.disk(device_ip, 'inodeusage')
        return Response({
            'code': 200,
            'data': res[device_ip]
        })


class DeviceDetailsSecurity(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, id):
        device_ip = Devices.objects.filter(id=id).exclude(is_domain=0).first().device_ip
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        is_active = salt_api.test_ping(device_ip)
        if not is_active.get(device_ip):
            return Response({
                'code': 400,
                'msg': '设备不在线'
            })
        arg = ['firewalld']
        res = salt_api.service(device_ip, 'status', arg=arg)
        return Response({
            'code': 200,
            'data': {'firewalld': res[device_ip]}
        })


class DeviceSyncPolicy(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request, device_uuid):
        policy_type = request.GET.get('policy_type')
        device = Devices.objects.filter(device_uuid=device_uuid).exclude(is_domain=0).first()
        if not device:
            return Response({'code': 200,
                             'data': {
                                 'code': status.HTTP_400_BAD_REQUEST,
                                 'success': False,
                             },
                             'msg': None})
        if not device.is_domain:
            return Response({'code': 200,
                             'data': {
                                 'code': status.HTTP_400_BAD_REQUEST,
                                 'success': False,
                             },
                             'msg': None})
        if policy_type == 'ssh':
            ssh_device = SSHDevicePolicy.objects.filter(device_id=device.id).first()
            if ssh_device:
                return Response({'code': 200,
                                 'data': {
                                     'code': status.HTTP_400_BAD_REQUEST,
                                     'success': True,
                                     'policy_type': policy_type,
                                     'policy': ssh_device.ssh
                                 },
                                 'msg': None})
        if policy_type == 'usb':
            usb_device = USBDevicePolicy.objects.filter(device_id=device.id).first()
            if usb_device:
                return Response({'code': 200,
                                 'data': {
                                     'code': status.HTTP_400_BAD_REQUEST,
                                     'success': True,
                                     'policy_type': policy_type,
                                     'policy': usb_device.usb
                                 },
                                 'msg': None})
        if policy_type == 'usb_storage':
            usb_device = USBDevicePolicy.objects.filter(device_id=device.id).first()
            if usb_device:
                return Response({'code': 200,
                                 'data': {
                                     'code': status.HTTP_400_BAD_REQUEST,
                                     'success': True,
                                     'policy_type': policy_type,
                                     'policy': 0 if usb_device.usb_rw_status == 'rw' else 1
                                 },
                                 'msg': None})
        if policy_type == 'wallpaper':
            wallpaper_device = WallpaperDevicePolicy.objects.filter(device_id=device.id).first()
            if wallpaper_device:
                if wallpaper_device.wallpaper_id:
                    wallpaper = Wallpaper.objects.filter(id=wallpaper_device.wallpaper_id).first()
                    return Response({'code': 200,
                                     'data': {
                                         'code': status.HTTP_400_BAD_REQUEST,
                                         'success': True,
                                         'policy_type': policy_type,
                                         'policy': f'file:///usr/share/backgrounds/{wallpaper.name}',
                                         'wallpaper_url': 'mainapi/wallpaper/{0}'.format(wallpaper.name)
                                     },
                                     'msg': None})
        if policy_type == 'idle-delay':
            lock_screen_device = ScreensaversDevicePolicy.objects.filter(device_id=device.id).first()
            if lock_screen_device:
                return Response({'code': 200,
                                 'data': {'code': status.HTTP_200_OK,
                                          'success': True,
                                          'policy_type': policy_type,
                                          'switch_status': lock_screen_device.switch_status,
                                          'policy': int(lock_screen_device.interval),
                                          },
                                 'msg': None}
                                )
            else:
                lock_screen_device = Screensavers.objects.all().first()
                if lock_screen_device.switch_status == 1:
                    screen_interval = int(lock_screen_device.interval) * 60
                else:
                    screen_interval = 0

                return Response({'code': 200,
                                 'data': {
                                     'code': status.HTTP_200_OK,
                                     'success': True,
                                     'policy_type': policy_type,
                                     'switch_status': lock_screen_device.switch_status,
                                     'policy': screen_interval,
                                 },
                                 'msg': None})
        if policy_type == 'firewalld':
            firewalld_deivce = FirewalldDevicePolicy.objects.filter(device_id=device.id).first()
            if firewalld_deivce:
                return Response({'code': 200,
                                 'data': {
                                     'code': status.HTTP_400_BAD_REQUEST,
                                     'success': True,
                                     'policy_type': policy_type,
                                     'policy': firewalld_deivce.firewalld
                                 },
                                 'msg': None}
                                )
        if policy_type == 'firewalld_level':
            firewalld_deivce = FirewalldDevicePolicy.objects.filter(device_id=device.id).first()
            if firewalld_deivce:
                return Response({'code': 200,
                                 'data': {
                                     'code': status.HTTP_400_BAD_REQUEST,
                                     'success': True,
                                     'policy_type': policy_type,
                                     'policy': firewalld_deivce.firewalld_level
                                 },
                                 'msg': None})
        return Response({'code': 200,
                         'data': {
                             'code': status.HTTP_400_BAD_REQUEST,
                             'success': False,
                         },
                         'msg': None})

    def post(self, request, device_uuid):
        policy_type = request.data.get('policy_type')
        policy = request.data.get('policy')
        device = Devices.objects.filter(device_uuid=device_uuid).exclude(is_domain=0).first()
        if not device:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
            })
        if not device.is_domain:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
            })
        if policy_type == 'ssh':
            ssh_device = SSHDevicePolicy.objects.filter(device_id=device.id).first()
            if ssh_device and ssh_device.ssh != policy:
                ssh_device.ssh = policy
                ssh_device.save()
        if policy_type == 'usb':
            usb_device = USBDevicePolicy.objects.filter(device_id=device.id).first()
            if usb_device and usb_device.usb != policy:
                usb_device.usb = policy
                usb_device.save()
        if policy_type == 'firewalld':
            firewalld_deivce = FirewalldDevicePolicy.objects.filter(device_id=device.id).first()
            if firewalld_deivce and firewalld_deivce.firewalld != policy:
                firewalld_deivce.firewalld = policy
                firewalld_deivce.save()
        if policy_type == 'firewalld_level':
            firewalld_deivce = FirewalldDevicePolicy.objects.filter(device_id=device.id).first()
            if firewalld_deivce and firewalld_deivce.firewalld_level != policy:
                firewalld_deivce.firewalld_level = policy
                firewalld_deivce.save()
        return Response({
            'code': status.HTTP_200_OK,
            'success': True,
        })


class DeviceCount(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        conn = get_redis_connection('redis')
        devices = Devices.objects.exclude(is_domain=0).all()
        device_count = devices.count()
        device_up_count = devices.filter(is_up=1).count()
        device_down_count = devices.filter(is_up=0).count()
        device_hostname = devices.values('device_hostname')
        device_active_count = 0
        for hostname in device_hostname:
            if conn.hget('actives', hostname.get('device_hostname')) == b'1': device_active_count += 1
        accounts = AccountEntity.objects.all()
        account_count = accounts.count()
        normal_count = accounts.filter(user_status=0).count()
        disable_count = accounts.filter(user_status=1).count()
        locked_count = accounts.filter(user_status=2).count()
        org_count = OrgnizationEntity.objects.all().count()
        jid_count = SaltJidDevice.objects.all().count()
        keys = conn.lrange('device_active_rate', 0, -1)
        device_active_rate = []
        for key in keys:
            day = key.decode('utf-8')
            count = conn.hlen(day)
            device_active_rate.append({'date': day, 'count': count})

        keys = conn.lrange('device_active_rate_hour', 0, -1)
        device_active_rate_hour = []
        for key in keys:
            hour = key.decode('utf-8')
            count = conn.hlen(hour)
            device_active_rate_hour.append({'date': hour, 'count': count})
        return Response({
            'code': 200,
            'data': {
                'device_count': device_count,
                'device_up_count': device_up_count,
                'device_down_count': device_down_count,
                'device_active_count': device_active_count,
                'account_count': account_count,
                'account_normal_count': normal_count,
                'account_disable_count': disable_count,
                'account_locked_count': locked_count,
                'org_count': org_count,
                'jid_count': jid_count,
                'device_active_rate': device_active_rate,
                'device_active_rate_hour': device_active_rate_hour,
            }
        })


class DevicesViewSet(ModelViewSet):
    queryset = Devices.objects.exclude(is_domain=0).all()
    serializer_class = DeviceSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    search_fields = ('device_cnname', 'device_ip', 'belong_user',)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    # throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    @classmethod
    def to_organ_name(cls, num):
        if num:
            org_id = AccountEntity.objects.get(id=num).org_info_id
            organ_name = OrgnizationEntity.objects.get(id=org_id).name
        else:
            organ_name = None

        return organ_name

    @classmethod
    def build_and_get_structure(cls, orgs_str):
        if not orgs_str:
            return OrgnizationEntity.objects.get(pk=DEPT_ROOT_ID)
        orgs = [o.strip() for o in orgs_str.split('/')]
        parent_org = None
        # 逐级创建部门
        for i, org in enumerate(orgs):
            parent_org, created = OrgnizationEntity.objects.get_or_create(
                name=org,
                parent_id=DEPT_ROOT_ID if (i == 0) else parent_org.id,
            )
        return parent_org

    @classmethod
    def bind_user_info(cls, number):
        """
        :param number: 设备主键id
        :return:
        """
        mid_id_user = cls.queryset.filter(id=number).values_list('bind_account__name', flat=True)
        mid_id_user = [i for i in mid_id_user if i is not None]
        if mid_id_user:
            res_data = ';'.join(mid_id_user)
        else:
            res_data = None

        return res_data

    def device_filter_queryset(self, queryset):
        """
        :param queryset: 表数据结果集合
        :return: 根据过滤条件筛选返回结果集合
        """
        queryset = queryset.exclude(is_domain=0)
        device_cnname = self.request.query_params.get('device_cnname', default=None)
        name = self.request.query_params.get('name', default=None)
        device_ip = self.request.query_params.get('device_ip', default=None)
        cn_name = self.request.query_params.get('cn_name', default=None)
        org_id = self.request.query_params.get('org_id', default=None)
        device_addr = self.request.query_params.get('device_addr', default=None)
        is_active = self.request.query_params.get('is_active', default=None)
        belong_user = self.request.query_params.get('belong_user', default=None)
        if device_cnname:
            queryset = queryset.filter(device_cnname__icontains=device_cnname)
        if device_ip:
            queryset = queryset.filter(device_ip__icontains=device_ip)
        if name:
            accounts_list = AccountEntity.objects.filter(name__startswith=name).all().values_list('id', flat=True)
            queryset = queryset.filter(accounts_id__in=accounts_list)
        if org_id:
            if self.request.user.org_global_switch_user == 1:
                queryset = queryset.filter(org_info_id=org_id)
            else:
                org_down_list = AccountList().organ_down_list(org_id)
                queryset = queryset.filter(org_info_id__in=org_down_list)
        if cn_name:
            accounts_list = AccountEntity.objects.filter(cn_name__contains=cn_name).values_list('id', flat=True)
            queryset = queryset.filter(accounts_id__in=accounts_list)
        if device_addr:
            queryset = queryset.filter(device_addr=device_addr)
        if is_active:
            conn = get_redis_connection('redis')
            devices_id_list = []
            for item in queryset:
                if conn.hget('actives', item.device_hostname) == b'1':
                    devices_id_list.append(item.id)
            if is_active == 'true':
                queryset = queryset.filter(id__in=devices_id_list)
            else:
                queryset = queryset.exclude(id__in=devices_id_list)
        if belong_user:
            queryset = queryset.filter(belong_user__contains=belong_user)

        return queryset

    def get_device_info(self):
        queryset = self.filter_queryset(self.get_queryset())
        serializer = self.get_serializer(queryset, many=True)
        res_data = json.dumps(serializer.data, ensure_ascii=False)
        redis_conn.hset('web_cache', 'device_info', res_data)
        close_old_connections()
        return res_data

    def list(self, request, *args, **kwargs):
        """缓存数据到redis"""
        redis_device_info = redis_conn.hget('web_cache', 'device_info')
        if redis_device_info:
            redis_device_info = redis_device_info.decode('utf-8')
            background_thread = threading.Thread(target=self.get_device_info)
            background_thread.start()
            return Response(json.loads(redis_device_info))
        else:
            queryset = self.queryset.all()[0:2]
            serializer = self.get_serializer(queryset, many=True)
            background_thread = threading.Thread(target=self.get_device_info)
            background_thread.start()
            return Response(serializer.data)

    def create(self, request, *args, **kwargs):
        """
        主动加域接口  必填  参数 device_auto_domain  device_ip  device_port  device_username device_passwd  device_cnname
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        query_data = request.data.copy()
        device_auto_domain = query_data.get('device_auto_domain', 1)
        device_auto_client = query_data.get('device_auto_client', 1)
        device_ip = query_data.get('device_ip')
        device_port = query_data.get('device_port')
        device_username = query_data.get('device_username')
        device_passwd = query_data.get('device_passwd')
        device_cnname = query_data.get('device_cnname')
        belong_user = query_data.get('belong_user')
        note = query_data.get('note')
        org_info_id = query_data.get('org_info', 1)
        self.queryset = Devices.objects.all()
        device_hostname = device_uuid = f'{device_cnname}.{os.environ.get("DOMAIN_NAME")}'
        device = self.queryset.filter(
            Q(device_cnname=device_cnname) | Q(device_ip=device_ip) | Q(device_hostname=device_hostname) | Q(
                device_uuid=device_uuid)
        )
        # logger.info(f'{device}')
        if device.exclude(is_domain=0).exists():
            return Response(f'主机名 或 IP 已存在', status=status.HTTP_400_BAD_REQUEST)
        device = device.first()
        # device = self.queryset.filter(device_ip=device_ip).first()
        # if device:
        #     if device.is_domain != 0:
        #         return Response(f'{device_ip}已存在', status=status.HTTP_400_BAD_REQUEST)
        if not device_cnname.islower() or ('@' in device_cnname) or ('_' in device_cnname):
            return Response(f'{device_cnname}主机名只能输入小写，数字和-', status=status.HTTP_400_BAD_REQUEST)
        try:
            ipaddress.ip_address(device_ip)
        except ValueError:
            return Response(f'{device_ip} ip格式不正确', status=status.HTTP_400_BAD_REQUEST)

        conn = get_redis_connection('redis')
        if conn.get(f'{device_ip}_adding_domain'):
            return Response('此设备正在加域请等待加域完成或稍后重试', status=status.HTTP_400_BAD_REQUEST)
        # 缓存加域状态
        conn.set(f'{device_ip}_adding_domain', 1, ex=300, nx=True)
        # 自动安装加域客户端
        # device_auto_domain = 1
        # device_auto_client = 1
        # 写入 salt_ssh配置
        data = {
            device_cnname: {
                "host": device_ip,
                "user": device_username,
                "passwd": device_passwd,
                "port": device_port
            }
        }
        # res_code, res_msg = auto_load_salt_shh(device_ip, device_port, device_username, device_passwd, device_cnname)
        # logger.info(f'{res_code},{res_msg}')
        # if res_code != 0:
        #     return Response(res_msg, status=status.HTTP_400_BAD_REQUEST)
        set_roster(data)
        # 测试是否联通
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        res = salt_api.ssh_ping_client(tgt=device_cnname)
        # logger.info(res)
        # salt_ssh = SaltSHH(roster_file)
        # res = salt_ssh.test_ping(tgt=device_cnname)
        logger.info(f'测试salt_ssh是否联通{res}')
        if not check_auto_result(res, device_cnname):
            return Response(f'终端{device_ip}的ssh账号{device_username}连接失败',
                            status=status.HTTP_408_REQUEST_TIMEOUT)
        query_data['device_hostname'] = device_hostname
        query_data['device_uuid'] = device_uuid
        query_data['device_auto_domain'] = 1
        query_data['org_info_id'] = org_info_id
        if not self.queryset.filter(device_cnname=device_cnname, device_ip=device_ip).exists():
            serializer = self.get_serializer(data=query_data)
            serializer.is_valid(raise_exception=True)
            self.perform_create(serializer)
        else:
            device.device_hostname = device_hostname
            device.device_uuid = device_uuid
            device.device_ip = device_ip
            device.device_cnname = device_cnname
            device.device_passwd = device_passwd
            device.device_auto_domain = device_auto_domain
            device.device_auto_client = device_auto_client
            device.org_info_id = org_info_id
            device.belong_user = belong_user
            device.note = note
            device.save()
        # 主动加域
        thread1 = threading.Thread(target=auto_install_client,
                                   args=(
                                       device_cnname, device_ip, device_username, device_passwd, device_auto_client,
                                       device_auto_domain))
        thread1.start()
        return Response('等待加域完成', status=status.HTTP_201_CREATED)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def devices_tmpl(self, request, *args, **kwargs):
        """
        设备主动加域模板下载
        :param request:  filename 文件名
        :param args:
        :param kwargs:
        :return:
        """

        def file_iterator(file_name, chunk_size=512):
            with open(file_name, 'rb') as f:
                while True:
                    c = f.read(chunk_size)
                    if c:
                        yield c
                    else:
                        break

        # 显示在弹出对话框中的默认的下载文件名
        if 'filename' in request.query_params.keys():
            the_file_name = request.query_params['filename']
        else:
            the_file_name = '主动添加终端模板.xlsx'
        # cur_path = os.path.abspath('.')
        download_url = BASE_DIR + settings.STATIC_ROOT
        response = StreamingHttpResponse(file_iterator(download_url + '主动添加终端模板.xlsx'))
        response['Content-Type'] = 'application/octet-stream'
        response['Content-Disposition'] = 'attachment;filename="{0}"'.format(the_file_name)

        return response

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def devices_import(self, request, *args, **kwargs):
        """
        终端主动加域导入模板数据
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        file = request.FILES.get('file', None)
        filename = file.name
        if not filename.endswith('.xlsx'):
            return Response('请上传xlsx格式文件', status=status.HTTP_400_BAD_REQUEST)
        wb = load_workbook(filename=BytesIO(file.read()))

        logger.info(wb.worksheets)
        rows = wb.worksheets[0].rows
        total = 0
        count = 0
        adding = 0
        allow_device = []

        conn = get_redis_connection('redis')
        for i, row in enumerate(rows):
            if i == 0 or i == 1:
                continue
            device_cnname = row[4].value
            device_ip = row[0].value
            if device_ip is None:
                return Response(f'{filename}导入成功', status=status.HTTP_200_OK)
            try:
                ipaddress.ip_address(device_ip)
            except ValueError:
                return Response(f'{device_ip} ip格式不正确', status=status.HTTP_400_BAD_REQUEST)
            if device_cnname.isupper() or ('@' in device_cnname) or ('_' in device_cnname):
                return Response(f'{device_cnname}主机名只能输入小写，数字 - .', status=status.HTTP_400_BAD_REQUEST)
            if len(device_cnname) > 32:
                return Response(f'{device_cnname}主机名长度不能超过32', status=status.HTTP_400_BAD_REQUEST)

            # for i, row in enumerate(rows):
            #     # 跳过第一行
            #     if i == 0 or i == 1:
            #         continue

            device_ip = row[0].value
            device_port = row[1].value
            device_username = row[2].value
            device_passwd = row[3].value
            device_cnname = row[4].value
            device_hostname = device_uuid = f'{device_cnname}.{os.environ.get("DOMAIN_NAME")}'
            # device_auto_domain = 2 if row[5].value == '仅管控' else 1
            device_auto_domain = 2
            org_info = row[5].value
            # is_up = row[7].value
            # note = row[8].value
            belong_user = row[6].value
            if conn.get(f'{device_ip}_adding_domain'):
                adding += 1
                continue
            # 缓存加域状态
            conn.set(f'{device_ip}_adding_domain', 1, ex=300, nx=True)

            if device_ip is None:
                continue
            device = Devices.objects.filter(
                Q(device_cnname=device_cnname) | Q(device_ip=device_ip) | Q(device_hostname=device_hostname) | Q(
                    device_uuid=device_uuid)
            ).first()
            if device:
                if device.is_domain or device.is_domain in (1, 2):
                    continue
            else:
                device = Devices.objects.create(device_hostname=device_hostname, device_cnname=device_cnname,
                                                device_ip=device_ip)
            device.device_ip = device_ip
            device.device_port = device_port
            device.device_username = device_username
            device.device_passwd = device_passwd
            device.device_cnname = device_cnname
            device.device_hostname = device_hostname
            device.device_uuid = device_hostname
            org = self.build_and_get_structure(org_info)
            device.org_info = org
            device.note = None
            device.belong_user = belong_user
            # if is_up:
            #     device.is_up = 1 if is_up == '开' else 0
            # else:
            #     device.is_up = 1
            device.is_up = 1
            device.save()
            data = {
                device_cnname: {
                    "host": device_ip,
                    "user": device_username,
                    "passwd": device_passwd,
                    "port": device_port
                }
            }
            allow_device.append(data)
            # set_roster(data)
            # # auto_load_salt_shh(device_ip, device_port, device_username, device_passwd, device_cnname)
            # thread1 = threading.Thread(target=auto_install_client,
            #                            args=(device_cnname, device_ip, device_passwd, 2, device_auto_domain))
            # thread1.start()
            # total += 1
        for data in allow_device:
            set_roster(data)
        for data in allow_device:
            cnname = list(data.keys())[0]
            thread1 = threading.Thread(target=auto_install_client,
                                       args=(
                                           cnname, data.get(cnname).get('host'), data.get(cnname).get('user'),
                                           data.get(cnname).get('passwd'),
                                           2, 2
                                       )
                                       )
            thread1.start()
        if allow_device:
            msg = '批量导入成功,请等待加域结果。。。'
        elif adding:
            msg = '导入的设备正在加域，请等待加域结果。。。'
        else:
            msg = '导入的设备均已在域中！'
        return Response(msg, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, permission_classes=[], authentication_classes=[])
    def devices_user_update(self, request, *args, **kwargs):
        """
        更新终端所属用户
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        file = request.FILES.get('file', None)
        filename = file.name
        if not filename.endswith('.xlsx'):
            return Response('请上传xlsx格式文件', status=status.HTTP_400_BAD_REQUEST)
        wb = load_workbook(filename=BytesIO(file.read()))
        rows = wb.worksheets[0].rows
        for i, row in enumerate(rows):
            if i == 0 or i == 1:
                continue
            device_ip = row[0].value
            if device_ip is None:
                return Response(f'{filename} 更新终端所属信息成功', status=status.HTTP_200_OK)
            try:
                ipaddress.ip_address(device_ip)
            except ValueError:
                return Response(f'{device_ip} ip格式不正确', status=status.HTTP_400_BAD_REQUEST)

            device_ip = row[0].value
            org_info = row[1].value
            belong_user = row[2].value
            org = self.build_and_get_structure(org_info)
            if org_info:
                Devices.objects.filter(device_ip=device_ip).update(org_info=org)
            if belong_user:
                Devices.objects.filter(device_ip=device_ip).update(belong_user=belong_user)

        return Response('更新终端所属信息成功', status=status.HTTP_200_OK)

    def download_device_info(self, queryset):
        conn = get_redis_connection('redis')
        devices_id_list = []
        for item in queryset:
            if conn.hget('actives', item.device_hostname) == b'1':
                devices_id_list.append(item.id)

        queryset = queryset.values_list('id', 'device_cnname', 'device_ip', 'org_info_id', 'belong_user', 'is_up',
                                        'device_hostname', 'os',
                                        'accounts_id', 'device_uid', 'device_sid')
        # head_data = [u'主键', u'终端别名', u'IP 地址', u'所属部门', u'所属用户', u'终端状态', u'在线状态',
        #              u'主机名',
        #              u'机器唯一标识码', u'操作系统', u'账号id']

        head_data = [u'主键', u'终端别名', u'IP 地址', u'所属部门', u'所属用户', u'终端状态',
                     u'主机名',
                     u'操作系统', u'账号id', u'设备ID', u'SID']
        df = pd.DataFrame(queryset, columns=head_data)
        df = df.where(pd.notna(df), None)
        df['终端状态'] = df['终端状态'].apply(lambda x: '启用' if x == 1 else '禁用')
        df['所属部门'] = df['所属部门'].apply(lambda x: get_up_3_organ_name(x))
        df['姓名'] = df['账号id'].apply(lambda x: AccountEntity.objects.get(id=x).cn_name if x is not None else None)
        df['所属用户'] = df['姓名'].values
        # 可登陆人员暂时
        # df['可登录人员'] = df['主键'].apply(
        #     lambda x: (";".join(queryset.filter(id=x).values_list('bind_account__name', flat=True)))
        #     if queryset.filter(id=x).values_list('bind_account__name', flat=True) else None
        # )
        df['可登录人员'] = df['主键'].apply(lambda x: self.bind_user_info(x))
        df['在线状态'] = df['主键'].apply(lambda x: '在线' if x in devices_id_list else '离线')
        df['操作系统'] = df['主键'].apply(
            lambda x: DevicesHardware.objects.get(device_id=x).os if DevicesHardware.objects.filter(
                device_id=x).exists() else None)
        df['产品机型'] = df['主键'].apply(
            lambda x: DevicesHardware.objects.get(device_id=x).productname if DevicesHardware.objects.filter(
                device_id=x).exists() else None)
        df['内核版本'] = df['主键'].apply(
            lambda x: DevicesHardware.objects.get(device_id=x).kernelrelease if DevicesHardware.objects.filter(
                device_id=x).exists() else None)
        df['制造商'] = df['主键'].apply(
            lambda x: DevicesHardware.objects.get(device_id=x).mainboard if DevicesHardware.objects.filter(
                device_id=x).exists() else None)
        df['处理器'] = df['主键'].apply(
            lambda x: '{} {}'.format(DevicesHardware.objects.get(device_id=x).cpu_model,
                                     DevicesHardware.objects.get(device_id=x).cpuarch)
            if DevicesHardware.objects.filter(device_id=x).exists() else None)
        df['内存'] = df['主键'].apply(
            lambda x: DevicesHardware.objects.get(device_id=x).mem_total if DevicesHardware.objects.filter(
                device_id=x).exists() else None)
        df['网络'] = df['主键'].apply(
            lambda x: DevicesHardware.objects.get(device_id=x).network if DevicesHardware.objects.filter(
                device_id=x).exists() else None)

        res_df = df[['终端别名', 'IP 地址', '所属部门', '所属用户', '终端状态', '在线状态',
                     '主机名', '设备ID', 'SID', '操作系统', '产品机型', '内核版本', '制造商', '处理器', '网络',
                     '可登录人员']]

        filename = f'nfs_domain_终端导出_{datetime.datetime.now().strftime("%Y%m%d%H%M%S")}.xlsx'
        res_df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"终端已导出完成, {filename}，总共{len(queryset)}条数据",
            'create_time': datetime.datetime.now(),
            'target_module': '终端导出',
            'message_user': self.request.user.username,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, self.request.user.username)
        close_old_connections()

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def device_switch(self, request, *args, **kwargs):
        """
        批量启用，禁用设备，删除，调整部门, 导出
        :param request:body {
                        is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        is_up: true|启用； false|禁用 空|不处理
                        is_delete : 0|不删除 1|删除 空|不处理
                        to_organ_id: 目标部门 空|不处理
                        is_down_file: 0|不下载 1|下载 空|不处理
                        is_power: 0:关机, 1:重启
                        filename: 文件名
                        ......: 筛选条件
                        }
        :param args:
        :param kwargs:
        :return:
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        is_up = request.data.get('is_up')
        is_delete = request.data.get('is_delete')
        to_organ_id = request.data.get('to_organ_id')
        is_down_file = request.data.get('is_down_file')
        is_power = request.data.get('is_power')
        filename = request.data.get('filename')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)
        if ids_list is None:
            return Response('ids_list 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)

        queryset = Devices.objects.filter(is_domain__in=[1, 2]).all()

        if is_all == 0:
            ids_list = [int(i) for i in ids_list.split(',')]
        else:
            queryset = self.device_filter_queryset(queryset)
            ids_list = queryset.values_list('id', flat=True)

        # 查询过滤结果集
        queryset = queryset.filter(id__in=ids_list)
        # 设备启用，禁用
        if is_up == 'true':
            for item in queryset:
                try:
                    hbacrule_enable(item.device_hostname)
                except Exception as exp:
                    logger.info(f"设备启用失败，{exp}")
                else:
                    item.is_up = 1
                    item.save()
            return Response(f'批量启用终端成功', status=status.HTTP_200_OK)
        elif is_up == 'false':
            for item in queryset:
                try:
                    hbacrule_disable(item.device_hostname)
                except Exception as exp:
                    logger.info(f"设备禁用失败，{exp}")
                else:
                    item.is_up = 0
                    item.save()
            return Response(f'批量禁用终端成功', status=status.HTTP_200_OK)
        else:
            pass

        # 设备删除
        conn = get_redis_connection('redis')
        public = RedisHelper()
        if is_delete == 1:
            for device in queryset:
                actives = True if conn.hget('actives', device.device_hostname) == b'1' else False
                # 提前获取设备相关的用户,避免删除后查不到                
                device_belong_user = AccountEntity.objects.filter(id=device.accounts_id).first()
                if actives:
                    # delete_freeipa.delay(DeviceSerializer(device).data)
                    signal = {
                        'event_type': 'ws_delete',
                        'device_ip': device.device_ip,
                        'device_hostname': device.device_hostname,
                        'success': 'deleting',
                        'msg': '退域策略已下发'
                    }
                    public.public(json.dumps(signal), chan='freeipa')
                else:
                    # 处理未被发送出去的指令
                    send_msg = {'sender': 'SERVER', 'sn': '1-1-3', 'data': {'exit_domain': 1}}
                    device_unsent_value = redis_conn.hget('client_websocket_unsent', device.device_hostname)
                    if device_unsent_value:
                        device_unsent_value = device_unsent_value.decode('utf-8')
                        device_unsent_value = json.loads(device_unsent_value)
                        device_unsent_value.append(send_msg)
                        redis_conn.hset('client_websocket_unsent', device.device_hostname,
                                        json.dumps(device_unsent_value, ensure_ascii=False))
                    else:
                        redis_conn.hset('client_websocket_unsent', device.device_hostname,
                                        json.dumps([send_msg], ensure_ascii=False))

                signal = {
                    'event_type': 'delete',
                    'device_ip': device.device_ip,
                    'device_hostname': device.device_hostname,
                    'success': True,
                    'msg': '{message} 退域成功'.format(message=device.device_hostname)
                }
                public.public(json.dumps(signal), chan='freeipa')
                logger.info(f'删除终端{device.device_hostname}成功')

                # 记录客户端日志(异步)
                client_log = {
                    'device_uuid': device.device_uuid,
                    'device_cnname': device.device_cnname,
                    'device_org_id': device.org_info.id if device.org_info else 0,
                    'account_name': device_belong_user.name if device_belong_user else '',
                    'account_cnname': device_belong_user.cn_name if device_belong_user else '',
                    'account_org_id': device_belong_user.org_info.id if device_belong_user else 0,
                    'module': ClientLogModule.DEVICE.value,
                    'action': ClientLogAction.EXIT_DOMAIN.value,
                    'operate_uuid': get_uuid(),
                    'result_status': 1,  # 退域结果
                    'request': json.dumps(request.data),
                    'response': json.dumps(signal),
                    'device_info_json': json.dumps({
                        'device_hostname': device.device_hostname,
                        'device_ip': device.device_ip
                    }),
                    'account_info_json': json.dumps({
                        'email': device_belong_user.email if device_belong_user else ''
                    })
                }
                client_log_add.delay(client_log=client_log)

            # queryset.delete()
            return Response(f'批量删除终端成功', status=status.HTTP_200_OK)

        if to_organ_id:
            queryset.update(org_info_id=to_organ_id)
            return Response(f'批量修改终端成功', status=status.HTTP_200_OK)

        if is_down_file == 1:
            # 保存文件到服务器
            if len(queryset) == 0:
                return Response('未找到满足条件的数据，导出失败', status=status.HTTP_200_OK)

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self.download_device_info, args=(queryset,))
            thread1.start()

            # output = io.BytesIO()
            # res_df.to_excel(output, encoding='utf-8', index=False)
            # output.seek(0)

            # 将文件传送到web
            # response = StreamingHttpResponse(output)
            # response['Content-Type'] = 'application/octet-stream'
            # response['Content-Disposition'] = 'attachment;filename="{0}"'.format(filename)

            return Response('终端导出中，请前往通知中心查看', status.HTTP_200_OK)

        if is_power in (0, 1):
            conn = get_redis_connection('redis')
            data_json = {
                "sender": "2",
                "sn": "9-8-7",
                "data": {
                    "adminname": request.user.username,
                    "op": is_power,  # 0:poweroff, 1:reboot
                    "display_time": 1 * 60,
                }
            }
            for item in queryset:
                if conn.hget('actives', item.device_hostname) != b'1':
                    continue
                msg_to_client(1, item.device_hostname, data_json)
            return Response('操作已下发', status.HTTP_200_OK)

    @action(methods=['GET', 'POST', 'DELETE'], detail=False, authentication_classes=[TokenAuthentication])
    def bind_device(self, request, *args, **kwargs):
        """用户获取可登录的终端"""
        conn = get_redis_connection('redis')
        if request.method == 'GET':
            bind_accounts = request.query_params.get('bind_accounts')
            bind_device_items = self.queryset.filter(bind_account=bind_accounts).values('id', 'device_hostname',
                                                                                        'device_ip', 'org_info__name')
            df = pd.DataFrame(bind_device_items)
            if bind_device_items.exists():
                df['is_active'] = df['device_hostname'].apply(lambda x: 1 if conn.hget('actives', x) == b'1' else 0)
            else:
                df['is_active'] = 0
            return Response(df.to_dict('records'), status=status.HTTP_200_OK)
        elif request.method == 'POST':
            """可登录人员添加终端"""
            bind_accounts = request.data.get('bind_accounts')
            devices_list = request.data.get('devices_list')
            bind_device_list = list(self.queryset.filter(bind_account=bind_accounts).values_list('id', flat=True))
            add_devices_list = list(set(devices_list).difference(set(bind_device_list)))
            delete_devices_list = list(set(bind_device_list).difference(set(devices_list)))
            bind_accounts_items = AccountEntity.objects.filter(id=bind_accounts).first()
            add_device_items = self.queryset.filter(id__in=add_devices_list)
            delete_device_items = self.queryset.filter(id__in=delete_devices_list)
            try:
                for i in add_device_items:
                    i.bind_account.add(bind_accounts_items)
                    # freeipa 人员可登录的终端
                    rule_name = 'hbac_' + i.device_hostname
                    hbacrule_add_user(rule_name, bind_accounts_items.name)
                for i in delete_device_items:
                    i.bind_account.remove(bind_accounts_items)
                    rule_name = 'hbac_' + i.device_hostname
                    hbacrule_remove_user(rule_name, bind_accounts_items.name)
            except Exception as exp:
                return Response(f"freeipa connect error: {exp}", status=status.HTTP_406_NOT_ACCEPTABLE)

            bind_device_items = self.queryset.filter(bind_account=bind_accounts).values('id', 'device_hostname',
                                                                                        'device_ip', 'org_info__name')
            df = pd.DataFrame(bind_device_items)
            if bind_device_items.exists():
                df['is_active'] = df['device_hostname'].apply(lambda x: 1 if conn.hget('actives', x) == b'1' else 0)
            else:
                df['is_active'] = None
            return Response(df.to_dict('records'), status=status.HTTP_200_OK)
        elif request.method == 'DELETE':
            bind_accounts = request.data.get('bind_accounts')
            devices_list = request.data.get('devices_list')
            bind_accounts_items = AccountEntity.objects.filter(id=bind_accounts).first()
            device_items = Devices.objects.exclude(is_domain=0).filter(id__in=devices_list)
            try:
                for i in device_items:
                    i.bind_account.remove(bind_accounts_items)
                    rule_name = 'hbac_' + i.device_hostname
                    hbacrule_remove_user(rule_name, bind_accounts_items.name)
            except Exception as exp:
                return Response(f"freeipa connect error: {exp}", status=status.HTTP_406_NOT_ACCEPTABLE)
            bind_device_items = self.queryset.filter(bind_account=bind_accounts).values('id', 'device_hostname',
                                                                                        'device_ip', 'org_info__name')
            df = pd.DataFrame(bind_device_items)
            if bind_device_items.exists():
                df['is_active'] = df['device_hostname'].apply(lambda x: 1 if conn.hget('actives', x) == b'1' else 0)
            else:
                df['is_active'] = None
            return Response(df.to_dict('records'), status=status.HTTP_200_OK)
        else:
            return Response(f'方法{request.method}不被允许', status=status.HTTP_200_OK)

    @action(methods=['POST'], detail=False, authentication_classes=[TokenAuthentication])
    def device_file_info(self, request, *args, **kwargs):
        """
        下载终端文件
        :param request: {'file_path':'/var/log/nfs-domain-client/nfs-domain-agent.log'}
        :param args:
        :param kwargs:
        :return: {'download_url':'http://172.30.98.132:8082/mainapi/client/********/'}
        """
        device_id = self.request.data.get('device_id')
        file_path = self.request.data.get('file_path')
        if (device_id is None) or (file_path is None):
            return Response('device_id和file_path为必输项', status=status.HTTP_400_BAD_REQUEST)
        device_items = self.queryset.filter(id=device_id)
        celery_task = compress_upload_file_policy(
            DeviceSerializer(device_items, many=True).data,  # celery参数不能是query_set类型
            {'file_path': file_path, 'domain_name': os.environ.get("DOMAIN_NAME"),
             'username': self.request.user.username},
            **{'policy_no': None},
        )
        return Response('正在下载', status=status.HTTP_200_OK)

    @action(methods=['GET'], detail=True, authentication_classes=[TokenAuthentication])
    def devices_user(self, request, *args, **kwargs):
        queryset = self.get_object()
        if queryset.device_passwd:
            encoded_data = base64.b64encode(queryset.device_passwd.encode("utf-8"))
            device_passwd = encoded_data.decode("utf-8")
        else:
            device_passwd = None
        device_username = queryset.device_username if queryset.device_username else None
        return Response(data={'device_username': device_username, 'device_passwd': device_passwd},
                        status=status.HTTP_200_OK)

    @action(methods=['GET'], detail=True, authentication_classes=[TokenAuthentication])
    def remote_addr(self, request, *args, **kwargs):
        queryset = self.get_object()
        if queryset.is_up != 1:
            return Response(f'终端已被禁用,无法ssh远程', status=status.HTTP_412_PRECONDITION_FAILED)
        if queryset.is_domain == 1:
            encoded_data = base64.b64encode(os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD").encode("utf-8"))
            device_passwd = encoded_data.decode("utf-8")
            device_username = os.environ.get("DOMAIN_SERVER_ADMIN")
            device_port = 22
        else:
            if queryset.device_passwd:
                encoded_data = base64.b64encode(queryset.device_passwd.encode("utf-8"))
                device_passwd = encoded_data.decode("utf-8")
            else:
                device_passwd = None
            device_username = queryset.device_username
            device_port = queryset.device_port if queryset.device_port else 22
        device_ip = queryset.device_ip
        remote_addr = f'http://{os.environ.get("DOMAIN_IP")}/webssh/?username={device_username}&password={device_passwd}&port={device_port}&hostname={device_ip}'
        return Response(remote_addr, status=status.HTTP_200_OK)

    @action(methods=['GET'], detail=True, authentication_classes=[TokenAuthentication])
    def remote_files(self, request, *args, **kwargs):
        queryset = self.get_object()
        if queryset.is_up != 1:
            return Response(f'终端已被禁用,无法远程文件', status=status.HTTP_412_PRECONDITION_FAILED)
        if queryset.is_domain == 1:
            encoded_data = base64.b64encode(os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD").encode("utf-8"))
            device_passwd = encoded_data.decode("utf-8")
            device_username = os.environ.get("DOMAIN_SERVER_ADMIN")
        else:
            if queryset.device_passwd:
                encoded_data = base64.b64encode(queryset.device_passwd.encode("utf-8"))
                device_passwd = encoded_data.decode("utf-8")
            else:
                device_passwd = None
            device_username = queryset.device_username
        device_ip = queryset.device_ip
        """http://172.22.1.163:8334/login?type=sftp&hostname=192.168.47.128&username=root&password=nfs12345"""
        remote_files = f'http://{os.environ.get("DOMAIN_IP")}:8334/login?type=sftp&hostname={device_ip}&username={device_username}'
        try:
            # 创建一个套接字对象并尝试连接到指定的IP地址和端口
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.settimeout(5)  # 设置超时时间（秒）
                s.connect((device_ip, 22))
            return Response({'remote_files':remote_files,'password':device_passwd}, status=status.HTTP_200_OK)
        except Exception as e:
            return Response(f"无法连接到 {device_ip}:22，错误信息: {str(e)}", status=status.HTTP_400_BAD_REQUEST)

    @action(methods=['GET'], detail=False, authentication_classes=[TokenAuthentication])
    def device_list(self, request, *args, **kwargs):
        """
        获取终端列表信息
        :param request: is_all: 0|非全选，1|全选
                        ids_list： [1,2,3,4,...]
        :param args:
        :param kwargs:
        :return:
        """
        is_all = self.request.query_params.get('is_all', None)
        ids_list = self.request.query_params.get('ids_list', '[]')
        ids_list = ids_list.split(',') if ids_list is not None else []
        queryset = self.get_queryset()
        # logger.info(f"**********{type(is_all)},{type(ids_list)}")
        if is_all is None:
            pass
        elif is_all == str(0):
            if not isinstance(ids_list, list):
                return Response('ids_list传参非法', status=status.HTTP_400_BAD_REQUEST)
            else:
                queryset = self.get_queryset().filter(id__in=ids_list)
        elif is_all == str(1):
            ids_list = self.filter_queryset(queryset).values_list('id', flat=True)
            return Response({'ids_list': ids_list}, status=status.HTTP_200_OK)
        else:
            return Response('is_all传参非法', status=status.HTTP_400_BAD_REQUEST)

        queryset = self.filter_queryset(queryset)

        user_data = request.user
        org_list = get_org_permission_list(user_data)
        if org_list:
            queryset = queryset.filter(org_info__in=org_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)


class BindAccountsViewSet(ModelViewSet):
    queryset = AccountEntity.objects.all()
    serializer_class = BindAccountSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = None
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)

    def list(self, request, *args, **kwargs):
        bind_devices = request.query_params.get('bind_device')
        bind_accounts_list = Devices.objects.filter(id=bind_devices).values_list('bind_account', flat=True)
        queryset = self.queryset.filter(id__in=bind_accounts_list)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    def create(self, request, *args, **kwargs):
        """
        可登录的终端添加人员
        :param request: bind_device    accounts_list:
        :param args:
        :param kwargs:
        :return:
        """
        bind_devices = request.data.get('bind_device')
        accounts_list = request.data.get('accounts_list')

        # 添加freeipa规则
        bind_accounts_list = list(Devices.objects.filter(id=bind_devices).values_list('bind_account', flat=True))
        add_accounts_list = list(set(accounts_list).difference(set(bind_accounts_list)))
        hostname = Devices.objects.get(id=bind_devices).device_hostname
        rule_name = 'hbac_' + hostname
        try:
            for accounts_id in add_accounts_list:
                account = AccountEntity.objects.get(id=accounts_id)
                hbacrule_add_user(rule_name, account.name)
        except Exception as exp:
            return Response(f"freeipa connect error: {exp}", status=status.HTTP_406_NOT_ACCEPTABLE)
        else:
            Devices.objects.filter(id=bind_devices).first().bind_account.set(accounts_list)

        queryset = self.queryset.filter(id__in=accounts_list)
        serializer = self.get_serializer(queryset, many=True)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    def destroy(self, request, *args, **kwargs):
        is_all = request.data.get('is_all')
        bind_devices = request.data.get('bind_device')
        accounts_list = request.data.get('accounts_list')
        if is_all == 1:
            bind_accounts_list = Devices.objects.filter(id=bind_devices).values_list('bind_account', flat=True)
            accounts_list = bind_accounts_list
        # freeipa 删除终端上可登录的用户
        try:
            hostname = Devices.objects.get(id=bind_devices).device_hostname
            rule_name = 'hbac_' + hostname
            for accounts_id in accounts_list:
                account = AccountEntity.objects.get(id=accounts_id)
                hbacrule_remove_user(rule_name, account.name)
                Devices.objects.filter(id=bind_devices).first().bind_account.remove(accounts_id)
        except Exception as exp:
            return Response(f"freeipa connect error: {exp}", status=status.HTTP_406_NOT_ACCEPTABLE)

        bind_accounts_list = Devices.objects.filter(id=bind_devices).values_list('bind_account', flat=True)
        queryset = self.queryset.filter(id__in=bind_accounts_list)
        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_204_NO_CONTENT)
