"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/11/16 下午3:33
@File: web_consumers.py
@Desc: 前端和服务器websocket通信
"""
import copy
import datetime
import json
import logging
import os
import time
import random
import asyncio
try:
    import aioredis
    aioredis_import = 'aioredis'
except ImportError:
    from redis import asyncio as aioredis
    aioredis_import = 'asyncio'

from channels.generic.websocket import AsyncWebsocketConsumer
from channels.layers import get_channel_layer
from django_redis import get_redis_connection
from asgiref.sync import async_to_sync, sync_to_async

from message_center.models import MessageCenter
from nfs_domain_mainserver.settings import SALTFILESYSTEMPATH
from system.models import DeviceOTP
from users.models import User

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')
channel_layer = get_channel_layer()


class WebConsumer(AsyncWebsocketConsumer):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.is_connected = None
        self.username = None
        self.client_info = None
        self.db_query_task = None
        self.active_event = asyncio.Event()  # 用于控制循环
        self.active_event.set()  # 初始化为活跃状态
        self.group_name = 'channel'  # 广播频道
        self.actions = {
            '0-0-0': self.unread_message,  # 0-0-0, 用户未读消息
            '0-0-1': self.device_otp,  # 0-0-1, 终端动态卸载口令
            '0-0-2': self.close_otp,  # 0-0-2, 前端离开页面
            '1-1-1': self.log_message,  # 1-1-1，获取终端文件日志内容
        }
        # 统一websocket服务端返回信息
        self.server_result_msg = {'sender': 'SERVER', 'web_sn': None, 'code': 200, 'data': None, 'msg': None}

    async def connect(self):
        if aioredis_import == 'aioredis':
            self.redis_conn = await aioredis.create_redis(
                # 使用3号库，websocket连接通道存在redis3号库中
                "redis://{}:{}/3".format(os.environ.get("REDIS_HOST"), os.environ.get('REDIS_PORT'))
            )
        else:
            self.redis_conn = await aioredis.from_url(
                # 使用3号库，websocket连接通道存在redis3号库中
                "redis://{}:{}/3".format(os.environ.get("REDIS_HOST"), os.environ.get('REDIS_PORT'))
            )
        # 当 WebSocket 连接建立时调用
        await self.accept()
        self.client_info = self.scope["client"] if "client" in self.scope.keys() else None
        user = self.scope['query_string'].decode('utf-8')
        username = user.split('user=')[-1]
        # logger.info(f"******{username}****{bool(username)}")
        if not bool(username):
            pass
        else:
            self.username = username
            web_channel_name = await self.redis_conn.hget('web_channel_name', username)
            if web_channel_name:
                web_channel_name = web_channel_name.decode('utf-8')
            if self.channel_name != web_channel_name:
                await self.redis_conn.hset('web_channel_name', username, self.channel_name)

        # 加入 chat group
        await self.channel_layer.group_add(
            self.group_name,  # 广播频道
            self.channel_name  # 通道名称
        )
        self.is_connected = True

    async def disconnect(self, close_code):
        # 当 WebSocket 连接关闭时调用
        self.active_event.clear()  # 清除事件以停止查询
        if self.db_query_task:
            self.db_query_task.cancel()  # 取消查询任务
        if self.username is not None:
            await self.redis_conn.hdel('web_channel_name', self.username)

        # 离开 chat group
        await self.channel_layer.group_discard(
            self.group_name,
            self.channel_name
        )
        self.is_connected = False
        logger.info(f'前端websocket连接关闭 {self.client_info[0]}')

    async def receive(self, text_data=None, bytes_data=None):
        # 当从 WebSocket 接收到消息时调用
        text_data_json = json.loads(text_data)
        message = text_data_json['message']

        # 在这里可以添加处理接收到的消息的逻辑
        # 发送消息给 特定通道
        await self.channel_layer.send(
            self.channel_name,
            {
                'type': 'push_message',
                'message': message
            }
        )

    async def push_message(self, event):
        # 当从 push_message 中收到消息时调用，广播消息
        message = event['message']
        server_msg = await self.msg_sn_input(message)
        web_channel_name = await self.redis_conn.hget('web_channel_name', self.username) if self.username is not None else None
        if web_channel_name:
            # Send message to WebSocket
            if self.is_connected:
                logger.info(f"向前端websocket发送的消息{server_msg}")
                await self.send(text_data=json.dumps({'message': server_msg}, ensure_ascii=False))
        else:
            pass

    async def send_message(self, event):
        server_msg = event['message']
        logger.info(f"向前端websocket发送的消息{event}")
        await self.send(text_data=json.dumps({'message': server_msg}, ensure_ascii=False))

    async def msg_sn_input(self, option):
        """
        通过指令转为相应的消息处理机制
        :param option: {"message": {"sender": "WEB","web_sn": "1-1-1","data": None,}}
        :return:
        """
        web_sn = option.get('web_sn')
        msg_data = option.get('data')

        # 检查选项是否在字典中
        if web_sn in self.actions:
            # 执行与选项对应的操作
            server_msg = await self.actions[web_sn](msg_data)
        else:
            # 执行默认操作
            server_msg = self.default_action(option)

        return server_msg

    @sync_to_async
    def unread_message(self, msg_data):
        """
        获取未读消息返回
        :param msg_data:
        :return:
        """
        logger.info(f"0-0-0, msg_data:{msg_data}")
        if isinstance(msg_data, dict):
            if 'message_user' in msg_data.items():
                username = msg_data.get('message_user', None)
            else:
                username = None
        else:
            username = None

        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        if User.objects.filter(username=self.username if username is None else username, is_superuser=1).exists():
            unread_message_total = MessageCenter.objects.filter(is_read=0).count()
        else:
            unread_message_total = MessageCenter.objects.filter(
                message_user=self.username if username is None else username, is_read=0).count()
        result_msg.update({
            'web_sn': '0-0-0',  # 未读消息
            'data': {'message_user': self.username if username is None else username,
                     'unread_message_total': unread_message_total}
        })
        return result_msg

    @sync_to_async
    def log_message(self, msg_data):
        """
        获取通知消息中心返回
        :param msg_data:
        :return:
        """
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        result_msg.update({'web_sn': '1-1-1', 'data': msg_data})
        return result_msg

    async def device_otp(self, msg_data):
        """
        获取终端动态口令监控
        :param msg_data: {"device_opt":""}
        :return:
        """
        logger.info(f"0-0-1, msg_data:{msg_data}")
        result_msg = copy.deepcopy(self.server_result_msg)  # 获取返回消息的统一格式
        device_otp = msg_data.get('device_otp', None)
        if device_otp is None or "":
            result_msg.update({
                'web_sn': '0-0-1',  # 未读消息
                'code': 400,
                'data': None,
                'msg': "device_opt传入参数错误"
            })
        else:
            self.active_event.set()
            self.db_query_task = asyncio.ensure_future(self.query_database(msg_data))
            latest_otp = await sync_to_async(DeviceOTP.objects.filter(id=1).first)()

            otp_time = datetime.datetime.now() - latest_otp.created_at
            otp_time = int(180 - otp_time.total_seconds())
            result_msg.update({
                'web_sn': '0-0-1',  # 未读消息
                'data': {'device_otp': latest_otp.code, 'otp_time': otp_time}
            })

        return result_msg

    async def query_database(self, msg_data):
        device_otp = msg_data.get('device_otp', None)
        result_msg = copy.deepcopy(self.server_result_msg)
        try:
            while self.active_event.is_set():  # 使用事件来控制循环:
                # await sync_to_async(YourModel.objects.first)()
                latest_otp = await sync_to_async(DeviceOTP.objects.filter(id=1).first)()
                if not latest_otp:
                    await sync_to_async(DeviceOTP.objects.create)(**{"id": 1,
                                                                     "code": str(random.randint(100000, 999999)),
                                                                     "created_at": datetime.datetime.now()
                                                                     })
                otp_time = datetime.datetime.now() - latest_otp.created_at
                otp_time = int(180 - otp_time.total_seconds())

                if (str(device_otp) != latest_otp.code) or (otp_time <= 0):
                    latest_code = str(random.randint(100000, 999999))
                    await sync_to_async(DeviceOTP.objects.filter(id=1).update)(
                        code=latest_code,
                        created_at=datetime.datetime.now())
                    device_otp = latest_code
                    otp_time = 180

                result_msg.update({
                    'web_sn': '0-0-1',  # 未读消息
                    'data': {'device_otp': device_otp, 'otp_time': otp_time}
                })
                # 发送消息给前端
                web_channel_name = await self.redis_conn.hget('web_channel_name',
                                                   self.username) if self.username is not None else None
                if web_channel_name:
                    # Send message to WebSocket
                    logger.info(f"向前端websocket发送的消息{result_msg}")
                    await self.send(text_data=json.dumps({'message': result_msg}, ensure_ascii=False))
                else:
                    break

                await asyncio.sleep(1)
        except Exception as exp:
            logger.info(f"0-0-1 循环任务发生异常，异常情况为{exp}")

    async def close_otp(self, msg_data):
        logger.info(f"0-0-2, msg_data:{msg_data}")
        self.active_event.clear()
        result_msg = copy.deepcopy(self.server_result_msg)
        result_msg.update({
            'web_sn': '0-0-2',  # 未读消息
            'data': msg_data
        })
        return result_msg

    @staticmethod
    def default_action(message):
        """
        默认收到信息转回去
        :param message:
        :return:
        """
        return message


def send_message_to_all_user(group_name, message):
    """
    点对面发送信息
    :param group_name: 广播频道名称
    :param message: 发送消息
    :return:
    """
    async_to_sync(channel_layer.group_send)(
        group_name,
        {
            "type": "send_message",
            "message": message
        }
    )


def send_message_to_web_user(channel_name, message):
    """
    点对点发送消息
    :param channel_name: 管道名称
    :param message: 对管道发送消息
    :return:
    """
    # 发送消息到特定通道
    async_to_sync(channel_layer.send)(
        channel_name,
        {
            "type": "send_message",
            "message": message,
        },
    )


def send_msg_to_web(message, is_broadcast, channel_tag='channel'):
    """
    消息中心发送消息
    判断通道是否在现，然后发送广播消息，或者是特定消息
    :param channel_tag: 要么是用户，要么是特定的通道组
    :param message: 消息
    :param is_broadcast: 广播内容  0 不是 1 是
    :return:
    """
    if is_broadcast == 0:
        n = 0
        username = message.get('message_user', None)
        while True:
            web_channel_name = redis_conn.hget('web_channel_name', username)
            if User.objects.filter(username=username, is_superuser=1).exists():
                unread_message_total = MessageCenter.objects.filter(is_read=0).count()
            else:
                unread_message_total = MessageCenter.objects.filter(message_user=username, is_read=0).count()
            if web_channel_name:
                web_channel_name = web_channel_name.decode('utf-8')
                message_dict = {
                    'sender': 'SERVER',
                    'code': 200,
                    'web_sn': '0-0-0',  # 未读消息
                    'data': {
                        'message_user': username,
                        'unread_message_total': unread_message_total,
                    },
                    'msg': None
                }
                send_message_to_web_user(web_channel_name, message_dict)
                break
            else:
                n = n + 1
                time.sleep(10)
                if n > 3:
                    break
    else:
        send_message_to_all_user(channel_tag, message)

    fifteen_days_ago = datetime.datetime.now() - datetime.timedelta(days=15)

    # 删除十五天之前的数据
    MessageCenter.objects.filter(create_time__lt=fifteen_days_ago).delete()

    # 删除15天之前的文件
    try:
        for filename in os.listdir(SALTFILESYSTEMPATH):
            if filename.startswith("nfs_domain_"):
                file_path = os.path.join(SALTFILESYSTEMPATH, filename)
                creation_time = datetime.datetime.fromtimestamp(os.path.getctime(file_path))

                if creation_time <= fifteen_days_ago:
                    os.remove(file_path)
                    logger.info(f"Deleted old file: {file_path}")
    except Exception as exp:
        logger.info(f"清除15天以上文件失败和处理信息{exp}")
