# -*- coding: utf-8 -*-
import logging
import os.path
from datetime import datetime

from django.conf import settings
from django.http import StreamingHttpResponse
from rest_framework import status
from rest_framework.views import APIView, Response
from utils.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from utils.permission import IsAdminUser
from utils.throttling import UserRateThrottle
from filemanager.models import FileManager
from utils.convert import hum_convert
from utils.utils import list_to_str, str_to_list

logger = logging.getLogger(__name__)


class FileManagerViewSet(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, file_id):
        """
        文件下载
        :param file_id:   文件id
        :param request:
            {
                "file_id":56
            }
        :return:
        """
        if not file_id:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '参数不存在,请重新输入'
            })
        file = FileManager.objects.get(pk=int(file_id))
        if not file:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '参数错误,请重新输入'
            })
        if not os.path.exists(file.path):
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '文件不存'
            })

        def file_iterator(file_name, chunk_size=1024):
            with open(file_name, 'rb') as f:
                while True:
                    c = f.read(chunk_size)
                    if c:
                        yield c
                    else:
                        break

        response = StreamingHttpResponse(file_iterator(file.path))
        response['Content-Type'] = 'application/octet-stream'
        response['Content-Disposition'] = 'attachment;filename="{0}"'.format(file.name)

        return response

    def put(self, request):
        """
        修改文件的任务ID
        :param request:
            {
                "file_id_list":'3,4' ,
                "task":12
            }
        :return:
        """

        req_data = request.data

        file_id_str = req_data.get('file_id_list')
        file_id_list = []
        if "," in file_id_str:
            file_id_list = file_id_str.split(",")
        else:
            file_id_list.append(file_id_str)

        task_id = req_data.get('task')
        if not file_id_list or not task_id:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '参数不存在,请重新输入'
            })

        from tasks.models import Tasks
        for file_id in file_id_list:
            task = Tasks.objects.get(pk=task_id, is_deleted=0)
            file = FileManager.objects.get(pk=file_id)
            if not task or not file:
                return Response({
                    'code': status.HTTP_404_NOT_FOUND,
                    'msg': '参数值错误,请重新输入'
                })
            file.task = task
            file.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '更新文件信息已完成'
        })

    def post(self, request):
        """
        上传文件
        param
            {
                "file":"2.txt",
                "desc":"",
                "version":"",
                "type":0
            }
        """
        req_data = request.data
        file_type = req_data.get("type")
        task_id = req_data.get("task")
        task = None
        if task_id:
            from tasks.models import Tasks
            task = Tasks.objects.get(pk=task_id)

        file_obj_list = request.FILES.getlist('file')
        if not file_obj_list:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请求参数错误'
            })

        file_id_list = []
        for file_obj in file_obj_list:
            filemanager_obj = FileManager()
            if task:
                filemanager_obj.task = task

            filemanager_obj.type = file_type
            filemanager_obj.name = file_obj.name
            user_id = request.user.id
            date_str = datetime.now().strftime("%Y-%m-%d")
            # 路径为 file_data/type/user_id/date/file
            dir_path = os.path.join(settings.FILES_PATH, str(req_data.get('type')), str(user_id), date_str)
            if not os.path.exists(dir_path):
                os.makedirs(dir_path)
            file_path = os.path.join(dir_path, file_obj.name)

            with open(file_path, 'wb') as f:
                for chunk in file_obj.chunks():
                    f.write(chunk)

            filemanager_obj.desc = req_data.get('desc')
            filemanager_obj.version = req_data.get('version')
            filemanager_obj.type = req_data.get('type')

            filemanager_obj.size = hum_convert(file_obj.size)

            filemanager_obj.path = file_path
            filemanager_obj.save()
            file_id_list.append(str(filemanager_obj.pk))

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '上传文件成功',
            'data': {
                "file_id_list": list_to_str(file_id_list)
            }
        })

    def delete(self, request):
        req_data = request.data
        file_id = req_data.get("file_id")
        file_id_list_str = req_data.get("file_id_list")
        if not file_id and not file_id_list_str:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '任务信息不存在'
            })
        if file_id_list_str:
            file_id_list = str_to_list(file_id_list_str)
        else:
            file_id_list = [file_id]
        try:
            files = FileManager.objects.filter(pk__in=file_id_list)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '任务信息不存在'
            })

        for file in files:
            # file 删除
            file_path = file.path
            if os.path.exists(file_path):
                os.remove(file_path)
            file.delete()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除文件成功',
            'data': {
                "file_id": file_id_list_str
            }
        })
