# -*- coding: utf-8 -*-
import logging
import os
from pathlib import  Path
import shutil
import json

from chunked_upload.views import ChunkedUploadView, ChunkedUploadCompleteView
from django.conf import settings
from django.http import JsonResponse
from rest_framework import status
from django.template.defaultfilters import filesizeformat
from rest_framework.response import Response

from utils.authentication import TokenAuthentication
from utils.permission import IsAdminUser
from utils.response import FitJSONRenderer
from utils.throttling import UserRateThrottle

from sysios.models import SysiosIso

logger = logging.getLogger(__name__)


class SysIosChunkedUploadView(ChunkedUploadView):
    """
    镜像模块：文件分块上传:
    1, 需继承ChunkedUploadView
    2, 文件需要提前分片
    3, 只能串行上传,不能并行
    4, 流程步骤如下
        4.1, 前端用for循环调用此接口,每次Headers的属性：Content-Range中的<start>-<end>/<total>的start,end会不同,total不变,
        4.2, 当start=total的时候,说明是最后一个分片,文件传送完毕，调用chunk_upload_complete接口完成收尾
    5, 分片文件的规则: filename.part{index}，文件原始名+.part{分片号码}，如文件[NFS4.iso]的分片文件： NFS4.iso.part0,NFS4.iso.part1,NFS4.iso.part2
    """
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def check_permissions(self, request):
        pass

    def get_response_data(self, chunked_upload, request):
        """重新封装返回对象"""
        parernt_response = super().get_response_data(chunked_upload, request)
        return {
            'data': parernt_response,
            'code': status.HTTP_200_OK
        }

    def post(self, request, *args, **kwargs):
        """
        重新封装post请求
        """
        # 调用父类的post()方法
        response = super().post(request, *args, **kwargs)
        # 如果是错误情况
        if response.status_code != 200:
            # 解析错误内容,获取固定错误属性:detail,offset
            response_json = json.loads(response.content.decode('utf-8'))
            data = None
            offset = response_json.get('offset', None)
            if offset is not None:
                data = {
                    'offset': offset
                }
            return JsonResponse({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': response_json.get('detail', None),
                'data': data,
            }, status=status.HTTP_400_BAD_REQUEST)
        else:
            return response


class SysIosChunkedUploadCompleteView(ChunkedUploadCompleteView):
    """
    镜像模块：文件分块上传完结接口:
    1, 分片上传完成后,调用此接口来做收尾工作
    2, 收尾工作如下：
        2.1, 组件自动将文件完成状态改为'complete'
        2.2, 将上传完毕的文件改名为原始文件名(上传的文件会以{upload_id}.part的格式存放在临时目录下)
        2.3, 将改名后的文件移到/docker/isofactory-docker/data/isofactory_basic_iso/目录下
    """
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    do_md5_check = False #排除md5校验

    def check_permissions(self, request):
        pass

    def get_response_data(self, chunked_upload, request):
        """重新封装返回对象"""
        super().get_response_data(chunked_upload, request)
        # 获取收尾后的文件信息
        complete_file_name = str(chunked_upload.filename)
        # 去掉后面的.part{xxx}
        complete_file_name = str(complete_file_name.split('.part')[0])
        docker_dir = os.path.join(os.path.dirname(settings.BASE_DIR), "docker")
        basic_iso_dir = f"{docker_dir}/isofactory-docker/data/isofactory_basic_iso/source"
        complete_file_path = Path(basic_iso_dir) / complete_file_name
        complete_file_size = os.path.getsize(complete_file_path)
        data = {
            'file_name': complete_file_name,
            'file_path': str(complete_file_path),
            'file_size': filesizeformat(int(complete_file_size))
        }
        return {
            'data': data,
            'code': status.HTTP_200_OK
        }

    def on_completion(self, uploaded_file, request):
        """
        完成上传后的收尾工作
        """
        complete_file_name = str(uploaded_file.name)
        # 去掉后面的.part{xxx}
        complete_file_name = str(complete_file_name.split('.part')[0])
        complete_file_path = str(os.path.join(settings.MEDIA_ROOT, uploaded_file.file.path))
        complete_file_size = uploaded_file.file.size

        # 暂时不用文件服务,只用目录挂载,挂载到[当前项目下的/docker/isofactory-docker/]目录
        # iso镜像文件挂载的基础目录
        docker_dir = os.path.join(os.path.dirname(settings.BASE_DIR), "docker")
        iso_base_dir = f"{docker_dir}/isofactory-docker/data"
        # 创建基础镜像固定目录
        basic_iso_dirname = 'isofactory_basic_iso'
        source_iso_name = str(complete_file_name)
        iso_file_dir = Path(iso_base_dir) / basic_iso_dirname
        iso_file_packages_dir = iso_file_dir / 'packages'
        iso_file_drive_dir = iso_file_packages_dir / 'drives'
        iso_file_software_dir = iso_file_packages_dir / 'softwares'
        iso_file_iso_dir = iso_file_dir / 'source'
        iso_file_path = iso_file_iso_dir / source_iso_name
        iso_file_size = filesizeformat(int(complete_file_size))
        try:
            # 创建目录
            iso_file_dir.mkdir(parents=True, exist_ok=True)
            iso_file_packages_dir.mkdir(parents=True, exist_ok=True)
            iso_file_drive_dir.mkdir(parents=True, exist_ok=True)
            iso_file_software_dir.mkdir(parents=True, exist_ok=True)
            iso_file_iso_dir.mkdir(parents=True, exist_ok=True)
            # 移动文件
            shutil.move(complete_file_path, iso_file_path)

            # 创建一条镜像记录,iso_type=0,status=1
            # 如果基础镜像文件(iso_type=0)存在就认为是重新上传,更新文件path和size
            sysiso_iso = SysiosIso.objects.filter(name=source_iso_name, iso_type=0).first()
            if sysiso_iso:
                sysiso_iso.source_iso_path = iso_file_path
                sysiso_iso.source_iso_size = iso_file_size
                sysiso_iso.iso_path = iso_file_path
                sysiso_iso.iso_size = iso_file_size
                sysiso_iso.updator = request.user.username
                sysiso_iso.save()
            else:
                SysiosIso.objects.create(
                    name=source_iso_name,  # 用文件名
                    is_config=0,
                    is_domain=0,
                    iso_type=0,  # 基础镜像
                    source_iso_name=source_iso_name,
                    source_iso_path=iso_file_path,
                    source_iso_size=iso_file_size,
                    iso_path=iso_file_path,
                    iso_size=iso_file_size,
                    status=1,  # 必须是成功
                    creator=request.user.username
                )
        except Exception as e:
            logger.error(f"镜像文件上传完成后的处理异常:{e}")

    def post(self, request, *args, **kwargs):
        """
        重新封装post请求
        """
        try:
            # 调用父类的post()方法
            response = super().post(request, *args, **kwargs)
            # 如果是错误情况
            if response.status_code != 200:
                # 解析错误内容,获取固定错误属性:detail,offset
                response_json = json.loads(response.content.decode('utf-8'))
                data = None
                offset = response_json.get('offset', None)
                if offset is not None:
                    data = {
                        'offset': offset
                    }
                return JsonResponse({
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': response_json.get('detail', None),
                    'data': data,
                }, status=status.HTTP_400_BAD_REQUEST)
            else:
                return response
        except FileNotFoundError as e1:
            # 如果是文件找不到,因为已被移走,直接返回提示
            return JsonResponse({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '文件已完成,不能重复操作',
            }, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e2:
            logger.error(f"系统异常:{e2}")
            return JsonResponse({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f"系统异常,ErrMsg:{e2}",
            }, status=status.HTTP_400_BAD_REQUEST)