import logging
import time
from io import BytesIO

from django.http import StreamingHttpResponse
from openpyxl.reader.excel import load_workbook

from nfs_domain_mainserver.settings import BASE_DIR
from utils.authentication import TokenAuthentication
from utils.throttling import UserRateThrottle
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.decorators import api_view, permission_classes, authentication_classes
from rest_framework.views import Response
from rest_framework import status
from rest_framework.viewsets import ModelViewSet
from rest_framework.decorators import action
from django.db.models import Q
from django.core.cache import caches
from django.conf import settings

from rest_framework import filters
from utils.response import FitJSONRenderer
from groups.models import Groups, GroupsDetail
from groups.serializers import GroupsSerializer, GroupsDetailSerializer
from utils.pagination import MyPagination
from utils.utils import get_serializer_error
from groups.group_order import get_group_order_id
from users.models import User
from utils.utils import str_to_list
from organizations.models import OrgnizationEntity
from devices.models import Devices
from accounts.models import AccountEntity

logger = logging.getLogger(__name__)


class GroupsViewSet(ModelViewSet):
    queryset = Groups.objects.filter(is_deleted=0).all()
    serializer_class = GroupsSerializer
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    # filter_backends = (filters.SearchFilter)
    # filter_fields = ("name")
    ordering = ('-order_id',)

    def filter_queryset(self, queryset):
        """
        :param queryset: 表数据结果集合
        :return: 根据过滤条件筛选返回结果集合
        """
        name = self.request.query_params.get('name', default=None)
        if name:
            queryset = queryset.filter(name__icontains=name)
        is_user = self.request.query_params.get('is_user', default=None)
        if is_user is not None:
            queryset = queryset.filter(is_user=is_user)
        return queryset

    def list(self, request, *args, **kwargs):
        is_all = self.request.query_params.get('is_all', None)
        ids_list = self.request.query_params.get('ids_list', None)
        ids_list = ids_list.split(',') if ids_list is not None else []
        queryset = self.get_queryset()
        # logger.info(f"**********{type(is_all)},{type(ids_list)}")
        if is_all is None:
            pass
        elif is_all == str(0):
            if not isinstance(ids_list, list):
                return Response('ids_list传参非法', status=status.HTTP_400_BAD_REQUEST)
            else:
                queryset = self.get_queryset().filter(id__in=ids_list)
        elif is_all == str(1):
            ids_list = self.filter_queryset(queryset).values_list('id', flat=True)
            return Response({'ids_list': ids_list}, status=status.HTTP_200_OK)
        else:
            return Response('is_all传参非法', status=status.HTTP_400_BAD_REQUEST)

        queryset = self.filter_queryset(queryset)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    def create(self, request, *args, **kwargs):
        num = len(self.queryset)
        if num > 3000:
            return Response('终端分组已达上限 3000个', status=status.HTTP_400_BAD_REQUEST)

        data = request.data
        name = request.data.get("name")
        remarks = request.data.get("remarks")
        is_user = request.data.get('is_user')

        if Groups.objects.filter(name=name, is_deleted=0, is_user=is_user).exists() > 0:
            return Response('分组名不可与其他分组名重名', status=status.HTTP_400_BAD_REQUEST)
        if remarks is not None and len(remarks) > 200:
            return Response('备注长度限制为 200 个字符以内', status=status.HTTP_400_BAD_REQUEST)

        data["create_user"] = request.user.id
        data["order_id"] = get_group_order_id()
        groups = GroupsSerializer(data=data)
        groups.is_valid(raise_exception=True)
        groups.save()
        return Response(groups.data, status=status.HTTP_200_OK)

    def update(self, request, pk):
        # 查询pk所指定的模型对象
        groups = Groups.objects.filter(id=pk, is_deleted=0)
        if not groups.exists():
            return Response('id 不存在', status=status.HTTP_400_BAD_REQUEST)

        name = request.data.get("name")
        history = Groups.objects.filter(name=name, is_deleted=0).exclude(id=pk)
        if len(history) > 0:
            return Response('分组名不可与其他分组名重名', status=status.HTTP_400_BAD_REQUEST)

        # 获取前端传入的请求体数据
        # 创建序列化器进行反序列化
        serializer = GroupsSerializer(instance=groups.first(), data=request.data)
        # 校验
        if serializer.is_valid():
            serializer.save()
            # 响应
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            error_str = get_serializer_error(serializer)
            return Response(f'参数错误: %s' % error_str, status=status.HTTP_400_BAD_REQUEST)

    def destroy(self, request, *args, **kwargs):
        groups = self.get_object()

        order_id = groups.order_id
        groups.is_deleted = 1
        groups.save()

        # 更新order_id
        update_groups_list = self.queryset.filter(order_id__gt=order_id, is_deleted=0)
        for update_groups in update_groups_list:
            update_groups.order_id = update_groups.order_id - 1
            update_groups.save()

        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def group_account_tmpl(self, request, *args, **kwargs):
        """
        终端分组用户模板导出
        :param request:  filename 文件名
        :param args:
        :param kwargs:
        :return:
        """

        def file_iterator(file_name, chunk_size=512):
            with open(file_name, 'rb') as f:
                while True:
                    c = f.read(chunk_size)
                    if c:
                        yield c
                    else:
                        break

        # 显示在弹出对话框中的默认的下载文件名
        if 'filename' in request.query_params.keys():
            the_file_name = request.query_params.get('filename')
        else:
            the_file_name = '终端分组用户导入模板.xlsx'
        download_url = BASE_DIR + settings.STATIC_ROOT
        response = StreamingHttpResponse(file_iterator(download_url + '终端分组用户导入模板.xlsx'))
        response['Content-Type'] = 'application/octet-stream'
        response['Content-Disposition'] = 'attachment;filename="{0}"'.format(the_file_name)

        return response

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def group_account_import(self, request, *args, **kwargs):
        """
        终端分组用户模板导入
        :param request:  filename 文件名
        :param args:
        :param kwargs:
        :return:

        """
        file = request.FILES.get('file', None)
        groups_id = request.data.get('groups_id', None)
        if (file is None) or (groups_id is None):
            return Response(f'必填参数错误 file: {file}; groups_id: {groups_id}', status=status.HTTP_400_BAD_REQUEST)
        filename = file.name
        if not filename.endswith('.xlsx'):
            return Response('请上传xlsx格式文件', status=status.HTTP_400_BAD_REQUEST)
        wb = load_workbook(filename=BytesIO(file.read()))
        rows = wb.worksheets[0].rows

        account_items = AccountEntity.objects.all()
        groups_detail_items = GroupsDetail.objects.all()
        for i, row in enumerate(rows):
            if i == 0 or i == 1:
                continue
            user_account = row[0].value
            if account_items.filter(name=user_account).exists():
                account_id = account_items.get(name=user_account).id
                groups_detail_items.create(groups_id=groups_id, range=3, account_id=account_id)
            else:
                pass

        return Response('导入数据成功', status=status.HTTP_200_OK)


@api_view(["GET"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def group_order_action(request):
    groups_id = request.query_params.get('groups_id')
    # 0 上移，1 下移
    action = request.query_params.get('action')
    try:
        groups = Groups.objects.filter(id=groups_id, is_deleted=0).first()
    except groups.DoesNotExist:
        return Response('id 不存在', status=status.HTTP_404_NOT_FOUND)

    group_list = Groups.objects.filter(is_deleted=0)
    order_id = groups.order_id

    # 默认order_id 为1，依次累加，通过倒叙排列
    if action == 0 or action == "0":
        # 0 上移
        if order_id == len(group_list):
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '组为最顶级组,无法上移!'
            })

        # 将上一个组的order_id 替换为当前order_id ,当前组的order_id +1
        tmp_order_id = order_id + 1
        update_group = Groups.objects.filter(order_id=tmp_order_id, is_deleted=0).first()
        update_group.order_id = order_id
        update_group.save()
        groups.order_id = order_id + 1
        groups.save()
    else:
        # 1 下移
        if order_id == 1:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '组为最底层组,无法下移!'
            })
        # 将下一个组的order_id 替换为当前order_id ,当前组的order_id -1
        tmp_order_id = order_id - 1
        update_group = Groups.objects.filter(order_id=tmp_order_id, is_deleted=0).first()
        update_group.order_id = order_id
        update_group.save()
        groups.order_id = order_id - 1
        groups.save()
    return Response({
        'code': status.HTTP_200_OK,
        'msg': '移动成功'
    })


class GroupsDetailViewSet(ModelViewSet):
    queryset = GroupsDetail.objects.all()
    serializer_class = GroupsDetailSerializer
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('-id')

    def check_null_value(self, queryset):
        # 删除退域或删除后的终端和部门
        org_list = OrgnizationEntity.objects.all()
        org_id_list = [x.id for x in org_list if x]
        org_queryset = queryset.filter(range=0)
        delete_org_queryset = org_queryset.exclude(org_id__in=org_id_list)
        for delete_org in delete_org_queryset:
            delete_org.delete()

        device_list = Devices.objects.all().exclude(is_domain=0)
        device_id_list = [x.id for x in device_list if x]
        device_queryset = queryset.filter(range=1)
        delete_device_queryset = device_queryset.exclude(device_id__in=device_id_list)
        for delete_device in delete_device_queryset:
            delete_device.delete()
        return True

    @classmethod
    def organ_down_id(cls, num):
        """父节点的id"""
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [num]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

        return organ_down_list

    def filter_queryset(self, queryset):
        """
        :param queryset: 表数据结果集合
        :return: 根据过滤条件筛选返回结果集合
        """
        self.check_null_value(queryset)
        groups_id = self.request.query_params.get('groups_id', default=None)
        range_param = self.request.query_params.get('range', default=None)
        name = self.request.query_params.get('name', default=None)

        if groups_id:
            queryset = GroupsDetail.objects.filter(groups_id=groups_id)

        if range_param:
            queryset = queryset.filter(range=range_param)

        if name:
            org_id_list = OrgnizationEntity.objects.filter(name__icontains=name).values_list('id', flat=True)
            device_id_list = Devices.objects.filter(device_hostname__icontains=name, is_domain=1).values_list('id',
                                                                                                              flat=True)
            groups_id_list = Groups.objects.filter(name__icontains=name).values_list('id', flat=True)
            account_id_list = AccountEntity.objects.filter(cn_name__icontains=name).values_list('id', flat=True)
            queryset = queryset.filter(Q(device_id__in=device_id_list) |
                                       Q(org_id__in=org_id_list) |
                                       Q(groups_id__in=groups_id_list) |
                                       Q(account_id__in=account_id_list))

        return queryset

    def create(self, request, *args, **kwargs):
        data = request.data.copy()
        group_data = {
            "groups_id": data["groups_id"],
            "range": data["range"]
        }
        if data["range"] == 0:
            org_id_list = str_to_list(data["org_id"])
            for org_id in org_id_list:
                logger.info("org_id %s" % org_id)
                logger.info("groups_id %s" % data["groups_id"])
                history = GroupsDetail.objects.filter(org_id=org_id, groups_id=data["groups_id"])
                logger.info("history %s" % history)
                if history:
                    continue
                group_data["org_id"] = org_id
                logger.info("group_data %s" % group_data)
                groups_detail = GroupsDetail.objects.create(**group_data)
                logger.info("groups_detail %s" % groups_detail)
        elif data['range'] == 1:
            device_id_list = str_to_list(data["device_id"])
            for device_id in device_id_list:
                history = GroupsDetail.objects.filter(device_id=device_id, groups_id=data["groups_id"])
                if history:
                    continue
                group_data["device_id"] = device_id
                groups_detail = GroupsDetail.objects.create(**group_data)
        elif data['range'] == 2:
            add_groups_list = str_to_list(data['add_groups_id'])
            for add_groups_id in add_groups_list:
                if self.queryset.filter(groups_id=add_groups_id, add_groups_id=data['groups_id']).exists():
                    return Response("不可循环嵌套", status=status.HTTP_400_BAD_REQUEST)
            for add_groups_id in add_groups_list:
                group_data['add_groups_id'] = add_groups_id
                if self.queryset.filter(groups_id=data['groups_id'], add_groups_id=add_groups_id, range=2).exists():
                    continue
                else:
                    groups_detail = GroupsDetail.objects.create(**group_data)
        elif data['range'] == 3:
            account_id_list = str_to_list(data['account_id'])
            for account_id in account_id_list:
                group_data['account_id'] = account_id
                if self.queryset.filter(groups_id=data['groups_id'], account_id=account_id, range=3).exists():
                    continue
                else:
                    groups_detail = GroupsDetail.objects.create(**group_data)
        else:
            return Response("range 参数错误", status=status.HTTP_400_BAD_REQUEST)
        return Response("添加数据成功", status=status.HTTP_200_OK)

    @action(methods=['delete'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_delete(self, request, *args, **kwargs):
        """
        批量删除
        :param request:body {
                        is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5
                        }
        :param args:
        :param kwargs:
        :return:
        """
        is_all = request.query_params.get('is_all')
        ids_list = request.query_params.get('ids_list')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)

        queryset = self.queryset

        if is_all == 0:
            try:
                ids_list = [int(i) for i in ids_list.split(',')]
            except Exception as exp:
                return Response(f'ids_list参数不符合规范{exp}', status=status.HTTP_400_BAD_REQUEST)
        else:
            queryset = self.filter_queryset(queryset)
            ids_list = queryset.values_list('id', flat=True)

        # 查询过滤结果集
        queryset = queryset.filter(id__in=ids_list)
        queryset.delete()
        return Response('数据移出成功', status=status.HTTP_200_OK)

    @action(methods=['post'], detail=False, authentication_classes=[TokenAuthentication])
    def groups_all(self, request, *args, **kwargs):
        """
        嵌套分组，添加全选信息
        :param request:
        query_params name 分组名称
        request.data  {
                    is_all: 0|不全选；1|全选
                    groups_id: 分组组别id
                    can_groups_set: 可选择分组集合
                    cannot_groups_set:  不能选择分组集合

        }
        :param args:
        :param kwargs:
        :return:
        """
        name = self.request.query_params.get('name', None)
        is_all = self.request.data.get('is_all', None)
        groups_id = self.request.data.get('groups_id', None)
        can_groups_set = self.request.data.get('can_groups_set', None)
        cannot_groups_set = self.request.data.get('cannot_groups_set', None)
        if is_all == 0:
            for i in can_groups_set:
                self.queryset.create(**{'groups_id': groups_id, 'range': 2, 'add_groups_id': i})
        else:
            if name:
                groups_queryset = Groups.objects.exclude(id=groups_id).filter(name__icontains=name)
            else:
                groups_queryset = Groups.objects.all().exclude(id=groups_id)
            cannot_groups_set = cannot_groups_set \
                                + list(self.queryset.filter(range=2).values_list('groups_id', flat=True)) \
                                + list(
                self.queryset.filter(groups_id=groups_id).values_list('add_groups_id', flat=True))
            groups_queryset = groups_queryset.exclude(id__in=cannot_groups_set).values_list('id', flat=True)
            for i in groups_queryset:
                self.queryset.create(**{'groups_id': groups_id, 'range': 2, 'add_groups_id': i})

        return Response('添加分组成功', status=status.HTTP_201_CREATED)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def organ_relationship(self, request, *args, **kwargs):
        """
        获取部门关系接口，防止部门循环嵌套，
        :param request: groups_id 组别id  groups_name 组别名称
        :param args:
        :param kwargs:
        :return:
        """
        groups_id = self.request.query_params.get('groups_id', None)
        groups_name = self.request.query_params.get('groups_name', None)
        if groups_id is None:
            return Response('请求参数错误', status=status.HTTP_400_BAD_REQUEST)

        queryset = self.queryset.filter(range='2').exclude(add_groups_id__isnull=True)

        # # 往下遍历关系部门
        # groups_down_items = queryset.filter(groups_id=groups_id)
        # groups_down_set = list(groups_down_items.values_list('add_groups_id', flat=True))
        # while True:
        #     groups_down_items = queryset.filter(groups_id__in=groups_down_items.values_list('add_groups_id', flat=True))
        #     if not groups_down_items.exists():
        #         break
        #     else:
        #         groups_down_set = groups_down_set + list(groups_down_items.values_list('add_groups_id', flat=True))
        #
        # # 往上遍历关系部门
        # groups_up_items = queryset.filter(add_groups_id=groups_id)
        # groups_up_set = list(groups_up_items.values_list('groups_id', flat=True))
        # while True:
        #     groups_up_items = queryset.filter(add_groups_id__in=groups_up_items.values_list('groups_id', flat=True))
        #     if not groups_up_items.exists():
        #         break
        #     else:
        #         groups_up_set = groups_up_set + list(groups_up_items.values_list('groups_id', flat=True))

        groups_set = list(
            set(list(queryset.values_list('groups_id', flat=True))
                + list(queryset.filter(groups_id=groups_id).values_list('add_groups_id', flat=True))))
        if queryset.filter(groups_id=int(groups_id)).exists():
            pass
        else:
            groups_set.append(int(groups_id))  # 自己不能添加自己

        # 根据groups_name，过滤集合信息
        if groups_name is not None:
            groups_name_list_1 = list(Groups.objects.filter(name__icontains=groups_name).values_list('id', flat=True))
            groups_name_list_2 = groups_set
            groups_set = list(set(groups_name_list_1) & set(groups_name_list_2))

        res_data = {'groups_id': int(groups_id), 'groups_set': sorted(groups_set)}

        return Response(res_data, status=status.HTTP_200_OK)
