# -*- coding: utf-8 -*-
import datetime
import json
import logging
import io
import os.path

import pytz
import threading

import pandas as pd
from django.conf import settings
from django.core.cache import caches
from django.db.models import Count
from django.http import HttpResponse, FileResponse
from django.utils import timezone
from django_redis import get_redis_connection
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.viewsets import GenericViewSet

from accounts.models import AccountEntity
from devices.models import Devices, DevicesHistory, DevicesHardware
from devices.serializers import DeviceSerializer, DeviceHistorySerializer
from saltapi.models import USBDevicePolicy, FirewalldDevicePolicy, SSHDevicePolicy, WallpaperDevicePolicy, \
    ScreensaversDevicePolicy, DevicePolicyInfo, WaterMarkDevicePolicy, ActivationDevicePolicy, FileRedirectionPolicy, \
    DesktopThemePolicy, SystemUpdatesPolicy, SoftwareRunsPolicy
from saltapi.utils import SaltAPI
from software.models import SoftwareInstalledModel, SoftwarePolicy, SoftwareInfoModel
from tasks.models import Tasks
from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.permission import IsAdminUser
from utils.response import FitJSONRenderer
from utils.throttling import UserRateThrottle
from patch_management.models import PatchInfo, PatchPolicy, PatchDeviceLog
from organizations.models import OrgnizationEntity
from system.models import DeviceStats
from reports.models import HistoryAccountOnline, HistoryDeviceOnline

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')


class DevicesViewSet(GenericViewSet):
    queryset = Devices.objects.all().exclude(is_domain=0)
    serializer_class = DeviceSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('id',)
    

    def get_time_length(self, start_time, end_time, init_data=0):
        res = {}
        # start_time = start_time.replace(hour=0).replace(minute=0).replace(second=0)
        # end_time = end_time.replace(hour=0).replace(minute=0).replace(second=0)
        time_delta = end_time - start_time
        date_length = time_delta.days + 1
        for offset in range(date_length):
            offset = offset * 24
            dt = start_time + datetime.timedelta(hours=offset)
            if isinstance(init_data, dict):
                res[dt] = init_data.copy()
            else:
                res[dt] = init_data
        return res

    @staticmethod
    def use_info():
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        sql = 'SELECT `id_serial`,`action`,`time` FROM `usb_event` order by `time` desc limit 10;'
        devices = Devices.objects.exclude(is_domain=0)
        usb_info = []
        #  耗时
        for device in devices:
            try:
                usb_info_res = salt_api.sqlite_fetch(device.device_hostname,
                                                     ['/usr/share/nfs-domain-client/db/data.db', '{0}'.format(sql)])
                # logging.info(f"***********{usb_info_res}*******************")
                if isinstance(usb_info_res[device.device_hostname], bool):
                    pass
                else:
                    if len(usb_info_res[device.device_hostname]) != 0:
                        try:
                            usb_info = usb_info + usb_info_res[device.device_hostname]
                        except Exception as exp:
                            logging.info(f"**{usb_info_res[device.device_hostname]}*********{exp}*********")
            except Exception as exp:
                logging.info(f"**{device.device_hostname}*********{exp}*********")
                continue
        try:
            df = pd.DataFrame(usb_info, columns=['id_serial', 'action', 'time'])
            try:
                df = df.sort_values(by='time', ascending=False)
                df['time'] = df['time'].apply(
                    lambda x: datetime.datetime.strptime(x, "%Y-%m-%dT%H:%M:%S.%f").strftime("%Y-%m-%d %H:%M:%S"))
            except Exception as exp:
                pass
            df['action'] = df['action'].apply(lambda x: '插入' if x == 'add' else '拔出')
            df_json = json.dumps(df.to_dict('records'), ensure_ascii=False)
        except Exception as exp:
            return 400, exp
        else:
            redis_conn.set('usb_info', df_json, 2 * 60 * 60)
            return 200, df_json

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def index_count(self, request, *args, **kwargs):
        """获取设首页计数"""
        cache = caches['redis']
        devices_id_list = []
        for item in self.queryset.exclude(is_domain=0):
            if redis_conn.hget('actives', item.device_hostname) == b'1':
                devices_id_list.append(item.id)
        devices_total = self.queryset.exclude(is_domain=0).count()
        online_devices_total = len(devices_id_list)
        offline_device_total = devices_total - len(devices_id_list)
        disable_device_total = self.queryset.filter(is_up=0).count()
        account_total = AccountEntity.objects.all().count()
        online_account_total = len(devices_id_list)
        offline_account_total = account_total - disable_device_total - online_account_total
        disable_account_total = AccountEntity.objects.filter(user_status=1).count()
        able_account_total = AccountEntity.objects.filter(user_status=0).count()
        task_total = Tasks.objects.filter(is_deleted=0).count()
        exec_task_total = Tasks.objects.filter(is_deleted=0, status=2).count()
        finish_task_total = Tasks.objects.filter(is_deleted=0, status=4).count()
        failed_task_total = Tasks.objects.filter(is_deleted=0, status=5).count()
        now_time = datetime.datetime.now()
        zero_today = now_time - datetime.timedelta(hours=now_time.hour, minutes=now_time.minute,
                                                   seconds=now_time.second, microseconds=now_time.microsecond)
        last_today = zero_today + datetime.timedelta(hours=23, minutes=59, seconds=59)
        zero_yerterday = zero_today - datetime.timedelta(days=1)
        last_yerterday = last_today - datetime.timedelta(days=1)
        last_week_start = now_time - datetime.timedelta(days=now_time.weekday() + 7)
        last_week_end = now_time - datetime.timedelta(days=now_time.weekday() + 1)
        last_weekly_task = Tasks.objects.filter(is_deleted=0).filter(last_update__gte=last_week_start,
                                                                     last_update__lte=last_week_end).count()
        last_day_task = Tasks.objects.filter(is_deleted=0).filter(last_update__gte=zero_yerterday,
                                                                  last_update__lte=last_yerterday).count()
        today_task = Tasks.objects.filter(is_deleted=0).filter(last_update__gte=zero_today,
                                                               last_update__lte=last_today).count()
        user_account = request.user
        if user_account.is_superuser != 1:
            task_total = Tasks.objects.filter(is_deleted=0, create_user__username=user_account.username).count()
            exec_task_total = Tasks.objects.filter(is_deleted=0, status=2,
                                                   create_user__username=user_account.username).count()
            finish_task_total = Tasks.objects.filter(is_deleted=0, status=4,
                                                     create_user__username=user_account.username).count()
            failed_task_total = Tasks.objects.filter(is_deleted=0, status=5,
                                                     create_user__username=user_account.username).count()
            last_weekly_task = Tasks.objects.filter(is_deleted=0).filter(last_update__gte=last_week_start,
                                                                         create_user__username=user_account.username,
                                                                         last_update__lte=last_week_end).count()
            last_day_task = Tasks.objects.filter(is_deleted=0).filter(last_update__gte=zero_yerterday,
                                                                      create_user__username=user_account.username,
                                                                      last_update__lte=last_yerterday).count()
            today_task = Tasks.objects.filter(is_deleted=0).filter(last_update__gte=zero_today,
                                                                   create_user__username=user_account.username,
                                                                   last_update__lte=last_today).count()
        if last_weekly_task == 0:
            if task_total == 0:
                weekly_task_comparison = '{:.2f}%'.format(0)
            else:
                weekly_task_comparison = '{:.2f}%'.format(100)
        else:
            weekly_task_comparison = '{:.2f}%'.format(100 * ((task_total - last_weekly_task) / last_weekly_task))
        if last_day_task == 0:
            if today_task == 0:
                day_task_comparison = '{:.2f}%'.format(0)
            else:
                day_task_comparison = '{:.2f}%'.format(100)
        else:
            day_task_comparison = '{:.2f}%'.format(100 * (today_task - last_day_task) / last_day_task)
        if cache.get('devices_online') is not None:
            df = cache.get('devices_online')
            df = df[['time', 'total', 'online_percent']]
            now_time = datetime.datetime.now()
            now_time_1 = now_time - datetime.timedelta(days=1)
            df1 = df[df['time'] >= now_time_1].copy()
            df1['time'] = df1['time'].dt.strftime("%Y-%m-%d %H:%M:%S")
            # df1['online_percent'] = df1['total'].apply(
            #     lambda x: '{:.2f}%'.format(100 * (x / devices_total)) if devices_total != 0 else "0%")
            df2 = df[df['time'] >= now_time_1].copy()
            df2['time'] = df2['time'].dt.strftime("%Y-%m-%d %H:%M:%S")
            # df2['online_percent'] = df2['total'].apply(
            #     lambda x: '{:.2f}%'.format(100 * (x / account_total)) if account_total != 0 else "0%")

            device_24hours = df1.to_dict('records')
            account_24hours = df2.to_dict('records')
        else:
            device_24hours = []
            account_24hours = []

        res = {
            'device_total': devices_total,
            'online_devices_total': online_devices_total,
            'offline_device_total': offline_device_total,
            'disable_device_total': disable_device_total,
            'device_24hours': device_24hours,
            'account_total': account_total,
            'online_account_total': online_account_total,
            'offline_account_total': offline_account_total,
            'disable_account_total': disable_account_total,
            'able_account_total': able_account_total,
            'account_24hours': account_24hours,
            'task_total': task_total,
            'exec_task_total': exec_task_total,
            'finish_task_total': finish_task_total,
            'failed_task_total': failed_task_total,
            'weekly_task_comparison': weekly_task_comparison,
            'day_task_comparison': day_task_comparison
        }
        return Response(res, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def policy_info(self, request, *args, **kwargs):
        """策略生效率"""
        soft_policy_items = SoftwareInstalledModel.objects.all()
        soft_efficiency = soft_policy_items.filter(
            installed_status=1).count() / soft_policy_items.count() if soft_policy_items.count() != 0 else 0
        usb_efficiency = (USBDevicePolicy.objects.filter(
            status=1).count() / USBDevicePolicy.objects.all().count()) if USBDevicePolicy.objects.all().count() != 0 else 0
        firewalld_efficiency = (FirewalldDevicePolicy.objects.filter(
            status=1).count() / FirewalldDevicePolicy.objects.all().count()) if FirewalldDevicePolicy.objects.all().count() != 0 else 0
        ssh_efficiency = (SSHDevicePolicy.objects.filter(
            status=1).count() / SSHDevicePolicy.objects.all().count()) if SSHDevicePolicy.objects.all().count() != 0 else 0
        wall_items = WallpaperDevicePolicy.objects.all()
        screen_items = ScreensaversDevicePolicy.objects.all()
        desktop_efficiency = ((wall_items.filter(status=1).count() + screen_items.filter(status=1).count()) / (
                wall_items.count() + screen_items.count())) if (wall_items.count() + screen_items.count()) != 0 else 0

        setting_efficiency = 1 if Devices.objects.count() != 0 else 0

        res = {
            'soft_efficiency': '{:.2f}'.format(100 * soft_efficiency),
            'usb_efficiency': '{:.2f}'.format(100 * usb_efficiency),
            'firewalld_efficiency': '{:.2f}'.format(100 * firewalld_efficiency),
            'ssh_efficiency': '{:.2f}'.format(100 * ssh_efficiency),
            'desktop_efficiency': '{:.2f}'.format(100 * desktop_efficiency),
            'setting_efficiency': '{:.2f}'.format(100 * setting_efficiency),
        }
        return Response(res, status=status.HTTP_200_OK)
        # device_list = Devices.objects.filter(is_domain=1).values_list('id', flat=True)
        # if DevicePolicyInfo.objects.all().exists():
        #     device_policy_items = DevicePolicyInfo.objects.filter(device_id__in=device_list).count()
        # else:
        #     device_policy_items = 0
        # if Devices.objects.filter(is_domain=1).exists():
        #     device_items = Devices.objects.filter(is_domain=1).count()
        # else:
        #     device_items = 0
        #
        # device_efficiency = device_policy_items/device_items if device_items != 0 else 1
        # res = {
        #     'device_efficiency': '{:.0f}'.format(100 * device_efficiency)
        # }
        # return Response(res, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def usb_info(self, request, *args, **kwargs):
        """USB 识别信息"""
        # if redis_conn.get('usb_info'):
        #     # 启动后台线程调用信息
        #     res = redis_conn.get('usb_info')
        #     res = json.loads(res)
        #     thread1 = threading.Thread(target=self.use_info)
        #     thread1.start()
        #     return Response(res, status=status.HTTP_200_OK)
        # else:
        #     code, msg = self.use_info()
        #     if code != 200:
        #         return Response(f'数据处理错误，{msg}', status=status.HTTP_417_EXPECTATION_FAILED)
        #     else:
        #         return Response(json.loads(msg), status=status.HTTP_200_OK)
        return Response([], status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def task_info(self, request, *args, **kwargs):
        task_items = Tasks.objects.filter(is_deleted=0).all()
        res = {
            'task_total': task_items.count(),
            'patch': task_items.filter(task_class=1).count(),
            'script': task_items.filter(task_class=0).count(),
            'software': task_items.filter(task_class=3).count(),
            'file': task_items.filter(task_class=2).count(),
            'other': task_items.exclude(task_class__in=[0, 1, 2, 3]).count()
        }
        return Response(res, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def software_info(self, request, *args, **kwargs):
        software_name = SoftwareInstalledModel.objects.values('name').distinct().order_by('name').annotate(
            Count('name'))
        return Response(software_name, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def devices_online(self, request, *args, **kwargs):
        cache = caches['redis']
        #  devices_count = self.queryset.filter(is_domain=1).count()
        if cache.get('devices_online') is not None:
            df = cache.get('devices_online')
            df = df[['time', 'total', 'online_percent']]
            now_time = datetime.datetime.now()
            now_time_1 = now_time - datetime.timedelta(days=1)
            now_time_7 = now_time - datetime.timedelta(days=7)
            df1 = df[df['time'] >= now_time_1].copy()
            df1['time'] = df1['time'].dt.strftime("%Y-%m-%d %H:%M:%S")
            # df1['online_percent'] = df1['total'].apply(
            #     lambda x: '{:.2f}%'.format(100 * (x / devices_count)) if devices_count != 0 else "0%")
            df7 = df[df['time'] >= now_time_7].copy()
            df7['time'] = df7['time'].dt.strftime("%Y-%m-%d %H:%M:%S")
            # df7['online_percent'] = df7['total'].apply(
            #     lambda x: '{:.2f}%'.format(100 * (x / devices_count)) if devices_count != 0 else "0%")
            return Response({'24hours': df1.to_dict('records'), '7days': df7.to_dict('records')},
                            status=status.HTTP_200_OK)
        else:
            return Response('数据正在采集中', status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def sub_domain(self, request, *args, **kwargs):
        start = request.query_params.get('start', None)
        end = request.query_params.get('end', None)
        start_time = datetime.datetime.strptime(start, '%Y-%m-%d %H:%M:%S')
        end_time = datetime.datetime.strptime(end, '%Y-%m-%d %H:%M:%S')
        # group_by = request.query_params.get('group_by', None)
        queryset = DevicesHistory.objects.filter(timestamp__gte=start_time, timestamp__lte=end_time)
        user_account = request.user
        if user_account.is_superuser != 1:
            queryset = queryset.filter(accounts_name=user_account.username)

        serializer = DeviceHistorySerializer(queryset, many=True)
        ops = self.get_time_length(
            start_time, end_time
        )
        ops_add = self.get_time_length(
            start_time, end_time
        )
        ops_exit = self.get_time_length(
            start_time, end_time
        )
        # logger.info(serializer.data)
        res_data = []
        for data in serializer.data:
            dt = datetime.datetime.strptime(data.get('timestamp'), '%Y-%m-%d %H:%M:%S').replace(hour=0, minute=0,
                                                                                                second=0)
            # logger.info(dt)
            if data.get('op_type') == 0:
                ops_exit[dt] = ops_exit.get(dt) + 1
            elif data.get('op_type') == 1:
                ops_add[dt] = ops_add.get(dt) + 1
        for k, v in ops.items():
            res_data.append({'datetime': k.strftime('%Y-%m-%d %H:%M:%S'),
                             'add': ops_add[k],
                             'exit': ops_exit[k]})
        return Response(sorted(res_data, key=lambda x: x['datetime']), status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def sub_domain_forms(self, request, *args, **kwargs):
        # start = request.query_params.get('start', None)
        # end = request.query_params.get('end', None)
        # org_info = request.query_params.get('org_info', None)
        # start_time = datetime.datetime.strptime(start, '%Y-%m-%d %H:%M:%S')
        # end_time = datetime.datetime.strptime(end, '%Y-%m-%d %H:%M:%S')
        # group_by = request.query_params.get('group_by', None)
        # queryset = DevicesHistory.objects.filter(op_type=1, timestamp__gte=start_time,
        #                                          timestamp__lte=end_time + datetime.timedelta(days=1)).order_by('-id')
        queryset = DevicesHistory.objects.filter(op_type=1).order_by('-id')
        user_account = request.user
        if user_account.is_superuser != 1:
            queryset = queryset.filter(accounts_name=user_account.username)
        # if org_info:
        #     queryset = queryset.filter(org_info__contains=org_info)

        # page = MyPagination()
        # page_chapter = page.paginate_queryset(queryset=queryset,
        #                                       request=request, view=self)
        serializer = DeviceHistorySerializer(queryset, many=True)

        return Response(serializer.data, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def sub_domain_forms_export(self, request, *args, **kwargs):
        queryset = DevicesHistory.objects.all()
        user_account = request.user
        if user_account.is_superuser != 1:
            queryset = queryset.filter(accounts_name=user_account.username)
        ids_list = request.query_params.get('ids_list', None)
        if ids_list:
            ids_list = [int(i) for i in ids_list.split(',')]
            queryset = queryset.filter(id__in=ids_list)

        head = [
            '用户账号', '姓名', '终端别名', 'IP', '终端所属部门', '加域日期', '加域时间'
        ]
        queryset = queryset.values(
            'accounts_cnname','accounts_name','device_cnname','device_ip','org_info', 'timestamp'
        )
        data_list = []
        for query in queryset:
            data_list.append(
                [query.get('accounts_name'),
                 query.get('accounts_cnname'),
                 query.get('device_cnname'),
                 query.get('device_ip'),
                 query.get('org_info'),
                 query.get('timestamp').date(),
                 query.get('timestamp').time().strftime('%H:%M:%S')
                 ]
            )

        df = pd.DataFrame(data_list, columns=head)
        output = io.BytesIO()
        df.to_excel(output, sheet_name='加域统计', encoding='utf-8', index=False)
        output.seek(0)

        data = output.getvalue()
        response = HttpResponse(data, content_type='application/ms-excel')
        response['Content-Type'] = 'application/ms-excel'
        response['Content-Disposition'] = 'attachment;filename=domain.xlsx'
        return response

    @action(methods=['get'], detail=False, authentication_classes=[], permission_classes=[], throttle_classes=[])
    def logo(self, request, *args, **kwarg):
        logo_path = settings.BASE_DIR + settings.STATIC_URL + 'logo.png'
        if not os.path.exists(logo_path):
            return Response(status=status.HTTP_404_NOT_FOUND)
        return FileResponse(open(logo_path, 'rb'), content_type='image/png')


class StatsDataScreenViewSet(GenericViewSet):
    """
    统计数据大屏的数据展示,共4个大类,11个小类:
        1,终端和人员活跃度
        -- 1.1,终端活跃度
        -- 1.2,人员活跃度
        2,统计数据
        -- 2.1,纳管终端数
        -- 2.2,终端上线率
        -- 2.3,人员上线率
        3,国产化推广进度
        -- 3.1,终端纳管进度
        -- 3.2,终端国产化进度
        4,终端健康情况
        -- 4.1,软件状态
        -- 4.2,策略情况
        -- 4.3补丁情况
        -- 4.4,任务执行
    """
    serializer_class = DeviceSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    
    # 当前时间
    now_time = datetime.datetime.now()
    # 今天 0 点
    zero_today = now_time - datetime.timedelta(hours=now_time.hour, minutes=now_time.minute,
                                                    seconds=now_time.second, microseconds=now_time.microsecond)
    # 今天 23:59:59
    last_today = zero_today + datetime.timedelta(hours=23, minutes=59, seconds=59)
    # 昨天 0 点
    zero_yerterday = zero_today - datetime.timedelta(days=1)
    # 昨天 23:59:59
    last_yerterday = last_today - datetime.timedelta(days=1)
    # 上周的第1天的0点
    last_week_start = zero_today - datetime.timedelta(days=zero_today.weekday() + 7)
    # 上周的第7天的23:59:59
    last_week_end = last_today - datetime.timedelta(days=last_today.weekday() + 1)
    # 本周周一的 0 点
    zero_cur_week = now_time - datetime.timedelta(days=now_time.weekday())
    zero_cur_week = zero_cur_week.replace(hour=0, minute=0, second=0, microsecond=0)
    # 本周周日的 23:59:59
    last_cur_week = zero_cur_week + datetime.timedelta(days=6)
    last_cur_week = last_cur_week.replace(hour=23, minute=59, second=59)
    
    # 终端数据
    device_queryset = Devices.objects.all().filter(is_delete=0)
    # 人员数据
    account_queryset = AccountEntity.objects.all()
    # 补丁数据
    patch_queryset = PatchInfo.objects.all()
    # 任务数据
    task_queryset = Tasks.objects.all().filter(is_deleted=0)
    # 机构数据
    org_queryset = OrgnizationEntity.objects.all()

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def device_user(self, request, *args, **kwargs):
        """
        1, 终端和人员活跃度数据统计:
            1,终端活跃度
            2,人员活跃度
        """        
        ######################################## 终端统计数据 ########################################
        # 缓存中的在线终端id集合
        devices_id_list = []
        device_org_map = {}
        for item in self.device_queryset.exclude(is_domain=0):
            if redis_conn.hget('actives', item.device_hostname) == b'1':
                devices_id_list.append(item.id)
                # 按着部门来区分集合
                if item.org_info:
                    if item.org_info.id not in device_org_map:
                        device_org_map[item.org_info.id] = []
                    device_org_map[item.org_info.id].append(item)
        # 终端总数
        devices_total = self.device_queryset.exclude(is_domain=0).count()
        # 在线终端数
        online_device_total = len(devices_id_list)
        # 离线终端数
        offline_device_total = devices_total - len(devices_id_list)
        # 禁用终端数
        disable_device_total = self.device_queryset.filter(is_up=0).count()
        # 终端上线率
        online_device_comparison = 100 * (online_device_total / devices_total) if devices_total > 0 else 0
        # 部门终端上线率的最高和最低
        if not device_org_map or online_device_total == 0:
            online_device_comparison_org_top = {}
            online_device_comparison_org_bottom = {}
        else:
            online_device_comparison_org_id_min = 0
            online_device_comparison_org_id_max = 0
            online_device_comparison_total_min = 0
            online_device_comparison_total_max = 0
            # 遍历出最高和最低的2个部门
            for org_id, items in device_org_map.items():
                if online_device_comparison_total_min == 0 or online_device_comparison_total_min > len(items):
                    online_device_comparison_total_min = len(items)
                    online_device_comparison_org_id_min = org_id
                if online_device_comparison_total_max == 0 or online_device_comparison_total_max < len(items):
                    online_device_comparison_total_max = len(items)
                    online_device_comparison_org_id_max = org_id
            # 获取部门对象
            device_org_min_name = ''
            device_org_max_name = ''
            if online_device_comparison_org_id_min >0:
                device_org_min = OrgnizationEntity.objects.filter(id=online_device_comparison_org_id_min).first()
                if device_org_min:
                    device_org_min_name = device_org_min.name
            if online_device_comparison_org_id_max >0:
                device_org_max = OrgnizationEntity.objects.filter(id=online_device_comparison_org_id_max).first()
                if device_org_max:
                    device_org_max_name = device_org_max.name
            # 结果封装
            # 疑问1: 部门上线率是除以[上线终端总数]还是[全部终端总数]?
            online_device_comparison_org_top = {
                'org_name': device_org_min_name,
                'comparison': round(100 * (online_device_comparison_total_max / online_device_total),2)
            }
            online_device_comparison_org_bottom = {
                'org_name': device_org_max_name,
                'comparison': round(100 * (online_device_comparison_total_min / online_device_total),2)
            }
            
        # 上周的终端上线数
        last_weekly_online_device_total = HistoryDeviceOnline.objects.filter(up_time__gte=self.last_week_start, 
                                    up_time__lte=self.last_week_end).values('device_uuid').distinct().count()
        # 本周的终端上线数
        cur_weekly_online_device_total = HistoryDeviceOnline.objects.filter(up_time__gte=self.zero_cur_week, 
                                            up_time__lte=self.last_cur_week).values('device_uuid').distinct().count()
        # 周环比(公式==(本周-上周)/上周)
        if last_weekly_online_device_total == 0:
            if cur_weekly_online_device_total == 0:
                weekly_online_device_comparison = 0
            else:
                weekly_online_device_comparison = 100 * 100
        else:
            weekly_online_device_comparison = round(100 * ((cur_weekly_online_device_total - last_weekly_online_device_total) / last_weekly_online_device_total), 2)
        # 本周终端上线率(柱状图)
        cur_week_online_device_comparison_chart = []
        # 除了今天之外的本周内其他几天的终端上线数的计算和处理
        for i in range(0, 7):
            cur_week_online_device_comparison_chart_item = {}
            # 本周每一天的 0 点
            cur_week_day_zero_1 = self.zero_cur_week + datetime.timedelta(days=i)
            cur_week_day_zero_1 = cur_week_day_zero_1.replace(hour=0, minute=0, second=0, microsecond=0)
            # 缓存key,以[yyyy-MM-dd]格式存放
            cur_week_day_online_device_key = cur_week_day_zero_1.strftime('%Y-%m-%d')
            cur_week_online_device_comparison_chart_item['date'] = cur_week_day_online_device_key
            # 1,今天的不放入缓存,因为会变化
            if cur_week_day_zero_1 == self.zero_today:
                cur_week_online_device_comparison_chart_item['total'] = devices_total
                cur_week_online_device_comparison_chart_item['online_total'] = online_device_total
                cur_week_online_device_comparison_chart_item['offline_total'] = offline_device_total
                cur_week_online_device_comparison_chart_item['comparison'] = online_device_comparison
                # 集合
                cur_week_online_device_comparison_chart.append(cur_week_online_device_comparison_chart_item)
                continue
            # 2,今天后面的几天是没有数据的,不用算,直接赋值为'0',也不放入缓存中
            if cur_week_day_zero_1 > self.zero_today:
                cur_week_online_device_comparison_chart_item['total'] = 0
                cur_week_online_device_comparison_chart_item['online_total'] = 0
                cur_week_online_device_comparison_chart_item['offline_total'] = 0
                cur_week_online_device_comparison_chart_item['comparison'] = 0
                # 集合
                cur_week_online_device_comparison_chart.append(cur_week_online_device_comparison_chart_item)
                continue
            
            # 3,排除了今天和今天之后的,剩下的都是今天之前的几天
            if redis_conn.hget('index_stats_device_online_cur_week', cur_week_day_online_device_key) is None:
                # 那一天的 23:59:59
                cur_week_day_last = self.zero_cur_week + datetime.timedelta(days=i)
                cur_week_day_last = cur_week_day_last.replace(hour=23, minute=59, second=59)
                # 那一天的上线终端数:取最高上线率,即只要有过上线记录的终端都算在内
                cur_week_day_online_device_total =  HistoryDeviceOnline.objects.filter(up_time__gte=cur_week_day_zero_1, 
                                    up_time__lte=cur_week_day_last).values('device_uuid').distinct().count()
                # 那一天的终端总数(截止到那一天23:59:59的终端总数)
                cur_week_day_device_total = self.device_queryset.exclude(is_domain=0).filter(add_domain_time__lte=cur_week_day_last).count()
                # 那一天的离线终端数
                cur_week_day_offline_device_total = cur_week_day_device_total - cur_week_day_online_device_total
                # 那一天的上线率
                if cur_week_day_device_total == 0:
                    cur_week_day_online_device_comparison = 0
                else:
                    cur_week_day_online_device_comparison = round(100 * (cur_week_day_online_device_total / cur_week_day_device_total), 2)
                # 放入缓存中
                cur_week_day_online_device_dict = {
                    'total': cur_week_day_device_total,
                    'online_total': cur_week_day_online_device_total,
                    'offline_total': cur_week_day_offline_device_total,
                    'comparison': cur_week_day_online_device_comparison
                }
                cur_week_day_online_device_json = json.dumps(cur_week_day_online_device_dict)
                redis_conn.hset('index_stats_device_online_cur_week', cur_week_day_online_device_key, cur_week_day_online_device_json)
                cur_week_online_device_comparison_chart_item['total'] = cur_week_day_device_total
                cur_week_online_device_comparison_chart_item['online_total'] = cur_week_day_online_device_total
                cur_week_online_device_comparison_chart_item['offline_total'] = cur_week_day_offline_device_total
                cur_week_online_device_comparison_chart_item['comparison'] = cur_week_day_online_device_comparison
            else:
                # 直接从缓存中获取即可
                index_stats_device_online_cur_week_caches = redis_conn.hget('index_stats_device_online_cur_week', cur_week_day_online_device_key)
                cur_week_day_online_device_cache_json = json.loads(index_stats_device_online_cur_week_caches)
                cur_week_online_device_comparison_chart_item['total'] = cur_week_day_online_device_cache_json.get('total')
                cur_week_online_device_comparison_chart_item['online_total'] = cur_week_day_online_device_cache_json.get('online_total')
                cur_week_online_device_comparison_chart_item['offline_total'] = cur_week_day_online_device_cache_json.get('offline_total')
                cur_week_online_device_comparison_chart_item['comparison'] = cur_week_day_online_device_cache_json.get('comparison')
            # 集合
            cur_week_online_device_comparison_chart.append(cur_week_online_device_comparison_chart_item)

        # 本周终端平均上线时长数据(柱状图)
        cur_week_online_device_avg_duration_chart = []
        for i in range(0, 7):
            # 每天的平均上线时长
            cur_week_online_device_avg_duration_item = {}
            # 每一天的 0 点
            cur_week_day_zero_2 = self.zero_cur_week + datetime.timedelta(days=i)
            cur_week_day_zero_2 = cur_week_day_zero_2.replace(hour=0, minute=0, second=0, microsecond=0)
            # 每一天的 23:59:59
            cur_week_day_last = self.zero_cur_week + datetime.timedelta(days=i)
            cur_week_day_last = cur_week_day_last.replace(hour=23, minute=59, second=59)
            # 缓存key,以[yyyy-MM-dd]格式存放
            cur_week_day_online_device_duration_key = cur_week_day_zero_2.strftime('%Y-%m-%d')
            cur_week_online_device_avg_duration_item['date'] = cur_week_day_online_device_duration_key
            # 1,今天后面的几天是没有数据的,不用算,直接赋值为'0',也不放入缓存中
            if cur_week_day_zero_2 > self.zero_today:
                cur_week_online_device_avg_duration_item['duration'] = 0
                # 集合
                cur_week_online_device_avg_duration_chart.append(cur_week_online_device_avg_duration_item)
                continue

            # 2,其他就是今天和今天之前的几天
            if redis_conn.hget('index_stats_device_online_duration_cur_week', cur_week_day_online_device_duration_key) is None:
                # 每一天的所有上线记录
                history_device_online = HistoryDeviceOnline.objects.filter(up_time__gte=cur_week_day_zero_2, up_time__lte=cur_week_day_last).all()
                # 每一天的上线时长总和
                cur_week_day_online_device_duration = 0
                if history_device_online:
                    for item in history_device_online:
                        # 如果下线时间是Null,说明没有下线,分2种情况：
                        # 1, 如果是今天,用当前时间来算;
                        # 2, 如果不是今天,用当天的[23:59:59]来.
                        # 都依据'四舍五入'原则换算成分钟
                        if item.down_time is None:
                            if cur_week_day_zero_2 == self.zero_today:
                                cur_week_day_online_device_duration += round((self.now_time.timestamp() - item.up_time.timestamp())/60,0)
                            else:
                                cur_week_day_online_device_duration += round((cur_week_day_last.timestamp() - item.up_time.timestamp())/60,0)
                        else:
                            cur_week_day_online_device_duration += round((item.down_time.timestamp() - item.up_time.timestamp())/60,0)
                # 除了今天的都放入缓存中
                if cur_week_day_zero_2 == self.zero_today:
                    pass
                else:
                    cur_week_day_online_device_duration_dict = {
                        'duration': cur_week_day_online_device_duration
                    }
                    cur_week_day_online_device_duration_json = json.dumps(cur_week_day_online_device_duration_dict)
                    redis_conn.hset('index_stats_device_online_duration_cur_week', cur_week_day_online_device_duration_key, cur_week_day_online_device_duration_json)
            else:
                index_stats_device_online_duration_cur_week_caches = redis_conn.hget('index_stats_device_online_duration_cur_week', cur_week_day_online_device_duration_key)
                cur_week_day_online_device_duration_json = json.loads(index_stats_device_online_duration_cur_week_caches)
                cur_week_day_online_device_duration = cur_week_day_online_device_duration_json.get('duration')
            # 封装
            cur_week_online_device_avg_duration_item['duration'] = cur_week_day_online_device_duration
            cur_week_online_device_avg_duration_chart.append(cur_week_online_device_avg_duration_item)

        # 近一周(前6天+今天)的终端上线时段(以小时为单位)活跃图(波浪图)
        day7_online_device_hour_chart = []
        # 7天里的小时段和小时段对应的总终端数
        day7_online_device_hour_chart_map = {}
        for i in range(0, 7):
            # 每一天的 0 点(前面的天数用今天减去天数)
            day7_day_zero = self.zero_today - datetime.timedelta(days=i)
            day7_day_zero = day7_day_zero.replace(hour=0, minute=0, second=0, microsecond=0)
            # 缓存key,以[yyyy-MM-dd]格式存放
            day7_day_online_device_hour_key = day7_day_zero.strftime('%Y-%m-%d')
            # 缓存处理
            if redis_conn.hget('index_stats_day7_online_device_hour', day7_day_online_device_hour_key) is None:
                # 24小时的数据对象
                day7_day_online_device_24hour = []
                # 每一天的所有上线记录,以小时为时段查询24次
                for j in range(24):
                    # 每一天的每一个小时的整点
                    day7_day_hour_zero = day7_day_zero + datetime.timedelta(hours=j)
                    #
                    day7_day_online_device_hour = str(day7_day_hour_zero.hour)
                    # 每一天的下一个小时的整点
                    next_j = j + 1
                    day7_day_next_hour_zero = day7_day_zero + datetime.timedelta(hours=next_j)
                    # 获取小时时段内的记录总数[>=x:00:00 and <(x+1):00:00]
                    day7_day_online_device_hour_count = HistoryDeviceOnline.objects.filter(up_time__gte=day7_day_hour_zero, up_time__lt=day7_day_next_hour_zero).count()
                    
                    day7_day_online_device_hour_dict = {
                        'hour': day7_day_online_device_hour,
                        'total': day7_day_online_device_hour_count
                    }
                    day7_day_online_device_24hour.append(day7_day_online_device_hour_dict)
                    # hour:total封装进map中
                    if day7_day_online_device_hour not in day7_online_device_hour_chart_map:
                        day7_online_device_hour_chart_map[day7_day_online_device_hour] = 0
                    day7_online_device_hour_chart_map[day7_day_online_device_hour] += day7_day_online_device_hour_count
                # 除了今天的都放入缓存中
                if day7_day_zero == self.zero_today:
                    pass
                else:
                    day7_day_online_device_24hour_json = json.dumps(day7_day_online_device_24hour)
                    redis_conn.hset('index_stats_online_device_day7_hour', day7_day_online_device_hour_key, day7_day_online_device_24hour_json)
            else:
                # 从缓存获取
                index_stats_online_device_day7_hour_caches = redis_conn.hget('index_stats_online_device_day7_hour', day7_day_online_device_hour_key)
                day7_day_online_device_hour_json = json.loads(index_stats_online_device_day7_hour_caches)
                for item in day7_day_online_device_hour_json:
                    day7_day_online_device_hour = item['hour']
                    day7_day_online_device_hour_total = item['total']
                    # hour:total封装进map中
                    if day7_day_online_device_hour not in day7_online_device_hour_chart_map:
                        day7_online_device_hour_chart_map[day7_day_online_device_hour] = 0
                    day7_online_device_hour_chart_map[day7_day_online_device_hour] += day7_day_online_device_hour_total
        # 数据处理
        if day7_online_device_hour_chart_map:
            for item_hour, item_total in day7_online_device_hour_chart_map.items():
                day7_online_device_hour_chart.append({
                    'hour': item_hour,
                    'total': item_total
                })

        ######################################## 人员统计数据 ########################################
        # 人员总数
        account_total = self.account_queryset.count()
        # 在线人员数
        device_account_active_id_list = []
        for item in self.device_queryset.exclude(is_domain=0):
            device_account_active = redis_conn.hget('device_user_active', item.device_hostname)
            if device_account_active:
                device_account_active = device_account_active.decode('utf-8')
                device_account_active = json.loads(device_account_active)
            device_account_active = device_account_active if isinstance(device_account_active, list) else []
            # 终端在线账号ID列表
            for item2 in self.account_queryset:
                if item2.name in device_account_active:
                    device_account_active_id_list.append(item2.id)
        online_account_total = len(device_account_active_id_list)
        # 禁用人员数
        disable_account_total = self.account_queryset.filter(user_status=1).count()
        # 离线人员数
        offline_account_total = account_total - disable_account_total - online_account_total
        # 人员上线率
        online_account_comparison = round(100 * (online_account_total / account_total),2) if account_total > 0 else 0
        # 部门人员上线率的最高和最低
        account_org_map = {}
        account_datas = self.account_queryset.filter(id__in=device_account_active_id_list)
        # 按着部门来集合
        for item in account_datas:
            if item.org_info:
                if item.id not in account_org_map:
                    account_org_map[item.org_info.id] = []
                account_org_map[item.org_info.id].append(item)
        #
        if not account_org_map or online_account_total == 0:
            online_account_comparison_org_top = {}
            online_account_comparison_org_bottom = {}
        else:
            online_account_comparison_org_id_min = 0
            online_account_comparison_org_id_max = 0
            online_account_comparison_total_min = 0
            online_account_comparison_total_max = 0
            # 遍历出最高和最低的2个部门
            for org_id, items in account_org_map.items():
                if online_account_comparison_total_min == 0 or online_account_comparison_total_min > len(items):
                    online_account_comparison_total_min = len(items)
                    online_account_comparison_org_id_min = org_id
                if online_account_comparison_total_max == 0 or online_account_comparison_total_max < len(items):
                    online_account_comparison_total_max = len(items)
                    online_account_comparison_org_id_max = org_id
            # 获取部门对象
            account_org_min_name = ''
            account_org_max_name = ''
            if online_account_comparison_org_id_min >0:
                account_org_min = OrgnizationEntity.objects.filter(id=online_account_comparison_org_id_min).first()
                if account_org_min:
                    account_org_min_name = account_org_min.name
            if online_account_comparison_org_id_max >0:
                account_org_max = OrgnizationEntity.objects.filter(id=online_account_comparison_org_id_max).first()
                if account_org_max:
                    account_org_max_name = account_org_max.name
            # 结果封装
            # 疑问2: 人员上线率是除以[上线人员总数]还是[全部人员总数]?
            if online_account_total == 0:
                online_account_comparison_total_max_comparison = 0
                online_account_comparison_total_min_comparison = 0
            else:
                online_account_comparison_total_max_comparison = round(100 * (online_account_comparison_total_max / online_account_total),2)
                online_account_comparison_total_min_comparison = round(100 * (online_account_comparison_total_min / online_account_total),2)
            online_account_comparison_org_top = {
                'org_name': account_org_min_name,
                'comparison': online_account_comparison_total_max_comparison
            }
            online_account_comparison_org_bottom = {
                'org_name': account_org_max_name,
                'comparison': online_account_comparison_total_min_comparison
            }
        
        # 上周的在线人员数
        last_weekly_account_total = HistoryAccountOnline.objects.filter(up_time__gte=self.last_week_start, 
                                    up_time__lte=self.last_week_end).values('account_name').distinct().count()
        # 本周的在线人员数
        cur_weekly_account_total = HistoryAccountOnline.objects.filter(up_time__gte=self.zero_cur_week, 
                                    up_time__lte=self.last_cur_week).values('account_name').distinct().count()
        # 周环比(公式==(本周-上周)/上周)
        if last_weekly_account_total == 0:
            if cur_weekly_account_total == 0:
                weekly_online_account_comparison = 0
            else:
                weekly_online_account_comparison = 100 * 100
        else:
            weekly_online_account_comparison = round(100 * ( (cur_weekly_account_total - last_weekly_account_total) / last_weekly_account_total ) , 2)
        
        # 本周人员上线率(柱状图)
        cur_week_online_account_comparison_chart = []
        # 除了今天之外的本周内其他几天的人员上线数的计算和处理
        for i in range(0, 7):
            cur_week_online_account_comparison_chart_item = {}
            # 本周每一天的 0 点
            cur_week_day_zero_3 = self.zero_cur_week + datetime.timedelta(days=i)
            cur_week_day_zero_3 = cur_week_day_zero_3.replace(hour=0, minute=0, second=0, microsecond=0)
            # 缓存key,以[yyyy-MM-dd]格式存放
            cur_week_day_online_account_key = cur_week_day_zero_3.strftime('%Y-%m-%d')
            cur_week_online_account_comparison_chart_item['date'] = cur_week_day_online_account_key
            # 1,今天的不放入缓存,因为会变化
            if cur_week_day_zero_3 == self.zero_today:
                cur_week_online_account_comparison_chart_item['total'] = account_total
                cur_week_online_account_comparison_chart_item['online_total'] = online_account_total
                cur_week_online_account_comparison_chart_item['offline_total'] = offline_account_total
                cur_week_online_account_comparison_chart_item['comparison'] = online_account_comparison
                # 集合
                cur_week_online_account_comparison_chart.append(cur_week_online_account_comparison_chart_item)
                continue
            # 2,今天后面几天是没有数据的,不用算,直接赋值为'0',也不放入缓存中
            if cur_week_day_zero_3 > self.zero_today:
                cur_week_online_account_comparison_chart_item['total'] = 0
                cur_week_online_account_comparison_chart_item['online_total'] = 0
                cur_week_online_account_comparison_chart_item['offline_total'] = 0
                cur_week_online_account_comparison_chart_item['comparison'] = 0
                # 集合
                cur_week_online_account_comparison_chart.append(cur_week_online_account_comparison_chart_item)
                continue
            
            # 3,排除了今天和今天之后的,剩下的都是今天之前的
            if redis_conn.hget('index_stats_account_online_cur_week', cur_week_day_online_account_key) is None:
                # 那一天的 23:59:59
                cur_week_day_last = self.zero_cur_week + datetime.timedelta(days=i)
                cur_week_day_last = cur_week_day_last.replace(hour=23, minute=59, second=59)
                # 那一天的上线数:取最高上线数,即只要有过上线记录的人员都算在内
                cur_week_day_online_account_total =  HistoryAccountOnline.objects.filter(up_time__gte=cur_week_day_zero_3, 
                                    up_time__lte=cur_week_day_last).values('account_name').distinct().count()
                # 那一天的人员总数(截止到那一天的 23:59:59的人员创建总数)
                cur_week_day_account_total = self.account_queryset.filter(created_at__lte=cur_week_day_last).count()
                # 那一天的离线人数
                cur_week_day_offline_account_total = cur_week_day_account_total - cur_week_day_online_account_total
                # 那一天的上线率
                if cur_week_day_account_total == 0:
                    cur_week_day_online_account_comparison = 0
                else:
                    cur_week_day_online_account_comparison = round(100 * (cur_week_day_online_account_total / cur_week_day_account_total), 2)
                # 放入缓存中
                cur_week_day_online_account_dict = {
                    'total': cur_week_day_account_total,
                    'online_total': cur_week_day_online_account_total,
                    'offline_total': cur_week_day_offline_account_total,
                    'comparison': cur_week_day_online_account_comparison
                }
                cur_week_day_online_account_json = json.dumps(cur_week_day_online_account_dict)
                redis_conn.hset('index_stats_account_online_cur_week', cur_week_day_online_account_key, cur_week_day_online_account_json)
                cur_week_online_account_comparison_chart_item['total'] = cur_week_day_account_total
                cur_week_online_account_comparison_chart_item['online_total'] = cur_week_day_online_account_total
                cur_week_online_account_comparison_chart_item['offline_total'] = cur_week_day_offline_account_total
                cur_week_online_account_comparison_chart_item['comparison'] = cur_week_day_online_account_comparison
            else:
                # 直接从缓存中获取即可
                index_stats_account_online_cur_week_caches = redis_conn.hget('index_stats_account_online_cur_week', cur_week_day_online_account_key)
                cur_week_day_online_account_cache_json = json.loads(index_stats_account_online_cur_week_caches)
                cur_week_online_account_comparison_chart_item['total'] = cur_week_day_online_account_cache_json.get('total')
                cur_week_online_account_comparison_chart_item['online_total'] = cur_week_day_online_account_cache_json.get('online_total')
                cur_week_online_account_comparison_chart_item['offline_total'] = cur_week_day_online_account_cache_json.get('offline_total')
                cur_week_online_account_comparison_chart_item['comparison'] = cur_week_day_online_account_cache_json.get('comparison')
            # 集合
            cur_week_online_account_comparison_chart.append(cur_week_online_account_comparison_chart_item)

        # 本周人员平均上线时长(柱状图)
        cur_week_online_account_avg_duration_chart = []
        for i in range(0, 7):
            # 每天的平均上线时长
            cur_week_online_account_avg_duration_item = {}
            # 每一天的 0 点
            cur_week_day_zero_4 = self.zero_cur_week + datetime.timedelta(days=i)
            cur_week_day_zero_4 = cur_week_day_zero_4.replace(hour=0, minute=0, second=0, microsecond=0)
            # 每一天的 23:59:59
            cur_week_day_last = self.zero_cur_week + datetime.timedelta(days=i)
            cur_week_day_last = cur_week_day_last.replace(hour=23, minute=59, second=59)
            # 缓存key,以[yyyy-MM-dd]格式存放
            cur_week_day_online_account_duration_key = cur_week_day_zero_4.strftime('%Y-%m-%d')
            cur_week_online_account_avg_duration_item['date'] = cur_week_day_online_account_duration_key
            # 1,今天后面的几天是没有数据的,不用算,直接赋值为'0',也不放入缓存中
            if cur_week_day_zero_4 > self.zero_today:
                cur_week_online_account_avg_duration_item['duration'] = 0
                # 集合
                cur_week_online_account_avg_duration_chart.append(cur_week_online_account_avg_duration_item)
                continue

            # 2,其他就是今天和今天之前的几天
            if redis_conn.hget('index_stats_account_online_duration_cur_week', cur_week_day_online_account_duration_key) is None:
                # 每一天的所有上线记录
                history_account_online = HistoryAccountOnline.objects.filter(up_time__gte=cur_week_day_zero_4, up_time__lte=cur_week_day_last).all()
                # 每一天的上线时长总和
                cur_week_day_online_account_duration = 0
                if history_account_online:
                    for item in history_account_online:
                        # 如果下线时间是Null,说明没有下线,分2种情况：
                        # 1, 如果是今天,用当前时间来算;
                        # 2, 如果不是今天,用当天的[23:59:59]来.
                        # 都依据'四舍五入'原则换算成分钟
                        if item.down_time is None:
                            if cur_week_day_zero_4 == self.zero_today:
                                cur_week_day_online_account_duration += round((self.now_time.timestamp() - item.up_time.timestamp())/60,0)
                            else:
                                cur_week_day_online_account_duration += round((cur_week_day_last.timestamp() - item.up_time.timestamp())/60,0)
                        else:
                            cur_week_day_online_account_duration += round((item.down_time.timestamp() - item.up_time.timestamp())/60,0)
                # 除了今天的都放入缓存中
                if cur_week_day_zero_4 == self.zero_today:
                    pass
                else:
                    cur_week_day_online_account_duration_dict = {
                        'duration': cur_week_day_online_account_duration
                    }
                    cur_week_day_online_account_duration_json = json.dumps(cur_week_day_online_account_duration_dict)
                    redis_conn.hset('index_stats_account_online_duration_cur_week', cur_week_day_online_account_duration_key, cur_week_day_online_account_duration_json)
            else:
                index_stats_account_online_duration_cur_week_caches = redis_conn.hget('index_stats_account_online_duration_cur_week', cur_week_day_online_account_duration_key)
                cur_week_day_online_account_duration_json = json.loads(index_stats_account_online_duration_cur_week_caches)
                cur_week_day_online_account_duration = cur_week_day_online_account_duration_json.get('duration')
            # 封装
            cur_week_online_account_avg_duration_item['duration'] = cur_week_day_online_account_duration
            cur_week_online_account_avg_duration_chart.append(cur_week_online_account_avg_duration_item)

        # 近一周(前7天)的人员上线时间活跃图(波浪图)
        day7_online_account_hour_chart = []
        # 7天里的小时段和小时段对应的总终端数
        day7_online_account_hour_chart_map = {}
        for i in range(0, 7):
            # 每一天的 0 点(前面的天数用今天减去天数)
            day7_day_zero = self.zero_today - datetime.timedelta(days=i)
            day7_day_zero = day7_day_zero.replace(hour=0, minute=0, second=0, microsecond=0)
            # 缓存key,以[yyyy-MM-dd]格式存放
            day7_day_online_account_hour_key = day7_day_zero.strftime('%Y-%m-%d')
            # 缓存处理
            if redis_conn.hget('index_stats_day7_online_account_hour', day7_day_online_account_hour_key) is None:
                # 24小时的数据对象
                day7_day_online_account_24hour = []
                # 每一天的所有上线记录,以小时为时段查询24次
                for j in range(24):
                    # 每一天的每一个小时的整点
                    day7_day_hour_zero = day7_day_zero + datetime.timedelta(hours=j)
                    #
                    day7_day_online_account_hour = str(day7_day_hour_zero.hour)
                    # 每一天的下一个小时的整点
                    next_j = j + 1
                    day7_day_next_hour_zero = day7_day_zero + datetime.timedelta(hours=next_j)
                    # 获取小时时段内的记录总数[>=x:00:00 and <(x+1):00:00]
                    day7_day_online_account_hour_count = HistoryAccountOnline.objects.filter(up_time__gte=day7_day_hour_zero, up_time__lt=day7_day_next_hour_zero).count()
                    day7_day_online_account_hour_dict = {
                        'hour': day7_day_online_account_hour,
                        'total': day7_day_online_account_hour_count
                    }
                    day7_day_online_account_24hour.append(day7_day_online_account_hour_dict)
                    # hour:total封装进map中
                    if day7_day_online_account_hour not in day7_online_account_hour_chart_map:
                        day7_online_account_hour_chart_map[day7_day_online_account_hour] = 0
                    day7_online_account_hour_chart_map[day7_day_online_account_hour] += day7_day_online_account_hour_count
                # 除了今天的都放入缓存中
                if day7_day_zero == self.zero_today:
                    pass
                else:
                    day7_day_online_account_24hour_json = json.dumps(day7_day_online_account_24hour)
                    redis_conn.hset('index_stats_online_account_day7_hour', day7_day_online_account_hour_key, day7_day_online_account_24hour_json)
            else:
                # 从缓存获取
                index_stats_online_account_day7_hour_caches = redis_conn.hget('index_stats_online_account_day7_hour', day7_day_online_account_hour_key)
                day7_day_online_account_hour_json = json.loads(index_stats_online_account_day7_hour_caches)
                for item in day7_day_online_account_hour_json:
                    day7_day_online_account_hour = item['hour']
                    day7_day_online_account_hour_total = item['total']
                    # hour:total封装进map中
                    if day7_day_online_account_hour not in day7_online_account_hour_chart_map:
                        day7_online_account_hour_chart_map[day7_day_online_account_hour] = 0
                    day7_online_account_hour_chart_map[day7_day_online_account_hour] += day7_day_online_account_hour_total
        # 数据处理
        if day7_online_account_hour_chart_map:
            for item_hour, item_total in day7_online_account_hour_chart_map.items():
                day7_online_account_hour_chart.append({
                    'hour': item_hour,
                    'total': item_total
                })

        ######################################## 返回对象封装 ########################################
        res = {
            # 终端数据
            'device_total': devices_total,
            'online_device_total': online_device_total,
            'offline_device_total': offline_device_total,
            'disable_device_total': disable_device_total,
            'online_device_comparison': online_device_comparison,
            'online_device_comparison_org_top': online_device_comparison_org_top,
            'online_device_comparison_org_bottom': online_device_comparison_org_bottom,
            'weekly_online_device_comparison': weekly_online_device_comparison,
            'cur_week_online_device_comparison_chart': cur_week_online_device_comparison_chart,
            'cur_week_online_device_avg_duration_chart': cur_week_online_device_avg_duration_chart,
            'day7_online_device_time_chart': day7_online_device_hour_chart,
            # 人员数据
            'account_total': account_total,
            'online_account_total': online_account_total,
            'offline_account_total': offline_account_total,
            'disable_account_total': disable_account_total,
            'online_account_comparison': online_account_comparison,
            'online_account_comparison_org_top': online_account_comparison_org_top,
            'online_account_comparison_org_bottom': online_account_comparison_org_bottom,
            'weekly_online_account_comparison': weekly_online_account_comparison,
            'cur_week_online_account_comparison_chart': cur_week_online_account_comparison_chart,
            'cur_week_online_account_avg_duration_chart': cur_week_online_account_avg_duration_chart,
            'day7_online_account_time_chart': day7_online_account_hour_chart,
        }
        return Response(res, status=status.HTTP_200_OK)

    
    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def summary(self, request, *args, **kwargs):
        """
        2, 汇总数据
            1,纳管终端数
            2,终端上线率
            3,人员上线率
        """
        # 1,纳管终端数 == 纳管终端进度配置数据中的计划纳管总数
        device_stats_queryset = DeviceStats.objects.first()
        device_stats_plan_total = device_stats_queryset.stats_max if device_stats_queryset.is_switch == 1 else 0
        # 2,终端上线率 == 上线终端 / 纳管终端总数
        # 缓存中的已上线终端
        device_id_list = []
        for item in self.device_queryset.exclude(is_domain=0):
            if redis_conn.hget('actives', item.device_hostname) == b'1':
                device_id_list.append(item.id)
        online_device_total = len(device_id_list)
        if device_stats_plan_total == 0:
            if online_device_total == 0:
                online_device_comparison = 0
            else:
                online_device_comparison = 100 * 100
        else:
            online_device_comparison = round(100 * (online_device_total / device_stats_plan_total),2)
        # 3,人员上线率
        device_account_active_id_list = []
        account_total = self.account_queryset.count()
        for item in self.device_queryset.exclude(is_domain=0):
            device_account_active = redis_conn.hget('device_user_active', item.device_hostname)
            if device_account_active:
                device_account_active = device_account_active.decode('utf-8')
            device_account_active = device_account_active if isinstance(device_account_active, list) else []
            # 终端关联账号ID列表
            for item2 in self.account_queryset:
                if item2.name in device_account_active:
                    device_account_active_id_list.append(item2.id)
        online_account_total = len(device_account_active_id_list)
        if account_total == 0:
            if online_account_total == 0:
                online_account_comparison = 0
            else:
                online_account_comparison = 100 * 100
        else:
            online_account_comparison = round(100 * (online_account_total / account_total),2)

        ######################################## 返回对象封装 ########################################
        res = {
            'device_total': device_stats_plan_total,
            'online_device_comparison': online_device_comparison,
            'online_account_comparison': online_account_comparison,
        }
        return Response(res, status=status.HTTP_200_OK)
    
    
    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def gch_schedule(self, request, *args, **kwargs):
        """
        3, 国产化推广进度
            1,终端纳管进度
            2,终端国产化进度
        """        
        ######################################## 终端纳管进度 ########################################
        # 计划纳管终端数 == 纳管总督进度配置中的数值
        device_stats_queryset = DeviceStats.objects.first()
        device_stats_plan_total = device_stats_queryset.stats_max if device_stats_queryset.is_switch == 1 else 0
        device_org_map = {}
        device_os_map = {}
        device_id_list = []
        for item in self.device_queryset.exclude(is_domain=0):
            device_id_list.append(item.id)
            # 按着部门来区分
            if item.org_info:
                if item.org_info.id not in device_org_map:
                    device_org_map[item.org_info.id] = []
                device_org_map[item.org_info.id].append(item)
            # 按操作系统来区分
            if item.os:
                if item.os not in device_os_map:
                    device_os_map[item.os] = []
                device_os_map[item.os].append(item)

        # 终端数
        device_total = self.device_queryset.exclude(is_domain=0).count()
        # 上周的加域终端数
        last_weekly_mgr_device_datas = self.device_queryset.exclude(is_domain=0).filter(add_domain_time__gte=self.last_week_start,
                                                                     add_domain_time__lte=self.last_week_end)
        last_weekly_mgr_device_total = last_weekly_mgr_device_datas.count()
        # 本周的加域终端数
        cur_weekly_mgr_device_datas = self.device_queryset.exclude(is_domain=0).filter(add_domain_time__gte=self.zero_cur_week,
                                                                     add_domain_time__lte=self.last_cur_week)
        cur_weekly_mgr_device_total = cur_weekly_mgr_device_datas.count()
        # 加域周环比 == (本周-上周)/上周)
        if last_weekly_mgr_device_total == 0:
            if cur_weekly_mgr_device_total == 0:
                weekly_mgr_device_comparison = 0
            else:
                weekly_mgr_device_comparison = 100 * 100
        else:
            weekly_mgr_device_comparison = round(100 * ((cur_weekly_mgr_device_total - last_weekly_mgr_device_total) / last_weekly_mgr_device_total),2)
        # 纳管终端率 == (终端数 / 计划纳管终端数)
        if device_stats_plan_total ==0:
            devices_plan_comparison = 0
        else:
            devices_plan_comparison = round(100 * (device_total / device_stats_plan_total),2)
        # 部门终端纳管率的最高和最低
        if not device_org_map or device_stats_plan_total == 0:
            online_device_comparison_org_top = {}
            online_device_comparison_org_bottom = {}
        else:
            online_device_comparison_org_id_min = 0
            online_device_comparison_org_id_max = 0
            online_device_comparison_total_min = 0
            online_device_comparison_total_max = 0
            # 遍历出最高和最低的2个部门
            for org_id, items in device_org_map.items():
                if online_device_comparison_total_min == 0 or online_device_comparison_total_min > len(items):
                    online_device_comparison_total_min = len(items)
                    online_device_comparison_org_id_min = org_id
                if online_device_comparison_total_max == 0 or online_device_comparison_total_max < len(items):
                    online_device_comparison_total_max = len(items)
                    online_device_comparison_org_id_max = org_id
            # 获取部门对象
            org_min_name = ''
            org_max_name = ''
            if online_device_comparison_org_id_min >0:
                org_min = OrgnizationEntity.objects.filter(id=online_device_comparison_org_id_min).first()
                if org_min:
                    org_min_name = org_min.name
            if online_device_comparison_org_id_max >0:
                org_max = OrgnizationEntity.objects.filter(id=online_device_comparison_org_id_max).first()
                if org_max:
                    org_max_name = org_max.name
            # 结果封装
            # 疑问3: 终端纳管率是除以[终端总数]还是[计划纳管终端总数]?
            online_device_comparison_org_top = {
                'org_name': org_min_name,
                'comparison': round(100 * (online_device_comparison_total_max / device_stats_plan_total),2)
            }
            online_device_comparison_org_bottom = {
                'org_name': org_max_name,
                'comparison': round(100 * (online_device_comparison_total_min / device_stats_plan_total),2)
            }

        ######################################## 终端国产化进度 ########################################
        # 国产化'操作系统'统计数据(饼图)
        gch_os_datas = []
        if not device_os_map:
            pass
        else:
            # 计算操作系统的占比
            for os_name, items in device_os_map.items():
                gch_os_item = {
                    'os': os_name,
                    'total': len(items),
                    'percent': round(100 * (len(items) / device_total),2)
                }
                gch_os_datas.append(gch_os_item)
        # 国产化'品牌'统计数据(饼图)
        gch_brand_datas = []
        device_brand_map = {}
        device_hardware_info_list = DevicesHardware.objects.filter(device_id__in=device_id_list)
        if not device_hardware_info_list:
            pass
        else:
            # 按品牌来区分
            for item in device_hardware_info_list:
                if item.productname not in device_brand_map:
                    device_brand_map[item.productname] = []
                device_brand_map[item.productname].append(item)
            # 计算品牌的占比
            for brand, items in device_brand_map.items():
                gch_brand_item = {
                    'brand': brand,
                    'total': len(items),
                    'percent': round(100 * (len(items) / device_total),2)
                }
                gch_brand_datas.append(gch_brand_item)


        ######################################## 返回对象封装 ########################################
        res = {
            'device_stats_total': device_stats_plan_total,
            'device_total': device_total,
            'weekly_mgr_device_comparison': weekly_mgr_device_comparison,
            'devices_plan_comparison': devices_plan_comparison,
            'online_device_comparison_org_top': online_device_comparison_org_top,
            'online_device_comparison_org_bottom': online_device_comparison_org_bottom,
            'gch_os_datas': gch_os_datas,
            'gch_brand_datas': gch_brand_datas
        }
        return Response(res, status=status.HTTP_200_OK)


    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def device_health(self, request, *args, **kwargs):
        """
        4, 终端健康情况
            1,软件状态
            2,策略情况
            3,补丁情况
            4,任务执行
        """
        ######################################## 软件状态 ########################################
        # 软件安装总数
        software_total = SoftwareInstalledModel.objects.count()
        # 上周安装软件数
        last_weekly_software_total = SoftwareInstalledModel.objects.filter(last_update__gte=self.last_week_start,
                                                                     last_update__lte=self.last_week_end).count()
        # 本周安装软件数
        cur_weekly_software_total = SoftwareInstalledModel.objects.filter(last_update__gte=self.zero_cur_week,
                                                                     last_update__lte=self.last_cur_week)
        # 周环比(公式==(本周-上周)/上周)
        if last_weekly_software_total == 0:
            if cur_weekly_software_total == 0:
                weekly_software_comparison = 0
            else:
                weekly_software_comparison = 100 *100
        else:
            weekly_software_comparison = round(100 * ((cur_weekly_software_total - last_weekly_software_total) / last_weekly_software_total),2)

        ######################################## 策略情况 ########################################
        ## 策略没有总表,需要一个一个的查询并汇总
        # USB策略
        usb_policy_queryset = USBDevicePolicy.objects.all()
        # 防火墙策略
        firewalld_policy_queryset = FirewalldDevicePolicy.objects.all()
        # SSH策略
        ssh_policy_queryset = SSHDevicePolicy.objects.all()
        # 桌面壁纸策略
        wallpaper_policy_queryset = WallpaperDevicePolicy.objects.all()
        # 屏保策略
        screen_policy_queryset = ScreensaversDevicePolicy.objects.all()
        # 水印策略
        watermark_policy_queryset = WaterMarkDevicePolicy.objects.all()
        # 激活策略
        active_policy_queryset = ActivationDevicePolicy.objects.all()
        # 文件重定向策略
        file_redirection_policy_queryset = FileRedirectionPolicy.objects.all()
        # 桌面主题策略
        desktop_theme_policy_queryset = DesktopThemePolicy.objects.all()
        # 系统更新策略
        sys_update_policy_queryset = SystemUpdatesPolicy.objects.all()
        # 软件运行策略
        software_run_policy_queryset = SoftwareRunsPolicy.objects.all()
        ###
        # 策略总数
        policy_total = usb_policy_queryset.count()  \
                        + firewalld_policy_queryset.count()  \
                        + ssh_policy_queryset.count()  \
                        + wallpaper_policy_queryset.count() \
                        + screen_policy_queryset.count()  \
                        + watermark_policy_queryset.count()  \
                        + active_policy_queryset.count()  \
                        + file_redirection_policy_queryset.count() \
                        + desktop_theme_policy_queryset.count() \
                        + sys_update_policy_queryset.count() \
                        + software_run_policy_queryset.count()
        # 策略成功生效数
        policy_success_total = usb_policy_queryset.filter(status=1).count()  \
                        + firewalld_policy_queryset.filter(status=1).count()  \
                        + ssh_policy_queryset.filter(status=1).count()  \
                        + wallpaper_policy_queryset.filter(status=1).count() \
                        + screen_policy_queryset.filter(status=1).count()  \
                        + watermark_policy_queryset.filter(status=1).count()  \
                        + active_policy_queryset.filter(status=1).count()  \
                        + file_redirection_policy_queryset.filter(status=1).count() \
                        + desktop_theme_policy_queryset.filter(status=1).count() \
                        + sys_update_policy_queryset.filter(status=1).count() \
                        + software_run_policy_queryset.filter(status=1).count()

        # 策略生效率
        if policy_total == 0:
            policy_efficiency = 0
        else:
            policy_efficiency = round(100 * (policy_success_total / policy_total),2)
        # 今天的策略生效数(没有状态变更日期,无法计算)
        today_policy_total = 0
        # 昨天的策略生效数(没有状态变更日期,无法计算)
        yesterday_policy_total = 0
        # 日环比(没有状态变更日期,无法计算)
        if yesterday_policy_total == 0:
            if today_policy_total == 0:
                day_policy_comparison = 0
            else:
                day_policy_comparison = 100 *100
        else:
            day_policy_comparison = round(100 * ((today_policy_total - yesterday_policy_total) / yesterday_policy_total),2)
        # 策略生效率最高和最低的部门
        policy_efficiency_org_top = {}
        policy_efficiency_org_bottom = {}

        ######################################## 补丁情况 ########################################
        # 设备相关补丁总数
        device_patch_total = PatchDeviceLog.objects.count()
        # 设备相关补丁已安装数
        device_patch_installed_total = PatchDeviceLog.objects.exclude(install_status=0).count()
        # 设备相关补丁已安装率
        if device_patch_total == 0:
            device_patch_installed_percent =  0
        else:
            device_patch_installed_percent = round(100 * ((device_patch_installed_total) / device_patch_total),2)
        # 昨天的设备相关补丁已安装数
        yesterday_device_patch_total = PatchDeviceLog.objects.exclude(install_status=0).filter(install_time__gte=self.zero_yerterday,
                                                                  install_time__lte=self.last_yerterday).count()
        # 今天的设备相关补丁已安装数
        today_device_patch_total = PatchDeviceLog.objects.exclude(install_status=0).filter(install_time__gte=self.zero_today,
                                                               install_time__lte=self.last_today).count()
        # 日环比
        if yesterday_device_patch_total == 0:
            if today_device_patch_total == 0:
                day_device_patch_comparison = 0
            else:
                day_device_patch_comparison = 100 *100
        else:
            day_device_patch_comparison = round(100 * ((today_device_patch_total - yesterday_device_patch_total) / yesterday_device_patch_total), 2)
        # 设备相关补丁安装率最高和最低的部门
        device_patch_org_map = {}
        # 1> 先找出已安装补丁的相关设备的所属机构
        path_device_id_list = []
        if device_patch_installed_total > 0:
            for item in PatchDeviceLog.objects.exclude(install_status=0):
                path_device_id_list.append(item.device_id)
            patch_device_list = self.device_queryset.filter(id__in=path_device_id_list)
            for item in patch_device_list:
                if item.org_info:
                    # 按着部门来区分
                    if item.org_info.id not in device_patch_org_map:
                        device_patch_org_map[item.org_info.id] = []
                    device_patch_org_map[item.org_info.id].append(item)

        # 2> 再从所属机构中找出最高和最低的部门
        if not device_patch_org_map or device_patch_installed_total == 0:
            patch_installed_comparison_org_top = {}
            patch_installed_comparison_org_bottom = {}
        else:
            patch_installed_comparison_org_id_min = 0
            patch_installed_comparison_org_id_max = 0
            patch_installed_comparison_total_min = 0
            patch_installed_comparison_total_max = 0
            # 遍历出最高和最低的2个部门
            for org_id, items in device_patch_org_map.items():
                if patch_installed_comparison_total_min == 0 or patch_installed_comparison_total_min > len(items):
                    patch_installed_comparison_total_min = len(items)
                    patch_installed_comparison_org_id_min = org_id
                if patch_installed_comparison_total_max == 0 or patch_installed_comparison_total_max < len(items):
                    patch_installed_comparison_total_max = len(items)
                    patch_installed_comparison_org_id_max = org_id
            # 获取部门对象
            patch_org_min_name = ''
            patch_org_max_name = ''
            if patch_installed_comparison_org_id_min >0:
                patch_org_min = OrgnizationEntity.objects.filter(id=patch_installed_comparison_org_id_min).first()
                if patch_org_min:
                    patch_org_min_name = patch_org_min.name
            if patch_installed_comparison_org_id_max >0:
                patch_org_max = OrgnizationEntity.objects.filter(id=patch_installed_comparison_org_id_max).first()
                if patch_org_max:
                    patch_org_max_name = patch_org_max.name
            # 结果封装
            patch_installed_comparison_org_top = {
                'org_name': patch_org_min_name,
                'comparison': round(100 * (patch_installed_comparison_total_max / device_patch_installed_total),2)
            }
            patch_installed_comparison_org_bottom = {
                'org_name': patch_org_max_name,
                'comparison': round(100 * (patch_installed_comparison_total_min / device_patch_installed_total),2)
            }

        ######################################## 任务执行 ########################################
        # 任务总数
        task_total = self.task_queryset.count()
        # 已完成任务数
        finish_task_total = self.task_queryset.filter(status=4).count()
        user_account = request.user
        # 不是超级管理员,只能看到自己的
        if user_account.is_superuser != 1:
            task_total = self.task_queryset.filter(create_user__username=user_account.username).count()
            finish_task_total = self.task_queryset.filter(status=4,
                                                     create_user__username=user_account.username).count()
        # 任务成功率
        if task_total == 0:
            finish_task_percent = 0
        else:
            finish_task_percent = round((100 * finish_task_total / task_total), 2)
        # 任务完成率最高和最低的部门
        task_org_map = {}
        for item in self.task_queryset.filter(status=4):
            # 按着部门来区分
            if item.org_id:
                if item.org_id not in task_org_map:
                    task_org_map[item.org_id] = []
                task_org_map[item.org_id].append(item)
        if not task_org_map or finish_task_total == 0:
            task_finish_comparison_org_top = {}
            task_finish_comparison_org_bottom = {}
        else:
            task_finish_comparison_org_id_min = 0
            task_finish_comparison_org_id_max = 0
            task_finish_comparison_total_min = 0
            task_finish_comparison_total_max = 0
            # 遍历出最高和最低的2个部门
            for org_id, items in task_org_map.items():
                if task_finish_comparison_total_min == 0 or task_finish_comparison_total_min > len(items):
                    task_finish_comparison_total_min = len(items)
                    task_finish_comparison_org_id_min = org_id
                if task_finish_comparison_total_max == 0 or task_finish_comparison_total_max < len(items):
                    task_finish_comparison_total_max = len(items)
                    task_finish_comparison_org_id_max = org_id
            # 获取部门对象
            task_org_min_name = ''
            task_org_max_name = ''
            if task_finish_comparison_org_id_min >0:
                task_org_min = OrgnizationEntity.objects.filter(id=task_finish_comparison_org_id_min).first()
                if task_org_min:
                    task_org_min_name = task_org_min.name
            if task_finish_comparison_org_id_max >0:
                task_org_max = OrgnizationEntity.objects.filter(id=task_finish_comparison_org_id_max).first()
                if task_org_max:
                    task_org_max_name = task_org_max.name
            # 结果封装
            if finish_task_total == 0:
                task_finish_comparison_total_max_comparison = 0
            else:
                task_finish_comparison_total_max_comparison = round(100 * (task_finish_comparison_total_max / finish_task_total),2)
            task_finish_comparison_org_top = {
                'org_name': task_org_min_name,
                'comparison': task_finish_comparison_total_max_comparison
            }
            if finish_task_total == 0:
                task_finish_comparison_total_min_comparison = 0
            else:
                task_finish_comparison_total_min_comparison = round(100 * (task_finish_comparison_total_min / finish_task_total),2)
            task_finish_comparison_org_bottom = {
                'org_name': task_org_max_name,
                'comparison': task_finish_comparison_total_min_comparison
            }

        ######################################## 返回对象封装 ########################################
        res = {
            # 软件数据
            'software_total': software_total,
            'weekly_software_comparison': weekly_software_comparison,
            # 策略数据
            'policy_efficiency': policy_efficiency,
            'day_policy_comparison': day_policy_comparison,
            'policy_efficiency_org_top': policy_efficiency_org_top,
            'policy_efficiency_org_bottom': policy_efficiency_org_bottom,
            # 补丁数据
            'patch_installed_percent': device_patch_installed_percent,
            'day_patch_comparison': day_device_patch_comparison,
            'patch_installed_org_top': patch_installed_comparison_org_top,
            'patch_installed_org_bottom': patch_installed_comparison_org_bottom,
            # 任务数据
            'finish_task_percent': finish_task_percent,
            'task_finish_org_top': task_finish_comparison_org_top,
            'task_finish_org_bottom': task_finish_comparison_org_bottom
        }
        return Response(res, status=status.HTTP_200_OK)
