#!/usr/bin/env python
# _*_ coding: UTF-8 -*-
"""
@Project :nfs_domain_mainserver
@File    :jobs_utils.py
@IDE     :PyCharm
@Author  :huchl
@Date    :2024/12/12 15:38
"""
import ipaddress
import logging
import threading

import pandas as pd

from devices.models import Devices
from groups.models import GroupsDetail
from jobs_management.models import JobResultInfo
from patch_management.models import PatchFile, PatchInfo
from saltapi import tasks as salt_task
from utils.sch_patch import PatchTask

logger = logging.getLogger(__name__)


class JobCalculation(object):

    def __init__(self):
        self.device_items = Devices.objects.exclude(is_domain=0).all()
        self.group_chat_items = GroupsDetail.objects.all()
        self.job_result_items = JobResultInfo.objects.all()

    def organ_method(self, range_value):
        """
        依据部门 计算终端数据
        Args:
            range_value:

        Returns: 终端数据列表

        """
        if range_value is None:
            res_device_list = []
        else:
            organ_list = range_value.split(',')
            if self.device_items.filter(org_info_id__in=organ_list).exists():
                res_device_list = self.device_items.filter(org_info_id__in=organ_list).values_list('id', flat=True)
            else:
                res_device_list = []
        return res_device_list

    @staticmethod
    def device_method(range_value):
        """
        依据终端 计算终端数据
        Args:
            range_value:

        Returns: 终端数据列表

        """
        if range_value is None:
            res_device_list = []
        else:
            res_device_list = range_value.split(',') if range_value is not None else []
        return res_device_list

    def ip_method(self, range_value):
        """
        依据IP 计算终端数据
        Args:
            range_value:

        Returns: 终端数据列表

        """
        res_device_list = []
        if range_value is not None:
            ip_list = range_value.split(',')
            mid_ip_list = []
            for i in ip_list:
                start_ip, end_ip = i.split('-')
                # 解析起始和结束IP地址为IPv4Address对象
                start_ip = ipaddress.IPv4Address(start_ip)
                end_ip = ipaddress.IPv4Address(end_ip)
                range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
                mid_ip_list = range_ip_list + mid_ip_list
            if self.device_items.filter(device_ip__in=mid_ip_list).exists():
                res_device_list = self.device_items.filter(device_ip__in=mid_ip_list).values_list('id', flat=True)
        else:
            pass
        return res_device_list

    def account_method(self, range_value):
        """
        依据账号 计算终端数据
        Args:
            range_value:

        Returns: 终端数据列表

        """
        if range_value is None:
            res_device_list = []
        else:
            account_list = range_value.split(',')
            if self.device_items.filter(bind_account__id__in=account_list).exists():
                res_device_list = self.device_items.filter(bind_account__id__in=account_list).values_list('id',
                                                                                                          flat=True)
            else:
                res_device_list = []
        return res_device_list

    def group_method(self, range_value):
        """
        依据群组 计算终端数据
        Args:
            range_value:

        Returns: 终端数据列表

        """
        if range_value is None:
            res_device_list = []
        else:
            group_chat_list = range_value.split(',')
            if self.group_chat_items.filter(groups_id__in=group_chat_list).exists():
                group_chat_filter_items = self.group_chat_items.filter(groups_id__in=group_chat_list)
                items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
                items_3_range = group_chat_filter_items.filter(range=3).values_list('account_id', flat=True)
                # 处理部门数据
                if self.device_items.filter(org_info_id__in=items_0_range).exists():
                    device_0_list = self.device_items.filter(org_info_id__in=items_0_range).values_list('id', flat=True)
                    device_0_list = [str(i) for i in device_0_list]
                else:
                    device_0_list = []
                # 处理设备信息
                device_1_list = [str(i) for i in items_1_range]
                # 处理账号信息
                if self.device_items.filter(bind_account__id__in=items_3_range).exists():
                    device_3_list = self.device_items.filter(bind_account__id__in=items_3_range).values_list('id',
                                                                                                             flat=True)
                    device_3_list = [str(i) for i in device_3_list]
                else:
                    device_3_list = []
                device_list = device_0_list + device_1_list + device_3_list
                res_device_list = list(set(device_list))
            else:
                res_device_list = []
        return res_device_list

    def get_device_list_string(self, row):
        """
        获取任务对应的终端数据
        (0, '按部门'), (1, '按终端'), (2, '按 IP 网段'), (3, '按人员'), (4, '按组')
        Args:
            row:

        Returns: 终端id字符串 ‘1，2，3，4，’

        """
        # logger.info(f"{row}")
        range_type = row.get('range_type')
        range_value = row.get('range_value')
        exception_range_type = row.get('exception_range_type')
        exception_range_value = row.get('exception_range_value')
        if range_type == 0:  # 部门处理
            res_device_list = self.organ_method(range_value)
        elif range_type == 1:  # 终端处理
            res_device_list = self.device_method(range_value)
        elif range_type == 2:  # ip处理
            res_device_list = self.ip_method(range_value)
        elif range_type == 3:  # 人员处理
            res_device_list = self.account_method(range_value)
        elif range_type == 4:  # 按组
            res_device_list = self.group_method(range_value)
        else:
            res_device_list = []

        # 排除例外的情况
        if len(res_device_list) > 0:
            if exception_range_type == 0:  # 部门
                exception_device_list = self.organ_method(exception_range_value)
            elif exception_range_type == 1:  # 终端
                exception_device_list = self.device_method(exception_range_value)
            elif exception_range_type == 2:  # ip地址
                exception_device_list = self.ip_method(exception_range_value)
            elif exception_range_type == 3:  # 用户
                exception_device_list = self.account_method(exception_range_value)
            elif exception_range_type == 4:  # 群组
                exception_device_list = self.group_method(exception_range_value)
            else:
                exception_device_list = []
        else:
            exception_device_list = []

        res_device_list = [int(i) for i in res_device_list]  # 强转整形
        exception_device_list = [int(i) for i in exception_device_list]  # 强转整形
        set1 = set(res_device_list)
        set2 = set(exception_device_list)
        result_set = set1 - set2
        res_device_list = list(result_set)
        res_device_list = [str(i) for i in res_device_list]
        res_device = ','.join(res_device_list) if len(res_device_list) != 0 else None

        return res_device

    def job_data_analysis(self, item: dict) -> bool:
        """
        更新任务或者新增任务，数据处理，为了进行数据下发
        Args:
            item: jobinfo - - > dict{}

        Returns: True/Fasle

        """
        if item.get('upload_files_ids') is None:
            return False
        if item.get('device_range_value') is None:
            return False
        df = pd.DataFrame([item])
        df['job_device_id'] = df['device_range_value'].str.split(',')
        df_exploded = df.explode('job_device_id').reset_index(drop=True)
        res_datas = df_exploded.to_dict('records')
        for res_data in res_datas:
            self.job_result_db(res_data)
        return True

    def job_result_db(self, item: dict):
        """
        处理传入数据，存入数据库和执行下发操作
        Args:
            item:

        Returns:

        """
        job_type = item.get('job_type')
        if job_type not in [1, 2, 3, 4]:  # 任务类型 1|软件 2|补丁，3|脚本，4|文件
            logger.info(f"任务数据不合法: {job_type}----{item}")
        else:
            if job_type in [1, 2, 3, 4]:
                """ 1|软件，3|脚本，4|文件 记录日志状态的地方 与 2|补丁 记录日志状态的地方不一样"""
                self.job_result_items.update_or_create(
                    defaults={
                        'is_send_status': 0,
                        'update_time': None,
                        'exe_status': 1,
                        "job_jid_id": None,
                        'job_is_delivery': item.get('job_is_delivery', 0),
                        'tar_name_path': item.get('tar_name_path', None),
                        'tar_name': item.get('tar_name', None)
                    },
                    job_info_id=item.get('id'),
                    job_device_id=int(item.get('job_device_id')),
                    job_file_ids=item.get('upload_files_ids')
                )
            if job_type == 1:
                self.software_transfer(item)
            elif job_type == 2:
                self.patch_transfer(item)
            elif job_type == 3:
                self.bash_transfer(item)
            elif job_type == 4:
                self.file_transfer(item)
            else:
                return False
        return True

    def software_transfer(self, item: dict):
        """
        软件下发
        Args:
            item:

        Returns:

        """
        file_tar_name = item.get('tar_name', None)
        if file_tar_name is None:
            return False
        param_dict = {
            'job_info_id': item.get('id'),
            'device_id': item.get('job_device_id'),
            'is_notify_user': item.get('is_notify_user',0),
            'is_user_cancel': item.get('is_user_cancel',0),
            "download_path": f'/home/{file_tar_name}',
            "file_name": file_tar_name,
        }
        salt_task.job_software_policy.delay(**param_dict)
        self.job_result_items.filter(job_info_id=item.get('id')).update(exe_status=2)
        return True

    def patch_transfer(self, item: dict):
        """
        补丁下发
        Args:
            item:

        Returns:

        """
        res_item = []
        patch_id_list = item.get('upload_files_ids').split(',') if item.get('upload_files_ids') else None
        patch_file_items = PatchFile.objects.all()
        patch_info_items = PatchInfo.objects.all()
        for n in patch_id_list:
            patch_file_id = patch_info_items.get(id=n).patch_file_id
            patch_file_name = patch_file_items.get(id=patch_file_id).name
            res_item.append({
                'device_id': item.get('job_device_id'),
                'device_hostname': self.device_items.get(id=item.get('job_device_id')).device_hostname,
                'patch_id': n,
                'patch_name': patch_file_name,
                'is_notify_user': item['is_notify_user'],
                'is_user_cancel': item['is_user_cancel'],
            })

        thread1 = threading.Thread(target=PatchTask().__bulk_data_delivery__, args=(res_item,))  # 将数据同步到缓存中
        thread1.start()

        self.job_result_items.filter(job_info_id=item.get('id')).update(exe_status=2)

    def bash_transfer(self, item: dict):
        """
        脚本下发
        Args:
            item:

        Returns:

        """
        file_tar_name = item.get('tar_name', None)
        if file_tar_name is None:
            return False
        param_dict = {
            'job_info_id': item.get('id'),
            'device_id': item.get('job_device_id'),
            'is_notify_user': item.get('is_notify_user',0),
            'is_user_cancel': item.get('is_user_cancel',0),
            "download_path": f'/home/{file_tar_name}',
            "file_name": file_tar_name,
        }
        salt_task.job_bash_policy.delay(**param_dict)
        self.job_result_items.filter(job_info_id=item.get('id')).update(exe_status=2)
        return True

    def file_transfer(self, item: dict):
        """
        文件下发
        Args:
            item:

        Returns:

        """
        file_tar_name = item.get('tar_name', None)
        if file_tar_name is None:
            return False
        param_dict = {
            'job_info_id': item.get('id'),
            'device_id': item.get('job_device_id'),
            'is_notify_user': item.get('is_notify_user',0),
            'is_user_cancel': item.get('is_user_cancel',0),
            'download_dir': '/home/',
            'download_path': f'/home/{file_tar_name}',
            'file_name': file_tar_name,
        }
        salt_task.job_file_policy.delay(**param_dict)
        self.job_result_items.filter(job_info_id=item.get('id')).update(exe_status=2)
        return True
