import datetime
import json
import logging
import os
import shutil
import threading
import tarfile

import pandas as pd
from django.db import close_old_connections
from django.http import StreamingHttpResponse
from django_redis import get_redis_connection
from rest_framework import status, mixins
from rest_framework.decorators import action
from rest_framework.parsers import MultiPartParser, JSONParser
from rest_framework.response import Response
from rest_framework.filters import OrderingFilter, SearchFilter
from rest_framework.viewsets import GenericViewSet

from django_filters.rest_framework import DjangoFilterBackend
from django.db.models import F
from accounts.models import AccountEntity
from devices.models import Devices
from jobs_management import models, serializers, filter
from jobs_management.jobs_utils import JobCalculation
from nfs_domain_mainserver import settings
from nfs_domain_mainserver.settings import HOSTNAME, REPREPRO_WEBSERVER_PORT
from organizations.models import OrgnizationEntity
from patch_management.models import PatchInfo, PatchDeviceLog
from software.apt_download_deb import AptPackageManager
from software.models import SoftwareInfoModel, SoftwareSource
from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer

logger = logging.getLogger(__name__)
conn = get_redis_connection('redis')


# Create your views here.

class JobFileViewSet(mixins.CreateModelMixin,
                     mixins.RetrieveModelMixin,
                     mixins.UpdateModelMixin,
                     mixins.DestroyModelMixin,
                     GenericViewSet):
    """
    文件：增删改查
    """
    parser_classes = [MultiPartParser, JSONParser]
    queryset = models.JobFile.objects.all()
    serializer_class = serializers.JobFileSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.JobFileFilter
    filterset_fields = '__all__'
    ordering = ['-create_time']

    def create(self, request, *args, **kwargs):
        file_obj = request.data.get('file')
        name = file_obj._name
        size = file_obj.size
        mime = file_obj.content_type
        if size == 0:
            return Response('提交的是空文件', status=status.HTTP_400_BAD_REQUEST)
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        instance = serializer.save(create=self.request.user.username, name=name, size=size, mime=mime,
                                   create_time=datetime.datetime.now())
        instance.path = f"{settings.FILES_PATH}/{instance.file.name}"
        instance.save()
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def file_download(self, request, *args, **kwargs):
        job_file_id = request.query_params.get('job_file_id')
        queryset = self.queryset.filter(id=job_file_id)
        if not queryset.exists():
            return Response('没找到对应文件', status=status.HTTP_400_BAD_REQUEST)
        else:
            file_path = queryset.first().path
            file_name = queryset.first().name

        def file_iterator(file_name, chunk_size=1024):
            with open(file_name, 'rb') as f:
                while True:
                    c = f.read(chunk_size)
                    if c:
                        yield c
                    else:
                        break

        response = StreamingHttpResponse(file_iterator(file_path))
        response['Content-Type'] = 'application/octet-stream'
        response['Content-Disposition'] = 'attachment;filename="{0}"'.format(file_name)

        return response


class JobInfoViewSet(mixins.CreateModelMixin,
                     mixins.RetrieveModelMixin,
                     mixins.ListModelMixin,
                     GenericViewSet):
    queryset = models.JobInfo.objects.filter(job_is_delete=0).all()
    serializer_class = serializers.JobInfoSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.JobInfoFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__'
    ordering = ['-create_time']

    @staticmethod
    def _clear_job_file_():
        """
        清理历史本地上传的补丁文件
        :return:
        """
        job_file_id_list = []
        upload_files_ids = models.JobInfo.objects.filter(job_is_delete=0, job_is_delivery=0).values_list(
            'upload_files_ids', flat=True)
        delete_tar_items = models.JobInfo.objects.filter(job_is_delete=1, job_is_delivery=0).values_list(
            'tar_name_path', flat=True)
        for i in upload_files_ids:
            job_file_id_list = job_file_id_list + i.split(',') if i else []
        if len(job_file_id_list) > 0:
            job_file_id_list = list(set(job_file_id_list))
        delete_items = models.JobFile.objects.exclude(id__in=job_file_id_list)
        for items in delete_items:
            abs_patch = f"{items.path}"
            if os.path.exists(abs_patch):
                os.remove(abs_patch)
        for items in delete_tar_items:
            if items is None:
                continue
            if os.path.exists(items):
                os.remove(items)
        delete_items.delete()
        close_old_connections()

    def tar_files_gz(self, time_string, upload_files_id_list, item):
        """
        任务上传文件统一进行压缩处理
        Args:
            time_string: 时间字符串
            upload_files_id_list: 文件id
            item: 任务数据字典

        Returns:

        """

        tar_name = f"archive_{time_string}.tar.gz"
        archive_name_path = f"{settings.FILES_PATH}/jobs_source/{tar_name}"
        file_names_path = models.JobFile.objects.filter(id__in=upload_files_id_list).values_list('path', flat=True)
        with tarfile.open(archive_name_path, "w:gz") as tar:
            for file_name in file_names_path:
                try:
                    # 只处理文件，不包含目录
                    if os.path.isfile(file_name):
                        # 将文件添加到 tar.gz 文件中
                        arcname = os.path.basename(file_name)
                        tar.add(file_name, arcname=arcname)

                except FileNotFoundError:
                    # 如果文件不存在，则跳过该文件
                    logger.info(f"File not found: {file_name}")
                    continue

        # 记录压缩包名称，这里简单打印出来，在实际应用中应该记录到日志或数据库中
        item['tar_name_path'] = archive_name_path
        item['tar_name'] = tar_name
        return item, None

    def delivery_tar_files_gz(self, time_string, upload_files_id_list, item):
        """
        软件分发处理 软件下载 压缩
        Args:
            time_string:
            upload_files_id_list:
            item:

        Returns:

        """
        file_names_path = f"{settings.FILES_PATH}/jobs_source/{time_string}"
        try:
            item['upload_files_ids_name'] = json.dumps(
                list(SoftwareInfoModel.objects.filter(id__in=upload_files_id_list)
                     .annotate(file_name=F('custome_filename')).values('id', 'file_name')), ensure_ascii=False)
            mirror_sources = [f"deb [trusted=yes] http://{HOSTNAME}:{REPREPRO_WEBSERVER_PORT}/server server main", ]
            software_source_queryset = SoftwareSource.objects.exclude(id=1).filter(is_delete=0).all()
            if software_source_queryset.exists():
                for software_item in software_source_queryset:
                    if software_item.type == 1:
                        # deb [ trusted=yes ] http://40.50.10.104/nfscdos 3.0 softwarecenter drivecenter depend tiger
                        mirror_string = f"deb [trusted=yes] http://{software_item.address}/nfscdos 3.0 softwarecenter drivecenter depend tiger"
                    else:
                        mirror_string = f"deb [trusted=yes] http://{software_item.address}:{software_item.domain_port}/server server main"
                    mirror_sources.append(mirror_string)

            # 从软件源，镜像下载
            package_name_version_list = SoftwareInfoModel.objects.filter(id__in=upload_files_id_list).values(
                "main_package", "main_version")
            package_name_version_list = list(package_name_version_list)
            # 下载软件包并压缩记录
            apt_manager = AptPackageManager(mirror_sources, package_name_version_list, file_names_path)
            apt_manager.run()
            # 不是分发任务上传文件统一进行压缩处理
            tar_name = f"archive_{time_string}.tar.gz"
            archive_name_path = f"{settings.FILES_PATH}/jobs_source/{tar_name}"

            with tarfile.open(archive_name_path, 'w:gz') as tar:
                # 遍历指定目录
                for file_item in os.listdir(file_names_path):
                    item_path = os.path.join(file_names_path, file_item)
                    # 只处理文件，不包含目录
                    if os.path.isfile(item_path):
                        # 将文件添加到 tar.gz 文件中
                        tar.add(item_path, arcname=file_item)

            item['tar_name_path'] = archive_name_path
            item['tar_name'] = tar_name
        except Exception as exp:
            return False, f"{exp}"
        finally:
            if os.path.exists(file_names_path):
                shutil.rmtree(file_names_path)

        return item, None

    def _change_item_(self, item):
        """
        统一修改前端传入body数据
        Args:
            item:

        Returns:

        """
        logger.info(f"{item}")
        time_string = datetime.datetime.now().strftime('%Y%m%d%H%M%S')
        device_range_value = JobCalculation().get_device_list_string(row=item)
        item['running_time'] = datetime.datetime.now()
        item['device_range_value'] = device_range_value
        upload_files_ids = item.get('upload_files_ids').split(',')
        upload_files_id_list = [int(i) for i in upload_files_ids]
        item['upload_files_ids_name'] = json.dumps(list(models.JobFile.objects.filter(id__in=upload_files_id_list)
                                                        .annotate(file_name=F('name'))
                                                        .values("id", 'file_name')), ensure_ascii=False)
        # 任务类型 1|软件 2|补丁，3|脚本，4|文件
        if int(item.get('job_type', 0)) == 1:
            if int(item.get('job_is_delivery', 0)) == 0:
                item, msg = self.tar_files_gz(time_string, upload_files_id_list, item)
            elif int(item.get('job_is_delivery', 0)) == 1:
                item, msg = self.delivery_tar_files_gz(time_string, upload_files_id_list, item)
        elif int(item.get('job_type', 0)) == 2:
            item['upload_files_ids_name'] = json.dumps(list(PatchInfo.objects.filter(id__in=upload_files_id_list)
                                                            .annotate(file_name=F('patch_file_name'))
                                                            .values('id', 'file_name')), ensure_ascii=False)
        elif int(item.get('job_type', 0)) == 3:
            item, msg = self.tar_files_gz(time_string, upload_files_id_list, item)
        elif int(item.get('job_type', 0)) == 4:
            item, msg = self.tar_files_gz(time_string, upload_files_id_list, item)
        else:
            return False, '任务类型错误'

        # 删除自定义字段
        item.pop('jobs_status')
        item.pop('jobs_percentage')
        item.pop('due_date')
        self.queryset.filter(id=item['id']).update(**item)
        job_thread = threading.Thread(target=JobCalculation().job_data_analysis, args=(item,))
        job_thread.start()

        return item, None

    def update_patch_job_status(self, queryset):
        """更新补丁任务状态"""
        patch_device_queryset = PatchDeviceLog.objects.all()
        job_info_ids_list = queryset.values_list('id', flat=True)
        job_res_queryset = models.JobResultInfo.objects.filter(job_info_id__in=job_info_ids_list)
        for obj in queryset:
            job_info_id = obj.id
            patch_ids = obj.upload_files_ids
            patch_ids = patch_ids.split(',') if patch_ids else []
            device_ids = obj.device_range_value
            device_ids = device_ids.split(',') if device_ids else []
            job_device_queryset = patch_device_queryset.filter(patch_id__in=patch_ids, device_id__in=device_ids)
            if job_device_queryset.exists():
                total_num = job_device_queryset.count()
                pending_num = job_device_queryset.filter(install_status=0, is_send_status=0).count()
                running_num = job_device_queryset.filter(install_status=0, is_send_status=1,
                                                         install_err_msg__isnull=True).count()
                success_num = job_device_queryset.filter(install_status__in=[1, 2]).count()
                failure_num = job_device_queryset.filter(install_status=0, is_send_status=1,
                                                         install_err_msg__isnull=False).count()
                # 1|未开始，2|执行中, 3|成功，4|失败
                if total_num == success_num:
                    update_time = job_device_queryset.filter(install_status__in=[1, 2]).order_by(
                        '-install_time').first().install_time
                    job_res_queryset.filter(job_info_id=job_info_id).update(exe_status=3, update_time=update_time)
                elif total_num == pending_num:
                    job_res_queryset.filter(job_info_id=job_info_id).update(exe_status=1, update_time=None)
                else:
                    if running_num > 0:
                        job_res_queryset.filter(job_info_id=job_info_id).update(exe_status=2, update_time=None)
                    elif failure_num > 0:
                        update_time = job_device_queryset.order_by('-install_time').first().install_time
                        job_res_queryset.filter(job_info_id=job_info_id).update(exe_status=4, update_time=update_time)
                    else:
                        update_time = job_device_queryset.filter(install_status__in=[1, 2]).order_by(
                            '-install_time').first().install_time
                        job_res_queryset.filter(job_info_id=job_info_id).update(exe_status=3, update_time=update_time)

    def list(self, request, *args, **kwargs):
        # 启动残留文件清理
        queryset = self.filter_queryset(self.get_queryset())
        cleanup_thread = threading.Thread(target=self._clear_job_file_)
        cleanup_thread.start()
        # 更新补丁任务状态
        patch_queryset = queryset.filter(job_type=2)
        update_thread = threading.Thread(target=self.update_patch_job_status, args=(patch_queryset,))
        update_thread.start()

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    def create(self, request, *args, **kwargs):
        item = request.data.copy()
        if item.get('upload_files_ids') is None:
            return Response('上传文件不为空', status=status.HTTP_400_BAD_REQUEST)
        item['creator'] = request.user.username
        serializer = self.get_serializer(data=item)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)

        tar_thread = threading.Thread(target=self._change_item_, args=(serializer.data,))
        tar_thread.start()

        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def job_type(self, request, *args, **kwargs):
        """
        任务类型
        Args:
            request:
            *args:
            **kwargs:

        Returns: 任务类型 0|软件 1|补丁，2|脚本，3|文件

        """
        res_data = [{'id': 1, 'name': '软件'}, {'id': 2, 'name': '补丁'},
                    {'id': 3, 'name': '脚本'}, {'id': 4, 'name': '文件'}]
        return Response(res_data, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def job_status(self, request, *args, **kwargs):
        """
        任务执行状态  0|未开始 1|执行，2|完成，3|失败
        Args:
            request:
            *args:
            **kwargs:

        Returns:  1|未开始 2|执行，3|完成，4|失败

        """
        res_data = [{'id': 1, 'name': '未开始'}, {'id': 2, 'name': '执行'},
                    {'id': 3, 'name': '完成'}, {'id': 4, 'name': '失败'}]
        return Response(res_data, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """批量操作 删除"""
        is_all = request.data.get('is_all')
        is_delete = request.data.get('is_delete')
        job_ids_str = request.data.get('ids_list')

        if is_all == 0:
            policy_ids_list = job_ids_str.split(',')
            queryset = self.queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            queryset = self.filter_queryset(self.get_queryset())
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if is_delete == 1:
            delete_job_info_id_list = queryset.values_list('id', flat=True)
            models.JobResultInfo.objects.filter(job_info_id__in=delete_job_info_id_list).delete()
            self._clear_job_file_()
            queryset.update(job_is_delete=1)

        return Response("修改完成", status=status.HTTP_200_OK)


class JobDeviceInfoViewSet(GenericViewSet):
    queryset = Devices.objects.all()
    serializer_class = serializers.JobDeviceInfoSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    ordering_fields = '__all__'
    ordering = ('-update_time',)

    @staticmethod
    def _user_name_(x):
        bind_accounts_list = Devices.objects.filter(id=x).values_list('bind_account', flat=True)
        items = AccountEntity.objects.filter(id__in=bind_accounts_list).first()
        if items:
            return items.cn_name
        else:
            return None

    @staticmethod
    def _user_account_(x):
        bind_accounts_list = Devices.objects.filter(id=x).values_list('bind_account', flat=True)
        items = AccountEntity.objects.filter(id__in=bind_accounts_list).first()
        if items:
            return items.name
        else:
            return None

    @staticmethod
    def _page_method_(cache_df, current, page_size):
        """
        df 数据分页方法
        :param cache_df:  pd.DF
        :param current:   当前页数
        :param page_size: 页面大小
        :return: struct_data ---> dict()
        """
        if len(cache_df):
            # 进行分页
            result = cache_df[(int(current) - 1) * int(page_size): int(current) * int(page_size)]
            total = len(cache_df)
            page_count = (total // int(page_size)) + 1

            struct_data = {
                "current": current,
                "data": result.to_dict('records'),
                'pageCount': page_count,
                "pageSize": page_size,
                "total": total,
            }

        else:
            struct_data = {
                "current": current,
                "data": [],
                'pageCount': 1,
                "pageSize": page_size,
                "total": 0,
            }

        return struct_data

    def filter_job_device_details_df(self, queryset):
        """任务执行详情过滤条件"""
        device_cnname = self.request.query_params.get('device_cnname')
        device_cnname__contains = self.request.query_params.get('device_cnname__contains')
        device_ip = self.request.query_params.get('device_ip')
        device_ip__contains = self.request.query_params.get('device_ip__contains')
        org_info_id = self.request.query_params.get('org_info_id')
        is_up = self.request.query_params.get('is_up')
        user_name__contains = self.request.query_params.get('user_name__contains')
        user_account__contains = self.request.query_params.get('user_account__contains')
        online_status = self.request.query_params.get('online_status')
        exe_status = self.request.query_params.get('exe_status')

        if device_cnname is not None:
            queryset = queryset[queryset["device_cnname"] == device_cnname]
        if device_cnname__contains:
            queryset = queryset[queryset["device_cnname"].str.contains(device_cnname__contains)]

        if device_ip is not None:
            queryset = queryset[queryset["device_ip"] == device_ip]
        if device_ip__contains:
            queryset = queryset[queryset["device_ip"].str.contains(device_ip__contains)]

        if org_info_id is not None:
            queryset = queryset[queryset["org_info_id"] == int(org_info_id)]

        if is_up is not None:
            queryset = queryset[queryset["is_up"] == int(is_up)]

        if user_name__contains:
            queryset = queryset[queryset["user_name"].str.contains(user_name__contains)]

        if user_account__contains:
            queryset = queryset[queryset["user_account"].str.contains(user_account__contains)]

        if online_status is not None:
            queryset = queryset[queryset["online_status"] == int(online_status)]

        if exe_status is not None:
            queryset = queryset[queryset["exe_status"] == int(exe_status)]

        return queryset.copy()

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def job_device_detail(self, request, *args, **kwargs):
        """
        根据传入任务id 获取终端上任务状态信息
        Args:
            request: job_info_id 任务id
            *args:
            **kwargs:

        Returns:

        """
        job_info_id = request.query_params.get('job_info_id')
        job_res_info_queryset = models.JobResultInfo.objects.filter(job_info_id=job_info_id).all()
        if not job_res_info_queryset.exists():
            return Response('没有找到相应id的任务', status=status.HTTP_400_BAD_REQUEST)
        devices_list = job_res_info_queryset.values_list('job_device_id', flat=True)
        queryset = self.queryset.filter(id__in=devices_list).exclude(is_domain=0).values(
            'id', 'device_cnname', 'device_ip', 'org_info_id', 'is_up', 'accounts_id', 'device_uuid'
        )
        df = pd.DataFrame(queryset)
        if len(df) != 0:
            organ_items = OrgnizationEntity.objects.all()
            df['user_name'] = df['accounts_id'].apply(lambda x: self._user_name_(x))
            df['user_account'] = df['accounts_id'].apply(lambda x: self._user_account_(x))
            df['org_info_name'] = df['org_info_id'].apply(lambda x: organ_items.get(id=x).name
            if organ_items.filter(id=x).exists() else None)
            df['online_status'] = df['device_uuid'].apply(lambda x: 1 if conn.hget('actives', x) == b'1' else 0)
            df['exe_status'] = df['id'].apply(lambda x: job_res_info_queryset.get(job_device_id=x).exe_status
            if job_res_info_queryset.filter(job_device_id=x).exists() else None)
            df['job_jid'] = df['id'].apply(lambda x: job_res_info_queryset.get(job_device_id=x).job_jid_id
            if job_res_info_queryset.filter(job_device_id=x).exists() else None)
            cache_df = self.filter_job_device_details_df(df)
        else:
            cache_df = pd.DataFrame()

        current = self.request.query_params.get('current', default=1)
        page_size = self.request.query_params.get('pageSize', default=10)
        # 进行分页
        struct_data = self._page_method_(cache_df, current, page_size)

        return Response(struct_data, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def job_detail_total(self, request, *args, **kwargs):
        job_info_id = request.query_params.get('job_info_id')
        job_res_info_queryset = models.JobResultInfo.objects.filter(job_info_id=job_info_id).all()
        if not job_res_info_queryset.exists():
            return Response('没有找到相应id的任务', status=status.HTTP_400_BAD_REQUEST)
        # 1|未开始，2|执行中, 3|成功，4|失败,
        res_data = {
            "pending": job_res_info_queryset.filter(exe_status=1).count(),
            "running": job_res_info_queryset.filter(exe_status=2).count(),
            "success": job_res_info_queryset.filter(exe_status=3).count(),
            "failure": job_res_info_queryset.filter(exe_status=4).count(),
            "total": job_res_info_queryset.count()
        }

        return Response(res_data, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_job_retry(self, request, *args, **kwargs):
        """
        任务批量 重试接口
        Args:
            request:
            *args:
            **kwargs:

        Returns:

        """
        is_all = request.data.get('is_all')
        is_retry = request.data.get('is_retry')
        job_info_id = request.data.get('job_info_id')
        job_device_ids = request.data.get('job_device_ids')

        job_res_info_queryset = models.JobResultInfo.objects.filter(job_info_id=job_info_id).all()

        if is_all == 0:
            job_device_ids_list = job_device_ids.split(',')
        elif is_all == 1:
            devices_list = job_res_info_queryset.values_list('job_device_id', flat=True)
            device_queryset = self.queryset.filter(id__in=devices_list).exclude(is_domain=0).values(
                'id', 'device_cnname', 'device_ip', 'org_info_id', 'is_up', 'accounts_id', 'device_uuid'
            )
            df = pd.DataFrame(device_queryset)
            if len(df) != 0:
                organ_items = OrgnizationEntity.objects.all()
                df['user_name'] = df['accounts_id'].apply(lambda x: self._user_name_(x))
                df['user_account'] = df['accounts_id'].apply(lambda x: self._user_account_(x))
                df['org_info_name'] = df['org_info_id'].apply(lambda x: organ_items.get(id=x).name
                if organ_items.filter(id=x).exists() else None)
                df['online_status'] = df['device_uuid'].apply(lambda x: 1 if conn.hget('actives', x) == b'1' else 0)
                df['exe_status'] = df['id'].apply(lambda x: job_res_info_queryset.get(job_device_id=x).exe_status
                if job_res_info_queryset.filter(job_device_id=x).exists() else None)
                df['job_jid'] = df['id'].apply(lambda x: job_res_info_queryset.get(job_device_id=x).job_jid_id
                if job_res_info_queryset.filter(job_device_id=x).exists() else None)
                cache_df = self.filter_job_device_details_df(df)

                job_device_ids_list = cache_df['id'].to_list()
            else:
                job_device_ids_list = []
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if is_retry == 1:
            if job_device_ids_list:
                job_info_dict = models.JobInfo.objects.filter(id=job_info_id).all().values()[0]
                job_device_ids_list = [str(i) for i in job_device_ids_list]
                job_info_dict.update({'device_range_value': ','.join(job_device_ids_list)})

                job_thread = threading.Thread(target=JobCalculation().job_data_analysis, args=(job_info_dict,))
                job_thread.start()
            else:
                pass

        return Response("重试已提交", status=status.HTTP_200_OK)
