# -*- coding: utf-8 -*-

import logging
import os
import threading

from elasticsearch import Elasticsearch
import pandas as pd
from dateutil.parser import parse
from datetime import datetime

from rest_framework.views import Response
from rest_framework import status, viewsets
from rest_framework.decorators import action

from utils.authentication import TokenAuthentication
from utils.permission import IsAdminUser
from utils.throttling import UserRateThrottle

from nfs_domain_mainserver.settings import SALTFILESYSTEMPATH
from devices.web_consumers import send_msg_to_web
from message_center.models import MessageCenter

logger = logging.getLogger(__name__)


class LogsViewSet(viewsets.ModelViewSet):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    @action(methods=['post'], detail=False, authentication_classes=[TokenAuthentication])
    def export(self, request, *args, **kwargs):
        DOMAIN_ELASTICSEARCH_HOST = os.environ.get("DOMAIN_ELASTICSEARCH_IP")
        DOMAIN_ELASTICSEARCH_PORT = os.environ.get("DOMAIN_ELASTICSEARCH_PORT")
        # 索引值,也是日志类型值
        es_index = request.data.get('es_index')

        if not DOMAIN_ELASTICSEARCH_HOST or not DOMAIN_ELASTICSEARCH_PORT:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': "ES环境变量[DOMAIN_ELASTICSEARCH_IP]或[DOMAIN_ELASTICSEARCH_PORT]不存在"
            })
        if not es_index:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': "索引值参数es_index不能为空"
            })

        try:
            # 创建Elasticsearch客户端,port必须是int
            es = Elasticsearch([{'host': str(DOMAIN_ELASTICSEARCH_HOST), 'port': int(DOMAIN_ELASTICSEARCH_PORT), 'scheme': 'http'}])
            
            # 判断索引是否存在ES中,不存在说明没有数据
            if not es.indices.exists(index=es_index):
                return Response({
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': "索引[%s]无数据" % es_index
                })
        except Exception as e:
            logger.error("ElasticSearch连接异常: %s" % e)
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg':'ElasticSearch连接失败'
            })

        # 后台处理下载
        exporter = ExportEsData(es_index=es_index, es_host=str(DOMAIN_ELASTICSEARCH_HOST), es_port=int(DOMAIN_ELASTICSEARCH_PORT), es_request=request)
        exporter_result = exporter.export_es_to_excel()
        return Response({
            'code': exporter_result['code'],
            'msg': exporter_result['msg']
        })

# Elasticsearch数据查询
class SearchEsData():
    def __init__(self,es_index, es_host, es_port, es_request):
        self.es_index = es_index
        self.es_host = es_host
        self.es_port = es_port
        self.es_request = es_request


# Elasticsearch数据导出类(导出为Excel格式,暂不支持其他格式)
class ExportEsData():
    def __init__(self,es_index, es_host, es_port, es_request):
        self.es_index = es_index
        self.es_host = es_host
        self.es_port = es_port
        self.es_request = es_request

    # 根据ES索引导出不同的数据
    def export_es_to_excel(self):
        if self.es_index == 'user-domain':
            # 用户加域退域
            return self._export_for_user_domain()
        elif self.es_index == 'user-login':
            # 用户登录登出
            return self._export_for_user_login()
        elif self.es_index == 'user-passwd':
            # 域控用户修改密码
            return self._export_for_user_passwd()
        elif self.es_index == 'nfsdomain-admin':
            # 管理员操作日志
            return self._export_for_nfsdomain_admin()
        elif self.es_index == 'system-audit':
            # 系统审计
            return self._export_for_system_audit()
        elif self.es_index == 'system-dpkg':
            # 系统软件安装
            return self._export_for_system_dpkg()
        elif self.es_index == 'system-start':
            # 系统启动
            return self._export_for_system_start()
        elif self.es_index == 'system-usb':
            # USB设备
            return self._export_for_system_usb()
        else:
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'不支持的类型: %s' % self.es_index
            }

    # 导出数据并发送消息
    def _thread_export_es_data(self, data, action_start_time, action_end_time, log_content_desc):
        # 文件名
        now = datetime.now()
        timestamp_str = now.strftime("%Y%m%d%H%M%S")
        filename = f'nfs-domain_{log_content_desc}日志_{timestamp_str}.xlsx'
        # 如果开始时间存在,则文件名重构为:'{action_start_time}--{action_end_time}_{log_content_desc}日志_{timestamp_str}.xlsx'
        if action_start_time:
            # 如果结束时间没有,则默认今天
            if not action_end_time:
                action_end_time = datetime.now()
            action_start_time_str = datetime.strptime(action_start_time, '%Y-%m-%d')
            action_end_time_str = datetime.strptime(action_end_time, '%Y-%m-%d')
            # 只要年-月-日
            filename = f'{action_start_time_str.date()}-{action_end_time_str.date()}_{log_content_desc}日志_{timestamp_str}.xlsx'

        # 将数据转换为DataFrame
        df = pd.DataFrame(data)
        df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"{log_content_desc}日志导出完成, {filename}, 总共{len(data)}条数据",
            'create_time': datetime.now(),
            'target_module': f"{log_content_desc}",
            'message_user': self.es_request.user.username,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, self.es_request.user.username)
    
    # 用户加域退域
    def _export_for_user_domain(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])                

            # 使用Scroll API获取所有文档
            data = []
            scroll_size = 1000  # 每次滚动的大小
            # 过滤条件
            device_cn_name = self.es_request.data.get('device_cn_name')
            device_org_id = self.es_request.data.get('device_org_id')
            device_user_name = self.es_request.data.get('device_user_name')
            device_user_cn_name = self.es_request.data.get('device_user_cn_name')
            action_type = self.es_request.data.get('action_type')
            action_result = self.es_request.data.get('action_result')
            action_start_time = self.es_request.data.get('action_start_time')
            action_end_time = self.es_request.data.get('action_end_time')

            # 拼装ES查询
            query_must_clauses = []
            # 时间范围查询(只有开始时间有值时才过滤)
            if action_start_time:
                # 如果结束时间没有,则默认今天
                if not action_end_time:
                    action_end_time = datetime.now()
                # 将字符串转换为ISO格式, 以适应Elasticsearch的日期字段,结束时间要加上'23:59:59'
                action_start_time_iso = datetime.strptime(action_start_time, '%Y-%m-%d').isoformat()
                action_end_time_iso = datetime.strptime(action_end_time + ' 23:59:59', '%Y-%m-%d %H:%M:%S').isoformat()
                query_must_clauses.append({
                    "range": {
                        "timestamp": {
                            "gte": action_start_time_iso,
                            "lte": action_end_time_iso
                        }
                    }
                })
            # 终端别名
            if device_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_cnname.keyword": device_cn_name
                    }
                })
            # 终端所属部门ID
            if device_org_id:
                query_must_clauses.append({
                    "term": {
                        "host.device_org_id.keyword": device_org_id
                    }
                })
            # 终端所属账号
            if device_user_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_user_name.keyword": device_user_name
                    }
                })
            # 终端所属账号昵称
            if device_user_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_user_cn_name.keyword": device_user_cn_name
                    }
                })
            # 行为类别
            if action_type:
                query_must_clauses.append({
                    "term": {
                        "action-info.action.keyword": action_type
                    }
                })
            # 行为结果
            if action_result:
                query_must_clauses.append({
                    "term": {
                        "action-info.result.keyword": action_result
                    }
                })
            # 构建查询
            query_body = {
                "query": {
                    "bool": {
                        "must": query_must_clauses
                    }
                }
            }
            # 执行搜索
            response = es.search(
                index=self.es_index,
                scroll='2m',
                size=scroll_size,
                body=query_body
            )

            sid = response['_scroll_id']
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            while scroll_size > 0:
                for doc in response['hits']['hits']:
                    # 拼装业务需要的属性
                    item = {}
                    source_data = doc.get('_source',{})
                    action_info = source_data.get('action-info')
                    device_info = source_data.get('device')
                    user_info = source_data.get('user')
                    org_info = source_data.get('org')
                    # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                    if action_info is None or device_info is None or user_info is None or org_info is None:
                        continue;
                                            
                    # 格式化时间
                    time_string = source_data.get('timestamp')
                    dt = parse(time_string)
                    formatted_time = dt.strftime('%Y-%m-%d %H:%M:%S')
                    item["行为时间"] = formatted_time
                    
                    # 转成中文描述
                    action_str = action_info.get('action', '')
                    if action_str == 'join':
                        item["行为"] = '加域'
                    elif action_str == 'remove':
                        item["行为"] = '退域'
                    action_result_str = action_info.get('result', '')
                    if action_result_str == 'success':
                        item["行为结果"] = '成功'
                    elif action_result_str == 'fail':
                        item["行为结果"] = '失败'
                    item["终端别名"] = device_info.get('device_cnname', '')
                    item["终端IP"] = device_info.get('ip', '')
                    item["终端所属用户"] = device_info.get('device_user_name', '')
                    item["终端所属用户昵称"] = device_info.get('device_user_cn_name', '')
                    item["终端所属部门"] = device_info.get('device_org_name', '')
                    data.append(item)

                response = es.scroll(scroll_id=sid, scroll='2m')
                sid = response['_scroll_id']
                scroll_size = len(response['hits']['hits'])

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._thread_export_es_data, args=(data, action_start_time, action_end_time, '域控用户加域退域',))
            thread1.start()

            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '域控用户加域退域日志导出中, 请稍后前往消息中心查看'
            }
        except Exception as exp:
            logger.error(f'_export_for_user_domain error %s' % exp)
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'导出异常: %s' % str(exp)
            }

    # 用户登录登出
    def _export_for_user_login(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])
                
            # 使用Scroll API获取所有文档
            data = []
            scroll_size = 1000  # 每次滚动的大小

            # 过滤条件
            device_cn_name = self.es_request.data.get('device_cn_name')
            org_id = self.es_request.data.get('org_id')
            account_name = self.es_request.data.get('account_name')
            account_cn_name = self.es_request.data.get('account_cn_name')
            action_type = self.es_request.data.get('action_type')
            action_result = self.es_request.data.get('action_result')
            action_start_time = self.es_request.data.get('action_start_time')
            action_end_time = self.es_request.data.get('action_end_time')

            # 拼装ES查询
            query_must_clauses = []
            # 时间范围查询(只有开始时间有值时才过滤)
            if action_start_time:
                # 如果结束时间没有,则默认今天
                if not action_end_time:
                    action_end_time = datetime.now()
                # 将字符串转换为ISO格式, 以适应Elasticsearch的日期字段,结束时间要加上'23:59:59'
                action_start_time_iso = datetime.strptime(action_start_time, '%Y-%m-%d').isoformat()
                action_end_time_iso = datetime.strptime(action_end_time + ' 23:59:59', '%Y-%m-%d %H:%M:%S').isoformat()
                query_must_clauses.append({
                    "range": {
                        "timestamp": {
                            "gte": action_start_time_iso,
                            "lte": action_end_time_iso
                        }
                    }
                })
            # 终端别名
            if device_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_cnname.keyword": device_cn_name
                    }
                })
            # 部门ID
            if org_id:
                query_must_clauses.append({
                    "term": {
                        "org.id.keyword": org_id
                    }
                })
            # 账号
            if account_name:
                query_must_clauses.append({
                    "term": {
                        "user.name.keyword": account_name
                    }
                })
            # 账号昵称
            if account_cn_name:
                query_must_clauses.append({
                    "term": {
                        "user.cn_name.keyword": account_cn_name
                    }
                })
            # 行为类别
            if action_type:
                query_must_clauses.append({
                    "term": {
                        "action-info.action.keyword": action_type
                    }
                })
            # 行为结果
            if action_result:
                query_must_clauses.append({
                    "term": {
                        "action-info.result.keyword": action_result
                    }
                })
            # 构建查询
            query_body = {
                "query": {
                    "bool": {
                        "must": query_must_clauses
                    }
                }
            }
            # 执行搜索
            response = es.search(
                index=self.es_index,
                scroll='2m',
                size=scroll_size,
                body=query_body
            )

            sid = response['_scroll_id']
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            while scroll_size > 0:
                for doc in response['hits']['hits']:
                    # 拼装业务需要的属性
                    item = {}
                    source_data = doc.get('_source',{})
                    action_info = source_data.get('action-info')
                    host_info = source_data.get('host')
                    user_info = source_data.get('user')
                    org_info = source_data.get('org')
                    # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                    if action_info is None or host_info is None or user_info is None or org_info is None:
                        continue;
                                            
                    # 格式化时间
                    time_string = source_data.get('timestamp')
                    dt = parse(time_string)
                    formatted_time = dt.strftime('%Y-%m-%d %H:%M:%S')
                    item["行为时间"] = formatted_time
                    
                    # 转成中文描述
                    action_str = action_info.get('action', '')
                    if action_str == 'login':
                        item["行为"] = '登录'
                    elif action_str == 'logout':
                        item["行为"] = '登出'
                    action_result_str = action_info.get('result', '')
                    if action_result_str == 'success':
                        item["行为结果"] = '成功'
                    elif action_result_str == 'fail':
                        item["行为结果"] = '失败'
                    item["终端别名"] = host_info.get('device_cnname', '')
                    item["终端IP"] = host_info.get('ip', '')
                    item["终端所属用户"] = host_info.get('device_user_name', '')
                    item["终端所属用户昵称"] = host_info.get('device_user_cn_name', '')
                    item["终端所属部门"] = host_info.get('device_org_name', '')
                    item["用户名"] = user_info.get('name', '')
                    item["用户昵称"] = user_info.get('cn_name', '')
                    item["用户所属部门"] = org_info.get('name', '')
                    data.append(item)

                response = es.scroll(scroll_id=sid, scroll='2m')
                sid = response['_scroll_id']
                scroll_size = len(response['hits']['hits'])

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._thread_export_es_data, args=(data, action_start_time, action_end_time, '域控用户登录登出',))
            thread1.start()

            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '域控用户登录登出日志导出中, 请稍后前往消息中心查看'
            }
        except Exception as exp:
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'导出异常: %s' % str(exp)
            }
          
    # 域控用户修改密码
    def _export_for_user_passwd(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])
                
            # 使用Scroll API获取所有文档
            data = []
            scroll_size = 1000  # 每次滚动的大小

            # 过滤条件
            device_cn_name = self.es_request.data.get('device_cn_name')
            org_id = self.es_request.data.get('org_id')
            account_name = self.es_request.data.get('account_name')
            account_cn_name = self.es_request.data.get('account_cn_name')
            action_result = self.es_request.data.get('action_result')
            action_start_time = self.es_request.data.get('action_start_time')
            action_end_time = self.es_request.data.get('action_end_time')

            # 拼装ES查询
            query_must_clauses = []
            # 时间范围查询(只有开始时间有值时才过滤)
            if action_start_time:
                # 如果结束时间没有,则默认今天
                if not action_end_time:
                    action_end_time = datetime.now()
                # 将字符串转换为ISO格式, 以适应Elasticsearch的日期字段,结束时间要加上'23:59:59'
                action_start_time_iso = datetime.strptime(action_start_time, '%Y-%m-%d').isoformat()
                action_end_time_iso = datetime.strptime(action_end_time + ' 23:59:59', '%Y-%m-%d %H:%M:%S').isoformat()
                query_must_clauses.append({
                    "range": {
                        "timestamp": {
                            "gte": action_start_time_iso,
                            "lte": action_end_time_iso
                        }
                    }
                })
            # 终端别名
            if device_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_cnname.keyword": device_cn_name
                    }
                })
            # 部门ID
            if org_id:
                query_must_clauses.append({
                    "term": {
                        "org.id.keyword": org_id
                    }
                })
            # 账号
            if account_name:
                query_must_clauses.append({
                    "term": {
                        "user.name.keyword": account_name
                    }
                })
            # 账号昵称
            if account_cn_name:
                query_must_clauses.append({
                    "term": {
                        "user.cn_name.keyword": account_cn_name
                    }
                })
            # 行为结果
            if action_result:
                query_must_clauses.append({
                    "term": {
                        "action-info.result.keyword": action_result
                    }
                })
            # 构建查询
            query_body = {
                "query": {
                    "bool": {
                        "must": query_must_clauses
                    }
                }
            }
            # 执行搜索
            response = es.search(
                index=self.es_index,
                scroll='2m',
                size=scroll_size,
                body=query_body
            )

            sid = response['_scroll_id']
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            while scroll_size > 0:
                for doc in response['hits']['hits']:
                    # 拼装业务需要的属性
                    item = {}
                    source_data = doc.get('_source',{})
                    action_info = source_data.get('action-info')
                    host_info = source_data.get('host')
                    user_info = source_data.get('user')
                    org_info = source_data.get('org')
                    # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                    if action_info is None or host_info is None or user_info is None:
                        continue;
                                            
                    # 格式化时间
                    time_string = source_data.get('timestamp')
                    dt = parse(time_string)
                    formatted_time = dt.strftime('%Y-%m-%d %H:%M:%S')
                    item["行为时间"] = formatted_time
                    
                    # 转成中文描述
                    item["行为"] = '修改密码'
                    action_result_str = action_info.get('result', '')
                    if action_result_str == 'success':
                        item["行为结果"] = '成功'
                    elif action_result_str == 'fail':
                        item["行为结果"] = '失败'
                    item["终端别名"] = host_info.get('device_cnname', '')
                    item["终端IP"] = host_info.get('ip', '')
                    item["终端所属用户"] = host_info.get('device_user_name', '')
                    item["终端所属用户昵称"] = host_info.get('device_user_cn_name', '')
                    item["终端所属部门"] = host_info.get('device_org_name', '')
                    item["用户名"] = user_info.get('name', '')
                    item["用户昵称"] = user_info.get('cn_name', '')
                    # 部门可能为空
                    if org_info is not None:
                        item["用户所属部门"] = org_info.get('name', '')
                    else:
                        item["用户所属部门"] = ''
                    data.append(item)

                response = es.scroll(scroll_id=sid, scroll='2m')
                sid = response['_scroll_id']
                scroll_size = len(response['hits']['hits'])

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._thread_export_es_data, args=(data, action_start_time, action_end_time, '域控用户修改密码',))
            thread1.start()

            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '域控用户修改密码日志导出中, 请稍后前往消息中心查看'
            }
        except Exception as exp:
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'导出异常: %s' % str(exp)
            }
    
    # 域控管理员操作
    def _export_for_nfsdomain_admin(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])
                
            # 使用Scroll API获取所有文档
            data = []
            scroll_size = 1000  # 每次滚动的大小
            
            # 过滤条件
            admin_name = self.es_request.data.get('admin_name')
            admin_nick_name = self.es_request.data.get('admin_nick_name')
            action_type = self.es_request.data.get('action_type')
            action_result = self.es_request.data.get('action_result')
            action_start_time = self.es_request.data.get('action_start_time')
            action_end_time = self.es_request.data.get('action_end_time')

            # 拼装ES查询
            query_must_clauses = []
            # 时间范围查询(只有开始时间有值时才过滤)
            if action_start_time:
                # 如果结束时间没有,则默认今天
                if not action_end_time:
                    action_end_time = datetime.now()
                # 将字符串转换为ISO格式, 以适应Elasticsearch的日期字段,结束时间要加上'23:59:59'
                action_start_time_iso = datetime.strptime(action_start_time, '%Y-%m-%d').isoformat()
                action_end_time_iso = datetime.strptime(action_end_time + ' 23:59:59', '%Y-%m-%d %H:%M:%S').isoformat()
                query_must_clauses.append({
                    "range": {
                        "timestamp": {
                            "gte": action_start_time_iso,
                            "lte": action_end_time_iso
                        }
                    }
                })
            # 管理员名称
            if admin_name:
                query_must_clauses.append({
                    "term": {
                        "admin.name.keyword": admin_name
                    }
                })
            # 管理员昵称
            if admin_nick_name:
                query_must_clauses.append({
                    "term": {
                        "admin.nick_name.keyword": admin_nick_name
                    }
                })
            # 行为类别
            if action_type:
                query_must_clauses.append({
                    "term": {
                        "operation.type.keyword": action_type
                    }
                })
            # 行为结果
            if action_result:
                query_must_clauses.append({
                    "term": {
                        "response.result.keyword": action_result
                    }
                })
            # 构建查询
            query_body = {
                "query": {
                    "bool": {
                        "must": query_must_clauses
                    }
                }
            }
            # 执行搜索
            response = es.search(
                index=self.es_index,
                scroll='2m',
                size=scroll_size,
                body=query_body
            )

            sid = response['_scroll_id']
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            while scroll_size > 0:
                for doc in response['hits']['hits']:
                    # 拼装业务需要的属性
                    item = {}
                    source_data = doc.get('_source',{})
                    operate_info = source_data.get('operation')
                    admin_info = source_data.get('admin')
                    response_info = source_data.get('response')
                    # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                    if operate_info is None or admin_info is None or response_info is None:
                        continue;
                                            
                    # 格式化时间
                    time_string = source_data.get('timestamp')
                    dt = parse(time_string)
                    formatted_time = dt.strftime('%Y-%m-%d %H:%M:%S')
                    item["行为时间"] = formatted_time
                    
                    # 转成中文描述
                    item["行为"] = operate_info.get('name', '')
                    operate_result_str = response_info.get('result', '')
                    if operate_result_str == 'success':
                        item["行为结果"] = '成功'
                    elif operate_result_str == 'fail':
                        item["行为结果"] = '失败'
                    item["管理员"] = admin_info.get('name', '')
                    item["管理员昵称"] = admin_info.get('nick_name', '')
                    data.append(item)

                response = es.scroll(scroll_id=sid, scroll='2m')
                sid = response['_scroll_id']
                scroll_size = len(response['hits']['hits'])

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._thread_export_es_data, args=(data, action_start_time, action_end_time, '域控管理员操作',))
            thread1.start()

            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '域控管理员操作日志导出中, 请稍后前往消息中心查看'
            }
        except Exception as exp:
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'导出异常: %s' % str(exp)
            }
          
    # 系统审计
    def _export_for_system_audit(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])

            # 使用Scroll API获取所有文档
            data = []
            scroll_size = 1000  # 每次滚动的大小
            
            # 过滤条件
            device_cn_name = self.es_request.data.get('device_cn_name')
            device_org_id = self.es_request.data.get('device_org_id')
            device_user_name = self.es_request.data.get('device_user_name')
            device_user_cn_name = self.es_request.data.get('device_user_cn_name')
            audit_type = self.es_request.data.get('audit_type')
            action_start_time = self.es_request.data.get('action_start_time')
            action_end_time = self.es_request.data.get('action_end_time')

            # 拼装ES查询
            query_must_clauses = []
            # 时间范围查询(只有开始时间有值时才过滤)
            if action_start_time:
                # 如果结束时间没有,则默认今天
                if not action_end_time:
                    action_end_time = datetime.now()
                # 将字符串转换为ISO格式, 以适应Elasticsearch的日期字段,结束时间要加上'23:59:59'
                action_start_time_iso = datetime.strptime(action_start_time, '%Y-%m-%d').isoformat()
                action_end_time_iso = datetime.strptime(action_end_time + ' 23:59:59', '%Y-%m-%d %H:%M:%S').isoformat()
                query_must_clauses.append({
                    "range": {
                        "timestamp": {
                            "gte": action_start_time_iso,
                            "lte": action_end_time_iso
                        }
                    }
                })
            # 终端别名
            if device_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_cnname.keyword": device_cn_name
                    }
                })
            # 终端所属部门ID
            if device_org_id:
                query_must_clauses.append({
                    "term": {
                        "host.device_org_id.keyword": device_org_id
                    }
                })
            # 终端所属账号
            if device_user_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_user_name.keyword": device_user_name
                    }
                })
            # 终端所属账号昵称
            if device_user_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_user_cn_name.keyword": device_user_cn_name
                    }
                })
            # 审计类型
            if audit_type:
                query_must_clauses.append({
                    "term": {
                        "audit.type.keyword": audit_type
                    }
                })
            # 构建查询
            query_body = {
                "query": {
                    "bool": {
                        "must": query_must_clauses
                    }
                }
            }
            # 执行搜索
            response = es.search(
                index=self.es_index,
                scroll='2m',
                size=scroll_size,
                body=query_body
            )

            sid = response['_scroll_id']
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            while scroll_size > 0:
                for doc in response['hits']['hits']:
                    # 拼装业务需要的属性
                    item = {}
                    source_data = doc.get('_source',{})
                    host_info = source_data.get('host')
                    user_info = source_data.get('user')
                    org_info = source_data.get('org')
                    audit_info = source_data.get('audit')
                    # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                    if audit_info is None or host_info is None or user_info is None:
                        continue;
                                            
                    # 格式化时间
                    time_string = source_data.get('timestamp')
                    dt = parse(time_string)
                    formatted_time = dt.strftime('%Y-%m-%d %H:%M:%S')
                    item["行为时间"] = formatted_time
                    
                    # 转成中文描述
                    item["行为"] = audit_info.get('name', '')
                    item["终端别名"] = host_info.get('device_cnname', '')
                    item["终端IP"] = host_info.get('ip', '')
                    item["终端所属用户"] = host_info.get('device_user_name', '')
                    item["终端所属用户昵称"] = host_info.get('device_user_cn_name', '')
                    item["终端所属部门"] = host_info.get('device_org_name', '')
                    # 用户可能为空
                    if user_info is not None:
                        item["用户名"] = user_info.get('name', '')
                        item["用户昵称"] = user_info.get('cn_name', '')
                    else:
                        item["用户名"] = ''
                        item["用户昵称"] = ''
                    # 部门可能为空
                    if org_info is not None:
                        item["用户所属部门"] = org_info.get('name', '')
                    else:
                        item["用户所属部门"] = ''
                    data.append(item)

                response = es.scroll(scroll_id=sid, scroll='2m')
                sid = response['_scroll_id']
                scroll_size = len(response['hits']['hits'])

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._thread_export_es_data, args=(data, action_start_time, action_end_time, '域控终端审计',))
            thread1.start()

            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '域控终端审计日志导出中, 请稍后前往消息中心查看'
            }
        except Exception as exp:
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'导出异常: %s' % str(exp)
            }
    
    # 系统安装软件
    def _export_for_system_dpkg(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])
                
            # 使用Scroll API获取所有文档
            data = []
            scroll_size = 1000  # 每次滚动的大小
            
            # 过滤条件
            device_cn_name = self.es_request.data.get('device_cn_name')
            device_org_id = self.es_request.data.get('device_org_id')
            device_user_name = self.es_request.data.get('device_user_name')
            device_user_cn_name = self.es_request.data.get('device_user_cn_name')
            soft_name = self.es_request.data.get('soft_name')
            soft_action = self.es_request.data.get('soft_action')
            #action_result = self.es_request.data.get('action_result')
            action_start_time = self.es_request.data.get('action_start_time')
            action_end_time = self.es_request.data.get('action_end_time')

            # 拼装ES查询
            query_must_clauses = []
            # 时间范围查询(只有开始时间有值时才过滤)
            if action_start_time:
                # 如果结束时间没有,则默认今天
                if not action_end_time:
                    action_end_time = datetime.now()
                # 将字符串转换为ISO格式, 以适应Elasticsearch的日期字段,结束时间要加上'23:59:59'
                action_start_time_iso = datetime.strptime(action_start_time, '%Y-%m-%d').isoformat()
                action_end_time_iso = datetime.strptime(action_end_time + ' 23:59:59', '%Y-%m-%d %H:%M:%S').isoformat()
                query_must_clauses.append({
                    "range": {
                        "timestamp": {
                            "gte": action_start_time_iso,
                            "lte": action_end_time_iso
                        }
                    }
                })
            # 终端别名
            if device_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_cnname.keyword": device_cn_name
                    }
                })
            # 终端所属部门ID
            if device_org_id:
                query_must_clauses.append({
                    "term": {
                        "host.device_org_id.keyword": device_org_id
                    }
                })
            # 终端所属账号
            if device_user_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_user_name.keyword": device_user_name
                    }
                })
            # 终端所属账号昵称
            if device_user_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_user_cn_name.keyword": device_user_cn_name
                    }
                })
            # 软件名称
            if soft_name:
                query_must_clauses.append({
                    "term": {
                        "soft.name.keyword": soft_name
                    }
                })
            # 行为类型
            if soft_action:
                query_must_clauses.append({
                    "term": {
                        "soft.action.keyword": soft_action
                    }
                })
            # 行为结果
            #if action_result:
            #    query_must_clauses.append({
            #        "term": {
            #            "soft.result.keyword": action_result
            #        }
            #    })
            # 构建查询
            query_body = {
                "query": {
                    "bool": {
                        "must": query_must_clauses
                    }
                }
            }
            # 执行搜索
            response = es.search(
                index=self.es_index,
                scroll='2m',
                size=scroll_size,
                body=query_body
            )

            sid = response['_scroll_id']
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            while scroll_size > 0:
                for doc in response['hits']['hits']:
                    # 拼装业务需要的属性
                    item = {}
                    source_data = doc.get('_source',{})
                    host_info = source_data.get('host')
                    soft_info = source_data.get('soft')
                    # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                    if soft_info is None or host_info is None:
                        continue;
                                            
                    # 格式化时间
                    time_string = source_data.get('timestamp')
                    dt = parse(time_string)
                    formatted_time = dt.strftime('%Y-%m-%d %H:%M:%S')
                    item["行为时间"] = formatted_time
                    
                    # 转成中文描述
                    action_type_str = soft_info.get('action','')
                    if action_type_str == 'install':
                        item["行为"] = '安装'
                    elif action_type_str == 'remove':
                        item["行为"] = '卸载'
                    item["终端别名"] = host_info.get('hostname', '')
                    item["终端IP"] = host_info.get('ip', '')
                    item["终端所属用户"] = host_info.get('device_user_name', '')
                    item["终端所属用户昵称"] = host_info.get('device_user_cn_name', '')
                    item["终端所属部门"] = host_info.get('device_org_name', '')
                    data.append(item)

                response = es.scroll(scroll_id=sid, scroll='2m')
                sid = response['_scroll_id']
                scroll_size = len(response['hits']['hits'])

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._thread_export_es_data, args=(data, action_start_time, action_end_time, '域控用户安装软件',))
            thread1.start()

            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '域控用户安装软件日志导出中, 请稍后前往消息中心查看'
            }
        except Exception as exp:
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'导出异常: %s' % str(exp)
            }
            
    # 系统启动
    def _export_for_system_start(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])
                
            # 使用Scroll API获取所有文档
            data = []
            scroll_size = 1000  # 每次滚动的大小
            
            # 过滤条件
            device_cn_name = self.es_request.data.get('device_cn_name')
            action_start_time = self.es_request.data.get('action_start_time')
            action_end_time = self.es_request.data.get('action_end_time')

            # 拼装ES查询
            query_must_clauses = []
            # 时间范围查询(只有开始时间有值时才过滤)
            if action_start_time:
                # 如果结束时间没有,则默认今天
                if not action_end_time:
                    action_end_time = datetime.now()
                # 将字符串转换为ISO格式, 以适应Elasticsearch的日期字段,结束时间要加上'23:59:59'
                action_start_time_iso = datetime.strptime(action_start_time, '%Y-%m-%d').isoformat()
                action_end_time_iso = datetime.strptime(action_end_time + ' 23:59:59', '%Y-%m-%d %H:%M:%S').isoformat()
                query_must_clauses.append({
                    "range": {
                        "timestamp": {
                            "gte": action_start_time_iso,
                            "lte": action_end_time_iso
                        }
                    }
                })
            # 终端别名
            if device_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_cnname.keyword": device_cn_name
                    }
                })
            # 构建查询
            query_body = {
                "query": {
                    "bool": {
                        "must": query_must_clauses
                    }
                }
            }
            # 执行搜索
            response = es.search(
                index=self.es_index,
                scroll='2m',
                size=scroll_size,
                body=query_body
            )

            sid = response['_scroll_id']
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            while scroll_size > 0:
                for doc in response['hits']['hits']:
                    # 拼装业务需要的属性
                    item = {}
                    source_data = doc.get('_source',{})
                    host_info = source_data.get('host')
                    # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                    if host_info is None:
                        continue;
                                            
                    # 格式化时间
                    time_string = source_data.get('timestamp')
                    dt = parse(time_string)
                    formatted_time = dt.strftime('%Y-%m-%d %H:%M:%S')
                    item["行为时间"] = formatted_time
                    
                    # 转成中文描述
                    item["终端别名"] = host_info.get('hostname', '')
                    item["终端IP"] = host_info.get('ip', '')
                    item["终端所属用户"] = host_info.get('device_user_name', '')
                    item["终端所属用户昵称"] = host_info.get('device_user_cn_name', '')
                    item["终端所属部门"] = host_info.get('device_org_name', '')
                    data.append(item)

                response = es.scroll(scroll_id=sid, scroll='2m')
                sid = response['_scroll_id']
                scroll_size = len(response['hits']['hits'])

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._thread_export_es_data, args=(data, action_start_time, action_end_time, '域控终端系统启动',))
            thread1.start()

            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '域控终端系统启动日志导出中, 请稍后前往消息中心查看'
            }
        except Exception as exp:
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'导出异常: %s' % str(exp)
            }
            
    # usb
    def _export_for_system_usb(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])

            # 使用Scroll API获取所有文档
            data = []
            scroll_size = 1000  # 每次滚动的大小
            
            # 过滤条件
            device_cn_name = self.es_request.data.get('device_cn_name')
            device_org_id = self.es_request.data.get('device_org_id')
            device_user_name = self.es_request.data.get('device_user_name')
            device_user_cn_name = self.es_request.data.get('device_user_cn_name')
            usb_name = self.es_request.data.get('usb_name')
            usb_type = self.es_request.data.get('usb_type')
            #action_result = self.es_request.data.get('action_result')
            action_start_time = self.es_request.data.get('action_start_time')
            action_end_time = self.es_request.data.get('action_end_time')

            # 拼装ES查询
            query_must_clauses = []
            # 时间范围查询(只有开始时间有值时才过滤)
            if action_start_time:
                # 如果结束时间没有,则默认今天
                if not action_end_time:
                    action_end_time = datetime.now()
                # 将字符串转换为ISO格式, 以适应Elasticsearch的日期字段,结束时间要加上'23:59:59'
                action_start_time_iso = datetime.strptime(action_start_time, '%Y-%m-%d').isoformat()
                action_end_time_iso = datetime.strptime(action_end_time + ' 23:59:59', '%Y-%m-%d %H:%M:%S').isoformat()
                query_must_clauses.append({
                    "range": {
                        "timestamp": {
                            "gte": action_start_time_iso,
                            "lte": action_end_time_iso
                        }
                    }
                })
            # 终端别名
            if device_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_cnname.keyword": device_cn_name
                    }
                })
            # 终端所属部门ID
            if device_org_id:
                query_must_clauses.append({
                    "term": {
                        "host.device_org_id.keyword": device_org_id
                    }
                })
            # 终端所属账号
            if device_user_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_user_name.keyword": device_user_name
                    }
                })
            # 终端所属账号昵称
            if device_user_cn_name:
                query_must_clauses.append({
                    "term": {
                        "host.device_user_cn_name.keyword": device_user_cn_name
                    }
                })
            # USB设备名称
            if usb_name:
                query_must_clauses.append({
                    "term": {
                        "usb_device.name.keyword": usb_name
                    }
                })
            # USB设备类型
            if usb_type:
                query_must_clauses.append({
                    "term": {
                        "usb_device.type.keyword": usb_type
                    }
                })
            # 行为结果
            #if action_result:
            #    query_must_clauses.append({
            #        "term": {
            #            "usb_device.result.keyword": action_result
            #        }
            #    })
            # 构建查询
            query_body = {
                "query": {
                    "bool": {
                        "must": query_must_clauses
                    }
                }
            }
            # 执行搜索
            response = es.search(
                index=self.es_index,
                scroll='2m',
                size=scroll_size,
                body=query_body
            )

            sid = response['_scroll_id']
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            while scroll_size > 0:
                for doc in response['hits']['hits']:
                    # 拼装业务需要的属性
                    item = {}
                    source_data = doc.get('_source',{})
                    usb_device_info = source_data.get('usb_device')
                    host_info = source_data.get('host')
                    # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                    if usb_device_info is None or host_info is None:
                        continue;
                    
                    # 格式化时间
                    time_string = source_data.get('timestamp')
                    dt = parse(time_string)
                    formatted_time = dt.strftime('%Y-%m-%d %H:%M:%S')
                    item["行为时间"] = formatted_time
                    
                    # 转成中文描述
                    item["USB设备名"] = usb_device_info.get('name', '')
                    item["USB设备类型"] = usb_device_info.get('type', '')
                    item["终端别名"] = host_info.get('hostname', '')
                    item["终端IP"] = host_info.get('ip', '')
                    item["终端所属用户"] = host_info.get('device_user_name', '')
                    item["终端所属用户昵称"] = host_info.get('device_user_cn_name', '')
                    item["终端所属部门"] = host_info.get('device_org_name', '')
                    data.append(item)

                response = es.scroll(scroll_id=sid, scroll='2m')
                sid = response['_scroll_id']
                scroll_size = len(response['hits']['hits'])

            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._thread_export_es_data, args=(data, action_start_time, action_end_time, '域控终端USB设备',))
            thread1.start()

            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '域控终端USB设备导出中, 请稍后前往消息中心查看'
            }
        except Exception as exp:
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'导出异常: %s' % str(exp)
            }


