# -*- coding: utf-8 -*-
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.filters import OrderingFilter
from rest_framework.response import Response
from rest_framework.viewsets import ReadOnlyModelViewSet

from devices.web_consumers import send_msg_to_web
from users.models import User
from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination

from message_center.models import MessageCenter
from message_center.serializers import MessageCenterSerializer
from message_center import filter
from utils.response import FitJSONRenderer


class MessageCenterViewSet(ReadOnlyModelViewSet):
    queryset = MessageCenter.objects.all()
    serializer_class = MessageCenterSerializer
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter)
    filterset_class = filter.MessageCenterFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__'
    ordering = ('-create_time',)

    def filter_queryset(self, queryset):
        queryset = super(MessageCenterViewSet, self).filter_queryset(self.queryset)

        username = self.request.user.username

        if User.objects.filter(username=username, is_superuser=1).exists():
            pass
        else:
            queryset = queryset.filter(message_user=username)

        return queryset

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def target_module_content(self, request, *args, **kwargs):
        """
        消息所属模块
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        target_module = self.queryset.values_list('target_module', flat=True).distinct()
        return Response({'target_module': target_module}, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def multiple_switch(self, request, *args, **kwargs):
        """
        消息中心批量操作, 批量操作已读未读
        :param request: body {
                        is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        is_read: 1|已读； 0|未读
                        ......: 筛选条件
                        }
        :param args:
        :param kwargs:
        :return:
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        is_read = request.data.get('is_read')

        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)
        if is_all == 1:
            queryset = self.filter_queryset(self.get_queryset())
        else:
            ids_list = [int(i) for i in ids_list.split(',')]
            queryset = self.get_queryset().filter(id__in=ids_list)

        if is_read == 1:
            queryset.update(is_read=1)
            send_msg_to_web({'message_user': self.request.user.username}, 0, self.request.user.username)
            return Response('消息已经标记为已读', status=status.HTTP_200_OK)
        elif is_read == 0:
            queryset.update(is_read=0)
            send_msg_to_web({'message_user': self.request.user.username}, 0, self.request.user.username)
            return Response('消息已经标记为未读', status=status.HTTP_200_OK)
        else:
            return Response('is_read 为必输选项', status=status.HTTP_400_BAD_REQUEST)
