import datetime
import io
import json
import logging
import os
import socket
import threading

import pandas as pd
from django.db import close_old_connections
from django.db.models import Count
from django_filters.rest_framework import DjangoFilterBackend
from django_redis import get_redis_connection
from rest_framework import viewsets, status, mixins
from rest_framework.decorators import action
from rest_framework.filters import OrderingFilter, SearchFilter
from rest_framework.parsers import MultiPartParser, JSONParser
from rest_framework.response import Response
from rest_framework.viewsets import GenericViewSet

from devices.web_consumers import send_msg_to_web
from message_center.models import MessageCenter
from nfs_domain_mainserver import settings
from accounts.models import AccountEntity
from nfs_domain_mainserver.settings import SALTFILESYSTEMPATH
from organizations.models import OrgnizationEntity
from patch_management.serializers import PatchPolicySerializer
from software.deb_util import getDebInfoForAptly
from software.models import SoftwareArch
from devices.models import Devices
from patch_management import models, filter, serializers
from users.role_permisson import get_org_permission_list
from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer
from patch_management.aptly_api_http import AptlyUtil
from utils.sch_patch import patch_task, PatchTask

conn = get_redis_connection('redis')
logger = logging.getLogger(__name__)


# Create your views here.
class PatchFileViewSet(viewsets.ModelViewSet):
    """
    文件：增删改查
    """
    parser_classes = [MultiPartParser, JSONParser]
    queryset = models.PatchFile.objects.all()
    serializer_class = serializers.PatchFileSerializer
    renderer_classes = (FitJSONRenderer,)
    ordering = ['-create_time']

    def create(self, request, *args, **kwargs):
        if not models.PatchSource.objects.filter(patch_source_enable=1).exists():
            return Response('未添加补丁源信息，不能上传补丁', status=status.HTTP_403_FORBIDDEN)
        file_obj = self.request.data.get('file')
        name = file_obj._name
        size = file_obj.size
        mime = file_obj.content_type
        if models.PatchInfo.objects.filter(patch_is_delete=0, patch_file_name=name).exists():
            return Response('上传补丁包已存在', status=status.HTTP_400_BAD_REQUEST)
        if size == 0:
            return Response('提交的是空文件', status=status.HTTP_400_BAD_REQUEST)
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        instance = serializer.save(create_by=self.request.user, name=name, size=size, mime=mime,
                                   create_time=datetime.datetime.now())
        instance.path = settings.MEDIA_URL + instance.file.name
        instance.save()
        abs_patch = settings.BASE_DIR + instance.path
        dict_deb = getDebInfoForAptly(abs_patch)
        instance.name = dict_deb.get('package')
        instance.save()
        headers = self.get_success_headers(serializer.data)
        res_data = serializer.data
        soft_arch_items = SoftwareArch.objects.all()
        if soft_arch_items.filter(name=dict_deb.get('architecture')).exists():
            res_data['patch_app_arch'] = soft_arch_items.get(name=dict_deb.get('architecture')).id
        else:
            """补丁包架构不存在报错"""
            return Response('补丁包暂时支持X86和ARM架构',status=status.HTTP_400_BAD_REQUEST)
        res_data['patch_app_arch_name'] = dict_deb.get('architecture')
        res_data['patch_ver'] = dict_deb.get('version')
        return Response(res_data, status=status.HTTP_201_CREATED, headers=headers)


class PatchTypeDictViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = models.PatchTypeDict.objects.filter(patch_is_delete=0).all()
    serializer_class = serializers.PatchTypeDictSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def update_patch_type(self, request, *args, **kwargs):
        """
        更新补丁类别 包括新增，更新，删除
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        data_list = self.request.data.get('data', [])
        if len(data_list) == 0:
            return Response('数据更新完成', status=status.HTTP_200_OK)
        else:
            queryset = models.PatchTypeDict.objects.all()
            for i in data_list:
                patch_type_name = i.get('patch_type_name')
                patch_is_delete = i.get('patch_is_delete')
                patch_id = i.get('id', None)
                if queryset.filter(id=patch_id, patch_is_delete=patch_is_delete,
                                   patch_type_name=patch_type_name).exists():
                    pass
                else:
                    if queryset.filter(id=patch_id).exists():
                        queryset.filter(id=patch_id).update(patch_is_delete=patch_is_delete,
                                                            patch_type_name=patch_type_name)
                    else:
                        queryset.create(patch_type_name=patch_type_name, patch_is_delete=patch_is_delete,
                                        patch_is_sys=0)

            return Response('数据更新完成', status=status.HTTP_200_OK)


class PatchSysDictViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = models.PatchSysDict.objects.filter(patch_is_delete=0).all()
    serializer_class = serializers.PatchSysDictSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def update_patch_sys(self, request, *args, **kwargs):
        """
        更新补丁应用系统 包括新增，更新，删除
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        data_list = self.request.data.get('data', [])
        if len(data_list) == 0:
            return Response('数据更新完成', status=status.HTTP_200_OK)
        else:
            queryset = models.PatchSysDict.objects.all()
            for i in data_list:
                patch_sys_ver = i.get('patch_sys_ver')
                patch_is_delete = i.get('patch_is_delete')
                patch_id = i.get('id', None)
                if queryset.filter(id=patch_id, patch_is_delete=patch_is_delete,
                                   patch_sys_ver=patch_sys_ver).exists():
                    pass
                else:
                    if queryset.filter(id=patch_id).exists():
                        queryset.filter(id=patch_id).update(patch_is_delete=patch_is_delete,
                                                            patch_sys_ver=patch_sys_ver)
                    else:
                        queryset.create(patch_sys_ver=patch_sys_ver, patch_is_delete=patch_is_delete, patch_is_sys=0)

            return Response('数据更新完成', status=status.HTTP_200_OK)


class PatchInfoViewSet(viewsets.ModelViewSet):
    queryset = models.PatchInfo.objects.filter(patch_is_delete=0).all()
    serializer_class = serializers.PatchInfoSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.PatchInfoFilter
    filterset_fields = '__all__'
    search_fields = ('patch_name', 'patch_no', 'patch_ver', 'patch_app_arch_name')
    ordering_fields = '__all__'
    ordering = ('-update_time',)

    def get_serializer_context(self):
        # 这里返回一个字典，其中包含request对象
        return {'request': self.request}

    @staticmethod
    def _status_name_(x):
        if x == 0:
            status_name = '未安装'
        elif x == 1:
            status_name = '已安装'
        elif x == 2:
            status_name = '已安装未重启'
        else:
            status_name = None
        return status_name

    @staticmethod
    def _user_all_name_(x):
        bind_accounts_list = Devices.objects.filter(id=x).values_list('bind_account', flat=True)
        items = AccountEntity.objects.filter(id__in=bind_accounts_list)
        if items:
            return ';'.join([i.cn_name for i in items])
        else:
            return None

    @staticmethod
    def _user_all_account_(x):
        bind_accounts_list = Devices.objects.filter(id=x).values_list('bind_account', flat=True)
        items = AccountEntity.objects.filter(id__in=bind_accounts_list)
        if items:
            return ';'.join([i.name for i in items])
        else:
            return None

    @staticmethod
    def get_org_info_name(x):
        org_info_id = Devices.objects.filter(id=x).first().org_info_id
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=org_info_id).first()
        if items:
            return items.name
        else:
            return None

    @staticmethod
    def get_online_status(x):
        device_hostname = Devices.objects.filter(id=x).first().device_hostname
        if conn.hget('actives', device_hostname):
            return '在线'
        else:
            return '离线'

    @staticmethod
    def _change_item_(item):
        """
        :param item: dict
        :return:
        """
        item['patch_source_name'] = models.PatchSource.objects.filter(id=item['patch_source']).first().patch_source_name
        item['patch_type_name'] = models.PatchTypeDict.objects.filter(id=item['patch_type']).first().patch_type_name
        temp_list = []
        for i in item['patch_app_arch'].split(','):
            if SoftwareArch.objects.filter(id=i).exists():
                name = SoftwareArch.objects.get(id=i).name
                temp_list.append(name)
        item['patch_app_arch_name'] = ','.join(temp_list)
        temp_mid_lst = []
        for i in item['patch_app_sys'].split(','):
            if models.PatchSysDict.objects.filter(id=i).exists():
                name = models.PatchSysDict.objects.get(id=i).patch_sys_ver
                temp_mid_lst.append(name)
        item['patch_app_sys_name'] = ','.join(temp_mid_lst)
        item['update_time'] = datetime.datetime.now()
        return item

    @staticmethod
    def _clear_patch_file_():
        """
        清理历史本地上传的补丁文件
        :return:
        """
        patch_file_id_list = models.PatchInfo.objects.filter(patch_is_delete=0).values_list('patch_file_id', flat=True)
        delete_items = models.PatchFile.objects.exclude(id__in=patch_file_id_list)
        for items in delete_items:
            abs_patch = settings.BASE_DIR + items.path
            if os.path.exists(abs_patch):
                os.remove(abs_patch)
        delete_items.delete()
        close_old_connections()


    def _download_patch_info_(self, patch_device_items):
        """
        生成补丁信息安装文
        """
        device_items = Devices.objects.all()
        df = pd.DataFrame(patch_device_items.values('id', 'device_id', 'patch_id', 'install_status', 'install_err_msg'))
        df['patch_no'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_no)
        df['patch_name'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_name)
        df['patch_ver'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_ver)
        df['patch_source_name'] = df['patch_id'].apply(
            lambda x: models.PatchInfo.objects.get(id=x).patch_source_name)
        df['patch_app_arch_name'] = df['patch_id'].apply(
            lambda x: models.PatchInfo.objects.get(id=x).patch_app_arch_name)
        df['patch_app_sys_name'] = df['patch_id'].apply(
            lambda x: models.PatchInfo.objects.get(id=x).patch_app_sys_name)
        df['patch_type_name'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_type_name)
        df['update_time'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).update_time)
        df['remark'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).remark)
        df['device_cnname'] = df['device_id'].apply(lambda x: device_items.get(id=x).device_cnname)
        df['user_name'] = df['device_id'].apply(lambda x: self._user_all_name_(x))
        df['user_account'] = df['device_id'].apply(lambda x: self._user_all_account_(x))
        df['device_ip'] = df['device_id'].apply(lambda x: device_items.get(id=x).device_ip)
        df['org_info_name'] = df['device_id'].apply(lambda x: self.get_org_info_name(x))
        df['is_up'] = df['device_id'].apply(lambda x: '禁用' if device_items.get(id=x).is_up == 0 else '启用')
        df['online_status'] = df['device_id'].apply(lambda x: self.get_online_status(x))
        df['install_status'] = df['install_status'].apply(lambda x: self._status_name_(x))

        df = df[
            ['patch_no', 'patch_name', 'patch_ver', 'patch_source_name', 'patch_app_arch_name',
             'patch_app_sys_name',
             'patch_type_name', 'update_time', 'remark', 'device_cnname', 'user_name', 'user_account', 'device_ip',
             'org_info_name', 'is_up', 'online_status', 'install_status', 'install_err_msg']]
        df.columns = ['补丁号', '补丁名称', '补丁版本', '所属源', '补丁适配架构', '补丁适配系统',
                      '补丁类型', '最近更新时间', '备注', '终端别名', '姓名', '用户账号', 'IP 地址',
                      '所属部门', '终端状态', '在线状态', '安装状态', '失败原因']

        filename = f'nfs_domain_补丁终端信息_补丁视角_{datetime.datetime.now().strftime("%Y%m%d%H%M%S")}.xlsx'
        df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"补丁终端信息已导出完成, {filename}，总共{len(df)}条数据",
            'create_time': datetime.datetime.now(),
            'target_module': '补丁终端导出',
            'message_user': self.request.user.username,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, self.request.user.username)
        close_old_connections()

    def list(self, request, *args, **kwargs):
        # 启动残留文件清理
        thread1 = threading.Thread(target=self._clear_patch_file_)
        thread1.start()

        is_all = self.request.query_params.get('is_all', None)
        ids_list = self.request.query_params.get('ids_list', None)
        ids_list = ids_list.split(',') if ids_list is not None else []
        queryset = self.get_queryset()
        # logger.info(f"**********{type(is_all)},{type(ids_list)}")
        if is_all is None:
            pass
        elif is_all == str(0):
            if not isinstance(ids_list, list):
                return Response('ids_list传参非法', status=status.HTTP_400_BAD_REQUEST)
            else:
                queryset = self.get_queryset().filter(id__in=ids_list)
        elif is_all == str(1):
            ids_list = self.filter_queryset(queryset).values_list('id', flat=True)
            return Response({'ids_list': ids_list}, status=status.HTTP_200_OK)
        else:
            return Response('is_all传参非法', status=status.HTTP_400_BAD_REQUEST)

        queryset = self.filter_queryset(queryset)
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)

        return Response(serializer.data)

    def create(self, request, *args, **kwargs):
        data_list = request.data.copy().get('data')
        for item in data_list:
            item = self._change_item_(item)
            # 根据补丁信息上传补丁包到对应的补丁源
            patch_source = item.get('patch_source')
            patch_type= item.get('patch_type')
            patch_file_id = item.get('patch_file_id')
            patch_file_item = models.PatchFile.objects.get(id=patch_file_id)
            i_default_patch_source = models.PatchSource.objects.filter(id=patch_source).first()
            if i_default_patch_source:
                i_default_source_host = "http://" + i_default_patch_source.patch_source_url + ":" + str(
                    i_default_patch_source.patch_source_port)
                # 查询补丁源信息 有的话 传递到工具类AptlyUtil(“xxxxx”)
                i_aptly_util = AptlyUtil(i_default_source_host)
                # repository 名称 ,根据web页面传递的patch_type参数 ，查询PatchTypeDict
                if patch_type:
                    i_patch_type_dic = models.PatchTypeDict.objects.get(pk=patch_type)
                    if i_patch_type_dic:
                        repo_name = i_patch_type_dic.patch_repo_name
                        i_aptly_util.add_package(patch_file=patch_file_item, repo_name=repo_name, filename=item['patch_file_name'])
                else:
                    print("patch type not exist")

            serializer = self.get_serializer(data=item)
            serializer.is_valid(raise_exception=True)
            self.perform_create(serializer)

        return Response('数据创建完成', status=status.HTTP_201_CREATED)

    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        item = self._change_item_(item)
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        patch_file_id = item.get('patch_file_id')
        # 补丁文件变更，变更上传信息
        if int(patch_file_id) != instance.patch_file_id:
            # 删除补丁源中包的信息
            i_patch_source = models.PatchSource.objects.get(pk=instance.patch_source)
            i_patch_file = models.PatchFile.objects.get(pk=instance.patch_file_id)
            i_source_host = "http://" + i_patch_source.patch_source_url + ":" + str(
                i_patch_source.patch_source_port)
            i_aptly = AptlyUtil(i_source_host)
            # repository 名称  查询PatchTypeDict
            i_patch_type_dic = models.PatchTypeDict.objects.get(pk=instance.patch_type)
            i_repo_name = i_patch_type_dic.patch_repo_name
            package_name = i_patch_file.name
            version = instance.patch_ver
            architecture = instance.patch_app_arch_name
            # keys = i_aptly.query_package_keys(repo_name="testrepo",package_name="libffi6",version="3.2.1-8",architecture="amd64")
            keys = i_aptly.query_package_keys(i_repo_name, package_name, version, architecture)
            i_aptly.del_package(i_repo_name, keys)
            # 上传新包到补丁服务器
            patch_source = item.get('patch_source')
            patch_type = item.get('patch_type')
            patch_file_item = models.PatchFile.objects.get(id=patch_file_id)
            i_default_patch_source = models.PatchSource.objects.filter(id=patch_source).first()
            if i_default_patch_source:
                i_default_source_host = "http://" + i_default_patch_source.patch_source_url + ":" + str(
                    i_default_patch_source.patch_source_port)
                # 查询补丁源信息 有的话 传递到工具类AptlyUtil(“xxxxx”)
                i_aptly_util = AptlyUtil(i_default_source_host)
                # repository 名称 ,根据web页面传递的patch_type参数 ，查询PatchTypeDict
                if patch_type:
                    i_patch_type_dic = models.PatchTypeDict.objects.get(pk=patch_type)
                    if i_patch_type_dic:
                        repo_name = i_patch_type_dic.patch_repo_name
                        i_aptly_util.add_package(patch_file=patch_file_item, repo_name=repo_name,
                                                 filename=item['patch_file_name'])
                else:
                    print("patch type not exist")
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        """删除补丁源数据"""
        instance = self.get_object()
        instance.patch_is_delete = 1
        instance.save()
        # self.perform_destroy(instance)
        # 查询补丁源信息 补丁文件信息
        i_patch_source = models.PatchSource.objects.get(pk=instance.patch_source)
        i_patch_file = models.PatchFile.objects.get(pk=instance.patch_file_id)
        try:
            if i_patch_source and i_patch_file:
                i_source_host = "http://" + i_patch_source.patch_source_url + ":" + str(
                    i_patch_source.patch_source_port)
                i_aptly = AptlyUtil(i_source_host)
                # repository 名称  查询PatchTypeDict
                i_patch_type_dic = models.PatchTypeDict.objects.get(pk=instance.patch_type)
                i_repo_name = i_patch_type_dic.patch_repo_name
                package_name = i_patch_file.name
                version = instance.patch_ver
                architecture = instance.patch_app_arch_name
                # keys = i_aptly.query_package_keys(repo_name="testrepo",package_name="libffi6",version="3.2.1-8",architecture="amd64")
                keys = i_aptly.query_package_keys(i_repo_name, package_name, version, architecture)
                i_aptly.del_package(i_repo_name, keys)
        except Exception as exp:
            logging.info(f'清补丁资源服务器资源失败, {exp}')

        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def uninstall_device(self, request, *args, **kwargs):
        """
        补丁获取没安装的终端
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        patch_info_id = self.request.query_params.get('patch_id')
        patch_info_id = int(patch_info_id)
        install_success_device_list = models.PatchDeviceLog.objects.filter(patch_id=patch_info_id).filter(
            install_status=1).values_list('device_id', flat=True)
        res_data = Devices.objects.exclude(is_domain=0).exclude(id__in=install_success_device_list).values()
        return Response(res_data, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def uninstall_patch(self, request, *args, **kwargs):
        """
        终端获取没安装的补丁
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        device_id = self.request.query_params.get('device_id')
        device_id = int(device_id)
        install_success_patch_list = models.PatchDeviceLog.objects.filter(device_id=device_id).filter(
            install_status=1).values_list('patch_id', flat=True)
        res_data = self.queryset.exclude(id__in=install_success_patch_list).values()
        return Response(res_data, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def patch2all(self, request, *args, **kwargs):
        """
        全选获取补丁id和名称
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        patch_name_info = self.filter_queryset(self.get_queryset()).values('id', 'patch_name')
        return Response(patch_name_info, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def patch_2_xlsx(self, request, *args, **kwargs):
        """补丁列表批量导出,删除"""
        patch_ids_str = self.request.data.get('ids_list')
        is_all = self.request.data.get('is_all')
        is_delete = self.request.data.get('is_delete')
        is_down_file = self.request.data.get('is_down_file')
        file_name = self.request.data.get('file_name')

        if is_all == 0:
            patch_ids_list = patch_ids_str.split(',')
            queryset = self.queryset.filter(id__in=patch_ids_list)
        elif is_all == 1:
            queryset = self.filter_queryset(self.get_queryset())
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if is_delete == 1:
            for instance in queryset:
                # 查询补丁源信息 补丁文件信息
                i_patch_source = models.PatchSource.objects.get(pk=instance.patch_source)
                i_patch_file = models.PatchFile.objects.get(id=instance.patch_file_id)
                if i_patch_source and i_patch_file:
                    i_source_host = "http://" + i_patch_source.patch_source_url + ":" + str(
                        i_patch_source.patch_source_port)
                    i_aptly = AptlyUtil(i_source_host)
                    # repository 名称  查询PatchTypeDict
                    i_patch_type_dic = models.PatchTypeDict.objects.get(pk=instance.patch_type)
                    i_repo_name = i_patch_type_dic.patch_repo_name
                    package_name = i_patch_file.name
                    version = instance.patch_ver
                    architecture = instance.patch_app_arch_name
                    # keys = i_aptly.query_package_keys(repo_name="testrepo",package_name="libffi6",version="3.2.1-8",architecture="amd64")
                    try:
                        keys = i_aptly.query_package_keys(i_repo_name, package_name, version, architecture)
                        i_aptly.del_package(i_repo_name, keys)
                    except Exception as exp:
                        logging.info(f'aptly 源中包{package_name} {i_repo_name} 异常 {exp}')

            # 补丁列表删除补丁时清理补丁安装历史记录
            models.PatchDeviceLog.objects.filter(patch_id__in=queryset.values_list('id', flat=True)).delete()
            queryset.update(patch_is_delete=1)

        if is_down_file == 1:
            patch_id_list = queryset.values_list('id', flat=True)
            # logging.info(f"安装*****{patch_id_list}")
            device_items = Devices.objects.all()
            patch_device_items = models.PatchDeviceLog.objects.filter(patch_id__in=patch_id_list,
                                                                      device_id__in=device_items.values_list('id',
                                                                                                             flat=True))
            if len(patch_device_items) <= 0:
                return Response("没查询到安装记录", status=status.HTTP_400_BAD_REQUEST)
            # 后台处理域终端下载
            thread1 = threading.Thread(target=self._download_patch_info_, args=(patch_device_items,))
            thread1.start()

            return Response('补丁信息导出中，请稍后前往消息中心查看', status=status.HTTP_200_OK)

        return Response('成功', status=status.HTTP_200_OK)


class PatchSourceViewSet(viewsets.ModelViewSet):
    queryset = models.PatchSource.objects.all()
    serializer_class = serializers.PatchSourceSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter)
    filterset_class = filter.PatchSourceFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__'
    ordering = ('update_time',)

    def update(self, request, *args, **kwargs):
        data = request.data.copy()
        data['update_time'] = datetime.datetime.now()
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()

        # 清理补丁列表
        patch_info_items = models.PatchInfo.objects.filter(patch_is_delete=0).all()
        for item in patch_info_items:
            # 查询补丁源信息 补丁文件信息
            try:
                i_patch_source = models.PatchSource.objects.get(pk=item.patch_source)
                i_patch_file = models.PatchFile.objects.get(pk=item.patch_file_id)
                if i_patch_source and i_patch_file:
                    i_source_host = "http://" + i_patch_source.patch_source_url + ":" + str(
                        i_patch_source.patch_source_port)
                    i_aptly = AptlyUtil(i_source_host)
                    # repository 名称  查询PatchTypeDict
                    i_patch_type_dic = models.PatchTypeDict.objects.get(pk=item.patch_type)
                    i_repo_name = i_patch_type_dic.patch_repo_name
                    package_name = i_patch_file.name
                    version = item.patch_ver
                    architecture = item.patch_app_arch_name
                    keys = i_aptly.query_package_keys(i_repo_name, package_name, version, architecture)
                    i_aptly.del_package(i_repo_name, keys)
            except Exception as exp:
                logging.info(f'清补丁资源服务器资源失败, {exp}')

        patch_info_items.update(patch_is_delete=1)
        # 补丁策略调整为禁用
        models.PatchPolicy.objects.update(status=0)

        self.perform_destroy(instance)
        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def test_ping(self, request, *args, **kwargs):
        """
        测试地址源的连通性
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        ip = self.request.query_params['ip']
        port = self.request.query_params['port']
        port = int(port)
        try:
            # 创建一个套接字对象并尝试连接到指定的IP地址和端口
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.settimeout(5)  # 设置超时时间（秒）
                s.connect((ip, port))
            return Response(f"成功连接到 {ip}:{port}", status=status.HTTP_200_OK)
        except Exception as e:
            return Response(f"无法连接到 {ip}:{port}，错误信息: {str(e)}", status=status.HTTP_400_BAD_REQUEST)


class PatchDevicesViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Devices.objects.exclude(is_domain=0)
    serializer_class = serializers.PatchDevicesSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter)
    filterset_class = filter.PatchDevicesFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__'
    ordering = ('id',)

    @staticmethod
    def _status_name_(x):
        if x == 0:
            status_name = '未安装'
        elif x == 1:
            status_name = '已安装'
        elif x == 2:
            status_name = '已安装未重启'
        else:
            status_name = None
        return status_name

    @staticmethod
    def _user_all_name_(x):
        bind_accounts_list = Devices.objects.filter(id=x).values_list('bind_account', flat=True)
        items = AccountEntity.objects.filter(id__in=bind_accounts_list)
        if items:
            return ';'.join([i.cn_name for i in items])
        else:
            return None

    @staticmethod
    def _user_all_account_(x):
        bind_accounts_list = Devices.objects.filter(id=x).values_list('bind_account', flat=True)
        items = AccountEntity.objects.filter(id__in=bind_accounts_list)
        if items:
            return ';'.join([i.name for i in items])
        else:
            return None

    @staticmethod
    def get_org_info_name(x):
        org_info_id = Devices.objects.filter(id=x).first().org_info_id
        organ_items = OrgnizationEntity.objects.all()
        items = organ_items.filter(id=org_info_id).first()
        if items:
            return items.name
        else:
            return None

    @staticmethod
    def get_online_status(x):
        device_hostname = Devices.objects.filter(id=x).first().device_hostname
        if conn.hget('actives', device_hostname):
            return '在线'
        else:
            return '离线'

    def _patch_device_info_(self, patch_device_items):
        df = pd.DataFrame(patch_device_items.values('id', 'device_id', 'patch_id', 'install_status', 'install_err_msg'))
        df['device_cnname'] = df['device_id'].apply(lambda x: Devices.objects.get(id=x).device_cnname)
        df['user_name'] = df['device_id'].apply(lambda x: self._user_all_name_(x))
        df['user_account'] = df['device_id'].apply(lambda x: self._user_all_account_(x))
        df['device_ip'] = df['device_id'].apply(lambda x: Devices.objects.get(id=x).device_ip)
        df['org_info_name'] = df['device_id'].apply(lambda x: self.get_org_info_name(x))
        df['is_up'] = df['device_id'].apply(lambda x: '禁用' if Devices.objects.get(id=x).is_up == 0 else '启用')
        df['online_status'] = df['device_id'].apply(lambda x: self.get_online_status(x))
        df['patch_no'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_no)
        df['patch_name'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_name)
        df['patch_ver'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_ver)
        df['patch_source_name'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_source_name)
        df['patch_app_arch_name'] = df['patch_id'].apply(
            lambda x: models.PatchInfo.objects.get(id=x).patch_app_arch_name)
        df['patch_app_sys_name'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_app_sys_name)
        df['patch_type_name'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).patch_type_name)
        df['update_time'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).update_time)
        df['remark'] = df['patch_id'].apply(lambda x: models.PatchInfo.objects.get(id=x).remark)
        df['install_status'] = df['install_status'].apply(lambda x: self._status_name_(x))

        df = df[
            ['device_cnname', 'user_name', 'user_account', 'device_ip', 'org_info_name', 'is_up', 'online_status',
             'patch_no', 'patch_name', 'patch_ver', 'patch_source_name', 'patch_app_arch_name', 'patch_app_sys_name',
             'patch_type_name', 'update_time', 'remark', 'install_status', 'install_err_msg']]
        df.columns = ['终端别名', '姓名', '用户账号', 'IP 地址', '所属部门', '终端状态', '在线状态',
                      '补丁号', '补丁名称', '补丁版本', '所属源', '补丁适配架构', '补丁适配系统',
                      '补丁类型', '最近更新时间', '备注', '安装状态', '失败原因']

        filename = f'nfs_domain_终端补丁信息_终端视角_{datetime.datetime.now().strftime("%Y%m%d%H%M%S")}.xlsx'
        df.to_excel(f"{SALTFILESYSTEMPATH}/{filename}", encoding='utf-8', index=False)

        # 将文件已经下载的消息发送给前端websocket
        websocket_msg = {
            'content': f"终端补丁信息已导出完成, {filename}，总共{len(df)}条数据",
            'create_time': datetime.datetime.now(),
            'target_module': '终端补丁导出',
            'message_user': self.request.user.username,
            'message_device': None,
            'download_path': f'/mainapi/client/{filename}/',
            'is_read': 0,
            'is_broadcast': 0,
        }
        MessageCenter.objects.create(**websocket_msg)
        send_msg_to_web(websocket_msg, 0, self.request.user.username)
        close_old_connections()

    def filter_queryset(self, queryset):
        queryset = super(PatchDevicesViewSet, self).filter_queryset(self.queryset)

        user_data = self.request.user
        org_list = get_org_permission_list(user_data)
        if org_list:
            queryset = queryset.filter(org_info__in=org_list)

        # 用户姓名模糊匹配
        if self.request.query_params.get('user_name__contains', None):
            params_dict = self.request.query_params.get('user_name__contains', None)
            queryset = queryset.filter(bind_account__cn_name__contains=params_dict)

        if self.request.query_params.get('user_account__contains', None):
            params_dict = self.request.query_params.get('user_account__contains', None)
            queryset = queryset.filter(bind_account__name__contains=params_dict)

        # 终端在离线过滤
        if self.request.query_params.get('online_status', None):
            params_dict = self.request.query_params['online_status']
            keys = conn.hgetall('actives')
            matching_keys = [key.decode('utf-8') for key, value in keys.items() if value == b'1']
            logging.info(f"{matching_keys}")
            if int(params_dict) == 0:
                queryset = queryset.exclude(device_hostname__in=matching_keys)
            else:
                queryset = queryset.filter(device_hostname__in=matching_keys)

        return queryset

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def device2all(self, request, *args, **kwargs):
        """
        全选终端，获取终端的主键id
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        device_id_list = self.filter_queryset(self.get_queryset()).values_list('id', flat=True)
        return Response({"device_list": device_id_list}, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def device_2_xlsx(self, request, *args, **kwargs):
        """终端列表批量导出 删除"""
        device_ids_str = self.request.data.get('ids_list')
        is_all = self.request.data.get('is_all')
        file_name = self.request.data.get('file_name')
        if is_all == 0:
            device_ids_list = device_ids_str.split(',')
            queryset = self.queryset.filter(id__in=device_ids_list)
        elif is_all == 1:
            queryset = self.filter_queryset(self.get_queryset())
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        device_id_list = queryset.values_list('id', flat=True)
        patch_device_items = models.PatchDeviceLog.objects.filter(device_id__in=device_id_list)
        if len(patch_device_items) <= 0:
            return Response("没查询到安装记录", status=status.HTTP_400_BAD_REQUEST)

        # 后台处理域终端下载
        thread1 = threading.Thread(target=self._patch_device_info_, args=(patch_device_items,))
        thread1.start()

        return Response('终端补丁信息导出中，请稍后前往消息中心查看', status=status.HTTP_200_OK)


class PatchDeviceLogViewSet(viewsets.GenericViewSet):
    queryset = models.PatchDeviceLog.objects.all()
    serializer_class = serializers.PatchDeviceLogSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    ordering_fields = '__all__'
    ordering = ('id',)

    @staticmethod
    def _status_name_(x):
        if x == 0:
            status_name = '未安装'
        elif x == 1:
            status_name = '已安装'
        elif x == 2:
            status_name = '已安装未重启'
        else:
            status_name = None
        return status_name

    @staticmethod
    def _user_name_(x, id_list=None):
        bind_accounts_list = Devices.objects.filter(id=x).values_list('bind_account', flat=True)
        if id_list:
            items = AccountEntity.objects.filter(id__in=bind_accounts_list).filter(id__in=id_list).first()
        else:
            items = AccountEntity.objects.filter(id__in=bind_accounts_list).first()
        if items:
            return items.cn_name
        else:
            return None

    @staticmethod
    def _user_account_(x, id_list=None):
        bind_accounts_list = Devices.objects.filter(id=x).values_list('bind_account', flat=True)
        if id_list:
            items = AccountEntity.objects.filter(id__in=bind_accounts_list).filter(id__in=id_list).first()
        else:
            items = AccountEntity.objects.filter(id__in=bind_accounts_list).first()

        if items:
            return items.name
        else:
            return None

    @staticmethod
    def _page_method_(cache_df, current, page_size):
        """
        df 数据分页方法
        :param cache_df:  pd.DF
        :param current:   当前页数
        :param page_size: 页面大小
        :return: struct_data ---> dict()
        """
        if len(cache_df):
            # 进行分页
            result = cache_df[(int(current) - 1) * int(page_size): int(current) * int(page_size)]
            total = len(cache_df)
            page_count = (total // int(page_size)) + 1

            struct_data = {
                "current": current,
                "data": result.to_dict('records'),
                'pageCount': page_count,
                "pageSize": page_size,
                "total": total,
            }

        else:
            struct_data = {
                "current": current,
                "data": [],
                'pageCount': 1,
                "pageSize": page_size,
                "total": 0,
            }

        return struct_data

    def filter_device2patch_df(self, queryset):
        """终端视角过滤条件"""
        patch_name = self.request.query_params.get('patch_name')
        patch_name__contains = self.request.query_params.get('patch_name__contains')
        patch_ver = self.request.query_params.get('patch_ver')
        patch_ver__contains = self.request.query_params.get('patch_ver__contains')
        patch_no = self.request.query_params.get('patch_no')
        patch_no__contains = self.request.query_params.get('patch_no__contains')
        patch_source = self.request.query_params.get('patch_source')
        patch_source__contains = self.request.query_params.get('patch_source__contains')
        patch_app_arch = self.request.query_params.get('patch_app_arch')
        patch_app_arch__contains = self.request.query_params.get('patch_app_arch__contains')
        patch_app_sys = self.request.query_params.get('patch_app_sys')
        patch_app_sys__contains = self.request.query_params.get('patch_app_sys__contains')
        patch_type = self.request.query_params.get('patch_type')
        patch_type__contains = self.request.query_params.get('patch_type__contains')
        install_status = self.request.query_params.get('install_status')

        if patch_name:
            queryset = queryset[queryset["patch_name"] == patch_name]
        if patch_name__contains:
            queryset = queryset[queryset["patch_name"].str.contains(patch_name__contains)]

        if patch_ver:
            queryset = queryset[queryset["patch_ver"] == patch_ver]
        if patch_ver__contains:
            queryset = queryset[queryset["patch_ver"].str.contains(patch_ver__contains)]

        if patch_no:
            queryset = queryset[queryset["patch_no"] == patch_no]
        if patch_no__contains:
            queryset = queryset[queryset["patch_no"].str.contains(patch_no__contains)]

        if patch_source:
            queryset = queryset[queryset["patch_source"] == patch_source]
        if patch_source__contains:
            queryset = queryset[queryset["patch_source"].str.contains(patch_source__contains)]

        if patch_app_arch:
            queryset = queryset[queryset["patch_app_arch"] == patch_app_arch]
        if patch_app_arch__contains:
            queryset = queryset[queryset["patch_app_arch"].str.contains(patch_app_arch__contains)]

        if patch_app_sys:
            queryset = queryset[queryset["patch_app_sys"] == patch_app_sys]
        if patch_app_sys__contains:
            queryset = queryset[queryset["patch_app_sys"].str.contains(patch_app_sys__contains)]

        if patch_type:
            queryset = queryset[queryset["patch_type"] == patch_type]
        if patch_type__contains:
            queryset = queryset[queryset["patch_type"].str.contains(patch_type__contains)]

        if install_status:
            queryset = queryset[queryset["install_status"] == int(install_status)]

        return queryset.copy()

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def device2patch(self, request, *args, **kwargs):
        """从终端视角看补丁"""
        device_id = self.request.query_params.get('device_id')
        policy_id = self.request.query_params.get('policy_id')
        patch_items = models.PatchInfo.objects.all()
        if policy_id:
            patch_ids = models.PatchPolicy.objects.filter(id=policy_id).first().patch_ids
            if patch_ids:
                patch_id_list = patch_ids.split(',')
                patch_id_list = [int(i) for i in patch_id_list]
            else:
                patch_id_list = []
        else:
            # 获取未被删除的补丁id
            patch_id_list = patch_items.filter(patch_is_delete=0).values_list('id',flat=True)

        patch_items = models.PatchInfo.objects.all()
        temp_patch_item = patch_items.filter(patch_is_delete=0).filter(id__in=patch_id_list).values()
        df = pd.DataFrame(temp_patch_item)
        if len(df) != 0:
            df['update_time'] = df['update_time'].dt.strftime('%Y-%m-%d %H:%M:%S')
            df['install_status'] = df['id'].apply(
                lambda x: self.queryset.get(patch_id=x, device_id=device_id).install_status
                if self.queryset.filter(patch_id=x, device_id=device_id).exists() else 0)
            df['install_status_name'] = df['install_status'].apply(lambda x: self._status_name_(x))
            df['install_time'] = df['id'].apply(
                lambda x: self.queryset.get(patch_id=x, device_id=device_id).install_time.strftime('%Y-%m-%d %H:%M:%S')
                if self.queryset.get(patch_id=x, device_id=device_id).install_time
                   and self.queryset.filter(patch_id=x, device_id=device_id).exists() else None)
            df['install_err_msg'] = df['id'].apply(
                lambda x: self.queryset.get(patch_id=x, device_id=device_id).install_err_msg
                if self.queryset.filter(patch_id=x, device_id=device_id).exists() else None)

            # 按条件查找, 搜索
            cache_df = self.filter_device2patch_df(df)
        else:
            cache_df = pd.DataFrame()

        current = self.request.query_params.get('current', default=1)
        page_size = self.request.query_params.get('pageSize', default=10)
        # 进行分页
        struct_data = self._page_method_(cache_df, current, page_size)

        return Response(struct_data, status=status.HTTP_200_OK)

    def filter_patch2device_df(self, queryset):
        """补丁视角过滤条件"""
        # logging.info(f"******\r\n{queryset.to_dict('records')}")
        device_cnname = self.request.query_params.get('device_cnname')
        device_cnname__contains = self.request.query_params.get('device_cnname__contains')
        device_ip = self.request.query_params.get('device_ip')
        device_ip__contains = self.request.query_params.get('device_ip__contains')
        org_info_id = self.request.query_params.get('org_info_id')
        is_up = self.request.query_params.get('is_up')
        user_name__contains = self.request.query_params.get('user_name__contains')
        user_account__contains = self.request.query_params.get('user_account__contains')
        online_status = self.request.query_params.get('online_status')
        install_status = self.request.query_params.get('install_status')

        if device_cnname:
            queryset = queryset[queryset["device_cnname"] == device_cnname]
        if device_cnname__contains:
            queryset = queryset[queryset["device_cnname"].str.contains(device_cnname__contains)]

        if device_ip:
            queryset = queryset[queryset["device_ip"] == device_ip]
        if device_ip__contains:
            queryset = queryset[queryset["device_ip"].str.contains(device_ip__contains)]

        if org_info_id:
            queryset = queryset[queryset["org_info_id"] == int(org_info_id)]

        if is_up:
            queryset = queryset[queryset["is_up"] == int(is_up)]

        if user_name__contains:
            queryset = queryset[queryset["user_name"].str.contains(user_name__contains)]

        if user_account__contains:
            queryset = queryset[queryset["user_account"].str.contains(user_account__contains)]

        if online_status:
            queryset = queryset[queryset["online_status"] == int(online_status)]

        if install_status:
            queryset = queryset[queryset["install_status"] == int(install_status)]

        return queryset.copy()

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def patch2device(self, request, *args, **kwargs):
        """从补丁视角看终端"""
        patch_id = self.request.query_params.get('patch_id')
        device_id_list = self.queryset.filter(patch_id=patch_id).values_list('device_id', flat=True)
        device_items = Devices.objects.filter(id__in=device_id_list).exclude(is_domain=0).values(
            'id', 'device_cnname', 'device_ip', 'org_info_id', 'is_up', 'device_uuid'
        )

        df = pd.DataFrame(device_items)
        if len(df) != 0:
            organ_items = OrgnizationEntity.objects.all()
            df['user_name'] = df['id'].apply(lambda x: self._user_name_(x))
            df['user_account'] = df['id'].apply(lambda x: self._user_account_(x))
            df['org_info_name'] = df['org_info_id'].apply(
                lambda x: organ_items.get(id=x).name if organ_items.filter(id=x).exists() else None)
            df['online_status'] = df['device_uuid'].apply(lambda x: 1 if conn.hget('actives', x) else 0)

            df['install_status'] = df['id'].apply(
                lambda x: self.queryset.get(device_id=x, patch_id=patch_id).install_status)
            df['install_status_name'] = df['install_status'].apply(lambda x: self._status_name_(x))
            df['install_time'] = df['id'].apply(
                lambda x: self.queryset.get(device_id=x, patch_id=patch_id).install_time.strftime(
                    '%Y-%m-%d %H:%M:%S') if self.queryset.get(device_id=x, patch_id=patch_id).install_time else None)
            df['install_err_msg'] = df['id'].apply(
                lambda x: self.queryset.get(device_id=x, patch_id=patch_id).install_err_msg)

            # 按条件查找, 搜索
            cache_df = self.filter_patch2device_df(df)
        else:
            cache_df = pd.DataFrame()

        current = self.request.query_params.get('current', default=1)
        page_size = self.request.query_params.get('pageSize', default=10)
        # 进行分页
        struct_data = self._page_method_(cache_df, current, page_size)

        return Response(struct_data, status=status.HTTP_200_OK)

    def filter_patch_details_df(self, queryset):
        """补丁视角过滤条件"""
        # logging.info(f"******\r\n{queryset.to_dict('records')}")
        device_cnname = self.request.query_params.get('device_cnname')
        device_cnname__contains = self.request.query_params.get('device_cnname__contains')
        device_ip = self.request.query_params.get('device_ip')
        device_ip__contains = self.request.query_params.get('device_ip__contains')
        org_info_id = self.request.query_params.get('org_info_id')
        is_up = self.request.query_params.get('is_up')
        user_name__contains = self.request.query_params.get('user_name__contains')
        user_account__contains = self.request.query_params.get('user_account__contains')
        online_status = self.request.query_params.get('online_status')
        apply_status = self.request.query_params.get('apply_status')

        if device_cnname is not None:
            queryset = queryset[queryset["device_cnname"] == device_cnname]
        if device_cnname__contains:
            queryset = queryset[queryset["device_cnname"].str.contains(device_cnname__contains)]

        if device_ip is not None:
            queryset = queryset[queryset["device_ip"] == device_ip]
        if device_ip__contains:
            queryset = queryset[queryset["device_ip"].str.contains(device_ip__contains)]

        if org_info_id is not None:
            queryset = queryset[queryset["org_info_id"] == int(org_info_id)]

        if is_up is not None:
            queryset = queryset[queryset["is_up"] == int(is_up)]

        if user_name__contains:
            queryset = queryset[queryset["user_name"].str.contains(user_name__contains)]

        if user_account__contains:
            queryset = queryset[queryset["user_account"].str.contains(user_account__contains)]

        if online_status is not None:
            queryset = queryset[queryset["online_status"] == int(online_status)]

        if apply_status is not None:
            queryset = queryset[queryset["apply_status"] == int(apply_status)]

        return queryset.copy()

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def policy_exe_details(self, request, *args, **kwargs):
        """获取策略执行状态"""
        policy_id = self.request.query_params.get('policy_id')
        policy_items = models.PatchPolicy.objects.all()
        if not policy_items.filter(id=policy_id).exists():
            return Response('policy_id error', status=status.HTTP_400_BAD_REQUEST)
        policy_obj = policy_items.get(id=policy_id)
        patch_id_list = policy_obj.patch_ids.split(',')
        devices_list = PatchPolicySerializer().mid_install_status(policy_obj)
        # logging.info(f'device list is  {devices_list}')
        res_item = Devices.objects.filter(id__in=devices_list).exclude(is_domain=0).values(
            'id', 'device_cnname', 'device_ip', 'org_info_id', 'is_up', 'device_uuid'
        )

        def apply_status_info(x):
            if self.queryset.filter(device_id=x, patch_id__in=patch_id_list).exists():
                if self.queryset.filter(device_id=x, patch_id__in=patch_id_list).filter(install_status=0).exists():
                    return_code = 0
                else:
                    return_code = 1
            else:
                return_code = 0
            return return_code

        df = pd.DataFrame(res_item)
        if len(df) != 0:
            organ_items = OrgnizationEntity.objects.all()
            user_id_list = policy_obj.range_value.split(',') if policy_obj.range_type == 3 else None
            df['user_name'] = df['id'].apply(lambda x: self._user_name_(x, user_id_list))
            df['user_account'] = df['id'].apply(lambda x: self._user_account_(x, user_id_list))
            df['org_info_name'] = df['org_info_id'].apply(
                lambda x: organ_items.get(id=x).name if organ_items.filter(id=x).exists() else None)
            df['online_status'] = df['device_uuid'].apply(lambda x: 1 if conn.hget('actives', x) == b'1' else 0)
            df['install_time'] = df['id'].apply(
                lambda x: self.queryset.filter(device_id=x, patch_id__in=patch_id_list).order_by(
                    '-install_time').first().install_time.strftime('%Y-%m-%d %H:%M:%S') if self.queryset.filter(
                    device_id=x, patch_id__in=patch_id_list).exists() else None)
            df['apply_status'] = df['id'].apply(lambda x: apply_status_info(x))
            df['install_status'] = None
            df['install_err_msg'] = None
            if len(patch_id_list) == 1:
                df['install_status'] = df['id'].apply(
                    lambda x: self.queryset.get(device_id=x, patch_id__in=patch_id_list).install_status
                    if self.queryset.filter(device_id=x, patch_id__in=patch_id_list).exists() else 0)
                df['install_err_msg'] = df['id'].apply(
                    lambda x: self.queryset.get(device_id=x, patch_id__in=patch_id_list).install_err_msg
                    if self.queryset.filter(device_id=x, patch_id__in=patch_id_list).exists() else None)

            cache_df = self.filter_patch_details_df(df)
        else:
            cache_df = pd.DataFrame()

        current = self.request.query_params.get('current', default=1)
        page_size = self.request.query_params.get('pageSize', default=10)
        # 进行分页
        struct_data = self._page_method_(cache_df, current, page_size)

        return Response(struct_data, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def policy_exe_totals(self, request, *args, **kwargs):
        policy_id = self.request.query_params.get('policy_id')
        policy_items = models.PatchPolicy.objects.all()
        if not policy_items.filter(id=policy_id).exists():
            return Response('policy_id error', status=status.HTTP_400_BAD_REQUEST)
        policy_obj = policy_items.get(id=policy_id)
        patch_id_list = policy_obj.patch_ids.split(',')
        devices_list = PatchPolicySerializer().mid_install_status(policy_obj)
        queryset_items = self.queryset.filter(device_id__in=devices_list, patch_id__in=patch_id_list)
        apply_totals = queryset_items.values('device_id').distinct().count()
        apply_false_count = queryset_items.filter(install_status=0).values('device_id').distinct().count()
        apply_success_count = apply_totals - apply_false_count

        res_data = {
            "apply_totals": apply_totals,
            "apply_success_count": apply_success_count,
            "apply_false_count": apply_false_count
        }

        return Response(res_data, status=status.HTTP_200_OK)


# 策略补丁View
class PatchPolicyViewSet(viewsets.ModelViewSet):
    queryset = models.PatchPolicy.objects.all()
    serializer_class = serializers.PatchPolicySerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = filter.PatchPolicyFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__'
    ordering = ('-update_time',)

    def filter_queryset(self, queryset):
        queryset = super(PatchPolicyViewSet, self).filter_queryset(self.queryset)
        user_data = self.request.user
        if user_data.is_superuser != 1:
            queryset = queryset.filter(creator=user_data.username)
        return queryset

    def update_device_range(self):
        """动态变更设备选择范围"""
        for item in self.queryset.values():
            device_range_value = PatchTask().get_device_list(row=item)
            self.queryset.filter(id=item['id']).update(device_range_value=device_range_value)

    @staticmethod
    def clear_patch_device_log():
        """清除设备补丁安装记录"""
        duplicates = models.PatchDeviceLog.objects.values('patch_id', 'device_id').annotate(count=Count('id')).filter(
            count__gt=1)

        for duplicate in duplicates:
            latest_record = models.PatchDeviceLog.objects.filter(patch_id=duplicate['patch_id'],
                                                                 device_id=duplicate['device_id']) \
                .order_by('-install_time').first()
            models.PatchDeviceLog.objects.exclude(pk=latest_record.pk).filter(patch_id=duplicate['patch_id'],
                                                                              device_id=duplicate['device_id']).delete()

    @staticmethod
    def _change_item_(item):
        device_range_value = PatchTask().get_device_list(row=item)
        item['device_range_value'] = device_range_value
        item['serial_number'] = datetime.datetime.today().strftime("%Y%m%d%H%M%S")
        return item

    @classmethod
    def _rsync_db_redis(cls):
        """
        将数据库策略同步到redis中
        :return:
        """
        df = pd.DataFrame(cls.queryset.values())
        if len(df) >= 1:
            df.replace({pd.NaT: None}, inplace=True)
            df['update_time'] = df['update_time'].apply(
                lambda x: x.strftime('%Y-%m-%d %H:%M:%S') if x is not None else None)
            df['execution_time'] = df['execution_time'].apply(
                lambda x: x.strftime('%Y-%m-%d %H:%M:%S') if x is not None else None)
            df['effective_date'] = df['effective_date'].apply(
                lambda x: x.strftime('%Y-%m-%d %H:%M:%S') if x is not None else None)
            df['expiration_date'] = df['expiration_date'].apply(
                lambda x: x.strftime('%Y-%m-%d %H:%M:%S') if x is not None else None)
            json_df = json.dumps(df.to_dict('records'), ensure_ascii=False)
        else:
            json_df = json.dumps(None, ensure_ascii=False)

        conn.hset('patch_db_cache', 'patch_policy', json_df)

        thread1 = threading.Thread(target=patch_task)  # 触发策略计算
        thread1.start()
        close_old_connections()

    def list(self, request, *args, **kwargs):
        # 更新范围
        self.update_device_range()

        queryset = self.filter_queryset(self.get_queryset())

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    def create(self, request, *args, **kwargs):
        if not models.PatchSource.objects.filter(patch_source_enable=1).exists():
            return Response('未添加补丁源信息，不能创建策略', status=status.HTTP_403_FORBIDDEN)
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        serializer = self.get_serializer(data=item)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)

        thread1 = threading.Thread(target=self._rsync_db_redis)  # 将数据同步到缓存中
        thread1.start()

        return Response('数据创建完成', status=status.HTTP_201_CREATED)

    def update(self, request, *args, **kwargs):
        if not models.PatchSource.objects.filter(patch_source_enable=1).exists():
            return Response('未添加补丁源信息，不能修改策略', status=status.HTTP_403_FORBIDDEN)
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        thread1 = threading.Thread(target=self._rsync_db_redis)  # 将数据同步到缓存中
        thread1.start()

        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        self.perform_destroy(instance)

        thread1 = threading.Thread(target=self._rsync_db_redis)  # 将数据同步到缓存中
        thread1.start()

        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def duplicate(self, request, *args, **kwargs):
        """
        复制策略
        :param request:
        :param args:
        :param kwargs:
        :return: status.HTTP_200_OK
        """
        if not models.PatchSource.objects.filter(patch_source_enable=1).exists():
            return Response('未添加补丁源信息，不能复制策略', status=status.HTTP_403_FORBIDDEN)
        try:
            policy_id = self.request.query_params['id']
        except Exception as e:
            return Response("缺少要复制的ID", status=status.HTTP_400_BAD_REQUEST)
        try:
            policyset = models.PatchPolicy.objects.get(pk=policy_id)
        except Exception as e:
            return Response("没有找到要复制的策略信息", status=status.HTTP_400_BAD_REQUEST)

        policyset_ = models.PatchPolicy()

        policyset_.name = policyset.name
        policyset_.range_type = policyset.range_type
        policyset_.range_value = policyset.range_value
        policyset_.device_range_value = policyset.device_range_value
        policyset_.exception_range_type = policyset.exception_range_type
        policyset_.exception_range_value = policyset.exception_range_value
        policyset_.exception_switch_type = policyset.exception_switch_type
        policyset_.status = 0  # 默认禁用
        policyset_.patch_ids = policyset.patch_ids
        policyset_.creator = request.user.username
        policyset_.remark = policyset.remark

        policyset_.save()
        return Response('复制完成', status=status.HTTP_201_CREATED)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """批量操作，启用，禁用， 删除"""
        is_all = self.request.data.get('is_all')
        is_delete = self.request.data.get('is_delete')
        is_status = self.request.data.get('is_status')
        policy_ids_str = self.request.data.get('ids_list')

        if is_all == 0:
            policy_ids_list = policy_ids_str.split(',')
            queryset = self.queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            queryset = self.filter_queryset(self.get_queryset())
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if is_delete == 1:
            queryset.delete()

        if is_status == 1:
            if not models.PatchSource.objects.filter(patch_source_enable=1).exists():
                return Response('未添加补丁源信息，不能启用策略', status=status.HTTP_403_FORBIDDEN)
            queryset.update(status=1)
        elif is_status == 0:
            queryset.update(status=0)
        else:
            pass

        thread1 = threading.Thread(target=self._rsync_db_redis)  # 将数据同步到缓存中
        thread1.start()

        return Response("修改完成", status=status.HTTP_200_OK)


class PatchDeliverySettingsViewSet(mixins.RetrieveModelMixin,
                                   mixins.UpdateModelMixin,
                                   GenericViewSet):
    queryset = models.PatchDeliverySettings.objects.all()
    serializer_class = serializers.PatchDeliverySettingsSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
