import json
import logging

from accounts import freeipa
from accounts.models import AccountEntity
from celery_task.celery import app
from policy.utils.translators import translate_shell_policy, translate_remote_assist_policy, \
    translate_desktop_theme_policy, translate_password_rule_policy
from remote_assist.service import remote_assist_policy_to_device

from saltapi.tasks import browser_policy, shell_policy, usb_policy, system_activation_policy, watermark_policy, \
    wallpaper_policy, device_system_updates_policy, desktop_theme_policy, root_policy, \
    password_rule_policy, device_screen_saver_policy

from devices.serializers import DeviceSerializer
from devices.models import Devices
from policy.models import Gpo
from policy.serializers import  PolicyGpoPolicySerializer
from policy.utils.translators import translate_browser_policies
from policy.utils.agent_utils import get_agent_policies
from policy.utils.translators2 import translate_systemconfig_policy, translate_systemtools_policy, \
                translate_peripheral_policy, translate_root_policy

logger = logging.getLogger(__name__)

@app.task(queue='default')
def trigger_gpo_update(user_ids=None, device_ids=None, args=None):
    if device_ids is None:
        device_ids = []
    if user_ids is None:
        user_ids = []
    if args is None:
        args = {}
    agents = [*map(lambda x: {'id': x, 'type': Gpo.GpoType.USER}, user_ids), *map(lambda x: {'id': x, 'type': Gpo.GpoType.MACHINE}, device_ids)]
    for agent in agents:
        id = agent['id']
        type = agent['type']
        policies = get_agent_policies(id, type)
        if type == Gpo.GpoType.MACHINE:
            devices_filter = Devices.objects.exclude(is_domain=0).all().filter(id=id)
            devices = DeviceSerializer(devices_filter, many=True).data
            # 浏览器策略下发相关调用
            browser_policies = translate_browser_policies(policies)
            if devices and len(devices):
                res = browser_policy.delay(
                        devices,  # celery参数不能是query_set类型
                        {'policies': browser_policies, 'type': 'machine'},
                        **args
                    )
            # 脚本策略下发
            shell_policies = translate_shell_policy(policies)
            if shell_policies and devices and len(devices):
                res = shell_policy.delay(
                    devices,
                    shell_policies,
                    **args
                )
            # root 策略下发
            """
            root_policies:{
                'enable':'true'/'false'
            }
            """
            root_policies = translate_root_policy(policies)
            if root_policies and devices and len(devices):
                res = root_policy.delay(devices,root_policies,**args)
            # 主题策略
            desktop_theme_policies = translate_desktop_theme_policy(policies)
            if desktop_theme_policies and devices and len(devices):
                res = desktop_theme_policy.delay(
                    devices,
                    desktop_theme_policies,
                    **args
                )

            # 外设管控,系统工具,系统设置项 策略下发
            if devices and len(devices):
                # 外设管控--相关策略下发调用
                res = handler_peripheral_policy(devices, policies)
                # 系统工具--相关策略下发调用
                res = handler_systemtools_policy(devices, policies)
                # 系统设置项--相关策略下发调用
                res = handler_systemconfig_policy(devices, policies)

            # 密码策略
            password_rule_policies = translate_password_rule_policy(policies)
            if password_rule_policies and devices and len(devices):
                password_rule_policy.delay(
                    devices,
                    password_rule_policies,
                    **args
                )
            # 远程策略下发
            remote_assist_policies = translate_remote_assist_policy(policies)
            if remote_assist_policies and devices and len(devices):
                for d in devices:
                    dd = Devices.objects.get(id=d['id'])
                    remote_assist_policy_to_device(dd, remote_assist_policies)
        # elif type == Gpo.GpoType.USER:
        #     devices_filter = Devices.objects.exclude(is_domain=0).all().filter(bind_account=id)
        #     browser_policies = translate_browser_policies(policies)
        #     devices = DeviceSerializer(devices_filter, many=True).data
        #     if devices and len(devices):
        #         res = browser_policy.delay(
        #                 devices,  # celery参数不能是query_set类型
        #                 {'policies': browser_policies, 'users': [id], 'type': 'user'},
        #                 **args
        #             )

    pass

def handler_peripheral_policy(devices, policies):
    """
    终端级别策略:外设管控--usb策略:
    gpo_policy:[
        {
            'path': '/policies/system/peripheral/usb',
            'value': true/false
        }
    """
    # 获取策略配置的配置项,没有就不处理
    peripheral_policies = translate_peripheral_policy(policies)
    # logger.info(f"machine.systemtools_policy.peripheral_policies==={peripheral_policies}")
    if not peripheral_policies:
        return None
    res = None
    # USB管控
    peripheral_usb_policy = peripheral_policies.get('usb')
    if peripheral_usb_policy is not None:
        res = usb_policy.delay(
            devices,
            {"usb": peripheral_usb_policy.get('enable'), "usb_rw_status": ""},
        )
    return res

def handler_systemtools_policy(devices, policies):
    """
    终端级别策略:系统工具
    1,系统激活策略
    2,水印策略
    """
    # 获取策略配置的配置项,没有就不处理
    sys_tools_policies = translate_systemtools_policy(policies)
    #logger.info(f"machine.systemtools_policy.sys_tools_policies==={sys_tools_policies}")
    if not sys_tools_policies:
        return None
    res = None
    # 系统激活策略
    sys_activation_policy = sys_tools_policies.get('activation')
    if sys_activation_policy is not None:
        sys_active = sys_activation_policy.get('active')
        if sys_active == 1:
            kms_server = sys_tools_policies.get('kms_server')
            res = system_activation_policy.delay(
                devices,
                kms_server,
                singal=False
            )
    # 水印策略
    sys_watermark_policy = sys_tools_policies.get('watermark')
    if sys_watermark_policy is not None:
        res = watermark_policy.delay(
            devices,
            sys_watermark_policy
        )
    return res

def handler_systemconfig_policy(devices, policies):
    """
    终端级别策略:系统配置项
    1,背景(壁纸)策略
    2,锁屏(屏幕保护)策略
    4,系统更新策略
    (主题、密码、远程策略已被处理)
    """
    # 获取策略配置的配置项,没有就不处理
    sys_conf_policies = translate_systemconfig_policy(policies)
    #logger.info(f"machine.systemtools_policy.sys_conf_policies==={sys_conf_policies}")
    if not sys_conf_policies:
        return None
    res = None
    # 背景(壁纸)策略
    sys_wallpaper_policy = sys_conf_policies.get('wallpaper')
    if sys_wallpaper_policy is not None:
        wallpaper_id = sys_wallpaper_policy.get('wallpaper_id')
        wallpaper_active = sys_wallpaper_policy.get('active')
        # 壁纸必须存在
        if wallpaper_active == 1 and int(wallpaper_id) > 0:
            res = wallpaper_policy.delay(
                devices,
                wallpaper_id
            )
    # 锁屏策略
    sys_screensavers_policy = sys_conf_policies.get('screensavers')
    if sys_screensavers_policy is not None:
        res = device_screen_saver_policy.delay(
            devices,
            sys_screensavers_policy,
            singal=False
        )

    # 系统更新策略
    sys_updates_policy = sys_conf_policies.get('sys_updates')
    if sys_updates_policy is not None:
        res = device_system_updates_policy.delay(
            devices,
            sys_updates_policy,
            singal=False
        )

    return res
