import logging
from functools import cmp_to_key

from organizations.views import get_all_parent, pidIsInSubs
from devices.models import Devices
from policy.models import Gpo, Ou, OuFilter
from groups.models import  GroupsDetail
from accounts.models import AccountEntity
from users.models import User

logger = logging.getLogger(__name__)

def get_agent_policies(id, type):
    agent=None
    if type == Gpo.GpoType.USER:
        agent= AccountEntity.objects.filter(id=id).prefetch_related("org_info").first()
    elif type == Gpo.GpoType.MACHINE:
        agent = Devices.objects.filter(id=id).prefetch_related("org_info").first()
    depts = []
    group_groups = []
    if agent and agent.org_info and agent.org_info.id:
        depts = get_all_parent(agent.org_info.id)
        group_groups = GroupsDetail.objects.filter(org_id=agent.org_info.id).all()
    agent_groups = None
    if type == Gpo.GpoType.USER:
        agent_groups = GroupsDetail.objects.filter(account_id=id).all()
    elif type == Gpo.GpoType.MACHINE:
        agent_groups = GroupsDetail.objects.filter(device_id=id).all()
    agent_groups = [*agent_groups, *group_groups]

    dept_filters = OuFilter.objects.filter(object_type=OuFilter.ObjectTypeChoices.DEPARTMENT,object_id__in=map(lambda x: x['id'], depts)).all()
    user_filters = OuFilter.objects.filter(object_type=OuFilter.ObjectTypeChoices.USER,object_id=id).all() if type == Gpo.GpoType.USER else []
    machine_filters = OuFilter.objects.filter(object_type=OuFilter.ObjectTypeChoices.DEVICE,object_id=id).all() if type == Gpo.GpoType.MACHINE else []
    agent_group_filters = OuFilter.objects.filter(object_type=OuFilter.ObjectTypeChoices.DEVICE_GROUP).filter(object_id__in=map(lambda x: x.id, agent_groups)).all()
    
    ou_filters = list({*dept_filters, *user_filters, *machine_filters, *agent_group_filters})

    def cmp_ou(x: Ou, y: Ou):
        # 暂时只将单一部门策略提出处
        def is_pure_dept_ou(ou: Ou):
            filters = ou.filters.all()
            if len(filters) != 1:
                return False
            return filters[0].object_type == OuFilter.ObjectTypeChoices.DEPARTMENT
        def is_pure_machine_ou(ou: Ou):
            filters = ou.filters.all()
            if len(filters) != 1:
                return False
            return filters[0].object_type == OuFilter.ObjectTypeChoices.DEVICE
        def is_pure_user_ou(ou: Ou):
            filters = ou.filters.all()
            if len(filters) != 1:
                return False
            return filters[0].object_type == OuFilter.ObjectTypeChoices.USER
        
        if x.is_inheritable != y.is_inheritable:
            return x.is_inheritable > y.is_inheritable
        inheritable_flag = 1 if x.is_inheritable else -1
        if is_pure_dept_ou(x) and is_pure_dept_ou(y):
            fx = x.filters.all()[0]
            fy = y.filters.all()[0]
            if pidIsInSubs(fx.id, fy.id): return inheritable_flag
            if pidIsInSubs(fy.id, fx.id): return -inheritable_flag
        # 纯粹单一部门/设备/用户策略 优先级为(强制时高于其他策略，非强制时低于其他策略)
        if is_pure_user_ou(x) != is_pure_user_ou(y): return inheritable_flag if is_pure_user_ou(x) else -inheritable_flag
        if is_pure_machine_ou(x) != is_pure_machine_ou(y): return inheritable_flag if is_pure_machine_ou(x) else -inheritable_flag
        if is_pure_dept_ou(x): return inheritable_flag
        if is_pure_dept_ou(y): return -inheritable_flag
        # 按策略创建人比较
        ux = User.objects.filter(username=x.creator).first()
        uy = User.objects.filter(username=y.creator).first()
        if ux and uy:
            if ux.is_superuser != uy.is_superuser:
                return inheritable_flag if ux.is_superuser else -inheritable_flag
            # TODO: 按许可范围比较逻辑
        # 获取创建用户所在部门，比较逻辑
        return (x.create_time - y.create_time).total_seconds() * inheritable_flag
        
    def cmp_gpo(x: Gpo, y: Gpo):
        return cmp_ou(x.ou, y.ou)

    # 按优先级从大到小
    ous = set()
    for f in ou_filters:
        if f.ou not in ous and f.ou is not None:
            ous.add(f.ou)
    ous = list(ous)
    ous.sort(key=cmp_to_key(cmp_ou))
    gpos = []
    for ou in ous:
        for gpo in ou.gpos.all():
            if gpo not in gpos:
                gpos.append(gpo)
    policies = {}
    for gpo in gpos:
        for policy in gpo.policies.all():
            if policy.path not in policies and gpo.status == 1:
                policies[policy.path] = policy
    return list(policies.values())

def get_ou_affected_agent_ids(ou):
    logger.info("======get_ou_affected_agent_ids start======")
    logger.info(f"ou_id: {ou.id}")
    if ou is None:
        return {
        'users': [],
        'devices': []
    }
    filters = OuFilter.objects.filter(ou__id=ou.id)
    if not filters or len(filters) == 0:
        pass
    user_plus = set()
    device_plus = set()
    user_delete = set()
    device_delete = set()
    for f in filters:
        us = []
        ds = []
        if f.object_type == OuFilter.ObjectTypeChoices.USER:
            us.append(f.object_id)
        elif f.object_type == OuFilter.ObjectTypeChoices.DEVICE:
            ds.append(f.object_id)
        elif f.object_type == OuFilter.ObjectTypeChoices.USER_GROUP or f.object_type == OuFilter.ObjectTypeChoices.DEVICE_GROUP:
            agent_group_items = GroupsDetail.objects.filter(groups_id=f.object_id).all()
            for agent_group_item in agent_group_items:
                if agent_group_item.range == 0:
                    org_ids = [agent_group_item.org_id]
                    for u in AccountEntity.objects.filter(org_info_id__in=org_ids).all():
                        us.append(u.id)
                    for d in Devices.objects.filter(org_info__in=org_ids).all():
                        ds.append(d.id)
                elif agent_group_item.range == 1:
                    ds.append(agent_group_item.device_id)
                elif agent_group_item.range == 2:
                    # agent_group, not impl
                    pass
                elif agent_group_item.range == 3:
                    us.append(agent_group_item.account_id)
        elif f.object_type == OuFilter.ObjectTypeChoices.DEPARTMENT:
            org_ids =[f.object_id]
            for u in AccountEntity.objects.filter(org_info_id__in=org_ids).all():
                us.append(u.id)
            for d in Devices.objects.filter(org_info__in=org_ids).all():
                ds.append(d.id)

        if f.expression == OuFilter.ExpressionChoices.AND:
            for u in us:
                user_plus.add(u)
            for d in ds:
                device_plus.add(d)
        elif f.expression == OuFilter.ExpressionChoices.NOT:
            for u in us:
                user_delete.add(u)
            for d in ds:
                device_delete.add(d)

    for u in user_delete:
        if u in user_plus:
            user_plus.remove(u)
    for d in device_delete:
        if d in device_plus:
            device_plus.remove(d)

    return {
        'users': list(user_plus),
        'devices': list(device_plus)
    }
