import json
import logging
import os
import uuid

from nfs_domain_mainserver import settings
from saltapi.models import Shell
logger = logging.getLogger(__name__)

NFS_BROWSER_USER_POLICY_PATH_PREFIX = '/policies/user/software/nfs-browser/'
NFS_BROWSER_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/software/nfs-browser/'
homepage_type_map = {
    "restore": 1,
    "customize": 4,
    "blank": 4,
    "newtab": 5,
    "restore_and_customize": 6
}

def translate_browser_policies(policies, policy_type ='system'):
    res = {}
    if policy_type == 'user':
        path = NFS_BROWSER_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = NFS_BROWSER_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')

    for policy in policies:
        if not policy.path.startswith(path):
            continue
        key = policy.path[len(path):]
        value = json.loads(policy.value)
        if key.startswith('advanced'):
            key = key[len('advanced/'):]
            if key == 'disable_copy':
                if value == True:
                    res['CopyPreventionSettings'] = {
                        "enable": ["*"],
                        "disable": [],
                        "minimum_data_size": 0
                        }
            elif key == 'disable_view_source':
                if value == True:
                    res["URLBlocklist"] = ["view-source:*"]
            elif key == 'disable_download':
                if value == True:
                    res["DownloadRestrictions"] = 3
            elif key == 'disable_developer_tool':
                if value == True:
                    res["DeveloperToolsAvailability"] = 2
            elif key == 'disable_printing':
                if value == True:
                    res["PrintingEnabled"] = False
            elif key == 'ntp/applications':
                if value is None: value = []
                res["NTPUseMostVisitedTiles"] = False
                res["CustomLinksList"] = value
            elif key == 'bookmark/list':
                if "ManagedBookmarks" not in res:
                    res["ManagedBookmarks"] = value
                else:
                    res["ManagedBookmarks"] = [*res["ManagedBookmarks"], *value]
            elif key == 'bookmark/toplevel_name':
                if "ManagedBookmarks" not in res:
                    res["ManagedBookmarks"] = [{"toplevel_name": value}]
                else:
                    res["ManagedBookmarks"] = [*res["ManagedBookmarks"], {"toplevel_name": value}]
        elif key.startswith('you'):
            key = key[len('you/'):]
            if key == 'auto_suggest':
                res["SearchSuggestEnabled"] = value
            elif key == 'anonymous_collection':
                res["UrlKeyedAnonymizedDataCollectionEnabled"] = value
            elif key == "background_color":
                res["BrowserThemeColor"] = value
        elif key.startswith('privacy_and_security'):
            key = key[len('privacy_and_security/'):]
            if key == 'cookies':
                if value == "all":
                    res["DefaultCookiesSetting"] = 1
                    res["BlockThirdPartyCookies"] = False
                elif value == "block_thirdparty":
                    res["DefaultCookiesSetting"] = 1
                    res["BlockThirdPartyCookies"] = True
                elif value == "block":
                    res["DefaultCookiesSetting"] = 2
                    res["BlockThirdPartyCookies"] = True
            if key == 'clean_cookies_data_when_close':
                if value == True:
                    res["SyncDisabled"] = True
                    res["ClearBrowsingDataOnExitList"] = ["cookies_and_other_site_data"]
            if key == 'preload':
                if value == True:
                    res["NetworkPredictionOptions"] = 0
                elif value == False:
                    res["NetworkPredictionOptions"] = 2
            if key == 'cookie_allow_urls':
                res["CookiesAllowedForUrls"] = value
            if key == 'cookie_deny_urls':
                res["CookiesBlockedForUrls"] = value
            elif key.startswith('security'):
                key = key[len('security/'):]
                if key == 'position':
                    if value == True:
                        res["DefaultGeolocationSetting"] = 1
                    elif value == False:
                        res["DefaultGeolocationSetting"] = 2
                if key == 'cam':
                    res["VideoCaptureAllowed"] = value
            elif key.startswith('premission'):
                key = key[len('premission/'):]
                if key == 'mic':
                    res["AudioCaptureAllowed"] = value
                if key == 'sensors':
                    if value == True:
                        res["DefaultSensorsSetting"] = 1
                    elif value == False:
                        res["DefaultSensorsSetting"] = 2
                if key == 'usb':
                    if value == True:
                        res["DefaultWebUsbGuardSetting"] = 3
                    elif value == False:
                        res["DefaultWebUsbGuardSetting"] = 2
                if key == 'serial':
                    if value == True:
                        res["DefaultSerialGuardSetting"] = 3
                    elif value == False:
                        res["DefaultSerialGuardSetting"] = 2
                if key == 'hid':
                    if value == True:
                        res["DefaultWebHidGuardSetting"] = 3
                    elif value == False:
                        res["DefaultWebHidGuardSetting"] = 2
                if key == 'notify':
                    if value == True:
                        res["DefaultNotificationsSetting"] = 1
                    elif value == False:
                        res["DefaultNotificationsSetting"] = 2
            elif key.startswith('content'):
                key = key[len('content/'):]
                if key == 'javascript':
                    if value == True:
                        res["DefaultJavaScriptSetting"] = 1
                    elif value == False:
                        res["DefaultJavaScriptSetting"] = 2
                # if key == 'pdf':
                #     res[""] = value
                if key == 'image':
                    if value == True:
                        res["DefaultImagesSetting"] = 1
                    elif value == False:
                        res["DefaultImagesSetting"] = 2
        elif key.startswith('startup'):
            key = key[len('startup/'):]
            if key.startswith('homepage'):
                key = key[len('homepage/'):]
                if key == 'type':
                    if value in homepage_type_map:
                        res["RestoreOnStartup"] = homepage_type_map[value]
                    if value == "blank":
                        res["RestoreOnStartupURLs"] = ['about:blank']
                elif key == 'urls':
                    if value is None: value = []
                    homepage_urls = value
                    res["RestoreOnStartupURLs"] = homepage_urls
        elif key.startswith('download'):
            key = key[len('download/'):]
            if key == 'location':
                # res["DefaultDownloadDirectory"] = value
                res["DownloadDirectory"] = value
            if key == 'ask_location':
                res["PromptForDownloadLocation"] = value
        elif key.startswith('system'):
            key = key[len('system/'):]
            if key == 'background_running':
                res["BackgroundModeEnabled"] = value
            if key == 'hardware_allercation':
                res["HardwareAccelerationModeEnabled"] = value
            # if key == 'hardware_decode':
            #     res["DeviceHardwareVideoDecodingEnabled"] = value
        elif key.startswith('appearence'):
            key = key[len('appearence/'):]
            if key == 'show_bookmark_bar':
                res["BookmarkBarEnabled"] = value
            if key == 'show_home_button':
                res["ShowHomeButton"] = value
            # if key == 'scale':
            #     res["DeviceDisplayResolution"] = value
            if key == 'set_default_browser':
                res["DefaultBrowserSettingEnabled"] = value


    return res


SHELL_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/management/shell/'
SHELL_USER_POLICY_PATH_PREFIX = '/policies/user/management/shell/'

def translate_shell_policy(policies, policy_type ='system'):
    if policy_type == 'user':
        path = SHELL_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = SHELL_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')

    p = {}
    for policy in policies:
        if not policy.path.startswith(path):
            continue
        p[policy.path[len(path):]] = json.loads(policy.value)

    # if p.get('shell_switch', 0) != 1:
    #     return None
    if p.get('shell_type') == 1:
        if 'file_content' in p:
            path = os.path.join('shell_policy', uuid.uuid4().hex, 'custom_shell.sh')
            p['shell_path'] = path
            shell_upload_path = os.path.join(settings.MEDIA_ROOT, path)
            if not os.path.exists(os.path.dirname(shell_upload_path)):
                os.makedirs(os.path.dirname(shell_upload_path))
            with open(shell_upload_path, 'w+') as dest:
                dest.write(p['file_content'])
            p.pop('file_content')
    elif p.get('shell_type') == 0:
        if 'shell_id' in p:
            p['shell_path'] = Shell.objects.get(id=p['shell_id']).file_path
    if 'shell_path' not in p or not p['shell_path'] or str.isspace(p['shell_path']):
        logger.warning("Shell policy: shell path cannot get or calculate, file_content or shell_id probably missing")
        return {}

    return {
        'file_path': p['shell_path']
    }

REMOTE_ASSIST_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/systemconfig/remote_assist/'
REMOTE_ASSIST_USER_POLICY_PATH_PREFIX = '/policies/user/systemconfig/remote_assist/'

def translate_remote_assist_policy(policies, policy_type ='system'):
    if policy_type == 'user':
        path = REMOTE_ASSIST_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = REMOTE_ASSIST_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')

    p = {}
    for policy in policies:
        if not policy.path.startswith(path):
            continue
        p[policy.path[len(path):]] = json.loads(policy.value)

    return p

DESKTOP_THEME_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/systemconfig/theme/'
DESKTOP_THEME_USER_POLICY_PATH_PREFIX = '/policies/user/systemconfig/theme/'

def translate_desktop_theme_policy(policies, policy_type ='system'):
    if policy_type == 'user':
        path = DESKTOP_THEME_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = DESKTOP_THEME_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')

    p = {}
    for policy in policies:
        if not policy.path.startswith(path):
            continue
        p[policy.path[len(path):]] = json.loads(policy.value)

    return p

PASSWORD_RULE_SYSTEM_POLICY_PATH_PREFIX = '/policies/system/systemconfig/security/password/'
PASSWORD_RULE_USER_POLICY_PATH_PREFIX = '/policies/user/systemconfig/security/password/'

def translate_password_rule_policy(policies, policy_type ='system'):
    if policy_type == 'user':
        path = PASSWORD_RULE_USER_POLICY_PATH_PREFIX
    elif policy_type == 'system':
        path = PASSWORD_RULE_SYSTEM_POLICY_PATH_PREFIX
    else:
        raise Exception('Wrong policy_type')

    p = {}
    for policy in policies:
        if not policy.path.startswith(path):
            continue
        p[policy.path[len(path):]] = json.loads(policy.value)

    level = p.get('level')
    pam_pwquality = {}
    pam_unix = {}
    if level == 'high':
        pam_pwquality['minlen'] = 15
        pam_pwquality['minclass'] = 3
        pam_pwquality['usercheck'] = 1
        pam_pwquality['dictcheck'] = 1
        pam_unix['remember'] = 3
    elif level == 'medium':
        pam_pwquality['minlen'] = 8
        pam_pwquality['minclass'] = 2
        pam_pwquality['usercheck'] = 1
        pam_unix['remember'] = 1
    elif level == 'low':
        pam_pwquality['minlen'] = 1
    elif level == 'custom':
        pam_pwquality['minlen'] = p['minlen']
        pam_pwquality['minclass'] = p['minclass']
        pam_pwquality['usercheck'] = p['usercheck']
        pam_unix['remember'] = p['remember']
        pam_pwquality['dictcheck'] = p['dictcheck']

    login_defs = {}
    if 'pass_max_days' in p:
        login_defs['PASS_MAX_DAYS'] = p.get('pass_max_days')
    if 'pass_warn_age' in p:
        login_defs['PASS_WARN_AGE'] = p.get('pass_warn_age')

    # if 'retry' in p:
    #     pam_pwquality['retry'] = p.get('retry')
    accountlock = {}
    if 'lock-interval' in p:
        accountlock['lock-interval'] = p.get('lock-interval')
    if 'max-try-times' in p:
        accountlock['max-try-times'] = p.get('max-try-times')
    faillock = {}
    if 'deny' in p:
        faillock['deny'] = p.get('deny')
    if 'unlock_time' in p:
        faillock['unlock_time'] = p.get('unlock_time')

    res = {
        'pam_pwquality': pam_pwquality,
        'pam_unix': pam_unix,
        'login_defs': login_defs,
        'accountlock': accountlock,
        'faillock': faillock
    }
    return res