import datetime
import logging
import json

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import status, viewsets
from rest_framework.decorators import action
from rest_framework.filters import OrderingFilter, SearchFilter
from rest_framework.views import Response

from groups.models import Groups, GroupsDetail
from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.response import FitJSONRenderer

from policy.models import Gpo, Ou, OuFilter, GpoPolicy
from policy.serializers import PolicyGpoSerializer, PolicyGpoPolicySerializer
from policy.utils.agent_utils import get_ou_affected_agent_ids, get_agent_policies
from policy.tasks import trigger_gpo_update

logger = logging.getLogger(__name__)

class GpoViewSet(viewsets.ModelViewSet):
    queryset = Gpo.objects.all()
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    # filterset_class = SoftwarePolicyFilter
    serializer_class = PolicyGpoSerializer
    filterset_fields = '__all__'
    ordering_fields = '__all__'
    ordering = ('-update_time',)
    
    def create(self, request, *args, **kwargs):
        if Gpo.objects.filter(name=request.data['name']):
            return Response('策略名称已存在', status=status.HTTP_400_BAD_REQUEST)
        request.data['creator'] = request.user.username
        request.data['updater'] = request.user.username
        res = super(GpoViewSet, self).create(request, *args, **kwargs)
        gpo = Gpo.objects.get(id=res.data['id'])
        affected_agents = get_ou_affected_agent_ids(gpo.ou)
        logger.info(f"gpo.create.affected_agents:{affected_agents}")
        # regenerate/publish policy
        trigger_gpo_update.delay(affected_agents['users'], affected_agents['devices'])
        return res

    def update(self, request, *args, **kwargs):
        gpo = Gpo.objects.filter(id=request.data['id']).first()
        if request.user.is_superuser == 0 and request.user.username != gpo.creator:
            return Response('您无权限修改其他用户创建的策略组', status=status.HTTP_400_BAD_REQUEST)
        
        item = request.data['updater'] = request.user.username
        raw_affected_agents = get_ou_affected_agent_ids(gpo.ou)
        res = super(GpoViewSet, self).update(request, *args, **kwargs)
        affected_agents = get_ou_affected_agent_ids(gpo.ou)
        affected_agents = {
            'users': list({*raw_affected_agents['users'], *affected_agents['users']}),
            'devices': list({*raw_affected_agents['devices'], *affected_agents['devices']}),
        }
        logger.info(f"gpo.update.affected_agents:{affected_agents}")
        # regenerate/publish policy
        trigger_gpo_update.delay(affected_agents['users'], affected_agents['devices'])
        return res

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """批量操作，启用，禁用， 删除"""
        is_all = self.request.data.get('is_all')
        is_delete = self.request.data.get('is_delete')
        is_status = self.request.data.get('is_status')
        ids_str = self.request.data.get('ids_list')

        if is_all == 0:
            ids_list = ids_str.split(',')
            queryset = self.queryset.filter(id__in=ids_list)
        elif is_all == 1:
            queryset = self.filter_queryset(self.get_queryset())
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if request.user.is_superuser == 0 and queryset.exclude(creator=request.user.username).count() > 0:
            return Response('您无权限修改其他用户创建的策略组', status=status.HTTP_400_BAD_REQUEST)

        ous = []
        for gpo in queryset.all():
            if gpo.ou:
                ous.append(gpo.ou)
        agent_list = []
        for ou in ous:
            agent_list.append(get_ou_affected_agent_ids(ou))
        affected_agents = {
            'users': list(set([item for sublist in map(lambda x: x['users'], agent_list) for item in sublist])),
            'devices': list(set([item for sublist in map(lambda x: x['devices'], agent_list) for item in sublist])),
        }
        logger.info(f"batch_operation.affected_agents:{affected_agents}")

        queryset.update(updater=request.user.username)

        if is_delete == 1:
            queryset.delete()

        if is_status == 1:
            queryset.update(status=1)
        elif is_status == 0:
            queryset.update(status=0)
        else:
            pass

        trigger_gpo_update.delay(affected_agents['users'], affected_agents['devices'])

        return Response("修改完成", status=status.HTTP_200_OK)

