import json
import logging
import time
import configparser
import os

from django.forms import model_to_dict

from devices.serializers import DeviceSerializer
from remote_assist.service import remote_assist_policy_to_device
from remote_assist.utils import get_redis_device_info
from utils.authentication import TokenAuthentication
from utils.throttling import UserRateThrottle
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.decorators import api_view, permission_classes, authentication_classes, action
from rest_framework.views import Response
from rest_framework import status, mixins
from rest_framework.viewsets import ModelViewSet, GenericViewSet
from distutils.version import LooseVersion
from django.core.cache import caches
from django.conf import settings
from django_redis import get_redis_connection

from saltapi.utils import SaltAPI
from utils.response import FitJSONRenderer
from remote_assist.models import RemoteAssist, UserRemoteAssistDevice, RemoteAssistDevice
from remote_assist.serializers import RemoteAssistSerializer, RADeviceSerializer
from utils.pagination import MyPagination
from utils.utils import get_serializer_error
from devices.models import Devices
from devices.utils import Ping
from users.models import User
from devices.client_consumers import msg_to_client
from nfs_domain_mainserver.settings import BASE_DIR
from policy.models import Gpo
from policy.utils.agent_utils import get_agent_policies
from policy.utils.translators import translate_remote_assist_policy

logger = logging.getLogger(__name__)
conn = get_redis_connection('redis')

CONFIG_PATH = os.path.join(BASE_DIR, 'nfs_domain_mainserver/conf_ini/config.ini')

class RemoteAssistViewSet(ModelViewSet):
    queryset = RemoteAssist.objects.all()
    serializer_class = RemoteAssistSerializer
    # pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    # filter_backends = (filters.SearchFilter)
    # filter_fields = ("name", "create_user_name", "exec_status", "policy_status")
    ordering = ('-id',)

    def update(self, request, pk):
        # 查询pk所指定的模型对象
        try:
            remote_assist = RemoteAssist.objects.get(id=pk)
        except remote_assist.DoesNotExist:
            return Response('id 不存在', status=status.HTTP_404_NOT_FOUND)
        # 获取前端传入的请求体数据
        # 创建序列化器进行反序列化
        serializer = RemoteAssistSerializer(instance=remote_assist, data=request.data)
        # 校验
        if serializer.is_valid():
            serializer.save()
            # 响应
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            error_str = get_serializer_error(serializer)
            return Response(f'参数错误: %s' % error_str, status=status.HTTP_400_BAD_REQUEST)


def update_redis_device_info(device, device_info):
    device_hostname = device.device_hostname
    cache = caches["redis"]

    # 直接从用户指定的通道名称构造通道组名称
    host_info = cache.get(settings.REMOET_ASSIST_KEY)
    host_info[device_hostname] = device_info
    cache.set(settings.REMOET_ASSIST_KEY, host_info)
    cache.persist(settings.REMOET_ASSIST_KEY)
    return True


@api_view(["GET"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def check_remote_assist(request):
    device_id = request.query_params.get('device_id')
    websockify_address = os.environ['WEBSOCKIFY_ADDRESS']
    # 设备是否存在
    device = Devices.objects.filter(id=device_id).exclude(is_domain=0).first()
    if not device:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '查询不到设备信息'
        })
    is_active = True if conn.hget('actives', device.device_hostname) == b'1' else False
    # 1、终端是否启用禁用或离线
    if not is_active:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '终端离线,无法远程协助'
        })
    if device.is_up == 0:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '当前终端已禁用,无法远程协助'
        })

    # [ziming]: 2025/01/03，组策略逻辑不写远程配置，不查询原有远程配置表
    # ra = RemoteAssistDevice.objects.filter(device=device).first()
    # if not ra:
    #     ra = RemoteAssist.objects.filter(pk=1).first()
    #     # TODO: 没有获取到策略配置，且默认开启了管控，下发默认配置给客户端
    #     if ra.global_enabled:
    #         ra_dict = model_to_dict(ra)
    #         remote_assist_policy_to_device(device, ra_dict)
    # if not ra.global_enabled or not ra.enabled:
    #     return Response({
    #         'code': status.HTTP_400_BAD_REQUEST,
    #         'msg': '终端远程协助已禁用'
    #     })

    policies = get_agent_policies(device_id, Gpo.GpoType.MACHINE)
    remote_assist_policies = translate_remote_assist_policy(policies)
    if remote_assist_policies:
        if remote_assist_policies.get('global_enabled') == 0 or remote_assist_policies.get('enabled') == 0:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '终端远程协助已禁用'
            })
        # TODO: 连接前下发一次策略，临时解决客户端无用户登录远程策略未生效问题
        remote_assist_policy_to_device(device, remote_assist_policies)

    # # 2、终端是否已经在远程中
    # device_exits_status = UserRemoteAssistDevice.objects.exclude(user=user).filter(device=device, is_delete=0).first()
    # if device_exits_status:
    #     return Response({
    #         'code': status.HTTP_400_BAD_REQUEST,
    #         'msg': '当前终端正在进行远程协助,不支持多个远程协助链接'
    #     })
    #
    # # 3、管理员是否在远程其他终端
    # user_exits_status = UserRemoteAssistDevice.objects.exclude(device=device).filter(user=user, is_delete=0).first()
    # if user_exits_status:
    #     return Response({
    #         'code': status.HTTP_400_BAD_REQUEST,
    #         'msg': '管理员正在远程中,不支持多个远程协助链接'
    #     })

    # 4、客户端版本是否高于1.2
    data_json = {
        "sender": "SERVER",
        "sn": "9-8-1",
        "data": {
            "adminname": request.user.username
        }
    }

    # 直接从用户指定的通道名称构造通道组名称
    channel_group_name = get_redis_device_info(device)
    logger.info("获取缓存中的通道名称 channel_group_name %s" % channel_group_name)

    if not channel_group_name:
        # 无 websocket
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '设备连接不成功,无法远程协助'
        })
    else:
        # 发送消息
        msg_to_client(1, channel_group_name, data_json)

    # 获取消息后，存储到缓存，取缓存
    # 获取缓存中的版本号
    client_version = None
    for i in range(20):
        remote_device_dict = conn.hget(settings.REMOET_ASSIST_KEY, device.device_hostname)
        if remote_device_dict:
            remote_device_dict = remote_device_dict.decode('utf-8')
            remote_device_dict = json.loads(remote_device_dict)
            client_version = remote_device_dict.get("version")
        else:
            time.sleep(3)

        if client_version:
            break

    if not client_version:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '设备终端版本获取不成功,无法远程协助'
        })

    # 最低版本号
    LOWEST_CLIENT_VERSION = "1.0.1-m35"
    if LooseVersion(client_version) < LooseVersion(LOWEST_CLIENT_VERSION):
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '终端客户端版本过低，请确保客户端版本高于V1.2'
        })

    # 5、给客户端发送远程连接规则
    remote_assist = RemoteAssist.objects.filter(pk=1).first()

    # 获取请求路径url
    data_json = {
        "sender": "SERVER",
        "sn": "9-8-5",
        "data": {
            "adminname": request.user.username,
            "op": "start",
            "connectconfirm": remote_assist.argee,
            "sessionidle": remote_assist.sessions,
            "sessionidletimeout": remote_assist.sessions_time,
            "reconnect": remote_assist.concat,
            "reconnectperiod": remote_assist.concat_time,
        }
    }
    msg_to_client(1, channel_group_name, data_json)

    for i in range(20):
        time.sleep(3)
        remote_device_dict = conn.hget(settings.REMOET_ASSIST_KEY, device.device_hostname)
        if remote_device_dict:
            remote_device_dict = remote_device_dict.decode('utf-8')
            remote_device_dict = json.loads(remote_device_dict)
            url = remote_device_dict.get("url")
            errmsg = remote_device_dict.get("errmsg")
            vnc_status = remote_device_dict.get("op")
            if url:
                url = url.replace("127.0.0.1", device.device_ip)
                logger.info("url %s errmsg %s" % (url, errmsg))
                data = {
                    "url": url,
                    "websockify_address": websockify_address
                } 
                return Response({
                    'code': status.HTTP_200_OK,
                    'data': data,
                    'msg': '可以启用远程协助'
                })
            else:
                if vnc_status == "start":
                    if errmsg:
                        return Response({
                            'code': status.HTTP_400_BAD_REQUEST,
                            'msg': "无法链接该终端.错误原因%s" % errmsg
                        })
                    else:
                        pass
                else:
                    pass
        else:
            time.sleep(3)

    return Response({
        'code': status.HTTP_400_BAD_REQUEST,
        'msg': '终端没响应'
    })


@api_view(["GET"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def stop_remote_assist(request):
    device_id = request.query_params.get('device_id')
    user = User.objects.filter(username=request.user.username).first()
    # 设备是否存在
    device = Devices.objects.filter(id=device_id).exclude(is_domain=0).first()
    if not device:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '查询不到设备信息'
        })

    history = UserRemoteAssistDevice.objects.filter(user=user, device=device)
    if history:
        history.delete()

    is_active = True if conn.hget('actives', device.device_hostname) == b'1' else False
    # 1、终端是否启用禁用或离线
    if not is_active:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '终端离线,无法关闭远程协助'
        })

    if device.is_up == 0:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '当前终端已禁用,无法关闭远程协助'
        })

    # 获取缓存中的通道名称
    # 获取缓存中的通道名称
    channel_group_name = get_redis_device_info(device)
    logger.info("channel_group_name %s" % channel_group_name)

    if not channel_group_name:
        # 无 websocket
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '设备连接不成功,无法关闭远程协助'
        })

    remote_assist = RemoteAssist.objects.filter(pk=1).first()

    # 获取请求路径url
    data_json = {
        "sender": "SERVER",
        "sn": "9-8-5",
        "data": {
            "adminname": request.user.username,
            "op": "stop",
            "connectconfirm": remote_assist.argee,
            "sessionidle": remote_assist.sessions,
            "sessionidletimeout": remote_assist.sessions_time,
            "reconnect": remote_assist.concat,
            "reconnectperiod": remote_assist.concat_time,
        }
    }

    msg_to_client(1, channel_group_name, data_json)

    for i in range(3):
        time.sleep(3)
        remote_device_dict = conn.hget(settings.REMOET_ASSIST_KEY, device.device_hostname)
        if remote_device_dict:
            remote_device_dict = remote_device_dict.decode('utf-8')
            remote_device_dict = json.loads(remote_device_dict)
            errmsg = remote_device_dict.get("errmsg")
            vnc_status = remote_device_dict.get("op")
            if vnc_status == 'stop':
                if errmsg:
                    return Response({
                        'code': status.HTTP_400_BAD_REQUEST,
                        'msg': errmsg
                    })
                else:
                    return Response({
                        'code': status.HTTP_200_OK,
                        'msg': '已关闭远程协助'
                    })
            else:
                pass
        else:
            time.sleep(3)

    return Response({
        'code': status.HTTP_400_BAD_REQUEST,
        'msg': "关闭远程连接超时"
    })


class RemoteAssistDeviceViewSet(ModelViewSet):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    queryset = Devices.objects.exclude(is_domain=0).prefetch_related('remote_assist').all()
    serializer_class = RADeviceSerializer
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)

    def update(self, request, *args, **kwargs):
        response = super().update(request, *args, **kwargs)

        instance = self.get_object()
        data = request.data.get('remote_assist', None)
        remote_assist_policy_to_device(instance, data)

        return response

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        instance = serializer.save()
        headers = self.get_success_headers(serializer.data)

        data = request.data.get('remote_assist', None)
        remote_assist_policy_to_device(instance, data)

        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """批量操作系统设置，部门操作"""
        is_all = self.request.data.get('is_all')
        org_id_list = self.request.data.get('org_id_list')
        activepolicy_ids_str = self.request.data.get('ids_list')
        
        global_enabled = self.request.data.get('global_enabled')
        enabled = self.request.data.get('enabled')
        argee = self.request.data.get('argee')
        need_passwd = self.request.data.get('need_passwd')
        passwd = self.request.data.get('passwd')

        # device_id_list = ActivationDevicePolicy.objects.filter(active=1).values_list('device_id', flat=True)

        if is_all == 0:
            device_queryset = self.filter_queryset(self.get_queryset())
            if org_id_list:
                device_queryset = device_queryset.filter(org_info_id__in=org_id_list)
            else:
                policy_ids_list = activepolicy_ids_str.split(',')
                device_queryset = device_queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            device_queryset = Devices.objects.exclude(is_domain=0).all()
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        if not device_queryset:
            return Response("没有符合要求的数据", status=status.HTTP_400_BAD_REQUEST)

        # device_queryset = device_queryset.filter(id__in=device_id_list)
        # if not device_queryset:
        #     return Response('没有选中加域的设备', status=status.HTTP_400_BAD_REQUEST)

        for device in device_queryset.only('id'):
            remote_assist_device_object, _ = RemoteAssistDevice.objects.get_or_create(device_id=device.id)
            remote_assist_device_object.global_enabled = global_enabled
            remote_assist_device_object.enabled = enabled
            remote_assist_device_object.argee = argee
            remote_assist_device_object.need_passwd = need_passwd
            remote_assist_device_object.passwd = passwd
            remote_assist_device_object.save()

            remote_assist_policy_to_device(remote_assist_device_object.device, dict({
                'global_enabled': global_enabled,
                'enabled': enabled, 
                'argee': argee, 
                'need_passwd': need_passwd, 
                'passwd':passwd
            }))

        # system_activation_policy.delay(
        #     DeviceSerializer(device_queryset, many=True).data,  # celery参数不能是query_set类型
        #     kms_server,
        #     singal=False
        # )

        return Response({'msg': '激活策略设置已提交'}, status=status.HTTP_200_OK)

