# -*- coding: utf-8 -*-

import logging
import os
import json

from elasticsearch import Elasticsearch
import pandas as pd
from dateutil.parser import parse
from datetime import datetime

from rest_framework.views import Response
from rest_framework import status, viewsets
from rest_framework.decorators import action

from utils.authentication import TokenAuthentication
from utils.permission import IsAdminUser
from utils.throttling import UserRateThrottle

from devices.models import Devices, DevicesHardware
from devices.serializers import DeviceSerializer

logger = logging.getLogger(__name__)


class ReportsViewSet(viewsets.ModelViewSet):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def hardwarelist(self, request, *args, **kwargs):
        '''
        终端设备硬件详情报表
        '''
        device_cnname = request.data.get('device_cnname')
        device_ip = request.data.get('device_ip')
        device_org_id = request.data.get('device_org_id')

        queryset = DevicesHardware.objects.select_related('device').all()
        if device_ip:
            queryset = queryset.filter(device__device_ip__icontains=device_ip)
        if device_cnname:
            queryset = queryset.filter(device__device_cnname=device_cnname)
        if device_org_id:
            queryset = queryset.filter(device__org_info_id=device_org_id)

        if not queryset.exists():
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': "无数据"
            })

        data = []
        for item in queryset:
            deviceObj = {}
            deviceObj['accounts_id'] = item.device.accounts_id
            deviceObj['device_ip'] = item.device.device_ip
            deviceObj['device_hostname'] = item.device.device_hostname
            deviceObj['device_cnname'] = item.device.device_cnname
            deviceObj['belong_user'] = item.device.belong_user
            deviceObj['org_info'] = item.device.org_info
            deviceObj['add_domain_time'] = item.device.add_domain_time
            deviceOrgInfo = {}
            deviceOrgInfo['org_name'] = item.device.org_info.name
            deviceOrgInfo['org_id'] = item.device.org_info.id
            deviceObj['org_info'] = deviceOrgInfo
            deviceHardwareObj = {}
            deviceHardwareObj['network'] = item.network
            deviceHardwareObj['cpu_model'] = item.cpu_model
            deviceHardwareObj['cpuarch'] = item.cpuarch
            deviceHardwareObj['mainboard'] = item.mainboard
            deviceHardwareObj['kernelrelease'] = item.kernelrelease
            deviceHardwareObj['productname'] = item.productname
            deviceHardwareObj['mem_total'] = item.mem_total
            deviceHardwareObj['os'] = item.os
            deviceHardwareObj['device'] = deviceObj
            data.append(deviceHardwareObj)
        #logger.info(f'data====%s' % data)
        # 返回
        return Response({
            'code': status.HTTP_200_OK,
            'msg':'',
            'data': data
        })

# Elasticsearch数据查询
class SearchEsData():
    def __init__(self,es_index, es_host, es_port, es_request):
        self.es_index = es_index
        self.es_host = es_host
        self.es_port = es_port
        self.es_request = es_request

    def get_device_software(self):
        try:
            # 创建Elasticsearch客户端
            es = Elasticsearch([{'host': self.es_host, 'port': self.es_port, 'scheme': 'http'}])

            # 过滤条件:终端唯一id
            device_ids = self.es_request.data.get('device_id')

            # 构建查询
            query_body = {
                "query": {
                    "term": {
                        "_id": device_ids
                    }
                }
            }
            logger.info(f"query_body==%s" % query_body)
            # 执行搜索
            response = es.search(
                index=self.es_index,
                body=query_body
            )

            # scroll_size这个时候===1
            scroll_size = len(response['hits']['hits'])
            
            if scroll_size <=0 :
                return {
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '没有数据'
                }

            data = []
            for doc in response['hits']['hits']:
                source_data = doc.get('_source',{})
                software_list = source_data.get('software')
                # 如果没有指定的属性,则认为此数据不合法,抛弃掉
                if software_list is None:
                    continue
                
                for software in software_list:
                    data.append(software)

            logger.info(f"data==%s" % data)
            # 返回
            return {
                'code': status.HTTP_200_OK,
                'msg': '',
                'data': data
            }
        except Exception as exp:
            logger.error(f'_export_for_user_domain error %s' % exp)
            return {
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f'查询异常: %s' % str(exp)
            }