# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import
import pprint
import textwrap

from . import nested_output


def output(data, summary=True):
    """
    The HighState Outputter is only meant to be used with the state.highstate
    function, or a function that returns highstate return data.
    """
    # for host, hostdata in data.items():
    return _format_host(data, summary)



def _format_host(data, summary):
    tabular = False
    rcounts = {}
    rdurations = []
    hstrs = []
    nchanges = 0
    num_failed = 0
    out = []

    if isinstance(data, int) or isinstance(data, str):

        # Data in this format is from saltmod.function,
        # so it is always a 'change'
        nchanges = 1
        hstrs.append(("{0}".format(data)))

    if isinstance(data, list):
        # Errors have been detected, list them in RED!
        for err in data:
            hstrs.append(
                ("{0}".format(err))
            )
    if isinstance(data, dict):
        # Verify that the needed data is present
        for tname, info in data.items():
            if isinstance(info, dict) and "__run_num__" not in info:
                err = (
                    "The State execution failed to record the order "
                    "in which all states were executed. The state "
                    "return missing data is:"
                )
                hstrs.insert(0, pprint.pformat(info))
                hstrs.insert(0, err)
        # Everything rendered as it should display the output
        for tname in sorted(data, key=lambda k: data[k].get("__run_num__", 0)):
            ret = data[tname]
            # Increment result counts
            rcounts.setdefault(ret["result"], 0)
            rcounts[ret["result"]] += 1
            rduration = ret.get("duration", 0)
            try:
                rdurations.append(float(rduration))
            except ValueError:
                rduration, _, _ = rduration.partition(" ms")
                try:
                    rdurations.append(float(rduration))
                except ValueError:
                    pass
                    # log.error('Cannot parse a float from duration %s', ret.get('duration', 0))


            schanged, ctext = _format_changes(ret["changes"])
            nchanges += 1 if schanged else 0

            comps = tname.split("_|-")

            # This isn't the prettiest way of doing this, but it's readable.
            retu = {
                'changes': ret["changes"],
                # '__id__': ret["__id__"],
                'comment': ret["comment"]
            }
            out.append(retu)
            for detail in ["start_time", "duration"]:
                ret.setdefault(detail, "")
            if ret["duration"] != "":
                ret["duration"] = "{0} ms".format(ret["duration"])

            changes = "     Changes:   " + ctext
            hstrs.append(("{0}".format(changes)))

            if "warnings" in ret:
                rcounts.setdefault("warnings", 0)
                rcounts["warnings"] += 1
                wrapper = textwrap.TextWrapper(
                    width=80, initial_indent=" " * 14, subsequent_indent=" " * 14
                )
                hstrs.append(
                    "Warnings: {0}".format(
                        wrapper.fill("\n".join(ret["warnings"])).lstrip()
                    )
                )

        # Append result counts to end of output
        colorfmt = "{0}"
        rlabel = {
            True: "Succeeded",
            False: "Failed",
            None: "Not Run",
            "warnings": "Warnings",
        }
        count_max_len = max([len(str(x)) for x in rcounts.items()] or [0])
        label_max_len = max([len(x) for x in rlabel.items()] or [0])
        print(label_max_len)
        line_max_len = label_max_len + count_max_len + 2  # +2 for ': '
        if summary:
            hstrs.append(
                colorfmt.format(
                    "\nSummary\n{0}".format("-" * line_max_len)
                )
            )

            def _counts(label, count):
                return "{0}: {1:>{2}}".format(
                    label, count, line_max_len - (len(label) + 2)
                )

            # Successful states
            changestats = []
            if None in rcounts and rcounts.get(None, 0) > 0:
                # test=True states
                changestats.append(
                    colorfmt.format(
                        "unchanged={0}".format(rcounts.get(None, 0)),
                    )
                )
            if nchanges > 0:
                changestats.append(
                    colorfmt.format(
                        "changed={0}".format(nchanges)
                    )
                )
            if changestats:
                changestats = " ({0})".format(", ".join(changestats))
            else:
                changestats = ""
            hstrs.append(
                colorfmt.format(
                    _counts(rlabel[True], rcounts.get(True, 0) + rcounts.get(None, 0)),
                )
                + changestats
            )

            # Failed states
            num_failed = rcounts.get(False, 0)
            hstrs.append(
                colorfmt.format(
                    _counts(rlabel[False], num_failed),
                )
            )

            num_warnings = rcounts.get("warnings", 0)
            if num_warnings:
                hstrs.append(
                    colorfmt.format(
                        _counts(rlabel["warnings"], num_warnings),
                    )
                )

            totals = "{0}\nTotal states run: {1:>{2}}".format(
                "-" * line_max_len,
                sum(rcounts.values()) - rcounts.get("warnings", 0),
                line_max_len - 7,
            )
            hstrs.append(colorfmt.format(totals))
            sum_duration = sum(rdurations)
            duration_unit = "ms"
            # convert to seconds if duration is 1000ms or more
            if sum_duration > 999:
                sum_duration /= 1000
                duration_unit = "s"
            total_duration = "Total run time: {0} {1}".format(
                "{0:.3f}".format(sum_duration).rjust(line_max_len - 5), duration_unit
            )
    return "\n".join(hstrs), out, num_failed


def _format_changes(changes):
    """
    Format the changes dict based on what the data is
    """

    if not changes:
        return False, ""

    if not isinstance(changes, dict):
        return True, "Invalid Changes data: {0}".format(changes)

    ret = changes.get("ret")
    if ret is not None and changes.get("out") == "highstate":
        ctext = ""
        changed = False
        for host, hostdata in ret.items():
            s, c = _format_host(host, hostdata)
            ctext += "\n" + "\n".join((" " * 14 + l) for l in s.splitlines())
            changed = changed or c
    else:
        changed = True
        ctext = nested_output.output(changes, nested_indent=14)
    # print(changed)
    # print(ctext)
    return changed, ctext