import os
import datetime
import logging
import json
from celery import states
from celery_task.celery import app
from django.dispatch import Signal, receiver
from django.conf import settings
from django.db.models import Q
from accounts.models import AccountEntity
from devices.serializers import DeviceSerializer
from devices.web_consumers import send_msg_to_web
from filemanager.models import FileManager
from jobs_management.models import JobResultInfo
from message_center.models import MessageCenter
from saltapi.models import SSHDevicePolicy, USBDevicePolicy, FirewalldDevicePolicy, WallpaperDevicePolicy, Wallpaper, \
    WaitDevicePolicy, ActivationDevicePolicy, ScreensaversDevicePolicy, Screensavers, \
    FirewalldWhiteBlackRosterDevicePolicy, \
    FirewalldWhiteBlackRosterPolicy, CaDevicePolicy, SaltJidDevice, WaterMarkDevicePolicy, WaterMark, \
    AccountLoginPolicy, DesktopThemePolicy, ScreenSaverPolicy, SystemUpdatesPolicy, FileRedirectionPolicy, \
    TrustedSitesPolicy, BrowserFavoritesPolicy, SoftwareRunsPolicy, LocalUserPWDPolicy, ShellDevicePolicy
from saltapi.utils import DefaultPolicy
from saltapi.output import check_output
from software.models import SoftwareInstalledModel, SoftwareInfoModel, SoftwarePolicy
from tasks.models import TaskDeviceExec, Tasks
from camanager.models import CaInfoModel
from utils.RedisHelper import RedisHelper
from software.utils import update_policy_status
from utils.redis_domain_device import RedisCacheDevice
from utils.rpc_client import SaltRpcClient
from utils.utils import auto_remove_salt_shh, del_roster
from utils.device_user_active import get_device_user_active, update_device_user_active
from sysios.models import SysiosIso
from sysios.utils import SysisoQueue
logger = logging.getLogger(__name__)

policy_singal = Signal(['singnal_type', 'msg'])


# create singnal

def policy_singnal_sender(data):
    singnal_type = 'policy'
    policy_singal.send(sender=policy_singnal_sender, singnal_type=singnal_type, msg=data)


@receiver(policy_singal, sender=policy_singnal_sender)
def policy_singnal_handler(sender, **kwargs):
    '''
    保存因设备不在线导致未执行的  策略
    同一个设备仅保留最新的策略 如： 先开启usb, 再关闭usb ，那么仅保留 关闭usb策略
    '''
    logger.info(f"*****kwargs******{kwargs}********")
    device_id = kwargs['msg']['device_id']
    policy = kwargs['msg']['policy']  # usb_policy
    arg = kwargs['msg']['arg']  # ['device-control.usb_enable']
    if policy in settings.JOB_POLICE:
        # 软件策略全保留
        WaitDevicePolicy.objects.create(device_id=device_id, policy=policy, arg=json.dumps(arg))
    else:
        # 仅保留最新的策略
        device = WaitDevicePolicy.objects.filter(device_id=device_id, policy=policy).first()
        if not device:
            WaitDevicePolicy.objects.create(device_id=device_id, policy=policy, arg=json.dumps(arg))
        else:
            device.arg = arg
            device.save()


# 系统待执行策略
def run_wait_policy(device):
    from saltapi import tasks
    device_policy = WaitDevicePolicy.objects.filter(device_id=device.id).all()
    if not device_policy: return
    for p in device_policy:
        try:
            fun = getattr(tasks, p.policy)
            if fun and p.policy in settings.DEVICE_POLICE:  # 跳过用户级的待执行策略
                fun.delay(
                    [DeviceSerializer(device).data],
                    p.arg,
                    signal=False
                )
            if fun and p.policy in settings.JOB_POLICE:  # 执行软件等待策略，不管成功失败
                # logger.info(f"type : {type(p.arg)}, p.arg : {p.arg}, ")
                arg_string = p.arg
                json_dict = json.loads(arg_string)
                fun.delay(**json_dict)
        except Exception as exp:
            logger.info(f"执行等待策略错误 源数据***{p.device_id},***{p.arg}, 错误信息{exp}")


# 用户待执行策略
def run_user_wait_policy(device):
    from saltapi import tasks
    device_policy = WaitDevicePolicy.objects.filter(device_id=device.id).all()
    if not device_policy: return
    for p in device_policy:
        fun = getattr(tasks, p.policy)
        if fun in settings.USER_POLICY:  # 跳过系统的待执行策略
            fun.delay(
                [DeviceSerializer(device).data],
                p.arg,
                signal=False
            )


# 系统默认策略
def run_default_policy(device):
    """加域后触发的策略"""
    from saltapi import tasks
    device_map = DeviceSerializer(device).data
    default_policy = DefaultPolicy()

    # usb policy
    if not WaitDevicePolicy.objects.filter(device_id=device_map.get('id'), policy='usb_policy').first():
        usb_device, is_create = USBDevicePolicy.objects.get_or_create(device_id=device.id)
        if is_create:
            logger.info('### run_default_policy usb ###')
            usb = int(default_policy.get_default_policy().get('usb'))
            usb_rw_status = default_policy.get_default_policy().get('usb_rw_status')
            usb_device.usb = usb
            usb_device.status = 2
            usb_device.usb_rw_status = usb_rw_status
            usb_device.save()
            # TODO: 执行usb策略

            tasks.usb_policy.delay(
                [device_map],
                {"usb": usb, "usb_rw_status": usb_rw_status},
                signal=False
            )

    # ssh policy
    ssh_switch = default_policy.get_default_policy().get('ssh_default_policy', None)
    if ssh_switch == '1':
        if not WaitDevicePolicy.objects.filter(device_id=device_map.get('id'), policy='ssh_policy').first():
            ssh_device, is_create = SSHDevicePolicy.objects.get_or_create(device_id=device.id)
            if is_create:
                logger.info('### run_default_policy ssh ###')
                ssh = int(default_policy.get_default_policy().get('ssh'))
                ssh_device.ssh = ssh
                ssh_device.status = 2
                ssh_device.save()
                # TODO: 执行ssh策略
                tasks.ssh_policy.delay(
                    [device_map],
                    ssh,
                    signal=False
                )
    else:
        logger.info('SSH策略未打开')
    # ca  policy
    cert_switch = default_policy.get_default_policy().get('cert', None)
    if cert_switch == '1':
        logger.info('准备执行证书下发安装策略')
        cainfos = CaInfoModel.objects.filter(is_deleted=0).filter(is_default_install=1)
        for cainfo in cainfos:
            if not WaitDevicePolicy.objects.filter(device_id=device_map.get('id'), policy='ca_apply_policy').first():
                ca_device, is_create = CaDevicePolicy.objects.get_or_create(device_id=device.id, ca_id=cainfo.id)
                if is_create:
                    logger.info('### run_default_policy CA ###')
                    ca_device.policy_no = 0
                    ca_device.device_id = device_map.get('id')
                    ca_device.ca_id = cainfo.id
                    ca_device.save()
            tasks.ca_apply_policy.delay(device_map,
                                        {'name': cainfo.name, 'custome_filename': cainfo.custome_filename},
                                        signal=False)
    else:
        logger.info('证书下发安装策略未打开')

    # firewalld
    firewalld_default_policy = default_policy.get_default_policy().get('firewalld_default_policy', None)
    if firewalld_default_policy == '1':
        firewall_device, is_create = FirewalldDevicePolicy.objects.get_or_create(device_id=device.id)
        if is_create:
            if not WaitDevicePolicy.objects.filter(device_id=device_map.get('id'), policy='firewalld_policy').first():
                firewalld = int(default_policy.get_default_policy().get('firewalld'))
                firewall_device.firewalld = firewalld
                firewall_device.status = 2
                firewall_device.save()
                tasks.firewalld_policy.delay(
                    [device_map],
                    firewalld,
                    signal=False
                )
            if tasks.get_os(device.os) != 'kylin':
                if not WaitDevicePolicy.objects.filter(device_id=device_map.get('id'),
                                                       policy='firewalld_level_policy').first():
                    firewalld_level = int(default_policy.get_default_policy().get('firewalld_level'))
                    firewall_device.firewalld_level = firewalld_level
                    firewall_device.status = 2
                    firewall_device.save()
                    tasks.firewalld_level_policy.delay(
                        [device_map],
                        firewalld_level,
                        signal=False
                    )

        # 防火墙黑白名单 策略
        if not WaitDevicePolicy.objects.filter(device_id=device_map.get('id'),
                                               policy='firewalld_black_white_roster_policy').first():
            firewalld_roster = FirewalldWhiteBlackRosterPolicy.objects.all().order_by('-id')
            pillar = ['Other', [], []]
            rule = int(default_policy.get_default_policy().get('firewalld_roster'))
            if rule == 0:
                pillar[0] = 'White'
            elif rule == 1:
                pillar[0] = 'Black'
            else:
                pillar[0] = 'Other'
            pillar[1] = list(firewalld_roster.filter(roster=0).values_list('site', flat=True))
            pillar[2] = list(firewalld_roster.filter(roster=1).values_list('site', flat=True))
            firewalld_white_black_device, is_create = FirewalldWhiteBlackRosterDevicePolicy.objects.get_or_create(
                device_id=device.id)
            if is_create:
                firewalld_white_black_device.roster = json.dumps(pillar)
                firewalld_white_black_device.status = 2
                tasks.firewalld_black_white_roster_policy.delay(
                    [DeviceSerializer(device).data],  # celery参数不能是query_set类型
                    json.dumps(pillar),
                    signal=False
                )
    else:
        logger.info('firewalld策略未打开')
    # 执行软件默认安装策略
    software_default_policy = default_policy.get_default_policy().get('software_default_policy', None)
    if software_default_policy == '1':
        logger.info('执行软件默认安装策略')
        tasks.pkg_install_default_policy.delay()
    else:
        logger.info('software策略未打开')
    # 执行浏览器默认策略
    browser_default_policy = default_policy.get_default_policy().get('browser_default_policy', None)
    if browser_default_policy:
        logger.info('执行浏览器默认策略')

        tasks.browser_default_policy.delay(device_map)
    else:
        logger.info('浏览器默认策略未打开')

        # screen_lock
        switch_status = Screensavers.objects.all().first().switch_status
        if switch_status == 1:  # 如果默认锁屏策略打开则改变加域时终端锁屏信息
            screen_interval = int(Screensavers.objects.all().first().interval) * 60
            if not WaitDevicePolicy.objects.filter(device_id=device_map.get('id'),
                                                   policy='screensavers_policy').first():
                screensavers_device, is_create = ScreensaversDevicePolicy.objects.get_or_create(device_id=device.id)
                if is_create:
                    logger.info('### run_default_policy screensavers_policy ###')
                    screensavers_device.interval = screen_interval
                    screensavers_device.switch_status = switch_status
                    screensavers_device.status = 2
                    screensavers_device.save()
                    tasks.screensavers_policy.delay(
                        [device_map],
                        {'interval': screen_interval, 'switch_status': switch_status},
                        signal=False
                    )
        else:
            logger.info('默认screensavers 锁屏策略未打开')

    # watermark policy
    watermark_default_policy = WaterMark.objects.all().first()
    if WaterMarkDevicePolicy.objects.filter(device_id=device.id).exists():
        pass
    else:
        if watermark_default_policy.watermark_switch_status == 1:
            watermark_params_dict = {
                "watermark_switch_status": watermark_default_policy.watermark_switch_status,
                "user_defined_text": watermark_default_policy.user_defined_text,
                "watermark_position": watermark_default_policy.watermark_position,
                "watermark_rotate": watermark_default_policy.watermark_rotate,
                "watermark_font": watermark_default_policy.watermark_font,
                "watermark_font_size": watermark_default_policy.watermark_font_size,
                "watermark_font_color": watermark_default_policy.watermark_font_color,
                "watermark_font_transparency": float(watermark_default_policy.watermark_font_transparency)
            }

            watermark_device, _ = WaterMarkDevicePolicy.objects.get_or_create(device_id=device.id)
            watermark_device.status = 2
            watermark_device.watermark_switch_status = watermark_default_policy.watermark_switch_status
            watermark_device.user_defined_text = watermark_default_policy.user_defined_text
            watermark_device.watermark_position = watermark_default_policy.watermark_position
            watermark_device.watermark_rotate = watermark_default_policy.watermark_rotate
            watermark_device.watermark_font = watermark_default_policy.watermark_font
            watermark_device.watermark_font_size = watermark_default_policy.watermark_font_size
            watermark_device.watermark_font_color = watermark_default_policy.watermark_font_color
            watermark_device.watermark_font_transparency = watermark_default_policy.watermark_font_transparency
            watermark_device.save()
            tasks.watermark_policy(
                [device_map],  # celery参数不能是query_set类型
                watermark_params_dict,
                signal=False
            )
        else:
            logger.info('默认watermark 水印策略未打开')

    # 系统更新配置策略
    system_update_items = SystemUpdatesPolicy.objects.all()
    if system_update_items.filter(device_id=device_map.get('id')).exists():
        device_system_update_policy_obj = system_update_items.get(device_id=device_map.get('id'))
        device_system_update_policy_obj.status = 2
        device_system_update_policy_obj.save()
        tasks.device_system_updates_policy.delay(
            [device_map],  # celery参数不能是query_set类型
            {
                "system_update_switch": device_system_update_policy_obj.system_update_switch,
                "update_style": device_system_update_policy_obj.update_style,
                "check_time_days": device_system_update_policy_obj.check_time_days,
                "check_time_time": device_system_update_policy_obj.check_time_time,
                "download_update_method": device_system_update_policy_obj.download_update_method
            },
            singal=False
        )
    else:
        if system_update_items.filter(id=1).exists():
            device_system_update_policy_obj = system_update_items.get(id=1)
            if device_system_update_policy_obj.system_update_switch == 0:
                logger.info('默认system_update_policy 系统更新配置默认策略未打开')
            else:
                action_device_items, _ = SystemUpdatesPolicy.objects.get_or_create(device_id=device.id)
                action_device_items.status = 2
                action_device_items.system_update_switch = device_system_update_policy_obj.system_update_switch
                action_device_items.update_style = device_system_update_policy_obj.update_style
                action_device_items.check_time_days = device_system_update_policy_obj.check_time_days
                action_device_items.check_time_time = device_system_update_policy_obj.check_time_time
                action_device_items.download_update_method = device_system_update_policy_obj.download_update_method
                action_device_items.save()

                tasks.device_system_updates_policy.delay(
                    [device_map],  # celery参数不能是query_set类型
                    {
                        "system_update_switch": device_system_update_policy_obj.system_update_switch,
                        "update_style": device_system_update_policy_obj.update_style,
                        "check_time_days": device_system_update_policy_obj.check_time_days,
                        "check_time_time": device_system_update_policy_obj.check_time_time,
                        "download_update_method": device_system_update_policy_obj.download_update_method
                    },
                    singal=False
                )
        else:
            logger.info('默认system_update_policy 系统更新配置默认策略未初始化')

    # 远程协助策略
    from remote_assist.service import get_device_remote_assist_policy, remote_assist_policy_to_device
    from django.forms import model_to_dict

    remote_assist = get_device_remote_assist_policy(device)
    if remote_assist.global_enabled:
        ra_dict = model_to_dict(remote_assist)
        remote_assist_policy_to_device(device, ra_dict)

    # 脚本策略
    shell_policy_filter = ShellDevicePolicy.objects.filter(device=device)
    if shell_policy_filter.exists():
        from saltapi.tasks import shell_policy
        sp = shell_policy_filter.first()
        shell_policy.delay([DeviceSerializer(device).data], {'file_path': sp.file_path})
    else:
        from saltapi.tasks import shell_policy
        default_shell_policy = DefaultPolicy().get_default_policy().get('shell_policy')
        try:
            if default_shell_policy:
                default_shell_policy = json.loads(default_shell_policy)
        except:
            pass
        if default_shell_policy and default_shell_policy['shell_switch']:
            ShellDevicePolicy.objects.update_or_create(device=device, defaults = default_shell_policy)
            shell_policy.delay([DeviceSerializer(device).data], {'file_path': default_shell_policy['file_path']})
        else:
            logger.info('默认shell_policy 脚本策略未初始化')

    # 触发系统策略开机下发
    from policy.tasks import trigger_gpo_update
    logger.info(f"============device policy startup push start======")
    trigger_gpo_update.delay([], [device.id])


# 用户默认策略
def run_user_default_policy(device):
    """用户登入触发的默认策略"""
    from saltapi import tasks

    device_map = DeviceSerializer(device).data
    screensavers_items = ScreensaversDevicePolicy.objects.all()
    # 下发屏幕锁屏策略
    if not WaitDevicePolicy.objects.filter(device_id=device_map.get('id'), policy='screensavers_policy').first():
        default_screensavers_item = Screensavers.objects.filter(id=1).first()
        if screensavers_items.filter(device_id=device.id).exists():
            # 自定义策略存在
            screensavers_device = screensavers_items.get(device_id=device.id)
            tasks.screensavers_policy.delay(
                [device_map],  # celery参数不能是query_set类型
                {'interval': screensavers_device.interval,
                 'switch_status': screensavers_device.switch_status}
            )
        else:
            # 默认锁屏策略
            if default_screensavers_item.switch_status == 1:
                default_interval = int(default_screensavers_item.interval) * 60
                tasks.screensavers_policy.delay(
                    [device_map],  # celery参数不能是query_set类型
                    {'interval': default_interval,
                     'switch_status': default_screensavers_item.switch_status}
                )

    else:
        if screensavers_items.filter(device_id=device.id).exists():
            # 自定义策略存在
            screensavers_device = screensavers_items.get(device_id=device.id)
            tasks.screensavers_policy.delay(
                [device_map],  # celery参数不能是query_set类型
                {'interval': screensavers_device.interval,
                 'switch_status': screensavers_device.switch_status}
            )
    if WaitDevicePolicy.objects.filter(device_id=device_map.get('id'), policy='wallpaper_policy').exists():
        default_wallpaper = Wallpaper.objects.filter(is_default=1).first()
        wallpaper_device, _ = WallpaperDevicePolicy.objects.get_or_create(device_id=device.id)
        if default_wallpaper and not WallpaperDevicePolicy.objects.filter(device_id=device.id, status=1).exists():
            wallpaper_device.wallpaper_id = default_wallpaper.id
            wallpaper_device.status = 2
            wallpaper_device.save()
            tasks.wallpaper_policy.delay(
                [device_map],
                default_wallpaper.id,
                signal=False
            )
    else:
        run_user_wait_policy(device)

    # 水印策略
    default_watermark = WaterMark.objects.all().first()
    watermark_device, _ = WaterMarkDevicePolicy.objects.get_or_create(device_id=device.id)
    logger.info(f"***{watermark_device.watermark_switch_status}")
    if _:
        if default_watermark.watermark_switch_status == 1:
            watermark_device.status = 2
            watermark_device.watermark_switch_status = default_watermark.watermark_switch_status
            watermark_device.user_defined_text = default_watermark.user_defined_text
            watermark_device.watermark_position = default_watermark.watermark_position
            watermark_device.watermark_rotate = default_watermark.watermark_rotate
            watermark_device.watermark_font = default_watermark.watermark_font
            watermark_device.watermark_font_size = default_watermark.watermark_font_size
            watermark_device.watermark_font_color = default_watermark.watermark_font_color
            watermark_device.watermark_font_transparency = default_watermark.watermark_font_transparency
            watermark_device.save()
            tasks.watermark_policy.delay(
                [device_map],  # celery参数不能是query_set类型
                {
                    "watermark_switch_status": default_watermark.watermark_switch_status,
                    "user_defined_text": default_watermark.user_defined_text,
                    "watermark_position": default_watermark.watermark_position,
                    "watermark_rotate": default_watermark.watermark_rotate,
                    "watermark_font": default_watermark.watermark_font,
                    "watermark_font_size": default_watermark.watermark_font_size,
                    "watermark_font_color": default_watermark.watermark_font_color,
                    "watermark_font_transparency": default_watermark.watermark_font_transparency
                },
                signal=False
            )
        else:
            watermark_device.delete()
    else:
        if watermark_device.watermark_switch_status == 1:
            watermark_device.status = 2
            watermark_device.save()
            tasks.watermark_policy.delay(
                [device_map],  # celery参数不能是query_set类型
                {
                    "watermark_switch_status": watermark_device.watermark_switch_status,
                    "user_defined_text": watermark_device.user_defined_text,
                    "watermark_position": watermark_device.watermark_position,
                    "watermark_rotate": watermark_device.watermark_rotate,
                    "watermark_font": watermark_device.watermark_font,
                    "watermark_font_size": watermark_device.watermark_font_size,
                    "watermark_font_color": watermark_device.watermark_font_color,
                    "watermark_font_transparency": watermark_device.watermark_font_transparency
                },
                signal=False
            )
        else:
            pass


def run_local_user_pwd_change_policy(device):
    from saltapi import tasks
    device_map = DeviceSerializer(device).data
    local_user_pwd, is_create = LocalUserPWDPolicy.objects.get_or_create(device_id=device.id)
    if is_create:
        tasks.local_user_pwd_change_policy.delay(
            [device_map],  # celery参数不能是query_set类型
            'user'
        )


def notify_browser_policy(device):
    publicer = RedisHelper()
    publicer.public(json.dumps({"event": "save_device_domain_info_to_reids", "device_id": device.id})
                    , chan='browser_policy_chan')


def save_device_domain_info_to_reids(device):
    RedisCacheDevice().update_redis_cache(device.device_hostname)
    # conn = get_redis_connection('redis')
    # conn.hset('device_domain_info', device.device_hostname, 1)


def clear_device_domain_info(device):
    # conn = get_redis_connection('redis')
    try:
        device.is_domain = 0
        device.is_up = 1
        device.org_info_id = 1
        device.belong_user = None
        device.note = None
        device.save()
        # device.delete()
        # conn.hdel('device_domain_info', device.device_hostname)
        RedisCacheDevice().delete_redis_cache(device.device_hostname)
    except Exception as exp:
        logger.info(f"**删除设备失败****{exp}****")
        # device.device_hostname = None
        device.save()
    try:
        del_roster(device.device_cnname)
    except Exception as exp:
        logger.info(f"**清除roster失败****{exp}****")


def clear_device_policy(device):
    use_device = USBDevicePolicy.objects.filter(device_id=device.id).first()
    if use_device:
        use_device.delete()
    ssh_device = SSHDevicePolicy.objects.filter(device_id=device.id).first()
    if ssh_device:
        ssh_device.delete()
    firewalld_device = FirewalldDevicePolicy.objects.filter(device_id=device.id).first()
    if firewalld_device:
        firewalld_device.delete()

    watermark_device = WaterMarkDevicePolicy.objects.filter(device_id=device.id).first()
    if watermark_device:
        watermark_device.delete()

    # # 退域后删除任务中的设备记录
    # task_exec_data = TaskDeviceExec.objects.filter(device__pk=device.id)
    # if task_exec_data:
    #     task_exec_data.delete()


def clear_software_installed_infos(device):
    from software.models import SoftwareInstalledModel
    installed_logs = SoftwareInstalledModel.objects.all().filter(device_info_id=device.id)
    if installed_logs:
        for log in installed_logs:
            log.delete()
    logger.info('delete software installed log for device ' + str(device.id))


def clear_task_infos(device):
    from tasks.models import TaskDeviceExec
    installed_logs = TaskDeviceExec.objects.filter(device=device)
    installed_logs.delete()
    logger.info('delete task installed log for device ' + str(device.id))


def clear_device_user_active(device):
    device_user_info = get_device_user_active()
    device_info = device_user_info.get(device.device_hostname)
    if device_info:
        del device_user_info[device.device_hostname]
        update_device_user_active(device_user_info)
    logger.info('delete device user active for device ' + str(device.id))


def clear_wait_policy(device, policy=None, job_info_id=None):
    device_wait_policys = WaitDevicePolicy.objects.filter(device_id=device.id).all()
    if job_info_id is not None:
        arg_string = f'"job_info_id": {job_info_id}'
        device_wait_policys.filter(policy=policy, arg__icontains=arg_string).delete()
    else:
        if policy:
            wait_policy = device_wait_policys.filter(policy=policy).first()
            if wait_policy:
                wait_policy.delete()
            return
        for wait_policy in device_wait_policys:
            wait_policy.delete()


def clear_device_jid(device):
    salt_jid = SaltJidDevice.objects.all()
    salt_jid.filter(Q(minion=device.device_hostname) | Q(minion=device.device_cnname)).delete()


def clear_user_policy(device):
    # 清除壁纸记录
    wallpaper_device = WallpaperDevicePolicy.objects.filter(device_id=device.id).first()
    if wallpaper_device:
        # wallpaper_device.wallpaper_id = None
        # wallpaper_device.save()
        wallpaper_device.delete()
    screensavers_device = ScreensaversDevicePolicy.objects.filter(device_id=device.id).first()
    if screensavers_device:
        screensavers_device.delete()


def save_policy_result(data):
    if data.get('event_type') == 'ssh_policy':
        ssh_device, _ = SSHDevicePolicy.objects.get_or_create(device_id=data.get('device_id'))
        ssh_device.ssh = data.get('arg')
        ssh_device.status = data.get('status')
        ssh_device.save()
    if data.get('event_type') == 'usb_policy':
        usb_device, _ = USBDevicePolicy.objects.get_or_create(device_id=data.get('device_id'))
        usb_device.usb = data.get('arg')
        usb_device.status = data.get('status')
        usb_device.save()
    if data.get('event_type') == 'wallpaper_policy':
        wallpaper_device, _ = WallpaperDevicePolicy.objects.get_or_create(device_id=data.get('device_id'))
        wallpaper_device.wallpaper_id = data.get('arg')
        wallpaper_device.status = data.get('status')
        wallpaper_device.save()
    if data.get('event_type') == 'firewalld_policy':
        firewalld_device, _ = FirewalldDevicePolicy.objects.get_or_create(device_id=data.get('device_id'))
        firewalld_device.firewalld = data.get('arg')
        firewalld_device.status = data.get('status')
        firewalld_device.save()
    if data.get('event_type') == 'firewalld_level_policy':
        firewalld_device, _ = FirewalldDevicePolicy.objects.get_or_create(device_id=data.get('device_id'))
        firewalld_device.firewalld_level = data.get('arg')
        firewalld_device.status = data.get('status')
        firewalld_device.save()
    if data.get('event_type') == 'system_activation_policy':
        activation_device, _ = ActivationDevicePolicy.objects.get_or_create(device_id=data.get('device_id'))
        activation_device.status = data.get('status')
        activation_device.save()


def save_policy_jid(fun_args, jid, device):
    from saltapi.tasks import get_os
    if not device.os:
        return
    if fun_args[0] in ['{os}.ssh-manage.ssh_enable'.format(os=get_os(device.os, device.os_release)),
                       '{os}.ssh-manage.ssh_disable'.format(os=get_os(device.os, device.os_release))]:
        ssh_device = SSHDevicePolicy.objects.filter(device_id=device.id).first()
        ssh_device.jid = jid
        ssh_device.save()

    if fun_args[0] in ['{os}.device-control.usb_disable'.format(os=get_os(device.os, device.os_release)),
                       '{os}.device-control.usb_enable'.format(os=get_os(device.os, device.os_release))]:
        usb_device = USBDevicePolicy.objects.filter(device_id=device.id).first()
        usb_device.jid = jid
        usb_device.save()
    if fun_args[0] in ['{os}.firewall-manage.enable_firewall'.format(os=get_os(device.os, device.os_release)),
                       '{os}.firewall-manage.set_firewall_level'.format(os=get_os(device.os, device.os_release))]:
        firewalld_device = FirewalldDevicePolicy.objects.filter(device_id=device.id).first()
        firewalld_device.jid = jid
        firewalld_device.save()
    if fun_args[0] in ['{os}.background-manage.set-background'.format(os=get_os(device.os, device.os_release))]:
        wallpaper_device = WallpaperDevicePolicy.objects.filter(device_id=device.id).first()
        wallpaper_device.jid = jid
        wallpaper_device.save()
    if fun_args[0] in ['{os}.kms-manage.kms-manage'.format(os=get_os(device.os, device.os_release))]:
        wallpaper_device = WallpaperDevicePolicy.objects.filter(device_id=device.id).first()
        wallpaper_device.jid = jid
        wallpaper_device.save()


def get_pillar(pillar_str):
    class_dic = {'pillar': {'signal': True}}
    if isinstance(pillar_str, str):
        try:
            exec(pillar_str, {}, class_dic)
        except:
            pass
        pillar_dict = class_dic.get('pillar')
    else:
        pillar_dict = pillar_str.get('pillar')
    logger.info("pillar_dict %s" % pillar_dict)
    return pillar_dict


def parse_policy_result(fun_args, jid, device, ret):
    from saltapi.tasks import SaltAPI, allow_access, check_task_status
    public = RedisHelper()
    policy_cmd = fun_args[0].split('.')
    # os = policy_cmd[0]
    policy = policy_cmd[1]
    signal = {}
    signal['device_id'] = device.id
    signal['device_ip'] = device.device_ip
    signal['device_hostname'] = device.device_hostname
    pillar = get_pillar(fun_args[1])
    _, _, num_failed = check_output.output(ret)

    logger.info("*******parse_policy_result********")
    logger.info("fun_args %s" % fun_args)
    logger.info("policy_cmd %s" % policy_cmd)
    logger.info("pillar %s" % pillar)
    logger.info("signal %s" % signal)
    logger.info("num_failed %s" % num_failed)
    if policy == 'freeipa-client-init':
        if policy_cmd[2] == 'exit_domain':
            signal['event_type'] = 'delete'
            signal['success'] = True
            signal['msg'] = '退域成功'
            signal['signal'] = pillar.get('signal')
            if num_failed:
                signal['success'] = False
                signal['msg'] = '退域失败'
            public.public(json.dumps(signal), chan='freeipa')
            return

        elif policy_cmd[2] == 'install':
            pass

        else:
            return

    if policy == 'user-manage':
        signal['signal'] = pillar.get('signal')
        if policy_cmd[2] == 'unlock_local_user':
            signal['event_type'] = 'unlock_local_user'
            if num_failed:
                signal['status'] = 3
                signal['success'] = False
                signal['msg'] = '解锁用户失败'
            else:
                signal['status'] = 1
                signal['success'] = True
                signal['msg'] = '解锁用户成功'

        elif policy_cmd[2] == 'lock_local_user':
            signal['event_type'] = 'unlock_local_user'
            if num_failed:
                signal['status'] = 3
                signal['success'] = False
                signal['msg'] = '锁用户失败'
            else:
                signal['status'] = 1
                signal['success'] = True
                signal['msg'] = '锁用户成功'

        elif policy_cmd[2] == 'unlock_domain_user':
            signal['event_type'] = 'unlock_domain_user'
            domain_user = pillar.get('domain_user_name')
            if num_failed:
                signal['status'] = 3
                signal['success'] = False
                signal['msg'] = f'解锁域用户{domain_user}失败'
            else:
                signal['status'] = 1
                signal['success'] = True
                signal['msg'] = f'解锁域用户{domain_user}成功'
        elif policy_cmd[2] == 'local_user_pwd_change':
            signal['event_type'] = 'local_user_pwd_change_policy'
            local_user = pillar.get('local_user')
            if num_failed:
                signal['status'] = 3
                signal['success'] = False
                signal['msg'] = f'本地用户密码重置 策略设置失败'
            else:
                signal['status'] = 1
                signal['success'] = True
                signal['msg'] = f'本地用户密码重置 策略设置成功'
            local_user_pwd, is_create = LocalUserPWDPolicy.objects.get_or_create(device_id=device.id)
            local_user_pwd.jid = jid
            local_user_pwd.status = signal['status']
            local_user_pwd.save()
        else:
            return
        public.public(json.dumps(signal), chan='policy')

    if policy == 'ssh-manage':
        if policy_cmd[2] == 'ssh_enable':
            arg = 1
        elif policy_cmd[2] == 'ssh_disable':
            arg = 0
        else:
            return
        signal['arg'] = arg
        signal['signal'] = pillar.get('signal')
        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = 'ssh 策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = 'ssh 策略设置成功'
            clear_wait_policy(device, 'ssh_policy')
        ssh_device, _ = SSHDevicePolicy.objects.get_or_create(device_id=device.id)
        ssh_device.jid = jid
        ssh_device.ssh = arg
        ssh_device.status = signal['status']
        if pillar.get('policy_no'):
            ssh_device.policy_no = pillar.get('policy_no')
        else:
            ssh_device.policy_no = None
        ssh_device.save()
        if pillar.get('policy_no'):
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'device-control':
        signal['event_type'] = 'usb_policy'
        if policy_cmd[2] == 'usb_enable':
            arg = 1
        else:
            arg = 0
        signal['arg'] = arg
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = 'usb 策略设置失败， 策略错误'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = 'usb 策略设置成功'
            clear_wait_policy(device, 'usb_policy')
        logger.info(f"***usb***** {jid},****{signal}")
        usb_device, _ = USBDevicePolicy.objects.get_or_create(device_id=device.id)
        usb_device.jid = jid
        usb_device.usb = signal['arg']
        usb_device.status = signal['status']
        if pillar.get('policy_no'):
            usb_device.policy_no = pillar.get('policy_no')
        else:
            usb_device.policy_no = None
        usb_device.policy_no = pillar.get('policy_no')
        usb_device.save()
        if pillar.get('policy_no'):
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'firewall-manage':
        salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
        firewalld_device, _ = FirewalldDevicePolicy.objects.get_or_create(device_id=device.id)
        if policy_cmd[2] == 'enable_firewall':
            signal['event_type'] = 'firewalld_policy'
            if pillar.get('enable') == 'true':
                arg = 1
            else:
                arg = 0
            signal['arg'] = arg
            signal['signal'] = pillar.get('signal')

            if num_failed:
                signal['status'] = 3
                signal['success'] = False
                signal['msg'] = '防火墙 策略设置失败， 策略错误'
            else:
                signal['status'] = 1
                signal['success'] = True
                signal['msg'] = '防火墙 策略设置成功'
                if arg == 1:
                    allow_access(salt_api, device.device_hostname, os=device.os, os_release=device.os_release)
                clear_wait_policy(device, 'firewalld_policy')
            firewalld_device.firewalld = signal['arg']
        elif policy_cmd[2] == 'set_firewall_level':
            signal['event_type'] = 'firewalld_level_policy'
            arg = int(pillar.get('level'))
            signal['arg'] = arg
            signal['signal'] = pillar.get('signal')

            if num_failed:
                signal['status'] = 3
                signal['success'] = False
                signal['msg'] = '防火墙等级 策略设置失败， 策略错误'
            else:
                signal['status'] = 1
                signal['success'] = True
                signal['msg'] = '防火墙等级 策略设置成功'
                clear_wait_policy(device, 'firewalld_level_policy')
                allow_access(salt_api, device.device_hostname, os=device.os, os_release=device.os_release)
            firewalld_device.firewalld_level = signal['arg']
        elif policy_cmd[2] == 'set_firewall_roster':
            firewalld_device, _ = FirewalldWhiteBlackRosterDevicePolicy.objects.get_or_create(device_id=device.id)
            signal['event_type'] = 'firewalld_black_white_roster_policy'
            arg = pillar.get('roster')
            signal['arg'] = arg
            signal['signal'] = pillar.get('signal')
            firewalld_device.roster = arg
            if num_failed:
                signal['status'] = 3
                signal['success'] = False
                signal['msg'] = '防火墙黑白名单 策略设置失败， 策略错误'
            else:
                signal['status'] = 1
                signal['success'] = True
                signal['msg'] = '防火墙黑白名单 策略设置成功'
                clear_wait_policy(device, 'firewalld_black_white_roster_policy')
        firewalld_device.jid = jid
        firewalld_device.status = signal['status']
        firewalld_device.policy_no = pillar.get('policy_no') if 'policy_no' in pillar.keys() else None
        firewalld_device.save()
        if pillar.get('policy_no'):
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'background-manage':
        wallpaper_name = pillar.get('wallpaper')
        wallpaper = Wallpaper.objects.filter(name=wallpaper_name).first()
        signal['event_type'] = 'wallpaper_policy'
        signal['arg'] = wallpaper.id
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '壁纸 策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '壁纸 策略设置成功'
            clear_wait_policy(device, 'wallpaper_policy')
        wallpaper_device, _ = WallpaperDevicePolicy.objects.get_or_create(device_id=device.id)
        wallpaper_device.jid = jid
        wallpaper_device.wallpaper_id = signal['arg']
        wallpaper_device.status = signal['status']
        wallpaper_device.policy_no = pillar.get('policy_no') if 'policy_no' in pillar.keys() else None
        wallpaper_device.save()
        if pillar.get('policy_no'):
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
        return
    if policy_cmd[-1] == 'set-lock-screen-time':
        # 获取屏幕锁定结果分析
        interval = pillar.get('time')
        switch_status = pillar.get('switch_status')
        signal['event_type'] = 'screensavers_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '锁屏 策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '锁屏 策略设置成功'
            clear_wait_policy(device, 'screensavers_policy')

        screensavers_device, _ = ScreensaversDevicePolicy.objects.get_or_create(device_id=device.id)
        screensavers_device.jid = jid
        screensavers_device.interval = int(interval)
        screensavers_device.status = signal['status']
        screensavers_device.policy_no = pillar.get('policy_no') if 'policy_no' in pillar.keys() else None
        screensavers_device.switch_status = switch_status
        screensavers_device.save()
        if pillar.get('policy_no'):
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'sudo-manage':
        # 获取开关root结果分析
        signal['event_type'] = 'check_sudo_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')
        name = pillar.get('domain_user')
        on_off = pillar.get('on_off')
        account_items = AccountEntity.objects.all()

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = 'root 策略设置失败'
            if on_off == 'on':
                is_root = 0
            else:
                is_root = 1
            account_items.filter(name=name).update(is_root=is_root)
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = 'root 策略设置成功'
            if on_off == 'on':
                is_root = 1
            else:
                is_root = 0
            account_items.filter(name=name).update(is_root=is_root)

        if pillar.get('policy_no'):
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'common_manage':
        # 获取离线登入结果分析
        if policy_cmd[2] == 'update_restart_sssd':
            signal['event_type'] = 'update_cache_sssd_policy'
            signal['arg'] = pillar
            signal['signal'] = pillar.get('signal')

            if num_failed:
                signal['status'] = 3
                signal['success'] = False
                signal['msg'] = '离线登入策略设置失败'
            else:
                signal['status'] = 1
                signal['success'] = True
                signal['msg'] = '离线登入策略设置成功'

            public.public(json.dumps(signal), chan='policy')
            return

    if policy == 'kms-manage':
        signal['signal'] = pillar.get('signal')
        signal['event_type'] = 'system_activation_policy'
        if num_failed:
            active = 0
            signal['status'] = 0
            signal['success'] = False
            signal['msg'] = '系统激活 策略设置失败， 策略错误'
        else:
            active = 1
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '系统激活 策略设置成功'
        activation_device, _ = ActivationDevicePolicy.objects.get_or_create(device_id=device.id)
        activation_device.status = signal['status']
        activation_device.active = active
        activation_device.jid = jid
        activation_device.save()
        public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'software-manage':
        signal['signal'] = pillar.get('signal')
        software_id = int(pillar.get('software_id'))
        software = SoftwareInfoModel.objects.filter(pk=software_id, is_delete=0).first()
        if not software:
            return
        softwareInstalledInfo = SoftwareInstalledModel.objects.filter(device_info_id=device.id).filter(
            main_package=software.main_package, main_version=software.main_version).first()
        if policy_cmd[2] == 'install' or policy_cmd[2] == 'source_install':
            signal['event_type'] = 'pkg_install_policy'
            if num_failed:
                signal['success'] = False
                signal['status'] = SoftwareInstalledModel.STATUS_FAILED
                signal['msg'] = f'软件{pillar.get("software_name")}安装失败'
            else:
                signal['success'] = True
                signal['status'] = SoftwareInstalledModel.STATUS_INSTALLED
                signal['msg'] = f'软件{pillar.get("software_name")}安装成功'
        elif policy_cmd[2] == 'remove':
            signal['event_type'] = 'pkg_remove_policy'
            if num_failed:
                signal['success'] = False
                signal['status'] = SoftwareInstalledModel.STATUS_FAILED
                signal['msg'] = f'软件{pillar.get("software_name")}卸载失败'
            else:
                signal['success'] = True
                signal['status'] = SoftwareInstalledModel.STATUS_REMOVED
                signal['msg'] = f'软件{pillar.get("software_name")}卸载成功'
        # 安装成功后 修改状态
        #  delete wait policy if exist
        waitDevicePolicy = WaitDevicePolicy.objects.filter(device_id=device.id, policy=signal['event_type'],
                                                           arg=int(pillar.get('software_id'))).first()
        if waitDevicePolicy:
            waitDevicePolicy.delete()
        softwareInstalledInfo.installed_status = signal['status']
        softwareInstalledInfo.save()
        software_policy_id = pillar.get('software_policy_id')
        logger.info("software_policy_id %s" % software_policy_id)
        if software_policy_id != 0:
            software_policy = SoftwarePolicy.objects.filter(id=software_policy_id, is_delete=0).first()
            if software_policy:
                if policy_cmd[2] == 'remove':
                    # 软件卸载的策略状态改为未设置
                    SoftwarePolicy.objects.filter(id=software_policy_id, is_delete=0).update(exec_status=0)
                else:
                    # 软件安装更新状态
                    update_policy_status(software_policy)
        task_id = pillar.get('task_id')
        if task_id != 0:
            logger.info("task_id %s" % task_id)
            task = Tasks.objects.filter(id=task_id).first()
            if task.quality != 2:
                # 任务中软件下发
                file_obj_id = FileManager.objects.get(software_id=software_id, task_id=task_id).id
                logger.info(f"task_id {task_id} file_obj {file_obj_id} device_id {device.id}")
                if signal['status'] == SoftwareInstalledModel.STATUS_INSTALLED:
                    TaskDeviceExec.objects.filter(task__pk=task_id, file_obj=str(file_obj_id),
                                                  device__pk=device.id).update(exec_status=4, jid=jid)
                else:
                    TaskDeviceExec.objects.filter(task__pk=task_id, file_obj=str(file_obj_id),
                                                  device__pk=device.id).update(exec_status=5, jid=jid)
                check_task_status(pillar.get("task_id"))
        public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'cert-manager':
        signal['event_type'] = 'ca_apply_policy'
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = 'CA 策略设置失败， 策略错误'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = 'CA 策略设置成功'
            clear_wait_policy(device, 'ca_apply_policy')

        ca_device, _ = CaDevicePolicy.objects.get_or_create(device_id=device.id)
        ca_device.jid = jid
        ca_device.ca_name = pillar.get('certname') if 'certname' in pillar.keys() else None
        ca_device.device_id = device.id
        ca_device.status = signal['status']
        ca_device.policy_no = pillar.get('policy_no') if 'policy_no' in pillar.keys() else None
        ca_device.save()
        if pillar.get('policy_no'):
            pass
        else:
            public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'task-manage':
        job_info_id = pillar.get("job_info_id")
        job_res_items = JobResultInfo.objects.filter(job_info_id=job_info_id)
        if not job_res_items:
            return

        if policy_cmd[2] == 'task_file':
            signal['event_type'] = 'job_file_policy'
            if num_failed:
                signal['success'] = False
                signal['status'] = 0
                signal['msg'] = '文件下发失败'
                job_res_items.update(exe_status=4, job_jid_id=jid, update_time=datetime.datetime.now())
            else:
                signal['success'] = True
                signal['status'] = 1
                signal['msg'] = '文件下发成功'
                job_res_items.update(exe_status=3, job_jid_id=jid, update_time=datetime.datetime.now())
            clear_wait_policy(device, 'job_file_policy', pillar.get("job_info_id"))
        elif policy_cmd[2] == 'task_shell':
            signal['event_type'] = 'job_bash_policy'
            if num_failed:
                signal['success'] = False
                signal['status'] = 0
                signal['msg'] = '脚本文件下发失败'
                job_res_items.update(exe_status=4, job_jid_id=jid, update_time=datetime.datetime.now())
            else:
                signal['success'] = True
                signal['status'] = 1
                signal['msg'] = '脚本文件下发成功'
                job_res_items.update(exe_status=3, job_jid_id=jid, update_time=datetime.datetime.now())
            clear_wait_policy(device, 'job_bash_policy', pillar.get("job_info_id"))
        elif policy_cmd[2] == 'task_software':
            signal['event_type'] = 'job_software_policy'
            if num_failed:
                signal['success'] = False
                signal['status'] = 0
                signal['msg'] = '软件下发失败'
                job_res_items.update(exe_status=4, job_jid_id=jid, update_time=datetime.datetime.now())
            else:
                signal['success'] = True
                signal['status'] = 1
                signal['msg'] = '软件下发成功'
                job_res_items.update(exe_status=3, job_jid_id=jid, update_time=datetime.datetime.now())
            clear_wait_policy(device, 'job_software_policy', pillar.get("job_info_id"))
        public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'reboot-manage':
        if policy_cmd[2] == 'reboot':
            arg = 1
        elif policy_cmd[2] == 'poweroff':
            arg = 0
        else:
            return
        signal['arg'] = arg
        signal['signal'] = pillar.get('signal')
        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '{} 策略设置失败'.format(policy_cmd[2])
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '{} 策略设置成功'.format(policy_cmd[2])
        public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'file_operations':
        if policy_cmd[2] == 'compress_and_upload':
            """将minion端文件从master的位置移动到固定位置"""
            domain_name = pillar.get('domain_name')
            upload_file_name = pillar.get('upload_file_name')
            signal['event_type'] = 'compress_upload_file_policy'
            signal['arg'] = pillar
            signal['signal'] = pillar.get('signal')
            username = pillar.get('username', None)
            download_path = None

            if num_failed:
                # signal['status'] = 3
                # signal['success'] = False
                # signal['msg'] = '获取客户端文件失败'
                msg_content = '获取客户端文件失败'
            else:
                if os.environ.get('DEPLOY_ARCH') == 'cluster':
                    # 通过salt api接口将日志文件下载到web容器内
                    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
                    source_file_path = f"/var/cache/salt/master/minions/{domain_name}/files/upload_file/{upload_file_name}"
                    save_path = settings.SALTFILESYSTEMPATH + '/' + upload_file_name
                    # destination_directory = '/srv/salt/file_system/'

                    # code, msg = SaltRpcClient().move_file(source_file_path, destination_directory)
                    if salt_api.api_files_download(source_file_path, save_path):
                        # signal['status'] = 1
                        # signal['success'] = True
                        # signal['msg'] = '获取客户端文件成功
                        # signal['download_url'] = f"/mainapi/client/{upload_file_name}/"
                        msg_content = "终端日志文件已导出完成"
                        download_path = f"/mainapi/client/{upload_file_name}/"
                    else:
                        # signal['status'] = 3
                        # signal['success'] = False
                        # signal['msg'] = '获取客户端文件失败'
                        msg_content = "获取客户端文件失败"
                else:
                    try:
                        source_file_path = f"/var/cache/salt/master/minions/{domain_name}/files/upload_file/{upload_file_name}"
                        destination_directory = '/srv/salt/file_system/'

                        code, msg = SaltRpcClient().move_file(source_file_path, destination_directory)
                        if code == 200:
                            # signal['status'] = 1
                            # signal['success'] = True
                            # signal['msg'] = '获取客户端文件成功
                            # signal['download_url'] = f"/mainapi/client/{upload_file_name}/"
                            msg_content = "终端日志文件已导出完成"
                            download_path = f"/mainapi/client/{upload_file_name}/"
                        else:
                            # signal['status'] = 3
                            # signal['success'] = False
                            # signal['msg'] = '获取客户端文件失败'
                            msg_content = "终端日志文件已导出完成"

                    except Exception as e:
                        logger.info('--- move file error ---')
                        logger.info(e)
                        # signal['status'] = 3
                        # signal['success'] = False
                        # signal['msg'] = '获取客户端文件失败'
                        msg_content = '获取客户端文件失败,'

            if pillar.get('policy_no'):
                pass
            else:
                # public.public(json.dumps(signal), chan='policy')
                websocket_msg = {
                    'content': msg_content,
                    'create_time': datetime.datetime.now(),
                    'target_module': '终端日志导出',
                    'message_user': username,
                    'message_device': None,
                    'download_path': download_path,
                    'is_read': 0,
                    'is_broadcast': 0,
                }
                MessageCenter.objects.create(**websocket_msg)
                send_msg_to_web(websocket_msg, 0, username)
            return

    if policy == 'watermark-manage':
        # 获取设置水印信息的结果
        signal['event_type'] = 'watermark_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '水印策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '水印策略设置成功'
            clear_wait_policy(device, 'watermark_policy')

        watermark_device, _ = WaterMarkDevicePolicy.objects.get_or_create(device_id=device.id)
        watermark_device.status = signal['status']
        watermark_device.jid = jid
        watermark_device.save()
        public.public(json.dumps(signal), chan='policy')

    if policy == 'login_out_script':
        # 获取设置登入登出策略的结果
        signal['event_type'] = 'account_login_logout_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '脚本策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '脚本策略设置成功'
            clear_wait_policy(device, 'account_login_logout_policy')

        policy_objs, _ = AccountLoginPolicy.objects.get_or_create(account_id=pillar.get("account_id"))
        policy_objs.status = signal['status']
        policy_objs.jid = jid
        policy_objs.save()
        public.public(json.dumps(signal), chan='policy')

    if policy == 'sysiso_policy':
        sysiso_id = pillar.get('sysiso_id')
        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = f"{pillar.get('name')}镜像制作失败"
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = f"{pillar.get('name')}镜像制作成功"
        sysios_iso = SysiosIso.objects.filter(id=sysiso_id).first()
        sysios_iso.status = signal['status']
        sysios_iso.jid = jid
        sysios_iso.save()
        # TODO 当任务成功或任务失败时，需删除redis队列中的此次任务,并开始队列中的下一个任务
        sysios_q = SysisoQueue()
        sysios_q.lrem(sysiso_id)
        sysios_q.run_next_task()
        if pillar.get('policy_no'):
            pass
        else:
            websocket_msg = {
                'content': signal['msg'],
                'create_time': datetime.datetime.now(),
                'target_module': '系统部署_镜像制作',
                'message_user': pillar.get('username'),
                'message_device': None,
                'download_path': f'mainapi/sysiso/file/download/{sysiso_id}' if signal['status']==1 else None,
                'is_read': 0,
                'is_broadcast': 0,
            }
            MessageCenter.objects.create(**websocket_msg)
            send_msg_to_web(websocket_msg, 0, pillar.get('username'))

    if policy_cmd[2] == 'theme':
        # 获取设置桌面主题的结果
        signal['event_type'] = 'account_desktop_theme_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '桌面主题策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '桌面主题策略设置成功'
            clear_wait_policy(device, 'account_desktop_theme_policy')

        policy_objs, _ = DesktopThemePolicy.objects.get_or_create(account_id=pillar.get("account_id"))
        policy_objs.status = signal['status']
        policy_objs.jid = jid
        policy_objs.save()
        public.public(json.dumps(signal), chan='policy')

    if policy_cmd[2] == 'lock-screen':
        # 获取设置桌面主题的结果
        signal['event_type'] = 'device_screen_saver_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '屏幕保护策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '屏幕保护策略设置成功'
            clear_wait_policy(device, 'device_screen_saver_policy')

        policy_objs, _ = ScreenSaverPolicy.objects.get_or_create(device_id=device.id)
        policy_objs.status = signal['status']
        policy_objs.jid = jid
        policy_objs.save()
        public.public(json.dumps(signal), chan='policy')

    if policy_cmd[-1] == 'auto_update_system':
        # 获取终端更新配置策略的结果
        signal['event_type'] = 'device_system_updates_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '终端更新配置策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '终端更新配置策略设置成功'
            clear_wait_policy(device, 'device_system_updates_policy')

        policy_objs, _ = SystemUpdatesPolicy.objects.get_or_create(device_id=device.id)
        policy_objs.status = signal['status']
        policy_objs.jid = jid
        policy_objs.save()
        public.public(json.dumps(signal), chan='policy')

    if policy_cmd[2] == 'add_blacklist':
        # 获取终端更新配置策略的结果
        signal['event_type'] = 'device_software_runs_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '终端软件运行策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '终端软件运行策略设置成功'
            clear_wait_policy(device, 'device_software_runs_policy')

        policy_objs, _ = SoftwareRunsPolicy.objects.get_or_create(device_id=device.id)
        policy_objs.status = signal['status']
        policy_objs.jid = jid
        policy_objs.save()
        public.public(json.dumps(signal), chan='policy')

    if policy_cmd[2] == 'folder-redirection':
        # 获取终端重定向策略的结果
        signal['event_type'] = 'device_file_redirection_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '终端重定向策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '终端重定向策略设置成功'
            clear_wait_policy(device, 'device_file_redirection_policy')

        policy_objs, _ = FileRedirectionPolicy.objects.get_or_create(device_id=device.id)
        policy_objs.status = signal['status']
        policy_objs.jid = jid
        policy_objs.save()
        public.public(json.dumps(signal), chan='policy')

    if len(policy_cmd) > 3 and policy_cmd[3] == 'popup_policy':
        # 获取设置桌面主题的结果
        signal['event_type'] = 'account_trusted_sites_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '可信任站点策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '可信任站点策略设置成功'
            clear_wait_policy(device, 'account_trusted_sites_policy')

        policy_objs, _ = TrustedSitesPolicy.objects.get_or_create(account_id=pillar.get("account_id"))
        policy_objs.status = signal['status']
        policy_objs.jid = jid
        policy_objs.save()
        public.public(json.dumps(signal), chan='policy')

    if len(policy_cmd) > 3 and policy_cmd[3] == 'mv_bookmark':
        # 获取设置桌面主题的结果
        signal['event_type'] = 'account_browser_favorites_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '游览器收藏夹策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '游览器收藏夹策略设置成功'
            clear_wait_policy(device, 'account_browser_favorites_policy')

        policy_objs, _ = BrowserFavoritesPolicy.objects.get_or_create(account_id=pillar.get("account_id"))
        policy_objs.status = signal['status']
        policy_objs.jid = jid
        policy_objs.save()
        public.public(json.dumps(signal), chan='policy')

    if len(policy_cmd) > 3 and policy_cmd[3] == 'push_browser_policies':
        # 获取设置浏览器策略的结果
        signal['event_type'] = 'browser_policy'
        signal['arg'] = pillar
        signal['signal'] = pillar.get('signal')

        if num_failed:
            signal['status'] = 3
            signal['success'] = False
            signal['msg'] = '浏览器策略设置失败'
        else:
            signal['status'] = 1
            signal['success'] = True
            signal['msg'] = '浏览器策略设置成功'
            clear_wait_policy(device, 'browser_policy')

        # task_id = pillar.get('task_id')
        # if task_id:
        #     app.AsyncResult(task_id).update_state(states=states.SUCCESS if signal['success'] else states.FAILURE)
        public.public(json.dumps(signal), chan='policy')
        return

    if policy == 'shell-manage':
        if policy_cmd[2] == 'shell':
            signal['event_type'] = 'shell_policy'
            if num_failed:
                signal['success'] = False
                signal['status'] = 0
                signal['msg'] = '脚本文件下发失败'
                ShellDevicePolicy.objects.filter(device_id=device.id).update(status=3, jid=jid)
            else:
                signal['success'] = True
                signal['status'] = 1
                signal['msg'] = '脚本文件下发成功'
                ShellDevicePolicy.objects.filter(device_id=device.id).update(status=1, jid=jid)
            # task_id = pillar.get('task_id')
            # if task_id:
            #     app.AsyncResult(task_id).update_state(states=states.SUCCESS if signal['success'] else states.FAILURE)

            public.public(json.dumps(signal), chan='policy')
            return