import datetime
import logging
import urllib.request
import urllib.parse
import zipfile
import json
import requests
import ssl
import os
from django.conf import settings
import salt.client
from salt.client.ssh.client import SSHClient
from salt.states import schedule
from salt.client.ssh.shell import Shell
import configparser
from celery_task import tasks

ssl._create_default_https_context = ssl._create_unverified_context
requests.packages.urllib3.disable_warnings()

logger = logging.getLogger(__name__)


class SaltAPI(object):
    __token_id = ''
    _instance = None

    def __init__(self, url, user, passwd):
        self.__url = url
        self.__user = user
        self.__password = passwd
        token = self.get_token_id()
        # print(token)
        self.__token_id = token

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = object.__new__(cls)
        return cls._instance

    def get_token_id(self):
        # user login and get token id
        params = {'eauth': 'pam', 'username': self.__user, 'password': self.__password}
        # params = {'eauth': 'sharedsecret', 'username': self.__user, 'password': self.__password}
        obj = urllib.parse.urlencode(params).encode('utf-8')
        url = str(self.__url) + '/login'
        req = urllib.request.Request(url, obj)
        try:
            opener = urllib.request.urlopen(req, timeout=60)
            content = json.loads(opener.read().decode('utf-8'))
            token_id = content['return'][0]['token']
        except Exception as e:
            print(e)
            return ""
        return token_id

    def post_request(self, data, prefix='/'):
        url = str(self.__url) + prefix
        headers = {'X-Auth-Token': self.__token_id, 'Content-type': 'application/json'}
        try:
            # 解析成json
            data = bytes(json.dumps(data), 'utf8')
            req = urllib.request.Request(url, data, headers)
            opener = urllib.request.urlopen(req, timeout=1800)
            # print(opener.read())
            res = opener.read()
            if isinstance(res, bytes):
                content = json.loads(res.decode('utf-8'))
            else:
                content = json.loads(opener.read())
        except Exception as e:
            return str(e)
        return content

    def get_events(self):
        url = f'{self.__url}/events?token={self.__token_id}'
        response = requests.get(url, stream=True, verify=False)
        if response.status_code == 200:
            for event in response.iter_lines():
                if event:
                    str_event = event.decode('utf-8')
                    if str_event.startswith('tag: '):
                        yield {'tag': str_event[len('tag: '):]}
                    elif str_event.startswith('data: '):
                        yield json.loads(str_event[len('data: '):])
                    #elif str_event.startswith('retry: '):
                    #    yield str_event

    def list_all_key(self):
        # salt-key -L
        params = {'client': 'wheel', 'fun': 'key.list_all'}
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]['data']['return']
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def delete_key(self, node_name):
        # salt-key -d node_name
        params = {'client': 'wheel', 'fun': 'key.delete', 'match': node_name}
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]['data']['success']
        else:
            return {"status": False, "message": "salt api error : " + content}

    def accept_key(self, node_name):
        # salt-key -a -y
        params = {'client': 'wheel', 'fun': 'key.accept', 'match': node_name}
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]['data']['success']
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def reject_key(self, node_name):
        params = {'client': 'wheel', 'fun': 'key.reject', 'match': node_name}
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]['data']['success']
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def test_ping(self, tgt):
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'test.ping',  # 调用的模块方法名
            'batch': None,
            'tgt_type': "glob"
            # 'arg': ''  # 模块需要用的参数
        }
        content = self.post_request(params)
        logger.info(f"test_ping........{content}")
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def remote_noarg_execution(self, tgt, fun, types="tgt_type"):
        # Execute commands without parameters
        params = {'client': 'local', 'tgt': tgt, 'fun': fun, types: 'list'}
        content = self.post_request(params)
        if isinstance(content, dict):
            ret = content['return'][0][tgt]
            # 如果返回结果是空，说明salt可能版本小于2017.7.0的版本，使用次方法处理
            if not ret and types != "expr_form":  # types != "expr_form" 很重要确保最多执行两次，否则肯能发送死循环
                return self.remote_noarg_execution(tgt, fun, types="expr_form")
            return ret
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def remote_noarg_execution_notgt(self, tgt, fun, types="tgt_type"):
        # Execute commands without parameters
        params = {'client': 'local', 'tgt': tgt, 'fun': fun, types: 'list'}
        content = self.post_request(params)
        if isinstance(content, dict):
            ret = content['return'][0]
            # 如果返回结果是空，说明salt可能版本小于2017.7.0的版本，使用次方法处理
            if not ret and types != "expr_form":  # types != "expr_form" 很重要确保最多执行两次，否则肯能发送死循环
                return self.remote_noarg_execution_notgt(tgt, fun, types="expr_form")
            return ret
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def remote_execution(self, tgt, fun, arg, types="tgt_type"):
        # Command execution with parameters
        params = {'client': 'local', 'tgt': tgt, 'fun': fun, 'arg': arg, types: 'list'}
        content = self.post_request(params)
        if isinstance(content, dict):
            ret = content['return'][0][tgt]
            # 如果返回结果是空，说明salt可能版本小于2017.7.0的版本，使用次方法处理
            if not ret and types != "expr_form":  # types != "expr_form" 很重要确保最多执行两次，否则肯能发送死循环
                return self.remote_execution(tgt, fun, arg, types="expr_form")
            return ret
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def remote_execution_notgt(self, tgt, fun, arg, types="tgt_type"):
        # Command execution with parameters
        params = {'client': 'local', 'tgt': tgt, 'fun': fun, 'arg': arg, types: 'list'}
        content = self.post_request(params)
        if isinstance(content, dict):
            ret = content['return'][0]
            # 如果返回结果是空，说明salt可能版本小于2017.7.0的版本，使用次方法处理
            if not ret and types != "expr_form":  # types != "expr_form" 很重要确保最多执行两次，否则肯能发送死循环
                return self.remote_execution_notgt(tgt, fun, arg, types="expr_form")
            return ret
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def shell_remote_execution(self, tgt, arg, types="tgt_type"):
        # Shell command execution with parameters
        # Changed in version 2017.7.0: Renamed from expr_form to tgt_type
        params = {'client': 'local', 'tgt': tgt, 'fun': 'cmd.run', 'arg': arg, types: 'list'}
        content = self.post_request(params)
        if isinstance(content, dict):
            ret = content['return'][0]
            # 如果返回结果是空，说明salt可能版本小于2017.7.0的版本，使用次方法处理
            if not ret and types != "expr_form":  # types != "expr_form" 很重要确保最多执行两次，否则肯能发送死循环
                return self.shell_remote_execution(tgt, arg, types="expr_form")
            return ret
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def grain(self, tgt, arg):
        # salt "*" grains.item uuid os
        params = {'client': 'local', 'tgt': tgt, 'fun': 'grains.item', 'arg': arg}
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def grains(self, tgt):
        # salt '*'  grains.items
        params = {'client': 'local', 'tgt': tgt, 'fun': 'grains.items'}
        content = self.post_request(params)
        if isinstance(content, dict):
            return {"status": True, "message": "", "data": content['return'][0]}
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def pillar_items2(self, tgt):
        # Grains.items
        params = {'client': 'local', 'tgt': tgt, 'fun': 'pillar.items', 'arg': [], 'tgt_type': 'list'}
        content = self.post_request(params)
        if isinstance(content, dict):
            return {"status": True, "message": "", "data": content['return'][0]}
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    # def target_remote_execution(self, tgt, fun, arg):
    #     # Use targeting for remote execution
    #     params = {'client': 'local', 'tgt': tgt, 'fun': fun, 'arg': arg, 'expr_form': 'nodegroup'}
    #     content = self.post_request(params)
    #     if isinstance(content, dict):
    #         jid = content['return'][0]['jid']
    #         return jid
    #     else:
    #         return {"status": False, "message": "Salt API Error : " + content}

    def deploy(self, tgt, arg):
        # Module deployment
        params = {'client': 'local', 'tgt': tgt, 'fun': 'state.sls', 'arg': arg}
        return self.post_request(params)

    def async_deploy(self, tgt, arg):
        # Asynchronously send a command to connected minions
        params = {'client': 'local_async', 'tgt': tgt, 'fun': 'state.sls', 'arg': arg}
        content = self.post_request(params)
        if isinstance(content, dict):
            if content['return'] == [{}]:
                return {"status": False, "message": "组件未启动"}
            return content['return'][0]['jid']
        else:
            return {"status": False, "message": "salt api error : " + content}

    def schedule_enable_job(self, tgt, arg):
        """
        应用定时任务
        salt * schedule.enable_job job1
        :param tgt:
        :param arg:
        :return:
        """
        params = {'client': 'local', 'tgt': tgt, 'fun': 'schedule.enable_job', 'arg': arg}
        content = self.post_request(params)
        logger.info("salt %s schedule.enable_job content %s" % (tgt, content))

    def schedule_disable_job(self, tgt, arg):
        """
        关闭定时任务
        salt * schedule.disable_job job1
        :param tgt:
        :param arg:
        :return:
        """
        # Asynchronously send a command to connected minions
        params = {'client': 'local', 'tgt': tgt, 'fun': 'schedule.disable_job', 'arg': arg}
        content = self.post_request(params)
        logger.info("salt %s schedule.disable content %s" % (tgt, content))

    def schedule_delete(self, tgt, arg):
        """
        删除定时任务
        salt * schedule.delete job1
        :param tgt:
        :param arg:
        :return:
        """
        # Asynchronously send a command to connected minions
        params = {'client': 'local', 'tgt': tgt, 'fun': 'schedule.delete', 'arg': arg}
        content = self.post_request(params)
        logger.info("salt %s schedule.delete %s content %s" % (tgt, arg, content))

    def refresh_pillar(self, tgt):
        """
        刷新所有机器上的pillar
        salt * saltutil.refresh_pillar
        :param tgt:
        :param arg:
        :return:
        """
        params = {'client': 'local', 'tgt': tgt, 'fun': 'saltutil.refresh_pillar'}
        content = self.post_request(params)
        logger.info("salt %s saltutil.refresh_pillar content %s" % (tgt, content))

    def state_apply(self, tgt):
        """
        开启并行模式
        salt * state.apply 'webserver,database' parallel=True
        :param tgt:
        :param arg:
        :return:
        """
        params = {'client': 'local', 'tgt': tgt, 'fun': 'state.apply',
                  'arg': """ 'webserver,database' parallel=True """}
        content = self.post_request(params)
        logger.info("salt %s state.apply 'webserver,database' parallel=True  %s" % (tgt, content))

    def target_deploy(self, tgt, arg, types="tgt_type"):
        # Based on the list forms deployment
        params = {'client': 'local', 'tgt': tgt, 'fun': 'state.sls', 'arg': arg, types: 'list'}
        content = self.post_request(params)
        if isinstance(content, dict):
            try:
                ret = content.get("return")[0]
                # 如果返回结果是空，说明salt可能版本小于2017.7.0的版本，使用次方法处理
                if not ret and types != "expr_form":  # types != "expr_form" 很重要确保最多执行两次，否则肯能发送死循环
                    return self.target_deploy(tgt, arg, types="expr_form")
                return ret
            except Exception as e:
                return {"status": False, "message": str(e)}
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def pillar_items(self, tgt, arg=[], types="tgt_type"):
        # Get pillar item
        if arg:
            params = {'client': 'local', 'tgt': tgt, 'fun': 'pillar.item', 'arg': arg, types: 'list'}
        else:
            params = {'client': 'local', 'tgt': tgt, 'fun': 'pillar.items', 'arg': arg, types: 'list'}
        content = self.post_request(params)
        if isinstance(content, dict):
            try:
                ret = content.get("return")[0]
                # 如果返回结果是空，说明salt可能版本小于2017.7.0的版本，使用次方法处理
                if not ret and types != "expr_form":  # types != "expr_form" 很重要确保最多执行两次，否则肯能发送死循环
                    return self.pillar_items(tgt, arg=[], types="expr_form")
                return ret
            except Exception as e:
                return {"status": False, "message": str(e)}
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def jobs_list(self):
        # Get Cache Jobs Default 24h '''
        url = self.__url + '/jobs/'
        headers = {'X-Auth-Token': self.__token_id}
        req = urllib.request.Request(url, headers=headers)
        try:
            opener = urllib.request.urlopen(req)
            content = json.loads(opener.read())
        except Exception as e:
            return str(e)
        if isinstance(content, dict):
            jid = content['return'][0]
            return jid
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def jobs_info(self, arg):
        # Get Job detail info '''
        url = self.__url + '/jobs/' + arg
        headers = {'X-Auth-Token': self.__token_id}
        req = urllib.request.Request(url, headers=headers)
        try:
            opener = urllib.request.urlopen(req)
            content = json.loads(opener.read())
        except Exception as e:
            return str(e)
        if isinstance(content, dict):
            jid = content['return'][0]
            return jid
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def kill_job(self, tgt, jid):
        # salt '*' saltutil.kill_job jid
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'saltutil.kill_job',  # 调用的模块方法名
            'arg': [jid]  # 模块需要用的参数
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def kill_all_jobs(self, tgt):
        # salt '*' saltutil.kill_all_jobs
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'saltutil.kill_all_jobs',  # 调用的模块方法名
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def stats(self):
        # Expose statistics on the running CherryPy server
        url = self.__url + '/stats'
        headers = {'X-Auth-Token': self.__token_id}
        req = urllib.request.Request(url, headers=headers)
        try:
            opener = urllib.request.urlopen(req)
            content = json.loads(opener.read())
        except Exception as e:
            return str(e)
        if isinstance(content, dict):
            return content
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def runner_status(self, arg):
        # Return minion status
        params = {'client': 'runner', 'fun': 'manage.' + arg}
        content = self.post_request(params)
        if isinstance(content, dict):
            jid = content['return'][0]
            return jid
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def job_jid_status(self, arg):
        # Return minion status
        params = {'client': 'runner', 'fun': 'jobs.lookup_jid', 'jid': arg}
        content = self.post_request(params)
        if isinstance(content, dict):
            jid = content['return'][0]
            return jid
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def runner(self, arg):
        # Return minion status
        params = {'client': 'runner', 'fun': arg}
        content = self.post_request(params)
        if isinstance(content, dict):
            jid = content['return'][0]
            return jid
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def events(self):
        # SSE get job info '''
        url = self.__url + '/events'
        headers = {'X-Auth-Token': self.__token_id}
        req = requests.get(url, stream=True, headers=headers)
        return req

    def hook(self, tag=""):
        # Fire an event in Salt with a custom event tag and data
        url = self.__url + '/hook/' + tag
        headers = {'X-Auth-Token': self.__token_id}
        # data = json.dumps({"gitfs": "update"})
        # data = bytes(data, 'utf8')
        req = urllib.request.Request(url, headers=headers, method="POST")
        try:
            opener = urllib.request.urlopen(req)
            content = json.loads(opener.read())
        except Exception as e:
            return str(e)
        if isinstance(content, dict):
            return content
        else:
            return {"status": False, "message": "Salt API Error : " + content}

    def manage_status(self, tgt):
        # salt '*' manage.status
        # 获取minion状态【上下线状态】
        params = {
            'client': 'runner',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'manage.status',  # 调用的模块方法名
            # 'arg': ''  # 模块需要用的参数
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def pkg_list(self, tgt):
        # salt '*' pkg.list
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'pkg.list_pkgs',  # 调用的模块方法名
            # 'arg': ''  # 模块需要用的参数
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def pkg_version(self, tgt, package_name):
        # salt '*' pkg.version httpd
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'pkg.version',  # 调用的模块方法名
            'arg': [package_name]  # 模块需要用的参数
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def pkg_install(self, tgt, package_name):
        # salt '192.168.3.54' pkg.install httpd
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'pkg.install',  # 调用的模块方法名
            'arg': [package_name]  # 模块需要用的参数
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def pkg_installs(self, tgt, pkgs):
        #  salt '*' pkg.install pkgs='["gcc", {"lrzsz": "0.12.20-27.1.el6"}]'
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'pkg.install',  # 调用的模块方法名
            'arg': pkgs  # 模块需要用的参数
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def pkg_remove(self, tgt, package_name):
        #  salt '*' pkg.remove httpd'
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'pkg.remove',  # 调用的模块方法名
            'arg': [package_name]  # 模块需要用的参数
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def pkg_removes(self, tgt, pkgs):
        #  salt '*' pkg.remove pkgs='["tcpdump","lrzsz","tree"]'
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用, local_async:异步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'pkg.remove',  # 调用的模块方法名
            'arg': pkgs  # 模块需要用的参数
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def cmd_run(self, tgt, arg):
        # salt '*' cmd.run 'rpm -q vsftpd'
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'cmd.run',  # 调用的模块方法名
            'arg': arg  # 模块需要用的参数,如 'df -h'
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def cmd_script(self, tgt, arg):
        # salt '*' cmd.script /root/a.sh
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'cmd.script',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def cp_get_file(self, tgt, arg):
        # 向minion发送文件
        # salt '*' cp.get_file salt://salt-file/test.txt /root
        params = {
            'client': 'local',  # client 表示调用方式， local:同步调用
            'tgt': tgt,  # tgt 表示要操作的minion， 值为minion_id
            'fun': 'cp.get_file',  # 调用的模块方法名
            'arg': arg  # 模块需要用的参数,如 'df -h'
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]  # {'192.168.3.45': '/root/test.txt', '192.168.3.54': '/root/test.txt'}

        return {"status": False, "message": "salt api error : " + content}

    def service(self, tgt, options, arg=[]):
        # salt '*' service.enable nginx
        # salt '*' service.disable nginx
        # salt '*' service.reload nginx
        # salt '*' service.restart nginx
        # salt '*' service.start nginx
        # salt '*' service.stop nginx
        # salt '*' service.status nginx
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'service.%s' % options,
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def service_start(self, tgt, arg=[]):
        # salt '*' service.start nginx
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'service.start',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def service_stop(self, tgt, arg=[]):
        # salt '*' service.stop nginx
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'service.stop',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def service_status(self, tgt, arg=[]):
        # salt '*' service.status nginx
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'service.status',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def service_restart(self, tgt, arg=[]):
        # salt '*' service.restart nginx
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'service.restart',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def disk(self, tgt, option, arg=[]):
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'disk.%s' % option,
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]

        return {"status": False, "message": "salt api error : " + content}

    def sqlite(self, tgt, option, arg):
        # salt "*" sqlite3.tables /tmp/db.sqlite3
        # salt "*" sqlite3.fetch /tmp/db.sqlite3
        # salt "*" sqlite3.modify /tmp/db.sqlite3
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'sqlite3.%s' % option,
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def sqlite_fetch(self, tgt, arg):
        # salt "*" sqlite3.fetch /tmp/db.sqlite3
        params = {
            'client': 'local',
            'tgt': tgt,
            'fun': 'sqlite3.fetch',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]
        return {"status": False, "message": "salt api error : " + content}

    def ssh_ping_client(self, tgt):
        # salt_ssh连接 测试连通性
        params = {
            'client': 'ssh',
            'tgt': tgt,
            'fun': 'test.ping',
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]

        return {"status": False, "message": "salt api error : " + content}

    def ssh_client(self, tgt, arg):
        # salt_ssh连接执行命令
        params = {
            'client': 'ssh',
            'tgt': tgt,
            'fun': 'cmd.run',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]

        return {"status": False, "message": "salt api error : " + content}

    def ssh_state_apply(self, tgt, arg):
        # salt_ssh连接执行命令
        params = {
            'client': 'ssh',
            'tgt': tgt,
            'fun': 'state.sls',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]

        return {"status": False, "message": "salt api error : " + content}

    def ssh_grains(self, tgt, arg):
        # salt_ssh连接执行命令
        params = {
            'client': 'ssh',
            'tgt': tgt,
            'fun': 'grains.item',
            'arg': arg
        }
        content = self.post_request(params)
        if isinstance(content, dict):
            return content['return'][0]

        return {"status": False, "message": "salt api error : " + content}

    def api_wallpaper_add(self, image_path, image_name):
        url = f"http://{os.environ['MASTER_HOST']}:{os.environ['MASTER_FLASK_PORT']}/wallpaper/"
        payload = {}
        try:
            with open(image_path, 'rb') as f:
                files = [
                    ('image', (image_name, f, 'image/jpg'))
                ]
                headers = {}
                response = requests.request("POST", url, headers=headers, data=payload, files=files)
                logger.info(response.text)
                res = json.loads(response.text)
                if res.get('code') != 200:
                    return False
                return True
        except FileNotFoundError:
            logging.error(f"文件未找到: {image_path}")
        except requests.exceptions.RequestException as e:
            logging.error(f"请求错误: {e}")
        except Exception as e:
            logging.error(f"未知错误: {e}")

    def api_wallpaper_modify(self, image_name, re_image_name):
        url = f"http://{os.environ['MASTER_HOST']}:{os.environ['MASTER_FLASK_PORT']}/wallpaper/"
        payload = json.dumps({
            "image_name": image_name,
            "re_image_name": re_image_name
        })
        headers = {
            'Content-Type': 'application/json'
        }
        try:
            response = requests.request("PUT", url, headers=headers, data=payload)
            logger.info(response.text)
            res = json.loads(response.text)
            if res.get('code') != 200:
                return False
            return True
        except requests.exceptions.RequestException as e:
            logging.error(f"请求错误: {e}")
        except Exception as e:
            logging.error(f"未知错误: {e}")

    def api_wallpaper_delete(self, image_name):
        url = f"http://{os.environ['MASTER_HOST']}:{os.environ['MASTER_FLASK_PORT']}/wallpaper/"
        payload = json.dumps({
            "images": [
                image_name
            ]
        })
        headers = {
            'Content-Type': 'application/json'
        }
        try:
            response = requests.request("DELETE", url, headers=headers, data=payload)
            logger.info(response.text)
            res = json.loads(response.text)
            if res.get('code') != 200:
                return False
            return True
        except requests.exceptions.RequestException as e:
            logging.error(f"请求错误: {e}")
        except Exception as e:
            logging.error(f"未知错误: {e}")

    def api_certfile_add(self, certfile_path, certfile_name):
        url = f"http://{os.environ['MASTER_HOST']}:{os.environ['MASTER_FLASK_PORT']}/certfile/"
        payload = {}
        try:
            with open(certfile_path, 'rb') as f:
                files = [
                    ('cert', (certfile_name, f, 'application/octet-stream'))
                ]
                headers = {}
                response = requests.request("POST", url, headers=headers, data=payload, files=files)
                logger.info(response.text)
                res = json.loads(response.text)
                if res.get('code') != 200:
                    return False
                return True
        except FileNotFoundError:
            logging.error(f"文件未找到: {certfile_path}")
        except requests.exceptions.RequestException as e:
            logging.error(f"请求错误: {e}")
        except Exception as e:
            logging.error(f"未知错误: {e}")

    def api_certfile_delete(self, certfile_name):
        url = f"http://{os.environ['MASTER_HOST']}:{os.environ['MASTER_FLASK_PORT']}/certfile/"
        payload = json.dumps({
            "images": [
                certfile_name
            ]
        })
        headers = {
            'Content-Type': 'application/json'
        }
        try:
            response = requests.request("DELETE", url, headers=headers, data=payload)
            logger.info(response.text)
            res = json.loads(response.text)
            if res.get('code') != 200:
                return False
            return True
        except requests.exceptions.RequestException as e:
            logging.error(f"请求错误: {e}")
        except Exception as e:
            logging.error(f"未知错误: {e}")

    def api_files_download(self, download_path, save_path):
        url = f"http://{os.environ['MASTER_HOST']}:{os.environ['MASTER_FLASK_PORT']}/files/?path={download_path}"
        try:
            with requests.get(url, stream=True) as r:
                r.raise_for_status()  # 确保请求成功
                with open(save_path, 'wb') as f:
                    for chunk in r.iter_content(chunk_size=8192):
                        # 将数据写入文件
                        f.write(chunk)
            return True
        except requests.exceptions.RequestException as e:
            logging.error(f"请求错误: {e}")
        except Exception as e:
            logging.error(f"未知错误: {e}")

    def api_files_move(self, source_path, destination_path):
        url = f"http://{os.environ['MASTER_HOST']}:{os.environ['MASTER_FLASK_PORT']}/files/"
        payload = json.dumps({
            "source_path": source_path,
            "destination_path": destination_path
        })
        headers = {
            'Content-Type': 'application/json'
        }
        try:
            response = requests.request("PUT", url, headers=headers, data=payload)
            logger.info(response.text)
            res = json.loads(response.text)
            if res.get('code') != 200:
                return False
            return True
        except requests.exceptions.RequestException as e:
            logging.error(f"请求错误: {e}")
        except Exception as e:
            logging.error(f"未知错误: {e}")

# shell = Shell({'_ssh_version': (4, 9), 'ignore_host_keys': True}, '192.168.11.176', 'root', 22, '123', timeout=60)
# result = shell.exec_cmd('ls')
# print(result)

def mkdir_ssh(path):
    if not os.path.exists(path):
        # shutil.rmtree(url)  #删除目录，包括目录下的所有文件
        os.mkdir(path)


class SaltSHH(object):
    def __init__(self, roster_file=None):
        c_path = os.path.join(settings.SALT_CONF, 'etc/salt/master')
        master_opts = salt.config.client_config(c_path)
        if not roster_file:
            roster_file = os.path.join(settings.SALT_CONF, 'etc/salt/roster')
        master_opts['roster_file'] = roster_file
        # master_opts['pki_dir'] = '*' #  '/var/cache/salt/master'
        self.salt_ssh = SSHClient(c_path=c_path, mopts=master_opts)

    def state_sls_test(self, tgt):
        # res = self.salt_ssh.cmd(tgt=tgt, fun='state.sls', arg=['test.install_nginx'])
        res = self.salt_ssh.cmd(tgt=tgt, fun='test.ping')
        # celery_task_obj = tasks.salt_ssh_state_sls.delay(tgt, fun, arg)
        # celery_task_obj = tasks.test.delay(8,9)
        # return celery_task_obj
        return res

    def state_sls_install_minion(self, tgt='*', fun='state.sls', arg=['test.run_saltminion']):
        '''
        base
            pkg
                installMinion
                    install_minion.py
        e.g:
        res = c.cmd(tgt='*', fun='state.sls', arg=['test.run_saltminion'])
    res = {
        '192.168.3.82': {
            'jid': '20220424070358511712',
            'return': {
                'service_|-nginx_run_|-nginx_|-running': {
                    'name': 'nginx',
                    'changes': {},
                    'result': True,
                    'comment': 'The service nginx is already running',
                    '__sls__': 'test.run_saltminion',
                    '__run_num__': 0,
                    'start_time': '15:03:58.771364',
                    'duration': 58.944,
                    '__id__': 'nginx_run'
                }
            },
            'retcode': 0,
            'out': 'highstate',
            'id': '192.168.3.82',
            'fun': 'state.pkg',
            'fun_args': [
                '/var/tmp/.nfs_f7ae4e_salt/salt_state.tgz',
                'test=None',
                'pkg_sum=37c1d1c1ce8bc008c907ad859b944c158add34792cdb1fe33204b1b11703e021',
                'hash_type=sha256'
            ]
        }
    }
    res =  {
        '192.168.3.82': {
            'jid': '20220424083959657066',
            'return': {
                'service_|-nginx_run_|-nginx_|-running': {
                    'name': 'nginx',
                    'changes': {},
                    'result': False,
                    'comment': 'Failed to start transient scope unit: Interactive authentication required.',
                    '__sls__': 'test.run_saltminion',
                    '__run_num__': 0,
                    'start_time': '16:39:59.842502',
                    'duration': 311.398,
                    '__id__': 'nginx_run'
                }
            },
            'retcode': 2,
            'out': 'highstate',
            'id': '192.168.3.82',
            'fun': 'state.pkg',
            'fun_args': [
                '/var/tmp/.nfs_f7ae4e_salt/salt_state.tgz',
                'test=None',
                'pkg_sum=7b33d5d850b7791a0ed86d32a6c1353b46445e90b69708f6244283c97f931bc3',
                'hash_type=sha256'
            ]
        }
    }

    # bug info
    res = {
        '192.168.3.45': {
            'stdout': '',
            'stderr': 'xxxx',
            'retcode': 1
        }
    }
        '''
        # res = self.salt_ssh.cmd_iter(tgt=tgt, fun=fun, arg=arg)
        celery_task_obj = tasks.install_minion.delay(tgt, fun, arg)
        return celery_task_obj

        # data = {
        #     'task': {'fun': 'state.sls', 'arg': arg},
        #     'failed': [],
        #     'success': [],
        #     'error': []
        # }
        # for ip, v in res:
        #     if v.get('retcode') == 1 and v.get('stderr', None):
        #         fail_device = {'ip': ip, 'stderr': v.get('stderr')}
        #         data.get('error').append(fail_device)
        #
        #     elif v.get('retcode') == 0 and not v.get('stderr', None):
        #         success_device = {
        #             'ip': ip,
        #             'jid': v.get('jid'),
        #             'return': v.get('return')
        #         }
        #         data.get('success').append(success_device)
        #
        #     elif v.get('retcode') == 2 and not v.get('stderr', None):
        #         fail_device = {
        #             'ip': ip,
        #             'jid': v.get('jid'),
        #
        #         }
        #         data.get('failed').append(fail_device)

    def test_ping(self, tgt, fun='test.ping'):
        '''
         e.g:
            res = c.cmd(tgt='*', fun='test.ping')
            res = {
                '192.168.3.82': {
                    'jid': '20220424090826076953',
                    'return': True,
                    'retcode': 0,
                    'id': '192.168.3.82',
                    'fun': 'test.ping',
                    'fun_args': []
                },
                '192.168.3.45': {
                    'jid': '20220424090827635708',
                    'return': True,
                    'retcode': 0,
                    'id': '192.168.3.45',
                    'fun': 'test.ping',
                    'fun_args': []
                }
            }
        '''
        res = self.salt_ssh.cmd(tgt=tgt, fun=fun)
        return res

    def ssh_cmd(self, tgt, arg):
        res = self.salt_ssh.cmd(tgt=tgt, fun='cmd.run', arg=arg)
        return res

    def ssh_grains(self, tgt, arg):
        res = self.salt_ssh.cmd(tgt=tgt, fun='grains.item', arg=arg)
        return res


class DefaultPolicy(object):
    def __init__(self):
        self.DEFAULTPOLICY = settings.DEFAULTPOLICY
        cfg = configparser.ConfigParser()
        cfg.read(self.DEFAULTPOLICY)
        self.cfg = cfg

    def get_default_policy(self):
        default_policy = {}
        if self.cfg.has_section("Policy"):
            for item in self.cfg.items('Policy'):
                default_policy[item[0]] = item[1]
            # ssh_policy = self.cfg.get('Policy', 'SSH')
            # usb_policy = self.cfg.get('Policy', 'USB')
            # default_policy['SSH'] = ssh_policy
            # default_policy['USB'] = usb_policy
            return default_policy
        return default_policy

    def set_default_policy(self, option, value):
        self.cfg.set('Policy', option, str(value))
        with open(self.DEFAULTPOLICY, 'w') as fp:
            self.cfg.write(fp)


if __name__ == '__main__':
    '''
    docker run --name salt --hostname salt -p 4505-4506:4505-4506 -p 8000:8000 -e SALT_SHARED_SECRET=123456 -d saltstack/salt

    '''
    # salt_api = SaltAPI('https://127.0.0.1:8000', 'salt', '123456')
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    all_key = salt_api.list_all_key()
    # print(all_key)  # {'minions': ['192.168.3.54'], 'minions_pre': ['192.168.3.45'], 'minions_rejected': [], 'minions_denied': [], 'local': ['master.pem', 'master.pub']}
    #                 minions: 已认证的
    #                 minions_pre： 待认证的
    #                 minions_rejected： 拒绝的

    # res = salt_api.accept_key('192.168.3.54')
    # print(res)
    # res = salt_api.test_ping('192.168.3.54')
    # print(res)

    # res = salt_api.manage_status('192.168.3.54')
    # {'up': [], 'down': ['192.168.3.54']}
    # {'up': ['192.168.3.54'], 'down': []}
    # print(res)

    # res = salt_api.deploy("*", 'test.run_saltminion')
    # logging.info(res)

    # res = salt_api.jobs_list()
    # print(res)

    # res = salt_api.jobs_info('20220407103055699706')
    # print(res)

    # res = salt_api.events()
    # print(res)

    # res = salt_api.pkg_list('192.168.3.54')
    # print(res)

    # print(len(pkgs.get('192.168.3.54')))  # 1473

    # res = salt_api.pkg_version('192.168.3.54','git')
    # print(res)

    # res = salt_api.cmd_run('192.168.3.54','rpm -qa |grep java')
    # print(res)

    # res = salt_api.pillar_items('192.168.3.54',['cwd'])
    # res = salt_api.grains('192.168.3.54')
    # print(res)

    # res = salt_api.cp_get_file('*', ['salt://salt-file/a.sh', '/root'])
    # print(res)

    # res = salt_api.cmd_script('*', '/root/a.sh')
    # print(res)

    # import getpass
    #
    # getpass.getuser()
