#!/usr/bin/env python
# _*_ coding: UTF-8 -*-
"""
@Project :nfs_domain_mainserver
@File    :apt_download_deb.py
@IDE     :PyCharm
@Author  :huchl
@Date    :2024/12/19 16:28
"""
import logging
import os
import subprocess

logger = logging.getLogger(__name__)


class AptPackageManager:
    def __init__(self, mirror_sources, package_name_version_list, download_directory):
        self.mirror_sources = mirror_sources
        self.package_name_version_list = package_name_version_list
        self.download_directory = download_directory
        self.sources_file_path = '/etc/apt/sources.list'

    def add_mirror_sources(self):
        existing_sources = set()

        # Add new sources
        # 覆盖写入
        with open(self.sources_file_path, 'w') as f:
            for source in self.mirror_sources:
                logger.info(f"Adding source: {source}")
                f.write(source + '\n')

    def remove_lines_from_file(self):
        # 读取文件内容
        with open(self.sources_file_path, 'r', encoding='utf-8') as file:
            lines = file.readlines()

        # 使用集合加速查找效率
        lines_to_remove_set = set(self.mirror_sources)

        # 过滤掉包含指定内容的行
        filtered_lines = [line for line in lines if not any(remove_line in line for remove_line in lines_to_remove_set)]

        # 写回更新后的内容
        with open(self.sources_file_path, 'w', encoding='utf-8') as file:
            file.writelines(filtered_lines)

    def update_apt_cache(self):
        logger.info("Updating APT cache...")
        subprocess.run(["apt-get", "update"], check=True)

    def download_package(self):
        for package_dict in self.package_name_version_list:
            package_name = package_dict.get("main_package", None)
            package_version = package_dict.get("main_version", None)
            if package_version:
                package_to_download = f"{package_name.lower()}={package_version}"
            else:
                package_to_download = package_name.lower()
            try:
                logger.info(f"Downloading package: {package_to_download} to {self.download_directory}")
                subprocess.run(["apt-get", "download", package_to_download], cwd=self.download_directory, check=True)
            except Exception as exp:
                logger.info(f"下载软件失败: {package_to_download} 原因 {exp}")


    def create_download_directory(self):
        os.makedirs(self.download_directory, exist_ok=True)
        logger.info(f"Checked/created directory: {self.download_directory}")

    def run(self):
        self.create_download_directory()
        self.add_mirror_sources()  # Add mirror sources
        self.update_apt_cache()  # Update APT cache
        self.download_package()  # Download package

    def update_apt(self):
        """
        更新本地镜像源
        Returns:

        """
        is_update_apt = self.add_mirror_sources()  # Add mirror sources
        if is_update_apt == 1:
            self.update_apt_cache()  # Update APT cache


if __name__ == "__main__":
    # Configuration
    mirror_sources = [
        "deb [trusted=yes] http://172.30.98.132:9778/server server main",
    ]
    package_name = "apipost"
    package_version = "x.y.z"  # Replace with the desired version (e.g., "1.2.3")
    download_directory = "/tmp/"

    # Initialize and run the package manager
    apt_manager = AptPackageManager(mirror_sources, [{}], download_directory)
    apt_manager.run()
