import datetime
import logging
import json
import threading
from rest_framework.filters import OrderingFilter, SearchFilter
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.views import APIView
from rest_framework.parsers import FormParser  # 解析urlencoded数据包
from rest_framework.parsers import MultiPartParser

from nfs_domain_mainserver.settings import HOSTNAME, REPREPRO_SSH_PORT, REPREPRO_WEBSERVER_PORT
from patch_management.aptly_api_http import AptlyUtil
from software.apt_download_deb import AptPackageManager
from software.filter import SoftwarePolicyFilter, SoftwareInfoFilter
from users.role_permisson import get_org_permission_list
from utils.authentication import TokenAuthentication
from utils.sch_patch import PatchTask
from utils.software_policy_analyse import SoftwarePolicyTask
from utils.throttling import AnonRateThrottle, UserRateThrottle
from rest_framework import permissions
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.decorators import api_view, permission_classes, authentication_classes
from rest_framework.views import Response
from rest_framework.pagination import PageNumberPagination
from rest_framework.decorators import action
from rest_framework import status, viewsets, mixins

from utils.response import FitJSONRenderer
from software.deb_util import getDebInfo
from software.models import SoftwareInfoModel, SoftwareInstalledModel, SoftwareType, SoftwareOS, SoftwareArch, \
    SoftwareSource, SoftwarePolicy
from devices.models import Devices
from devices.serializers import DeviceSerializer
from software.serializers import SoftwareInfoModelSerializer, SoftwareTypeSerializer, SoftwareOSSerializer, \
    SoftwareArchSerializer, SoftwareSourceSerializer, SoftwarePolicySerializer, SoftwareModelSerializer
from organizations.models import OrgnizationEntity
from accounts.models import AccountEntity

from django.db.models import Q
from saltapi import tasks
from utils.RedisHelper import RedisHelper
from utils.pagination import get_page_data
from software import software_sync
from filemanager.models import FileManager
from utils.pagination import MyPagination
from software.deb_util import getDebSection
from django.conf import settings

logger = logging.getLogger(__name__)


# 上传deb包 处理
class DebUploadView(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)
    parsers_classes = [FormParser, MultiPartParser]

    def put(self, request, filename, format=None):

        if filename.endswith('.deb'):
            logger.info('upload deb to server')
        else:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '不是正确到deb安装包',
                'data': {}
            })

        file_obj = request.FILES.get('file')
        filename = file_obj.name
        full_filename = settings.DEB_FOLDER + filename
        logger.info("full_filename %s" % full_filename)
        # save file to local
        with open(full_filename, 'wb') as f:
            for chunk in file_obj.chunks():
                f.write(chunk)

        deb_info = getDebInfo(full_filename)
        if not deb_info:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '上传deb包无法解析',
                'data': {}
            })

        # check package and version exist
        arch_items = SoftwareArch.objects.filter(name=deb_info['architecture'])
        if arch_items.exists():
            s_count = SoftwareInfoModel.objects.filter(main_package=deb_info['package'],
                                                       main_version=deb_info['version'],
                                                       arch_id=arch_items.first().id,
                                                       is_delete=0)
            if s_count:
                return Response({
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': 'deb包已经存在',
                    'data': {}
                })
        else:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': f"包的架构暂不支持 {deb_info['architecture']}",
                'data': {}
            })

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '上传deb包完成',
            'data': deb_info
        })


# 软件信息分页列表  新增
class SoftwareInfoList(APIView):
    authentication_classes = (TokenAuthentication,)

    # permission_classes = (IsAuthenticated, IsAdminUser)
    # throttle_classes = (UserRateThrottle,)

    def add_software(self, codename, custome_filename, full_filename, main_package):
        redis_publish = RedisHelper()
        software_data = {
            'event_type': 'includedeb',
            'msg': 'include deb to reprepro',
            'filename': custome_filename,
            'full_filename': full_filename,
            'codename': codename,
            'main_package': main_package
        }
        # send to redis channel queue
        redis_publish.public(json.dumps(software_data), chan='reprepro_channel')

    def add_software_list(self, package_list):
        i_default_source_host = f"http://{HOSTNAME}:{REPREPRO_SSH_PORT}"
        # 查询补丁源信息 有的话 传递到工具类AptlyUtil(“xxxxx”)
        i_aptly_util = AptlyUtil(i_default_source_host)
        for i in package_list:
            i_aptly_util.batch_add_package(patch_file=i.get('full_filename'), repo_name="server-repo",
                                           filename=i.get('custome_filename'))

    def get_rate(self, softwares):
        device_num = Devices.objects.exclude(is_domain=0).all().count()
        all_install = SoftwareInstalledModel.objects.filter(main_package=softwares.main_package,
                                                            main_version=softwares.main_version,
                                                            installed_status=SoftwareInstalledModel.STATUS_INSTALLED,
                                                            ).exclude(device_info__is_domain=0)

        user_data = self.request.user
        org_list = get_org_permission_list(user_data)
        if org_list:
            device_num = Devices.objects.exclude(is_domain=0).filter(org_info__in=org_list)
            all_install = all_install.filter(device_info__org_info__in=org_list)

        success_install = list(all_install.values_list('device_info_id', flat=True))
        success_num = len(list(set(success_install)))
        return "%s/%s 终端" % (success_num, device_num)

    #  分页查询
    def get(self, request, *args, **kwargs):
        softwares = SoftwareInfoModel.objects.filter(is_delete=0).order_by("-last_update")
        query_name = request.query_params.get('name')
        source = request.query_params.get('source')
        arch_name = request.query_params.get('arch_name')
        os_name = request.query_params.get('os_name')
        type = request.query_params.get('type')
        if query_name:
            softwares = softwares.filter(Q(name__icontains=query_name) | Q(main_package__icontains=query_name))

        if source:
            softwares = softwares.filter(Q(source__name__contains=source))

        if arch_name:
            softwares = softwares.filter(arch_name=arch_name)

        if os_name:
            os_name_query = (
                    Q(os_name__icontains=f',{os_name},') |
                    Q(os_name=os_name) |
                    Q(os_name__icontains=f'{os_name},') |
                    Q(os_name__icontains=f',{os_name}')
            )
            softwares = softwares.filter(os_name_query)

        if type:
            softwares = softwares.filter(software_type__id=type)

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(softwares, request)
        except Exception as exp:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'error',
                'success': False,
                'data': []
            })
        res_list = []
        for software in ret:
            software_info = {
                'sid': software.id,
                'name': software.name,
                'main_package': software.main_package,
                'main_version': software.main_version,
                'description': software.description,
                'remarks': software.remarks,
                'created_at': software.created_at,
                'last_update': software.last_update,
                'custome_filename': software.custome_filename,
                'is_default_install': software.is_default_install,
                'type': software.software_type.name,
                'type_id': software.software_type.id,
                'source': software.source.name,
                'source_id': software.source.id,
                'rate': self.get_rate(software),
                'deb_size': software.deb_size,
                'os_name': software.os_name.split(',') if software.os_name else None,
                'os_id': software.os_id.split(',') if software.os_id else None,
                'arch_id': software.arch_id.split(',') if software.arch_id else None,
                'arch_name': software.arch_name.split(',') if software.arch_name else None,
            }
            res_list.append(software_info)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '查询软件信息列表完成',
            'total': len(softwares),
            'success': True,
            'data': res_list
        })

    def get_depende_deb(self, dependency_info, codename):
        package_list = []
        for deb_name in dependency_info:
            deb_path = settings.DEB_FOLDER + deb_name
            tmp_deb_info = getDebInfo(deb_path)
            tmp_section = getDebSection(deb_path)
            package = {
                "codename": codename,
                "custome_filename": deb_name,
                "full_filename": deb_path,
                "Section": tmp_section,
                "main_package": tmp_deb_info['package'],
            }
            package_list.append(package)
        return package_list

    # 新增软件信息
    def post(self, request, *args, **kwargs):
        """
        :param request:
            {
                "name": "test-python3-apt",
                "main_package": "python3-apt",
                "main_version": "2.2.1",
                "custome_filename": "python3-apt_2.2.1_amd64.deb",
                "dependency_info": [],
                "type_id": 8,
                "os_id": [
                    1,
                    4
                ],
                "arch_id": [
                    1,
                    2
                ]
            }
        :param args:
        :param kwargs:
        :return:
        """
        req_data = request.data
        name = req_data.get('name')
        custome_filename = req_data.get('custome_filename')
        description = req_data.get('description')
        remarks = req_data.get('remarks')
        type_id = req_data.get('type_id')
        os_id_list = req_data.get('os_id')
        arch_id_list = req_data.get('arch_id')
        arch_all = SoftwareArch.objects.filter(id__in=arch_id_list).all()
        os_all = SoftwareOS.objects.filter(id__in=os_id_list).all()

        full_filename = settings.DEB_FOLDER + custome_filename
        deb_info = getDebInfo(full_filename)

        if not deb_info:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '上传的软件包错误,无法解析',
                'data': {}
            })

        software = SoftwareInfoModel.objects.filter(main_package=deb_info['package'],
                                                    main_version=deb_info['version'],
                                                    arch_id=json.dumps(arch_id_list[-1], ensure_ascii=False),
                                                    is_delete=0).first()
        if software:
            # 判断 是否存在, 存在不能新建
            return Response({
                'code': status.HTTP_226_IM_USED,
                'msg': '软件包已存在'
            })
        else:
            software = SoftwareInfoModel.objects.create(
                name=name,
                main_version=deb_info['version'],
                main_package=deb_info['package'],
                custome_filename=custome_filename,
                description=description,
                remarks=remarks,
                software_type_id=type_id,
                source_id=1,
                created_at=datetime.datetime.now(),
                deb_size=deb_info['deb_size'],
                os_id=','.join([str(num) for num in os_id_list]) if os_id_list else None,
                arch_id=','.join([str(num) for num in arch_id_list]) if arch_id_list else None,
                arch_name=','.join(arch_all.values_list('name', flat=True)) if arch_all.values_list('name',
                                                                                                    flat=True) else None,
                os_name=','.join(os_all.values_list('name', flat=True)) if os_all.values_list('name',
                                                                                              flat=True) else None,
            )

        Section = getDebSection(full_filename)
        package_list = [{"custome_filename": custome_filename,
                         "full_filename": full_filename,
                         "Section": Section,
                         "main_package": deb_info['package'],
                         }]

        self.add_software_list(package_list)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '新增软件信息成功',
            'data': {
                "id": software.id
            }
        })

    # 删除软件信息
    def delete(self, request, *args, **kwargs):
        for i in request.data.get('id'):
            try:
                software = SoftwareInfoModel.objects.filter(pk=i, is_delete=0).only('main_version',
                                                                                    'main_package').first()
            except Exception as e:
                return Response({
                    'code': status.HTTP_404_NOT_FOUND,
                    'msg': 'id %s 软件信息不存在' % i
                })
            software_delete([software])
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除软件信息完成'
        })


def software_delete(software_list):
    for software in software_list:
        software_installed = SoftwareInstalledModel.objects.filter(main_package=software.main_package,
                                                                   main_version=software.main_version).first()
        if software_installed:
            software_installed.delete()
        if software.source.id == 1:
            # 上传包到aptly源
            i_default_source_host = f"http://{HOSTNAME}:{REPREPRO_SSH_PORT}"
            # 查询补丁源信息 有的话 传递到工具类AptlyUtil(“xxxxx”)
            i_aptly_util = AptlyUtil(i_default_source_host)

            # 平台源清除reprepro中的软件
            # 删除aptly源中对应软件包
            # remove_deb(software.main_package, arch.name)
            keys = i_aptly_util.query_package_keys('server-repo', software.main_package, software.main_version,
                                                   None)
            i_aptly_util.batch_del_package('server-repo', keys)

        software.is_delete = 1
        software.save()


@api_view(["DELETE"])
@permission_classes((permissions.AllowAny,))
def software_batch_delete(request):
    """
    软件批量删除
    :param request:
    :return:
    """
    try:
        query_name = request.data.get('name')
        source = request.data.get('source')
        arch = request.data.get('arch')
        os = request.data.get('os')
        type = request.data.get('type')
        ids_list_str = request.data.get('ids_list')
        is_all = request.data.get('is_all', 0)
        if "," in ids_list_str:
            ids_list = ids_list_str.split(",")
        else:
            ids_list = [ids_list_str]
        if is_all == 1:
            softwares = SoftwareInfoModel.objects.filter(is_delete=0).order_by("-last_update")
            if query_name:
                softwares = softwares.filter(Q(name__contains=query_name) | Q(main_version__contains=query_name))

            if source:
                softwares = softwares.filter(Q(source__name__contains=source))

            if arch:
                softwares = softwares.filter(arch__id=arch)

            if os:
                softwares = softwares.filter(os__id=os)

            if type:
                softwares = softwares.filter(software_type__id=type)

            if query_name:
                software_list = softwares.filter(name=query_name, is_delete=0).only('main_version', 'main_package')
            else:
                software_list = softwares.filter(is_delete=0).only('main_version', 'main_package')
        else:
            software_list = SoftwareInfoModel.objects.filter(id__in=ids_list, is_delete=0).only('main_version',
                                                                                                'main_package')
        software_delete(software_list)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除软件信息完成'
        })
    except Exception as e:
        return Response({
            'code': status.HTTP_404_NOT_FOUND,
            'msg': 'ids_list %s 软件信息不存在' % ids_list
        })


# 软件信息 详细 修改  删除
class SoftwareInfoDetail(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 软件详细信息
    def get(self, request, *args, **kwargs):
        uid = kwargs.get('sid')
        try:
            software = SoftwareInfoModel.objects.get(pk=uid, is_delete=0)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '查询不到软件信息'
            })
        # ser_account = AccountEntitySerializer(account, many=True)
        if software:
            res_obj = {
                'sid': software.id,
                'name': software.name,
                'main_package': software.main_package,
                'main_version': software.main_version,
                'description': software.description,
                'remarks': software.remarks,
                'created_at': software.created_at,
                'last_update': software.last_update,
                'custome_filename': software.custome_filename,
                'is_default_install': software.is_default_install,
                'type': software.software_type.name,
                'type_id': software.software_type.id,
                'source': software.source.name,
                'source_id': software.source.id,
                # TODO
                'rate': "0/15 终端",
                'deb_size': software.deb_size,
                'os_name': software.os_name.split(',') if software.os_name else None,
                'os_id': software.os_id.split(',') if software.os_id else None,
                'arch_id': software.arch_id.split(',') if software.arch_id else None,
                'arch_name': software.arch_name.split(',') if software.arch_name else None,
            }

            return Response({
                'code': status.HTTP_200_OK,
                'msg': '查询软件信息完成',
                'data': res_obj
            })
        else:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '获取不到软件信息',
                'data': {}
            })

    # 更新软件信息
    def put(self, request, *args, **kwargs):
        uid = kwargs.get('sid')
        req_data = request.data
        try:
            software = SoftwareInfoModel.objects.get(pk=uid, is_delete=0)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '未查询到指定到软件信息'
            })

        main_package = req_data.get('main_package')
        if main_package and main_package != software.main_package:
            # 判断 是否存在
            check_set = SoftwareInfoModel.objects.filter(main_package=main_package,
                                                         main_version=req_data.get('main_version'), is_delete=0)
            if check_set:
                return Response({
                    'code': status.HTTP_226_IM_USED,
                    'msg': '软件信息已存在'
                })
            software.main_package = main_package

        name = req_data.get('name')
        if name and name != software.name:
            # 判断 是否存在
            check_set = SoftwareInfoModel.objects.filter(name=name, is_delete=0)
            if check_set:
                return Response({
                    'code': status.HTTP_226_IM_USED,
                    'msg': '软件信息已存在'
                })
            software.name = name

        if req_data.get('main_version'):
            software.main_version = req_data.get('main_version')
        if req_data.get('custome_filename'):
            software.custome_filename = req_data.get('custome_filename')
        if req_data.get('dependency_info') is not None:
            software.dependency_info = req_data.get('dependency_info')
        if req_data.get('description'):
            software.description = req_data.get('description')
        if req_data.get('remarks'):
            software.remarks = req_data.get('remarks')
        if req_data.get('type_id'):
            software.software_type_id = req_data.get('type_id')

        if req_data.get('os_id'):
            software.os.clear()
            os_id = req_data.get('os_id')
            for software_os_id in os_id:
                software_os = SoftwareOS.objects.filter(id=software_os_id).first()
                software.os.add(software_os)

        if req_data.get('arch_id'):
            software.arch.clear()
            arch_id = req_data.get('arch_id')
            arch_all = SoftwareArch.objects.filter(id__in=arch_id).all()
            for software_arch in arch_all:
                software.arch.add(software_arch)
        software.save()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '更新软件信息完成'
        })


#  软件默认安装策略   软件列表
class DefaulInstalledSoftwareInfos(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 查询全部
    def get(self, request, *args, **kwargs):
        # softwares = SoftwareInfoModel.objects.filter(is_default_install=1)
        softwares = SoftwareInfoModel.objects.filter(is_delete=0)

        res_list = []
        for software in softwares:
            res_list.append({
                'sid': software.id,
                'name': software.name,
                'main_package': software.main_package,
                'main_version': software.main_version,
                'dependency_info': software.dependency_info,
                'description': software.description,
                'remarks': software.remarks,
                'created_at': software.created_at,
                'last_update': software.last_update,
                'custome_filename': software.custome_filename,
                'is_default_install': software.is_default_install,
            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '查询软件信息完成',
            'total': len(softwares),
            'success': True,
            'data': res_list
        })


# 软件默认安装策略   添加   删除
class DefaulInstalledSoftwareInfoDetail(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 添加默认安装软件
    def put(self, request, *args, **kwargs):
        sid = kwargs.get('sid')
        req_data = request.data
        try:
            software = SoftwareInfoModel.objects.get(pk=sid, is_delete=0)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '软件信息不存在'
            })
        # 设置标志位
        software.is_default_install = 1

        software.save()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '添加默认安装软件完成'
        })

    # 删除默认安装软件
    def delete(self, request, *args, **kwargs):
        uid = kwargs.get('sid')
        try:
            software = SoftwareInfoModel.objects.get(pk=uid, is_delete=0)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '软件信息不存在'
            })
        # 设置标志位
        software.is_default_install = 0

        software.save()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除默认安装软件完成'
        })


# 软件安装 一个软件 一个终端  单独安装  卸载
class SoftwareSingleInstall(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def post(self, request, *args, **kwargs):

        req_data = request.data
        sid = req_data.get('software_id')
        did = req_data.get('device_id')

        try:
            software = SoftwareInfoModel.objects.get(pk=sid, is_delete=0)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '软件信息不存在'
            })

        try:
            device = Devices.objects.get(pk=did)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '终端设备信息不存在'
            })
        for arch in software.arch.all():
            if arch.name == device.os_arch:
                break
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '此软件适配的架构与终端系统架构不符'
            })
        celery_task = tasks.pkg_install_policy.delay(
            DeviceSerializer(device).data,
            software.id
        )
        logger.info(celery_task)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '软件安装命令发送成功'
        })

    def put(self, request, *args, **kwargs):

        req_data = request.data
        sid = req_data.get('software_id')
        did = req_data.get('device_id')

        try:
            software = SoftwareInfoModel.objects.get(pk=sid, is_delete=0)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '软件信息不存在'
            })

        try:
            device = Devices.objects.get(pk=did)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '终端设备不存在'
            })

        softwareInstalledInfo = SoftwareInstalledModel.objects.filter(device_info_id=device.id,
                                                                      main_package=software.main_package,
                                                                      main_version=software.main_version).first()
        # if softwareInstalledInfo:
        #     softwareInstalledInfo.is_histroy_install = 1
        #     softwareInstalledInfo.save()
        if softwareInstalledInfo is None:
            err_msg = '软件安装信息不存在'
            logger.error(err_msg)
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': err_msg
            })

        celery_task = tasks.pkg_remove_policy.delay(
            DeviceSerializer(device).data,
            software.id
        )
        logger.info(celery_task)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '软件卸载命令发送成功'
        })


# 软件批量安装
class SoftwareBatchInstall(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def post(self, request, *args, **kwargs):

        req_data = request.data
        software_ids = req_data.get('software_ids')
        device_ids = []
        org_ids = req_data.get('org_ids')
        logger.info('批量安装软件部门id %s', org_ids)
        if not org_ids or not software_ids:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '请求参数不完整'
            })

        softwares = []
        devices = []
        for sid in software_ids:
            try:
                software = SoftwareInfoModel.objects.get(pk=sid, is_delete=0)
            except Exception as e:
                logger.error('软件信息不存在 ,id is %s', sid)
                return Response({
                    'code': status.HTTP_404_NOT_FOUND,
                    'msg': '软件信息不存在'
                })
            softwares.append(software)
        # 通过部门ID --> 终端用户--> 设备 的关系 查找要安装软件到设备
        for oid in org_ids:
            try:
                org = OrgnizationEntity.objects.get(pk=oid)
            except Exception as e:
                logger.error('部门信息不存在 ,id is %s', oid)
                return Response({
                    'code': status.HTTP_404_NOT_FOUND,
                    'msg': '部门信息不存在'
                })
            accounts = AccountEntity.objects.filter(org_info=org)
            for account in accounts:
                account_devices = Devices.objects.filter(accounts_id=account.id)
                for device in account_devices:
                    devices.append(device)
        if not devices:
            return Response({
                'code': status.HTTP_406_NOT_ACCEPTABLE,
                'msg': '部门下没有设备信息可用'
            })

        for dev in devices:
            softwareInstalledInfos = []
            for soft in softwares:
                softwareInstalledInfo = SoftwareInstalledModel.objects.filter(main_package=soft.main_package,
                                                                              main_version=soft.main_version).filter(
                    device_info=dev).first()
                if not softwareInstalledInfo:
                    softwareInstalledInfo = SoftwareInstalledModel()
                    softwareInstalledInfo.software_info = soft
                    softwareInstalledInfo.name = soft.name
                    softwareInstalledInfo.main_package = soft.main_package
                    softwareInstalledInfo.main_version = soft.main_version
                    softwareInstalledInfo.device_info = dev
                    softwareInstalledInfo.save()
                softwareInstalledInfos.append(softwareInstalledInfo.id)
                # pkg_name = software.main_package
                # tgt_devices = [dev]

                # celery_task = tasks.pkg_install_policy.delay(DeviceSerializer(tgt_devices,many=True).data,pkg_name,softwareInstalledInfo.id)
                # logger.info(celery_task)
            celery_task = tasks.pkg_install_batch_policy.delay(
                DeviceSerializer(dev).data,
                # SoftwareInstalledSerializer(softwareInstalledInfos, many=True).data
                softwareInstalledInfos
            )
            logger.info(celery_task)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '批量安装命令已发送'
        })


# 软件安装信息
class SoftwareInstalledList(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, *args, **kwargs):
        device_id = kwargs.get('did')
        if not device_id:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '缺少设备参数',
                'success': False,
                'data': []
            })
        softwareInstalleds = SoftwareInstalledModel.objects.all()
        # device_id = request.query_params.get('device_id')
        if device_id:
            softwareInstalleds = softwareInstalleds.filter(device_info_id=device_id).exclude(software_info__isnull=True)
        else:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'device id not found'
            })
        res_list = []
        for softwareInstalled in softwareInstalleds:
            res_list.append({
                'sid': softwareInstalled.software_info.id,
                'name': softwareInstalled.name,
                'main_package': softwareInstalled.main_package,
                'main_version': softwareInstalled.main_version,
                'installed_status': softwareInstalled.installed_status,
            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '获取软件安装情况信息成功',
            'total': len(softwareInstalleds),
            'success': True,
            'data': res_list
        })


# 所有设备列表  不分页
class DeviceInfos(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    # 查询全部
    def get(self, request, *args, **kwargs):
        # softwares = SoftwareInfoModel.objects.filter(is_default_install=1)
        devices = Devices.objects.exclude(is_domain=0).all()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': 'get software list ok',
            'total': len(devices),
            'success': True,
            'data': DeviceSerializer(devices, many=True).data
        })


class SoftwareTypeAPIView(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        is_all = self.request.query_params.get('is_all', None)
        if is_all:
            software_type = SoftwareType.objects.all()
        else:
            software_type = SoftwareType.objects.filter(is_delete=0).all()
        return Response({
            'code': status.HTTP_200_OK,
            'data': SoftwareTypeSerializer(software_type, many=True).data
        })

    def post(self, request):
        """
        :param request:
            {
                "data":[
                    {
                        "id": 11,
                        "name": "test1",
                        "type": 1,
                        "is_delete":0
                    },
                    {
                        "id": 12,
                        "name": "test2",
                        "type": 1,
                        "is_delete":0
                    }
                ]
            }
        :return:
        """
        data_list = request.data.get('data')
        for data in data_list:
            if not data.get("id"):
                SoftwareType.objects.create(**data)
            else:
                SoftwareType.objects.filter(id=data["id"]).update(**data)
                if data.get("is_delete") == 1:
                    software_type_id = data.get('id')
                    other_id = SoftwareType.objects.get(name='其他').id
                    SoftwareInfoModel.objects.filter(software_type_id=software_type_id).update(
                        software_type_id=other_id)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '添加成功'
        })

    def put(self, request):
        for d in request.data.get('data'):
            id = d.get('id')
            name = d.get('name')

            if SoftwareType.objects.filter(name=name).first():
                return Response({
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '软件类型已存在'
                })

            software_type = SoftwareType.objects.filter(id=id).first()
            software_type.name = name
            software_type.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '修改成功'
        })

    def delete(self, request):
        for i in request.data.get('id'):
            software_type = SoftwareType.objects.filter(id=i).first()
            if software_type:
                # return Response({
                #     'code': status.HTTP_400_BAD_REQUEST,
                #     'msg': '软件类型不存在'
                # })
                software_type.is_delete = 1
                software_type.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除成功'
        })


class SoftwareOSAPIView(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        is_all = self.request.query_params.get('is_all', None)
        if is_all:
            software_os = SoftwareOS.objects.all()
        else:
            software_os = SoftwareOS.objects.filter(is_delete=0).all()

        return Response({
            'code': status.HTTP_200_OK,
            'data': SoftwareOSSerializer(software_os, many=True).data
        })

    def post(self, request):
        """
        :param request:
         {
                "data":[
                    {
                        "name": "test1",
                        "is_delete":0
                    },
                    {
                        "name": "test2",
                        "is_delete":0
                    }
                ]
            }
        :return:
        """
        data_list = request.data.get('data')
        for data in data_list:
            if not data.get("id"):
                SoftwareOS.objects.create(**data)
            else:
                if data.get("is_delete") == 1:
                    software_os_id = data.get('id')
                    other_id = SoftwareOS.objects.get(name='其他')
                    res_data = SoftwareOS.objects.get(id=software_os_id).softwareinfomodel_set.all()
                    for i in res_data:
                        i.os.add(other_id)
                SoftwareOS.objects.filter(id=data["id"]).update(**data)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '添加成功'
        })

    def put(self, request):
        for d in request.data.get('data'):
            id = d.get('id')
            name = d.get('name')

            if SoftwareOS.objects.filter(name=name).first():
                return Response({
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '软件适配系统已存在'
                })

            software_os = SoftwareOS.objects.filter(id=id).first()
            software_os.name = name
            software_os.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '修改成功'
        })

    def delete(self, request):
        for i in request.data.get('id'):
            software_os = SoftwareOS.objects.filter(id=i).first()
            if software_os:
                # return Response({
                #     'code': status.HTTP_400_BAD_REQUEST,
                #     'msg': '软件适配系统不存在'
                # })
                software_os.is_delete = 1
                software_os.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除成功'
        })


class SoftwareArchAPIView(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        software_arch = SoftwareArch.objects.filter(is_delete=0).all()
        return Response({
            'code': status.HTTP_200_OK,
            'data': SoftwareArchSerializer(software_arch, many=True).data
        })

    def post(self, request):
        """
        :param request:
            {
                "data":[
                    {
                        "name": "x86_64",
                        "is_delete":0
                    },
                    {
                        "name": "sw_64",
                        "is_delete":0
                    }
                ]
            }
        :return:
        """
        data_list = request.data.get('data')
        for data in data_list:
            if not data.get("id"):
                SoftwareArch.objects.create(**data)
            else:
                SoftwareArch.objects.filter(id=data["id"]).update(**data)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '添加成功'
        })

    def put(self, request):
        for d in request.data.get('data'):
            id = d.get('id')
            name = d.get('name')

            if SoftwareArch.objects.filter(name=name).first():
                return Response({
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': '软件适配架构已存在'
                })

            software_arch = SoftwareArch.objects.filter(id=id).first()
            software_arch.name = name
            software_arch.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '修改成功'
        })

    def delete(self, request):
        for i in request.data.get('id'):
            software_arch = SoftwareArch.objects.filter(id=i).first()
            if software_arch:
                # return Response({
                #     'code': status.HTTP_400_BAD_REQUEST,
                #     'msg': '软件适配架构不存在'
                # })
                software_arch.is_delete = 1
                software_arch.save()
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除成功'
        })


class SoftwareSourceAPIView(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        name = request.query_params.get('name')
        if name:
            source = SoftwareSource.objects.filter(name__icontains=name, is_delete=0).all()
        else:
            source = SoftwareSource.objects.filter(is_delete=0).all()

        page_status, res_list, total = get_page_data(source, request, SoftwareSourceSerializer)
        if page_status:
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '完成分页查询',
                'total': total,
                'success': True,
                'data': res_list
            })
        else:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '查询失败'
            })

    def post(self, request):
        name = request.data.get('name')
        address = request.data.get('address')
        source_status = request.data.get('status', 0)
        remarks = request.data.get('remarks', "")
        link_status = request.data.get('link_status', 0)
        # 软件源端口号，默认为80，此处为兼容端口号改变
        domain_port = request.data.get('domain_port', "")
        source_port = request.data.get('source_port', 80)
        if SoftwareSource.objects.filter(name=name,is_delete=0).exists():
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '软件源名称重复,请重试!'
            })

        # 用户创建的软件源默认为1，自定义软件源
        source_type = 1
        create_data = {
            "name": name,
            "address": address,
            "type": source_type,
            "status": source_status,
            "remarks": remarks,
            "link_status": link_status,
            "domain_port": domain_port,
            "source_port": source_port
        }

        source = SoftwareSource.objects.create(**create_data)

        mirror_sources = [
            f"deb [trusted=yes] http://{HOSTNAME}:{REPREPRO_WEBSERVER_PORT}/server server main", ]

        if source_type == 1:
            #deb [ trusted=yes ] http://40.50.10.104/nfscdos 3.0 softwarecenter drivecenter depend tiger
            mirror_string = f"deb [trusted=yes] http://{address}:{source_port}/nfscdos 3.0 softwarecenter drivecenter depend tiger"
        else:
            mirror_string = f"deb [trusted=yes] http://{address}:{domain_port}/server server main"
        mirror_sources.append(mirror_string)

        apt_manager = AptPackageManager(mirror_sources, [], None)
        update_apt_thread = threading.Thread(target=apt_manager.update_apt)
        update_apt_thread.start()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '添加成功',
            'data': {
                "id": source.id
            }
        })

    def put(self, request):
        data = request.data
        if not data.get("id"):
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': 'id不能为空'
            })
        else:
            if not SoftwareSource.objects.filter(id=data["id"]):
                return Response({
                    'code': status.HTTP_400_BAD_REQUEST,
                    'msg': 'id不存在'
                })
            SoftwareSource.objects.filter(id=data["id"]).update(**data)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '修改成功'
        })

    def delete(self, request):
        name = request.data.get('name')
        ids_list_str = request.data.get('ids_list')
        is_all = request.data.get('is_all', 0)
        if "," in ids_list_str:
            ids_list = ids_list_str.split(",")
        else:
            ids_list = [ids_list_str]
        if is_all == 1:
            if name:
                res_data = SoftwareSource.objects.filter(~Q(id=1)).filter(name=name, is_delete=0)
            else:
                res_data = SoftwareSource.objects.filter(~Q(id=1)).filter(is_delete=0)
        else:
            res_data = SoftwareSource.objects.filter(~Q(id=1)).filter(id__in=ids_list, is_delete=0)

        mirror_sources = []
        for item in res_data:
            if item.type == 1:
                # deb [ trusted=yes ] http://40.50.10.104/nfscdos 3.0 softwarecenter drivecenter depend tiger
                mirror_string = f"deb [trusted=yes] http://{item.address}:{item.source_port}/nfscdos 3.0 softwarecenter drivecenter depend tiger"
            else:
                mirror_string = f"deb [trusted=yes] http://{item.address}:{item.domain_port}/server server main"
            mirror_sources.append(mirror_string)

        apt_manager = AptPackageManager(mirror_sources, [], None)
        remove_apt_thread = threading.Thread(target=apt_manager.remove_lines_from_file)
        remove_apt_thread.start()

        source_id_list = res_data.values_list('id',flat=True)
        SoftwareInfoModel.objects.filter(source_id__in=source_id_list).delete()
        res_data.update(is_delete=1)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除成功'
        })


@api_view(["PUT"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def source_update_status(request):
    name = request.data.get('name')
    ids_list_str = request.data.get('ids_list')
    source_status = request.data.get('status')
    is_all = request.data.get('is_all', 0)
    if "," in ids_list_str:
        ids_list = ids_list_str.split(",")
    else:
        ids_list = [ids_list_str]
    if is_all == 1:
        if name:
            SoftwareSource.objects.filter(name=name, is_delete=0).update(status=source_status)
        else:
            SoftwareSource.objects.filter(is_delete=0).update(status=source_status)
    else:
        SoftwareSource.objects.filter(id__in=ids_list, is_delete=0).update(status=source_status)
    return Response({
        'code': status.HTTP_200_OK,
        'msg': '修改成功'
    })


@api_view(["GET"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def test_software_source(request):
    """
    连接测试软件源
    :param request:
    :return:
    """
    domain_port = request.query_params.get('domain_port')
    address = request.query_params.get('address')
    if None in [domain_port, address]:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': 'address或者domain_port不能为空'
        })
    address_url = f"http://{address}:{domain_port}"
    actives = software_sync.test_software_source(address_url)
    if actives:
        msg = '软件源连接测试成功'
        return Response({
            'code': status.HTTP_200_OK,
            'msg': msg,
            'data': {
                "actives": actives
            }
        })
    else:
        msg = '软件源连接测试失败'
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': msg,
            'data': {
                "actives": actives
            }
        })


@api_view(["GET"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def sync_software_source(request):
    """
    同步软件源
    :param request:
    :return:
    """
    id = request.query_params.get('id')
    if not id:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': 'id不能为空'
        })
    source = SoftwareSource.objects.filter(id=id).first()
    if source.status == 0:
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '软件源已被禁用无法同步'
        })
    actives = software_sync.sync_software_source(id)
    if actives:
        msg = '软件源同步成功'
    else:
        msg = '软件源同步失败'

    return Response({
        'code': status.HTTP_200_OK,
        'msg': msg,
        'data': {
            "actives": actives
        }
    })


@api_view(["POST"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def add_software_task(request):
    """
    下发软件任务(支持批量)
    :param request:
        {
            "ids_list":"171,172"
        }
    :return:
    """
    is_all = request.data.get('is_all')
    ids_list_str = request.data.get('ids_list')

    if is_all is None or not ids_list_str:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': 'is_all和ids_list 为必输选项不能为空'
        })

    if is_all == 0:
        if "," in ids_list_str:
            ids_list = ids_list_str.split(",")
        else:
            ids_list = [ids_list_str]
        softwares = SoftwareInfoModel.objects.filter(pk__in=ids_list, is_delete=0)
    else:
        softwares = SoftwareInfoModel.objects.filter(is_delete=0)

    query_name = request.data.get('name')
    source = request.data.get('source')
    arch = request.data.get('arch')
    os = request.data.get('os')
    type = request.data.get('type')

    if query_name:
        softwares = softwares.filter(Q(name__contains=query_name) | Q(main_version__contains=query_name))

    if source:
        softwares = softwares.filter(Q(source__name__contains=source))

    if arch:
        softwares = softwares.filter(arch__id=arch)

    if os:
        softwares = softwares.filter(os__id=os)

    if type:
        softwares = softwares.filter(software_type__id=type)

    return_list = []
    for software in softwares:

        return_list.append({
            "id": software.id,
            "name": software.custome_filename,
        })

    return Response({
        'code': status.HTTP_200_OK,
        'data': {
            "file_data": return_list
        }
    })


class SoftwarePolicyViewSet(viewsets.ModelViewSet):
    queryset = SoftwarePolicy.objects.all()
    serializer_class = SoftwarePolicySerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = SoftwarePolicyFilter
    filterset_fields = '__all__'
    ordering_fields = '__all__'
    ordering = ('-update_time',)

    def filter_queryset(self, queryset):
        queryset = super(SoftwarePolicyViewSet, self).filter_queryset(self.queryset)
        user_data = self.request.user
        if user_data.is_superuser != 1:
            queryset = queryset.filter(creator=user_data.username)
        return queryset

    def update_device_range(self):
        """动态变更设备选择范围"""
        for item in self.queryset.values():
            apply_range_value = SoftwarePolicyTask().get_apply_range_list(row=item)
            self.queryset.filter(id=item['id']).update(apply_range_value=apply_range_value)

    @staticmethod
    def _change_item_(item):
        apply_range_value = SoftwarePolicyTask().get_apply_range_list(row=item)
        item['apply_range_value'] = apply_range_value
        item['serial_number'] = datetime.datetime.today().strftime("%Y%m%d%H%M%S")
        return item

    def list(self, request, *args, **kwargs):
        # 更新范围
        self.update_device_range()
        queryset = self.filter_queryset(self.get_queryset())

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    def create(self, request, *args, **kwargs):
        item = request.data.copy()
        if self.queryset.filter(name=item['name']).exists():
            return Response('策略名称重复', status=status.HTTP_400_BAD_REQUEST)
        item = self._change_item_(item)
        item['creator'] = request.user.username
        serializer = self.get_serializer(data=item)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)

        thread1 = threading.Thread(target=SoftwarePolicyTask().software_calculation_policy)  # 触发软件策略下发
        thread1.start()

        return Response('数据创建完成', status=status.HTTP_201_CREATED)

    def update(self, request, *args, **kwargs):
        item = request.data.copy()
        item = self._change_item_(item)
        item['creator'] = request.user.username
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=item, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        thread1 = threading.Thread(target=SoftwarePolicyTask().software_calculation_policy)  # 触发软件策略下发
        thread1.start()

        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        self.perform_destroy(instance)

        thread1 = threading.Thread(target=SoftwarePolicyTask().software_calculation_policy)  # 触发软件策略下发
        thread1.start()

        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """批量操作，启用，禁用， 删除"""
        is_all = self.request.data.get('is_all')
        is_delete = self.request.data.get('is_delete')
        is_status = self.request.data.get('is_status')
        policy_ids_str = self.request.data.get('ids_list')

        if is_all == 0:
            policy_ids_list = policy_ids_str.split(',')
            queryset = self.queryset.filter(id__in=policy_ids_list)
        elif is_all == 1:
            queryset = self.filter_queryset(self.get_queryset())
        else:
            return Response("is_all 参数错误", status=status.HTTP_400_BAD_REQUEST)

        thread1 = threading.Thread(target=SoftwarePolicyTask().software_calculation_policy)  # 触发软件策略下发

        if is_delete == 1:
            queryset.delete()
            thread1.start()
        if is_status == 1:
            queryset.update(status=1)
            thread1.start()
        elif is_status == 0:
            queryset.update(status=0)
            thread1.start()
        else:
            pass

        return Response("修改完成", status=status.HTTP_200_OK)


class SoftwareInfoViewSet(viewsets.ModelViewSet):
    queryset = SoftwareInfoModel.objects.filter(is_delete=0).all()
    serializer_class = SoftwareModelSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    filter_backends = (DjangoFilterBackend, OrderingFilter, SearchFilter)
    filterset_class = SoftwareInfoFilter
    filterset_fields = '__all__'
    search_fields = ('name', 'main_version')
    ordering_fields = '__all__'
    ordering = ('-id',)

    def list(self, request, *args, **kwargs):
        is_all = self.request.query_params.get('is_all', None)
        ids_list = self.request.query_params.get('ids_list', None)
        ids_list = ids_list.split(',') if ids_list is not None else []
        queryset = self.get_queryset()
        # logger.info(f"**********{type(is_all)},{type(ids_list)}")
        if is_all is None:
            pass
        elif is_all == str(0):
            if not isinstance(ids_list, list):
                return Response('ids_list传参非法', status=status.HTTP_400_BAD_REQUEST)
            else:
                queryset = self.get_queryset().filter(id__in=ids_list)
        elif is_all == str(1):
            ids_list = self.filter_queryset(queryset).values_list('id', flat=True)
            return Response({'ids_list': ids_list}, status=status.HTTP_200_OK)
        else:
            return Response('is_all传参非法', status=status.HTTP_400_BAD_REQUEST)

        queryset = self.filter_queryset(queryset)
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)

        return Response(serializer.data)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def os_name_db(self, request, *args, **kwargs):
        # 获取符合条件的 os_name 值
        os_names = self.queryset.filter(is_delete=0).values_list('os_name', flat=True)

        # 将 os_name 字符串拆分成列表
        all_os_names = list(SoftwareOS.objects.filter(is_delete=0).values_list('name', flat=True))

        if os_names:
            for os_name in os_names:
                all_os_names.extend(os_name.split(','))
            unique_os_names = sorted(set(all_os_names))
        else:
            unique_os_names = all_os_names
        res_data = {"os_names": unique_os_names}
        return Response(res_data, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def arch_name_db(self, request, *args, **kwargs):
        # 获取符合条件的 arch_name 值
        arch_names = self.queryset.filter(is_delete=0).values_list('arch_name', flat=True)

        # 将 arch_name 字符串拆分成列表
        all_arch_names = list(SoftwareArch.objects.filter(is_delete=0).values_list('name', flat=True))

        if arch_names:
            for arch_name in arch_names:
                all_arch_names.extend(arch_name.split(','))
            unique_arch_names = sorted(set(all_arch_names))
        else:
            unique_arch_names = all_arch_names
        res_data = {"arch_names": unique_arch_names}
        return Response(res_data, status=status.HTTP_200_OK)
