# -*- coding: utf-8 -*-

import logging
import os
import json

from elasticsearch import Elasticsearch
import pandas as pd
from dateutil.parser import parse
from datetime import datetime

from rest_framework.views import Response
from rest_framework import status, viewsets
from rest_framework.decorators import action

from utils.authentication import TokenAuthentication
from utils.permission import IsAdminUser
from utils.throttling import UserRateThrottle

from softwarecenter.models import SoftwareCenter, SoftwareCenterDevice
from devices.models import Devices

logger = logging.getLogger(__name__)


class SoftwareCenterViewSet(viewsets.ModelViewSet):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def alllist(self, request, *args, **kwargs):
        '''
        终端和其安装的软件列表
        '''
        # 终端别名
        device_cnname = request.data.get('device_cnname')

        queryset = SoftwareCenterDevice.objects.select_related('software_center','device').filter(status=1).all()
        if device_cnname:
            queryset = queryset.filter(device__device_cnname=device_cnname)
        if not queryset.exists():
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': "无数据"
            })

        data = []
        for item in queryset:
            deviceObj = {}
            deviceObj['device_uuid'] = item.device.device_uuid
            deviceObj['accounts_id'] = item.device.accounts_id
            deviceObj['device_ip'] = item.device.device_ip
            deviceObj['device_hostname'] = item.device.device_hostname
            deviceObj['device_cnname'] = item.device.device_cnname
            deviceObj['belong_user'] = item.device.belong_user
            deviceObj['org_info'] = item.device.org_info
            deviceObj['add_domain_time'] = item.device.add_domain_time
            deviceOrgInfo = {}
            deviceOrgInfo['org_name'] = item.device.org_info.name
            deviceOrgInfo['org_id'] = item.device.org_info.id
            deviceObj['org_info'] = deviceOrgInfo
            softwareCenterObj = {}
            softwareCenterObj['package_name'] = item.software_center.package_name
            softwareCenterObj['title'] = item.software_center.title
            softwareCenterObj['cmd'] = item.software_center.cmd
            deviceObj['software_center'] = softwareCenterObj
            data.append(deviceObj)
        #logger.info(f'data====%s' % data)
        # 返回
        return Response({
            'code': status.HTTP_200_OK,
            'msg':'',
            'data': data
        })
    
    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def listbydevice(self, request, *args, **kwargs):
        '''
        某个终端和其安装的软件列表
        '''
        # 终端uuid
        device_uuid = request.data.get('device_uuid')
        if not device_uuid:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': "device_uuid不能为空"
            })

        device = Devices.objects.filter(device_uuid=device_uuid).exclude(is_domain=0).first()
        if not device:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': "终端不存在"
            })

        queryset = SoftwareCenterDevice.objects.select_related('software_center','device').filter(status=1).all()
        if device_uuid:
            queryset = queryset.filter(device__device_uuid=device_uuid)
        if not queryset.exists():
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': "无数据"
            })
        
        deviceObj = {}
        deviceObj['device_uuid'] = device.device_uuid
        deviceObj['accounts_id'] = device.accounts_id
        deviceObj['device_ip'] = device.device_ip
        deviceObj['device_hostname'] = device.device_hostname
        deviceObj['device_cnname'] = device.device_cnname
        deviceObj['belong_user'] = device.belong_user
        deviceObj['org_info'] = device.org_info
        deviceObj['add_domain_time'] = device.add_domain_time
        deviceOrgInfo = {}
        deviceOrgInfo['org_name'] = device.org_info.name
        deviceOrgInfo['org_id'] = device.org_info.id
        deviceObj['org_info'] = deviceOrgInfo

        softwareList = []
        for item in queryset:
            softwareCenterObj = {}
            softwareCenterObj['package_name'] = item.software_center.package_name
            softwareCenterObj['title'] = item.software_center.title
            softwareCenterObj['cmd'] = item.software_center.cmd
            softwareList.append(softwareCenterObj)
            deviceObj['software_center'] = softwareList
        #logger.info(f'data====%s' % data)
        # 返回
        return Response({
            'code': status.HTTP_200_OK,
            'msg':'',
            'data': deviceObj
        })

