# -*- coding: utf-8 -*-
import logging
import shutil
import datetime
from django.conf import settings

from celery_task.celery import app
from saltapi.utils import SaltAPI
from sysios.models import SysiosIso
from utils.RedisHelper import RedisHelper
from message_center.models import MessageCenter
from devices.web_consumers import send_msg_to_web
from software.apt_download_deb import AptPackageManager
from software.models import SoftwareSource

logger = logging.getLogger(__name__)

def iso_install_deb(install_deb):
    mirror_sources = [f"deb [trusted=yes] http://{settings.HOSTNAME}:{settings.REPREPRO_WEBSERVER_PORT}/server server main", ]
    software_source_queryset = SoftwareSource.objects.exclude(id=1).filter(is_delete=0).all()
    if software_source_queryset.exists():
        for software_item in software_source_queryset:
            if software_item.type == 1:
                # deb [ trusted=yes ] http://40.50.10.104/nfscdos 3.0 softwarecenter drivecenter depend tiger
                mirror_string = f"deb [trusted=yes] http://{software_item.address}/nfscdos 3.0 softwarecenter drivecenter depend tiger"
            else:
                mirror_string = f"deb [trusted=yes] http://{software_item.address}:{software_item.domain_port}/server server main"
            mirror_sources.append(mirror_string)
    # 下载驱动包
    if install_deb.get('drives'):
        drives_manager = AptPackageManager(mirror_sources, install_deb.get('drives'), install_deb.get('drives_path'))
        drives_manager.run()
    # 下载软件包
    if install_deb.get('softwares'):
        softwares_manager = AptPackageManager(mirror_sources, install_deb.get('softwares'), install_deb.get('softwares_path'))
        softwares_manager.run()


def copy_iso(iso_file_source_path, iso_file_target_path):
    # 复制iso文件
    # 如果文件路径不同,必复制
    if str(iso_file_source_path).strip() != str(iso_file_target_path).strip():
        try:
            shutil.copy2(iso_file_source_path, iso_file_target_path)
        except shutil.SameFileError as sf:
            # 如果是同一文件,不处理
            logger.error(
                f"iso_file_source_path({iso_file_source_path}) is same to iso_file_target_path({iso_file_target_path})")
            pass

@app.task(queue='default')
def sysiso_policy(isofactory, iso_obj, **kwargs):
    """
    镜像管理-异步任务:自定义镜像策略设置
    """
    # public = RedisHelper()
    #copy_iso(iso_obj.get('iso_file_source_path'), iso_obj.get('iso_file_target_path'))
    iso_install_deb(iso_obj.get('install_deb'))
    salt_api = SaltAPI(settings.SALT_URL, settings.SALT_AUTH, settings.SALT_PASSWD)
    signal = {'event_type': 'sysiso_policy', 'isofactory': isofactory}
    signal.update(kwargs)
    pillar = {
        'web_server': iso_obj.get('web_server'),
        'sysiso_id': iso_obj.get('id'),
        'name': iso_obj.get('name'),
        'arch':f'5.0-{iso_obj.get("arch")}',
        'username':iso_obj.get('username'),
        'workdir':iso_obj.get('workdir'),
        'iso_file':iso_obj.get('iso_file'),
        'source_iso_file_name': iso_obj.get('source_iso_file_name')
    }
    sls = ['sysiso.sysiso_policy', "pillar={pillar}".format(pillar=pillar), 'queue=true']
    # 调用接口
    result = salt_api.async_deploy(isofactory, sls)
    logger.info('####### celery sysiso_policy ######')
    logger.info(result)
    logger.info('#######---######')
    #　接口返回的结果处理
    if isinstance(result, dict):
        signal['success'] = False
        signal['msg'] = f"{iso_obj.get('name')}镜像制作失败， {result.get('message')}"
        signal['status'] = 3
    else:
        signal['success'] = True
        signal['msg'] = f"{iso_obj.get('name')}镜像制作任务已执行"
        signal['status'] = 2
    # 更新策略状态
    sysios_iso = SysiosIso.objects.filter(id=iso_obj.get('id')).first()
    sysios_iso.status = signal['status']
    sysios_iso.save()
    if 'policy_no' in kwargs.keys():
        pass
    else:
        if signal['status'] == 3:
            websocket_msg = {
                'content': signal['msg'],
                'create_time': datetime.datetime.now(),
                'target_module': '系统部署_镜像制作',
                'message_user': pillar.get('username'),
                'message_device': None,
                'download_path': None,
                'is_read': 0,
                'is_broadcast': 0,
            }
            MessageCenter.objects.create(**websocket_msg)
            send_msg_to_web(websocket_msg, 0, iso_obj.get('username'))
            # 若任务失败，此处的失败是因为saltminion不在线导致，任务不能发送给saltminion，需要抛出错误，让celery通过信号的形式将这个失败的任务发送给sysiso_policy_task_failure
            raise Exception(signal['msg'])

