import json
import logging
from django_redis import get_redis_connection
from celery.signals import task_failure, task_success
from sysios.tasks import sysiso_policy
from sysios.models import SysiosIso
logger = logging.getLogger(__name__)

redis_conn = get_redis_connection('redis')

class ISOFactory(object):
    def __init__(self):
        self.id = 1
        self.device_ip = 'isofactory'
        self.device_hostname = 'isofactory'

class SysisoQueue(object):
    def __init__(self):
        self.queue_name = 'sysiso_queue'
        self.queue_progress = 'sysiso_progress'

    def rpush(self, sysios_iso):
        '''
        添加任务到队列
        [ {'id': sysios_iso.id, 'name': sysios_iso.name, 'username':'admin'} ]
        '''
        redis_conn.rpush(self.queue_name, json.dumps(sysios_iso))

    def lrem(self, sysios_iso_id):
        '''
        从队列中删除任务
        '''
        l = redis_conn.lrange(self.queue_name, 0, -1)
        if not l:
            return
        for item in l:
            sysios_iso = json.loads(item)
            if sysios_iso_id == sysios_iso.get('id'):
                redis_conn.lrem(self.queue_name, 0, item)
                break

    def get_tasks(self):
        '''
        检查任务队列是否有任务存在
        '''
        l = redis_conn.lrange(self.queue_name, 0, -1)
        return l if l else False

    def run_task(self, sysios_iso):
        '''
        判断任务队列中是否有任务：
            -- 若无任务： 开始celery异步任务，并将sysios_iso_id加入到 redis任务队列
            -- 若有任务： 只将并将sysios_iso_id加入到 redis任务队列
        待任务结束（无论成功与失败）将任务从redis队列中删除，并开始下一个任务
        '''
        if not self.get_tasks():
            sysiso_policy.delay(isofactory="isofactory", iso_obj=sysios_iso)
            self.rpush(sysios_iso)
            return True
        self.rpush(sysios_iso)
        return False

    def run_next_task(self):
        if not self.get_tasks():
            return
        next_sysios_iso = json.loads(redis_conn.lindex(self.queue_name, 0))  # 获取 队列中第一个任务
        sysios_iso = SysiosIso.objects.filter(id=next_sysios_iso.get('id')).first()
        if sysios_iso:
            sysiso_policy.delay(isofactory="isofactory", iso_obj=next_sysios_iso)
        else:
            self.lrem(next_sysios_iso.get('id'))
            self.run_next_task()
        return

    def set_task_progress(self, sysios_iso_id, progress):
        redis_conn.hset(self.queue_progress, sysios_iso_id, progress)

    def get_task_progress(self, sysios_iso_id):
        return redis_conn.hget(self.queue_progress, sysios_iso_id)

@task_failure.connect(sender=sysiso_policy)
def sysiso_policy_task_failure(sender=None, **kwargs):
    '''
    celery 信号
        task_failure捕获指定任务的错误信号
    捕获 sysiso_policy 异步任务的错误信号，当sysiso_policy失败出错时，处理其他任务

    logger.info(kwargs)
    {
        'signal': <Signal: task_failure providing_args={'args', 'einfo', 'traceback', 'exception', 'task_id', 'kwargs'}>,
        'task_id': '501e3226-2b68-4911-b428-c390cf000b9f', 'exception': Exception('镜像制作失败， 组件未启动',),
        'args': [],
        'kwargs': {'isofactory': 'isofactory', 'iso_obj': {'id': 16, 'name': 'xxx3', 'username': 'admin'}},
        'traceback': <traceback object at 0x7fae47716c08>,
        'einfo': <ExceptionInfo: Exception('镜像制作失败， 组件未启动',)>
    }
    '''
    sysios_q = SysisoQueue()
    logger.info("====sysiso_policy_task_failure====")
    logger.info(kwargs)
    task_id = kwargs.get('task_id')
    isofactory = kwargs.get('kwargs').get('isofactory')
    iso_obj = kwargs.get('kwargs').get('iso_obj')
    sysios_iso = SysiosIso.objects.filter(id=iso_obj.get('id')).first()
    if sysios_iso:
        sysios_iso.status = 3
        sysios_iso.save()
    sysios_q.lrem(iso_obj.get('id'))
    sysios_q.run_next_task()
