# -*- coding: utf-8 -*-
import json
import logging
import os
import re
import shutil
from datetime import datetime

from django.conf import settings
from rest_framework.views import Response, APIView
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.pagination import PageNumberPagination
from rest_framework.viewsets import ModelViewSet
from django.db.models import Q
from django.http import StreamingHttpResponse
from django.template.defaultfilters import filesizeformat

from utils.authentication import TokenAuthentication
from utils.pagination import MyPagination
from utils.permission import IsAdminUser
from utils.response import FitJSONRenderer
from utils.throttling import UserRateThrottle, AnonRateThrottle
from pathlib import  Path

from sysios.serializers import SysiosIsoSerializer, SysiosFactorySerializer, SysiosArchSerializer, \
    SysiosLanguageSerializer, SysiosKeyboardSerializer, SysiosTimezoneSerializer, SysiosDomainSerializer
from sysios.models import SysiosIso, SysiosFactory, SysiosArch, SysiosConfig, SysiosDrive, \
    SysiosSoftware, SysiosLanguage, SysiosKeyboard, SysiosTimezone, SysiosDomain
from patch_management.aptly_api_http import AptlyUtil
from utils.RedisHelper import RedisHelper
from software.models import SoftwareInfoModel
from sysios.utils import SysisoQueue

logger = logging.getLogger(__name__)
# 镜像名规则：字母、数字、下划线、连接符、点
ISO_NAME_PATTERN = r'^[a-zA-Z0-9_\-.]+$'

class SysiosIsoViewSet(ModelViewSet):
    """
    镜像管理
    """
    queryset = SysiosIso.objects.all()
    serializer_class = SysiosIsoSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    redis_helper = RedisHelper()

    def list(self, request, *args, **kwargs):
        """
        镜像分页查询
        """
        name = request.GET.get('name')
        sysios_factory_id = request.GET.get('sysios_factory_id')
        sysios_arch_id = request.GET.get('sysios_arch_id')
        query_status = request.GET.get('status')
        base_version = request.GET.get('base_version')
        creator = request.GET.get('creator')
        # 列表只查询[基于基础镜像构建的镜像]
        queryset = self.queryset.filter(iso_type=1)
        if name:
            queryset = queryset.filter(Q(name__icontains=name))
        if sysios_factory_id:
            queryset = queryset.filter(sysios_factory_id=sysios_factory_id)
        if sysios_arch_id:
            queryset = queryset.filter(sysios_arch_id=sysios_arch_id)
        if query_status:
            queryset = queryset.filter(status=query_status)
        if base_version:
            queryset = queryset.filter(Q(base_version__icontains=base_version))
        if creator:
            queryset = queryset.filter(Q(creator__icontains=creator))
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response(data=[], status=status.HTTP_200_OK)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        sysios_q = SysisoQueue()
        for item in ret:
            # 获取厂商名称
            sysios_factory = SysiosFactory.objects.filter(id=item.sysios_factory_id).first()
            # 获取架构名称
            sysios_arch = SysiosArch.objects.filter(id=item.sysios_arch_id).first()
            # 策略状态
            status_str = '未执行'
            if item.status == 0:
                status_str = '未执行'
            elif item.status == 1:
                status_str = '成功'
            elif item.status == 2:
                status_str = '执行中'
            elif item.status == 3:
                status_str = '失败'
            progress = sysios_q.get_task_progress(item.id)
            item_info = {
                'id': item.id,
                'name': item.name,
                'sysios_factory_id': item.sysios_factory_id,
                'sysios_arch_id': item.sysios_arch_id,
                'sysios_factory_name': sysios_factory.name if sysios_factory else '',
                'sysios_arch_name': sysios_arch.name if sysios_arch else '',
                'is_config': True if item.is_config == 1 else False,
                'is_config_val': '开' if item.is_config == 1 else '关',
                'sysios_config_id': item.sysios_config_id,
                # 'sysios_config_filepath': item.sysios_config_filepath,
                'is_domain': True if item.is_domain == 1 else False,
                'is_domain_val': '开' if item.is_domain == 1 else '关',
                'sysios_domain_id': item.sysios_domain_id,
                'base_version': item.base_version,
                'remarks': item.remarks,
                'source_iso_name': item.source_iso_name,
                'source_iso_size': item.source_iso_size,
                'source_iso_path': item.source_iso_path,
                'iso_path': item.iso_path,
                'iso_size': item.iso_size,
                'status': item.status,
                'status_val': status_str,
                'jid': item.jid,
                'creator': item.creator,
                'updator': item.updator,
                'create_at': item.created_at.strftime(format_str),
                'update_at': item.update_at.strftime(format_str) if item.update_at else '',
                'progress': int(progress) if progress else 0
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def all(self, request, *args, **kwargs):
        """
        获取所有的镜像文件简要数据(id,name,iso_path,iso_size,用于下拉框选择)
        """
        name = request.GET.get('name')
        # 下拉框里的是全部'已构建成功'的镜像
        queryset = self.queryset.filter(status=1)
        if name:
            queryset = queryset.filter(Q(name__icontains=name))
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")
        res_list = []
        for item in queryset:
            item_info = {
                'id': item.id,
                'name': item.name,
            }
            res_list.append(item_info)
        return Response(res_list, status=status.HTTP_200_OK)

    def retrieve(self, request, *args, **kwargs):
        """
        获取镜像明细
        """
        # 获取镜像基本信息
        sysios_iso = self.get_object()
        # 获取自定义系统配置
        sysios_config_obj = {}
        sysios_config = SysiosConfig.objects.filter(id=sysios_iso.sysios_config_id).first()
        if sysios_config:
            sysios_config_obj = {
                'id': sysios_config.id,
                'local_user': sysios_config.local_user,
                'local_user_pwd': sysios_config.local_user_pwd,
                'root_permission':True if sysios_config.root_permission == 1 else False,
                'root_permission_val': '开' if sysios_config.root_permission == 1 else '关',
                'sysios_language_id': sysios_config.sysios_language_id,
                'sysios_keyboard_id': sysios_config.sysios_keyboard_id,
                'sysios_timezone_id': sysios_config.sysios_timezone_id,
                'is_active': True if sysios_config.is_active == 1 else  False,
                'is_active_val': '开' if sysios_config.is_active == 1 else '关',
                'active_url': sysios_config.active_url,
                'after_deploy': True if sysios_config.after_deploy == 1 else False,
                'after_deploy_val': '重启' if sysios_config.after_deploy == 1 else '关机',
            }
        # 获取驱动集合
        sysios_drive_obj = []
        sysios_drive = SysiosDrive.objects.filter(sysios_iso_id=sysios_iso.id).all()
        if sysios_drive:
            for item in sysios_drive:
                sysios_drive_item = {
                    'id': item.id,
                    'drive_id': item.drive_id,
                    'name': item.name,
                    'path': item.path,
                }
                sysios_drive_obj.append(sysios_drive_item)
        # 获取软件集合
        sysios_software_obj = []
        sysios_software = SysiosSoftware.objects.filter(sysios_iso_id=sysios_iso.id).all()
        if sysios_software:
            for item in sysios_software:
                sysios_software_item = {
                    'id': item.id,
                    'software_id': item.software_id,
                    'name': item.name,
                    'path': item.path,
                    'is_uninstall': True if item.is_uninstall == 1 else False,
                    'is_uninstall_val': '是' if item.is_uninstall == 1 else '否',
                }
                sysios_software_obj.append(sysios_software_item)
        # 加域配置
        sysios_domain_obj = {}
        sysios_domain = SysiosDomain.objects.filter(id=sysios_iso.sysios_domain_id).first()
        if sysios_domain:
            last_colon_index = str(sysios_domain.url).rfind(':')
            sysios_domain_obj = {
                'id': sysios_domain.id,
                'url': sysios_domain.url[:last_colon_index],
                'port': sysios_domain.url[last_colon_index+1:],
                'admin_user': sysios_domain.admin_user,
                'admin_passwd': sysios_domain.admin_passwd,
            }
        # 策略状态
        status_str = '未执行'
        if sysios_iso.status == 0:
            status_str = '未执行'
        elif sysios_iso.status == 1:
            status_str = '成功'
        elif sysios_iso.status == 2:
            status_str = '执行中'
        elif sysios_iso.status == 3:
            status_str = '失败'
        sysios_q = SysisoQueue()
        progress = sysios_q.get_task_progress(sysios_iso.id)
        # 返回对象
        return Response({
            'id': sysios_iso.id,
            'name': sysios_iso.name,
            'sysios_factory_id': sysios_iso.sysios_factory_id,
            'sysios_arch_id': sysios_iso.sysios_arch_id,
            'is_config': True if sysios_iso.is_config == 1 else False,
            'is_config_val': '开' if sysios_iso.is_config == 1 else '关',
            'sysios_config_id': sysios_iso.sysios_config_id,
            # 'sysios_config_filepath': sysios_iso.sysios_config_filepath,
            'is_domain': True if sysios_iso.is_domain == 1 else False,
            'is_domain_val': '开' if sysios_iso.is_domain == 1 else '关',
            'sysios_domain_id': sysios_iso.sysios_domain_id,
            'base_version': sysios_iso.base_version,
            'remarks': sysios_iso.remarks,
            'source_iso_name': sysios_iso.source_iso_name,
            'source_iso_size': sysios_iso.source_iso_size,
            'source_iso_path': sysios_iso.source_iso_path,
            'iso_path': sysios_iso.iso_path,
            'iso_size': sysios_iso.iso_size,
            'status': sysios_iso.status,
            'jid': sysios_iso.jid,
            'iso_type': sysios_iso.iso_type,
            'iso_type_val': '已构建镜像' if sysios_iso.iso_type == 1 else '本地上传镜像',
            'creator': sysios_iso.creator,
            'create_at': sysios_iso.created_at.strftime("%Y-%m-%d %H:%M:%S"),
            'updator': sysios_iso.updator,
            'update_at': sysios_iso.update_at.strftime("%Y-%m-%d %H:%M:%S") if sysios_iso.update_at else '',
            'sysios_config': sysios_config_obj,
            'sysios_drive': sysios_drive_obj,
            'sysios_software': sysios_software_obj,
            'sysios_domain': sysios_domain_obj,
            'progress': int(progress) if progress else 0
        },status=status.HTTP_200_OK)


    def create(self, request, *args, **kwargs):
        """
        创建镜像:
        1,如果自定义系统配置开启,需要保存自定义系统配置
        2,如果加域配置开启,需要保存加域配置
        3,如果驱动集合不为空,需要保存驱动集合
        4,如果软件集合不为空,需要保存软件集合
        """
        # 复制请求对象
        req_data = request.data.copy()
        # 0,请求参数检查
        # 镜像信息检查
        name = req_data.get('name')
        if not name or not req_data.get('sysios_factory_id') \
                or not req_data.get('sysios_arch_id') \
                or not req_data.get('base_version'):
            return Response('镜像信息缺少必要属性(name,sysios_factory_id,sysios_arch_id,base_version)',
                            status=status.HTTP_400_BAD_REQUEST)
        # 镜像名的规则检查
        if not re.match(ISO_NAME_PATTERN, name):
            return Response('镜像名(name)只允许字母,数字,下划线,连字符和点', status=status.HTTP_400_BAD_REQUEST)
        if self.queryset.filter(name=name).count() > 0:
            return Response(f'镜像名(name={name})已存在', status=status.HTTP_400_BAD_REQUEST)
        # 镜像文件检查
        # 如果是选择已有的镜像,需要判断原镜像是否存在
        source_iso_name = req_data.get('source_iso_name', None)
        # 如果为None,就是本地上传,否则就是选择镜像
        if source_iso_name is not None:
            # 根据镜像名查询已存在的镜像信息
            source_sysios_iso = self.queryset.filter(name=source_iso_name).first()
            if not source_sysios_iso:
                return Response(f'镜像(source_iso_name:{source_iso_name})不存在', status=status.HTTP_400_BAD_REQUEST)
            source_iso_name = source_sysios_iso.source_iso_name
            source_iso_path = source_sysios_iso.source_iso_path
            source_iso_size = source_sysios_iso.source_iso_size
        else:
            # 否则就是本地上传基础镜像,需要判断是否上传成功
            iso_path = req_data.get('iso_path', None)
            iso_size = req_data.get('iso_size', None)
            if iso_path is None or iso_size is None:
                return Response('本地上传缺少镜像文件信息(iso_path,iso_size)', status=status.HTTP_400_BAD_REQUEST)
            if not os.path.exists(iso_path):
                return Response(f'镜像文件(iso_path:{iso_path})不存在', status=status.HTTP_400_BAD_REQUEST)
            # 获取基础镜像的信息
            iso_file_name = Path(iso_path).name
            basic_sysios_iso = self.queryset.filter(source_iso_name=iso_file_name, iso_type=0).last()
            if not basic_sysios_iso:
                return Response(f'镜像文件(iso_path:{iso_path})不存在', status=status.HTTP_400_BAD_REQUEST)
            source_iso_name = basic_sysios_iso.source_iso_name
            source_iso_path = basic_sysios_iso.source_iso_path
            source_iso_size = iso_size
        # 自定义系统配置检查(如果开启)
        is_config = req_data.get('is_config', False)
        if is_config:
            req_data_sysios_config = req_data.get('sysios_config', {})
            if not req_data_sysios_config:
                return Response('自定义系统配置信息不能为空', status=status.HTTP_400_BAD_REQUEST)
            local_user = req_data_sysios_config.get('local_user')
            local_user_pwd = req_data_sysios_config.get('local_user_pwd')
            local_user_pwd_re = req_data_sysios_config.get('local_user_pwd_re')
            if not local_user or not local_user_pwd or not local_user_pwd_re:
                return Response('自定义系统配置信息缺少必要属性(local_user,local_user_pwd,local_user_pwd_re)',
                            status = status.HTTP_400_BAD_REQUEST
                )
            if local_user_pwd != local_user_pwd_re:
                return Response('自定义系统配置信息中两次密码不一致',status = status.HTTP_400_BAD_REQUEST)
        # 驱动集合检查(如果有值)
        req_data_sysios_drive_ids_list = []
        req_data_sysios_drive_ids = req_data.get('sysios_drive_ids',None)
        if req_data_sysios_drive_ids is not None:
            req_data_sysios_drive_ids_array = str(req_data_sysios_drive_ids).split(',')
            for item_id in req_data_sysios_drive_ids_array:
                # 检查是否为整型
                try:
                    req_data_sysios_drive_ids_list.append(int(item_id))
                except (ValueError, TypeError):
                    # 只记录
                    logger.warning(f"驱动ID(ID:item_id)不是整型")
        # 软件集合检查(如果有值)
        req_data_sysios_software_list = []
        req_data_sysios_software = req_data.get('sysios_software', [])
        if req_data_sysios_software:
            for item in req_data_sysios_software:
                req_data_sysios_software_item = {
                    'id': item.get('id', 0),
                    'is_uninstall': item.get('is_uninstall', 0),
                }
                req_data_sysios_software_list.append(req_data_sysios_software_item)
        # 加域配置检查(如果开启)
        is_domain = req_data.get('is_domain', False)
        if is_domain:
            req_data_sysios_domain = req_data.get('sysios_domain', {})
            if not req_data_sysios_domain:
                return Response('加域配置信息不能为空', status=status.HTTP_400_BAD_REQUEST)
            url = req_data_sysios_domain.get('url')
            port = req_data_sysios_domain.get('port')
            admin_user = req_data_sysios_domain.get('admin_user')
            admin_passwd = req_data_sysios_domain.get('admin_passwd')
            if not url or not port or not admin_user or not admin_passwd:
                return Response('加域配置信息缺少必要属性(url,port,admin_user,admin_passwd)',
                                status=status.HTTP_400_BAD_REQUEST
                            )
        try:
            # 1,先保存系统配置(如果系统配置开启)
            sysios_config_id = None
            if is_config:
                req_data_sysios_config = req_data.get('sysios_config', {})
                sysios_config = SysiosConfig.objects.create(
                    local_user=req_data_sysios_config.get('local_user'),
                    local_user_pwd=req_data_sysios_config.get('local_user_pwd'),
                    root_permission=1 if req_data_sysios_config.get('root_permission', False) == True else 0,
                    sysios_language_id=req_data_sysios_config.get('sysios_language_id'),
                    sysios_keyboard_id=req_data_sysios_config.get('sysios_keyboard_id'),
                    sysios_timezone_id=req_data_sysios_config.get('sysios_timezone_id'),
                    is_active=1 if req_data_sysios_config.get('is_active', False) == True else 0,
                    active_url=req_data_sysios_config.get('active_url'),
                    after_deploy=1 if req_data_sysios_config.get('after_deploy', False) == True else 0,
                )
                sysios_config_id = sysios_config.id
            # 2,再保存加域配置(如果开启)
            sysios_domain_id = None
            if is_domain:
                req_data_sysios_domain = req_data.get('sysios_domain', {})
                address = f'%s:%s' %( req_data_sysios_domain.get('url'), req_data_sysios_domain.get('port'))
                sysios_domain = SysiosDomain.objects.create(
                    url=address,
                    admin_user=req_data_sysios_domain.get('admin_user'),
                    admin_passwd=req_data_sysios_domain.get('admin_passwd'),
                )
                sysios_domain_id = sysios_domain.id
            # 3,再保存镜像基本信息
            sysios_iso = SysiosIso.objects.create(
                name=req_data.get('name'),
                sysios_factory_id=req_data.get('sysios_factory_id'),
                sysios_arch_id=req_data.get('sysios_arch_id'),
                is_config=1 if is_config else 0,
                sysios_config_id=sysios_config_id,
                # sysios_config_filepath = req_data.get('sysios_config_filepath'),
                is_domain=1 if is_domain else 0,
                sysios_domain_id=sysios_domain_id,
                base_version=req_data.get('base_version'),
                remarks=req_data.get('remarks'),
                iso_type=1,#镜像类型-构建镜像
                source_iso_name=source_iso_name,
                source_iso_path=source_iso_path,
                source_iso_size=source_iso_size,
                iso_path=req_data.get('iso_path'),
                iso_size=req_data.get('iso_size'),
                creator=request.user.username
            )
            # 4,再保存驱动集合
            if req_data_sysios_drive_ids_list:
                for item_id in req_data_sysios_drive_ids_list:
                    # 查询驱动(驱动也是软件)
                    if item_id:
                        software_drive_item = SoftwareInfoModel.objects.filter(id=item_id).first()
                        if software_drive_item:
                            SysiosDrive.objects.create(
                                drive_id=item_id,
                                name=software_drive_item.main_package,
                                sysios_iso_id=sysios_iso.id
                            )
            # 5,再保存软件集合
            if req_data_sysios_software_list:
                for item in req_data_sysios_software_list:
                    # 查询软件
                    item_id = int(item.get('id', 0))
                    software_item = SoftwareInfoModel.objects.filter(id=item_id).first()
                    if software_item:
                        SysiosSoftware.objects.create(
                            software_id=item_id,
                            name=software_item.main_package,
                            sysios_iso_id=sysios_iso.id,
                            is_uninstall=1 if item.get('is_uninstall', False) ==True else 0
                        )
            # 镜像文件处理
            source_iso_file_name, arch, install_deb = self._handler_iso_file(sysios_iso.id)
            # 放入到队列中
            sysios_q = SysisoQueue()
            if sysios_q.run_task(
                    {
                        'web_server': os.environ.get("DOMAIN_IP"),
                        'id': sysios_iso.id,
                        'name': sysios_iso.name,
                        'username': sysios_iso.creator,
                        'workdir': "/data/%s" % sysios_iso.name,
                        'iso_file': "/data/%s/source/%s" % (sysios_iso.name, Path(sysios_iso.source_iso_path).name),
                        'source_iso_file_name': source_iso_file_name,
                        'arch': arch,
                        'install_deb': install_deb
                    }
            ):
                return Response('镜像制作中,稍后在消息中心查看', status=status.HTTP_200_OK)
            return Response('当前有镜像制作中,此镜像制作已进入到队列中排队', status=status.HTTP_200_OK)
        except Exception as e:
            logger.error(f"新增镜像异常:{e}")
            return Response('新增镜像系统异常', status=status.HTTP_400_BAD_REQUEST)


    def update(self, request, *args, **kwargs):
        """
        修改镜像:
        1,如果自定义系统配置开启,需要保存自定义系统配置
        2,如果加域配置开启,需要保存加域配置
        3,如果驱动集合不为空,需要保存驱动集合
        4,如果软件集合不为空,需要保存软件集合
        """
        # 镜像信息
        sysios_iso = self.get_object()
        sysios_iso_id = sysios_iso.id
        # 复制请求对象
        req_data = request.data.copy()
        # 0,请求参数检查
        if not req_data.get('sysios_factory_id') \
                or not req_data.get('sysios_arch_id') \
                or not req_data.get('base_version'):
            return Response('镜像信息缺少必要属性(sysios_factory_id,sysios_arch_id,base_version)',
                            status=status.HTTP_400_BAD_REQUEST)
        # 如果是选择已有的镜像,需要判断原镜像是否存在
        source_iso_name = req_data.get('source_iso_name', None)
        # 如果不为None,就是选择镜像,如果为None,就是本地上传
        if source_iso_name is not None:
            #if source_iso_name == sysios_iso.name:
            #    return Response(f'镜像文件(source_iso_name:{source_iso_name})不能选择自己', status=status.HTTP_400_BAD_REQUEST)
            # 根据镜像名查询已存在的镜像信息
            source_sysios_iso = self.queryset.filter(name=source_iso_name).first()
            if not source_sysios_iso:
                return Response(f'镜像(source_iso_name:{source_iso_name})不存在', status=status.HTTP_400_BAD_REQUEST)
            # source_iso_name要用镜像的源镜像信息
            source_iso_name = source_sysios_iso.source_iso_name
            source_iso_path = source_sysios_iso.source_iso_path
            source_iso_size = source_sysios_iso.source_iso_size
        else:
            # 否则就是本地上传基础镜像,需要判断是否上传成功
            iso_path = req_data.get('iso_path', None)
            iso_size = req_data.get('iso_size', None)
            if iso_path is None or iso_size is None:
                return Response('本地上传缺少镜像文件信息(iso_path,iso_size)', status=status.HTTP_400_BAD_REQUEST)
            if not os.path.exists(iso_path):
                return Response(f'镜像文件(iso_path:{iso_path})不存在', status=status.HTTP_400_BAD_REQUEST)
            # 根据path查找基础镜像的信息
            iso_file_name = Path(iso_path).name
            basic_sysios_iso = self.queryset.filter(source_iso_name=iso_file_name, iso_type=0).last()
            if not basic_sysios_iso:
                return Response(f'镜像文件(iso_path:{iso_path})不存在', status=status.HTTP_400_BAD_REQUEST)
            source_iso_name = basic_sysios_iso.source_iso_name
            source_iso_path = basic_sysios_iso.source_iso_path
            source_iso_size = iso_size
        # 自定义系统配置检查(如果开启)
        is_config = req_data.get('is_config', False)
        if is_config:
            req_data_sysios_config = req_data.get('sysios_config', {})
            if not req_data_sysios_config:
                return Response('自定义系统配置信息不能为空', status=status.HTTP_400_BAD_REQUEST)
            local_user = req_data_sysios_config.get('local_user')
            local_user_pwd = req_data_sysios_config.get('local_user_pwd')
            local_user_pwd_re = req_data_sysios_config.get('local_user_pwd_re')
            if not local_user or not local_user_pwd or not local_user_pwd_re:
                return Response('自定义系统配置信息缺少必要属性(local_user,local_user_pwd,local_user_pwd_re)',
                                status=status.HTTP_400_BAD_REQUEST)
            if local_user_pwd != local_user_pwd_re:
                return Response('自定义系统配置信息中两次密码不一致', status=status.HTTP_400_BAD_REQUEST)
        # 驱动集合检查(如果有值)
        req_data_sysios_drive_ids_list = []
        req_data_sysios_drive_ids = req_data.get('sysios_drive_ids', None)
        if req_data_sysios_drive_ids is not None:
            req_data_sysios_drive_ids_array = str(req_data_sysios_drive_ids).split(',')
            for item_id in req_data_sysios_drive_ids_array:
                # 检查是否为整型
                try:
                    req_data_sysios_drive_ids_list.append(int(item_id))
                except (ValueError, TypeError):
                    # 只记录
                    logger.warning(f"驱动ID(ID:item_id)不是整型")
        # 软件集合检查(如果有值)
        req_data_sysios_software_list = None
        req_data_sysios_software_ids = req_data.get('sysios_software', None)
        if req_data_sysios_software_ids is not None:
            req_data_sysios_software_list = []
            for item in req_data_sysios_software_ids:
                req_data_sysios_software_id_item = {
                    'id': item.get('id', 0),
                    'is_uninstall': item.get('is_uninstall', 0),
                }
                req_data_sysios_software_list.append(req_data_sysios_software_id_item)
        # 加域配置检查(如果开启)
        is_domain = req_data.get('is_domain', False)
        if is_domain:
            req_data_sysios_domain = req_data.get('sysios_domain', {})
            if not req_data_sysios_domain:
                return Response('加域配置信息不能为空', status=status.HTTP_400_BAD_REQUEST)
            url = req_data_sysios_domain.get('url')
            port = req_data_sysios_domain.get('port')
            admin_user = req_data_sysios_domain.get('admin_user')
            admin_passwd = req_data_sysios_domain.get('admin_passwd')
            if not url or not port or not admin_user or not admin_passwd:
                return Response('加域配置信息缺少必要属性(url,port,admin_user,admin_passwd)',
                                status=status.HTTP_400_BAD_REQUEST
                                )
        # 1,先更新自定义系统配置(如果开启),有记录就更新,没记录就创建
        sysios_config_id = None
        if is_config:
            req_data_sysios_config = req_data.get('sysios_config', {})
            sysios_config = SysiosConfig.objects.filter(id=sysios_iso.sysios_config_id).first()
            if not sysios_config:
                sysios_config = SysiosConfig.objects.create(
                    local_user=req_data_sysios_config.get('local_user'),
                    local_user_pwd=req_data_sysios_config.get('local_user_pwd'),
                    root_permission=1 if req_data_sysios_config.get('root_permission', False) ==True else 0,
                    sysios_language_id=req_data_sysios_config.get('sysios_language_id'),
                    sysios_keyboard_id=req_data_sysios_config.get('sysios_keyboard_id'),
                    sysios_timezone_id=req_data_sysios_config.get('sysios_timezone_id'),
                    is_active=1 if req_data_sysios_config.get('is_active', False) ==True else 0,
                    active_url=req_data_sysios_config.get('active_url'),
                    after_deploy=1 if req_data_sysios_config.get('after_deploy', False) ==True else 0,
                )
            else:
                SysiosConfig.objects.filter(id=sysios_iso.sysios_config_id).update(
                    local_user=req_data_sysios_config.get('local_user'),
                    local_user_pwd=req_data_sysios_config.get('local_user_pwd'),
                    root_permission=1 if req_data_sysios_config.get('root_permission', False) ==True else 0,
                    sysios_language_id=req_data_sysios_config.get('sysios_language_id'),
                    sysios_keyboard_id=req_data_sysios_config.get('sysios_keyboard_id'),
                    sysios_timezone_id=req_data_sysios_config.get('sysios_timezone_id'),
                    is_active=1 if req_data_sysios_config.get('is_active', False) ==True else 0,
                    active_url=req_data_sysios_config.get('active_url'),
                    after_deploy=1 if req_data_sysios_config.get('after_deploy', False) ==True else 0,
                )
            sysios_config_id = sysios_config.id
        # 2,再保存加域配置(如果开启)
        sysios_domain_id = None
        if is_domain:
            req_data_sysios_domain = req_data.get('sysios_domain', {})
            address = f'%s:%s' % (req_data_sysios_domain.get('url'), req_data_sysios_domain.get('port'))
            sysios_domain = SysiosDomain.objects.filter(id=sysios_iso.sysios_domain_id).first()
            if sysios_domain:
                SysiosDomain.objects.filter(id=sysios_iso.sysios_domain_id).update(
                    url=address,
                    admin_user=req_data_sysios_domain.get('admin_user'),
                    admin_passwd=req_data_sysios_domain.get('admin_passwd'),
                )
            else:
                sysios_domain = SysiosDomain.objects.create(
                    url=address,
                    admin_user=req_data_sysios_domain.get('admin_user'),
                    admin_passwd=req_data_sysios_domain.get('admin_passwd'),
                )
            sysios_domain_id = sysios_domain.id
        # 3,再更新镜像基本信息
        SysiosIso.objects.filter(id=sysios_iso.id).update(
            sysios_factory_id=req_data.get('sysios_factory_id'),
            sysios_arch_id=req_data.get('sysios_arch_id'),
            is_config=1 if is_config else 0,
            sysios_config_id=sysios_config_id,
            # sysios_config_filepath = req_data.get('sysios_config_filepath'),
            is_domain=1 if is_domain else 0,
            sysios_domain_id=sysios_domain_id,
            base_version=req_data.get('base_version'),
            remarks=req_data.get('remarks'),
            source_iso_name=source_iso_name,
            source_iso_path=source_iso_path,
            source_iso_size=source_iso_size,
            iso_path=req_data.get('iso_path'),
            iso_size=req_data.get('iso_size'),
            updator=request.user.username,
            update_at=datetime.now()
        )
        # 4,再保存驱动集合(先删除再创建)
        if req_data_sysios_drive_ids_list:
            SysiosDrive.objects.filter(sysios_iso_id=sysios_iso_id).delete()
            for item_id in req_data_sysios_drive_ids_list:
                # 查询软件
                software_drive_item = SoftwareInfoModel.objects.filter(id=item_id).first()
                if software_drive_item:
                    SysiosDrive.objects.create(
                        drive_id=item_id,
                        name=software_drive_item.main_package,
                        sysios_iso_id=sysios_iso.id,
                    )
        # 5,再保存软件集合(先删除再创建)
        if req_data_sysios_software_list is not None:
            SysiosSoftware.objects.filter(sysios_iso_id=sysios_iso_id).delete()
            for item in req_data_sysios_software_list:
                # 查询软件
                item_id = int(item.get('id', 0))
                software_item = SoftwareInfoModel.objects.filter(id=item_id).first()
                if software_item:
                    SysiosSoftware.objects.create(
                        software_id=item_id,
                        name=software_item.main_package,
                        sysios_iso_id=sysios_iso.id,
                        is_uninstall=1 if item.get('is_uninstall', False) ==True else 0,
                    )
        # 镜像文件处理
        source_iso_file_name, arch, install_deb = self._handler_iso_file(sysios_iso_id)
        # 放入到队列中
        sysios_q = SysisoQueue()
        if sysios_q.run_task(
                {
                    'web_server': os.environ.get("DOMAIN_IP"),
                    'id': sysios_iso.id,
                    'name': sysios_iso.name,
                    'username': sysios_iso.creator,
                    'workdir': "/data/%s" % sysios_iso.name,
                    'iso_file': "/data/%s/source/%s" % (sysios_iso.name, Path(sysios_iso.source_iso_path).name),
                    'source_iso_file_name': source_iso_file_name,
                    'arch': arch,
                    'install_deb': install_deb
                }
        ):
            return Response('镜像制作中,稍后在消息中心查看', status=status.HTTP_200_OK)
        return Response('当前有镜像制作中,此镜像制作已进入到队列中排队', status=status.HTTP_200_OK)


    def destroy(self, request, *args, **kwargs):
        """
        删除镜像:
        1,同时删除自定义系统配置
        2,同时删除驱动集合
        3,同时删除软件集合
        4,同时删除加域配置
        """
        # 镜像信息检查
        sysios_iso = self.get_object()
        # 制作中的镜像不能删除
        if sysios_iso.status == 2:
            return Response('制作中的镜像不能删除', status=status.HTTP_400_BAD_REQUEST)
        # 删除关联数据
        SysiosDrive.objects.filter(sysios_iso_id=sysios_iso.id).delete()
        SysiosSoftware.objects.filter(sysios_iso_id=sysios_iso.id).delete()
        # 删除对应的镜像文件
        self._delete_iso_file(sysios_iso)
        # 删除镜像
        self.perform_destroy(sysios_iso)
        # 删除关联数据
        SysiosConfig.objects.filter(id=sysios_iso.sysios_config_id).delete()
        SysiosDomain.objects.filter(id=sysios_iso.sysios_domain_id).delete()
        # 返回对象
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除镜像成功'
        },status=status.HTTP_200_OK)

    def _handler_iso_file(self, sysios_iso_id):
        """
        镜像文件处理:
        1,在isofactory-docker目录下创建此镜像的根目录(以镜像名为目录名)和其子目录(iso,drives,softwares)，如果已存在则不再创建
        2,将iso文件复制到子目录iso下，如果已存在则不复制
        3,拼装isofactory.json文件,并保存到isofactory-docker目录下，如果存在则覆盖
        4,驱动和软件下载,需要将驱动和软件下载到对应的目录(drives/softwares)下
        """
        """
                镜像文件处理:
                1,在isofactory-docker目录下创建此镜像的根目录(以镜像名为目录名)和其子目录(iso,drives,softwares)，如果已存在则不再创建
                2,将iso文件复制到子目录iso下，如果已存在则不复制
                3,拼装isofactory.json文件,并保存到isofactory-docker目录下，如果存在则覆盖
                4,驱动和软件下载,需要将驱动和软件下载到对应的目录(drives/softwares)下
                """
        # 检查
        sysios_iso = self.queryset.filter(id=sysios_iso_id).first()
        if not sysios_iso:
            logger.error(f"缺少镜像基本数据")
            return
        sysios_config = sysios_iso.sysios_config
        sysios_domain = sysios_iso.sysios_domain
        removeblacklist = []
        sysios_software = SysiosSoftware.objects.filter(sysios_iso_id=sysios_iso_id, is_uninstall=1).all()
        if sysios_software:
            for item in sysios_software:
                item_software = SoftwareInfoModel.objects.filter(id=item.software_id).first()
                if item_software:
                    removeblacklist.append(item_software.main_package)
        # 从文件路径中获取到文件
        # 判断镜像来源类型
        iso_type = sysios_iso.iso_type
        # 如果是已构镜像,则需要拿到它的源镜像,也就是它的基础镜像
        if iso_type == 1:
            iso_file_source_path = sysios_iso.source_iso_path
            iso_file_source_size = sysios_iso.source_iso_size
        # 否则就是本地上传,也就是基础镜像,源镜像信息就是自己
        else:
            iso_file_source_path = sysios_iso.iso_path
            iso_file_source_size = sysios_iso.iso_size
        # 从文件路径获取文件全名
        iso_file_name = Path(iso_file_source_path).name
        # iso镜像文件挂载的基础目录
        docker_dir = os.path.join(os.path.dirname(settings.BASE_DIR), "docker")
        iso_base_dir = f"{docker_dir}/isofactory-docker/data"
        # 根据镜像名创建镜像的根目录和子目录
        iso_file_dir = Path(iso_base_dir) / sysios_iso.name
        iso_file_packages_dir = iso_file_dir / 'packages'
        iso_file_drive_dir = iso_file_packages_dir / 'drives'
        iso_file_software_dir = iso_file_packages_dir / 'softwares'
        iso_file_iso_dir = iso_file_dir / 'source'
        iso_file_target_path = iso_file_iso_dir / iso_file_name
        # 更新iso_path和iso_size
        sysios_iso.iso_path = iso_file_dir / f'{sysios_iso.name}.iso'
        sysios_iso.iso_size = iso_file_source_size
        sysios_iso.save()
        # 创建目录
        iso_file_dir.mkdir(parents=True, exist_ok=True)
        iso_file_packages_dir.mkdir(parents=True, exist_ok=True)
        iso_file_drive_dir.mkdir(parents=True, exist_ok=True)
        iso_file_software_dir.mkdir(parents=True, exist_ok=True)
        iso_file_iso_dir.mkdir(parents=True, exist_ok=True)
        install_deb = {}  # iso需要安装的驱动，软件以及保存位置
        update_deb = []
        # 下载驱动和软件,并更新关系表中的path
        sysios_drive_list = SysiosDrive.objects.filter(sysios_iso_id=sysios_iso_id).all()
        if sysios_drive_list:
            update_deb.append('drives')
            install_deb['drives'] = []
            install_deb['drives_path'] = iso_file_drive_dir.as_posix()
            for item in sysios_drive_list:
                drive_id = int(item.drive_id)
                software = SoftwareInfoModel.objects.filter(id=drive_id).first()
                #
                if software:
                    install_deb['drives'].append(
                        {'main_package': software.main_package, 'main_version': software.main_version}
                    )
                    iso_file_drive_dir_path = iso_file_drive_dir / software.custome_filename
                    item.path = iso_file_drive_dir_path
                    item.save()
        # 下载软件
        sysios_software_list = SysiosSoftware.objects.filter(sysios_iso_id=sysios_iso_id).all()
        if sysios_software_list:
            update_deb.append('softwares')
            install_deb['softwares'] = []
            install_deb['softwares_path'] = iso_file_software_dir.as_posix()
            for item in sysios_software_list:
                software_id = int(item.software_id)
                software = SoftwareInfoModel.objects.filter(id=software_id).first()
                if software:
                    install_deb['softwares'].append(
                        {'main_package': software.main_package, 'main_version': software.main_version}
                    )
                    iso_file_software_dir_path = iso_file_software_dir / software.custome_filename
                    item.path = iso_file_software_dir_path
                    item.save()
        arch = ''
        if sysios_iso.sysios_arch.name:
            if sysios_iso.sysios_arch.name == 'X86':
                arch = 'amd64'
            else:
                arch = sysios_iso.sysios_arch.name
        # 拼接JSON格式
        iso_factory_json = {
            f"5.0-{arch}": {
                "web_server": os.environ.get("DOMAIN_IP"),
                "name": sysios_iso.name,
                "arch": arch,
                "iso_file": "/data/%s/source/%s" % (sysios_iso.name, iso_file_name),
                "workdir": "/data/%s" % sysios_iso.name,
                "comp_args": "xz",
                "custom_settings": True if sysios_iso.is_config == 1 else False,
                "system_settings": {
                    "user": sysios_config.local_user if sysios_config else '',
                    "passwd": sysios_config.local_user_pwd if sysios_config else '',
                    "root": True if sysios_config and sysios_config.root_permission == 1 else False,
                    "language": sysios_config.sysios_language.value if sysios_config else '',
                    "timezone": sysios_config.sysios_timezone.value if sysios_config else '',
                    "after-pxe": True if sysios_config and sysios_config.after_deploy == 1 else False,
                    "keyboard": sysios_config.sysios_keyboard.value if sysios_config else '',
                    "license": True if sysios_config and sysios_config.is_active == 1 else False,
                    "license-server-ip": sysios_config.active_url if sysios_config else '',
                    "domain_join": True if sysios_iso.is_domain == 1 else False,
                    "domain_url": sysios_domain.url if sysios_domain else '',
                    "domain_admin_user": sysios_domain.admin_user if sysios_domain else '',
                    "domain_admin_passwd": sysios_domain.admin_passwd if sysios_domain else '',
                },
                "datadir": "/data/%s" % sysios_iso.name,
                "update_deb": update_deb,
                "removeblacklist": removeblacklist,
                "clear": "true",
            }
        }
        iso_factory_json_file_path = iso_file_dir / 'ISOconfig.json'
        # 创建文件并写入内容
        with open(iso_factory_json_file_path, 'w') as f:
            json.dump(iso_factory_json, f, indent=4, ensure_ascii=False)

        # 复制iso文件
        # 如果文件路径不同,必复制
        # if str(iso_file_source_path).strip() != str(iso_file_target_path).strip():
        #     try:
        #         shutil.copy2(iso_file_source_path, iso_file_target_path)
        #     except shutil.SameFileError as sf:
        #         # 如果是同一文件,不处理
        #         logger.error(f"iso_file_source_path({iso_file_source_path}) is same to iso_file_target_path({iso_file_target_path})")
        #         pass
        return iso_file_name, arch, install_deb

    def _delete_iso_file(self, sysios_iso:SysiosIso):
        """
        镜像文件删除:
        1,删除isofactory-docker目录下的镜像目录
        """
        # iso镜像文件挂载的基础目录
        docker_dir = os.path.join(os.path.dirname(settings.BASE_DIR), "docker")
        iso_base_dir = f"{docker_dir}/isofactory-docker/data"
        iso_file_dir = Path(iso_base_dir) / sysios_iso.name
        try:
            # 如果存在,就删除
            shutil.rmtree(iso_file_dir)
            sysios_q = SysisoQueue()
            sysios_q.lrem(sysios_iso.id)
        except Exception as e:
            logger.error(f"镜像目录删除异常:{e}")

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def batch_operation(self, request, *args, **kwargs):
        """
        批量操作
        :param request: body {
                        is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        is_delete : 0|不删除 1|删除 空|不处理
                        }
        :param args:
        :param kwargs:
        :return:
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        is_delete = request.data.get('is_delete')
        if is_all is None:
            return Response('is_all 为必输选项', status=status.HTTP_400_BAD_REQUEST)

        is_all = int(is_all)

        queryset = self.queryset
        try:
            ids_list = [int(i) for i in ids_list.split(',')]
        except Exception as exp:
            return Response(f'ids_list参数不符合规范{exp}', status=status.HTTP_400_BAD_REQUEST)

        if is_all == 0:
            queryset = queryset.filter(id__in=ids_list)
        else:
            # queryset = self.filter_queryset(queryset)
            ids_list = list(set(queryset.values_list('id', flat=True)) - set(ids_list))
            # 查询过滤结果集
            queryset = queryset.filter(id__in=ids_list)

        if is_delete == 1:
            # 判断镜像状态
            for item in queryset.all():
                # 制作中的镜像不能删除
                if item.status == 2:
                    return Response(f'制作中的镜像 {item.name} 不能删除,请过滤', status=status.HTTP_400_BAD_REQUEST)
            # 删除操作
            for sysios_iso in queryset.all():
                # 删除关联数据
                SysiosDrive.objects.filter(sysios_iso_id=sysios_iso.id).delete()
                SysiosSoftware.objects.filter(sysios_iso_id=sysios_iso.id).delete()
                # 删除对应的镜像文件
                self._delete_iso_file(sysios_iso)
                # 删除镜像
                self.perform_destroy(sysios_iso)
                # 删除关联数据
                SysiosConfig.objects.filter(id=sysios_iso.sysios_config_id).delete()
                SysiosDomain.objects.filter(id=sysios_iso.sysios_domain_id).delete()
            # queryset.delete()
            return Response('批量删除记录', status=status.HTTP_200_OK)
        return Response('无任何操作', status=status.HTTP_200_OK)

    @action(methods=['post'], detail=False, authentication_classes=[], permission_classes=[], throttle_classes=[])
    def build_progress(self, request, *args, **kwargs):
        '''
        接收制作iso进度
        '''
        data = request.data
        name = data.get('name')
        progress = data.get('progress')
        sysios_iso = self.queryset.filter(name=name).first()
        if not sysios_iso:
            return Response(f'{name}镜像不存在', status=status.HTTP_400_BAD_REQUEST)
        sysios_q = SysisoQueue()
        sysios_q.set_task_progress(sysios_iso.id, progress)
        return Response(f'{name} 进度 {progress}% 提交成功', status=status.HTTP_200_OK)

class SysiosFactoryViewSet(ModelViewSet):
    """
    系统厂商管理
    """
    queryset = SysiosFactory.objects.all()
    serializer_class = SysiosFactorySerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def list(self, request, *args, **kwargs):
        """
        系统厂商分页查询
        """
        queryset = self.queryset
        name = request.GET.get('name')
        if name:
            queryset = queryset.filter(Q(name__icontains=name))
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            item_info = {
                'id': item.id,
                'name': item.name,
                'remarks': item.remarks,
                'create_at': item.created_at.strftime(format_str),
                'update_at': item.update_at.strftime(format_str) if item.update_at else '',
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)

    @action(methods=['post'], detail=False, authentication_classes=[TokenAuthentication])
    def save(self, request, *args, **kwargs):
        """
        保存系统厂商数据(包括新增,修改,删除)
        1, id不存在或为空就是新增
        2, id存在,is_delete=0就是修改
        3, id存在,is_delete=1就是删除
        """
        data_list = request.data.get('data')
        # 参数检查
        for item in data_list:
            if not item.get('name') or not item.get('is_delete'):
                return Response('系统厂商缺少必要属性(name,is_delete)', status=status.HTTP_400_BAD_REQUEST)
        for data in data_list:
            if not data.get("id"):
                SysiosFactory.objects.create({
                    'name': data.get('name'),
                    'remarks': data.get('remarks')
                })
            else:
                # 修改
                if data.get("is_delete") == 0:
                    factory_id = data.get('id')
                    self.queryset.filter(id=factory_id).update({
                        'name': data.get('name'),
                        'remarks': data.get('remarks')
                    })
                #删除
                elif data.get("is_delete") == 1:
                    factory_id = data.get('id')
                    self.queryset.filter(id=factory_id).delete()

        return Response('保存成功', status=status.HTTP_200_OK)

class SysiosArchViewSet(ModelViewSet):
    """
    系统架构管理
    """
    queryset = SysiosArch.objects.all()
    serializer_class = SysiosArchSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def list(self, request, *args, **kwargs):
        """
        系统架构分页查询
        """
        name = request.GET.get('name')
        queryset = self.queryset
        if name:
            queryset = queryset.filter(Q(name__icontains=name))
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            item_info = {
                'id': item.id,
                'name': item.name,
                'create_at': item.created_at.strftime(format_str),
                'update_at': item.update_at.strftime(format_str) if item.update_at else '',
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)

    @action(methods=['post'], detail=False, authentication_classes=[TokenAuthentication])
    def save(self, request, *args, **kwargs):
        """
        保存系统架构数据(包括新增,修改,删除)
        1, id不存在或为空就是新增
        2, id存在,is_delete=0就是修改
        3, id存在,is_delete=1就是删除
        """
        data_list = request.data.get('data')
        # 参数检查
        for item in data_list:
            if not item.get('name') or not item.get('is_delete'):
                return Response('系统架构缺少必要属性(name,is_delete)', status=status.HTTP_400_BAD_REQUEST)
        for data in data_list:
            if not data.get("id"):
                SysiosArch.objects.create({
                    'name': data.get('name')
                })
            else:
                # 修改
                if data.get("is_delete") == 0:
                    arch_id = data.get('id')
                    self.queryset.filter(id=arch_id).update({
                        'name': data.get('name')
                    })
                #删除
                elif data.get("is_delete") == 1:
                    arch_id = data.get('id')
                    self.queryset.filter(id=arch_id).delete()

        return Response('保存成功', status=status.HTTP_200_OK)

class SysiosLanguageViewSet(ModelViewSet):
    """
    系统语言管理
    """
    queryset = SysiosLanguage.objects.all()
    serializer_class = SysiosLanguageSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def list(self, request, *args, **kwargs):
        """
        系统语言分页查询
        """
        name = request.GET.get('name')
        queryset = self.queryset
        if name:
            queryset = queryset.filter(Q(name__icontains=name))
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            item_info = {
                'id': item.id,
                'name': item.name,
                'create_at': item.created_at.strftime(format_str),
                'update_at': item.update_at.strftime(format_str) if item.update_at else '',
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)

    @action(methods=['post'], detail=False, authentication_classes=[TokenAuthentication])
    def save(self, request, *args, **kwargs):
        """
        保存系统语言数据(包括新增,修改,删除)
        1, id不存在或为空就是新增
        2, id存在,is_delete=0就是修改
        3, id存在,is_delete=1就是删除
        """
        data_list = request.data.get('data')
        # 参数检查
        for item in data_list:
            if not item.get('name') or not item.get('is_delete'):
                return Response('系统语言缺少必要属性(name,is_delete)', status=status.HTTP_400_BAD_REQUEST)
        for data in data_list:
            if not data.get("id"):
                SysiosLanguage.objects.create({
                    'name': data.get('name')
                })
            else:
                # 修改
                if data.get("is_delete") == 0:
                    language_id = data.get('id')
                    self.queryset.filter(id=language_id).update({
                        'name': data.get('name')
                    })
                #删除
                elif data.get("is_delete") == 1:
                    language_id = data.get('id')
                    self.queryset.filter(id=language_id).delete()

        return Response('保存成功', status=status.HTTP_200_OK)

class SysiosKeyboardViewSet(ModelViewSet):
    """
    系统键盘布局管理
    """
    queryset = SysiosKeyboard.objects.all()
    serializer_class = SysiosKeyboardSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def list(self, request, *args, **kwargs):
        """
        系统键盘布局分页查询
        """
        name = request.GET.get('name')
        queryset = self.queryset
        if name:
            queryset = queryset.filter(Q(name__icontains=name))
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            item_info = {
                'id': item.id,
                'name': item.name,
                'create_at': item.created_at.strftime(format_str),
                'update_at': item.update_at.strftime(format_str) if item.update_at else '',
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)

    @action(methods=['post'], detail=False, authentication_classes=[TokenAuthentication])
    def save(self, request, *args, **kwargs):
        """
        保存系统键盘布局数据(包括新增,修改,删除)
        1, id不存在或为空就是新增
        2, id存在,is_delete=0就是修改
        3, id存在,is_delete=1就是删除
        """
        data_list = request.data.get('data')
        # 参数检查
        for item in data_list:
            if not item.get('name') or not item.get('is_delete'):
                return Response('系统键盘布局缺少必要属性(name,is_delete)', status=status.HTTP_400_BAD_REQUEST)
        for data in data_list:
            if not data.get("id"):
                SysiosKeyboard.objects.create({
                    'name': data.get('name')
                })
            else:
                # 修改
                if data.get("is_delete") == 0:
                    item_id = data.get('id')
                    self.queryset.filter(id=item_id).update({
                        'name': data.get('name')
                    })
                #删除
                elif data.get("is_delete") == 1:
                    item_id = data.get('id')
                    self.queryset.filter(id=item_id).delete()

        return Response('保存成功', status=status.HTTP_200_OK)

class SysiosTimezoneViewSet(ModelViewSet):
    """
    系统时区管理
    """
    queryset = SysiosTimezone.objects.all()
    serializer_class = SysiosTimezoneSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def list(self, request, *args, **kwargs):
        """
        系统时区分页查询
        """
        name = request.GET.get('name')
        queryset = self.queryset
        if name:
            queryset = queryset.filter(Q(name__icontains=name))
        if not queryset.exists():
            return Response(data=[], status=status.HTTP_200_OK)
        # 倒序
        queryset = queryset.order_by("-id")

        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(queryset, request)
        except:
            return Response('error', status=status.HTTP_404_NOT_FOUND)
        # 定义时间格式
        format_str = "%Y-%m-%d %H:%M:%S"
        res_list = []
        for item in ret:
            item_info = {
                'id': item.id,
                'name': item.name,
                'create_at': item.created_at.strftime(format_str),
                'update_at': item.update_at.strftime(format_str) if item.update_at else '',
            }
            res_list.append(item_info)
        return Response({
            'total': len(queryset),
            'data': res_list
        }, status=status.HTTP_200_OK)

    @action(methods=['post'], detail=False, authentication_classes=[TokenAuthentication])
    def save(self, request, *args, **kwargs):
        """
        保存系统时区数据(包括新增,修改,删除)
        1, id不存在或为空就是新增
        2, id存在,is_delete=0就是修改
        3, id存在,is_delete=1就是删除
        """
        data_list = request.data.get('data')
        # 参数检查
        for item in data_list:
            if not item.get('name') or not item.get('is_delete'):
                return Response('系统时区缺少必要属性(name,is_delete)',status=status.HTTP_400_BAD_REQUEST)
        for data in data_list:
            if not data.get("id"):
                SysiosTimezone.objects.create({
                    'name': data.get('name')
                })
            else:
                # 修改
                if data.get("is_delete") == 0:
                    item_id = data.get('id')
                    self.queryset.filter(id=item_id).update({
                        'name': data.get('name')
                    })
                #删除
                elif data.get("is_delete") == 1:
                    item_id = data.get('id')
                    self.queryset.filter(id=item_id).delete()

        return Response('保存成功',status=status.HTTP_200_OK)

class FileUploadViewSet(APIView):
    """
    基础镜像文件上传:
    1,这个基础镜像是其他镜像构建的基础,目录名唯一：isofactory_basic_iso,统一放在/isofactory-docker/data/下
    2,将iso文件上传到目录isofactory_basic_iso下
    3,在表sysios_iso中添加一条记录,iso_type=0,表明是基础镜像
    """
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def post(self, request, *args, **kwargs):
        # 获取上传的文件
        file = request.FILES.get('file', None)
        if file is None:
            return Response('缺少必要属性(file)', status=status.HTTP_400_BAD_REQUEST)
        # 暂时不用文件服务,只用目录挂载,挂载到[当前项目下的/docker/isofactory-docker/]目录
        # iso镜像文件挂载的基础目录
        docker_dir = os.path.join(os.path.dirname(settings.BASE_DIR), "docker")
        iso_base_dir = f"{docker_dir}/isofactory-docker/data"
        # 创建基础镜像固定目录
        basic_iso_dirname = 'isofactory_basic_iso'
        # 基础镜像的名称采用文件名
        source_iso_name = str(file.name)
        iso_file_dir = Path(iso_base_dir) / basic_iso_dirname
        iso_file_packages_dir = iso_file_dir / 'packages'
        iso_file_drive_dir = iso_file_packages_dir /'drives'
        iso_file_software_dir = iso_file_packages_dir / 'softwares'
        iso_file_iso_dir = iso_file_dir  / 'source'
        iso_file_path = iso_file_iso_dir / str(file.name)
        iso_file_size = filesizeformat(file.size)
        try:
            # 创建目录
            iso_file_dir.mkdir(parents=True, exist_ok=True)
            iso_file_packages_dir.mkdir(parents=True, exist_ok=True)
            iso_file_drive_dir.mkdir(parents=True, exist_ok=True)
            iso_file_software_dir.mkdir(parents=True, exist_ok=True)
            iso_file_iso_dir.mkdir(parents=True, exist_ok=True)
            # 上传文件
            with open(iso_file_path, 'wb+') as destination:
                for chunk in file.chunks():
                    destination.write(chunk)
            # 创建一条镜像记录,iso_type=0,status=1
            # 如果基础镜像文件(iso_type=0)存在就认为是重新上传,更新文件path和size
            sysiso_iso = SysiosIso.objects.filter(name=source_iso_name, iso_type=0).first()
            if sysiso_iso:
                sysiso_iso.source_iso_path=iso_file_path
                sysiso_iso.source_iso_size=iso_file_size
                sysiso_iso.iso_path=iso_file_path
                sysiso_iso.iso_size=iso_file_size
                sysiso_iso.updator=request.user.username
                sysiso_iso.save()
            else:
                SysiosIso.objects.create(
                    name=source_iso_name, #用文件名
                    is_config=0,
                    is_domain=0,
                    iso_type=0,#基础镜像
                    source_iso_name=source_iso_name,
                    source_iso_path=iso_file_path,
                    source_iso_size=iso_file_size,
                    iso_path=iso_file_path,
                    iso_size=iso_file_size,
                    status=1,#必须是成功
                    creator=request.user.username
                )
            # 返回对象
            data = {
                'file_name': str(file.name),
                'file_path': str(iso_file_path),
                'file_size': file.size
            }
            return Response({
                'code': status.HTTP_200_OK,
                'data': data,
                'msg': None
            }, status=status.HTTP_200_OK)
        except Exception as e:
            logger.error(f"文件上传失败:{e}")
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '上传异常'
            },status=status.HTTP_400_BAD_REQUEST)

class FileDownloadViewSet(APIView):
    """
    文件下载
    """
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request, file_id):
        # client = Fdfs_client('path/to/client.conf')  # 配置 FastDFS 客户端
        # # 下载文件
        # result = client.download_to_buffer(file_id)
        # if result:
        #     response = HttpResponse(result, content_type='application/octet-stream')
        #     response['Content-Disposition'] = f'attachment; filename="{file_id.split("/")[-1]}"'
        #     return response
        # else:
        #     return Response({'error': 'File not found'}, status=404)
        def file_iterator(file_name, chunk_size=512):
            with open(file_name, 'rb') as f:
                while True:
                    c = f.read(chunk_size)
                    if c:
                        yield c
                    else:
                        break
        iso = SysiosIso.objects.filter(id=int(file_id)).first()
        if iso:
            file_path = iso.iso_path
            try:
                response = StreamingHttpResponse(file_iterator(file_path))
                response['Content-Type'] = 'application/octet-stream'
                response['Content-Disposition'] = 'attachment; filename={file}.iso'.format(file=iso.name)
                return response
            except Exception:
                return Response({'code': status.HTTP_400_BAD_REQUEST,'msg': '下载异常'},status=status.HTTP_400_BAD_REQUEST)
        return Response({'code': status.HTTP_404_NOT_FOUND,'msg': '文件不存在'},status=status.HTTP_404_NOT_FOUND)
