# -*- coding: utf-8 -*-
import base64
import datetime
import logging
import os
import json
import random
import requests
from rest_framework import status, viewsets, mixins
from rest_framework.decorators import action
from rest_framework.views import APIView, Response
from rest_framework.viewsets import GenericViewSet

from accounts import models
from accounts import freeipa
from accounts.models import AccountEntity
from devices.models import Devices
from nfs_domain_mainserver.settings import HOSTNAME
from organizations.models import OrgnizationEntity, OrgLabel
from patch_management.aptly_api_http import AptlyUtil
from system import serializers
from users.models import User
from users.role_permisson import get_org_permission_list
from utils.authentication import DomainLicenseAuth, TokenAuthentication
from utils.pagination import MyPagination
from utils.permission import IsAdminUser
from utils.response import FitJSONRenderer
from utils.throttling import UserRateThrottle
from utils.sch_soap_task_member import SoapMember
from utils.utils import sm4_decrypt
from system.models import SystemConfigs, DomainRule, DeviceLimit, DeviceOTP, DeviceStats
from utils.utils import read_licenseB, bytes_to_string

logger = logging.getLogger(__name__)


class SystemConfig(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        DOMAIN_NAME = os.environ.get("DOMAIN_NAME")
        DOMAIN_IP = os.environ.get("DOMAIN_IP")
        DOMAIN_SERVER_ADMIN = os.environ.get("DOMAIN_SERVER_ADMIN")
        DOMAIN_SERVER_ADMIN_PASSWD = os.environ.get("DOMAIN_SERVER_ADMIN_PASSWD")

        sys_ntp_server, _ = SystemConfigs.objects.get_or_create(code='ntp_server')
        DOMAIN_NTP_SERVER = sys_ntp_server.value

        sys_use_ntp_server, _ = SystemConfigs.objects.get_or_create(code='use_ntp_server')
        useNTPServer = True if sys_use_ntp_server.value == 'True' else False

        return Response({
            'code': 200,
            'data': {
                'DOMAIN_NAME': DOMAIN_NAME,
                'DOMAIN_IP': DOMAIN_IP,
                'DOMAIN_SSH': 'http://{0}:8888/?username={1}&password={2}&port=22'.format(
                    DOMAIN_IP, DOMAIN_SERVER_ADMIN,
                    base64.b64encode(DOMAIN_SERVER_ADMIN_PASSWD.encode('utf-8')).decode('utf-8')
                ),
                'DOMAIN_NTP_SERVER': DOMAIN_NTP_SERVER,
                'useNTPServer': useNTPServer,
            }
        })

    def put(self, req, *args, **kwargs):
        ntp_server = req.data.get('DOMAIN_NTP_SERVER')
        use_ntp_server = req.data.get('useNTPServer')

        sys_ntp_server, _ = SystemConfigs.objects.get_or_create(code='ntp_server')
        if ntp_server is not None:
            logger.info(ntp_server)
            sys_ntp_server.value = ntp_server
            sys_ntp_server.save()

        sys_use_ntp_server, _ = SystemConfigs.objects.get_or_create(code='use_ntp_server')
        if use_ntp_server is not None:
            logger.info(use_ntp_server)
            sys_use_ntp_server.value = 'True' if use_ntp_server == True else 'False'
            sys_use_ntp_server.save()

        opts = ''
        if use_ntp_server == False:
            opts = 'off'
        else:
            opts = ntp_server or sys_ntp_server.value

        if not opts:
            sys_use_ntp_server.value = 'False'
            sys_use_ntp_server.save()
        else:
            # from utils.rpc_client import RpcClient
            # rpc = RpcClient()
            # rpc.ntp_rpc_interface(opts)
            # res = subprocess.run(['docker', 'exec', 'nfs-domain-mainserver_freeipa_1', 'bash', 'ntp.sh', opts])
            # logger.info(res)
            pass

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '修改成功',
        })


class TerminalConfigViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = models.AccountEntity.objects.all()
    serializer_class = serializers.TerminalConfigSerializer
    pagination_class = MyPagination
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    ordering = ('name',)

    @classmethod
    def organ_down_list(cls, num):
        """获取父级及以下的部门"""
        organ_down_list = []
        items = OrgnizationEntity.objects.exclude(id=num).filter(parent=num)
        if not items.exists():
            return [int(num)]
        else:
            organ_down_list = organ_down_list + list(items.values_list('id', flat=True))
            while True:
                # logger.info(f'**********{items}*********')
                items = OrgnizationEntity.objects.filter(parent__in=items.values_list('id', flat=True))
                if not items.exists():
                    break
                else:
                    organ_down_list = organ_down_list + list(items.values_list('id', flat=True))

            organ_down_list.append(int(num))
        return organ_down_list

    @classmethod
    def make_hostname(cls, account_name):
        """
        主机名生成规则
        :return: 主机名
        """
        domain_rule_obj = DomainRule.objects.first()
        account_items = AccountEntity.objects.all()
        org_label_items = OrgLabel.objects.all()
        org_info_items = OrgnizationEntity.objects.all()
        device_items = Devices.objects.all()
        account_id = account_items.filter(name=account_name).first().org_info_id
        if domain_rule_obj.hostname_first_type == 0:
            """部门缩写,上海银行下一层级部门简称"""
            search_id = account_id
            if domain_rule_obj.is_up_search == 1:
                while True:
                    if org_label_items.filter(org_id=search_id).exists():
                        break
                    else:
                        if search_id == 1:
                            search_id = account_id
                            break
                        search_id = org_info_items.filter(id=search_id).first().parent_id
            else:
                search_id = account_id

            if org_label_items.filter(org_id=search_id).exists():
                hostname_first = org_label_items.get(org_id=search_id).org_label
            else:
                hostname_first = domain_rule_obj.other_org_label
        elif domain_rule_obj.hostname_first_type == 1:
            """自定义"""
            hostname_first = domain_rule_obj.hostname_first_name
        elif domain_rule_obj.hostname_first_type == 2:
            """用户账号"""
            hostname_first = account_name
        elif domain_rule_obj.hostname_first_type == 3:
            """工号"""
            if account_items.filter(name=account_name, create_type=2).exists():
                user_prefix = SoapMember().user_prefix
                hostname_first = account_name.split(f'{user_prefix}')[-1]
            else:
                hostname_first = account_name
        elif domain_rule_obj.hostname_first_type == 4:
            """IP"""
            hostname_first = None
        elif domain_rule_obj.hostname_first_type == 5:
            """MAC"""
            hostname_first = None
        else:
            hostname_first = None

        if domain_rule_obj.hostname_second_type == 0:
            """部门缩写, 上海银行下一层级部门简称"""
            search_id = account_id
            if domain_rule_obj.is_up_search == 1:
                while True:
                    if org_label_items.filter(org_id=search_id).exists():
                        break
                    else:
                        if search_id == 1:
                            search_id = account_id
                            break
                        search_id = org_info_items.filter(id=search_id).first().parent_id
            else:
                search_id = account_id
            if org_label_items.filter(org_id=search_id).exists():
                hostname_second = org_label_items.get(org_id=search_id).org_label
            else:
                hostname_second = domain_rule_obj.other_org_label
        elif domain_rule_obj.hostname_second_type == 1:
            hostname_second = domain_rule_obj.hostname_second_name
        elif domain_rule_obj.hostname_second_type == 2:
            hostname_second = account_name
        elif domain_rule_obj.hostname_second_type == 3:
            if account_items.filter(name=account_name, create_type=2).exists():
                user_prefix = SoapMember().user_prefix
                hostname_second = account_name.split(f'{user_prefix}')[-1]
            else:
                hostname_second = account_name
        elif domain_rule_obj.hostname_second_type == 4:
            hostname_second = None
        elif domain_rule_obj.hostname_second_type == 5:
            hostname_second = None
        else:
            hostname_second = None

        def get_host(mid_name):
            m = 2
            original_name = mid_name
            while True:
                if device_items.filter(device_cnname=mid_name).exists():
                    mid_name = f"{original_name}-{m}"
                    m = m + 1
                else:
                    break

            return mid_name

        if hostname_first is None:
            hostname = None
        else:
            if hostname_second is not None:
                hostname = f"{hostname_first}-{hostname_second}"
            else:
                hostname = f"{hostname_first}"
            hostname = get_host(hostname)

        return hostname

    def get_queryset(self):
        queryset = self.queryset
        # 根据部门过滤数据，超级管理员不需要
        user = User.objects.filter(username=self.request.user.username).first()
        org_list = get_org_permission_list(user)

        # 根据部门筛选数据
        if org_list:
            queryset = queryset.filter(org_info_id__in=org_list)

        return queryset

    def filter_queryset(self, queryset):
        """
        :param queryset: 表数据结果集合
        :return: 根据过滤条件筛选返回结果集合
        """
        cn_name = self.request.query_params.get('cn_name', default=None)
        name = self.request.query_params.get('name', default=None)
        address = self.request.query_params.get('address', default=None)
        org_id = self.request.query_params.get('org_id', default=None)
        is_root = self.request.query_params.get('is_root', default=None)
        system_status = self.request.query_params.get('system_status', default=None)
        policy_status = self.request.query_params.get('policy_status', default=None)
        device_items = Devices.objects.all().exclude(is_domain=0)
        if cn_name is not None:
            accounts_id_list = device_items.filter(device_cnname__contains=cn_name).values_list('accounts_id',
                                                                                                flat=True)
            queryset = queryset.filter(id__in=accounts_id_list)
        if name is not None:
            queryset = queryset.filter(name__contains=name)
        if address is not None:
            accounts_id_list = device_items.filter(device_ip__contains=address).values_list('accounts_id', flat=True)
            queryset = queryset.filter(id__in=accounts_id_list)
        if org_id is not None:
            queryset = queryset.filter(org_info_id__in=self.organ_down_list(org_id))
        if is_root is not None:
            queryset = queryset.filter(is_root=int(is_root))
        if system_status is not None:
            queryset = queryset.filter(system_status=int(system_status))
        if policy_status is not None:
            queryset = queryset.filter(policy_status=int(policy_status))

        return queryset

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def multiple_root(self, request, *args, **kwargs):
        """
        批量root权限开启和关闭
        :param request: body {is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        is_root： 0：未启用, 1: 启用}
                        query_params{......: 筛选条件}
        """
        is_all = request.data.get('is_all')
        ids_list = request.data.get('ids_list')
        is_root = request.data.get('is_root')

        if is_root is None:
            return Response({'code': 4001, 'msg': 'is_root 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        is_root = int(is_root)  # 将字符转换为数字

        if is_root not in [0, 1]:
            return Response({'code': 4001, 'msg': 'is_root 传参错误'}, status=status.HTTP_400_BAD_REQUEST)

        if int(is_all) == 1:
            queryset = self.filter_queryset(self.get_queryset())
            if is_root == 0:
                # 移除root账号权限到freeipa中hbac的sudo规则中
                name_list = queryset.exclude(is_root=0).values_list('name', flat=True)
                try:
                    # 批量移除root权限
                    # process = Process(target=freeipa.sudorule_all_remove_user,
                    #                           kwargs={'a_cn': 'sudo_admin', 's_user': name_list})
                    # process.start()
                    freeipa.sudorule_all_remove_user(a_cn='sudo_admin', s_user=name_list)
                except Exception as exp:
                    return Response({'code': 4001, 'msg': f'关闭root权限失败，错误信息: {exp}'},
                                    status=status.HTTP_400_BAD_REQUEST)
                queryset.exclude(is_root=0).update(is_root=is_root)
            else:
                # 上传root账号权限到freeipa中hbac的sudo规则中
                name_list = queryset.exclude(is_root=1).values_list('name', flat=True)
                try:
                    # process = Process(target=freeipa.sudorule_all_add_user,
                    #                           kwargs={'a_cn': 'sudo_admin', 's_user': name_list})
                    # process.start()
                    freeipa.sudorule_all_add_user(a_cn='sudo_admin', s_user=name_list)
                except Exception as exp:
                    return Response({'code': 4001, 'msg': f'批量开启root权限失败，错误信息: {exp}'},
                                    status=status.HTTP_400_BAD_REQUEST)

                queryset.exclude(is_root=1).update(is_root=is_root)

        elif int(is_all) == 0:
            if ids_list is None:
                return Response({'code': 4001, 'msg': 'ide_list 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)
            else:
                ids_list = [int(i) for i in ids_list.split(',')]
                if int(is_root) == 1:
                    # 上传root账号权限到freeipa中hbac的sudo规则中
                    name_list = self.queryset.exclude(is_root=1).filter(id__in=ids_list).values_list('name', flat=True)
                    try:
                        # 批量移除root权限
                        # process = Process(target=freeipa.sudorule_all_add_user,
                        #                           kwargs={'a_cn': 'sudo_admin', 's_user': name_list})
                        # process.start()
                        freeipa.sudorule_all_add_user(a_cn='sudo_admin', s_user=name_list)
                    except Exception as exp:
                        return Response({'code': 4001, 'msg': f'关闭root权限失败，错误信息: {exp}'},
                                        status=status.HTTP_400_BAD_REQUEST)
                    self.queryset.exclude(is_root=1).filter(id__in=ids_list).update(is_root=is_root)
                else:
                    # 移除root账号权限到freeipa中hbac的sudo规则中
                    name_list = self.queryset.exclude(is_root=0).filter(id__in=ids_list).values_list('name', flat=True)
                    try:
                        # logger.info(f'******* add s_user*****{name_list}')
                        # 批量移除root权限
                        # process = Process(target=freeipa.sudorule_all_remove_user,
                        #                           kwargs={'a_cn': 'sudo_admin', 's_user': name_list})
                        # process.start()
                        freeipa.sudorule_all_remove_user(a_cn='sudo_admin', s_user=name_list)
                    except Exception as exp:
                        return Response({'code': 4001, 'msg': f'关闭root权限失败，错误信息: {exp}'},
                                        status=status.HTTP_400_BAD_REQUEST)
                    self.queryset.exclude(is_root=0).filter(id__in=ids_list).update(is_root=is_root)

        else:
            return Response({'code': 4001, 'msg': 'is_all 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if is_root == 0:
            return Response({'code': 2000, 'msg': '关闭root账号成功'}, status=status.HTTP_200_OK)
        else:
            return Response({'code': 2000, 'msg': '开启root账号成功'}, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def multiple_org_root(self, request, *args, **kwargs):
        """
        批量系统激活
        :param request: body {
                        org_info_id: 部门id， 列表
                        is_root： 0：未启用, 1: 启用, 空|不操作
                        }
        """
        organ_id = request.data.get('org_info_id')
        is_root = request.data.get('is_root')

        if is_root is None:
            return Response({'code': 4001, 'msg': 'is_root 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if organ_id is None:
            return Response({'code': 4001, 'msg': 'is_organ 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if not isinstance(organ_id, list):
            return Response({'code': 4001, 'msg': 'is_organ必须传列表'}, status=status.HTTP_400_BAD_REQUEST)

        if is_root == 0:
            name_list = self.queryset.exclude(is_root=0).filter(org_info_id__in=organ_id).values_list('name', flat=True)
            try:
                freeipa.sudorule_all_remove_user(a_cn='sudo_admin', s_user=name_list)
            except Exception as exp:
                return Response({'code': 4001, 'msg': f'部门关闭root权限失败，错误信息: {exp}'},
                                status=status.HTTP_400_BAD_REQUEST)

            self.queryset.exclude(is_root=0).filter(org_info_id__in=organ_id).update(is_root=0)
        elif is_root == 1:
            name_list = self.queryset.exclude(is_root=1).filter(org_info_id__in=organ_id).values_list('name', flat=True)
            try:
                freeipa.sudorule_all_add_user(a_cn='sudo_admin', s_user=name_list)
            except Exception as exp:
                return Response({'code': 4001, 'msg': f'部门关闭root权限失败，错误信息: {exp}'},
                                status=status.HTTP_400_BAD_REQUEST)

            self.queryset.exclude(is_root=1).filter(org_info_id__in=organ_id).update(is_root=1)

        if is_root == 0:
            return Response({'code': 2000, 'msg': '部门关闭root账号成功'}, status=status.HTTP_200_OK)
        else:
            return Response({'code': 2000, 'msg': '部门开启root账号成功'}, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, permission_classes=[], authentication_classes=[])
    def change_password(self, request, *args, **kwargs):
        """
        修改终端账号密码
        :param request: username， old_password, new_password
        :param args:
        :param kwargs:
        :return:
        """
        # uuid = request.data['hostname']
        username = request.data['username']
        old_password = request.data['old_password']
        new_password = request.data['new_password']
        sm = SoapMember()
        oldPassword = sm4_decrypt(old_password, sm.soap_sm4_keys)
        newPassword = sm4_decrypt(new_password, sm.soap_sm4_keys)
        # logger.info(f"\n**************start***********\n"
        #             f"终端修改密码传入信息： \n"
        #             f"username: {username}\n"
        #             f"old_password: xxxxxxx\n"
        #             f"new_password: xxxxxxx\n"
        #             f"**************end************\n")
        # check pwd_prohibit_mod
        account = models.AccountEntity.objects.filter(name=username).first()
        if not account:
            return Response({'code': 4001, 'msg': '用户不存在'}, status=status.HTTP_200_OK)
            # if not models.AccountEntity.objects.filter(name=f'{sm.user_prefix}{username}').first():
        elif account.create_type == 2:
            res = freeipa.user_find(username)
            if not res:
                return Response(
                    {'status': 'error', 'message': '上行认证无此子系统用户信息！', 'code': 20013, 'data': ''})
            username = res.get('result').get('sn')[0]
            if sm.client.connect():
                soap_data = {
                    "appId": sm.appid,
                    "userId": username.lstrip(sm.user_prefix),
                    "oldPassword": oldPassword,
                    "newPassword": newPassword,
                    "authType": 'M_PASSWORD',
                    "otpCode": None,
                }
                res = sm.update_password_emop(soap_data)
                if res.get('status') != 'success':
                    return Response({'code': 4001, 'msg': res.get('message')}, status=status.HTTP_200_OK)
                return Response({'code': 2000, 'msg': '密码修改成功'}, status=status.HTTP_200_OK)

        # default_policy = DefaultPolicy()
        # if default_policy.get_default_policy().get('pwd_prohibit_mod') == '1':
        pwd_prohibit_mod = models.PasswordAccountProhibitModification.objects.filter(id=1).first()
        if pwd_prohibit_mod.status == 1:
            if pwd_prohibit_mod.range == 0:
                if not pwd_prohibit_mod.org_id or str(account.org_info_id) not in pwd_prohibit_mod.org_id.split(','):
                    return Response({'code': 4001, 'msg': '禁止修改密码'}, status=status.HTTP_200_OK)
            elif pwd_prohibit_mod.range == 1:
                if not pwd_prohibit_mod.account_id or str(account.id) not in pwd_prohibit_mod.account_id.split(','):
                    return Response({'code': 4001, 'msg': '禁止修改密码'}, status=status.HTTP_200_OK)
            else:
                return Response({'code': 4001, 'msg': '禁止修改密码'}, status=status.HTTP_200_OK)

        # web用户密码强度校验
        # re_code, re_msg = check_pwd(new_password)
        # if not re_code:
        #     return Response({'code': 4001, 'msg': re_msg}, status=status.HTTP_200_OK)

        # if not Devices.objects.filter(device_uuid=uuid, accounts_id__isnull=False).exists():
        #     return Response({'code': 4001, 'msg': f"UUID {uuid}为绑定域用户，请先绑定"}, status=status.HTTP_200_OK)
        # else:
        #     accounts_id = Devices.objects.get(device_uuid=uuid).accounts_id
        #     username = models.AccountEntity.objects.get(id=accounts_id).name

        # 原密码校验
        res = freeipa.check_user(username, oldPassword)
        logger.info(f"校验密码结果 {res}")
        if res['exist'] and (res['msg'] == '登录成功' or res['msg'] == '密码过期'):
            try:
                msg = freeipa.get_ipa_client().change_password(username, newPassword, oldPassword)
                logger.info(f"修改{username}密码，信息为{msg.__dict__}")
            except Exception as exp:
                if 'Password is too simple' in str(exp):
                    return Response({'code': 4001, 'msg': f'{username}密码修改失败，密码过于简单'},
                                    status=status.HTTP_200_OK)
                else:
                    return Response({'code': 4001, 'exp': f'{exp}', 'msg': f'{username}密码修改失败'},
                                    status=status.HTTP_200_OK)

            if msg.status_code != 200:
                msg_dict = msg.__dict__
                if 'Too soon to change password' in msg_dict:
                    return Response({'code': 4001, 'msg': f'{username}密码修改过于频繁'},
                                    status=status.HTTP_200_OK)
                else:
                    return Response({'code': 4001, 'msg': f'{username}密码修改失败，{msg.__dict__}'},
                                    status=status.HTTP_200_OK)
        else:
            return Response({'code': 4001, 'msg': f'校验原账号信息失败，错误信息为{res["exist"]},{res["msg"]}'},
                            status=status.HTTP_200_OK)

        return Response({'code': 2000, 'msg': f'{username}密码修改成功'}, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, permission_classes=[], authentication_classes=[])
    def automatic_hostname(self, request, *args, **kwargs):
        """
        自动生成主机名称
        :param request: account 账号
        :param args:
        :param kwargs:
        :return:
        """
        domain_rule_obj = DomainRule.objects.first()
        edit_hostname_switch = domain_rule_obj.edit_hostname_switch
        hostname_switch = domain_rule_obj.hostname_switch
        account_name = self.request.query_params.get('account_name', None)
        if (account_name is None) or (not account_name):
            return Response({
                'code': 2000,
                'hostname_switch': hostname_switch,
                'edit_hostname_switch': edit_hostname_switch,
                'hostname': None,
                'msg': None
            }, status=status.HTTP_200_OK)

        if not AccountEntity.objects.filter(name=account_name).exists():
            return Response({'code': 4001, 'msg': f'{account_name}账号不存在'}, status=status.HTTP_200_OK)

        if domain_rule_obj.hostname_switch == 1:
            hostname = self.make_hostname(account_name)
            return Response(
                {
                    'code': 2000,
                    'hostname_switch': hostname_switch,
                    'edit_hostname_switch': edit_hostname_switch,
                    'hostname': hostname,
                    'msg': None
                },
                status=status.HTTP_200_OK
            )
        else:
            return Response(
                {
                    'code': 2000,
                    'hostname_switch': hostname_switch,
                    'edit_hostname_switch': edit_hostname_switch,
                    'hostname': None,
                    'msg': None
                },
                status=status.HTTP_200_OK
            )

    @action(methods=['get'], detail=False, permission_classes=[], authentication_classes=[])
    def check_hostname(self, request, *args, **kwargs):
        """
        终端检查用户别名是否存在
        :param request: hostname 
        :param args: 
        :param kwargs: 
        :return: 
        """
        domain_auth = DomainLicenseAuth()
        if not domain_auth.license_b:
            return Response({'code': 4001, 'msg': '服务器未激活,请联系管理员'}, status=status.HTTP_200_OK)
        mac = os.environ.get("DOMAIN_MAC")
        license_info = read_licenseB(mac, domain_auth.license_b)

        if not license_info.get('register_res'):
            return Response({'code': 4001, 'msg': '服务器激活码错误,请联系管理员'}, status=status.HTTP_200_OK)

        max_users = bytes_to_string(license_info.get('max_users_res', 0))

        domain_name = os.environ.get("DOMAIN_NAME")
        if not domain_name:
            return Response({'code': 4001, 'msg': '未设置域名,请联系管理员'}, status=status.HTTP_200_OK)

        if Devices.objects.exclude(is_domain=0).all().count() >= int(max_users):
            return Response({'code': 4001, 'msg': '超出最大终端管控数,请联系管理员'}, status=status.HTTP_200_OK)

        hostname = self.request.query_params.get('hostname', None)
        if not hostname:
            return Response({'code': 4000, 'msg': 'hostname 为必输项'}, status=status.HTTP_200_OK)
        hostname = f'{hostname}.{os.environ.get("DOMAIN_NAME")}'
        if Devices.objects.exclude(is_domain=0).filter(device_uuid=hostname).exists():
            return Response({'code': 4001, 'msg': 'hostname 已存在'}, status=status.HTTP_200_OK)
        else:
            return Response({'code': 2000, 'domain_name': os.environ.get("DOMAIN_NAME"), 'msg': 'hostname没被使用'},
                            status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, permission_classes=[], authentication_classes=[])
    def check_pwd_prohibit_mod(self, request, *args, **kwargs):
        # check pwd_prohibit_mod
        name = request.query_params.get('name')
        account = models.AccountEntity.objects.filter(name=name).first()
        if not account:
            return Response({'code': 4001, 'msg': '用户不存在'}, status=status.HTTP_200_OK)
        pwd_prohibit_mod = models.PasswordAccountProhibitModification.objects.filter(id=1).first()
        if pwd_prohibit_mod.status == 1:
            if pwd_prohibit_mod.range == 0:
                if not pwd_prohibit_mod.org_id or str(account.org_info_id) not in pwd_prohibit_mod.org_id.split(','):
                    return Response({'code': 4001, 'msg': '禁止修改密码'}, status=status.HTTP_200_OK)
            elif pwd_prohibit_mod.range == 1:
                if not pwd_prohibit_mod.account_id or str(account.id) not in pwd_prohibit_mod.account_id.split(','):
                    return Response({'code': 4001, 'msg': '禁止修改密码'}, status=status.HTTP_200_OK)
            else:
                return Response({'code': 4001, 'msg': '禁止修改密码'}, status=status.HTTP_200_OK)
        return Response({'code': 2000, 'msg': '允许修改密码'}, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, permission_classes=[], authentication_classes=[])
    def terminal_version(self, request, *args, **kwargs):
        """
        源中的终端版本信息
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        nfs4_codename = 'nfs-4.0'
        nfs5_codename = 'nfs-5.0'
        package_name = "nfs-domain-client"
        aptly_method_class = AptlyUtil()
        nfs4_code, nfs4_msg = aptly_method_class.get_package_versions(repo_name=nfs4_codename,
                                                                      package_name=package_name)
        nfs5_code, nfs5_msg = aptly_method_class.get_package_versions(repo_name=nfs5_codename,
                                                                      package_name=package_name)
        if not nfs4_code:
            return Response({'code': 4000, 'nfs4_msg': nfs4_msg}, status=status.HTTP_200_OK)
        if not nfs5_code:
            return Response({'code': 4000, 'nfs4_msg': nfs5_msg}, status=status.HTTP_200_OK)

        return Response({'code': 2000, 'nfs4-ver': nfs4_msg, 'nfs5-ver': nfs5_msg}, status=status.HTTP_200_OK)


class DomainRuleViewSet(mixins.RetrieveModelMixin,
                        mixins.UpdateModelMixin,
                        GenericViewSet):
    queryset = DomainRule.objects.all()
    serializer_class = serializers.DomainRuleSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    ordering = ('-id',)


class DeviceLimitViewSet(viewsets.ModelViewSet):
    queryset = DeviceLimit.objects.all()
    serializer_class = serializers.DeviceLimitSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    ordering = ('-id',)

    def list(self, request, *args, **kwargs):
        instance = self.queryset.first()
        serializer = self.get_serializer(instance)
        return Response(serializer.data)

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop('partial', False)
        instance = self.get_object()

        request_is_limit_switch = request.data.get("is_limit_switch")
        request_limit_rate = request.data.get("limit_rate")
        request_limit_conn_perip = request.data.get("limit_conn_perip")
        if (request_is_limit_switch != instance.is_limit_switch) or (request_limit_rate != instance.limit_rate) or (
                request_limit_conn_perip != instance.limit_conn_perip):
            url = f"http://{HOSTNAME}:6505/nginx/limit/"
            data = {"limit": request_is_limit_switch,
                    "limit_rate": f"{request_limit_rate}m" if request_limit_rate is not None else None,
                    "limit_conn_perip": int(request_limit_conn_perip) if request_limit_conn_perip is not None else None}
            # 发送POST请求，注意headers中指定了Content-Type为application/json
            logger.info(f"********{data}")
            response = requests.post(url, data=json.dumps(data), headers={'Content-Type': 'application/json'})

            # 检查请求是否成功
            logger.info(f"********{response.text}")
            if response.status_code == 200:
                logger.info("数据发送成功，服务器响应:", response.text)
            else:
                logger.info(f"请求失败，状态码：{response.status_code}, 服务器响应: {response.text}")
                return Response(f"{response.text}", status=response.status_code)

        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    def perform_update(self, serializer):
        serializer.save()

    def partial_update(self, request, *args, **kwargs):
        kwargs['partial'] = True
        return self.update(request, *args, **kwargs)


class DeviceOTPViewSet(mixins.ListModelMixin,
                       mixins.UpdateModelMixin,
                       GenericViewSet):
    queryset = DeviceOTP.objects.all()
    serializer_class = serializers.DeviceOTPSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    pagination_class = None
    ordering = ('-id',)

    def list(self, request, *args, **kwargs):
        latest_otp = DeviceOTP.objects.filter(id=1).first()
        if not latest_otp:
            DeviceOTP.objects.create(
                **{"id": 1,
                   "code": str(random.randint(100000, 999999)),
                   "created_at": datetime.datetime.now()
                   }
            )
        else:
            if (datetime.datetime.now() - latest_otp.created_at) > datetime.timedelta(seconds=180):
                latest_otp.code = str(random.randint(100000, 999999))
                latest_otp.created_at = datetime.datetime.now()
                latest_otp.save()
            else:
                pass

        instance = DeviceOTP.objects.filter(id=1).first()
        serializer = self.get_serializer(instance)
        return Response(serializer.data)

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop('partial', False)
        instance = DeviceOTP.objects.filter(id=1).first()
        instance.code = str(random.randint(100000, 999999))
        instance.created_at = datetime.datetime.now()
        instance.save()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    @action(methods=['get'], detail=False, permission_classes=[], authentication_classes=[])
    def check_device_otp(self, request, *args, **kwargs):
        """
        检查终端的卸载动态口令
        """
        uninstall_otp = self.request.query_params.get('device_otp', None)
        latest_otp = DeviceOTP.objects.filter(id=1).first()
        if not latest_otp:
            DeviceOTP.objects.create(
                **{"id": 1,
                   "code": str(random.randint(100000, 999999)),
                   "created_at": datetime.datetime.now()
                   }
            )
        # logger.info(f"{uninstall_otp}----{latest_otp.code}")
        otp_time = datetime.datetime.now() - latest_otp.created_at
        otp_time = int(180 - otp_time.total_seconds())
        if (latest_otp.code == str(uninstall_otp)) and (otp_time >= 0):
            latest_otp.code = str(random.randint(100000, 999999))
            latest_otp.created_at = datetime.datetime.now()
            latest_otp.save()
            return Response({'code': 2000, 'msg': '卸载口令正确'}, status=status.HTTP_200_OK)
        else:
            return Response({'code': 4001, 'msg': '卸载口令不正确或者已过期'}, status=status.HTTP_200_OK)


class DeviceStatsViewSet(mixins.RetrieveModelMixin,
                        mixins.UpdateModelMixin,
                        GenericViewSet):
    """
    终端纳管统计设置:
    1,终端纳管统计进度开关
    2,终端纳管总数值
    """
    queryset = DeviceStats.objects.all()
    serializer_class = serializers.DeviceStatsSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
