# -*- coding: utf-8 -*-
import datetime
import ipaddress
import logging
import threading
import json

import pandas as pd
from rest_framework import status
from rest_framework.views import APIView, Response
from rest_framework.viewsets import ModelViewSet

from accounts.models import AccountEntity
from devices.client_consumers import msg_to_client
from patch_management.models import PatchFile, PatchInfo, PatchDeviceLog
from saltapi.models import WaitDevicePolicy
from software.models import SoftwareInfoModel, SoftwareInstalledModel
from tasks.serializers import TasksSerializer
from utils.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from rest_framework.pagination import PageNumberPagination
from rest_framework import permissions
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.decorators import api_view, permission_classes, authentication_classes, action
from django.db.models import Q
from django_redis import get_redis_connection

from utils.pagination import MyPagination
from utils.permission import IsAdminUser
from utils.response import FitJSONRenderer
from utils.sch_patch import PatchTask
from utils.throttling import UserRateThrottle
from tasks.models import Tasks, TaskDeviceExec
from filemanager.models import FileManager
from utils.utils import str_to_list, list_to_str
from devices.models import Devices
from organizations.models import OrgnizationEntity
from tasks.task_action import start_task, schedule_task_action
from utils.time_utils import run_get_next_time
from groups.models import GroupsDetail

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')


def get_rate(task_id):
    """
    获取完成率
    """
    total_device = TaskDeviceExec.objects.filter(task__pk=task_id).values_list('device_id', flat=False)
    total_device_num = len(list(set(list(total_device.values_list('device_id', flat=False)))))
    success_device_num = len(list(set(list(total_device.filter(exec_status=4).values_list('device_id', flat=False)))))
    if total_device_num == 0:
        rate_str = "0/0,0"
        return rate_str
    rate = round(success_device_num / (total_device_num) * 100, 2)
    rate_str = "%s/%s,%s" % (success_device_num, total_device_num, rate)
    return rate_str


def patch_rate(task_id):
    """
    获取补丁完成率
    """
    task_obj = Tasks.objects.get(id=task_id)
    patch_id_string = task_obj.file_ids_range
    task_range = task_obj.task_range

    # 任务范围，0 按部门、1按终端、2 按分组, 3 人员， 4 ip段
    device_list = []
    if task_obj.range in ['0', 0]:
        # 部门筛选
        org_id = task_range.split(',')
        org_device_list = Devices.objects.filter(org_info__in=org_id).exclude(is_domain=0).distinct()
        for device in org_device_list:
            device_list.append(device)
    elif task_obj.range in ['1', 1]:
        # 终端筛选
        check_id = task_range
        check_id_list = str_to_list(check_id)
        for check_id in check_id_list:
            if Devices.objects.filter(pk=check_id).exists():
                device = Devices.objects.get(pk=check_id)
                device_list.append(device)
            else:
                pass
    elif task_obj.range in ['2', 2]:
        # 分组筛选
        groups_id = task_range
        groups_id_list = str_to_list(groups_id)
        group_chat_items = GroupsDetail.objects.all()
        if group_chat_items.filter(groups_id__in=groups_id_list).exists():
            group_chat_filter_items = group_chat_items.filter(groups_id__in=groups_id_list)
            items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
            items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
            items_3_range = group_chat_filter_items.filter(range=3).values_list('account_id', flat=True)
            # 处理部门数据
            if Devices.objects.filter(org_info_id__in=items_0_range).exclude(is_domain=0).exists():
                device_0_list = Devices.objects.filter(org_info_id__in=items_0_range).exclude(
                    is_domain=0).values_list('id', flat=True)
                device_0_list = list(device_0_list)
            else:
                device_0_list = []
            # 处理设备信息
            device_1_list = list(items_1_range)
            # 处理账号信息
            if Devices.objects.filter(bind_account__id__in=items_3_range).exclude(is_domain=0).exists():
                device_3_list = Devices.objects.filter(bind_account__id__in=items_3_range).exclude(
                    is_domain=0).values_list('id',
                                             flat=True)
                device_3_list = list(device_3_list)
            else:
                device_3_list = []
            device_list = device_0_list + device_1_list + device_3_list
            device_list = Devices.objects.filter(id__in=list(set(device_list)))

    elif task_obj.range in ['3', 3]:
        # 按人员筛选
        user_id = task_range
        account_list = user_id.split(',')
        if Devices.objects.filter(bind_account__id__in=account_list).exclude(is_domain=0).exists():
            device_list = Devices.objects.filter(bind_account__id__in=account_list).exclude(is_domain=0)
            device_list = list(device_list)
        else:
            device_list = []
    elif task_obj.range in ['4', 4]:
        # 根据ip段范围获取设备
        task_range = task_range
        ip_list = task_range.split(',')
        mid_ip_list = []
        for i in ip_list:
            start_ip, end_ip = i.split('-')
            # 解析起始和结束IP地址为IPv4Address对象
            start_ip = ipaddress.IPv4Address(start_ip)
            end_ip = ipaddress.IPv4Address(end_ip)
            range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
            mid_ip_list = range_ip_list + mid_ip_list
        if Devices.objects.filter(device_ip__in=mid_ip_list).exclude(is_domain=0).exists():
            device_list = Devices.objects.filter(device_ip__in=mid_ip_list).exclude(is_domain=0)
            device_list = list(device_list)
        else:
            device_list = []

    patch_id_list = patch_id_string.split(',') if patch_id_string is not None else []
    device_id_list = [i.id for i in device_list]
    patch_device_items = PatchDeviceLog.objects.filter(device_id__in=device_id_list, patch_id__in=patch_id_list)
    total_device_num = patch_device_items.count()
    success_device_num = patch_device_items.filter(install_status__in=[1, 2]).count()
    fail_device_num = patch_device_items.filter(install_status=0).filter(install_err_msg__isnull=False).count()
    if patch_device_items.exists():
        install_time = patch_device_items.order_by('-install_time').first().install_time.strftime('%Y-%m-%d %H:%M:%S')
    else:
        install_time = ""

    if total_device_num == 0:
        rate_str = "0/0,0"
        return rate_str, ""
    rate = round(success_device_num / (total_device_num) * 100, 2)
    rate_str = "%s/%s,%s" % (success_device_num, total_device_num, rate)
    if success_device_num == total_device_num:
        task_obj.status = 4
    elif fail_device_num == total_device_num:
        task_obj.status = 5
    task_obj.save()
    return rate_str, install_time


class TasksListViewSet(APIView):
    authentication_classes = (TokenAuthentication,)

    # permission_classes = (IsAuthenticated, IsAdminUser)
    # throttle_classes = (UserRateThrottle,)

    # def get_rate(self, task_id):
    #     """
    #     获取完成率
    #     """
    #     total_device = TaskDeviceExec.objects.filter(task__pk=task_id)
    #     success_device = total_device.filter(exec_status=4)
    #     if len(total_device) == 0:
    #         rate_str = "0/0,0"
    #         return rate_str
    #     rate = round(len(success_device) / (len(total_device)) * 100, 2)
    #     rate_str = "%s/%s,%s" % (len(success_device), len(total_device), rate)
    #     return rate_str

    # 任务分页查询
    def get(self, request, *args, **kwargs):
        create_user = request.user
        if create_user.is_superuser == 1:
            task = Tasks.objects.filter(is_deleted=0)
        else:
            task = Tasks.objects.filter(create_user=create_user, is_deleted=0)

        name = request.query_params.get('name')
        if name:
            task = task.filter(name__icontains=name)

        create_user_name = request.query_params.get('create_user_name')
        if create_user_name:
            task = task.filter(create_user__username__icontains=create_user_name)

        task_class = request.query_params.get('task_class')
        if task_class:
            task = task.filter(task_class=task_class)

        quality = request.query_params.get('quality')
        if quality:
            task = task.filter(quality=quality)

        task_status = request.query_params.get('status')
        if task_status:
            task = task.filter(status=task_status)

        range = request.query_params.get('range')
        if range:
            task = task.filter(range=range)

        task = task.order_by("-id")
        total = task.count()
        page = PageNumberPagination()
        page.page_size = 10  # 默认每页显示记录
        page.page_query_param = 'current'  # 默认查询参数名为 page
        page.page_size_query_param = 'pageSize'
        page.max_page_size = 100000  # 后台控制显示的最大记录条数，防止用户输入的查询条数过大
        try:
            ret = page.paginate_queryset(task, request)
        except:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': 'error',
                'success': False,
                'data': []
            })
        res_list = []
        for task in ret:
            if task.quality == 2:
                exec_time = run_get_next_time(task.exec_time)
            else:
                exec_time = task.exec_time
            if task.task_class == 1:
                rate, success_time = patch_rate(task.id)
            else:
                rate = get_rate(task.id)

            task = Tasks.objects.get(id=task.id)
            res_list.append({
                'id': task.id,
                'name': task.name,
                'task_class': task.task_class,
                'quality': task.quality,
                'create_user': task.create_user.pk,
                'create_user_name': task.create_user.username,
                'exec_time': exec_time,
                'rate': rate,
                'status': task.status,
                'remarks': task.remarks
            })
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '完成任务分页查询',
            'total': total,
            'success': True,
            'data': res_list
        })


class TasksViewSet(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, *args, **kwargs):
        task_id = kwargs.get('task_id')
        if not task_id:
            return Response({
                'code': status.HTTP_401_UNAUTHORIZED,
                'msg': '请求参数有误'
            })

        try:
            task_obj = Tasks.objects.get(pk=task_id, is_deleted=0)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '任务不存在'
            })
        if task_obj:
            file_obj_list = FileManager.objects.filter(task=task_id)

            file_info_list = []
            for file_obj in file_obj_list:
                file_info = {
                    "file_id": file_obj.pk,
                    "file_name": file_obj.name
                }
                file_info_list.append(file_info)

            if task_obj.task_class == 1:
                file_ids_range = task_obj.file_ids_range
                patch_is_list = file_ids_range.split(',') if file_ids_range is not None else []
                file_info_list = []
                for items in PatchInfo.objects.filter(id__in=patch_is_list).values('id', 'patch_name'):
                    file_info_list.append({'file_id': items['id'], 'file_name': items['patch_name']})

            check_id = ""
            task_device_exec = TaskDeviceExec.objects.filter(task=task_obj)
            devices_items = Devices.objects.exclude(is_domain=0).values_list('id', flat=True)
            task_device_exec = task_device_exec.filter(device_id__in=devices_items)
            check_id_list = []
            check_id_status_dict = {}
            for item in task_device_exec:
                check_id_list.append(item.device.pk)
                check_id_status_dict[str(item.device.pk)] = item.exec_status

            check_id_list = [str(x.device.pk) for x in task_device_exec if task_device_exec]
            check_id_list_str = list_to_str(check_id_list)
            if task_obj.quality == 2:
                exec_time = run_get_next_time(task_obj.exec_time)
            else:
                exec_time = task_obj.exec_time
            if task_obj.task_class == 1:
                rate, success_time = patch_rate(task_obj.id)
            else:
                rate = get_rate(task_obj.id)
                success_time = task_obj.success_time if task_obj.success_time else ""

            task_obj = Tasks.objects.get(pk=task_id, is_deleted=0)
            res_obj = {
                'id': task_obj.id,
                'name': task_obj.name,
                'task_class': task_obj.task_class,
                'quality': task_obj.quality,
                'range': task_obj.range,
                'rate': rate,
                'create_user': task_obj.create_user.pk,
                'create_user_name': task_obj.create_user.username,
                'created_at': task_obj.created_at.strftime('%Y-%m-%d %H:%M:%S'),
                'exec_time': exec_time,
                'success_time': success_time,
                'check_id': check_id_list_str,
                'org_id': eval(task_obj.org_id) if task_obj.org_id else "",
                'check_id_status': check_id_status_dict,
                'status': task_obj.status,
                'remarks': task_obj.remarks,
                'file_data': file_info_list,
                'is_software_source': task_obj.is_software_source,
                'need_reboot': task_obj.need_reboot,
                'need_prompt': task_obj.need_prompt,
                'install_complete_time': task_obj.install_complete_time
            }
            return Response({
                'code': status.HTTP_200_OK,
                'msg': '获取任务信息完成',
                'data': res_obj
            })
        else:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '获取任务信息发生错误',
                'data': {}
            })

    def post(self, request, *args, **kwargs):
        """
        创建任务
        param
            {
                "name":"test_task",
                "task_class":0,
                "quality":0,
                "range":0,
                "status":0,
                "org_id":[1,2],
                "check_id":'1,2',
                "file_path":'/tmp',
                "exec_time":"1676511079,1676511179",
                "remarks":"test remarks"
            }
        """
        req_data = request.data.copy()
        task_class = req_data.get('task_class')
        file_id_str = req_data.get('file_id_list')
        task_range = req_data.get('task_range')
        device_list = []
        # 任务范围，0 按部门、1按终端、2 按分组, 3 人员， 4 ip段
        if req_data.get('range') in ['0', 0]:
            # 部门筛选
            org_id = task_range.split(',')
            org_device_list = Devices.objects.filter(org_info__in=org_id).exclude(is_domain=0).distinct()
            for device in org_device_list:
                device_list.append(device)
        elif req_data.get('range') in ['1', 1]:
            # 终端筛选
            check_id = task_range
            check_id_list = str_to_list(check_id)
            for check_id in check_id_list:
                device = Devices.objects.get(pk=check_id)
                device_list.append(device)
        elif req_data.get('range') in ['2', 2]:
            # 分组筛选
            groups_id = task_range
            groups_id_list = str_to_list(groups_id)
            group_chat_items = GroupsDetail.objects.all()
            if group_chat_items.filter(groups_id__in=groups_id_list).exists():
                group_chat_filter_items = group_chat_items.filter(groups_id__in=groups_id_list)
                items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
                items_3_range = group_chat_filter_items.filter(range=3).values_list('account_id', flat=True)
                # 处理部门数据
                if Devices.objects.filter(org_info_id__in=items_0_range).exclude(is_domain=0).exists():
                    device_0_list = Devices.objects.filter(org_info_id__in=items_0_range).exclude(
                        is_domain=0).values_list('id', flat=True)
                    device_0_list = list(device_0_list)
                else:
                    device_0_list = []
                # 处理设备信息
                device_1_list = list(items_1_range)
                # 处理账号信息
                if Devices.objects.filter(bind_account__id__in=items_3_range).exclude(is_domain=0).exists():
                    device_3_list = Devices.objects.filter(bind_account__id__in=items_3_range).exclude(
                        is_domain=0).values_list('id',
                                                 flat=True)
                    device_3_list = list(device_3_list)
                else:
                    device_3_list = []
                device_list = device_0_list + device_1_list + device_3_list
                device_list = Devices.objects.filter(id__in=list(set(device_list)))

        elif req_data.get('range') in ['3', 3]:
            # 按人员筛选
            user_id = task_range
            account_list = user_id.split(',')
            if Devices.objects.filter(bind_account__id__in=account_list).exclude(is_domain=0).exists():
                device_list = Devices.objects.filter(bind_account__id__in=account_list).exclude(is_domain=0)
                device_list = list(device_list)
            else:
                device_list = []
        elif req_data.get('range') in ['4', 4]:
            # 根据ip段范围获取设备
            task_range = task_range
            ip_list = task_range.split(',')
            mid_ip_list = []
            for i in ip_list:
                start_ip, end_ip = i.split('-')
                # 解析起始和结束IP地址为IPv4Address对象
                start_ip = ipaddress.IPv4Address(start_ip)
                end_ip = ipaddress.IPv4Address(end_ip)
                range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
                mid_ip_list = range_ip_list + mid_ip_list
            if Devices.objects.filter(device_ip__in=mid_ip_list).exclude(is_domain=0).exists():
                device_list = Devices.objects.filter(device_ip__in=mid_ip_list).exclude(is_domain=0)
                device_list = list(device_list)
            else:
                device_list = []

        task = Tasks.objects.create(
            name=req_data.get('name'),
            task_class=req_data.get('task_class'),
            file_ids_range=file_id_str,
            quality=req_data.get('quality'),
            range=req_data.get('range'),
            task_range=task_range,
            status=2 if req_data.get('quality') == 2 else 0,
            rate="0/0,0",
            exec_time=req_data.get('exec_time'),
            remarks=req_data.get('remarks'),
            file_path=req_data.get('file_path'),
            create_user=request.user,
            org_id=str(req_data.get('org_id')) if req_data.get('org_id') else None,
            groups_id=str(req_data.get('groups_id')) if req_data.get('groups_id') else None,
            need_reboot=int(req_data.get('need_reboot')) if req_data.get('need_reboot', None) is not None else None,
            need_prompt=int(req_data.get('need_prompt')) if req_data.get('need_prompt', None) is not None else None,
            install_complete_time=req_data.get('install_complete_time', None),
            is_software_source=req_data.get('is_software_source', 0)
        )

        if int(task_class) == 1:
            file_id_str = file_id_str
        else:
            if file_id_str:
                file_id_list = str_to_list(file_id_str)
                file_items = FileManager.objects.filter(id__in=file_id_list)
                if file_items.filter(task__isnull=True):
                    file_items.update(task=task)
                else:
                    file_id_list = []
                    for i in file_items.values('name', 'software_id', 'main_package', 'desc', 'version', 'size', 'type',
                                               'path', 'created_at', 'last_update'):
                        i['task'] = task
                        file_id = FileManager.objects.create(**i)
                        file_id_list.append(file_id.id)
            else:
                file_id_list = []

        if int(task_class) == 1:
            # 补丁源添加补丁信息
            res_item = []
            patch_id_list = file_id_str.split(',')
            patch_file_items = PatchFile.objects.all()
            patch_info_items = PatchInfo.objects.all()
            for i in device_list:
                for n in patch_id_list:
                    patch_file_id = patch_info_items.get(id=n).patch_file_id
                    patch_file_name = patch_file_items.get(id=patch_file_id).name
                    res_item.append({
                        'device_id': str(i.id),
                        'device_hostname': i.device_hostname,
                        'patch_id': n,
                        'patch_name': patch_file_name
                    })

            thread1 = threading.Thread(target=PatchTask().__bulk_data_delivery__, args=(res_item,))  # 将数据同步到缓存中
            thread1.start()

            task.exec_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            task.status = 2 if len(device_list) != 0 else 0
            task.save()
        else:
            device_list = list(set(device_list))  # 去重，兼容终端分组重复选择
            for device in device_list:
                #  去除被禁用的终端
                if device.is_up == 0:
                    continue
                if int(req_data.get('task_class')) != 2:
                    for i in file_id_list:
                        TaskDeviceExec.objects.create(
                            task=task,
                            device=device,
                            file_obj=i,
                            exec_status=2 if task.quality == 2 else 0,
                            exec_time=task.exec_time
                        )
                else:
                    TaskDeviceExec.objects.create(
                        task=task,
                        device=device,
                        exec_status=2 if task.quality == 2 else 0,
                        exec_time=task.exec_time
                    )

            logger.info("create task %s success" % task.id)

            if int(req_data.get('is_software_source', 0)) == 1:
                logger.info("websocket task %s start" % task.id)
                # send install msg to device_client
                send_msg = {'sender': 'SERVER', 'sn': '9-2-1', 'data': None}
                task_file_items = FileManager.objects.filter(task_id=task.id)
                deb_package_list = []
                for item in task_file_items:
                    deb_package_list.append({'name': item.name.lower(), 'version': item.version})
                    software_info_id = SoftwareInfoModel.objects.filter(custome_filename=item.main_package,
                                                                        is_delete=0).first().id
                    for device_items in device_list:
                        SoftwareInstalledModel.objects.create(
                            name=item.name,
                            main_package=item.name,
                            main_version=item.version,
                            created_at=datetime.datetime.now(),
                            last_update=datetime.datetime.now(),
                            is_success_install=0,
                            is_histroy_install=0,
                            installed_status=2,
                            device_info_id=device_items.id,
                            software_info_id=software_info_id,
                            task_id=task.id
                        )
                send_msg.update(
                    {'data': {
                        'taskid': str(task.id),
                        'task_name': task.name,
                        'need_reboot': task.need_reboot,
                        'need_immediate': task.need_prompt,
                        'install_complete_time': task.install_complete_time,
                        'debpackage': deb_package_list
                    }})
                task.exec_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
                task.status = 2
                task.save()
                # 查找终端对应的通道下发消息
                for items in device_list:
                    try:
                        channel_client_name = redis_conn.hget('client_channel_name', items.device_hostname)
                        logger.info(f"********{channel_client_name}*****")
                        if channel_client_name:
                            channel_client_name = channel_client_name.decode('utf-8')
                            msg_to_client(1, channel_client_name, send_msg)
                            TaskDeviceExec.objects.filter(task_id=task.id, device_id=items.id).update(exec_status=2)
                        else:
                            # 处理未被发送出去的指令
                            device_unsent_value = redis_conn.hget('client_websocket_unsent', items.device_hostname)
                            if device_unsent_value:
                                device_unsent_value = device_unsent_value.decode('utf-8')
                                device_unsent_value = json.loads(device_unsent_value)
                                device_unsent_value.append(send_msg)
                                redis_conn.hset('client_websocket_unsent', items.device_hostname,
                                                json.dumps(device_unsent_value, ensure_ascii=False))
                            else:
                                redis_conn.hset('client_websocket_unsent', items.device_hostname,
                                                json.dumps([send_msg], ensure_ascii=False))
                    except Exception as exp:
                        logger.info(f"获取主机{items.device_hostname}对应的管道失败{exp}")
            else:
                start_task(task.id)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '新增任务成功'
        })

    def put(self, request):
        """
        修改任务
        :param request:
        is_all 是否全选，0不选取，1全选
        task_id_list task_id 列表
        task_action 任务操作，0 暂停、1 取消、2 删除、3失败重试
            {
                "is_all": 0，
                "task_action": 0，
                "file_path": "/tmp"，
                "task_id_list":"3,4",
                "name":"test_task",
                "task_class":0,
                "quality":0,
                "range":0,
                "status":0,
                "exec_time":"1676511079,1676511179",
                "remarks":"test remarks"
            }
        :return:
        """
        query_name = request.query_params.get('name')
        query_task_class = request.query_params.get('task_class')
        query_quality = request.query_params.get('quality')
        query_create_user_name = request.query_params.get('create_user_name')
        query_status = request.query_params.get('status')
        req_data = request.data
        is_all = req_data.get('is_all')
        task_id_list_str = req_data.get('task_id_list')
        task_id_list = str_to_list(task_id_list_str)
        create_user = request.user

        try:
            if is_all == 1:
                if create_user.is_superuser:
                    tasks = Tasks.objects.filter(is_deleted=0)
                else:
                    tasks = Tasks.objects.filter(create_user=create_user, is_deleted=0)
            else:
                if create_user.is_superuser:
                    tasks = Tasks.objects.filter(id__in=task_id_list, is_deleted=0)
                else:
                    tasks = Tasks.objects.filter(create_user=create_user, id__in=task_id_list, is_deleted=0)

        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '任务不存在'
            })

        # 兼容批量操作和更新
        if query_name:
            tasks = tasks.filter(name__icontains=query_name)
        if query_task_class:
            tasks = tasks.filter(task_class=query_task_class)
        if query_quality:
            tasks = tasks.filter(quality=query_quality)
        if query_create_user_name:
            tasks = tasks.filter(create_user__username__icontains=query_create_user_name)
        if query_status:
            tasks = tasks.filter(status=query_status)

        if req_data.get('name'):
            tasks.update(name=req_data.get('name'))

        if req_data.get('task_class'):
            tasks.update(task_class=req_data.get('task_class'))

        if req_data.get('quality'):
            tasks.update(quality=req_data.get('quality'))

        if req_data.get('range'):
            tasks.update(range=req_data.get('range'))

        if req_data.get('status'):
            tasks.update(status=req_data.get('status'))

        if req_data.get('exec_time'):
            tasks.update(exec_time=req_data.get('exec_time'))

        if req_data.get('remarks'):
            tasks.update(remarks=req_data.get('remarks'))

        if req_data.get('file_path'):
            tasks.update(file_path=req_data.get('file_path'))

        # 用户操作
        task_action = req_data.get('task_action')
        if task_action == 0:
            # 暂停任务
            action = "disable"
            schedule_task_action(tasks, action)
            tasks = tasks.filter(~Q(status=4))
            if tasks:
                tasks.update(status=1)
        elif task_action == 1:
            # 取消任务
            action = "disable"
            schedule_task_action(tasks, action)
            tasks = tasks.filter(~Q(status=4))
            if tasks:
                tasks.update(status=3)
        elif task_action == 2:
            # 删除任务
            action = "delete"
            schedule_task_action(tasks, action)
            # 删除任务删除等待策略
            wait_device_policy = WaitDevicePolicy.objects.all()
            for item in tasks:
                if wait_device_policy.filter(arg__contains=f'"task_id": {item.id},').exists():
                    wait_device_policy.filter(arg__contains=f'"task_id": {item.id},').delete()
                else:
                    pass
            tasks.update(is_deleted=1)

        else:
            # task_action = 3
            # 恢复任务 和 失败重试
            action = "enable"
            tasks = tasks.filter(~Q(status=4))
            schedule_task_action(tasks, action)
            if tasks:
                tasks.update(status=2)

            for task in tasks:
                start_task(task.pk)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '更新任务信息已完成'
        })

    def delete(self, request):
        req_data = request.data
        id = req_data.get('id')
        task_id_list_str = req_data.get('task_id_list')
        if task_id_list_str:
            task_id_list = str_to_list(task_id_list_str)
        else:
            task_id_list = [id]
        try:
            task = Tasks.objects.filter(pk__in=task_id_list)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '任务信息不存在'
            })

        # task 删除
        task.update(is_deleted=1)
        # 删除任务删除等待策略
        wait_device_policy = WaitDevicePolicy.objects.all()
        for item in task:
            if wait_device_policy.filter(arg__contains=f'"task_id": {item.id},').exists():
                wait_device_policy.filter(arg__contains=f'"task_id": {item.id},').delete()
            else:
                pass
        logger.info("delete task %s success" % task_id_list_str)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '删除任务成功'
        })


class TaskDeviceExecViewSet(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, IsAdminUser)
    throttle_classes = (UserRateThrottle,)

    def put(self, request):
        """
        更新状态
        :param request:
            is_all 是否全选，0不选取，1全选
            task_id  任务id
            device_id_list 设备id 列表
            device_action 设备操作，0、取消、 1重试
            {
                "is_all":0,
                "task_id":"8",
                "device_id_list":"2",
                "device_action":0
            }
        :return:
        """
        req_data = request.data
        is_all = req_data.get('is_all')
        task_id = req_data.get('task_id')
        device_id_list = str_to_list(req_data.get('device_id_list'))

        try:
            if is_all == 1:
                task_device_exec_obj = TaskDeviceExec.objects.filter(task__pk=task_id)
            else:
                task_device_exec_obj = TaskDeviceExec.objects.filter(task__pk=task_id, device__pk__in=device_id_list)
        except Exception as e:
            return Response({
                'code': status.HTTP_404_NOT_FOUND,
                'msg': '设备不存在'
            })

        device_action = req_data.get('device_action')
        if device_action == 0:
            # 取消
            task_device_exec_obj.update(exec_status=3)

            # 批量取消时,全部设备都为取消，将任务状态更新为取消
            total_device = TaskDeviceExec.objects.filter(task__pk=task_id)
            canceled_device = total_device.filter(exec_status=3)
            if len(total_device) == len(canceled_device):
                Tasks.objects.filter(pk=task_id).update(status=3)

        else:
            # 重试
            for device_id in device_id_list:
                task_device_exec_obj.filter(device__pk=device_id).update(exec_status=2)
                start_task(task_id, device_id)
            total_device = TaskDeviceExec.objects.filter(task__pk=task_id)
            success_device = total_device.filter(exec_status=4)
            if len(total_device) == len(success_device):
                Tasks.objects.filter(pk=task_id).update(status=4)
        return Response({
            'code': status.HTTP_200_OK,
            'msg': '设备操作已完成'
        })


@api_view(["POST"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def schedule_task(request):
    """
    周期任务操作，关闭周期任务和启用  (只支持周期任务)
    :param request:
        {
            "ids_list":"21",
            "action":"disable"
        }
    :return:
    """
    try:
        ids_list_str = request.data.get('ids_list')
        action = request.data.get('action')
        if not ids_list_str:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': 'id不存在'
            })
        if "," in ids_list_str:
            ids_list = ids_list_str.split(",")
        else:
            ids_list = [ids_list_str]
        task_list = Tasks.objects.filter(pk__in=ids_list)
        schedule_task_action(task_list, action)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '任务操作完成'
        })
    except Exception as e:
        logger.info(e)
        return Response({
            'code': status.HTTP_404_NOT_FOUND,
            'msg': 'id %s 不存在' % ids_list
        })


@api_view(["GET"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def task_detail_device(request):
    """
    任务详情设备列表
    :param request:
        {
            "task_id":"21"
        }
    :return:
    """
    try:
        task_id = request.query_params.get('task_id')
        task_id = int(task_id)
        task = Tasks.objects.filter(id=task_id)
        if not task.exists():
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': 'id 不存在'
            })
        task_class_range = task.first().task_class
        task_type_range = task.first().range
        task_range = task.first().task_range
        patch_ids_range = task.first().file_ids_range
        patch_ids_list = patch_ids_range.split(',')
        conn = get_redis_connection('redis')

        if task_class_range == 1:
            device_list = []
            if task_type_range == 0:
                # 部门筛选
                org_id = task_range.split(',')
                org_device_list = Devices.objects.filter(org_info__in=org_id).exclude(
                    is_domain=0).distinct().values_list('id', flat=True)
                for device in org_device_list:
                    device_list.append(device)
            elif task_type_range == 1:
                # 终端筛选
                check_id = task_range
                check_id_list = str_to_list(check_id)
                for check_id in check_id_list:
                    device = Devices.objects.get(pk=check_id)
                    device_list.append(device.id)
            elif task_type_range == 2:
                # 分组筛选
                groups_id = task_range
                groups_id_list = str_to_list(groups_id)
                group_chat_items = GroupsDetail.objects.all()
                if group_chat_items.filter(groups_id__in=groups_id_list).exists():
                    group_chat_filter_items = group_chat_items.filter(groups_id__in=groups_id_list)
                    items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                    items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
                    items_3_range = group_chat_filter_items.filter(range=3).values_list('account_id', flat=True)
                    # 处理部门数据
                    if Devices.objects.filter(org_info_id__in=items_0_range).exists():
                        device_0_list = Devices.objects.filter(org_info_id__in=items_0_range).values_list('id',
                                                                                                          flat=True)
                        device_0_list = list(device_0_list)
                    else:
                        device_0_list = []
                    # 处理设备信息
                    device_1_list = list(items_1_range)
                    # 处理账号信息
                    if Devices.objects.filter(bind_account__id__in=items_3_range).exists():
                        device_3_list = Devices.objects.filter(bind_account__id__in=items_3_range).values_list('id',
                                                                                                               flat=True)
                        device_3_list = list(device_3_list)
                    else:
                        device_3_list = []
                    device_list = device_0_list + device_1_list + device_3_list
                    device_list = Devices.objects.filter(id__in=list(set(device_list)))

            elif task_type_range == 3:
                # 按人员筛选
                user_id = task_range
                account_list = user_id.split(',')
                if Devices.objects.filter(bind_account__id__in=account_list).exists():
                    device_list = Devices.objects.filter(bind_account__id__in=account_list).values_list('id', flat=True)
                    device_list = [device_list]
                else:
                    device_list = []
            elif task_type_range == 4:
                # 根据ip段范围获取设备
                task_range = task_range
                ip_list = task_range.split(',')
                mid_ip_list = []
                for i in ip_list:
                    start_ip, end_ip = i.split('-')
                    # 解析起始和结束IP地址为IPv4Address对象
                    start_ip = ipaddress.IPv4Address(start_ip)
                    end_ip = ipaddress.IPv4Address(end_ip)
                    range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
                    mid_ip_list = range_ip_list + mid_ip_list
                if Devices.objects.filter(device_ip__in=mid_ip_list).exists():
                    device_list = Devices.objects.filter(device_ip__in=mid_ip_list).values_list('id', flat=True)
                    device_list = list(device_list)
                else:
                    device_list = []

            return_list = []
            devices_items = Devices.objects.filter(id__in=device_list)
            for item in devices_items:
                is_active = conn.hget('actives', item.device_hostname)
                if is_active == b"1":
                    is_active = 1
                else:
                    is_active = 0
                if item.is_domain == 0:
                    is_active = 2
                organ_name = OrgnizationEntity.objects.get(id=item.org_info_id).name
                if PatchDeviceLog.objects.filter(device_id=item.id, patch_id__in=patch_ids_list,
                                                 install_status=0).exists():
                    exec_status = 5
                else:
                    exec_status = 4

                account = AccountEntity.objects.filter(id=item.accounts_id)
                cn_name = account.first().cn_name if account.exists() else None

                tmp_data = {
                    "device_id": item.id,
                    "device_cnname": item.device_cnname,
                    "device_ip": item.device_ip,
                    "org_name": organ_name,
                    "cn_name": cn_name,
                    "jid": None,
                    "uid": item.accounts_id,
                    "is_domain": item.is_domain,
                    "exec_status": exec_status,
                    "is_active": is_active,
                }
                return_list.append(tmp_data)

        else:
            task_device_exec_list = TaskDeviceExec.objects.filter(task__id=task_id)
            return_list = []

            task_device_id = None
            for task_device_exec in task_device_exec_list:
                if task_device_id == task_device_exec.device_id:
                    continue
                else:
                    task_device_id = task_device_exec.device_id

                is_active = conn.hget('actives', task_device_exec.device.device_hostname)
                if is_active == b"1":
                    is_active = 1
                else:
                    is_active = 0
                if task_device_exec.device.is_domain == 0:
                    is_active = 2

                account = AccountEntity.objects.filter(id=task_device_exec.device.accounts_id)
                cn_name = account.first().cn_name if account.exists() else None

                tmp_data = {
                    "device_id": task_device_exec.device_id,
                    "device_cnname": task_device_exec.device.device_cnname,
                    "device_ip": task_device_exec.device.device_ip,
                    "org_name": task_device_exec.device.org_info.name,
                    "uid": task_device_exec.device.accounts_id,
                    "jid": task_device_exec.jid,
                    "cn_name": cn_name,
                    "is_domain": task_device_exec.device.is_domain,
                    "exec_status": task_device_exec.exec_status,
                    "is_active": is_active,
                }
                return_list.append(tmp_data)

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '获取任务信息完成',
            'data': return_list
        })
    except Exception as e:
        logger.info(e)
        return Response({
            'code': status.HTTP_404_NOT_FOUND,
            'msg': 'id %s 不存在' % task_id
        })


class TasksInfoViewSet(ModelViewSet):
    queryset = Tasks.objects.all()
    serializer_class = TasksSerializer
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    ordering = ('id',)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def tasks_result_info(self, request, *args, **kwargs):
        """
        :param request:  query_params 中 task_id 和 device_id 为必输项 task_class == 1  补丁
        :param args:
        :param kwargs:
        :return:
        """
        task_info_id = self.request.query_params.get('task_id')
        task_info_id = int(task_info_id) if task_info_id else None
        task_info_device_id = self.request.query_params.get('device_id')
        task_info_device_id = int(task_info_device_id) if task_info_device_id else None
        task_class = self.queryset.get(id=task_info_id).task_class
        if (not isinstance(task_info_id, int)) or (not isinstance(task_info_device_id, int)):
            return Response('请求参数错误', status=status.HTTP_400_BAD_REQUEST)
        try:
            if task_class != 1:
                queryset = FileManager.objects.filter(task_id=task_info_id)
                task_type_info = task_class
                df = pd.DataFrame(data=queryset.values('id', 'name', 'task_id'))
                df['file_name'] = df['name']
                df['file_type'] = df['task_id'].apply(lambda x: task_type_info)
                if task_type_info != 2:
                    df['exec_status'] = df['id'].apply(lambda x: TaskDeviceExec.objects.get(
                        task_id=task_info_id,
                        device_id=task_info_device_id,
                        file_obj=str(x) if x else x
                    ).exec_status)
                else:
                    df['exec_status'] = df['id'].apply(lambda x: TaskDeviceExec.objects.get(
                        task_id=task_info_id,
                        device_id=task_info_device_id,
                    ).exec_status)

                res_df = df[['file_name', 'file_type', 'exec_status']]
            else:
                task_obj = Tasks.objects.get(id=task_info_id)
                patch_ids_range = task_obj.file_ids_range
                patch_ids_list = patch_ids_range.split(',')
                queryset = PatchDeviceLog.objects.filter(device_id=task_info_device_id).filter(
                    patch_id__in=patch_ids_list)
                df = pd.DataFrame(data=queryset.values('id', 'install_status', 'patch_id'))
                df['patch_no'] = df['patch_id'].apply(lambda x: PatchInfo.objects.get(id=x).patch_no)
                df['patch_name'] = df['patch_id'].apply(lambda x: PatchInfo.objects.get(id=x).patch_name)
                df['patch_ver'] = df['patch_id'].apply(lambda x: PatchInfo.objects.get(id=x).patch_ver)
                df['patch_source_name'] = df['patch_id'].apply(lambda x: PatchInfo.objects.get(id=x).patch_source_name)
                df['patch_type_name'] = df['patch_id'].apply(lambda x: PatchInfo.objects.get(id=x).patch_type_name)
                res_df = df[['id', 'patch_no', 'patch_name', 'patch_ver', 'patch_source_name', 'patch_type_name',
                             'install_status']]

            return Response(res_df.to_dict('records'), status=status.HTTP_200_OK)
        except Exception as exp:
            return Response(f"处理数据错误,{exp}", status=status.HTTP_417_EXPECTATION_FAILED)
