# -*- coding: utf-8 -*-
import logging

from django.utils import timezone
from django.conf import settings as dj_settings
from django.core.cache import cache

from rest_framework import serializers
from rest_framework.exceptions import APIException
from rest_framework import status
from rest_framework_jwt.serializers import jwt_payload_handler
from rest_framework_jwt.serializers import jwt_encode_handler

from users.models import User, Token, Permission
from users.utils import get_login_failed_attempts, normalize_cache_key, LOGIN_ATTEMPT_PREFIX, incr_login_failed_attempts
from utils.utils import get_user_ip

logger = logging.getLogger(__name__)


class AuthenticationFailed(APIException):
    # http status code : 200
    status_code = status.HTTP_200_OK
    default_detail = 'Incorrect authentication credentials.'

    def __init__(self, detail=None):
        res = {
            'code': 4001,
            'msg': detail or self.default_detail
        }
        self.detail = res


def gen_token(user_obj):
    # 签发token
    payload = jwt_payload_handler(user_obj)
    token = jwt_encode_handler(payload)
    return token


class AuthTokenSerializer(serializers.Serializer):
    username = serializers.CharField()
    password = serializers.CharField()

    # device_ip = serializers.CharField(required=False)

    def __init__(self, *a, **kw):
        super(AuthTokenSerializer, self).__init__(*a, **kw)

    def validate(self, attrs):
        # check_res = check_license()
        # if check_res['error']:
        #     raise serializers.ValidationError(check_res['error_msg'])
        login_id = attrs.get('username')
        password = attrs.get('password')

        if login_id and password:
            try:
                user = User.objects.get(username=login_id)
            except Exception as e:
                logger.error("login error %s"%e)
                raise AuthenticationFailed("登录失败")
            ip = get_user_ip(self.context['request'])
            failed_attempt = get_login_failed_attempts(username=login_id)
            if failed_attempt >= dj_settings.LOGIN_ATTEMPT_LIMIT:
                # add_audit_log(login_id, ip, 'system_log', '登录', 'fail')

                if user.is_staff:
                    lock_time = int(dj_settings.LOGIN_ATTEMPT_TIMEOUT / 60)
                    raise AuthenticationFailed('管理员禁用登录 %s 分钟' % lock_time)
                if user.is_active == 0 and user.is_staff == 0:
                    user.freeze_user()
                    cache_key = normalize_cache_key(user.username, prefix=LOGIN_ATTEMPT_PREFIX)
                    if cache_key:
                        cache.set(cache_key, 0, dj_settings.LOGIN_ATTEMPT_TIMEOUT)
                    raise AuthenticationFailed('该帐户已禁用，请与管理员联系')
            else:
                if user.is_active == 1:
                    # add_audit_log(login_id, ip, 'system_log', '登录', 'fail')

                    raise AuthenticationFailed('该帐户已禁用，请与管理员联系')
                if not user.check_password(password):
                    # add_audit_log(login_id, ip, 'system_log', '登录', 'fail')

                    failed_attempt = incr_login_failed_attempts(username=login_id)
                    if failed_attempt >= dj_settings.LOGIN_ATTEMPT_LIMIT:
                        chance = 0
                    else:
                        chance = dj_settings.LOGIN_ATTEMPT_LIMIT - failed_attempt
                    raise AuthenticationFailed('用户名或密码错误，剩余次数 {}'.format(chance))
                # add_audit_log(login_id, ip, 'system_log', '登录', 'success')
                key = gen_token(user)
                token_obj = Token.objects.filter(username=login_id)
                if token_obj:
                    # delete token 配合authentication可以实现单点登录
                    # token_obj.key = key
                    # token_obj.last_accessed = timezone.now()
                    token_obj.update(key=key)
                    user.last_login_ip = ip
                    user.last_accessed = timezone.now()
                    user.save()
                    return key, user
                Token.objects.create(key=key, username=login_id)
                user.last_login_ip = ip
                user.last_accessed = timezone.now()
                user.save()
                return key, user
        raise AuthenticationFailed('缺少用户名或密码')


class UserInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = '__all__'


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = '__all__'


class PermissionSerializer(serializers.ModelSerializer):
    """
    权限序列化
    """

    class Meta:
        model = Permission
        fields = '__all__'


class UserListSerializer(serializers.ModelSerializer):
    """
    用户列表序列化
    """
    role_name = serializers.SerializerMethodField()

    class Meta:
        model = User
        fields = ['id', 'nickname', 'username', 'created_at', 'role_name', 'phone', 'email', 'remark', 'is_netdom',
                  'last_login_ip', 'is_active', 'last_accessed', 'perms', 'org_perms', 'is_superuser']

    def get_role_name(self, obj):
        if obj.is_superuser == 1:
            role_name = '超级管理员'
        else:
            role_name = '管理员'
        return role_name


class UserModifySerializer(serializers.ModelSerializer):
    """
    用户编辑序列化
    """
    role_name = serializers.SerializerMethodField()

    class Meta:
        model = User
        fields = ['id', 'nickname', 'username', 'created_at', 'role_name', 'phone', 'email', 'remark', 'is_netdom',
                  'last_login_ip', 'is_active', 'last_accessed', 'perms', 'org_perms', 'is_superuser']

    def get_role_name(self, obj):
        if obj.is_superuser == 1:
            role_name = '超级管理员'
        else:
            role_name = '管理员'
        return role_name


class UserCreateSerializer(serializers.ModelSerializer):
    """
    创建用户序列化
    """
    username = serializers.CharField(required=True)
    role_name = serializers.SerializerMethodField()

    class Meta:
        model = User
        fields = ['id', 'nickname', 'username', 'password', 'is_staff', 'created_at', 'role_name', 'phone', 'email',
                  'remark', 'is_netdom', 'last_login_ip', 'is_active', 'last_accessed', 'perms', 'org_perms',
                  'is_superuser']

    def validate_username(self, username):
        if User.objects.filter(username=username):
            raise serializers.ValidationError(username + ' 账号不可重复')
        if 5 <= len(username) <= 32:
            pass
        else:
            raise serializers.ValidationError(username + ' 账号需为 5~32位')
        return username

    def get_role_name(self, obj):
        if obj.is_superuser == 1:
            role_name = '超级管理员'
        else:
            role_name = '管理员'
        return role_name
