# -*- coding: utf-8 -*-

import re
import logging
from hashlib import sha1

from django.core.cache import cache
from django.utils.http import urlquote
from django.conf import settings

from users.models import User

logger = logging.getLogger(__name__)

LOGIN_ATTEMPT_PREFIX = 'UserLoginAttempt_'

def normalize_cache_key(value, prefix=None, token=None, max_length=200):
    key = value if prefix is None else prefix + value
    key = key if token is None else key + '_' + token
    return urlquote(key)[:max_length]


def get_login_failed_attempts(username=None, ip=None):

    if username is None and ip is None:
        return 0

    username_attempts = ip_attempts = 0

    if username:
        cache_key = normalize_cache_key(username, prefix=LOGIN_ATTEMPT_PREFIX)
        username_attempts = cache.get(cache_key, 0)

    if ip:
        cache_key = normalize_cache_key(ip, prefix=LOGIN_ATTEMPT_PREFIX)
        ip_attempts = cache.get(cache_key, 0)

    return max(username_attempts, ip_attempts)

def admin_get_login_failed_attempts(username=None):
    if username is None:
        return 0
    username_attempts = 0
    if username:
        cache_key = normalize_cache_key(username, prefix=LOGIN_ATTEMPT_PREFIX)
        username_attempts = cache.get(cache_key, 0)
    return username_attempts

def admin_incr_login_failed_attempts(username=None):
    timeout = settings.LOGIN_ATTEMPT_TIMEOUT
    username_attempts = 1
    if username:
        cache_key = normalize_cache_key(username, prefix=LOGIN_ATTEMPT_PREFIX)
        num = admin_get_login_failed_attempts(username)
        if num >= settings.LOGIN_ATTEMPT_LIMIT:
            cache.set(cache_key, username_attempts, timeout)
            return num
        attempts = cache.get(cache_key)
        if not attempts:
            cache.set(cache_key, username_attempts, timeout)
            return username_attempts
        username_attempts = attempts + 1
        cache.set(cache_key, username_attempts, timeout)
    return username_attempts


def check_admin_user(username):
    user = User.objects.filter(username=username).first()
    if not user.is_staff:
        return False
    admin_failed_attempt = admin_incr_login_failed_attempts(username)
    return admin_failed_attempt

def incr_login_failed_attempts(username=None, ip=None):
    """Increase login failed attempts by 1 for both username and ip.

    Arguments:
    - `username`:
    - `ip`:

    Returns new value of failed attempts.
    """

    admin_failed_attempt = check_admin_user(username)
    if admin_failed_attempt:
        return admin_failed_attempt

    timeout = settings.LOGIN_ATTEMPT_TIMEOUT
    username_attempts = 1
    ip_attempts = 1
    if username:
        cache_key = normalize_cache_key(username, prefix=LOGIN_ATTEMPT_PREFIX)
        try:
            username_attempts = cache.incr(cache_key)
        except ValueError:
            cache.set(cache_key, 1, timeout)

    if ip:
        cache_key = normalize_cache_key(ip, prefix=LOGIN_ATTEMPT_PREFIX)
        try:
            ip_attempts = cache.incr(cache_key)
        except ValueError:
            cache.set(cache_key, 1, timeout)

    return max(username_attempts, ip_attempts)


email_re = re.compile(
    r"(^[-!#$%&*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&*+/=?^_`{}|~0-9A-Z]+)*"  # dot-atom
    # quoted-string, see also http://tools.ietf.org/html/rfc2822#section-3.2.5
    r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*"'
    r')@((?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)$)'  # domain
    r'|\[(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}\]$', re.IGNORECASE)  # literal form, ipv4 address (SMTP 4.1.3)

def is_valid_email(email):
    """A heavy email format validation.
    """
    return True if email_re.match(email) is not None else False

def sha(i):
    sha1_obj = sha1()
    sha1_obj.update(str(i).encode('utf-8'))
    return sha1_obj.hexdigest()  # 明文密码经过加密处理