# -*- coding: utf-8 -*-
import os
import time
import hashlib
import requests
import logging
import json
import base64
from django.conf import settings
from rest_framework.decorators import action
from rest_framework.views import APIView
from rest_framework.parsers import JSONParser  # 解析json数据包
from rest_framework.parsers import FormParser  # 解析urlencoded数据包
from rest_framework.parsers import MultiPartParser  # 解析from-data数据包
from rest_framework.renderers import JSONRenderer  # 渲染模块,仅渲染json格式
from rest_framework.permissions import IsAuthenticated
from rest_framework.views import Response
from rest_framework import status, viewsets, mixins
from rest_framework.viewsets import ModelViewSet
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.decorators import api_view, permission_classes, authentication_classes

from accounts.freeipa import add_user, user_del
from users.filters import UserFilter
from users.models import Permission
from utils.authentication import TokenAuthentication, DomainLicenseAuth
from utils.check_passwd import check_pwd
from utils.pagination import MyPagination
from utils.permission import IsAdminUser
from utils.response import FitJSONRenderer
from utils.throttling import AnonRateThrottle, UserRateThrottle
from utils.utils import page_func, get_user_ip, get_mac_address, read_licenseB, bytes_to_string, get_qrcode

from users.serializers import AuthTokenSerializer, PermissionSerializer, UserCreateSerializer, UserListSerializer, \
    UserModifySerializer
from users import models, serializers
from users.role_permisson import get_permission_list, get_org_permission_list
from users.utils import sha, is_valid_email

# from auditlog.utils import system_log, add_audit_log

logger = logging.getLogger(__name__)


class CheckDomainLicense(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request):
        domain_auth = DomainLicenseAuth()
        if not domain_auth.license_b:
            return Response({
                'code': 4001,
                'msg': '服务器未激活'
            })
        # mac = get_mac_address()
        mac = os.environ.get("DOMAIN_MAC")
        license_info = read_licenseB(mac, domain_auth.license_b)

        if not license_info.get('register_res'):
            return Response({
                'code': 4001,
                'msg': '服务器激活码有误'
            })
        expiration = bytes_to_string(license_info.get('expiration_res', 0))
        if int(expiration) < int(time.strftime('%Y%m%d')):
            return Response({
                'code': 4001,
                'msg': '序列号已过期'
            })
        max_users = bytes_to_string(license_info.get('max_users_res', 0))
        return Response({
            'code': 2000,
            'activation': 'success',
            'msg': '服务器已激活, 用户数:{0}, 有效期：{1}'.format(int(max_users), expiration)
        })


class QRCodeActivationDomain(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (AnonRateThrottle,)

    def get(self, request):
        domain_auth = DomainLicenseAuth()
        serial = request.GET.get('serial')
        # mac = get_mac_address()
        mac = os.environ.get("DOMAIN_MAC")
        if not request.data.get('main_host'):
            main_host = domain_auth.remote_auth_host
        else:
            main_host = request.data.get('main_host').strip('/')
        url = '{}{}?mac={}&serial={}'.format(main_host, domain_auth.remote_auth_api_qrcode, mac, serial)
        image_qrcode = get_qrcode(url)
        # return HttpResponse(image_qrcode, content_type="image/jpg")
        return Response({
            'code': 2000,
            'image': base64.b64encode(image_qrcode).decode()
        })

    def post(self, request):
        domain_auth = DomainLicenseAuth()
        license_b = request.data.get('license_b')
        # mac = get_mac_address()
        mac = os.environ.get("DOMAIN_MAC")
        license_info = read_licenseB(mac, license_b)
        if not license_info.get('register_res'):
            return Response({
                'code': 4001,
                'msg': '服务器激活码有误'
            })
        domain_auth.add_domain_license('', license_b)
        expiration = bytes_to_string(license_info.get('expiration_res', 0))
        max_users = bytes_to_string(license_info.get('max_users_res', 0))
        return Response({
            'code': 2000,
            'activation': 'success',
            'msg': '服务器已激活, 用户数:{0}, 有效期：{1}'.format(int(max_users), expiration)
        })


class ActivationDomain(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    throttle_classes = (UserRateThrottle,)

    def post(self, request):
        domain_auth = DomainLicenseAuth()
        if not request.data.get('main_host'):
            main_host = domain_auth.remote_auth_host.strip('/')
        else:
            main_host = request.data.get('main_host').strip('/')
        url = '{}{}'.format(main_host, domain_auth.remote_auth_api)
        timestamp = int(time.time())
        token = '{}{}{}'.format(domain_auth.remote_auth_key, domain_auth.remote_auth_salt, timestamp).encode()
        headers = {
            'Timestamp': str(timestamp),
            'Token': hashlib.md5(token).hexdigest(),
            'accept': 'application/json; charset=utf-8',
            "Content-Type": "application/json",
            'Connection': 'close',
        }
        # mac = get_mac_address()
        mac = os.environ.get("DOMAIN_MAC")
        data = {
            'mac': mac,
            'serial': request.data.get('serial')
        }

        try:
            response = requests.post(
                url=url,
                headers=headers,
                data=json.dumps(data),
                timeout=15,  # 15 / s
                verify=False,

            )
            response.encoding = 'utf-8'
            res_data = json.loads(response.text)
        except Exception as e:
            logger.info(e)
            return Response({
                'code': 4001,
                'mac': mac,
                'activation': 'failed',
                'msg': '请求超时，请重试，或选择其他方式激活'
            })
        if res_data.get('code') != 2000:
            return Response({
                'code': 2000,
                'activation': 'failed',
                'msg': res_data.get('msg')
            })
        license_b = res_data.get('license_b')
        license_a = res_data.get('license_a')
        domain_auth.add_domain_license(license_a, license_b)
        mac = os.environ.get("DOMAIN_MAC")
        license_info = read_licenseB(mac, domain_auth.license_b)
        expiration = bytes_to_string(license_info.get('expiration_res', 0))
        max_users = bytes_to_string(license_info.get('max_users_res', 0))
        return Response({
            'code': 2000,
            'activation': 'success',
            'msg': '服务器已激活, 用户数:{0}, 有效期：{1}'.format(int(max_users), expiration)
        })


class ObtainAuthToken(APIView):
    """
    Returns auth token if username and password are valid.
    For example:
        curl -d "username=foo@example.com&password=123456" http://127.0.0.1:8000/login/
    """
    throttle_classes = (AnonRateThrottle,)
    permission_classes = ()
    parser_classes = (FormParser, MultiPartParser, JSONParser,)
    renderer_classes = (JSONRenderer,)

    def post(self, request, *args, **kwargs):
        if not request.data.get('username') or not request.data.get('password'):
            return Response({
                'code': 4001,
                'msg': '缺少用户名或密码'
            })
        context = {'request': request}
        serializer = AuthTokenSerializer(data=request.data, context=context)
        if serializer.is_valid():
            key, user = serializer.validated_data
            return Response({
                'code': 2000,
                'token': '{}-{}'.format('nfs', key),
                'is_superuser': user.is_superuser,
                'user_permission': get_permission_list(user)
            })
        return Response(serializer.errors,
                        status=status.HTTP_200_OK)


class User(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    throttle_classes = (UserRateThrottle,)

    def get(self, request, *args, **kwargs):
        role_type = 0
        user = models.User.objects.filter(username=request.user.username).first()
        # token = models.Token.objects.filter(username=request.user.username).first()
        return Response({
            'code': 2000,
            'id': user.id,
            'username': user.username,
            'nickname': user.nickname if user.nickname else user.username,
            'create_time': user.created_at,
            'last_login_i1p': user.last_login_ip if user.last_login_ip else "",
            'is_active': user.is_active,
            'last_accessed': user.last_accessed.strftime("%Y-%m-%d %H:%M:%S") if user.last_accessed else "",
            'is_superuser': user.is_superuser,
            'role': user.is_superuser,
            'user_role': 'NFS_ROOT',  # 放开前端权限
            'permission': get_permission_list(user),
            'org_permission': get_org_permission_list(user)
        })

    def put(self, request, *args, **kwargs):
        filter_dict = {
            'password': request.data.get('password', False),
            'nickname': request.data.get('nickname', False)
        }
        if filter_dict.get('password'):
            if sha(request.data.get('original_password')) != request.user.password:
                return Response({
                    'code': 4001,
                    'msg': '原密码错误'
                })
            if request.data.get('password') != request.data.get('re_password'):
                return Response({
                    'code': 4001,
                    'msg': '两次密码不一致'
                })
            if sha(request.data.get('password')) == request.user.password:
                return Response({
                    'code': 4001,
                    'msg': '新密码与旧密码相同'
                })
            # 密码合规性检查
            code, msg = check_pwd(request.data.get('password'))
            if not code:
                return Response({'code': 4001, 'msg': msg}, status=status.HTTP_400_BAD_REQUEST)
            request.user.update_password(request.data.get('password'))
        if filter_dict.get('nickname'):
            request.user.update_nickname(request.data.get('nickname'))
        return Response({
            'code': 2000,
            'msg': '修改成功'
        })


class CreateAdminUser(APIView):
    authentication_classes = ()
    permission_classes = ()
    throttle_classes = (UserRateThrottle,)

    def post(self, request, *args, **kwargs):
        if not request.data.get('username') or not request.data.get('password'):
            return Response({
                'code': 4001,
                'msg': '缺少用户名或密码'
            })
        try:
            models.User.objects.get(username=request.data.get('username'))
            return Response({
                'code': 4001,
                'msg': '用户已存在'
            })
        except:
            if request.data.get('username') == 'root@admin.com':
                role_type = 1
            elif request.data.get('username') == 'secadm@admin.com':
                role_type = 2
            else:
                role_type = 3
            user = models.User.objects.create(username=request.data.get('username'), is_staff=1, is_active=0,
                                              role=role_type)
            user.make_password(request.data.get('password'))
            return Response({
                'code': 2000,
                'msg': '用户创建成功'
            })


class Logout(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    throttle_classes = (UserRateThrottle,)

    def post(self, request, *args, **kwargs):
        token = models.Token.objects.filter(username=request.user.username).first()
        token.delete()
        return Response({
            'code': 2000,
            'msg': '已退出'
        })


class UserAdminSystem(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def put(self, request, *args, **kwargs):
        role_type = request.user.get_role_display()
        user_list = request.data.get('user')
        active = request.data.get('active', 1)
        for id in user_list:
            query = models.User.objects.filter(id=id)
            user = query.first()
            if user.get_role_display() == 'NFS_SECADM':
                return Response({
                    'code': 4001,
                    'msg': '不可解锁/锁定自身'
                })
            query.update(is_active=active)
            ip = get_user_ip(request)
            # if active == 0:
            #     log_msg = system_log.get('admin_un_active_user').format(user.username)
            # else:
            #     log_msg = system_log.get('admin_active_user').format(user.username)
            # add_audit_log(request.user.username, ip, 'system_log', log_msg, 'success')

        return Response({'code': 2000, 'msg': '修改成功'})

    def get(self, request, id=None):
        # check user
        if id:
            user = models.User.objects.filter(id=id).first()
            # token = models.Token.objects.filter(username=user.username).first()

            if user.is_superuser == 1:
                role_name = '超级管理员'
            else:
                role_name = '管理员'

            if user.role_name:
                role_name = user.role_name
            return Response({
                'code': 2000,
                'id': user.id,
                'username': user.username,
                'nickname': user.nickname if user.nickname else user.username,
                'create_time': user.created_at,
                'role': user.is_superuser,
                'role_name': role_name,
                'phone': user.phone,
                'email': user.email,
                'remark': user.remark,
                'last_login_ip': user.last_login_ip if user.last_login_ip else "",
                'is_active': user.is_active,
                'last_accessed': user.last_accessed.strftime("%Y-%m-%d %H:%M:%S") if user.last_accessed else "",
            })
        # search user
        querys = []
        query = models.User.objects.all()
        if request.GET.get('username'):
            query = query.filter(username__icontains=request.GET.get('username')).all()
        if request.GET.get('nickname'):
            query = query.filter(nickname__icontains=request.GET.get('nickname')).all()
        if request.GET.get('is_active'):
            query = query.filter(is_active=request.GET.get('is_active')).all()
        if request.GET.get('role'):
            role = int(request.GET.get('role'))
            query = query.filter(role=role).all()
        if request.GET.get('last_login_ip'):
            token_query = models.Token.objects.filter(last_login_ip=request.GET.get('last_login_ip')).all()
            if token_query:
                for i in query:
                    for t in token_query:
                        if i.username == t.username:
                            querys.append(i)
        else:
            querys = query
        page_info, query_list = page_func(request, querys)
        # if querys:
        #     page_info, query_list = page_func(request, querys)
        # else:
        #     page_info, query_list = page_func(request, query)
        res_list = []
        if len(query_list) == 0:
            return Response({
                'success': True,
                # 'has_next_page':page_info.get('has_next_page'),
                'pageCount': page_info.get('page_count'),
                'current': page_info.get('current_page'),
                'pageSize': page_info.get('per_page'),
                'total': len(query),
                'data': res_list
            })
        for user in query_list:
            res_dict = {}
            # last_accessed = ""
            # last_login_ip = ""
            # token = models.Token.objects.filter(username=user.username).first()
            # if token:
            #     last_accessed = token.last_accessed
            #     last_login_ip = token.last_login_ip
            res_dict["nickname"] = user.nickname
            res_dict["id"] = user.id
            res_dict['role'] = user.is_superuser
            if user.is_superuser == 1:
                res_dict['role_name'] = '超级管理员'
            else:
                res_dict['role_name'] = '管理员'

            if user.role_name:
                res_dict['role_name'] = user.role_name
            res_dict["username"] = user.username
            res_dict["create_time"] = user.created_at
            res_dict["last_login_ip"] = user.last_login_ip
            res_dict["last_accessed"] = user.last_accessed.strftime("%Y-%m-%d %H:%M:%S") if user.last_accessed else ""
            res_dict["is_active"] = user.is_active
            res_list.append(res_dict)

        return Response({
            'success': True,
            # 'has_next_page':page_info.get('has_next_page'),
            'pageCount': page_info.get('page_count'),
            'current': page_info.get('current_page'),
            'pageSize': page_info.get('per_page'),
            'total': len(query),
            'data': res_list
        })

    def post(self, request, *args, **kwargs):
        """
        新增用户接口
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        # role_type = request.user.get_role_display()
        if not request.data.get('username') or not request.data.get('password'):
            return Response({'code': 4001, 'msg': '缺少用户名或密码'}, status=status.HTTP_400_BAD_REQUEST)
        # if not is_valid_email(request.data.get('username')):
        #     return Response({'code': 4001, 'msg': '登录名需要使用邮箱格式'}, status=status.HTTP_400_BAD_REQUEST)
        if request.data.get('password') != request.data.get('re_password'):
            return Response({'code': 4001, 'msg': '两次密码不一致'}, status=status.HTTP_400_BAD_REQUEST)
        if models.User.objects.filter(username=request.data.get('username')).exists():
            return Response({'code': 4001, 'msg': '用户已存在'}, status=status.HTTP_400_BAD_REQUEST)
        role_name = request.data.get('role_name')
        if role_name is not None:
            if '超级管理员' in role_name:
                return Response({'code': 4001, 'msg': '不能创建超级管理员'}, status=status.HTTP_400_BAD_REQUEST)

        password = request.data.get('password')
        code, msg = check_pwd(password)
        if not code:
            return Response({'code': 4001, 'msg': msg}, status=status.HTTP_400_BAD_REQUEST)

        sha1_obj = models.sha1()
        sha1_obj.update(str(password).encode('utf-8'))
        password = sha1_obj.hexdigest()

        role = 0
        is_staff = 1

        datadict = {
            'username': request.data.get('username'),
            'nickname': request.data.get('nickname'),
            'password': password,
            'phone': request.data.get('phone'),
            # 'role': role,
            'role_name': role_name,
            'email': request.data.get('email'),
            'remark': request.data.get('remark'),
            'is_active': int(request.data.get('is_active')),
            'is_staff': is_staff
        }

        models.User.objects.create(**datadict)

        return Response({'code': 2000, 'msg': '用户创建成功'}, status=status.HTTP_200_OK)


class RestUserPassword(APIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)

    def post(self, request, *args, **kwargs):
        user_list = request.data.get('user')
        for id in user_list:
            query = models.User.objects.filter(id=id)
            query.update(password=sha(settings.DEFAULT_USER_PASSWORD))

            user = query.first()
            ip = get_user_ip(request)
            # log_msg = system_log.get('admin_rest_user_pwd').format(user.username)
            # add_audit_log(request.user.username, ip, 'system_log', log_msg, 'success')

        return Response({'code': 2000, 'msg': '修改成功'})


class UserInfoViewSet(viewsets.GenericViewSet,
                      mixins.UpdateModelMixin):
    queryset = models.User.objects.all()
    serializer_class = serializers.UserInfoSerializer
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('username',)

    def filter_queryset(self, queryset):
        """
        :param queryset: 表数据结果集合
        :return: 根据过滤条件筛选返回结果集合
        """
        username = self.request.query_params.get('username', default=None)
        nickname = self.request.query_params.get('nickname', default=None)
        is_active = self.request.query_params.get('is_active', default=None)
        if username is not None:
            queryset = queryset.filter(username__contains=username)
        if nickname is not None:
            queryset = queryset.filter(nickname__contains=nickname)
        if is_active is not None:
            queryset = queryset.filter(is_active=is_active)

        return queryset

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def change_password(self, request, *args, **kwargs):
        """
        web列表，修改密码
        :param request: body  {user_id,original_password,password,re_password}
        :param args:
        :param kwargs:
        :return:
        """
        user_id = request.data.get('user_id')
        org_password = request.data.get('original_password')
        password = request.data.get('password')
        re_password = request.data.get('re_password')

        request_user_name = request.user
        request_info = self.queryset.get(username=request_user_name)
        if password != re_password:
            return Response({'code': 4001, 'msg': '两次密码不一致'}, status=status.HTTP_400_BAD_REQUEST)

        # 密码加密存储
        sha1_obj = models.sha1()
        sha1_obj.update(str(password).encode('utf-8'))
        password = sha1_obj.hexdigest()

        if request_info.is_superuser == 1:
            # 超级管理员修改密码，不需要原密码
            self.queryset.filter(id=user_id).update(password=password)
        else:
            # 原密码加密和数据库校验检查
            sha1_obj.update(str(org_password).encode('utf-8'))
            db_org_password = self.queryset.get(id=user_id).password
            if db_org_password != sha1_obj.hexdigest():
                return Response({'code': 4001, 'msg': '原密码错误'}, status=status.HTTP_400_BAD_REQUEST)
            sha1_obj.update(str(password).encode('utf-8'))
            password = sha1_obj.hexdigest()
            self.queryset.filter(id=user_id).update(password=password)

        return Response({'code': 2000, 'msg': '修改成功'}, status=status.HTTP_200_OK)

    @action(methods=['delete'], detail=False, authentication_classes=[TokenAuthentication])
    def user_delete(self, request, *args, **kwargs):
        """
        批量删除web账号
        :param request: query_params {is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        ......: 筛选条件
                        }
        """
        is_all = self.request.query_params.get('is_all', default=None)
        ids_list = self.request.query_params.get('ids_list', default=None)

        # 检查是否未超级管理员账号操作
        if self.queryset.get(username=request.user).is_superuser != 1:
            return Response({'code': 4001, 'msg': '操作账号必须为超级管理员账号'}, status=status.HTTP_400_BAD_REQUEST)

        # 检查是否全选
        if int(is_all) == 1:
            queryset = self.filter_queryset(self.get_queryset())
            queryset = queryset.exclude(is_superuser=1).delete()
        elif int(is_all) == 0:
            if ids_list is None:
                return Response({'code': 4001, 'msg': 'ide_list 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)
            else:
                ids_list = [int(i) for i in ids_list.split(',')]
            queryset = self.queryset.exclude(is_superuser=1).filter(id__in=ids_list).delete()
        else:
            return Response({'code': 4001, 'msg': 'is_all 传参错误'}, status=status.HTTP_400_BAD_REQUEST)

        return Response({'code': 2000, 'msg': '批量删处成功'}, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def user_activate(self, request, *args, **kwargs):
        """
        批量 启用|禁用 web账号
        :param request: query_params {is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        active： 0: 启用， 1：未启用
                        ......: 筛选条件
                        }
        """
        is_all = self.request.query_params.get('is_all', default=None)
        ids_list = self.request.query_params.get('ids_list', default=None)
        active = self.request.query_params.get('active', default=None)

        # 检查是否未超级管理员账号操作
        if self.queryset.get(username=request.user).is_superuser != 1:
            return Response({'code': 4001, 'msg': '操作账号必须为超级管理员账号'}, status=status.HTTP_400_BAD_REQUEST)

        if active is None:
            return Response({'code': 4001, 'msg': 'active 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        active = int(active)  # 将字符转换为数字

        if active not in [0, 1]:
            return Response({'code': 4001, 'msg': 'active 传参错误'}, status=status.HTTP_400_BAD_REQUEST)

        if int(is_all) == 1:
            queryset = self.filter_queryset(self.get_queryset())
            queryset = queryset.exclude(is_superuser=1).update(is_active=active)
        elif int(is_all) == 0:
            if ids_list is None:
                return Response({'code': 4001, 'msg': 'ide_list 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)
            else:
                ids_list = [int(i) for i in ids_list.split(',')]
            queryset = self.queryset.exclude(is_superuser=1).filter(id__in=ids_list).update(is_active=active)
        else:
            return Response({'code': 4001, 'msg': 'is_all 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)

        if active == 0:
            return Response({'code': 2000, 'msg': '批量启用成功'}, status=status.HTTP_200_OK)
        else:
            return Response({'code': 2000, 'msg': '批量禁用成功'}, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def user_passwd(self, request, *args, **kwargs):
        """
        批量 修改 web账号 密码
        :param request: query_params {is_all: 0|不全选， 1|全选，
                        ids_list: 选中的条目主键  1,2,3,4,5，
                        password：
                        confirm_password:
                        ......: 筛选条件
                        }
        """
        is_all = self.request.query_params.get('is_all', default=None)
        ids_list = self.request.query_params.get('ids_list', default=None)
        password = self.request.query_params.get('password', default=None)
        confirm_password = self.request.query_params.get('re_password', default=None)

        # 检查是否未超级管理员账号操作
        if self.queryset.get(username=request.user).is_superuser != 1:
            return Response({'code': 4001, 'msg': '操作账号必须为超级管理员账号'}, status=status.HTTP_400_BAD_REQUEST)

        # 检查密码是否一致
        if password is None:
            return Response({'code': 4001, 'msg': 'password 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)
        code, msg = check_pwd(password)
        if not code:
            return Response({'code': 4001, 'msg': msg}, status=status.HTTP_400_BAD_REQUEST)
        if password != confirm_password:
            return Response({'code': 4001, 'msg': '两次密码不一致'}, status=status.HTTP_400_BAD_REQUEST)

        # 密码加密存储
        sha1_obj = models.sha1()
        sha1_obj.update(str(password).encode('utf-8'))
        password = sha1_obj.hexdigest()

        if int(is_all) == 1:
            queryset = self.filter_queryset(self.get_queryset())
            queryset = queryset.exclude(is_superuser=1).update(password=password)
        elif int(is_all) == 0:
            if ids_list is None:
                return Response({'code': 4001, 'msg': 'ids_list 为必输选项'}, status=status.HTTP_400_BAD_REQUEST)
            else:
                ids_list = [int(i) for i in ids_list.split(',')]
            queryset = self.queryset.exclude(is_superuser=1).filter(id__in=ids_list).update(password=password)
        else:
            return Response({'code': 4001, 'msg': 'is_all 传参数错误'}, status=status.HTTP_400_BAD_REQUEST)

        return Response({'code': 2000, 'msg': '批量修改密码成功'}, status=status.HTTP_200_OK)

    @action(methods=['get'], detail=False, authentication_classes=[TokenAuthentication])
    def role_spinner(self, request, *args, **kwargs):
        """
        角色下拉框选项
        :param request:
        :param args:
        :param kwargs:
        :return: {0: '普通用户',1: '超级管理员',2: '密级管理员',3: '审计管理员',}
        """
        data = {
            0: '普通用户',
            1: '超级管理员',
            2: '密级管理员',
            3: '审计管理员',
        }
        return Response({'code': 2000, 'msg': data}, status=status.HTTP_200_OK)

    @action(methods=['put'], detail=False, authentication_classes=[TokenAuthentication])
    def user_update(self, request, *args, **kwargs):
        """
        更新用户基本信息
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        user_id = self.request.query_params.get('id', default=None)
        if not user_id:
            return Response({'code': 4001, 'msg': '主键id为必输项'}, status=status.HTTP_400_BAD_REQUEST)
        user_id = int(user_id)
        if not self.queryset.filter(id=user_id).exists():
            return Response({'code': 4001, 'msg': '主键id的记录不存在'}, status=status.HTTP_400_BAD_REQUEST)
        user_info = self.queryset.get(id=user_id)

        role_name = request.data.get('role_name')
        if role_name is not None:
            if '超级管理员' in role_name:
                return Response({'code': 4001, 'msg': '不能更新为超级管理员'}, status=status.HTTP_400_BAD_REQUEST)
        if user_info.is_superuser == 1:
            is_superuser = 1
        else:
            is_superuser = 0

        models.User.objects.filter(id=user_id).update(**{
            'nickname': request.data.get('nickname'),
            'phone': request.data.get('phone'),
            'is_superuser': is_superuser,
            'role_name': request.data.get('role_name'),
            'email': request.data.get('email'),
            'remark': request.data.get('remark'),
        })
        user = models.User.objects.get(id=user_id)
        if user.is_superuser == 1:
            role_name = '超级管理员'
        else:
            role_name = '管理员'
        if user.role_name:
            role_name = user.role_name
        return Response({
            'code': 2000,
            'id': user.id,
            'username': user.username,
            'nickname': user.nickname if user.nickname else user.username,
            'create_time': user.created_at,
            'role': user.is_superuser,
            'role_name': role_name,
            'phone': user.phone,
            'email': user.email,
            'remark': user.remark,
            'last_login_ip': user.last_login_ip if user.last_login_ip else "",
            'is_active': user.is_active,
            'last_accessed': user.last_accessed.strftime("%Y-%m-%d %H:%M:%S") if user.last_accessed else "",
        }, status=status.HTTP_200_OK)


class PermissionViewSet(ModelViewSet):
    """
    权限：增删改查
    """
    queryset = Permission.objects.all()
    serializer_class = PermissionSerializer
    renderer_classes = (FitJSONRenderer,)
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    pagination_class = None
    search_fields = ['name']
    ordering = ['pk']


class UserViewSet(ModelViewSet):
    queryset = models.User.objects.all()
    serializer_class = serializers.UserSerializer
    filterset_class = UserFilter
    renderer_classes = (FitJSONRenderer,)
    pagination_class = MyPagination
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAdminUser,)
    throttle_classes = (UserRateThrottle,)
    ordering = ('username',)

    def get_serializer_class(self):
        # 根据请求类型动态变更serializer
        if self.action == 'create':
            return UserCreateSerializer
        elif self.action == 'list':
            return UserListSerializer
        return UserModifySerializer

    def create(self, request, *args, **kwargs):
        # 创建用户
        request_data = request.data.copy()
        password = request.data['password']
        re_password = request.data['re_password']
        if 'role_name' in request.data.keys():
            role_name = request.data['role_name']
        else:
            role_name = None

        is_netdom = request.data['is_netdom']

        # 检查管理员角色
        user = request.user
        if role_name == '超级管理员':
            if user.is_superuser == 1:
                return Response('超级管理员仅一位', status=status.HTTP_400_BAD_REQUEST)
            else:
                return Response('超级管理员仅一位', status=status.HTTP_400_BAD_REQUEST)
        else:
            request_data['is_superuser'] = 0

        # 检查密码
        code, msg = check_pwd(password)
        if not code:
            return Response(msg, status=status.HTTP_400_BAD_REQUEST)
        if password != re_password:
            return Response('两次密码不一致', status=status.HTTP_400_BAD_REQUEST)

        request_data.pop('re_password')
        request_data['is_staff'] = 1

        # 密码加密存储
        sha1_obj = models.sha1()
        sha1_obj.update(str(password).encode('utf-8'))
        password = sha1_obj.hexdigest()

        request_data['password'] = password

        # 是否加域
        if is_netdom == 1:
            try:
                add_user(request_data['username'],
                         request_data['username'] if request_data['nickname'] is None else request_data['username'])
            except Exception as exp:
                Response(f'账户允许加域异常，异常信息为： {exp}', status=status.HTTP_400_BAD_REQUEST)
        else:
            try:
                user_del(request_data['username'])
            except Exception as exp:
                Response(f'账户不允许加域异常，异常信息为： {exp}', status=status.HTTP_400_BAD_REQUEST)

        serializer = self.get_serializer(data=request_data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    def update(self, request, *args, **kwargs):
        is_netdom = request.data['is_netdom']
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        # 是否开启加域
        if is_netdom != instance.is_netdom:
            if is_netdom == 1:
                try:
                    add_user(instance.username, instance.username if instance.nickname is None else instance.nickname)
                except Exception as exp:
                    Response(f'账户允许加域异常，异常信息为： {exp}', status=status.HTTP_400_BAD_REQUEST)
            else:
                try:
                    user_del(instance.username)
                except Exception as exp:
                    Response(f'账户不允许加域异常，异常信息为： {exp}', status=status.HTTP_400_BAD_REQUEST)

        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        if instance.is_superuser == 1:
            return Response('超级管理员账号不可删除', status=status.HTTP_204_NO_CONTENT)
        self.perform_destroy(instance)
        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(methods=['put'], detail=False, permission_classes=[IsAuthenticated],  # perms_map={'put':'change_password'}
            url_name='change_password')
    def change_password(self, request, pk=None):
        """
        修改密码
        """
        user = request.user
        old_password = request.data['old_password']
        sha1_obj = models.sha1()
        sha1_obj.update(str(old_password).encode('utf-8'))
        old_password = sha1_obj.hexdigest()

        if old_password == user.password:
            new_password1 = request.data['new_password1']
            new_password2 = request.data['new_password2']
            if new_password1 == new_password2:
                sha1_obj.update(str(new_password1).encode('utf-8'))
                new_password1 = sha1_obj.hexdigest()
                user.password = new_password1
                user.save()
                return Response('密码修改成功!', status=status.HTTP_200_OK)
            else:
                return Response('新密码两次输入不一致!', status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response('旧密码错误!', status=status.HTTP_400_BAD_REQUEST)

    # perms_map={'get':'*'}, 自定义action控权
    @action(methods=['get'], detail=False, url_name='my_info', permission_classes=[IsAuthenticated])
    def my_info(self, request, pk=None):
        """
        初始化用户信息
        """
        user = request.user
        data = {
            'id': user.id,
            'username': user.username,
            'nickname': user.nickname if user.nickname else user.username,
            'create_at': user.created_at.strftime("%Y-%m-%d %H:%M:%S") if user.created_at else "",
            'is_active': user.is_active,
            'last_accessed': user.last_accessed.strftime("%Y-%m-%d %H:%M:%S") if user.last_accessed else "",
            'is_superuser': user.is_superuser,
            'permission': get_permission_list(user),
            'org_permission': get_org_permission_list(user)
        }

        return Response(data, status=status.HTTP_200_OK)


@api_view(["GET"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def set_org_global_switch(request):
    """
    打开关闭仅展示部门直属人员或终端
    :param request:
        range 0 人员 1 device
        action 0 关 1 开
        {
            "range:0,
            "action":0
        }
    :return:
    """
    try:
        action = request.query_params.get('action')
        range_param = request.query_params.get('range')
        if not action:
            return Response({
                'code': status.HTTP_400_BAD_REQUEST,
                'msg': '参数range 或 action 不存在'
            })

        user_id = request.user.id
        user = models.User.objects.get(id=user_id)
        if int(range_param) == 0:
            user.org_global_switch_user = action
        else:
            user.org_global_switch_device = action
        user.save()

        return Response({
            'code': status.HTTP_200_OK,
            'msg': '操作完成'
        })
    except Exception as e:
        logger.info(e)
        return Response({
            'code': status.HTTP_404_NOT_FOUND,
            'msg': 'user_id %s 不存在' % user_id
        })


@api_view(["GET"])
@authentication_classes((TokenAuthentication,))
@permission_classes((IsAuthenticated, IsAdminUser,))
def org_global_switch(request):
    """
    仅展示部门直属人员或终端
    :param request:
    :return:
        {
            "range":0
        }
    """
    user_id = request.user.id
    user = models.User.objects.get(id=user_id)
    range_param = request.query_params.get('range')

    if not range_param:
        return Response({
            'code': status.HTTP_400_BAD_REQUEST,
            'msg': '参数range不存在'
        })

    org_global_switch = 0
    if int(range_param) == 0:
        org_global_switch = user.org_global_switch_user
    else:
        org_global_switch = user.org_global_switch_device

    return Response({
        'code': status.HTTP_200_OK,
        'data': {
            "org_global_switch": org_global_switch
        },
        'msg': '操作完成'
    })
