# Copyright (c) 2021-2022 nfs-license Ltd.
import datetime
import logging
import jwt
import time
import hashlib
import os
import configparser
from gmssl import sm4
from django.utils import timezone

from rest_framework import status
from rest_framework_jwt.authentication import BaseJSONWebTokenAuthentication
from rest_framework.exceptions import APIException
from rest_framework_jwt.authentication import jwt_decode_handler


from users.models import Token, User
from utils.utils import sm4_hash, sm4_decrypt


from django.conf import settings
logger = logging.getLogger(__name__)


class DomainLicenseAuth(object):
    def __init__(self):
        self.LICENSE = settings.LICENSE
        cfg = configparser.ConfigParser()
        cfg.read(self.LICENSE)
        self.cfg = cfg
        if not self.cfg.has_section("DOMAIN_LICENSE"):
            self.cfg.add_section("DOMAIN_LICENSE")
            self.cfg.write(open(self.LICENSE, 'w'))
        if not self.cfg.has_section("KMS"):
            self.cfg.add_section("KMS")
            self.cfg.write(open(self.LICENSE, 'w'))
        if not self.cfg.has_section("LDAP"):
            self.cfg.add_section("LDAP")
            self.cfg.write(open(self.LICENSE, 'w'))

    def get_domain_license(self):
        license = {}
        if self.cfg.has_section("AGENT_LICENSE"):
            license_a = self.cfg.get('DOMAIN_LICENSE', 'license_a')
            license_b = self.cfg.get('DOMAIN_LICENSE', 'license_b')
            label = self.cfg.get('DOMAIN_LICENSE', 'DOMAINLABLE')
            if not license_a or not license_b:
                return license
            license['LICENSE_A'] = license_a
            license['LICENSE_B'] = license_b
            license['LABEL'] = label
            return license
        return license

    def add_domain_license(self, a, b):
        if not self.cfg.has_section("DOMAIN_LICENSE"):
            self.cfg.add_section("DOMAIN_LICENSE")
        self.cfg.set('DOMAIN_LICENSE', 'license_a', a)
        self.cfg.set('DOMAIN_LICENSE', 'license_b', b)
        # self.cfg.set('DOMAIN_LICENSE', 'DOMAINLABLE', lable)
        with open(self.LICENSE, 'w') as fp:
            self.cfg.write(fp)

    @property
    def license_a(self):
        if not self.cfg.has_option('DOMAIN_LICENSE', 'license_a'):
            return False
        return self.cfg.get('DOMAIN_LICENSE', 'license_a')

    @property
    def license_b(self):
        if not self.cfg.has_option('DOMAIN_LICENSE', 'license_b'):
            return False
        return self.cfg.get('DOMAIN_LICENSE', 'license_b')

    @property
    def label(self):
        if not self.cfg.has_option('DOMAIN_LICENSE', 'DOMAINLABLE'):
            return False
        return self.cfg.has_option('DOMAIN_LICENSE', 'DOMAINLABLE')

    @property
    def remote_auth_host(self):
        return self.cfg.get('DOMAIN_LICENSE', 'cloudaccount_os_nfschina')

    @property
    def remote_auth_api(self):
        return self.cfg.get('DOMAIN_LICENSE', 'cloudaccount_os_nfschina_api')

    @property
    def remote_auth_api_qrcode(self):
        return self.cfg.get('DOMAIN_LICENSE', 'cloudaccount_os_nfschina_api_qrcode')

    @property
    def remote_auth_key(self):
        return self.cfg.get('DOMAIN_LICENSE', 'APIKEY')

    @property
    def remote_auth_salt(self):
        return self.cfg.get('DOMAIN_LICENSE', 'SALT')

    def update_license_a(self, a):
        if not self.cfg.has_section("DOMAIN_LICENSE"):
            self.cfg.add_section("DOMAIN_LICENSE")
        self.cfg.set('DOMAIN_LICENSE', 'license_a', a)
        self.cfg.write(open(self.LICENSE, 'w'))

    def update_license_b(self, b):
        if not self.cfg.has_section("DOMAIN_LICENSE"):
            self.cfg.add_section("DOMAIN_LICENSE")
        self.cfg.set('DOMAIN_LICENSE', 'license_b', b)
        self.cfg.write(open(self.LICENSE, 'w'))

    def add_kms_active(self, active):
        if not self.cfg.has_section("KMS"):
            self.cfg.add_section("KMS")
        self.cfg.set('KMS', 'active', str(active))
        with open(self.LICENSE, 'w') as fp:
            self.cfg.write(fp)

    @property
    def kms_active(self):
        if not self.cfg.has_option('KMS', 'active'):
            return False
        return self.cfg.get('KMS', 'active')

    def add_kms_server(self, kms_server):
        if not self.cfg.has_section("KMS"):
            self.cfg.add_section("KMS")
        self.cfg.set('KMS', 'kms_server', kms_server)
        with open(self.LICENSE, 'w') as fp:
            self.cfg.write(fp)

    @property
    def kms_server(self):
        if not self.cfg.has_option('KMS', 'kms_server'):
            return False
        return self.cfg.get('KMS', 'kms_server')


class AuthenticationFailed(APIException):
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = 'Incorrect authentication credentials.'

    def __init__(self, detail=None):
        self.detail = detail or self.default_detail


class DeviceRemoteWipedException(AuthenticationFailed):
    pass

def get_token_time_out(payload):
    timeStamp = payload.get('exp')
    timeArray = time.localtime(timeStamp)
    otherStyleTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
    return otherStyleTime

class TokenAuthentication(BaseJSONWebTokenAuthentication):

    def authenticate(self, request):
        msg = 'Token 无效.'
        auth = request.META.get('HTTP_AUTHORIZATION', None)
        if not auth:
            raise AuthenticationFailed(msg)
        auth = auth.split('-',1)
        if auth[0].lower() != 'nfs':
            raise AuthenticationFailed(msg)
        if len(auth) == 1:
            raise AuthenticationFailed(msg)
        elif len(auth) > 2:
            raise AuthenticationFailed(msg)

        key = auth[1]
        return self.authenticate_v1(request, key)

    def authenticate_v1(self, request, key):
        try:
            token = Token.objects.get(key=key)
        except :
            raise AuthenticationFailed('Token 无效')
        try:
            User.objects.get(username=token.username)
        except :
            raise AuthenticationFailed('用户不存在')

        # 根据token拿到payload  | token => payload
        try:
            payload = jwt_decode_handler(key)
            # otherStyleTime = get_token_time_out(payload)
            # logger.info(otherStyleTime)
        except jwt.ExpiredSignature:
            raise AuthenticationFailed('Token 已过期')
        # 根据payload拿到user  | payload => user
        user = self.authenticate_credentials(payload)

        if not user.is_active:
            token.last_accessed = timezone.now()
            token.save()
            return (user, token)
        else:
            raise AuthenticationFailed('用户已锁定')

    def authenticate_credentials(self, payload):
        """
        Returns an active user that matches the payload's user id and email.
        """
        return User.objects.filter(username = payload.get('username')).first()


class AnonUser(object):
    def __init__(self):
        self.username = 'AnonUser'

    def is_authenticated(self):
        return self.username


class OpenUrlAuthentication(BaseJSONWebTokenAuthentication):

    def authenticate(self, request):
        timestamp = request.META.get('HTTP_TIMESTAMP')
        token = request.META.get('HTTP_TOKEN')
        # print(request.META)
        if not timestamp or not token:
            raise AuthenticationFailed('请求非法')

        ret = hashlib.md5('{}{}{}'.format(settings.APIKEY, settings.SALT, timestamp).encode())
        re_token = ret.hexdigest()
        if token != re_token:
            raise AuthenticationFailed('请求非法')

        local_timestamp = time.time()
        if settings.INTRVAL_TIME < (
                datetime.datetime.fromtimestamp(local_timestamp) - datetime.datetime.fromtimestamp(float(timestamp))).seconds:
            raise AuthenticationFailed('请求非法')

        return (AnonUser(), token)


def apis_authentication(appid: str, timestamp: str, sig: str):
    if not appid or not timestamp or not sig:
        return
    # local_timestamp = time.time()
    # INTRVAL_TIME = 600
    # if INTRVAL_TIME < (
    #         datetime.datetime.fromtimestamp(local_timestamp) - datetime.datetime.fromtimestamp(float(timestamp))
    # ).seconds:
    #     return
    encrypted_data = sm4_decrypt(sig, '11HDESaAhiHHugD2')
    if f'{appid}#{timestamp}' != encrypted_data:
        return
    return True
