"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/8/25 上午10:35
@File: ldap_server.py
@Desc: 通过ldap协议修改freeipa组织单元和人员
"""
import concurrent.futures
import configparser
import logging
import os
import ldap3.utils.conv as ldap_utils

from ldap3 import Server, Connection, ALL, MODIFY_REPLACE, RESTARTABLE, SUBTREE
from nfs_domain_mainserver import settings
from nfs_domain_mainserver.settings import BASE_DIR

CONFIG_PATH = os.path.join(BASE_DIR, 'nfs_domain_mainserver/conf_ini/config.ini')

logger = logging.getLogger(__name__)


class LDAPManager:
    def __init__(self):
        """
        LDAP管理器的初始化函数。
        """
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH, encoding="utf-8")
        self.soap_node_name = config.get('soap_api', 'soap_node_name')
        self.server_address = settings.DOMAIN_SERVER_FQDN
        self.server_port = 389
        self.user = settings.DOMAIN_SERVER_ADMIN
        self.user_passwd = settings.DOMAIN_SERVER_ADMIN_PASSWD
        self.base_dc = settings.DOMAIN_SERVER_RELM.lower().split('.')
        self.base_dc = ','.join([f"dc={i}" for i in self.base_dc])
        self.user_dn = f'uid={self.user},cn=users,cn=accounts,{self.base_dc}'
        self.search_dn = f"ou=groups,{self.base_dc}"
        logging.info(f"用户的完整DN，{self.user_dn}")

        self.server = Server(self.server_address, port=self.server_port, get_info=ALL)

        # 使用RESTARTABLE策略进行连接，确保在断线时可以重连
        self.connection = Connection(self.server, user=self.user_dn, password=self.user_passwd, auto_bind=True,
                                     client_strategy=RESTARTABLE)

    def base_search(self, base_dn, search_filter):
        """
        查询条件是否存在
        :param base_dn: 查询目录
        :param search_filter: 查询条件
        :return: Ture|False
        """
        self.connection.search(base_dn, search_filter, SUBTREE)
        entries = self.connection.entries
        return entries

    def add_ou(self, ou, base_dn):
        """
        创建组织单元。
        :param ou: 要创建的OU的名称
        :param base_dn: 基础的DN，例如: 'dc=example,dc=com'
        :return: 创建是否成功
        """
        ou = ldap_utils.escape_filter_chars(ou)
        base_dn = ldap_utils.escape_filter_chars(base_dn)
        dn = f"ou={ou},{base_dn}"
        # 执行LDAP搜索以检查OU是否存在

        # search_filter = f'(ou={ou})'
        # logger.info(f"??? {ou},???{base_dn}")
        # entries = self.base_search(base_dn, search_filter)

        # 检查是否存在OU
        # if entries:
        #     logger.info(f"The OU 'ou={ou},{base_dn}' exists.")
        #     return True
        # else:
        #     pass

        # logger.info(f"添加组织单元 {dn}")
        attributes = {
            'objectClass': ['organizationalUnit'],
            'ou': ou
        }
        try:
            res = self.connection.add(dn, attributes=attributes)
            # logger.info(f"添加组织单元结果 {res}")
            return True
        except Exception as e:
            logger.info(f"添加组织单元出错: {e}")
            return False

    def modify_ou_name(self, ou_dn, new_name):
        """
        更新组织单元名称
        :param ou_dn: 原来的 'ou=OldOU,dc=example,dc=com'
        :param new_name: 新的名称
        :return: 更新是否成功
        """
        base_dn = self.base_dc
        for i in ou_dn:
            if i == self.soap_node_name:
                i = 'groups'
            base_dn = f"ou={i},{base_dn}"
        modifications = {'ou': [(MODIFY_REPLACE, [new_name])]}
        self.connection.modify(base_dn, modifications)
        logger.info(f"更新部门结果{self.connection.result['result']},,0 success")
        if self.connection.result['result'] == 0:
            return True
        else:
            return False

    def add_hierarchy_ou(self, ou_list):
        """
        逐层创建OU。
        :param ou_list: 一个包含要逐层创建的OU名称的列表，例如: ['ou1', 'ou2', 'ou3']
        :return: 创建是否全部成功
        """
        logger.info(f"{ou_list}")
        base_dn = self.base_dc  # base_dn: 基础的DN
        for ou in ou_list:
            if ou == self.soap_node_name:
                ou = 'groups'
            if not self.add_ou(ou, base_dn):
                return False
            base_dn = f"ou={ou},{base_dn}"
        return True

    def add_person(self, res_data):
        """
        添加人员。
        :param res_data: {}
        :return: 操作是否成功
        """
        # logger.info(f"{res_data}")
        ou_list = res_data.get('ou')
        name = res_data.get('old_name')
        cn_name = res_data.get('cn_name')
        base_dn = self.base_dc
        for i in ou_list:
            if i == self.soap_node_name:
                i = 'groups'
            base_dn = f"ou={i},{base_dn}"
        person_dn = f"cn={name},{base_dn}"
        search_filter = f'(cn={name})'
        entries = self.base_search(base_dn, search_filter)
        if entries:
            logger.info(f"The cn '{person_dn}' exists.")
            return True
        else:
            logger.info(f"需要添加的人员{person_dn}")
            attributes = {
                'objectClass': ['person', 'top'],
                'cn': cn_name,
                'sn': cn_name,
            }
            try:
                return self.connection.add(person_dn, attributes=attributes)
            except Exception as e:
                logger.info(f"添加人员出错: {e}")
                return False

    def query_update_person(self, person, new_ou_dn, cn_name):
        # logger.info(f"{person},{new_ou_dn},{cn_name}")
        search_filter = f'(cn={person})'
        # 执行LDAP搜索
        self.base_search(self.search_dn, search_filter)

        # 获取结果
        if self.connection.entries:
            for entry in self.connection.entries:
                person_dn = entry.entry_dn
                # logger.info(f'DN: {entry.entry_dn}')
                base_dn = self.base_dc
                for i in new_ou_dn:
                    if i == self.soap_node_name:
                        i = 'groups'
                    base_dn = f"ou={i},{base_dn}"
                base_dn = f"cn={person},{base_dn}"
                # logger.info(f"老人员组织单元{person_dn}......./ 新{base_dn}")
                if person_dn != base_dn:
                    attributes = {
                        'objectClass': ['person', 'top'],
                        'cn': cn_name,
                        'sn': cn_name,
                    }

                    search_filter = f'(cn={person})'
                    entries = self.base_search(self.search_dn, search_filter)
                    if entries:
                        logger.info(f"需要变更的人员组织单元{person_dn}......./ {base_dn}")
                        try:
                            res = self.connection.delete(person_dn)
                            logger.info(f"删除节点结果{res}")
                            res = self.connection.add(base_dn, attributes=attributes)

                            logger.info(f"需要变更的人员组织单元的结果{res}")
                            return res
                        except Exception as e:
                            logger.info(f"修改人员组织单元出错: {e}")
                            return False
                    else:
                        logger.info(f"The cn '{person_dn}' not exists.")
                        res = self.connection.add(base_dn, attributes=attributes)
                        return res


        else:
            return None

    def modify_person(self, person, old_ou_dn, new_ou_dn, cn_name):
        """
        修改人员的组织单元。
        :param person: 人员
        :param old_ou_dn: 旧的组织单元
        :param new_ou_dn: 新组织单元
        :param cn_name: 新组织单元
        :return: 操作是否成功
        """

        old_base_dn = self.base_dc
        for i in old_ou_dn:
            if i == self.soap_node_name:
                i = 'groups'
            old_base_dn = f"ou={i},{old_base_dn}"
        person_dn = f"cn={person},{old_base_dn}"
        base_dn = self.base_dc
        for i in new_ou_dn:
            if i == self.soap_node_name:
                i = 'groups'
            base_dn = f"ou={i},{base_dn}"
        base_dn = f"cn={person},{base_dn}"

        attributes = {
            'objectClass': ['person', 'top'],
            'cn': cn_name,
            'sn': cn_name,
        }

        search_filter = f'(cn={person})'
        entries = self.base_search(old_base_dn, search_filter)
        if entries:
            logger.info(f"需要变更的人员组织单元{person_dn}......./ {base_dn}")
            try:
                res = self.connection.delete(person_dn)
                logger.info(f"删除节点结果{res}")
                res = self.connection.add(base_dn, attributes=attributes)

                logger.info(f"需要变更的人员组织单元的结果{res}")
                return res
            except Exception as e:
                logger.info(f"修改人员组织单元出错: {e}")
                return False
        else:
            logger.info(f"The cn '{person_dn}' not exists.")
            res = self.connection.add(base_dn, attributes=attributes)
            return res

    def delete_person(self, res_data):
        """
        删除人员。
        :param res_data: {}
        :return: 操作是否成功
        """
        # logger.info(res_data)
        ou_list = res_data.get('ou')
        name = res_data.get('old_name')
        base_dn = self.base_dc
        for i in ou_list:
            if i == self.soap_node_name:
                i = 'groups'
            base_dn = f"ou={i},{base_dn}"
        person_dn = f"cn={name},{base_dn}"
        search_filter = f'(cn={name})'
        entries = self.base_search(base_dn, search_filter)
        if entries:
            logger.info(f"需要删除的人员{person_dn}")
            try:
                return self.connection.delete(person_dn)
            except Exception as e:
                logger.info(f"删除人员出错: {e}")
                return False
        else:
            logger.info(f"The cn '{person_dn}' not exists.")
            return True

    def close(self):
        """
        关闭连接
        :return:
        """
        if self.connection:
            self.connection.unbind()
            logger.info("Disconnected from LDAP server.")

    def thread_modify_ou(self, res_data, choice_type=0):
        """
        将人员信息更新到ldap
        :param res_data: [{},{},{},{}]
        :param choice_type: 0: 变更组织单元； 1：更新组织单元; 2|新增组织单元人员；3|修改组织单元人员；4|删除组织单元人员
        :return: 返回操作失败的数据
        """
        try:
            # logger.info(f'{res_data}')
            logger.info(f"数据总数{len(res_data)}开启1个线程，状态为{choice_type}，进行ldap账号创建或更新或者删除")
            with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
                # 提交多个任务到线程池
                if choice_type == 0:
                    # 新增组织部门
                    futures = [executor.submit(self.add_hierarchy_ou, *(i['ou'],)) for i in res_data]
                elif choice_type == 1:
                    # 更新组织部门
                    futures = [executor.submit(self.modify_ou_name, *(i['ou'], i['name'],)) for i in res_data]
                elif choice_type == 2:
                    # 新增组织单元人员
                    futures = [executor.submit(self.add_person, *(i,)) for i in res_data]
                elif choice_type == 3:
                    # 修改组织单元人员
                    futures = [executor.submit(self.modify_person, *(i['old_name'], i['old_ou'], i['ou'], i['cn_name']))
                               for i in res_data]
                elif choice_type == 4:
                    # 删除组织单元人员
                    futures = [executor.submit(self.delete_person, *(i,)) for i in res_data]
                elif choice_type == 5:
                    # 变更人员组织部门信息
                    futures = [executor.submit(self.query_update_person, *(i['old_name'], i['ou'], i['cn_name']))
                               for i in res_data]

                # 使用as_completed()方法获取已完成任务的结果
                for future in concurrent.futures.as_completed(futures):
                    result = future.result()
        except Exception as e:
            logger.info(f"ldap操作异常 {e}")
        finally:
            self.close()
