import datetime
import json
import logging
import operator
import sqlalchemy
import numpy as np
import pandas as pd

from django.conf import settings
from django.db import close_old_connections

from devices.models import Devices
from users.models import User
from saltapi.models import PolicyInfo

logger = logging.getLogger(__name__)

DB_NAME = settings.DATABASES['default']['NAME']
DB_USER = settings.DATABASES['default']['USER']
DB_PASSWORD = settings.DATABASES['default']['PASSWORD']
DB_HOST = settings.DATABASES['default']['HOST']
DB_PORT = settings.DATABASES['default']['PORT']

database_url = 'mysql+pymysql://{}:{}@{}:{}/{}'.format(DB_USER, DB_PASSWORD, DB_HOST, DB_PORT, DB_NAME)

def get_superuser_list():
    superuser_list = list(User.objects.filter(is_superuser=1).values_list('username', flat=True))
    return superuser_list

def computational_priority(df):
    """
    计算多策略优先级，返回策略编号
    :param df: dataframe数据结构
    :return: policy_no
    """
    superuser_list = get_superuser_list()
    admin_query = df.query("policy_create_user in @superuser_list ")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        return admin_query.policy_no.values[0]
    else:
        df = df.sort_values(by='policy_update_time', ascending=False)
        return df.policy_no.values[0]


def software_id(df):
    """
    计算同一设备的软件对应策略
    :param df:
    :return:
    """
    df = df.replace(np.NaN, '')
    software_id_list = []
    res_software = []
    for i in df['software_id'].values.tolist():
        if isinstance(i, str):
            if ',' in i:
                software_id_list = software_id_list + i.split(',')
            else:
                software_id_list.append(i)
    while '' in software_id_list:
        software_id_list.remove('')
    software_id_list = sorted(set(software_id_list))
    for i in software_id_list:
        queryset = df.query("software_id.str.contains('{}')".format(i))
        if len(queryset):
            policy_no = computational_priority(queryset)
            res_software.append((i, policy_no))
        else:
            pass
    if len(res_software):
        return json.dumps(res_software)
    else:
        return json.dumps([('', '0')])


def default_ca_switch(df):
    """
    计算同一设备对应的usb开关状态
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['default_ca_switch'].notna()]['default_ca_switch'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and default_ca_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.default_ca_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query("default_ca_switch in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.default_ca_switch.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def default_ca_id(df):
    """
    计算同一设备的默认证书对应的策略
    :param df:
    :return:
    """
    df = df.replace(np.NaN, '')
    ca_id_list = []
    res_ca = []
    for i in df['default_ca_id'].values.tolist():
        if isinstance(i, str):
            if ',' in i:
                ca_id_list = ca_id_list + i.split(',')
            else:
                ca_id_list.append(i)
    while '' in ca_id_list:
        ca_id_list.remove('')
    ca_id_list = sorted(set(ca_id_list))
    for i in ca_id_list:
        queryset = df.query("default_ca_id.str.contains('{}')".format(i))
        if len(queryset):
            policy_no = computational_priority(queryset)
            res_ca.append((i, policy_no))
        else:
            pass

    if len(res_ca):
        return json.dumps(res_ca)
    else:
        return json.dumps([('', '0')])


def usb_switch(df):
    """
    计算同一设备对应的usb开关状态
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    # logger.info(f"******\r\n{df['usb_switch']}")
    choice_list = list(set(df[df['usb_switch'].notna()]['usb_switch'].values.tolist()))
    # logger.info(f"******{choice_list}")
    admin_query = df.query("policy_create_user in @superuser_list and usb_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.usb_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query("usb_switch in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.usb_switch.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def usb_rw_status(df):
    """
    计算同一设备对应的usb 只读控制
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['usb_rw_status'].notna()]['usb_rw_status'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and usb_rw_status in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (str(admin_query.usb_rw_status.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query("usb_rw_status in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (str(df.usb_rw_status.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def firewalld_switch(df):
    """
    计算同一设备对应的usb开关状态
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['firewalld_switch'].notna()]['firewalld_switch'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and firewalld_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.firewalld_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query("firewalld_switch in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.firewalld_switch.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def firewalld_level(df):
    """
    计算同一设备对应的防火墙 级别
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['firewalld_level'].notna()]['firewalld_level'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and firewalld_level in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.firewalld_level.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query("firewalld_level in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.firewalld_level.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def firewalld_white_switch(df):
    """
    计算同一设备对应的防火墙白名单开关
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['firewalld_white_switch'].notna()]['firewalld_white_switch'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and firewalld_white_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.firewalld_white_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query("firewalld_white_switch in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.firewalld_white_switch.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def firewalld_white_list(df):
    """
    计算同一设备对应的防火墙白名单列表
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['firewalld_white_list'].notna()]['firewalld_white_list'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and firewalld_white_list in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (admin_query.firewalld_white_list.values[0], admin_query.policy_no.values[0])
    else:
        df = df.query("firewalld_white_list in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (df.firewalld_white_list.values[0], df.policy_no.values[0])
    return json.dumps(res_tuple)


def firewalld_black_switch(df):
    """
    计算同一设备对应的防火墙黑名单开关
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['firewalld_black_switch'].notna()]['firewalld_black_switch'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and firewalld_black_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.firewalld_black_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query("firewalld_black_switch in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.firewalld_black_switch.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def firewalld_black_list(df):
    """
    计算同一设备对应的防火墙白名单列表
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['firewalld_black_list'].notna()]['firewalld_black_list'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and firewalld_black_list in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (admin_query.firewalld_black_list.values[0], admin_query.policy_no.values[0])
    else:
        df = df.query("firewalld_black_list in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (df.firewalld_black_list.values[0], df.policy_no.values[0])
    return json.dumps(res_tuple)


def remote_ssh_switch(df):
    """
    计算同一设备对应的ssh开关
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['remote_ssh_switch'].notna()]['remote_ssh_switch'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and remote_ssh_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.remote_ssh_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query("remote_ssh_switch in @choice_list").sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.remote_ssh_switch.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def screen_lock_switch(df):
    """
    计算同一设备对应的锁屏保护开关
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['screen_lock_switch'].notna()]['screen_lock_switch'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and screen_lock_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.screen_lock_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query('screen_lock_switch in @choice_list').sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.screen_lock_switch.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def screen_lock_time(df):
    """
    计算同一设备对应的锁屏时间
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['screen_lock_time'].notna()]['screen_lock_time'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and screen_lock_time in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.screen_lock_time.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query('screen_lock_time in @choice_list').sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.screen_lock_time.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def wallpaper_id(df):
    """
    计算同一设备对应的壁纸id
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['wallpaper_id'].notna()]['wallpaper_id'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and wallpaper_id in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.wallpaper_id.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query('wallpaper_id in @choice_list').sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.wallpaper_id.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def root_switch(df):
    """
    计算同一设备对应的root权限开关
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['root_switch'].notna()]['root_switch'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and root_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.root_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query('root_switch in @choice_list').sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.root_switch.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def activation_switch(df):
    """
    计算同一设备对应的激活服务器开关
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['activation_switch'].notna()]['activation_switch'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and activation_switch in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(admin_query.activation_switch.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query('activation_switch in @choice_list').sort_values(by='policy_update_time', ascending=False)
        res_tuple = (int(df.activation_switch.values[0]), df.policy_no.values[0])

    return json.dumps(res_tuple)


def activation_address(df):
    """
    计算同一设备对应的激活服务器地址
    :param df:
    :return:
    """
    superuser_list = get_superuser_list()

    choice_list = list(set(df[df['activation_address'].notna()]['activation_address'].values.tolist()))
    admin_query = df.query("policy_create_user in @superuser_list and activation_address in @choice_list")
    if len(admin_query):
        admin_query = admin_query.sort_values(by='policy_update_time', ascending=False)
        res_tuple = (str(admin_query.activation_address.values[0]), admin_query.policy_no.values[0])
    else:
        df = df.query('activation_address in @choice_list').sort_values(by='policy_update_time', ascending=False)
        res_tuple = (str(df.activation_address.values[0]), df.policy_no.values[0])
    return json.dumps(res_tuple)


def analyse_policy(x):
    res_dict = {}
    if len(x) >= 1:
        # 有多条策略的情况
        # 将策略内容转存到对应的列
        x = pd.concat([x, x['policy_context'].apply(pd.Series)], axis=1).drop('policy_context', axis=1)
        # logger.info(f"******\r\n{x}")
        x = pd.concat([x, x[0].apply(pd.Series)], axis=1).drop(0, axis=1)
        x = pd.concat([x, x[1].apply(pd.Series)], axis=1).drop(1, axis=1)
        x = pd.concat([x, x[2].apply(pd.Series)], axis=1).drop(2, axis=1)
        x = pd.concat([x, x[3].apply(pd.Series)], axis=1).drop(3, axis=1)
        x = pd.concat([x, x[4].apply(pd.Series)], axis=1).drop(4, axis=1)
        x = pd.concat([x, x[5].apply(pd.Series)], axis=1).drop(5, axis=1)
        res_dict['device_id'] = x.device_id.values[-1]
        if 'software_id' in x.columns:
            res_dict['software_id'] = x.groupby('device_id').apply(software_id).loc[res_dict['device_id']]
        else:
            res_dict['software_id'] = json.dumps([('', '0')])
        if 'usb_switch' in x.columns:
            res_dict['usb_switch'] = x.groupby('device_id').apply(usb_switch).loc[res_dict['device_id']]
        else:
            res_dict['usb_switch'] = json.dumps((0, '0'))
        if 'usb_rw_status' in x.columns:
            res_dict['usb_rw_status'] = x.groupby('device_id').apply(usb_rw_status).loc[res_dict['device_id']]
        else:
            res_dict['usb_rw_status'] = json.dumps(('rw', '0'))

        if 'firewalld_switch' in x.columns:
            res_dict['firewalld_switch'] = x.groupby('device_id').apply(firewalld_switch).loc[res_dict['device_id']]
        else:
            res_dict['firewalld_switch'] = json.dumps((0, '0'))

        if 'firewalld_level' in x.columns:
            res_dict['firewalld_level'] = x.groupby('device_id').apply(firewalld_level).loc[res_dict['device_id']]
        else:
            res_dict['firewalld_level'] = json.dumps((1, '0'))

        if 'firewalld_white_switch' in x.columns:
            res_dict['firewalld_white_switch'] = x.groupby('device_id').apply(firewalld_white_switch).loc[
                res_dict['device_id']]
        else:
            res_dict['firewalld_white_switch'] = json.dumps((0, '0'))

        if 'firewalld_white_list' in x.columns:
            x['firewalld_white_list'] = x['firewalld_white_list'].apply(
                lambda m: ','.join(m) if isinstance(m, list) else None)
            res_dict['firewalld_white_list'] = x.groupby('device_id').apply(firewalld_white_list).loc[
                res_dict['device_id']]
        else:
            res_dict['firewalld_white_list'] = json.dumps(('', '0'))

        if 'firewalld_black_switch' in x.columns:
            res_dict['firewalld_black_switch'] = x.groupby('device_id').apply(firewalld_black_switch).loc[
                res_dict['device_id']]
        else:
            res_dict['firewalld_black_switch'] = json.dumps((0, '0'))

        if 'firewalld_black_list' in x.columns:
            x['firewalld_black_list'] = x['firewalld_black_list'].apply(
                lambda m: ','.join(m) if isinstance(m, list) else None)
            res_dict['firewalld_black_list'] = x.groupby('device_id').apply(firewalld_black_list).loc[
                res_dict['device_id']]
        else:
            res_dict['firewalld_black_list'] = json.dumps(('', '0'))
        if 'remote_ssh_switch' in x.columns:
            res_dict['remote_ssh_switch'] = x.groupby('device_id').apply(remote_ssh_switch).loc[res_dict['device_id']]
        else:
            res_dict['remote_ssh_switch'] = json.dumps((0, '0'))
        if 'screen_lock_switch' in x.columns:
            res_dict['screen_lock_switch'] = x.groupby('device_id').apply(screen_lock_switch).loc[res_dict['device_id']]
        else:
            res_dict['screen_lock_switch'] = json.dumps((0, '0'))
        if 'screen_lock_time' in x.columns:
            res_dict['screen_lock_time'] = x.groupby('device_id').apply(screen_lock_time).loc[res_dict['device_id']]
        else:
            res_dict['screen_lock_time'] = json.dumps((0, '0'))
        if 'wallpaper_id' in x.columns:
            res_dict['wallpaper_id'] = x.groupby('device_id').apply(wallpaper_id).loc[res_dict['device_id']]
        else:
            res_dict['wallpaper_id'] = json.dumps((0, '0'))
        if 'root_switch' in x.columns:
            res_dict['root_switch'] = x.groupby('device_id').apply(root_switch).loc[res_dict['device_id']]
        else:
            res_dict['root_switch'] = json.dumps((0, '0'))
        if 'activation_switch' in x.columns:
            res_dict['activation_switch'] = x.groupby('device_id').apply(activation_switch).loc[res_dict['device_id']]
        else:
            res_dict['activation_switch'] = json.dumps((0, '0'))
        if 'activation_address' in x.columns:
            res_dict['activation_address'] = x.groupby('device_id').apply(activation_address).loc[res_dict['device_id']]
        else:
            res_dict['activation_address'] = json.dumps(('https://cloudaccount.os.nfschina.com/', '0'))
        if 'default_ca_switch' in x.columns:
            res_dict['default_ca_switch'] = x.groupby('device_id').apply(default_ca_switch).loc[res_dict['device_id']]
        else:
            res_dict['default_ca_switch'] = json.dumps((0, '0'))
        if 'default_ca_id' in x.columns:
            res_dict['default_ca_id'] = x.groupby('device_id').apply(default_ca_id).loc[res_dict['device_id']]
        else:
            res_dict['default_ca_id'] = json.dumps([('', '0')])
    else:
        pass

    return res_dict


def policy_main():
    """
    计算策略优先级，按时推送策略到终端
    优先级顺序，创建者超级管理员创建策略最高，同级管理者，最后更新时间的策略优先级最高
    :return: 输入每个设备的策略输出到数据库
    """
    start_time = datetime.datetime.now()
    res_df = pd.DataFrame()
    header = (
        'policy_name', 'policy_no', 'policy_device_list', 'policy_create_user', 'policy_update_time', 'policy_context')
    policy_items = PolicyInfo.objects.all().exclude(policy_run_status=3).filter(policy_device_list__isnull=False)
    if policy_items.exists():
        policy_items_list = policy_items.values_list('policy_name',
                                                     'policy_no',
                                                     'policy_device_list',
                                                     'policy_create_user',
                                                     'policy_update_time',
                                                     'policy_context')
        policy_items.exclude(policy_run_status=3).update(policy_run_status=2)
        df = pd.DataFrame(policy_items_list, columns=header)
        df['policy_context'] = df['policy_context'].apply(
            lambda x: sorted(json.loads(x), key=operator.itemgetter('policy_type')))
        df = df.drop(['policy_device_list'], axis=1).join(
            df['policy_device_list'].str.split(',', expand=True).stack().reset_index(level=1, drop=True).rename(
                'device_id'))
        device_list = [str(i) for i in Devices.objects.exclude(is_domain=0).values_list('id', flat=True)]
        df = df.query("device_id in @device_list")
        if len(df):
            res_df['policy_context'] = df.groupby('device_id').apply(analyse_policy)
            res_df = res_df.reset_index(drop=True)  # 重建索引
            res_df = pd.concat([res_df, res_df['policy_context'].apply(pd.Series)], axis=1).drop('policy_context',
                                                                                                 axis=1)
        else:
            pass
    else:
        pass
    end_time = datetime.datetime.now()
    engine = sqlalchemy.create_engine(database_url, echo=False)
    res_df.to_sql('device_policy_info', engine, if_exists='replace')
    logger.info(f"计算每个设备策略花费的时间{end_time - start_time}")
    close_old_connections()

    return res_df

# if __name__ == '__main__':
#     policy_main()
