"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/7/25 下午3:53
@File: request_api.py
@Desc: 调用http请求封装
"""
import requests
from requests.exceptions import RequestException
import time


class HttpRequest:
    def __init__(self, base_url, token, token_refresh_interval=1800, max_retries=3, retry_interval=1):
        self.base_url = base_url
        self.token = token
        self.token_refresh_interval = token_refresh_interval
        self.last_token_refresh_time = None
        self.session = requests.Session()
        self.session.headers.update({'Authorization': f'Token {token}'})
        self.max_retries = max_retries
        self.retry_interval = retry_interval

    def _authenticate(self):
        current_time = time.time()
        if not self.last_token_refresh_time or current_time - self.last_token_refresh_time > self.token_refresh_interval:
            self._get_new_token()
            self.last_token_refresh_time = current_time

    def _get_new_token(self):
        # 实现获取新Token的逻辑，可以在这里发送认证请求，获取新的Token
        pass

    def _handle_exceptions(self, func):
        def wrapper(*args, **kwargs):
            retries = self.max_retries
            while retries > 0:
                try:
                    self._authenticate()
                    return func(*args, **kwargs)
                except RequestException as e:
                    # 处理HTTP请求异常
                    retries -= 1
                    if retries > 0:
                        time.sleep(self.retry_interval)
                    else:
                        raise e

        return wrapper

    def _request(self, method, url, *args, **kwargs):
        full_url = self.base_url + url
        response = method(full_url, *args, **kwargs)
        response.raise_for_status()
        return response

    def get(self, url, params=None, **kwargs):
        wrapped_method = self._handle_exceptions(self.session.get)
        return self._request(wrapped_method, url, params=params, **kwargs)

    def put(self, url, data=None, **kwargs):
        wrapped_method = self._handle_exceptions(self.session.put)
        return self._request(wrapped_method, url, data=data, **kwargs)

    def post(self, url, data=None, json=None, **kwargs):
        wrapped_method = self._handle_exceptions(self.session.post)
        return self._request(wrapped_method, url, data=data, json=json, **kwargs)

    def delete(self, url, **kwargs):
        wrapped_method = self._handle_exceptions(self.session.delete)
        return self._request(wrapped_method, url, **kwargs)

    def close(self):
        self.session.close()
