"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/9/11 上午9:32
@File: sch_patch.py
@Desc: 定时动态下发补丁到终端
       通过 线程数，控制并发数量
       通过线程阻塞，查找指令的返回状态，是否释放线程，进行带宽限制
"""
import datetime
import ipaddress
import json
import logging
import time
import concurrent.futures

import pandas as pd
from django_redis import get_redis_connection

from devices.client_consumers import msg_to_client
from patch_management.models import PatchPolicy, PatchDeviceLog, PatchInfo, PatchFile, PatchSource, \
    PatchDeliverySettings
from devices.models import Devices
from groups.models import GroupsDetail
from django.db import close_old_connections

logger = logging.getLogger(__name__)
redis_conn = get_redis_connection('redis')


class PatchTask(object):

    def __init__(self):
        self.policy_items = PatchPolicy.objects.all()
        self.device_items = Devices.objects.exclude(is_domain=0).all()
        self.patch_device_log = PatchDeviceLog.objects.all()
        self.group_chat_items = GroupsDetail.objects.all()
        self.patch_info = PatchInfo.objects.filter(patch_is_delete=0).all()
        self.patch_file = PatchFile.objects.all()
        self.patch_delivery_settings = PatchDeliverySettings.objects.all().first()

    def get_device_list(self, row):
        """获取策略对应的终端数据"""
        # logger.info(f"{row}")
        range_type = row.get('range_type')
        range_value = row.get('range_value')
        exception_switch_type = row.get('exception_switch_type')
        exception_range_type = row.get('exception_range_type')
        exception_range_value = row.get('exception_range_value')
        res_device_list = []
        if range_type == 0:
            # 部门处理
            organ_list = range_value.split(',')
            if self.device_items.filter(org_info_id__in=organ_list).exists():
                res_device_list = self.device_items.filter(org_info_id__in=organ_list).values_list('id', flat=True)
        elif range_type == 1:
            # 终端处理
            res_device_list = range_value.split(',')
        elif range_type == 2:
            # ip处理
            ip_list = range_value.split(',')
            mid_ip_list = []
            for i in ip_list:
                start_ip, end_ip = i.split('-')
                # 解析起始和结束IP地址为IPv4Address对象
                start_ip = ipaddress.IPv4Address(start_ip)
                end_ip = ipaddress.IPv4Address(end_ip)
                range_ip_list = [str(ipaddress.IPv4Address(ip)) for ip in range(int(start_ip), int(end_ip) + 1)]
                mid_ip_list = range_ip_list + mid_ip_list
            if self.device_items.filter(device_ip__in=mid_ip_list).exists():
                res_device_list = self.device_items.filter(device_ip__in=mid_ip_list).values_list('id', flat=True)
        elif range_type == 3:
            # 人员处理
            account_list = range_value.split(',')
            if self.device_items.filter(bind_account__id__in=account_list).exists():
                res_device_list = self.device_items.filter(bind_account__id__in=account_list).values_list('id',
                                                                                                          flat=True)
        elif range_type == 4:
            # 群聊处理
            group_chat_list = range_value.split(',')
            if self.group_chat_items.filter(groups_id__in=group_chat_list).exists():
                group_chat_filter_items = self.group_chat_items.filter(groups_id__in=group_chat_list)
                items_0_range = group_chat_filter_items.filter(range=0).values_list('org_id', flat=True)
                items_1_range = group_chat_filter_items.filter(range=1).values_list('device_id', flat=True)
                items_3_range = group_chat_filter_items.filter(range=3).values_list('account_id', flat=True)
                # 处理部门数据
                if self.device_items.filter(org_info_id__in=items_0_range).exists():
                    device_0_list = self.device_items.filter(org_info_id__in=items_0_range).values_list('id', flat=True)
                    device_0_list = [str(i) for i in device_0_list]
                else:
                    device_0_list = []
                # 处理设备信息
                device_1_list = [str(i) for i in items_1_range]
                # 处理账号信息
                if self.device_items.filter(bind_account__id__in=items_3_range).exists():
                    device_3_list = self.device_items.filter(bind_account__id__in=items_3_range).values_list('id',
                                                                                                             flat=True)
                    device_3_list = [str(i) for i in device_3_list]
                else:
                    device_3_list = []
                device_list = device_0_list + device_1_list + device_3_list
                res_device_list = list(set(device_list))
        else:
            pass

        # 排除例外的情况
        if (exception_switch_type == 1) and res_device_list:  # 例外开关已开启
            if exception_range_type == 1:  # 例外范围为设备
                res_device_list = [int(i) for i in res_device_list]  # 强转整形
                exception_range_value = [int(i) for i in exception_range_value.split(',')]  # 强转整形
                set1 = set(res_device_list)
                set2 = set(exception_range_value)
                result_set = set1 - set2
                res_device_list = list(result_set)

        res_device_list = list(set(res_device_list))  # 去重处理
        res_device_list = [str(i) for i in res_device_list]
        res_device = ','.join(res_device_list) if len(res_device_list) != 0 else None

        return res_device

    def get_patch_name(self, patch_id):
        """
        获取补丁包的名称
        :param patch_id:
        :return: name
        """
        try:
            patch_file_id = self.patch_info.get(id=patch_id).patch_file_id
            patch_file_name = self.patch_file.get(id=patch_file_id).name
        except Exception as exp:
            # logger.info(f"获取补丁包名异常：{exp}")
            patch_file_name = None

        return patch_file_name

    def __data_analyze__(self):
        """数据库数据拆分，需要处理的策略数据"""
        device_all_items = Devices.objects.all()
        # 从缓存中获取数据
        cache_patch_policy = redis_conn.hget('patch_db_cache', 'patch_policy')
        if cache_patch_policy:
            cache_patch_policy = cache_patch_policy.decode('utf-8')
            cache_patch_policy = json.loads(cache_patch_policy)
        else:
            cache_patch_policy = self.policy_items.values()
        df = pd.DataFrame(cache_patch_policy)
        if len(df) > 0:
            # 统一时间格式，方便后续比较
            df['update_time'] = pd.to_datetime(df['update_time'])
            # df['execution_time'] = df['execution_time'].apply(
            #     lambda x: datetime.datetime.strptime(x, '%Y-%m-%d %H:%M:%S') if x is not None else None)
            df['device_list'] = df.apply(self.get_device_list, axis=1)
            df['effective_date'] = df['effective_date'].apply(lambda x: self.get_exe_time(x))
            df['expiration_date'] = df['expiration_date'].apply(lambda x: self.get_exe_time(x))
            df['device_id'] = df['device_list'].str.split(',')
            df = df.explode('device_id', ignore_index=True)
            df['patch_id'] = df['patch_ids'].str.split(',')
            df = df.explode('patch_id', ignore_index=True)
            df = df.dropna(subset=['device_id', 'patch_id'])
            df['device_ip'] = df['device_id'].apply(lambda x: device_all_items.get(id=x).device_ip)
            df['device_hostname'] = df['device_id'].apply(lambda x: device_all_items.get(id=x).device_hostname)
            df['patch_name'] = df['patch_id'].apply(lambda x: self.get_patch_name(x))
            df.sort_values(by='update_time', ascending=True)
            # 获取当前时间
            current_time = datetime.datetime.now()

            # 筛选条件：
            # 1. 获取当前时间大于 a 且小于 b 的行，并且 a 和 b 都不是空值
            condition1 = (df['effective_date'].notna()) & (df['expiration_date'].notna()) & (current_time >= df['effective_date']) & (current_time <= df['expiration_date'])

            # 2. 获取 a 或 b 为 NaN 的行
            condition2 = df['effective_date'].isna() | df['expiration_date'].isna()

            # 合并条件
            df = df[condition1 | condition2]
            # 获取启用的策略
            df = df[df['status'] == 1]

        # logger.info(f"****需要执行的策略数*****{df.to_dict('records')}")

        return df.to_dict('records')

    def __data_compare__(self, res_items):
        """
        拆分数据和结果数据比较，得出需要下发数据
        :param res_items:
        :return: {
                   'id': 18, 'name': '策略2', 'serial_number': '20230908145354',
                   'range_type': 1, 'range_value': '1,2,3',
                   'status': 1, 'patch_ids': '4', 'creator': 'xhy',
                   'update_time': Timestamp('2023-09-08 14:53:54.482480'),
                   'remark': '记录创建人员信息', 'device_list': '3'
        }
        """
        patch_id = res_items.get('patch_id')
        device_id = res_items.get('device_id')
        # 下发没有执行过的机器
        if self.patch_device_log.filter(patch_id=patch_id, device_id=device_id).exists():
            if self.patch_device_log.filter(patch_id=patch_id, device_id=device_id, is_send_status=0).exists():
                # 再次下发
                send_data = res_items
            elif self.patch_device_log.filter(patch_id=patch_id, device_id=device_id,
                                              install_err_msg='下载失败').exists():
                # 下载失败的再次下发
                send_data = res_items
            else:
                send_data = None
        else:
            # 初次下发
            send_data = res_items

        return send_data

    def __data_to_device__(self, res_items):
        """
        指令下发到终端
        :param res_items:
        {
        'sender': '2',
        'sn': '9-6-6',
        'data': {
                 'id': 18,'name': '策略2','serial_number': '20230908145354','range_type': 1,'range_value': '4,7,11',
                 'status': 1,'patch_ids': '3008,3010','creator': 'xhy','update_time': Timestamp('2023-09-08 14:53:54.482480'),
                 'remark': '记录创建人员信息','device_list': '4,7,11','device_id': '11','patch_id': '3010',
                 'device_ip': '172.30.98.75','device_hostname': 'xiechuan-11-pc.os-domain.cpu-os.com',
                 'patch_name': 'atom-amd64.deb'
                 }
        }
        """
        # logger.info(f"***res_times:{res_items}***")
        device_hostname = res_items.get('device_hostname')
        if 'update_time' in res_items:
            res_items['update_time'] = res_items['update_time'].strftime('%Y-%m-%d %H:%M:%S')

        send_items = {
            'device_id': res_items['device_id'],
            'patch_id': res_items['patch_id'],
            'patch_name': res_items['patch_name'],
            'device_hostname': res_items['device_hostname'],
            'is_notify_user': res_items['is_notify_user'],
            'is_user_cancel': res_items['is_user_cancel'],
        }
        data_json = {
            "sender": "2",
            "sn": "9-6-6",  # 补丁下发命令
            "data": send_items
        }
        """数据下发到终端方法"""
        n = 0
        is_send_status_res = 0
        while True:
            redis_channel_name = redis_conn.hget('client_channel_name', device_hostname)
            if redis_channel_name:
                redis_channel_name = redis_channel_name.decode('utf-8')
                if res_items['patch_name'] is not None:
                    msg_to_client(1, redis_channel_name, data_json)
                    is_send_status_res = 1
                    break
                else:
                    break
            else:
                n = n + 1
                time.sleep(10)
                if n >= 3:
                    break

        # 扫描redis缓存记录指令状态，判断是否退出
        patch_id = res_items.get('patch_id')
        device_id = res_items.get('device_id')
        if self.patch_device_log.filter(patch_id=patch_id, device_id=device_id).exists():
            self.patch_device_log.filter(patch_id=patch_id, device_id=device_id, install_status=0).update(
                is_send_status=is_send_status_res,
                send_msg_time=datetime.datetime.now() if is_send_status_res == 1 else None
            )
        else:
            if res_items['patch_name'] is not None:
                if (patch_id is not None) and (device_id is not None):
                    self.patch_device_log.update_or_create(
                        patch_id=patch_id,
                        device_id=device_id,
                        defaults={
                            'install_status': 0,
                            'install_time': datetime.datetime.now(),
                            'install_err_msg': None,
                            'is_send_status': is_send_status_res,
                            'send_msg_time': datetime.datetime.now() if is_send_status_res == 1 else None,
                            'is_receive_status': 0
                        }
                    )
                else:
                    pass
            else:
                pass


    def __query_device_patch__(self, device_hostname):
        """重启查询设备补丁"""
        if self.device_items.filter(device_hostname=device_hostname).exists():
            device_obj = self.device_items.get(device_hostname=device_hostname)
            data_json = {
                "sender": "2",
                "sn": "9-5-6",  # 查询终端所有补丁状态
                "data": {'device_id': device_obj.id}
            }
            """数据下发到终端方法"""
            n = 0
            while True:
                redis_channel_name = redis_conn.hget('client_channel_name', device_hostname)
                if redis_channel_name:
                    redis_channel_name = redis_channel_name.decode('utf-8')
                    msg_to_client(1, redis_channel_name, data_json)
                    break
                else:
                    n = n + 1
                    if n > 6:
                        break

    def __data_distribution__(self, res_data):
        """多线程 获取可以下发的数据"""
        send_item_list = []
        with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
            # 提交多个任务到线程池
            futures = [executor.submit(self.__data_compare__, i) for i in res_data]
            # 使用as_completed()方法获取已完成任务的结果
            for future in concurrent.futures.as_completed(futures):
                result = future.result()
                if result:
                    send_item_list.append(result)

        return send_item_list

    def __bulk_data_delivery__(self, data_list):
        """大量数据下发处理, 下发控制数据"""
        threading_no = 1
        threading_pool = concurrent.futures.ThreadPoolExecutor(threading_no)
        threading_pool.map(self.__data_to_device__, data_list)
        threading_pool.shutdown(wait=True)

    @staticmethod
    def get_exe_time(execution_time):
        """
        获取策略的执行时间
        :param execution_time:
        :return:
        """
        if execution_time is None:
            exe_time = None
        else:
            if isinstance(execution_time, str):
                exe_time = datetime.datetime.strptime(execution_time, '%Y-%m-%d %H:%M:%S')
            else:
                exe_time = execution_time.strftime('%Y-%m-%d %H:%M:%S')

        return exe_time


def patch_task(device_hostname=None):
    start_time = datetime.datetime.now()
    logger.info(f"****{start_time}  patch task start******")
    if PatchSource.objects.filter(patch_source_enable=1).exists():
        class_patch = PatchTask()
        if device_hostname is not None:
            class_patch.__query_device_patch__(device_hostname)
        # 查看需要安装的补丁
        else:
            res_data = class_patch.__data_analyze__()
            # logger.info(f"*****\r\n{res_data}")
            if res_data:
                send_items_list = class_patch.__data_distribution__(res_data)
            else:
                send_items_list = []
            if send_items_list:
                class_patch.__bulk_data_delivery__(send_items_list)
    close_old_connections()
    logger.info(f"****{start_time} patch task end******")
