"""
-*- coding: utf-8 -*-
@Author: huchl
@Time: 2023/7/28 下午2:01
@File: sch_soap_task_member.py
@Desc: 定时获取SOAP成员信息，全量和增量获取
"""
import concurrent.futures
import configparser
import datetime
import glob
import json
import logging
import os
import time
import sqlalchemy
import pandas as pd
import numpy as np
from pandas import errors as pd_errors
from python_freeipa import ClientMeta, exceptions

from django.db.models import Max
from accounts.models import AccountEntity, AuthSource
from nfs_domain_mainserver import settings
from nfs_domain_mainserver.settings import BASE_DIR
from organizations.models import OrgnizationEntity
from utils.ldap_server import LDAPManager
from utils.zeep_api import SOAPClient

from utils.utils import sm4_hash, java_sm4_hash
from accounts.freeipa import reset_user_pwd, check_user

logger = logging.getLogger(__name__)

CONFIG_PATH = os.path.join(BASE_DIR, 'nfs_domain_mainserver/conf_ini/config.ini')

DB_NAME = settings.DATABASES['default']['NAME']
DB_USER = settings.DATABASES['default']['USER']
DB_PASSWORD = settings.DATABASES['default']['PASSWORD']
DB_HOST = settings.DATABASES['default']['HOST']
DB_PORT = settings.DATABASES['default']['PORT']

database_url = 'mysql+pymysql://{}:{}@{}:{}/{}'.format(DB_USER, DB_PASSWORD, DB_HOST, DB_PORT, DB_NAME)
engine = sqlalchemy.create_engine(
    database_url,
    echo=False,  # SQL 回显功能
    pool_size=5,  # 连接池中保持的连接数
    max_overflow=10,  # 连接池达到最大值后可以创建的额外连接数
    pool_timeout=30,  # 尝试获取连接的超时时间（秒）
    pool_recycle=1800,  # 防止连接超过这个时间而不被关闭和重新创建（秒），有助于避免数据库端口长时间空闲而断开连接的问题
)


class SoapMember:
    '''
    shanghaiyinhgna conf
    '''

    def __init__(self, url_choice=0):
        config = configparser.ConfigParser()
        config.read(CONFIG_PATH, encoding="utf-8")
        auth_sources = AuthSource.objects.filter().all().first()
        self.auth_sources = auth_sources if auth_sources else None
        self.appid = config.get('soap_api', 'appid')
        self.soap_ip = config.get('soap_api', 'soap_ip')
        self.soap_port = config.get('soap_api', 'soap_port')
        if url_choice == 0:
            self.soap_url = config.get('soap_api', 'soap_url_0')
        else:
            self.soap_url = config.get('soap_api', 'soap_url_1')
        self.soap_organ_file_patch = config.get('soap_api', 'soap_organ_file_path')
        self.soap_organ_file_patch = os.path.dirname(settings.BASE_DIR) + self.soap_organ_file_patch
        self.soap_organ_root_id = config.get('soap_api', 'soap_organ_root_id')
        self.soap_node_name = config.get('soap_api', 'soap_node_name')
        self.soap_encoding = config.get('soap_api', 'soap_encoding')
        self.soap_sm4_keys = config.get('soap_api', 'soap_sm4_keys')
        self.soap_update_password_emop = config.get('soap_api', 'soap_update_password_emop')
        self.soap_acquire_members = config.get('soap_api', 'soap_acquire_members')
        self.soap_auth_by_sm4 = config.get('soap_api', 'soap_auth_by_sm4')
        self.soap_send_code = config.get('soap_api', 'soap_send_code')
        self.soap_acquire_one_day_members = config.get('soap_api', 'soap_acquire_one_day_members')
        self.soap_one_recodes = {
            'add_user': 0, 'update_user': 0, 'fail_user': 0, 'add_group': 0, 'update_group': 0, 'fail_group': 0,
            'clash_user': 0, 'clash_group': 0, 'sync_at': datetime.datetime.now(), 'sync_status': 1,
            'sync_err_msg': {},
        }
        wsdl_url = f"http://{self.soap_ip}:{self.soap_port}/{self.soap_url}"
        if self.auth_sources:
            wsdl_url = f"http://{auth_sources.addr}/{self.soap_url}"
        self.client = SOAPClient(wsdl_url=wsdl_url)
        self.user_prefix = config.get('soap_api', 'user_prefix')

    @classmethod
    def get_latest_file(cls, directory_path):
        # 获取目录中所有文件的列表
        files = glob.glob(os.path.join(directory_path, '*'))

        # 如果目录中没有文件，则返回None
        if not files:
            return None

        # 使用文件的修改时间作为排序依据，获取最新的文件
        latest_file = max(files, key=os.path.getctime)

        return latest_file

    def update_freeipa_name(self):
        """定时任务 每次更新freeipa中的三层部门信息"""
        account_freeipa_db_items = AccountEntity.objects.filter(create_type=2).all()
        df = pd.DataFrame(account_freeipa_db_items.values('org_info_id', 'name', 'cn_name'))
        start_time = datetime.datetime.now()
        if len(df):
            df['old_name'] = df['name'].apply(lambda x: x.split(self.user_prefix)[-1])
            # 更新freeipa部门信息
            res_update_data = df.to_dict('records')
            self.update_user_to_freeipa(res_update_data, 0)
            # 更新ldap组织单元人员
            df['ou'] = df['org_info_id'].apply(lambda x: self.get_up_3_organ_name(x, 1))
            df = df.dropna(subset=['ou'])
            df = df[df['ou'].apply(lambda x: self.soap_node_name in x)]
            res_update_data_1 = df.to_dict('records')
            LDAPManager().thread_modify_ou(res_update_data_1, 5)
        else:
            pass

        end_time = datetime.datetime.now()
        logger.info(f"freeipa_ldap更新账号部门时间 {end_time - start_time}")

    def update_organ_to_db(self):
        """将部门信息更新到db中"""
        try:
            organ_db_items = OrgnizationEntity.objects.all()
            latest_file = self.get_latest_file(self.soap_organ_file_patch)
            if not latest_file:
                return 400, f'在{self.soap_organ_file_patch}没找到最新的文件'

            abs_file_patch = latest_file
            logger.info(f"文件路径{abs_file_patch}")
            try:
                df = pd.read_csv(abs_file_patch, sep='|', encoding=self.soap_encoding, header=None)
            except pd_errors.EmptyDataError:
                logger.info(f"处理{self.soap_organ_file_patch}为空文件")
                return 200, '同步完成'
            except exceptions as exp:
                logger.info(f"处理{self.soap_organ_file_patch}部门文件错误： {exp}")
                return 400, f"部门文件错误： {exp}"
            df = df[[0, 1, 2, 4]]
            df.columns = ['id', 'name', 'parent_id', 'seq_num']
            df['remarks'] = None
            # 排除已经存在的部门表中的数据
            # remarks='外部源',标志位
            if organ_db_items.filter(remarks='外部源', create_type=2).exists():
                organ_db_max_id = organ_db_items.get(remarks='外部源', create_type=2).id
            else:
                maximum_value = df['id'].max()
                organ_db_max_id = organ_db_items.order_by('-id').first().id + 1
                if int(maximum_value) > int(organ_db_max_id):
                    organ_db_max_id = int(maximum_value) + 10000
                seq_num__max = organ_db_items.filter(parent_id=1).aggregate(Max('seq_num'))['seq_num__max']
                seq_num = (1 + seq_num__max) if seq_num__max is not None else 0
                df.loc[len(df.index)] = [str(organ_db_max_id), self.soap_node_name, 1, seq_num, '外部源']  # 向df中插入根目录

            exists_id_list = [str(i) for i in organ_db_items.filter(create_type=2).values_list('id', flat=True)]

            # 同步删除数据soap不存在的，数据库存在的部门
            # soap_id_list = df['id'].drop_duplicates().tolist()
            # organ_db_items.filter(create_type=2).exclude(id__in=soap_id_list).delete()

            # 根据id列的值去重
            df = df.drop_duplicates(subset=['id'])

            # 不清楚根部门的parent_id值，调试处理
            df.replace({np.nan: None}, inplace=True)
            df['parent_id'] = df['parent_id'].apply(lambda x: organ_db_max_id if x is None else x)
            df.loc[df['remarks'] == '外部源', 'parent_id'] = 1
            df['create_type'] = 2
            df['created_at'] = datetime.datetime.now()

            # 排除已经在数据库的部门数据
            filtered_create_df = df[~df['id'].isin(exists_id_list)]
            if len(filtered_create_df) != 0:
                filtered_create_df_sql = filtered_create_df.copy()
                filtered_create_df_sql.set_index('id', inplace=True)
                # 批量创建数据
                filtered_create_df_sql.to_sql('org_info', engine, if_exists='append')
                # 创建freeipa中OU单元信息
                filtered_create_df['ou'] = filtered_create_df['id'].apply(lambda x: self.get_up_3_organ_name(x, 1))
                filtered_create_df = filtered_create_df.dropna(subset=['ou'])

                filtered_create_df = filtered_create_df[
                    filtered_create_df['ou'].apply(lambda x: self.soap_node_name in x)]
                LDAPManager().thread_modify_ou(filtered_create_df.to_dict('records'), 0)
            # 需要更新的数据
            filtered_update_df = df[df['id'].isin(exists_id_list)]
            if len(filtered_update_df) != 0:
                res_update_data = filtered_update_df.to_dict('records')
                # logger.info(f"update 数据 {res_update_data}")
                for item in res_update_data:
                    if 'index' in item:
                        del item['index']
                    # logger.info(f"****{item}")
                    organ_db_items.filter(id=item['id']).update(**item)

                # 更新freeipa中OU单元信息
                filtered_update_df['ou'] = filtered_update_df['id'].apply(lambda x: self.get_up_3_organ_name(x, 1))
                filtered_update_df = filtered_update_df.dropna(subset=['ou'])
                filtered_update_df = filtered_update_df[
                    filtered_update_df['ou'].apply(lambda x: self.soap_node_name in x)]
                res_update_data_1 = filtered_update_df.to_dict('records')
                LDAPManager().thread_modify_ou(res_update_data_1, 1)

            # 统计同步一次数据
            self.soap_one_recodes['add_group'] = len(filtered_create_df)
            self.soap_one_recodes['update_group'] = len(filtered_update_df)
            self.soap_one_recodes['fail_group'] = 0
        except Exception as exp:
            return 400, exp
        else:
            return 200, '部门同步完成'

    @staticmethod
    def get_up_3_organ_name(organ_id, choice_type=0):
        """
        获取部门名层级
        :param choice_type: 0:获取 顶部三层部门名； 1：获取所有部门名
        :param organ_id: 账号对应的部门id
        :return: 返回信息 name1/name2/name3
        """
        up_3_organ_name = []
        organ_items = OrgnizationEntity.objects.all()
        organ_id_temp = organ_id
        organ_db_max_id = organ_items.get(remarks='外部源', create_type=2).id
        try:
            if choice_type == 0:
                break_id = organ_db_max_id
            else:
                break_id = 1
            while True:
                if organ_items.filter(id=organ_id_temp).exclude(parent_id=organ_id_temp).exists():
                    organ_obj = organ_items.get(id=organ_id_temp)
                    up_3_organ_name.append(organ_obj.name)
                    organ_id_temp = organ_obj.parent_id
                    if organ_obj.id == break_id:
                        break
                else:
                    # logger.info(f"获取父节点失败 id为{organ_id_temp}")
                    break
        except Exception as e:
            pass
            # logger.info(f'获取上级部门信息错误，错误信息为{e}')
        finally:
            if len(up_3_organ_name) != 0:
                up_3_organ_name.reverse()
                if choice_type == 0:
                    up_3_organ_name = '/'.join(up_3_organ_name[3:6])
                else:
                    pass
            else:
                up_3_organ_name = None
            return up_3_organ_name

    def create_user_freeipa(self, res_data, client):
        """创建freeipa账号"""
        organ_id = res_data.get('org_info_id')
        organ_3_name = self.get_up_3_organ_name(organ_id)
        a_uid = res_data.get('name')
        gn = f"{res_data.get('cn_name')}({organ_3_name})"
        pwd = res_data.get('password')
        max_life = datetime.datetime.now() + datetime.timedelta(days=365 * 30)
        max_life = max_life.strftime("%Y-%m-%d %H:%M:%SZ")
        res_name_data = None
        try:
            client.user_show(a_uid)
            return res_name_data, 1
        except exceptions.NotFound:
            logger.info(f"Account {a_uid} does not exist.")
            for i in range(3):
                try:
                    # o_sn 姓氏 大小写转换会去检查，不要变动，默认为账号名称
                    client.user_add(a_uid, o_givenname=gn, o_sn=a_uid, o_cn=gn, o_displayname=gn, o_gecos=gn,
                                    o_userpassword=pwd if pwd else settings.DEFAULT_USER_PASSWORD,
                                    o_krbpasswordexpiration=max_life, o_loginshell='/bin/bash')
                    logger.info(f'创建账号 {a_uid} 完成')
                    break
                except Exception as exp:
                    if i < 3 - 1:
                        logger.info(f"Retrying in 1 seconds... (Attempt {i + 1}/{3})")
                        logger.info(f"创建账号失败，错误信息{exp}")
                        time.sleep(1)
                    else:
                        res_name_data = a_uid
        except Exception as exp:
            logger.info(f"查询账号失败 {exp}")

        return res_name_data, 1

    def update_user_freeipa(self, res_data, client):
        """更新freeipa账号"""
        organ_id = res_data.get('org_info_id')
        organ_3_name = self.get_up_3_organ_name(organ_id)
        a_uid = res_data.get('name')
        gn = f"{res_data.get('cn_name')}({organ_3_name})"
        res_name_data = None
        try:
            user_info = client.user_show(a_uid)
            freeipa_gn = user_info['result']['givenname'][0]
        except Exception as exp:
            freeipa_gn = None
        logger.info(f"**********{freeipa_gn}******{gn}")
        if freeipa_gn == gn:
            return res_name_data, 0
        else:
            for i in range(3):
                try:
                    client.user_mod(a_uid, o_givenname=gn, o_cn=gn, o_displayname=gn, o_gecos=gn,
                                    o_loginshell='/bin/bash')
                    logger.info(f'更新账号 {a_uid} 完成')
                    break
                except Exception as exp:
                    if i < 3 - 1:
                        logger.info(f"Retrying in 1 seconds... (Attempt {i + 1}/{3})")
                        logger.info(f"更新账号失败，错误信息{exp}")
                        time.sleep(1)
                    else:
                        res_name_data = a_uid

        return res_name_data, 0

    @staticmethod
    def delete_user_freeipa(res_data, client):
        """删除freeipa账号"""
        a_uid = res_data.get('name')
        res_name_data = None
        for i in range(3):
            try:
                client.user_del(a_uid=a_uid)
                logger.info(f'删除账号 {a_uid} 完成')
                break
            except Exception as exp:
                if i < 3 - 1:
                    logger.info(f"Retrying in 1 seconds... (Attempt {i + 1}/{3})")
                    logger.info(f"删除账号失败，错误信息{exp}")
                    time.sleep(1)
                else:
                    res_name_data = a_uid

        return res_name_data, 2

    def update_user_to_freeipa(self, res_data, is_create_update):
        """
        将人员信息更新到freeipa
        :param res_data: [{},{},{},{}]
        :param is_create_update:  1|创建数据； 0|更新数据; 2|删除数据
        :return:
        """
        try:
            freeipa_client = ClientMeta(settings.DOMAIN_SERVER_FQDN, verify_ssl=False)
            freeipa_client.login(settings.DOMAIN_SERVER_ADMIN, settings.DOMAIN_SERVER_ADMIN_PASSWD)
            create_err_name_list = []
            update_err_name_list = []
            delete_err_name_list = []
            logger.info(
                f"数据总数{len(res_data)}开启3个线程，状态为{is_create_update}，进行freeipa 账号创建或更新或者删除")
            with concurrent.futures.ThreadPoolExecutor(max_workers=3) as executor:
                # 提交多个任务到线程池
                if is_create_update == 1:
                    futures = [executor.submit(self.create_user_freeipa, *(i, freeipa_client)) for i in res_data]
                elif is_create_update == 0:
                    futures = [executor.submit(self.update_user_freeipa, *(i, freeipa_client)) for i in res_data]
                elif is_create_update == 2:
                    futures = [executor.submit(self.delete_user_freeipa, *(i, freeipa_client)) for i in res_data]

                # 使用as_completed()方法获取已完成任务的结果
                for future in concurrent.futures.as_completed(futures):
                    result = future.result()
                    if result[1] == 1:
                        if result[0]:
                            create_err_name_list.append(result[0])
                    elif result[1] == 0:
                        if result[0]:
                            update_err_name_list.append(result[0])
                    elif result[1] == 2:
                        if result[0]:
                            delete_err_name_list.append(result[0])
                    else:
                        pass
            freeipa_client.logout()
            if is_create_update == 1:
                return create_err_name_list
            elif is_create_update == 0:
                return update_err_name_list
            elif is_create_update == 2:
                return delete_err_name_list
        except Exception as e:
            logger.info(f"freeipa连接异常 {e}")

    def update_user_to_db_freeipa(self, res_data):
        """将人员信息更新到db和freeipa"""
        try:
            if len(res_data) == 0:
                return 200, None
            acs_items = AccountEntity.objects.all()
            organ_db_items = OrgnizationEntity.objects.all()
            organ_db_max_id = organ_db_items.get(remarks='外部源', create_type=2).id
            df = pd.DataFrame(res_data)
            if 'Password' not in df.columns:
                df['Password'] = None
            if 'operType' not in df.columns:
                df['operType'] = None
            # df = df[['NAME', 'Code', 'DEPCODE', 'Password', 'operType']]  # 测试
            df = df[['NAME', 'ID', 'DEPCODE', 'Password', 'operType']]
            df.columns = ['cn_name', 'name', 'org_info_id', 'password', 'create_update']
            df['old_name'] = df['name']
            df['name'] = df['name'].apply(lambda x: f"{self.user_prefix}{x}")
            df['org_info_id'] = df['org_info_id'].apply(
                lambda x: organ_db_max_id if (x == self.soap_organ_root_id) or (x is None) else x)
            df['create_type'] = 2
            df['ou'] = df['org_info_id'].apply(lambda x: self.get_up_3_organ_name(x, 1))
            df = df.dropna(subset=['ou'])  # 清除 ou列中空值可能到 df为空
            if len(df) == 0:
                return 200, None
            df = df[df['ou'].apply(lambda x: self.soap_node_name in x)]
            acs_name_list = list(acs_items.filter(create_type=2).values_list('name', flat=True))
            # 数据在内存中存在但是不存在数据库的数据，表示要新增数据库数据
            filtered_create_df = df[~df['name'].isin(acs_name_list)]
            # logger.info(f"create  数据 {filtered_create_df.to_dict('records')}")
            if len(filtered_create_df) != 0:
                filtered_create_df['user_status'] = 0
                filtered_create_df['is_root'] = 0
                filtered_create_df['system_status'] = 1
                filtered_create_df['policy_status'] = 1
                filtered_create_df['is_extra_status'] = 0
                filtered_create_df['created_at'] = datetime.datetime.now()
                res_create_data = filtered_create_df.to_dict('records')
                # 批量创建freeipa账号数据
                start_time = datetime.datetime.now()
                create_err_name_list = self.update_user_to_freeipa(res_create_data, 1)
                end_time = datetime.datetime.now()

                # 批量创建freeipa组织单元人员
                LDAPManager().thread_modify_ou(res_create_data, 2)

                logger.info(f"freeipa创建账号时间 {end_time - start_time}")
                filtered_create_df = filtered_create_df.drop(columns=['password', 'create_update', 'ou', 'old_name'])
                # 排除创建失败的数据
                filtered_create_df = filtered_create_df[~filtered_create_df['name'].isin(create_err_name_list)]
                # 统计同步一次数据
                self.soap_one_recodes['add_user'] = len(filtered_create_df)
                self.soap_one_recodes['fail_user'] = len(create_err_name_list)
                self.soap_one_recodes['sync_err_msg']['fail_user_create_msg'] = create_err_name_list if len(
                    create_err_name_list) > 0 else None
                res_create_data = filtered_create_df.to_dict('records')
                filtered_create_df.set_index('name', inplace=True)
                logger.info(f'filtered_create_df 写入DB的数据 {len(res_create_data)}')
                try:
                    filtered_create_df.to_sql('account_info', engine, if_exists='append')
                except Exception as e:
                    logger.info(f'filtered_create_df 写入DB失败，{e}')
                    # 清除freeipa中添加的记录
                    delete_err_name_list = self.update_user_to_freeipa(res_create_data, 2)

            # 数据存在内存也存在数据库中的，表示要更新数据库数据
            filtered_update_df = df[df['name'].isin(acs_name_list)]
            # logger.info(f"update  数据 {filtered_update_df}")
            if len(filtered_update_df) != 0:
                res_update_data = filtered_update_df.to_dict('records')
                start_time = datetime.datetime.now()
                update_err_name_list = self.update_user_to_freeipa(res_update_data, 0)
                end_time = datetime.datetime.now()
                logger.info(f"freeipa更新账号时间 {end_time - start_time}")

                # 批量更新ldap组织单元人员
                filtered_update_df['old_org_info_id'] = filtered_update_df['name'].apply(
                    lambda x: acs_items.get(create_type=2, name=x).org_info_id if acs_items.filter(create_type=2,
                                                                                                   name=x).exists() else organ_db_max_id)
                filtered_update_df['old_ou'] = filtered_update_df['old_org_info_id'].apply(
                    lambda x: self.get_up_3_organ_name(x, 1))
                res_update_data_1 = filtered_update_df.to_dict('records')
                LDAPManager().thread_modify_ou(res_update_data_1, 3)

                # 删除password列
                filtered_update_df = filtered_update_df.drop(
                    columns=['password', 'create_update', 'ou', 'old_name', 'old_ou', 'old_org_info_id'])
                # logger.info(f"update 错误信息名单 数据 {update_err_name_list}")
                # 排除更新失败的数据
                filtered_update_df = filtered_update_df[~filtered_update_df['name'].isin(update_err_name_list)]
                # 统计同步一次数据
                self.soap_one_recodes['update_user'] = len(filtered_update_df)
                self.soap_one_recodes['fail_user'] = self.soap_one_recodes['fail_user'] + len(update_err_name_list)
                self.soap_one_recodes['sync_err_msg']['fail_user_update_msg'] = update_err_name_list if len(
                    update_err_name_list) > 0 else None

                res_update_data = filtered_update_df.to_dict('records')
                # logger.info(f"update 数据 {res_update_data}")
                for item in res_update_data:
                    acs_items.filter(name=item['name']).update(**item)

            self.soap_one_recodes['clash_user'] = 0
            self.soap_one_recodes['clash_group'] = 0
            self.soap_one_recodes['sync_at'] = datetime.datetime.now()
            self.soap_one_recodes['sync_status'] = 1
        except Exception as exp:
            return 400, exp
        else:
            return 200, self.soap_one_recodes

    def delete_user_to_db_freeipa(self, res_data):
        """将人员信息从db和freeipa删除"""
        try:
            if len(res_data) == 0:
                return 200, None
            acs_items = AccountEntity.objects.all()
            organ_db_items = OrgnizationEntity.objects.all()
            organ_db_max_id = organ_db_items.get(remarks='外部源', create_type=2).id
            df = pd.DataFrame(res_data)
            df = df[['userId', 'operType']]
            df.columns = ['name', 'create_update']
            df['old_name'] = df['name']
            df['name'] = df['name'].apply(lambda x: f"{self.user_prefix}{x}")
            df['org_info_id'] = df['name'].apply(
                lambda x: acs_items.get(create_type=2, name=x).org_info_id if acs_items.filter(create_type=2,
                                                                                               name=x).exists() else organ_db_max_id)
            df['ou'] = df['org_info_id'].apply(lambda x: self.get_up_3_organ_name(x, 1))
            df = df.dropna(subset=['ou'])
            df = df[df['ou'].apply(lambda x: self.soap_node_name in x)]
            res_delete_data = df.to_dict('records')
            delete_err_name_list = self.update_user_to_freeipa(res_delete_data, 2)

            # 批量删除ldap组织单元人员
            LDAPManager().thread_modify_ou(res_delete_data, 4)

            # 排除删除freeipa 账号失败的账号
            filtered_delete_df = df[~df['name'].isin(delete_err_name_list)]
            name_delete_list = filtered_delete_df['name'].tolist()
            acs_items.filter(name__in=name_delete_list).delete()
            logger.info(f"需要删除的账号总数为{len(res_data)},"
                        f"freeipa上删除失败的账号数{len(delete_err_name_list)},"
                        f"数据删除的账号数{len(name_delete_list)}")
        except Exception as e:
            return 400, e
        else:
            return 200, self.soap_one_recodes

    def append_recodes_to_db(self, one_recodes):
        """
        将同步一次的数据记录在数据库中
        :param one_recodes: 库表记录字段
        :return:
        """
        if one_recodes is None:
            one_recodes = self.soap_one_recodes

        one_recodes['sync_err_msg'] = json.dumps(one_recodes['sync_err_msg'], ensure_ascii=False)
        one_recodes = [one_recodes]
        logger.info(f"写入记录数据 {one_recodes}")
        df = pd.DataFrame(one_recodes)
        df.to_sql('accounts_auth_records', engine, index=False, if_exists='append')

    def get_member_all(self):
        """从soap中获取所有成员数据"""
        page_num = 1
        res_user_list = []
        while True:
            query_params = {"appId": self.appid, "pageNum": page_num}
            res_data = self.client.call(self.soap_acquire_members, query_params)
            if res_data:
                res_data = json.loads(res_data)
                soap_res_code = res_data.get('code')
                soap_res_user_list = res_data.get('userInfoDtoStr', [])
                soap_res_user_list = [json.loads(i) for i in soap_res_user_list]
                # 没有查询导数据， 终止
                if len(soap_res_user_list) == 0:
                    break
                # 查询返回码失败，终止
                if soap_res_code == 0:
                    page_num = page_num + 1
                    res_user_list = res_user_list + soap_res_user_list
                else:
                    break
            else:
                # 接口异常 终止
                break

        # logger.info(f"接口请求数据{res_user_list}")

        return res_user_list

    def get_member_day(self):
        """从soap中获取当天产生成员数据"""
        page_num = 1
        res_user_list = []
        res_c_u_user_list = []  # 创建和更新的记录
        res_delete_user_list = []  # 删除的记录
        for i in range(1):
            query_params = {"appId": self.appid, "pageNum": page_num}
            res_data = self.client.call(self.soap_acquire_one_day_members, query_params)
            if res_data:
                res_data = json.loads(res_data)
                soap_res_code = res_data.get('code')
                soap_res_user_list = res_data.get('userInfoDtoStr', [])
                soap_res_user_list = [json.loads(i) for i in soap_res_user_list]
                # 没有查询导数据， 终止
                if len(soap_res_user_list) == 0:
                    break
                # 查询返回码失败，终止
                if soap_res_code == 0:
                    page_num = page_num + 1
                    res_user_list = res_user_list + soap_res_user_list
                else:
                    break
            else:
                # 接口异常 终止
                break

        for i in res_user_list:
            # 区分出需要删除的账号，更新，创建
            oper_type = i.get('operType', '')
            if "CREATE" in oper_type:
                res_c_u_user_list.append(i)
            elif "UPDATE" in oper_type:
                res_c_u_user_list.append(i)
            elif "DELETE" in oper_type:
                res_delete_user_list.append(i)
            else:
                pass

        return res_c_u_user_list, res_delete_user_list

    def send_code(self, data):
        '''
        统一认证发送验证码
        '''
        code_res = self.client.call(self.soap_send_code, data.get("userId"), self.appid)
        res = json.loads(code_res)
        return res

    def get_auth_by_sm4(self, data):
        nfs_user = f'{self.user_prefix}{data.get("userId")}'
        password = data.get('password')
        connect_code = self.client.connect()
        if connect_code == 1:
            if self.auth_sources:
                auth_type = data.get('authType')
                if auth_type == 'M_OTP':
                    # 统一认证验证工号和手机验证码，若成功继续调用统一认证校验工号和密码
                    hash = java_sm4_hash(json.dumps(data))
                    auth_m_otp_res = self.client.call(self.soap_auth_by_sm4, hash)
                    if not auth_m_otp_res:
                        return {
                            'status': 'error',
                            'message': f'获取 SOAP 数据错误',
                            'code': 1,
                            'data': ''
                        }
                    auth_m_otp_res = json.loads(auth_m_otp_res)
                    if auth_m_otp_res.get('status') != 'success':
                        return {
                            'status': 'error',
                            'message': auth_m_otp_res.get('message'),
                            'code': auth_m_otp_res.get('code'),
                            'data': ''
                        }
                data = {
                    "appId": data.get('appId'),
                    "userId": data.get('userId'),
                    "password": data.get('password'),
                    "requestId": data.get('requestId'),
                    "authType": 'M_PASSWORD',
                    "requestName": None,
                    "requestReason": None,
                    'authFlag': "1"
                }
                hash = java_sm4_hash(json.dumps(data))
                # logger.info(hash)
                auth_sm4_res = self.client.call(self.soap_auth_by_sm4, hash)
                if not auth_sm4_res:
                    ret = {
                        'status': 'error',
                        'message': f'获取 SOAP 数据错误',
                        'code': 1,
                        'data': ''
                    }
                else:
                    # decrypt = java_sm4_decrypt(auth_sm4_res)
                    auth_sm4_res = json.loads(auth_sm4_res)
                    if auth_sm4_res.get('status') != 'success':
                        ret = {
                            'status': 'error',
                            'message': auth_sm4_res.get('message'),
                            'code': auth_sm4_res.get('code'),
                            'data': ''
                        }
                    else:
                        stu = reset_user_pwd(nfs_user, password, o_krbpasswordexpiration=True)
                        if stu == 'notfound':
                            res_data = self.client.call(self.soap_acquire_members, {"appId": self.appid, "pageNum": 1})
                            if not res_data:
                                ret = {
                                    'status': 'error',
                                    'message': f'获取 SOAP 数据错误',
                                    'code': 1,
                                    'data': ''
                                }
                            else:
                                res_data = json.loads(res_data)
                                users = res_data.get('userInfoDtoStr')
                                soap_res_user_list = []
                                for i in users:
                                    if i.get('ID') == data.get("userId"):
                                        i['password'] = password
                                        soap_res_user_list.append(i)
                                        break
                                self.update_user_to_db_freeipa(soap_res_user_list)
                                # user = res_data.get('userInfoDtoStr')[0]
                                # add_user(nfs_user, user.get('Name'), password)
                                ret = {
                                    'status': 'success',
                                    'message': auth_sm4_res.get('message'),
                                    'code': auth_sm4_res.get('code'),
                                    'data': sm4_hash(json.dumps({
                                        'token': auth_sm4_res.get('token'),
                                        'userId': nfs_user.lower()
                                    }), self.soap_sm4_keys)
                                }
                        else:
                            ret = {
                                'status': 'success',
                                'message': auth_sm4_res.get('message'),
                                'code': auth_sm4_res.get('code'),
                                'data': sm4_hash(json.dumps({
                                    'token': auth_sm4_res.get('token'),
                                    'userId': nfs_user.lower()
                                }), self.soap_sm4_keys)
                            }
                logger.info(ret)
                if ret.get('code') != 10001:
                    return ret
        # freeipa auth
        check_res = check_user(nfs_user, password)
        if not check_res.get('exist'):
            return {
                'status': 'error',
                'message': '用户不存在或密码错误',
                'code': 1,
                'data': ''
            }
        return {
            'status': 'success',
            'message': '',
            'code': 0,
            'data': sm4_hash(json.dumps({
                'token': '',
                'userId': nfs_user.lower()
            }), self.soap_sm4_keys)
        }

    def update_password_emop(self, data):
        connect_code = self.client.connect()
        if connect_code != 1:
            return {
                'status': 'error',
                'message': f'SOAP 服务连接异常',
                'code': 1,
                'data': ''
            }
        nfs_user = f'{self.user_prefix}{data.get("userId")}'
        hash = java_sm4_hash(json.dumps(data))
        # logger.info(hash)
        res_data = self.client.call(self.soap_update_password_emop, hash)
        if not res_data:
            return {
                'status': 'error',
                'message': f'获取 SOAP 数据错误',
                'code': 1,
            }
        res_data = json.loads(res_data)
        if res_data.get('status') != 'success':
            return {
                'status': 'error',
                'message': res_data.get('message'),
                'code': res_data.get('code'),
            }
        stu = reset_user_pwd(nfs_user, data.get('newPassword'), o_krbpasswordexpiration=True)
        if stu == 'notfound':
            acquire_res_data = self.client.call(self.soap_acquire_members, {"appId": self.appid, "pageNum": 1})
            acquire_res_data = json.loads(acquire_res_data)
            users = acquire_res_data.get('userInfoDtoStr')
            soap_res_user_list = []
            for i in users:
                if i.get('ID') == data.get("userId"):
                    i['password'] = data.get('password')
                    soap_res_user_list.append(i)
                    break
            self.update_user_to_db_freeipa(soap_res_user_list)
        elif stu == 'unknown':
            logger.info('freeipa update pwd error')
            return {
                'status': 'error',
                'message': '更新数据时错误',
                'code': 1,
            }
        return {
            'status': 'success',
            'message': res_data.get('message'),
            'code': res_data.get('code'),
        }


def get_soapmember():
    obj = AuthSource.objects.last()
    if not obj:
        return False
    return SoapMember()
